/**
 *
 */
package com.yeejoin.amos.iot.business.controller;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Sort;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.yeejoin.amos.iot.business.enums.AlarmStatus;
import com.yeejoin.amos.iot.business.param.AlarmPageParam;
import com.yeejoin.amos.iot.business.param.DeleteAlarmParam;
import com.yeejoin.amos.iot.business.service.intfc.IAlarmService;
import com.yeejoin.amos.iot.core.common.request.CommonPageable;
import com.yeejoin.amos.iot.core.common.request.CommonRequest;
import com.yeejoin.amos.iot.core.enums.QueryOperatorEnum;
import com.yeejoin.amos.iot.mongodb.service.intfc.IMetricDataQueryService;
import com.yeejoin.amos.op.core.common.query.DaoCriteria;
import com.yeejoin.amos.op.core.common.response.CommonResponse;
import com.yeejoin.amos.op.core.util.CommonResponseUtil;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;

/**
 * 告警控制器
 *
 * @author as-youjun
 *
 */
@RestController
@RequestMapping("/alarm")
@Api(value = "alarm", tags = {"告警管理"})
public class AlarmController extends BaseController {

    @Autowired
    private IAlarmService alarmService;

    @Autowired
    private IMetricDataQueryService metricDataQueryService;

    /**
     *
     * <pre>
     * 分页查询告警数据（基础数据）
     * </pre>
     *
     * @param requests 查询条件集合，可以为空
     * @param pageable 分页参数，默认pageNumber=0&pageSize=10
     * @return CommonResponse
     */
    @ApiOperation(httpMethod = "POST", value = "分页查询告警数据（基础数据）", notes = "分页查询告警数据（基础数据）")
    @RequestMapping(value = "/list", produces = "application/json;charset=UTF-8", method = RequestMethod.POST)
    public CommonResponse queryAlarmList(
            @ApiParam(value = "查询参数", required = true) @RequestBody(required = false) List<CommonRequest> requests,
            @ApiParam(value = "分页对象", required = true, defaultValue = "pageNumber=0&pageSize=10") CommonPageable pageable) {
        List<DaoCriteria> criterias = buildDaoCriteriasNoDelete(requests);
        criterias = criterias.stream().filter(e ->  !"orgCode".equals(e.getPropertyName())).collect(Collectors.<DaoCriteria>toList());
        if (!"SUPER".equals(getUserId())) {
            DaoCriteria daoCriteria = new DaoCriteria();
            daoCriteria.setPropertyName("orgCode");
            daoCriteria.setOperator(QueryOperatorEnum.LIKE.getName());
            daoCriteria.setValue(getCompanyOrgCode() + "%");
            criterias.add(daoCriteria);
        }
        pageable.setSort(Sort.unsorted());
        Map<String, Object> map = alarmService.findAllByParam(criterias, pageable);
        return CommonResponseUtil.success(map);
    }

    /**
     *
     * <pre>
     * 清除告警数据（只改变数据状态）
     * </pre>
     *
     * @param requests 要删除的条件实体
     * @return CommonResponse
     */
    @ApiOperation(httpMethod = "POST", value = "清除告警数据（只改变数据状态）", notes = "清除告警数据（只改变数据状态）")
    @RequestMapping(value = "/delete", produces = "application/json;charset=UTF-8", method = RequestMethod.POST)
    public CommonResponse deleteAlarm(
            @ApiParam(value = "清除告警数据的参数", required = true) @RequestBody(required = false) DeleteAlarmParam requests) {
        alarmService.deleteAlarm(requests);
        return CommonResponseUtil.success();
    }

    /**
     *
     * <pre>
     * 当前告警统计
     * </pre>
     * @return CommonResponse
     */
    @ApiOperation(httpMethod = "GET", value = "当前告警统计", notes = "当前告警统计")
    @RequestMapping(value = "/presentAlarm-static", produces = "application/json;charset=UTF-8", method = RequestMethod.GET)
    public CommonResponse queryPresentAlarmStatic() {
        List<DaoCriteria> daoCriterias = buildDaoCriteriasNoParam();
        daoCriterias.add(new DaoCriteria("status", QueryOperatorEnum.EQUAL.getName(), AlarmStatus.RAISE.getValue()));
        Map<String, Object> result = alarmService.queryPresentAlarmStatic(daoCriterias);
        return CommonResponseUtil.success(result);
    }

    /**
     *
     * <pre>
     * 根据状态统计告警数据
     * </pre>
     * @return CommonResponse
     */
    @ApiOperation(httpMethod = "GET", value = "根据状态统计告警数据", notes = "根据状态统计告警数据")
    @RequestMapping(value = "/alarmStaticByStatus", produces = "application/json;charset=UTF-8", method = RequestMethod.GET)
    public CommonResponse alarmStaticByStatus() {
        return CommonResponseUtil.success(alarmService.alarmStaticByStatus(getOrgCode()));
    }

    @ApiOperation(httpMethod = "POST", value = "根据设备属性获取告警信息", notes = "根据设备属性获取告警信息")
    @RequestMapping(value = "/equipAttr", produces = "application/json;charset=UTF-8", method = RequestMethod.POST)
    public CommonResponse queryAlarmByEquipAttr(@ApiParam(value = "查询参数", required = true) @RequestBody(required = true) List<CommonRequest> requests) {
        HashMap<String, Object> param = new HashMap<>();
        HashMap<String, Object> attrs = new HashMap<>();
        for (CommonRequest cr : requests) {
            attrs.put(cr.getName(), cr.getValue());
        }
        param.put("orgCode", getCompanyOrgCode());
        param.put("attrs", attrs);
        List list = alarmService.queryAlarmByEquipAttr(param);
        return CommonResponseUtil.success(list);
    }

    @ApiOperation(value = "根据状态统计告警指标数据", notes = "根据状态统计告警指标数据")
    @PostMapping(value = "/alarmPage")
    public CommonResponse alarmPage(@ApiParam(value = "查询参数", required = true) @RequestBody AlarmPageParam alarmPageParam) {
        return CommonResponseUtil.success(metricDataQueryService.alarmPage(alarmPageParam));
    }
}
