package com.yeejoin.amos.iot.business.controller;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import javax.servlet.http.HttpServletRequest;
import javax.validation.ConstraintViolationException;

import com.alibaba.fastjson.JSON;
import com.yeejoin.amos.component.feign.model.FeignClientResult;
import com.yeejoin.amos.feign.privilege.model.CompanyModel;
import com.yeejoin.amos.feign.privilege.model.DepartmentModel;
import com.yeejoin.amos.feign.privilege.model.RoleModel;
import com.yeejoin.amos.iot.business.param.CompanyBo;
import com.yeejoin.amos.iot.business.param.DepartmentBo;
import com.yeejoin.amos.iot.business.param.RoleBo;
import com.yeejoin.amos.iot.business.remote.RemoteSecurityService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.TransactionSystemException;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.client.RestTemplate;
import org.typroject.tyboot.core.foundation.context.RequestContext;

import com.alibaba.fastjson.JSONObject;
import com.yeejoin.amos.component.feign.config.InnerInvokException;
import com.yeejoin.amos.feign.privilege.Privilege;
import com.yeejoin.amos.feign.privilege.model.AgencyUserModel;
import com.yeejoin.amos.iot.business.param.ReginParams;
import com.yeejoin.amos.iot.context.RedisUtil;
import com.yeejoin.amos.iot.core.common.request.CommonRequest;
import com.yeejoin.amos.iot.core.enums.QueryOperatorEnum;
import com.yeejoin.amos.iot.core.util.StringUtil;
import com.yeejoin.amos.op.core.common.query.DaoCriteria;
import com.yeejoin.amos.op.core.util.ApplicationConfig;

import springfox.documentation.annotations.ApiIgnore;

import static org.typroject.tyboot.component.cache.Redis.redisTemplate;

/**
 * 基础控制器
 * 
 * @author as-youjun
 *
 */
@ApiIgnore
@RestController
@RequestMapping("/base")
public class BaseController {
	
	@Autowired
	protected RedisUtil redisUtil;
	
	
	@Autowired
	protected HttpServletRequest request;
	
	@Autowired
	private ApplicationConfig applicationConfig;
	private  RestTemplate restTemplate;
	private static String X_ACCESS_TOKEN = null;
	/**
	 * 成功返回状态
	 */
	protected static final String SUCCESS = "SUCCESS";

	/**
	 * 失败返回状态
	 */
	protected static final String FAILED = "FAILED";

	/**
	 * 默认页大小
	 */
	protected static final int DEFAULT_PAGE_SIZE = 10;

	@Autowired
	private RemoteSecurityService remoteSecurityService;

	protected String getToken() {
		String token = request.getHeader("token");
		return token;
	}
	
	
	/**
	 * 当前登录用户信息
	 */
	public AgencyUserModel getUserInfo() {

		try {
			String token = request.getHeader("token");
			String product = request.getHeader("product");
			String appKey = request.getHeader("appKey");
			RequestContext.setToken(token);
			RequestContext.setProduct(product);
			RequestContext.setAppKey(appKey);
			AgencyUserModel result = Privilege.agencyUserClient.getme().getResult();
			return result;
		} catch (InnerInvokException e) {
			System.err.println("get user info fail");
		}
		return null;

	}

	public String getUserId() {
		return getUserInfo().getUserId();
	}
	
	
	
	
	public String getCurrentCompanyNameOfUser(){
		ReginParams reginParams = this.getSelectedOrgInfo();
		if(reginParams == null || reginParams.getCompany() == null){
			return "";
		}
		return reginParams.getCompany().getCompanyName();
	}
	
	
	/**
	 * 获取公司orgcode
	 * @return
	 */
	protected String getOrgCode(){
		ReginParams reginParams = this.getSelectedOrgInfo();
		if(reginParams == null){
			return "";
		}
		if (reginParams.getDepartment() == null || reginParams.getDepartment().getOrgCode() == null) {
			return reginParams.getCompany().getOrgCode();
		}

		return reginParams.getDepartment().getOrgCode();
	}

	protected String getCompanyOrgCode(){
		ReginParams reginParams = this.getSelectedOrgInfo();
		if(reginParams == null){
			return "";
		}
		return reginParams.getCompany().getOrgCode();
	}

	/**
	 * 
	 * <pre>
	 * 生成查询条件
	 * </pre>
	 * 
	 * @param queryRequests
	 * @return
	 */
	public List<DaoCriteria> buildDaoCriterias(List<CommonRequest> queryRequests) {
		List<DaoCriteria> daoCriterias = new ArrayList<DaoCriteria>();
		String orgCode = null;
		Boolean flag = true;
		if(queryRequests != null && !queryRequests.isEmpty()){
			for (CommonRequest query : queryRequests) {
				DaoCriteria criteria = new DaoCriteria();
				if(StringUtil.isNotEmpty(query.getValue())){
					if(query.getName().equals("orgCode")){
						orgCode = query.getValue().toString();
						continue;
					}
					
					if("visibility".equals(query.getName()) && "0".equals(query.getValue())){
						flag = false;
					}
					criteria.setPropertyName(query.getName());
					String column = criteria.getPropertyName();
					if (!(query.getValue() instanceof Collection<?>) 
							&& column.substring(column.length()-2,column.length()).toUpperCase().equals("ID")) {
						criteria.setValue(Long.valueOf(query.getValue().toString()));
					} else {
						criteria.setValue(query.getValue());
					}
					
					String operator = query.getType();
					if (!StringUtil.isNotEmpty(operator)) {
						criteria.setOperator(QueryOperatorEnum.EQUAL.getName());
					}else if(operator.equals(QueryOperatorEnum.LIKE.getName())){
						setLikeCriteria(criteria,query);
					}else if(operator.equals(QueryOperatorEnum.BIGGER_EQUAL.getName())){
						criteria.setOperator(QueryOperatorEnum.getEnum(operator).getName());
					}else if(operator.equals(QueryOperatorEnum.LESS_EQUAL.getName())){
						criteria.setOperator(QueryOperatorEnum.getEnum(operator).getName());
					}else {
						criteria.setOperator(QueryOperatorEnum.getEnum(operator)
								.getName());
					}
					daoCriterias.add(criteria);
				}
			}	
		}
		if(flag){
			
			//获取当前用户code			
			orgCode=getCompanyOrgCode();
			daoCriterias = buildOrgDaoCriteriaOutChildren(daoCriterias,orgCode);
//			if(isDirector()){
//				daoCriterias = buildOrgDaoCriteriaInChildren(daoCriterias,orgCode);
//			}else{
//				daoCriterias = buildOrgDaoCriteriaOutChildren(daoCriterias);
//			}
		}
		daoCriterias.add(new DaoCriteria("isDelete", QueryOperatorEnum.EQUAL.getName(), false));
		return daoCriterias;

	}
	
	public List<DaoCriteria> buildDaoCriteriasNoDelete(List<CommonRequest> queryRequests) {
		List<DaoCriteria> daoCriterias = new ArrayList<DaoCriteria>();
		String orgCode = null;
		Boolean flag = true;
		if(queryRequests != null && !queryRequests.isEmpty()){
			for (CommonRequest query : queryRequests) {
				DaoCriteria criteria = new DaoCriteria();
				if(StringUtil.isNotEmpty(query.getValue())){
					if(query.getName().equals("orgCode")){
						orgCode = query.getValue().toString();
						continue;
					}
					if("visibility".equals(query.getName()) && "0".equals(query.getValue())){
						flag = false;
					}
					criteria.setPropertyName(query.getName());
					String column = criteria.getPropertyName();
					if (!(query.getValue() instanceof Collection<?>) 
							&& column.substring(column.length()-2,column.length()).toUpperCase().equals("ID")) {
						criteria.setValue(Long.valueOf(query.getValue().toString()));
					} else {
						criteria.setValue(query.getValue());
					}
					
					String operator = query.getType();
					if (!StringUtil.isNotEmpty(operator)) {
						criteria.setOperator(QueryOperatorEnum.EQUAL.getName());
					}else if(operator.equals(QueryOperatorEnum.LIKE.getName())){
						setLikeCriteria(criteria,query);
					}else if(operator.equals(QueryOperatorEnum.BIGGER_EQUAL.getName())){
						criteria.setOperator(QueryOperatorEnum.getEnum(operator).getName());
					}else if(operator.equals(QueryOperatorEnum.LESS_EQUAL.getName())){
						criteria.setOperator(QueryOperatorEnum.getEnum(operator).getName());
					}else {
						criteria.setOperator(QueryOperatorEnum.getEnum(operator)
								.getName());
					}
					daoCriterias.add(criteria);
				}
			}	
		}
		if(flag){
			daoCriterias = buildOrgDaoCriteriaOutChildren(daoCriterias,orgCode);
//			if(isDirector()){
//				daoCriterias = buildOrgDaoCriteriaInChildren(daoCriterias,orgCode);
//			}else{
//				daoCriterias = buildOrgDaoCriteriaOutChildren(daoCriterias);
//			}
		}
		return daoCriterias;

	}
	
	public List<DaoCriteria> buildDaoCriteriasNoParam() {
		List<DaoCriteria> daoCriterias = new ArrayList<DaoCriteria>();
		String orgCode = getCompanyOrgCode();
		daoCriterias = buildOrgDaoCriteriaOutChildren(daoCriterias,orgCode);
//		if(isDirector()){
//			daoCriterias = buildOrgDaoCriteriaInChildren(daoCriterias,orgCode);
//		}else{
//			daoCriterias = buildOrgDaoCriteriaOutChildren(daoCriterias);
//		}
		return daoCriterias;
	}
	
	/**
	 * 
	 * <pre>
	 * 生成通过当前用户所属组织机构（包含子机构）过滤的查询条件
	 * </pre>
	 *
	 * @param daoCriterias 查询条件集合
	 * @return
	 */
//	public List<DaoCriteria> buildOrgDaoCriteriaInChildren(List<DaoCriteria> daoCriterias,String orgCode) 
//	{
//		/**
//		 * 当前用户所属公司
//		 */
//		String companyCode = orgCode;
//		if(companyCode == null){
//			CompanyBo company = getUserInfo().getCompany();
//			companyCode = company.getCompCode();
//		}
//		
//		//添加所属公司的过滤条件
//		DaoCriteria compDaoCriteria = new DaoCriteria();
//		compDaoCriteria.setPropertyName("orgCode");
//		compDaoCriteria.setOperator(QueryOperatorEnum.EQUAL.getName());
//		compDaoCriteria.setValue(companyCode);
//		daoCriterias.add(compDaoCriteria);
//		
//		
//		//添加子公司下的过滤条件
//		DaoCriteria childCompDaoCriteria = new DaoCriteria();
//		childCompDaoCriteria.setPropertyName("orgCode");
//		childCompDaoCriteria.setOperator(QueryOperatorEnum.LIKE.getName());
//		childCompDaoCriteria.setValue(companyCode+"*%");
//		daoCriterias.add(childCompDaoCriteria);
//		return daoCriterias;
//	}
	
	/**
	 * 
	 * <pre>
	 * 生成通过当前用户所属组织机构（不包含子机构）过滤的查询条件
	 * </pre>
	 *
	 * @param daoCriterias 查询条件集合
	 * @return
	 */
	public List<DaoCriteria> buildOrgDaoCriteriaOutChildren(List<DaoCriteria> daoCriterias, String orgCode) 
	{
		/**
		 * 当前用户所属公司
		 */
		
		DaoCriteria compDaoCriteria = new DaoCriteria();
		compDaoCriteria.setPropertyName("orgCode");
		compDaoCriteria.setOperator(QueryOperatorEnum.EQUAL.getName());
		compDaoCriteria.setValue(orgCode);
		daoCriterias.add(compDaoCriteria);
		return daoCriterias;
	}

	/**
	 * 对like查询语句的内容进行特殊字符转义
	 * @param criteria
	 * @param query
	 */
	@SuppressWarnings("unchecked")
	public void setLikeCriteria(DaoCriteria criteria,CommonRequest query){
		if (query.getValue() instanceof List) {
			List<String> value = (List<String>) query.getValue();
			if (!value.isEmpty()) {
				value = value.stream().map(s -> {
					String str = s.replace("%","/%").replace("_","/_");
					return "%" + str + "%";
				}).collect(Collectors.toList());
				criteria.setValue(value);
				criteria.setOperator(QueryOperatorEnum.LIKE.getName());
			}
		} else {
			String value = (String) query.getValue();
			if(value != null){
				value = value.replace("%","/%").replace("_","/_");
				criteria.setValue("%"+value+"%");
				criteria.setOperator(QueryOperatorEnum.LIKE.getName());
			}
		}
	}
	
	/**
	 * <pre>
	 * 提取错误异常中的错误消息
	 * </pre>
	 * 
	 * <p>
	 * 当实体类中对属相注解了以下类似的注解，需要用try.catch语句捕获异常，使用 #getErrorMessage(Exception)
	 * 提取出异常信息NotNull(message = "属性不能为空！")
	 * </p>
	 * @see {@code NotBlank} ... NotNull、NotEmpty...
	 * @see {@code Valid}
	 * 
	 * @param e
	 * @return
	 */
	public String getErrorMessage(Exception e){
		StringBuilder message = new StringBuilder();
		if(e instanceof TransactionSystemException){
			TransactionSystemException exception = (TransactionSystemException) e;
			if(exception.getRootCause() instanceof ConstraintViolationException) {
				ConstraintViolationException root = (ConstraintViolationException) exception.getRootCause();
				root.getConstraintViolations().forEach(constraintViolation->{message.append(constraintViolation.getMessageTemplate());});
			}else{
				message.append(e.getMessage());
			}
		} else if (e instanceof ConstraintViolationException) {
			((ConstraintViolationException) e).getConstraintViolations().forEach(constraintViolation -> message.append(constraintViolation.getMessageTemplate()));
		} else {
			message.append("操作异常！");
		}
		return message.toString();
	}
	
	protected ReginParams getSelectedOrgInfo() {
		String json = (String) redisUtil.hget("regoin_"+getUserId(), getToken().substring(0, getToken().indexOf('-', 1)));
		if (ObjectUtils.isEmpty(json)) {
			json = selectDefaultOrgInfo();
		}
		return JSONObject.parseObject(json, ReginParams.class);
	}

	private String selectDefaultOrgInfo() {

		String token = (String) request.getHeader("token");
		token = ObjectUtils.isEmpty(token) ? (String) request.getHeader("X-Access-Token") : token;
		String product = (String) request.getHeader("product");
		String appKey = (String) request.getHeader("appKey");

		RequestContext.setToken(token);
		RequestContext.setProduct(product);
		RequestContext.setAppKey(appKey);

		FeignClientResult feignClientResult;
		AgencyUserModel userModel=null;
		try {
			feignClientResult = Privilege.agencyUserClient.getme();
			userModel = (AgencyUserModel) feignClientResult.getResult();

		} catch (InnerInvokException e) {
			e.printStackTrace();
		}
		ReginParams regionParam = new ReginParams();
		if(userModel != null){
			String userId = userModel.getUserId();
			Map<Long, List<RoleModel>> orgRoles = userModel.getOrgRoles();
			List<RoleModel> roleModels = null;
			if(!ObjectUtils.isEmpty(orgRoles)) {
				for (Map.Entry<Long, List<RoleModel>> entry : orgRoles.entrySet()) {
					roleModels = entry.getValue();
					break;
				}
			}
			ReginParams reginParams = JSON.parseObject((String) redisTemplate.opsForValue().get(buildKey(userId, token)), ReginParams.class);

			if(reginParams == null && userModel.getCompanys().size() > 0){
				CompanyModel companyModel = userModel.getCompanys().get(0);

				List<DepartmentModel> deptList = remoteSecurityService.getDepartmentTreeByCompanyId(token, product, appKey, companyModel.getSequenceNbr().toString());
				if(deptList.size() > 0){
					CompanyBo companyBo = convertCompanyModelToBo(companyModel);
					DepartmentBo departmentBo = convertDepartmentModelToBo(deptList.get(0));
					regionParam.setCompany(companyBo);
					regionParam.setDepartment(departmentBo);
				}
				if(!ObjectUtils.isEmpty(roleModels)){
					regionParam.setRole(convertRoleModelToBo(roleModels.get(0)));
				}
				redisTemplate.opsForValue().set(buildKey(userId, token), JSONObject.toJSONString(regionParam));
				return JSONObject.toJSONString(regionParam);
			} else {
				return JSONObject.toJSONString(reginParams);
			}
		}
		return null;
	}

	private DepartmentBo convertDepartmentModelToBo(DepartmentModel departmentModel){
		DepartmentBo departmentBo = new DepartmentBo();
		if(departmentModel != null) {
			departmentBo.setCompanySeq(departmentModel.getCompanySeq().toString());
			departmentBo.setDepartmentDesc(departmentModel.getDepartmentDesc());
			departmentBo.setDepartmentName(departmentModel.getDepartmentName());
//			departmentBo.setLevel(departmentModel.getLevel());
			departmentBo.setOrgCode(departmentModel.getOrgCode());
			departmentBo.setParentId(departmentModel.getParentId().toString());
			departmentBo.setDeptOrgCode(departmentModel.getDeptOrgCode().toString());
			departmentBo.setSequenceNbr(departmentModel.getSequenceNbr().toString());
		}
		return departmentBo;
	}

	private RoleBo convertRoleModelToBo(RoleModel roleModel) {
		RoleBo roleBo = new RoleBo();
		if(roleModel != null){
			roleBo.setRoleName(roleModel.getRoleName());
			roleBo.setRoleType(roleModel.getRoleType());
			roleBo.setSequenceNbr(roleModel.getSequenceNbr().toString());
		}
		return roleBo;
	}

	/**
	 * 	Model 转 Bo
	 */
	private CompanyBo convertCompanyModelToBo(CompanyModel companyModel){

		CompanyBo companyBo = new CompanyBo();
		if(companyModel != null) {
			companyBo.setAddress(companyModel.getAddress());
			companyBo.setCompanyName(companyModel.getCompanyName());
//			companyBo.setCompanyOrgCode(Long.parseLong(companyModel.getCompanyOrgCode().toString()));
			companyBo.setEmail(companyModel.getEmail());
			companyBo.setLandlinePhone(companyModel.getLandlinePhone());
			companyBo.setLongitude(companyModel.getLongitude());
			companyBo.setLatitude(companyModel.getLatitude());
			companyBo.setLevel(companyModel.getLevel());
			companyBo.setOrgCode(companyModel.getOrgCode());
			companyBo.setSequenceNbr(companyModel.getSequenceNbr());
			companyBo.setParentId(companyModel.getParentId().toString());
		}
		return companyBo;
	}

	private String buildKey(String userId, String token) {
		return "region_" + userId + "_" + token;
	}

	protected void saveSelectedOrgInfo(ReginParams reginParams) {
		redisUtil.hset("regoin_"+getUserId(), getToken().substring(0, getToken().indexOf('-', 1)), JSONObject.toJSONString(reginParams));
	}
}
