package com.yeejoin.amos.iot.business.controller;

import com.alibaba.fastjson.JSON;
import com.yeejoin.amos.component.feign.config.InnerInvokException;
import com.yeejoin.amos.feign.privilege.Privilege;
import com.yeejoin.amos.feign.privilege.model.AgencyUserModel;
import com.yeejoin.amos.feign.privilege.model.CompanyModel;
import com.yeejoin.amos.iot.business.param.CompanyBo;
import com.yeejoin.amos.op.core.common.response.CommonResponse;
import com.yeejoin.amos.op.core.util.ApplicationConfig;
import com.yeejoin.amos.op.core.util.CommonResponseUtil;
import com.yeejoin.amos.op.core.util.Constants;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.MediaType;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.client.RestTemplate;
import org.typroject.tyboot.core.foundation.context.RequestContext;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("/company")
@Api(value = "company", tags = {"公司"})
public class CompanyController extends BaseController {

    @Autowired
    private ApplicationConfig applicationConfig;

    @ApiOperation(httpMethod = "POST", value = "获取公司权限树", notes = "获取公司权限树（包含子公司）")
    @RequestMapping(value = "/manger-tree", produces = "application/json;charset=UTF-8", method = RequestMethod.GET)
    public CommonResponse mangerTree() {
//        String url = applicationConfig.getParamValueStr("params.remoteurl");
//        HttpHeaders headers = new HttpHeaders();
//        MediaType type = MediaType
//                .parseMediaType("application/json; charset=UTF-8");
//        headers.setContentType(type);
//        headers.add("Accept", MediaType.APPLICATION_JSON.toString());
//        headers.add("X-From-Service", "true");
//        HttpEntity<String> formEntity = new HttpEntity<String>(headers);
//        List<HashMap> companys = null;
        try {
//            RestTemplate restTemplate = new RestTemplate();
//            CommonResponse commonResponse = restTemplate.exchange(url + "v1/company/subtree/" + getSelectedOrgInfo().getCompany().getSequenceNbr(),
//                    HttpMethod.GET, formEntity, CommonResponse.class).getBody();
//            if (commonResponse.getResult().equals(Constants.RESULT_SUCCESS)) {
//                companys = JSON.parseArray(JSON.toJSONString(commonResponse.getDataList()), HashMap.class);
//            }
            CompanyBo selectCompany = this.getSelectedOrgInfo().getCompany();
            if (ObjectUtils.isEmpty(selectCompany)) {
                CompanyModel companyModel = getUserInfo().getCompanys().get(0);
                selectCompany = new CompanyBo();
                BeanUtils.copyProperties(companyModel, selectCompany);
            }
            Map<String, Object> mapCompany = Obj2Map(selectCompany);
            mapCompany.put("orgCode", selectCompany.getOrgCode());
            mapCompany.put("levelName", selectCompany.getCompanyName());
            mapCompany.put("title", selectCompany.getCompanyName());
            mapCompany.put("value", String.valueOf(selectCompany.getSequenceNbr()));
            //mapCompany.put("children", mapList(companys));
            mapCompany.put("children", getCompanyChild(selectCompany.getSequenceNbr()));
            List resultList = new ArrayList();
            resultList.add(mapCompany);
            return CommonResponseUtil.success(resultList);
        } catch (Exception e) {
            e.printStackTrace();
            return CommonResponseUtil.failure();
        }
    }

    private List<HashMap> mapList(List<HashMap> list) {
        for (HashMap hm : list) {
            hm.put("value", hm.get("sequenceNbr"));
            hm.put("title", hm.get("companyName"));
            if (!ObjectUtils.isEmpty(hm.get("children"))) {
                hm.put("children", mapList((List<HashMap>) hm.get("children")));;
            }
        }
        return list;
    }
    
    
    

    private Map<String, Object> Obj2Map(Object obj) throws Exception {
        Map<String, Object> map = new HashMap<String, Object>();
        Field[] fields = obj.getClass().getDeclaredFields();
        for (Field field : fields) {
            field.setAccessible(true);
            map.put(field.getName(), String.valueOf(field.get(obj)));
        }
        return map;
    }
    
    
    /**
	 * 根据公司ID查询下的子公司
	 */
	public List<CompanyModel> getCompanyChild(Long companySeq) {

		Map<String, Object> map = new HashMap<>();
		List<CompanyModel> childrens = new ArrayList<CompanyModel>();
		try {
			String token = request.getHeader("token");
			String product = request.getHeader("product");
			String appKey = request.getHeader("appKey");
			RequestContext.setToken(token);
			RequestContext.setProduct(product);
			RequestContext.setAppKey(appKey);
			CompanyModel result = Privilege.companyClient.seleteOne(companySeq).getResult();
			if(result.getChildren()!=null && !result.getChildren().isEmpty()) {
				childrens = (List<CompanyModel>) result.getChildren();
			}
			
		} catch (InnerInvokException e) {
			System.err.println("get user info fail");
		}
		return childrens;

	}
}
