package com.yeejoin.amos.iot.business.controller;

import com.alibaba.fastjson.JSONObject;
import com.yeejoin.amos.iot.business.param.MongoPageParam;
import com.yeejoin.amos.iot.business.param.QueryMetricDataParam;
import com.yeejoin.amos.iot.core.common.response.CommonPage;
import com.yeejoin.amos.iot.mongodb.MongoCondition;
import com.yeejoin.amos.iot.mongodb.MongoSort;
import com.yeejoin.amos.iot.mongodb.dao.repository.EquipMetricDataRepository;
import com.yeejoin.amos.iot.mongodb.entity.EqpMetricData;
import com.yeejoin.amos.iot.mongodb.service.intfc.IEquipMetricDataService;
import com.yeejoin.amos.iot.mongodb.service.intfc.IMetricDataQueryService;
import com.yeejoin.amos.op.core.common.response.CommonResponse;
import com.yeejoin.amos.op.core.util.CommonResponseUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * 设备指标数据管理控制器
 *
 * @author as-youjun
 */
@RestController
@RequestMapping("/metricdata")
@Api(value = "metricdata", tags = {"设备指标数据管理"})
public class EquipMetricDataController extends BaseController {

    @Autowired
    private IEquipMetricDataService equipMetricDataService;

    @Autowired
    private IMetricDataQueryService metricDataQueryService;

    @Autowired
    private EquipMetricDataRepository equipMetricDataRepository;

    @ApiOperation(httpMethod = "POST", value = "条件查询指标数据", notes = "条件查询指标数据")
    @RequestMapping(value = "/findMetricData", produces = "application/json;charset=UTF-8", method = RequestMethod.POST)
    public CommonResponse findMetricData(
            @ApiParam(value = "参数", required = true) @RequestBody QueryMetricDataParam param) {
        List<EqpMetricData> data = equipMetricDataService.findByParam(param);
        return CommonResponseUtil.success(data);
    }

    @ApiOperation(httpMethod = "POST", value = "条件查询指标数据", notes = "条件查询指标数据")
    @RequestMapping(value = "/pageByParam", produces = "application/json;charset=UTF-8", method = RequestMethod.POST)
    public CommonResponse pageByParam(
            @ApiParam(value = "参数", required = true) @RequestBody QueryMetricDataParam param) {
        List<EqpMetricData> data = equipMetricDataRepository.pageByParam(param);
        return CommonResponseUtil.success(data);
    }

    @ApiOperation(httpMethod = "POST", value = "条件查询指标数据", notes = "条件查询指标数据")
    @RequestMapping(value = "/testBasicQuery", produces = "application/json;charset=UTF-8", method = RequestMethod.POST)
    public CommonResponse testBasicQuery(
            @ApiParam(value = "参数", required = true) @RequestBody QueryMetricDataParam param) {
        List<EqpMetricData> data = equipMetricDataRepository.testBasicQuery(param);
        return CommonResponseUtil.success(data);
    }

    @ApiOperation(httpMethod = "POST", value = "条件查询指标数据", notes = "条件查询指标数据")
    @RequestMapping(value = "/testTextQuery", produces = "application/json;charset=UTF-8", method = RequestMethod.POST)
    public CommonResponse testTextQuery(
            @ApiParam(value = "参数", required = true) @RequestBody QueryMetricDataParam param) {
        List<EqpMetricData> data = equipMetricDataRepository.testTextQuery(param);
        return CommonResponseUtil.success(data);
    }

    @ApiOperation(value = "通用条件[指标数据列表]", notes = "通用条件[指标数据列表]")
    @PostMapping(value = "/listMongoData")
    public CommonResponse listMongoData(
            @ApiParam(value = "参数", required = true) @RequestBody MongoPageParam param) {
        String collectionName = param.getCollectionName();
        List<MongoCondition> conditions = param.getConditions();
        Integer skip = param.getSkip();
        Integer limit = param.getLimit();
        List<MongoSort> sorts = param.getSorts();
        List<JSONObject> data = metricDataQueryService.list(collectionName, conditions, skip, limit, sorts);
        Long count = metricDataQueryService.count(collectionName, conditions);
        return CommonResponseUtil.success(new CommonPage<>(data, null, count));
    }

    @ApiOperation(value = "通用条件[指标数据详情]", notes = "通用条件[指标数据详情]")
    @PostMapping(value = "/findMongoData")
    public CommonResponse findMongoData(
            @ApiParam(value = "参数", required = true) @RequestBody MongoPageParam param) {
        String collectionName = param.getCollectionName();
        List<MongoCondition> conditions = param.getConditions();
        JSONObject data = metricDataQueryService.findOne(collectionName, conditions);
        return CommonResponseUtil.success(data);
    }

    @ApiOperation(value = "通用条件[统计指标数据]", notes = "通用条件[统计指标数据]")
    @PostMapping(value = "/statisticMongoData")
    public CommonResponse statisticMongoData(
            @ApiParam(value = "参数", required = true) @RequestBody MongoPageParam param) {
        String collectionName = param.getCollectionName();
        List<MongoCondition> conditions = param.getConditions();
        String statisticType = param.getStatisticType();
        String groupBy = param.getGroupBy();
        String sumBy = param.getSumBy();
        List<MongoSort> sorts = param.getSorts();
        List<JSONObject> data = metricDataQueryService.statistic(collectionName, conditions, statisticType, groupBy, sumBy, sorts);
        return CommonResponseUtil.success(data);
    }


}
