package com.yeejoin.amos.iot.business.controller;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.yeejoin.amos.iot.business.constants.IotConstant;
import com.yeejoin.amos.iot.business.entity.Capability;
import com.yeejoin.amos.iot.business.entity.CapabilityAttribute;
import com.yeejoin.amos.iot.business.entity.Equipment;
import com.yeejoin.amos.iot.business.entity.EquipmentTemplate;
import com.yeejoin.amos.iot.business.entity.Subgraph;
import com.yeejoin.amos.iot.business.entity.mybatis.EquipmentAnBo;
import com.yeejoin.amos.iot.business.entity.mybatis.EquipmentBo;
import com.yeejoin.amos.iot.business.enums.DataType;
import com.yeejoin.amos.iot.business.enums.EquipmentType;
import com.yeejoin.amos.iot.business.param.EqpTemplPageParam;
import com.yeejoin.amos.iot.business.param.EquipParam;
import com.yeejoin.amos.iot.business.param.InputColumn;
import com.yeejoin.amos.iot.business.service.intfc.ICapabilityService;
import com.yeejoin.amos.iot.business.service.intfc.ICapacityPropertyService;
import com.yeejoin.amos.iot.business.service.intfc.IDeviceService;
import com.yeejoin.amos.iot.business.service.intfc.IEqpTemplService;
import com.yeejoin.amos.iot.business.service.intfc.IEquipmentService;
import com.yeejoin.amos.iot.business.service.intfc.IMetricDataService;
import com.yeejoin.amos.iot.business.service.intfc.ITopoService;
import com.yeejoin.amos.iot.core.enums.QueryOperatorEnum;
import com.yeejoin.amos.op.core.common.query.DaoCriteria;
import com.yeejoin.amos.op.core.common.response.CommonResponse;
import com.yeejoin.amos.op.core.common.response.TreeResponse;
import com.yeejoin.amos.op.core.util.CommonResponseUtil;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;

@RestController
@RequestMapping("/iot")
@Api(value = "iot", tags = { "提供给外部的接口" })
public class ExportController extends BaseController{
	@Autowired
	private ITopoService iTopoService;
	@Autowired
	private IDeviceService iDeviceService;
	@Autowired
	private IEquipmentService equipmentService;
	@Autowired
	private IEqpTemplService eqpTemplService;
	@Autowired
	private ICapabilityService iCapabilityService;
	@Autowired
	private IMetricDataService metricDataService;
	@Autowired
	private ICapacityPropertyService iCapacityPropertyService;
	@ApiOperation(httpMethod = "GET", value = "根据公司id获取视图树", notes = "根据公司id获取视图树")
	@RequestMapping(value = "/subgraph-orgcode/{orgCode}", produces = "application/json;charset=UTF-8", method = RequestMethod.GET)
	public CommonResponse getSubviewByOrgCode(@ApiParam(value = "orgCode", required = true, defaultValue = "1")
	@PathVariable String orgCode) {
		List<Subgraph> subgraphList = iTopoService.getSubviewByOrgCode(orgCode);
		List<TreeResponse> results = new ArrayList<>();
		for (Subgraph subgraph : subgraphList) {
			TreeResponse treeResponse = new TreeResponse();
			treeResponse.setKey(subgraph.getId().toString());
			treeResponse.setValue(subgraph.getOrgCode());
			treeResponse.setLabel(subgraph.getName());
			treeResponse.setType("IOT");			
			results.add(treeResponse);
		}
		return CommonResponseUtil.success(results);
	}

	@PostMapping(value = "/subgraph-entity", produces = "application/json;charset=UTF-8")
	@ApiOperation(value = "查询子图数据", notes = "查询子图数据")
	public CommonResponse fetchSubgraphEntity(@RequestBody EquipParam param,@ApiParam(value = "分页对象", required = true, defaultValue = "page=0&size=10")  Pageable pageable) {
		Page<Object>  pages = iDeviceService.findEqpBySubgraph(param.getSubgraphId(), param.getOrgCode(),pageable);
		List<EquipmentAnBo> reuslts= new ArrayList<>();
		List<Object> lists = pages.getContent();
		for (Object object : lists) {
			Object[] obj = (Object[]) object;
			reuslts.add(getEquipmentAnBo(obj));
		}
		Map<String, Object> map = new HashMap<>();
		map.put("content", reuslts);
		map.put("totalsElement", pages.getTotalElements());
		return CommonResponseUtil.success(map);
	}
	
	@GetMapping(value = "/quota-tree/{id}", produces = "application/json;charset=UTF-8")
	@ApiOperation(value = "查询设备的指标数据", notes = "查询设备的指标数据")
	public CommonResponse fetchQuotaTree(@PathVariable String id){
		List<Capability> capabilitys = iCapabilityService.findByEqpTmpl(Long.parseLong(id));
		List<TreeResponse> results = new ArrayList<>();
		for (Capability capability : capabilitys) {
			Long cpId = capability.getId();
			List<CapabilityAttribute> capabilityAttributes = iCapacityPropertyService.findByCapabilityId(cpId);
			TreeResponse treeResponse = new TreeResponse();
			treeResponse.setKey(cpId.toString());
			treeResponse.setValue(capability.getId().toString());
			treeResponse.setLabel(capability.getName());
			treeResponse.setChildren(getChildren(capabilityAttributes,null));		
			results.add(treeResponse);
		}
		return CommonResponseUtil.success(results);
	}

	@ApiOperation(httpMethod = "GET", value = "根据公司id获取设备数", notes = "根据公司id获取设备数")
	@RequestMapping(value = "/device-tree/{orgCode}", produces = "application/json;charset=UTF-8", method = RequestMethod.GET)
	public CommonResponse getDeviceTreeByOrgCode(
			@ApiParam(value = "orgCode", required = true, defaultValue = "1") @PathVariable String orgCode) {
		List<Subgraph> subgraphList = iTopoService.getSubviewByOrgCode(orgCode);
		List<TreeResponse> results = new ArrayList<>();
		for (Subgraph subgraph : subgraphList) {
			Long subgraphId = subgraph.getId();
			List<EquipmentBo> equipments = iDeviceService.findEqpBySubgraphId(subgraphId, orgCode);
			TreeResponse subgraphTreeResponse = new TreeResponse();
			subgraphTreeResponse.setKey(subgraph.getId().toString());
			subgraphTreeResponse.setValue(subgraph.getOrgCode());
			subgraphTreeResponse.setLabel(subgraph.getName());
			subgraphTreeResponse.setType("IOT");
			subgraphTreeResponse.setChildren(bulidEquTree(equipments));
			results.add(subgraphTreeResponse);
		}
		return CommonResponseUtil.success(results);

	}
	@ApiOperation(httpMethod = "POST", value = "列明查询面积图数据", notes = "列明查询面积图数据")
	@RequestMapping(value = "/area/loadData", method = RequestMethod.POST, produces = "application/json;charset=UTF-8")
	public CommonResponse loadAreaDate(@ApiParam("列信息") @RequestBody List<InputColumn> columnInfo) {
		return CommonResponseUtil.success(loadDate(columnInfo, IotConstant.AREA_CHAR));
	}
	@ApiOperation(httpMethod = "POST", value = "列名查询状图数据", notes = "列名查询柱状图数据")
	@RequestMapping(value = "/bar/loadData", method = RequestMethod.POST, produces = "application/json;charset=UTF-8")
	public CommonResponse loadBarDate(@ApiParam("列信息") @RequestBody List<InputColumn> columnInfo) {
		return CommonResponseUtil.success(loadDate(columnInfo, IotConstant.BAR_CHAR));
		
	}
	@ApiOperation(httpMethod = "POST", value = "列名查询环形图数据", notes = "列名查询环形图数据")
	@RequestMapping(value = "/doughunt/loadData", method = RequestMethod.POST, produces = "application/json;charset=UTF-8")
	public CommonResponse loadDoughuntDate(@ApiParam("列信息") @RequestBody List<InputColumn> columnInfo) {
		return CommonResponseUtil.success(loadDate(columnInfo, IotConstant.DOUGHUNT_CHAR));

	}
	@ApiOperation(httpMethod = "POST", value = "列明查询条形图数据", notes = "列明查询条形图数据")
	@RequestMapping(value = "/hbar/loadData", method = RequestMethod.POST, produces = "application/json;charset=UTF-8")
	public CommonResponse loadHbarDate(@ApiParam("列信息") @RequestBody List<InputColumn> columnInfo) {
		return CommonResponseUtil.success(loadDate(columnInfo, IotConstant.HBAR_CHAR));

	}
	@ApiOperation(httpMethod = "POST", value = "列明查询线形图数据", notes = "列明查询线形图数据")
	@RequestMapping(value = "/line/loadData", method = RequestMethod.POST, produces = "application/json;charset=UTF-8")
	public CommonResponse loadLineDate(@ApiParam("列信息") @RequestBody List<InputColumn> columnInfo) {
		return CommonResponseUtil.success(loadDate(columnInfo, IotConstant.LINE_CHAR));
		
	}
	@ApiOperation(httpMethod = "POST", value = "列名查询饼图数据", notes = "列名查询饼图数据")
	@RequestMapping(value = "/pie/loadData", method = RequestMethod.POST, produces = "application/json;charset=UTF-8")
	public CommonResponse loadPieDate(@ApiParam("列信息") @RequestBody List<InputColumn> columnInfo) {
		return CommonResponseUtil.success(loadDate(columnInfo, IotConstant.PIE_CHAR));

	}

	@ApiOperation(httpMethod = "POST", value = "列名查询玫瑰数据", notes = "列名查询玫瑰数据")
	@RequestMapping(value = "/rose/loadData", method = RequestMethod.POST, produces = "application/json;charset=UTF-8")
	public CommonResponse loadRoseDate(@ApiParam("列信息") @RequestBody List<InputColumn> columnInfo) {
		return CommonResponseUtil.success(loadDate(columnInfo, IotConstant.ROSE_CHAR));
		
	}
	
	@ApiOperation(httpMethod = "POST", value = "根据设备id和指标项获取设备指标", notes = "根据设备id和指标项获取设备指标")
	@RequestMapping(value = "/queryEquipQuotaData/{equipId}", produces = "application/json;charset=UTF-8", method = RequestMethod.POST)
	public CommonResponse queryEquipQuotaData(
			@ApiParam(value = "equipId") @PathVariable Long equipId,
			@ApiParam("列信息") @RequestBody String[] quotaKey){
		String userName = getUserInfo().getUserName();
		Map<String, Object> result =  metricDataService.queryEquipQuotaData(equipId, quotaKey,userName);
		return CommonResponseUtil.success(result);
	}
	@ApiOperation(httpMethod = "GET", value = "查询设备类型下的所有设备", notes = "查询设备类型下的所有设备")
	@RequestMapping(value = "/eqpTempl/equipments/{eqpTemplSerial}", produces = "application/json;charset=UTF-8", method = RequestMethod.POST)
	public CommonResponse fetchDeviceList(
			@ApiParam(value = "设备模版序列", required = true) @PathVariable String eqpTemplSerial){
		List<DaoCriteria> tempDaoCriterias = new ArrayList<DaoCriteria>();
		DaoCriteria tmplDaoCriteria = new DaoCriteria("serialNum",QueryOperatorEnum.EQUAL.getName(),eqpTemplSerial);
		tempDaoCriterias.add(tmplDaoCriteria);
		List<EquipmentTemplate> lists = eqpTemplService.findAll(tempDaoCriterias);
		List<DaoCriteria> daoCriterias = new ArrayList<DaoCriteria>();
		for (EquipmentTemplate equipmentTemplate : lists) {
			DaoCriteria eqpTmplDaoCriteria = new DaoCriteria("eqpTmplId",QueryOperatorEnum.EQUAL.getName(),equipmentTemplate.getId());
			daoCriterias.add(eqpTmplDaoCriteria);
		}
		buildOrgDaoCriteriaOutChildren(daoCriterias, getCompanyOrgCode());
		List<Equipment> equipment = equipmentService.findAll(daoCriterias);
		return CommonResponseUtil.success(equipment);
	}
	@GetMapping(value = "/device/quota-tree/{eqpTemplSerial}", produces = "application/json;charset=UTF-8")
	@ApiOperation(value = "查询设备模版序列的指标数据", notes = "查询设备模版序列的指标数据")
	public CommonResponse fetchQuotaTrees(@PathVariable String eqpTemplSerial){
		List<DaoCriteria> tempDaoCriterias = new ArrayList<DaoCriteria>();
		DaoCriteria tmplDaoCriteria = new DaoCriteria("serialNum",QueryOperatorEnum.EQUAL.getName(),eqpTemplSerial);
		tempDaoCriterias.add(tmplDaoCriteria);
		List<EquipmentTemplate> lists = eqpTemplService.findAll(tempDaoCriterias);
		List<Capability> capabilitys = new ArrayList<>();
		for (EquipmentTemplate equipmentTemplate : lists) {
			List<Capability> capabilits = equipmentTemplate.getCapabilities();
			capabilitys.addAll(capabilits);
		}
		List<TreeResponse> results = new ArrayList<>();
		for (Capability capability : capabilitys) {
			Long cpId = capability.getId();
			List<CapabilityAttribute> capabilityAttributes = iCapacityPropertyService.findByCapabilityId(cpId);
			TreeResponse treeResponse = new TreeResponse();
			treeResponse.setKey(cpId.toString());
			treeResponse.setValue(capability.getId().toString());
			treeResponse.setLabel(capability.getName());
			treeResponse.setChildren(getChildren(capabilityAttributes,null));		
			results.add(treeResponse);
		}
		return CommonResponseUtil.success(results);
	}
	
	private Object loadDate(List<InputColumn> columnInfo, String charType){
		Object resultMap;
		if (charType.equals(IotConstant.PIE_CHAR) || charType.equals(IotConstant.DOUGHUNT_CHAR)
				|| charType.equals(IotConstant.ROSE_CHAR)) {

			 resultMap = metricDataService.getEchartPStatisData(columnInfo);
		} else {
			 resultMap = metricDataService.getEchartStatisData(columnInfo);
		}
		return resultMap;
	}
	private List<TreeResponse> bulidEquTree(List<EquipmentBo> equipments){
		List<TreeResponse> equResults = new ArrayList<>();
		for (EquipmentBo equipment : equipments) {
			Long eqpTmpId = equipment.getEqpTmplId();
			List<Capability> capabilitys = iCapabilityService.findByEqpTmpl(eqpTmpId);
			TreeResponse devTreeResponse = new TreeResponse();
			devTreeResponse.setKey("E"+equipment.getId().toString());
			devTreeResponse.setValue("E"+equipment.getEqpTmplId().toString());
			devTreeResponse.setLabel(equipment.getName());
			devTreeResponse.setType("Dev");
			devTreeResponse.setChildren(bulidCapTree(capabilitys,equipment));
			equResults.add(devTreeResponse);
		}
		return equResults;
	}
	private List<TreeResponse> bulidCapTree(List<Capability> capabilitys,EquipmentBo equipment){
		List<TreeResponse> capResults = new ArrayList<>();
		for (Capability capability : capabilitys) {
			Long cpId = capability.getId();
			List<CapabilityAttribute> capabilityAttributes = iCapacityPropertyService.findByCapabilityId(cpId);
			TreeResponse treeResponse = new TreeResponse();
			treeResponse.setKey("C"+cpId.toString());
			treeResponse.setValue("C"+capability.getName());
			treeResponse.setLabel(capability.getName());
			treeResponse.setType("Cap");
			treeResponse.setChildren(getChildren(capabilityAttributes,equipment));
			capResults.add(treeResponse);
		}
		return capResults;
	}
	private List<TreeResponse> getChildren(List<CapabilityAttribute> capabilityAttributes,EquipmentBo equipment){
		List<TreeResponse> results = new ArrayList<>();
		for (CapabilityAttribute capabilityAttribute : capabilityAttributes) {
			TreeResponse treeResponse = new TreeResponse();
			StringBuffer sb = new StringBuffer(capabilityAttribute.getCapability().getId().toString());
			sb.append("-");
			sb.append(capabilityAttribute.getId());
			sb.append("-");
			sb.append(capabilityAttribute.getName());
			treeResponse.setValue(sb.toString());
			treeResponse.setKey(sb.toString());
			treeResponse.setLabel(capabilityAttribute.getDisplayName());
			treeResponse.setType(DataType.getVName(capabilityAttribute.getDataType()));
			if(equipment !=null)
			treeResponse.setfString1(equipment.getId().toString());
			results.add(treeResponse);
		}
		return results;
	}
	private EquipmentAnBo getEquipmentAnBo(Object[] obj){
		EquipmentAnBo equipmentAnBo = new EquipmentAnBo();
		equipmentAnBo.setId(obj[0].toString());
		equipmentAnBo.setEqpTmplId(obj[1].toString());
		equipmentAnBo.setModel(obj[3].toString());
		equipmentAnBo.setName(obj[2].toString());
		equipmentAnBo.setType(EquipmentType.getName(Integer.parseInt(obj[4].toString())));
		equipmentAnBo.setIndustryId(obj[5].toString());
		return equipmentAnBo;
	}
	
}
