package com.yeejoin.amos.iot.business.controller;

import com.yeejoin.amos.iot.business.entity.EqpTmplInterface;
import com.yeejoin.amos.iot.business.entity.Equipment;
import com.yeejoin.amos.iot.business.entity.Intfc;
import com.yeejoin.amos.iot.business.entity.IntfcAttribute;
import com.yeejoin.amos.iot.business.exceptions.IntfcException;
import com.yeejoin.amos.iot.business.service.intfc.IEqpTmplInterfaceService;
import com.yeejoin.amos.iot.business.service.intfc.IEquipmentService;
import com.yeejoin.amos.iot.business.service.intfc.IInterfacePropertyService;
import com.yeejoin.amos.iot.business.service.intfc.IIntfcService;
import com.yeejoin.amos.iot.core.common.request.CommonPageable;
import com.yeejoin.amos.iot.core.common.request.CommonRequest;
import com.yeejoin.amos.iot.core.enums.QueryOperatorEnum;
import com.yeejoin.amos.op.core.common.query.DaoCriteria;
import com.yeejoin.amos.op.core.common.response.CommonResponse;
import com.yeejoin.amos.op.core.util.CommonResponseUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Sort;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * <pre>
 * 接口数据控制器
 * </pre>
 *
 * @author as-shibaobao
 * @version $Id: IntfcController.java, v 0.1 2018年1月26日 下午3:33:54 as-shibaobao Exp $
 */
@RestController
@RequestMapping("/intfc")
@Api(value = "intfc", tags = {"接口数据管理"})
public class IntfcController extends BaseController {

    @Autowired
    private IIntfcService intfcService;

    @Autowired
    private IInterfacePropertyService iInterfacePropertyService;

    @Autowired
    private IEqpTmplInterfaceService eqpTmplInterfaceService;

    @Autowired
    private IEquipmentService equipmentService;

    /**
     * <pre>
     * 新增接口数据（基础数据和接口属性）/编辑接口基础数据
     * </pre>
     *
     * @param intfc 接口对象
     * @return CommonResponse
     * @throws IntfcException
     */
    @ApiOperation(httpMethod = "POST", value = "保存接口数据（基础数据和接口属性）/编辑接口基础数据", notes = "保存接口数据（基础数据和接口属性）/编辑接口基础数据")
    @RequestMapping(value = "/add", produces = "application/json;charset=UTF-8", method = RequestMethod.POST)
    public CommonResponse saveBasicInfoAndAttrs(
            @ApiParam(value = "接口对象", required = true) @RequestBody Intfc intfc) throws IntfcException {
        intfc.setOrgCode(getCompanyOrgCode());

        //通过实体id是否为空区分新增还是修改，为空则为新增，否则为修改
        intfc = intfcService.saveBasicInfoAndAttrs(intfc);
        return CommonResponseUtil.success(intfc);
    }

    /**
     * <pre>
     * 删除接口
     * </pre>
     *
     * @param id 接口ID
     * @return CommonResponse
     * @throws IntfcException
     */
    @ApiOperation(httpMethod = "DELETE", value = "删除接口 ", notes = "删除接口 ")
    @RequestMapping(value = "/delete/{id}", produces = "application/json;charset=UTF-8", method = RequestMethod.DELETE)
    public CommonResponse delIntfc(
            @ApiParam(value = "接口ID", required = true) @PathVariable Long id) throws IntfcException {
        intfcService.delIntfc(id);
        return CommonResponseUtil.success();
    }

    /**
     * <pre>
     * 分页查询接口数据（基础数据）
     * </pre>
     *
     * @param requests （查询条件，可以为空）
     * @param pageable （分页参数，默认pageNumber=0&pageSize=10）
     * @return CommonResponse
     */
    @ApiOperation(httpMethod = "POST", value = "分页查询接口数据（基础数据）", notes = "分页查询接口数据（基础数据）")
    @RequestMapping(value = "/list", produces = "application/json;charset=UTF-8", method = RequestMethod.POST)
    public CommonResponse queryIntfc(
            @ApiParam(value = "查询参数", required = true) @RequestBody(required = false) List<CommonRequest> requests,
            @ApiParam(value = "分页对象", required = true, defaultValue = "pageNumber=0&pageSize=10") CommonPageable pageable) {
        List<Intfc> result = new ArrayList<Intfc>();
        //如果查询参数中无visibility且当前用户ID不为1（superadmin），则需要查询公共接口
        boolean isquery = true;
        boolean isPublic = true;
        if (requests != null) {
            for (CommonRequest commonRequest : requests) {
                if (commonRequest.getName() != null && commonRequest.getName().equals("visibility")) {
                    isquery = false;
                    isPublic = commonRequest.getValue().equals("0");
                    break;
                }
            }
        }
        if (isquery && !"SUPER".equals(getUserInfo().getUserId())) {
            List<DaoCriteria> daoCriterias = new ArrayList<DaoCriteria>();
            DaoCriteria visDaoCriteria = new DaoCriteria();
            visDaoCriteria.setPropertyName("visibility");
            visDaoCriteria.setOperator(QueryOperatorEnum.EQUAL.getName());
            visDaoCriteria.setValue(0);
            daoCriterias.add(visDaoCriteria);
            daoCriterias.add(new DaoCriteria("orgCode", QueryOperatorEnum.LIKE.getName(), getOrgCode() + "%"));
            result.addAll(intfcService.findAll(daoCriterias));
        }
        //根据条件查询
        List<DaoCriteria> criterias = buildDaoCriterias(requests);
        if (!isPublic) {
            criterias.add(new DaoCriteria("orgCode", QueryOperatorEnum.LIKE.getName(), getOrgCode() + "%"));
        }
        criterias.add(new DaoCriteria("createTime", QueryOperatorEnum.ORDER_BY.getName(), "DESC"));
        result.addAll(intfcService.findAll(criterias));
        return CommonResponseUtil.success(result);
    }

    /**
     * <pre>
     * 根据接口ID分页查询接口详情数据（接口属性）
     * </pre>
     *
     * @param pageable
     * @return
     */
    @ApiOperation(httpMethod = "POST", value = "根据接口ID分页查询接口详情数据（接口属性）", notes = "根据接口ID分页查询接口详情数据（接口属性）")
    @RequestMapping(value = "/attributes/{id}", produces = "application/json;charset=UTF-8", method = RequestMethod.POST)
    public CommonResponse queryIntfcAttributeList(
            @ApiParam(value = "接口ID", required = true) @PathVariable("id") Long id,
            @ApiParam(value = "分页对象", required = true, defaultValue = "pageNumber=0&pageSize=10") CommonPageable pageable) {
        List<DaoCriteria> criterias = new ArrayList<>();
        criterias.add(new DaoCriteria("intfcId", QueryOperatorEnum.EQUAL.getName(), id));
        pageable.setSort(Sort.unsorted());
        Page<IntfcAttribute> result = intfcService.queryIntfcAttributeList(criterias, pageable);
        
        return CommonResponseUtil.success(new PageImpl<IntfcAttribute>(result.getContent(), pageable, result.getTotalElements()));
    }

    /**
     * <pre>
     * 编辑接口属性
     * </pre>
     *
     * @param id             接口ID
     * @param intfcAttribute 接口属性对象
     * @throws IntfcException
     * @returnCommonResponse
     */
    @ApiOperation(httpMethod = "POST", value = "编辑接口属性", notes = "编辑接口属性")
    @RequestMapping(value = "/{id}/attribute/edit", produces = "application/json;charset=UTF-8", method = RequestMethod.POST)
    public CommonResponse editAttribute(
            @ApiParam(value = "接口ID", required = true) @PathVariable("id") Long id,
            @ApiParam(value = "接口属性对象", required = true) @RequestBody IntfcAttribute intfcAttribute) throws IntfcException {
        intfcAttribute.setIntfcId(id);

        intfcAttribute = intfcService.editAttribute(intfcAttribute);
        return CommonResponseUtil.success(intfcAttribute, "编辑成功！");
    }

    /**
     * <pre>
     * 删除接口属性
     * </pre>
     *
     * @return CommonResponse
     * @throws IntfcException
     */
    @ApiOperation(httpMethod = "DELETE", value = "删除接口属性 ", notes = "删除接口属性 ")
    @RequestMapping(value = "/attribute/delete/{id}", produces = "application/json;charset=UTF-8", method = RequestMethod.DELETE)
    public CommonResponse delAttribute(
            @ApiParam(value = "接口属性ID", required = true) @PathVariable Long id) throws IntfcException {
        iInterfacePropertyService.delOneAttribute(id);
        return CommonResponseUtil.success();
    }

    @ApiOperation(httpMethod = "GET", value = "接口统计", notes = "接口统计")
    @RequestMapping(value = "/statistics", produces = "application/json;charset=UTF-8", method = RequestMethod.GET)
    public com.yeejoin.amos.op.core.common.response.CommonResponse countIntfcInfo() {
        List<DaoCriteria> daoCriteria = buildDaoCriteriasNoParam();
        Map<String, Long> intfcCount = intfcService.countIntfcInfo(daoCriteria);
        return CommonResponseUtil.success(intfcCount);
    }

    @ApiOperation(httpMethod = "GET", value = "查询设备类型下接口", notes = "查询设备类型下接口")
    @RequestMapping(value = "/{eqpTemplId}/intfcs", produces = "application/json;charset=UTF-8", method = RequestMethod.GET)
    public CommonResponse fetchIntfcList(
            @ApiParam(value = "设备类型ID", required = true) @PathVariable Long eqpTemplId) {
        List<DaoCriteria> daoCriterias = new ArrayList<DaoCriteria>();
        DaoCriteria eqpTmplDaoCriteria = new DaoCriteria();
        eqpTmplDaoCriteria.setPropertyName("eqpTmplId");
        eqpTmplDaoCriteria.setOperator(QueryOperatorEnum.EQUAL.getName());
        eqpTmplDaoCriteria.setValue(eqpTemplId);
        daoCriterias.add(eqpTmplDaoCriteria);
        List<EqpTmplInterface> eqpTmplInterfaces = eqpTmplInterfaceService.findAll(daoCriterias);
        List<Intfc> intfcs = new ArrayList<Intfc>();
        for (EqpTmplInterface eqpTmplInterface : eqpTmplInterfaces) {
            Intfc intfc = intfcService.findOne(eqpTmplInterface.getIntfcId());
            intfcs.add(intfc);
        }
        return CommonResponseUtil.success(intfcs);
    }

    @SuppressWarnings("unchecked")
    @ApiOperation(httpMethod = "GET", value = "查询设备下接口", notes = "查询设备下接口")
    @RequestMapping(value = "/{eqpTemplId}/{eqpId}/intfcs", produces = "application/json;charset=UTF-8", method = RequestMethod.GET)
    public CommonResponse fetchIntfcList(
            @ApiParam(value = "设备类型ID", required = true) @PathVariable Long eqpTemplId,
            @ApiParam(value = "设备ID", required = true) @PathVariable Long eqpId) {
        //获取设备支持的接口
        List<Long> intfcIdsList = null;
        if (eqpId != 0) {
            intfcIdsList = new ArrayList<Long>();
            Equipment equipment = equipmentService.findOne(eqpId);
            Map<String, Object> valueMap = equipment.getCfgParamMap();
            Object obj = valueMap.get("selIntfcIds");
            if (obj != null) {
                intfcIdsList = (List<Long>) obj;
            }
        }
        //获取设备模板下的所有接口
        List<DaoCriteria> daoCriterias = new ArrayList<DaoCriteria>();
        DaoCriteria eqpTmplDaoCriteria = new DaoCriteria();
        eqpTmplDaoCriteria.setPropertyName("eqpTmplId");
        eqpTmplDaoCriteria.setOperator(QueryOperatorEnum.EQUAL.getName());
        eqpTmplDaoCriteria.setValue(eqpTemplId);
        daoCriterias.add(eqpTmplDaoCriteria);
        List<EqpTmplInterface> eqpTmplInterfaces = eqpTmplInterfaceService.findAll(daoCriterias);
        List<Intfc> intfcs = new ArrayList<Intfc>();
        for (EqpTmplInterface eqpTmplInterface : eqpTmplInterfaces) {
            Intfc intfc = intfcService.findOne(eqpTmplInterface.getIntfcId());
            if (intfcIdsList != null)//编辑或者查看,需要填充设备对应接口是否启用
            {
                if (!intfcIdsList.contains(eqpTmplInterface.getIntfcId())) {
                    intfc.setIsUser(false);
                }
            }
            intfcs.add(intfc);
        }

        return CommonResponseUtil.success(intfcs);
    }
}
