/**
 *
 */
package com.yeejoin.amos.iot.business.controller;

import com.yeejoin.amos.iot.business.entity.MetricData;
import com.yeejoin.amos.iot.business.entity.mybatis.MetricDataBo;
import com.yeejoin.amos.iot.business.entity.mybatis.MetricDataVo;
import com.yeejoin.amos.iot.business.param.MetricDataPageParam;
import com.yeejoin.amos.iot.business.service.intfc.IMetricDataService;
import com.yeejoin.amos.iot.business.service.intfc.IProtocolService;
import com.yeejoin.amos.iot.core.common.request.CommonPageable;
import com.yeejoin.amos.iot.core.common.response.CommonPage;
import com.yeejoin.amos.iot.core.enums.QueryOperatorEnum;
import com.yeejoin.amos.op.core.common.query.DaoCriteria;
import com.yeejoin.amos.op.core.common.response.CommonResponse;
import com.yeejoin.amos.op.core.util.CommonResponseUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Random;

/**
 * 指标数据控制器
 *
 * @author as-youjun
 */
@RestController
@RequestMapping("/mericdata")
@Api(value = "mericdata", tags = {"指标数据管理"})
public class MetricDataController extends BaseController {

    @Autowired
    private IMetricDataService metricDataService;

    @Autowired
    private IProtocolService protocolService;

    private Random random = new Random(100);

    @ApiOperation(httpMethod = "GET", value = "测试保存指标数据", notes = "测试保存指标数据")
    @RequestMapping(value = "/testSave", produces = "application/json;charset=UTF-8", method = RequestMethod.GET)
    public CommonResponse testSave() {
        MetricData metricData = new MetricData();
        long longValue = random.nextLong();
        if (longValue < 0) {
            longValue = 0 - longValue;
        } else if (longValue == 0) {
            longValue = 1;
        }
        longValue = random.nextLong();
        if (longValue < 0) {
            longValue = 0 - longValue;
        } else if (longValue == 0) {
            longValue = 1;
        }
        metricData.setCpblId(longValue);
        longValue = random.nextLong();
        if (longValue < 0) {
            longValue = 0 - longValue;
        } else if (longValue == 0) {
            longValue = 1;
        }
        metricData.setEqpId(longValue);
        longValue = random.nextLong();
        if (longValue < 0) {
            longValue = 0 - longValue;
        } else if (longValue == 0) {
            longValue = 1;
        }
        metricData.setEqpTmplId(longValue);
        metricData.setExeTime(new Date());
        metricData.setIsDelete(false);
        metricDataService.save(metricData);
        return CommonResponseUtil.success(metricData);
    }

    @ApiOperation(httpMethod = "POST", value = "分页查询指标数据", notes = "分页查询指标数据")
    @RequestMapping(value = "/pageMetricData", produces = "application/json;charset=UTF-8", method = RequestMethod.POST)
    public CommonPage<MetricDataVo> pageMetricData(
            @ApiParam(value = "查询参数", required = true) @RequestBody MetricDataPageParam request,
            @ApiParam(value = "分页对象", required = true, defaultValue = "pageNumber=0&pageSize=10") CommonPageable pageable) {
        if (pageable == null) {
            pageable = new CommonPageable(1, DEFAULT_PAGE_SIZE);
        }
        request.setPageNumber(pageable.getPageNumber());
        request.setPageSize(pageable.getPageSize());
        request.setOffset(pageable.getPageNumber() * pageable.getPageSize());
        CommonPage<MetricDataVo> page = metricDataService.pageMetricData(request);
        return page;
    }

    @ApiOperation(httpMethod = "GET", value = "获取设备当前能力数据", notes = "获取设备当前能力数据")
    @RequestMapping(value = "/equip/{id}", produces = "application/json;charset=UTF-8", method = RequestMethod.GET)
    public CommonResponse getMetricData(@ApiParam(value = "设备ID", required = true) @PathVariable long id) {
        List<DaoCriteria> daoCriterias = new ArrayList<DaoCriteria>();
        DaoCriteria eqpId = new DaoCriteria();
        eqpId.setPropertyName("eqpId");
        eqpId.setOperator(QueryOperatorEnum.EQUAL.getName());
        eqpId.setValue(id);
        daoCriterias.add(eqpId);

        List<MetricDataBo> list = metricDataService.findAllMetricDataByEquipId(id);
        return CommonResponseUtil.success(list);
    }

    @ApiOperation(httpMethod = "GET", value = "获取设备当前能力数据", notes = "获取设备当前能力数据")
    @RequestMapping(value = "/equip/{id}/{attrId}", produces = "application/json;charset=UTF-8", method = RequestMethod.GET)
    public CommonResponse getMetricData(@ApiParam(value = "设备ID", required = true) @PathVariable long id, @ApiParam(value = "设备能力ID", required = true) @PathVariable long attrId) {
        List<DaoCriteria> daoCriterias = new ArrayList<DaoCriteria>();
        DaoCriteria eqpId = new DaoCriteria();
        eqpId.setPropertyName("eqpId");
        eqpId.setOperator(QueryOperatorEnum.EQUAL.getName());
        eqpId.setValue(id);

        DaoCriteria cpblId = new DaoCriteria();
        cpblId.setPropertyName("cpblId");
        cpblId.setOperator(QueryOperatorEnum.EQUAL.getName());
        cpblId.setValue(attrId);

        daoCriterias.add(eqpId);
        daoCriterias.add(cpblId);

        List<MetricData> list = metricDataService.findAllMetricData(daoCriterias);
        return CommonResponseUtil.success(list);
    }

}
