package com.yeejoin.amos.iot.business.controller;

import com.yeejoin.amos.iot.business.entity.Protocol;
import com.yeejoin.amos.iot.business.exceptions.IntfcException;
import com.yeejoin.amos.iot.business.exceptions.ProtocolException;
import com.yeejoin.amos.iot.business.service.intfc.IProtocolService;
import com.yeejoin.amos.iot.core.common.request.CommonPageable;
import com.yeejoin.amos.iot.core.common.request.CommonRequest;
import com.yeejoin.amos.iot.core.enums.QueryOperatorEnum;
import com.yeejoin.amos.op.core.common.query.DaoCriteria;
import com.yeejoin.amos.op.core.common.response.CommonResponse;
import com.yeejoin.amos.op.core.util.CommonResponseUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * <pre>
 * 协议控制器
 * </pre>
 *
 * @author as-lixiangyang
 * @version $Id: ProtocolController.java, v 0.1 2018年1月29日 上午10:48:00 as-lixiangyang Exp $
 */
@RestController
@RequestMapping("/protocol")
@Api(value = "protocol", tags = {"协议管理"})
public class ProtocolController extends BaseController {

    private final static Logger log = LoggerFactory.getLogger(ProtocolController.class);

    @Autowired
    private IProtocolService protocolService;

    /**
     * <pre>
     * 新增协议/编辑基本信息/编辑脚本信息
     * </pre>
     *
     * @param protocol 协议实体
     * @return CommonResponse
     * @throws ProtocolException
     */
    @ApiOperation(httpMethod = "POST", value = "新增协议", notes = "新增协议")
    @RequestMapping(value = "/save", produces = "application/json;charset=UTF-8", method = RequestMethod.POST)
    public CommonResponse save(@ApiParam(value = "新增协议数据", required = true) @RequestBody Protocol protocol) throws ProtocolException {
        try {
            protocol.setOrgCode(getCompanyOrgCode());

            //通过实体id是否为空区分新增还是修改，为空则为新增，否则为修改
            protocolService.saveProtocol(protocol);
            return CommonResponseUtil.success();
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            return CommonResponseUtil.failure("新增协议数据失败！");
        }
    }

    /**
     * <pre>
     * 分页查询协议数据
     * </pre>
     *
     * @param requests 查询条件集合，可以为空
     * @param pageable 分页参数，默认pageNumber=0&pageSize=10
     * @return CommonResponse
     */
    @ApiOperation(httpMethod = "POST", value = "分页查询协议数据", notes = "分页查询协议数据")
    @RequestMapping(value = "/list", produces = "application/json;charset=UTF-8", method = RequestMethod.POST)
    public CommonResponse queryProtocolList(
            @ApiParam(value = "查询参数", required = true) @RequestBody(required = false) List<CommonRequest> requests,
            @ApiParam(value = "分页对象", required = true, defaultValue = "pageNumber=0&pageSize=10") CommonPageable pageable) {
        List<Protocol> result = new ArrayList<Protocol>();
        //如果查询参数中无visibility且当前用户ID不为1（superadmin），则需要查询公共协议
        boolean isquery = true;
        boolean isPublic = true;
        if (requests != null) {
            for (CommonRequest commonRequest : requests) {
                if (commonRequest.getName() != null && commonRequest.getName().equals("visibility")) {
                    isquery = false;
                    isPublic = commonRequest.getValue().equals("0");
                    break;
                }
            }
        }
        if (isquery && !"SUPER".equals(getUserInfo().getUserId())) {
            List<DaoCriteria> daoCriterias = new ArrayList<DaoCriteria>();
            DaoCriteria visDaoCriteria = new DaoCriteria();
            visDaoCriteria.setPropertyName("visibility");
            visDaoCriteria.setOperator(QueryOperatorEnum.EQUAL.getName());
            visDaoCriteria.setValue(0);
            daoCriterias.add(visDaoCriteria);
//            daoCriterias.add(new DaoCriteria("orgCode", QueryOperatorEnum.LIKE.getName(), getCompanyOrgCode() + "%"));
            result.addAll(protocolService.findAll(daoCriterias));
//            return CommonResponseUtil.success(result);
        }
        //根据条件查询
        List<DaoCriteria> criterias = buildDaoCriterias(requests);
        if (!isPublic) {
            criterias.add(new DaoCriteria("orgCode", QueryOperatorEnum.LIKE.getName(), getCompanyOrgCode() + "%"));
        }
        criterias.add(new DaoCriteria("createTime", QueryOperatorEnum.ORDER_BY.getName(), "DESC"));
        result.addAll(protocolService.findAll(criterias));
        return CommonResponseUtil.success(result);
    }

    /**
     * <pre>
     * 根据协议ID获取详情数据
     * </pre>
     *
     * @param id 协议ID
     * @return CommonResponse
     */
    @ApiOperation(httpMethod = "GET", value = "根据协议ID获取详情数据", notes = "根据协议ID获取详情数据")
    @RequestMapping(value = "/{id}/detail", produces = "application/json;charset=UTF-8", method = RequestMethod.GET)
    public CommonResponse queryProtocolDetail(
            @ApiParam(value = "协议ID", required = true) @PathVariable("id") Long id) {
        Protocol protocol = protocolService.queryById(id);
        return CommonResponseUtil.success(protocol);
    }

    /**
     * <pre>
     * 删除协议
     * </pre>
     *
     * @param id 协议ID
     * @return CommonResponse
     * @throws IntfcException
     */
    @ApiOperation(httpMethod = "DELETE", value = "删除协议 ", notes = "删除协议 ")
    @RequestMapping(value = "/delete/{id}", produces = "application/json;charset=UTF-8", method = RequestMethod.DELETE)
    public CommonResponse delProtocol(
            @ApiParam(value = "协议ID", required = true) @PathVariable Long id) throws IntfcException {
        protocolService.delProtocol(id);
        return CommonResponseUtil.success();
    }

    @ApiOperation(httpMethod = "GET", value = "协议统计", notes = "协议统计")
    @RequestMapping(value = "/statistics", produces = "application/json;charset=UTF-8", method = RequestMethod.GET)
    public CommonResponse countProtocolInfo() {
        List<DaoCriteria> daoCriteria = buildDaoCriteriasNoParam();
        Map<String, Long> intfcCount = protocolService.countProtocolInfo(daoCriteria);
        return CommonResponseUtil.success(intfcCount);
    }
}
