package com.yeejoin.amos.iot.business.controller;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.yeejoin.amos.iot.business.entity.NodeVo;
import com.yeejoin.amos.iot.business.entity.RuleNode;
import com.yeejoin.amos.iot.business.param.NodeVoParam;
import com.yeejoin.amos.iot.business.param.RuleConfigParam;
import com.yeejoin.amos.iot.business.param.RuleDataParam;
import com.yeejoin.amos.iot.business.service.intfc.INodeVoService;
import com.yeejoin.amos.iot.business.service.intfc.IRuleNodeService;
import com.yeejoin.amos.op.core.common.response.CommonResponse;
import com.yeejoin.amos.op.core.common.response.TreeResponse;
import com.yeejoin.amos.op.core.util.CommonResponseUtil;

/**
 * 
 * <pre>
 * 规则部署模块
 * </pre>
 *
 * @author as-chenjiajun
 * @version $Id: RuleDeployController.java, v 0.1 2018年4月10日 下午4:14:17 as-chenjiajun Exp $
 */
@RestController
@RequestMapping("/ruledeploy")
@Api(value = "ruledeploy", tags = { "规则部署模块" })
public class RuleDeployController extends BaseController
{
	@Autowired
	private INodeVoService nodeVoService;
	
	@Autowired
	private IRuleNodeService IRuleNodeService;
	
	@ApiOperation(httpMethod = "POST", value = "新增/编辑规则部署", notes = "新增/编辑规则部署")
	@RequestMapping(value = "/rule-data", produces = "application/json;charset=UTF-8", method = RequestMethod.POST)
	public CommonResponse addEditRuleNodeAndLinks(
			@ApiParam(value = "规则部署参数", required = true) @RequestBody RuleDataParam ruleData)
	{
		nodeVoService.addEditRuleNodeAndLinks(ruleData);
		return CommonResponseUtil.success();
	}

	@ApiOperation(httpMethod = "DELETE", value = "删除node", notes = "删除node")
	@RequestMapping(value = "/nodevo/{id}", produces = "application/json;charset=UTF-8", method = RequestMethod.DELETE)
	public CommonResponse deleteNode(
			@ApiParam(value = "nodeID", required = true) @PathVariable String id)
	{
		nodeVoService.deleteNode(id);
		return CommonResponseUtil.success();
	}

	@ApiOperation(httpMethod = "PUT", value = "编辑node", notes = "编辑node")
	@RequestMapping(value = "/nodevoname", produces = "application/json;charset=UTF-8", method = RequestMethod.PUT)
	public CommonResponse editNodeName(
			@ApiParam(value = "node参数", required = true) @RequestBody NodeVo nodeVo)
	{
		nodeVoService.editNodeName(nodeVo);
		return CommonResponseUtil.success();
	}

	@ApiOperation(httpMethod = "POST", value = "新增node", notes = "新增node")
	@RequestMapping(value = "/nodevo", produces = "application/json;charset=UTF-8", method = RequestMethod.POST)
	public CommonResponse addNode(
			@ApiParam(value = "node参数", required = true) @RequestBody NodeVoParam nodeVo)
	{
		nodeVoService.addNode(nodeVo);
		return CommonResponseUtil.success();
	}

	@ApiOperation(httpMethod = "PUT", value = "编辑node", notes = "编辑node")
	@RequestMapping(value = "/nodevo", produces = "application/json;charset=UTF-8", method = RequestMethod.PUT)
	public CommonResponse editNode(
			@ApiParam(value = "node配置参数", required = true) @RequestBody RuleConfigParam param)
	{
		nodeVoService.editNode(param);
		return CommonResponseUtil.success();
	}

	@ApiOperation(httpMethod = "GET", value = "获取node", notes = "获取node")
	@RequestMapping(value = "/nodevo/{id}", produces = "application/json;charset=UTF-8", method = RequestMethod.GET)
	public CommonResponse getNode(
			@ApiParam(value = "nodeID", required = true) @PathVariable String id)
	{
		RuleConfigParam ruleConfigParam = nodeVoService.fetchNode(id);
		return CommonResponseUtil.success(ruleConfigParam);
	}
	/**
	 * 
	 * <pre>
	 * 获取规则节点元素
	 * </pre>
	 *
	 * @return
	 */
	@ApiOperation(httpMethod = "GET", value = "获取规则节点元素", notes = "获取规则节点元素")
	@RequestMapping(value = "/elements", produces = "application/json;charset=UTF-8", method = RequestMethod.GET)
	public CommonResponse getRuleElements()
	{
		List<RuleNode> ruleNodeList = IRuleNodeService.findAll();
		List<TreeResponse> treeResponseList = new ArrayList<TreeResponse>();
		Map<String, List<TreeResponse>> subTreeRespMap = new HashMap<String, List<TreeResponse>>();
		//转换为树节点
		for (RuleNode ruleNode : ruleNodeList)
		{
			TreeResponse treeResponse = new TreeResponse();
			treeResponse.setKey(ruleNode.getId());
			treeResponse.setValue(ruleNode.getValue());
			treeResponse.setLabel(ruleNode.getDisplayName());
			treeResponse.setType(ruleNode.getType());
			treeResponse.setIcon(ruleNode.getIcon());
			treeResponse.setObject(ruleNode);
			if (ruleNode.getParentId() == null)
			{
				treeResponseList.add(treeResponse);
			}else 
			{
				List<TreeResponse> treeResponses = subTreeRespMap.get(ruleNode.getParentId());
				if (treeResponses == null)
				{
					treeResponses = new ArrayList<TreeResponse>();
				}
				treeResponses.add(treeResponse);
				subTreeRespMap.put(ruleNode.getParentId(), treeResponses);
			}
		}
		//填充子节点
		for (TreeResponse treeResponse : treeResponseList)
		{
			treeResponse.setChildren(subTreeRespMap.get(treeResponse.getKey()));
		}
		return CommonResponseUtil.success(treeResponseList);
	}
}
