package com.yeejoin.amos.iot.business.controller;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.yeejoin.amos.iot.business.entity.Rule;
import com.yeejoin.amos.iot.business.param.RuleDataParam;
import com.yeejoin.amos.iot.business.param.RulePageParam;
import com.yeejoin.amos.iot.business.service.intfc.IRuleService;
import com.yeejoin.amos.iot.core.common.response.CommonPage;
import com.yeejoin.amos.op.core.common.response.CommonResponse;
import com.yeejoin.amos.op.core.util.CommonResponseUtil;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import io.swagger.annotations.Authorization;

/**
 * 
 * <pre>
 * 规则库管理模块
 * </pre>
 *
 * @author as-chenjiajun
 * @version $Id: RuleManageController.java, v 0.1 2018年4月10日 下午4:13:57 as-chenjiajun Exp $
 */
@RestController
@RequestMapping("/rulemanage")
@Api(value = "rulemanage", tags = { "规则库管理模块" })
public class RuleManageController extends BaseController
{

	@Autowired
	private IRuleService ruleService;

	@ApiOperation(httpMethod = "GET", value = "根据ID查询规则", notes = "根据ID查询规则")
	@RequestMapping(value = "/rule/{id}", produces = "application/json;charset=UTF-8", method = RequestMethod.GET)
	public CommonResponse fetchRuleEntity(
			@ApiParam(value = "ID", required = true, defaultValue = "1") @PathVariable String id)
	{
		RuleDataParam result = ruleService.fetchRuleDataByID(id);
		return CommonResponseUtil.success(result);
	}

	@ApiOperation(httpMethod = "POST", value = "查询规则数据", notes = "查询规则数据")
	@RequestMapping(value = "/rule-list", produces = "application/json;charset=UTF-8", method = RequestMethod.POST)
	public CommonResponse fetchRuleEntityList(
			@ApiParam(value = "查询参数", required = true) @RequestBody RulePageParam request)
	{
		String orgCode = request.getOrgCode();
		//如果过滤条件为空，则查询登录用户及子用户的数据
		if (orgCode == null)
		{
			request.setOrgCode(getCompanyOrgCode());
		}
		CommonPage<Rule> page = ruleService
				.pageRuleData(request);
		return CommonResponseUtil.success(page);
	}

	@ApiOperation(httpMethod = "POST", value = "保存规则", notes = "保存规则")
	@RequestMapping(value = "/rule", produces = "application/json;charset=UTF-8", method = RequestMethod.POST)
	public CommonResponse saveRuleEntity(
			@ApiParam(value = "规则对象", required = true) @RequestBody Rule rule)
	{
		try {
			rule = ruleService.saveRuleEntity(rule);
			return CommonResponseUtil.success(rule);
		} catch (Exception e) {
			return CommonResponseUtil.failure(getErrorMessage(e));
		}
	}

	@ApiOperation(httpMethod = "PUT", value = "编辑规则", notes = "编辑规则")
	@RequestMapping(value = "/rule", produces = "application/json;charset=UTF-8", method = RequestMethod.PUT)
	public CommonResponse editRuleEntity(
			@ApiParam(value = "规则对象", required = true) @RequestBody Rule rule)
	{
		try {
			rule = ruleService.editRuleEntity(rule);
			return CommonResponseUtil.success(rule);
		} catch (Exception e) {
			return CommonResponseUtil.failure(getErrorMessage(e));
		}
	}

	@ApiOperation(httpMethod = "PUT", value = "启用/停用规则", notes = "启用/停用规则")
	@RequestMapping(value = "/rule-enabled/{id}", produces = "application/json;charset=UTF-8", method = RequestMethod.PUT)
	public CommonResponse startOrStopRule(
			@ApiParam(value = "规则id集合", required = true) @PathVariable String id)
	{
		ruleService.startOrStopRule(id);
		return CommonResponseUtil.success();
	}

	@ApiOperation(httpMethod = "PUT", value = "启用/停用全部规则", notes = "启用/停用全部规则")
	@RequestMapping(value = "/all-rule-enabled/{status}", produces = "application/json;charset=UTF-8", method = RequestMethod.PUT)
	public CommonResponse startOrStopRules(
			@ApiParam(value = "规则id集合", required = true) @PathVariable Boolean status)
	{
		ruleService.startOrStopRules(status, getCompanyOrgCode());
		return CommonResponseUtil.success();
	}

	@ApiOperation(httpMethod = "DELETE", value = "删除规则", notes = "删除规则")
	@RequestMapping(value = "/rule/{id}", produces = "application/json;charset=UTF-8", method = RequestMethod.DELETE)
	public CommonResponse deleteRule(
			@ApiParam(value = "设备ID", required = true) @PathVariable String id)
	{
		ruleService.deleteRule(id);
		return CommonResponseUtil.success();
	}

}
