package com.yeejoin.amos.iot.business.controller;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.yeejoin.amos.iot.business.constants.SystemDicType;
import com.yeejoin.amos.iot.business.dao.repository.IDicDao;
import com.yeejoin.amos.iot.business.entity.Dic;
import com.yeejoin.amos.iot.business.enums.AttributeType;
import com.yeejoin.amos.iot.business.enums.CapabilityType;
import com.yeejoin.amos.iot.business.enums.DataType;
import com.yeejoin.amos.iot.business.enums.Visibility;
import com.yeejoin.amos.op.core.common.response.CommonResponse;
import com.yeejoin.amos.op.core.util.CommonResponseUtil;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;


/**
 * 
 * <pre>
 * 枚举、系统字典控制器
 * </pre>
 *
 * @author as-shibaobao
 * @version $Id: SysDicController.java, v 0.1 2018年1月29日 上午11:31:48 as-shibaobao Exp $
 */
@RestController
@RequestMapping("/dic")
@Api(value = "dic", tags = { "系统字典数据、枚举管理" })
public class SysDicController extends BaseController {

	/**
	 * 系统字典无业务逻辑，直接访问DAO
	 */
	@Autowired
	private IDicDao dicDao;


	
	/**
	 * 
	 * <pre>
	 * 获取设备原型行业种类
	 * </pre>
	 *
	 * @return
	 */
	@ApiOperation(httpMethod = "GET", value = "获取设备原型行业种类", notes = "获取设备原型行业种类")
	@RequestMapping(value = "/equiptemplate-industry", produces = "application/json;charset=UTF-8", method = RequestMethod.GET)
	public CommonResponse queryEquipTemplateIndustry() {
		List<Dic> comms = dicDao.findByType(SystemDicType.EquipTemplateIndustry);
		return CommonResponseUtil.success(comms);
	}
	
	/**
	 * 
	 * <pre>
	 * 获取通讯方式
	 * </pre>
	 *
	 * @return
	 */
	@ApiOperation(httpMethod = "GET", value = "获取通讯方式", notes = "获取通讯方式")
	@RequestMapping(value = "/community", produces = "application/json;charset=UTF-8", method = RequestMethod.GET)
	public CommonResponse queryCommunityProtocol() {
		List<Dic> comms = dicDao.findByType(SystemDicType.CommunityProtocol);
		return CommonResponseUtil.success(comms);
	}

	/**
	 * 
	 * <pre>
	 * 获取数据类型
	 * </pre>
	 *
	 * @return
	 */
	@ApiOperation(httpMethod = "GET", value = "获取数据类型", notes = "获取数据类型")
	@RequestMapping(value = "/data-type", produces = "application/json;charset=UTF-8", method = RequestMethod.GET)
	public CommonResponse queryDataType() {
		List<Map<String,Object>> list = new ArrayList<>();
		for (DataType dt : DataType.values())
		{
			Map<String,Object> m = new HashMap<>();
			
			m.put("code", dt.getValue());
			m.put("name", dt.getName());
			
			list.add(m);
		}
		return CommonResponseUtil.success(list);
	}
	
	/**
	 * 
	 * <pre>
	 * 获取属性类型
	 * </pre>
	 *
	 * @return
	 */
	@ApiOperation(httpMethod = "GET", value = "获取属性类型", notes = "获取属性类型")
	@RequestMapping(value = "/attr-type", produces = "application/json;charset=UTF-8", method = RequestMethod.GET)
	public CommonResponse queryAttributeType() {
		List<Map<String,Object>> list = new ArrayList<>();
		for (AttributeType at : AttributeType.values())
		{
			Map<String,Object> m = new HashMap<>();
			
			m.put("code", at.getValue());
			m.put("name", at.getName());
			
			list.add(m);
		}
		return CommonResponseUtil.success(list);
	}
	
	/**
	 * 
	 * <pre>
	 * 获取可见枚举性
	 * </pre>
	 *
	 * @return
	 */
	@ApiOperation(httpMethod = "GET", value = "获取可见枚举性", notes = "获取可见枚举性")
	@RequestMapping(value = "/visibility", produces = "application/json;charset=UTF-8", method = RequestMethod.GET)
	public CommonResponse queryVisibility() {
		List<Map<String,Object>> list = new ArrayList<>();
		for (Visibility vis : Visibility.values())
		{
			Map<String,Object> m = new HashMap<>();
			
			m.put("code", vis.getValue());
			m.put("name", vis.getName());
			
			list.add(m);
		}
		return CommonResponseUtil.success(list);
	}
	
	
	/**
	 * 
	 * <pre>
	 * 获取脚本语言类型
	 * </pre>
	 *
	 * @return
	 */
	@ApiOperation(httpMethod = "GET", value = "获取脚本语言类型", notes = "获取脚本语言类型")
	@RequestMapping(value = "/script-type", produces = "application/json;charset=UTF-8", method = RequestMethod.GET)
	public CommonResponse queryScriptLanguage() {
		List<Dic> scripts = dicDao.findByType(SystemDicType.ScriptLang);
		return CommonResponseUtil.success(scripts);
	}
	
	/**
	 * 
	 * <pre>
	 * 获取能力分类
	 * </pre>
	 *
	 * @return
	 */
	@ApiOperation(httpMethod = "GET", value = "获取能力分类", notes = "获取能力分类")
	@RequestMapping(value = "/capability-type", produces = "application/json;charset=UTF-8", method = RequestMethod.GET)
	public CommonResponse queryCapabilityType() {
		List<Map<String,Object>> list = new ArrayList<>();
		for (CapabilityType ct : CapabilityType.values())
		{
			Map<String,Object> m = new HashMap<>();
			
			m.put("code", ct.getValue());
			m.put("name", ct.getName());
			
			list.add(m);
		}
		return CommonResponseUtil.success(list);
	}
}
