package com.yeejoin.amos.iot.business.controller;

import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.yeejoin.amos.iot.business.entity.Equipment;
import com.yeejoin.amos.iot.business.entity.Subgraph;
import com.yeejoin.amos.iot.business.param.OrgImgParam;
import com.yeejoin.amos.iot.business.param.SubgraphParam;
import com.yeejoin.amos.iot.business.param.TopoNodeParam;
import com.yeejoin.amos.iot.business.service.intfc.IOrgImgRelationshipsService;
import com.yeejoin.amos.iot.business.service.intfc.ITopoConnectionService;
import com.yeejoin.amos.iot.business.service.intfc.ITopoNodeService;
import com.yeejoin.amos.iot.business.service.intfc.ITopoService;
import com.yeejoin.amos.iot.core.common.datastructure.TreeNode;
import com.yeejoin.amos.op.core.common.response.CommonResponse;
import com.yeejoin.amos.op.core.util.CommonResponseUtil;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;

/**
 * 
 * <pre>
 * 拓扑管理
 * </pre>
 *
 * @author as-chenjiajun
 * @version $Id: TopoController.java, v 0.1 2018年3月8日 下午1:33:13 as-chenjiajun Exp $
 */
@RestController
@RequestMapping("/topo")
@Api(value = "topo", tags = { "拓扑管理" })
public class TopoController extends BaseController
{
	@Autowired
	private ITopoService iTopoService;
	
	@Autowired
	private IOrgImgRelationshipsService iOrgImgRelationshipsService;
	
	@Autowired
	private ITopoConnectionService iTopoConnectionService;
	
	@Autowired
	private ITopoNodeService iTopoNodeService;
	
	@ApiOperation(httpMethod = "POST", value = "新增/编辑子图", notes = "新增/编辑子图")
	@RequestMapping(value = "/subgraph-persistence", produces = "application/json;charset=UTF-8", method = RequestMethod.POST)
	public CommonResponse addSubgraph(
			@ApiParam(value = "子图参数", required = true) @RequestBody Subgraph subgraph) {
		Long subgraphID = iTopoService.addEditSubgraph(subgraph,getCompanyOrgCode());
		return CommonResponseUtil.success(subgraphID);
	}
	
	@ApiOperation(httpMethod = "POST", value = "新增/编辑子图", notes = "新增/编辑子图")
	@RequestMapping(value = "/subgraph-data-persistence", produces = "application/json;charset=UTF-8", method = RequestMethod.POST)
	public CommonResponse addSubgraphNodeAndConns(
			@ApiParam(value = "子图参数", required = true) @RequestBody SubgraphParam subgraph) {
		Long subgraphID = iTopoService.addEditSubgraphData(subgraph);
		return CommonResponseUtil.success(subgraphID);
	}
	
	@ApiOperation(httpMethod = "DELETE", value = "删除子图", notes = "删除子图")
	@RequestMapping(value = "/subgraph-eliminate/{id}", produces = "application/json;charset=UTF-8", method = RequestMethod.DELETE)
	public CommonResponse deleteSubgraph(
			@ApiParam(value = "视图主键ID", required = true) @PathVariable Long id) {
		iTopoService.delSubgraph(id);
		return CommonResponseUtil.success();
	}
	
	@ApiOperation(httpMethod = "GET", value = "根据公司id获取视图数据", notes = "根据公司id获取视图数据")
	@RequestMapping(value = "/subgraph-orgcode/{orgCode}", produces = "application/json;charset=UTF-8", method = RequestMethod.GET)
	public CommonResponse getSubviewByOrgCode(@ApiParam(value = "orgCode", required = true, defaultValue = "1")
	@PathVariable String orgCode) {
		List<Subgraph> subgraphList = iTopoService.getSubviewByOrgCode(orgCode);
		return CommonResponseUtil.success(subgraphList);
	}
	
	@ApiOperation(httpMethod = "GET", value = "查询子图数据", notes = "查询子图数据")
	@RequestMapping(value = "/subgraph-entity/{id}", produces = "application/json;charset=UTF-8", method = RequestMethod.GET)
	public CommonResponse fetchSubgraphEntity(@ApiParam(value = "ID", required = true, defaultValue = "1")
	@PathVariable Long id) {
		Subgraph subgraph = iTopoService.getSubgraphById(id);
		return CommonResponseUtil.success(subgraph);
	}
	
	@ApiOperation(httpMethod = "POST", value = "查询子图数据", notes = "查询子图数据")
	@RequestMapping(value = "/subgraph-entitys", produces = "application/json;charset=UTF-8", method = RequestMethod.POST)
	public CommonResponse fetchSubgraphInfos(@RequestBody SubgraphParam param) {
		param.setOrgCode(getCompanyOrgCode());
//		param.setIsDirector(isDirector());
		List<TreeNode> subgraphs = iTopoService.getSubgraphsByCondition(param);
		return CommonResponseUtil.success(subgraphs);
	}
	
	@ApiOperation(httpMethod = "POST", value = "设备名称编辑", notes = "设备名称编辑")
	@RequestMapping(value = "/device-name-update", produces = "application/json;charset=UTF-8", method = RequestMethod.POST)
	public CommonResponse updatedeviceBasicInfo(
			@ApiParam(value = "设备名称", required = true) @RequestBody TopoNodeParam param) {
		Long result = iTopoService.updateDeviceName(param);
		return CommonResponseUtil.success(result);
	}
	
	@ApiOperation(httpMethod = "DELETE", value = "删除节点", notes = "删除节点")
	@RequestMapping(value = "/node-eliminate/{id}", produces = "application/json;charset=UTF-8", method = RequestMethod.DELETE)
	public CommonResponse deleteDevice(
			@ApiParam(value = "节点ID", required = true) @PathVariable Long id) {
		iTopoNodeService.delNodeById(id);
		return CommonResponseUtil.success();
	}
	
	@ApiOperation(httpMethod = "DELETE", value = "删除线路", notes = "删除线路")
	@RequestMapping(value = "/link-eliminate/{id}", produces = "application/json;charset=UTF-8", method = RequestMethod.DELETE)
	public CommonResponse deleteLink(
			@ApiParam(value = "线路ID", required = true) @PathVariable Long id) {
		iTopoConnectionService.delConnection(id);
		return CommonResponseUtil.success();
	}
	
	@ApiOperation(httpMethod = "DELETE", value = "删除线路", notes = "删除线路")
	@RequestMapping(value = "/connection-eliminate/{ids}", produces = "application/json;charset=UTF-8", method = RequestMethod.DELETE)
	public CommonResponse deleteDevice(
			@ApiParam(value = "线路ID集合", required = true) @PathVariable List<Long> ids) {
		iTopoConnectionService.deleteConnections(ids);
		return CommonResponseUtil.success();
	}
	
	@ApiOperation(httpMethod = "GET", value = "查询指定子图下的设备", notes = "查询指定子图下的设备")
	@RequestMapping(value = "/subgraph-equipments/{name}", produces = "application/json;charset=UTF-8", method = RequestMethod.GET)
	public CommonResponse fetchSubgraphEquipments(@PathVariable String name) {
		List<Equipment> equipments = iTopoService.getSubgraphEqpsByName(name,getCompanyOrgCode());
		return CommonResponseUtil.success(equipments);
	}
	
	@ApiOperation(httpMethod = "GET", value = "获取设备最新的一条实时数据", notes = "获取设备最新的一条实时数据")
	@RequestMapping(value = "/real-data/{id}", produces = "application/json;charset=UTF-8", method = RequestMethod.GET)
	public CommonResponse getRealTimeData(@ApiParam(value = "ID", required = true) @PathVariable Long id) {
		Map<String, Object> resultMap = iTopoService.getRealTimeData(id);
		return CommonResponseUtil.success(resultMap);
	}
	
	@ApiOperation(httpMethod = "POST", value = "新增/编辑组织架构背景图片", notes = "新增/编辑组织架构背景图片")
	@RequestMapping(value = "/org-img", produces = "application/json;charset=UTF-8", method = RequestMethod.POST)
	public CommonResponse editOrgImg(
			@ApiParam(value = "编辑参数", required = true) @RequestBody OrgImgParam param)
	{
		iOrgImgRelationshipsService.editOrgImg(param);
		return CommonResponseUtil.success();
	}
	
	@ApiOperation(httpMethod = "GET", value = "根据orgid查询图片", notes = "根据orgid查询图片")
	@RequestMapping(value = "/imgs/{id}", produces = "application/json;charset=UTF-8", method = RequestMethod.GET)
	public CommonResponse fetchImgsEntity(
			@ApiParam(value = "ID", required = true, defaultValue = "1") @PathVariable String id)
	{
		OrgImgParam result = iOrgImgRelationshipsService.fetchImgsEntity(id);
		return CommonResponseUtil.success(result);
	}
	
	@ApiOperation(httpMethod = "GET", value = "根据orgid查询图片", notes = "根据orgid查询图片")
	@RequestMapping(value = "/simple-imgs/{id}", produces = "application/json;charset=UTF-8", method = RequestMethod.GET)
	public CommonResponse fetchSimpleImgsEntity(
			@ApiParam(value = "ID", required = true, defaultValue = "1") @PathVariable String id)
	{
		OrgImgParam result = iOrgImgRelationshipsService.fetchSimpleImgsEntity(id);
		return CommonResponseUtil.success(result);
	}
	
	@ApiOperation(httpMethod = "POST", value = "新增/编辑子图背景图", notes = "新增/编辑子图背景图")
	@RequestMapping(value = "/subgraph-img", produces = "application/json;charset=UTF-8", method = RequestMethod.POST)
	public CommonResponse editSubgraphName(
			@ApiParam(value = "子图参数", required = true) @RequestBody Subgraph subgraph) {
		iTopoService.addEditSubgraphImg(subgraph);
		return CommonResponseUtil.success();
	}
	
	@ApiOperation(httpMethod = "GET", value = "启用或者关闭视图", notes = "启用或者关闭视图")
	@RequestMapping(value = "/rule-switch/{id}/{enableMonitor}", produces = "application/json;charset=UTF-8", method = RequestMethod.GET)
	public CommonResponse oneOpenOrClose(@PathVariable Long id,@PathVariable Boolean enableMonitor) {
		iTopoService.oneOpenOrClose(id,enableMonitor);
		return CommonResponseUtil.success();
	}
	
	@ApiOperation(httpMethod = "GET", value = "查询指定子图下的设备", notes = "查询指定子图下的设备")
	@RequestMapping(value = "/single-subgraph-equipments/{id}", produces = "application/json;charset=UTF-8", method = RequestMethod.GET)
	public CommonResponse fetchSubgraphEquipments(@PathVariable Long id) {
		List<Equipment> equipments = iTopoService.getSubgraphEqpsByID(id);
		return CommonResponseUtil.success(equipments);
	}
	
}
