/**
 * NewHeight.com Inc.
 * Copyright (c) 2008-2010 All Rights Reserved.
 */
package com.yeejoin.amos.iot.business.dao.mapper;

import com.yeejoin.amos.iot.business.entity.mybatis.CapacityInterfaceDataVo;
import com.yeejoin.amos.iot.business.entity.mybatis.CapacityPropertyDataVo;
import com.yeejoin.amos.iot.business.entity.mybatis.DeviceInterfaceDataVo;
import com.yeejoin.amos.iot.business.entity.mybatis.EqpCount;
import com.yeejoin.amos.iot.business.entity.mybatis.EquipmentBo;
import com.yeejoin.amos.iot.business.entity.mybatis.EquipmentTemplateVo;
import com.yeejoin.amos.iot.business.entity.mybatis.EquipmentVo;
import com.yeejoin.amos.iot.business.entity.mybatis.IntfcAttrBo;
import com.yeejoin.amos.iot.business.param.DevicePageParam;
import com.yeejoin.amos.iot.business.param.EqpTemplPageParam;
import com.yeejoin.amos.iot.business.param.QueryEqpCountParam;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;
import java.util.Map;

/**
 * <pre>
 * 设备操作mapper接口
 * </pre>
 *
 * @author as-shibaobao
 * @version $Id: DeviceMapper.java, v 0.1 2018年1月31日 下午3:45:39 as-shibaobao Exp
 * $
 */
@Mapper
public interface DeviceMapper {

    /**
     * <pre>
     * 获取设备总数量
     * </pre>
     *
     * @param param
     * @return
     */
    long countDeviceData(DevicePageParam param);

    /**
     * 获取设备列表
     *
     * @param param
     * @return
     */
    List<EquipmentTemplateVo> pageDeviceData(DevicePageParam param);

    /**
     * <pre>
     * 根据设备ID获取设备接口信息
     * </pre>
     *
     * @param id
     * @return
     */
    List<DeviceInterfaceDataVo> getDeviceInterfaceById(Long id);

    /**
     * <pre>
     * 根据能力id查询能力接口
     * </pre>
     *
     * @param id
     * @return
     */
    List<CapacityInterfaceDataVo> getCapacityInterfaceByCapId(Long capId);

    /**
     * <pre>
     * 根据能力id查询能力属性
     * </pre>
     *
     * @param id
     * @return
     */
    List<CapacityPropertyDataVo> getCapacityPropertyById(Long id);

    /**
     * 查询能力集合对应的能力属性
     *
     * @param cpblIds
     * @return
     */
    List<CapacityPropertyDataVo> getCapacityPropertyByCpblIds(List<Long> cpblIds);

    /**
     * 根据能力属性id集合查询能力属性
     *
     * @param ids
     * @return
     */
    List<CapacityPropertyDataVo> getCapacityPropertyByCpblProIds(List<Long> ids);

    /**
     * 查询接口集合对应的接口属性
     *
     * @param intfcIds
     * @return
     */
    List<IntfcAttrBo> findIntfcAttrByIntfcIds(List<Long> intfcIds);

    List<EquipmentBo> findEqpBySubgraphId(@Param("subgraphId") Long subgraphId, @Param("orgCode") String orgCode);

    /**
     * 查询设备数量
     *
     * @param param
     * @return
     */
    List<EqpCount> findEqpCount(QueryEqpCountParam param);

    /**
     * 查询设备类型
     *
     * @param param
     * @return
     */
    List<EquipmentTemplateVo> getEqpTmpModelByOrgCode(String orgCode);

    /**
     * 分页查询设备
     *
     * @param param
     * @return
     */
    List<EquipmentVo> pageDeviceDataByMybatis(EqpTemplPageParam requestParam);

    /**
     * 分页查询计算总数
     *
     * @param param
     * @return
     */
    long countDeviceDataByMybatis(EqpTemplPageParam requestParam);

    List<EquipmentVo> listByOrgCodeAndTmplId(@Param("orgCode") String orgCode, @Param("eqpTmplId") Long eqpTmplId);

    List<Map> findTempAttr(Long eqpId);

    List<Map> qryEqpCountByTemplAttrs(@Param("orgCode") String orgCode, @Param("attrNames") List<String> attrName);

    List<EquipmentVo> listDeviceAndDeviceCpblAttr(@Param("equipmentId") Long equipmentId);

    List<EquipmentVo> listDeviceCpblAttr(@Param("equipmentId") long equipmentId);
}
