/**
 *
 */
package com.yeejoin.amos.iot.business.dao.mapper;

import com.yeejoin.amos.iot.business.entity.Node;
import com.yeejoin.amos.iot.business.entity.Subgraph;
import com.yeejoin.amos.iot.business.entity.mybatis.*;
import com.yeejoin.amos.iot.business.param.*;
import org.apache.ibatis.annotations.Mapper;

import java.util.List;

/**
 * 采集/控制/诊断等核心业务查询数据mapper
 *
 * @author as-youjun
 */
@Mapper
public interface MonitorMapper {

    /**
     * 查询监测设备数量
     *
     * @param param
     * @return
     */
    long countMonitorEquipment(QueryMonitorEqpParam param);

    /**
     * 查询监测设备
     *
     * @param param
     * @return
     */
    List<EquipmentBo> findMonitorEquipment(QueryMonitorEqpParam param);

    /**
     * 查询设备模板的能力
     *
     * @param param
     * @return
     */
    List<CapabilityBo> findCpblByEqpTmplIds(QueryCapabilityParam param);

    /**
     * 根据设备id查询设备能力
     *
     * @param id
     * @return
     */
    List<CapabilityBo> getCapabilityByDeviceId(Long id);

    /**
     * 查询设备模板的能力的接口
     *
     * @param param
     * @return
     */
    List<IntfcBo> findIntfcOfCpblByEqpTmplIds(QueryIntfcParam param);

    /**
     * @param eqpId
     * @return
     */
    List<EqpNodeSubGraphBo> findEqpNodeGraphByEqpId(Long eqpId);

    /**
     * 查询设备模板的属性集合
     *
     * @param param
     * @return
     */
    List<EqpTmplAttributeBo> findEqpTmplAttrByEqpTmplIds(
            QueryEqpTmplAttrParam param);

    /**
     * 查询设备接口集合
     *
     * @param param
     * @return
     */
    List<EqpTmplIntfcBo> findEqpTmplIntfcByEqpTmplIds(
            QueryEqpTmplIntfcParam param);

    /**
     * 根据设备id查询视图名称
     *
     * @return
     */
    Subgraph getSubgraphByEqbId(Long eqpId);

    /**
     * 根据设备id查询node
     *
     * @return
     */
    Node getNodeByEqbId(Long eqpId);

    /**
     * 查询设备id、节点id、视图id
     *
     * @param eqpId
     * @return
     */
    List<EqpGraphNodeBo> findEqpGraphNodeByEqpId(Long eqpId);

    /**
     * 查询监听端口信息
     *
     * @param param
     * @return
     */
    List<ListenPortBo> findListenPortBo(QueryListenPortParam param);
    /**
     * 查询设备监听端口信息
     *
     * @param param
     * @return
     */
	List<EquipmentBo> findMonitorEquipmentPorts(QueryMonitorEqpParam param);
	/**
     * 查询设备原型监听端口信息
     *
     * @param param
     * @return
     */
	List<ListenPortBo> findTempListenPortBo(QueryListenPortParam param);
    /**
     * 查询监测设备，优化后
     *
     * @param param
     * @return
     */
	List<EquipmentBo> findMonitorEquipmentDet(QueryMonitorEqpParam param);
	/**
     * 查询长连接监测设备
     *
     * @param param
     * @return
     */
    List<EquipmentBo> findLongConMonitorEqp(QueryMonitorEqpParam param);

}
