package com.yeejoin.amos.iot.business.dao.repository;

import java.util.List;

import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import com.yeejoin.amos.iot.business.entity.Dic;
import com.yeejoin.amos.op.core.jpa.IBaseRepository;

/**
 * 
 * <pre>
 * 系统数据字典Dao接口
 * </pre>
 *
 * @author as-shibaobao
 * @version $Id: IDicDao.java, v 0.1 2018年1月26日 下午2:29:53 as-shibaobao Exp $
 */
@Repository("dicDao")
public interface IDicDao extends BaseDao<Dic, Long> {
	default Long findIdByTypeAndCode(String type, String code) {
		Dic systemDic = this.findByTypeAndCode(type, code);
		if (null != systemDic) {
			return systemDic.getId();
		} else {
			return null;
		}

	}

	/**
	 * 根据type，code获取SystemDic
	 * 
	 * @param type
	 * @param code
	 * @return
	 */
	@Query(value = "from Dic where type = ?1 AND code = ?2")
	public Dic findByTypeAndCode(String type, String code);

	/**
	 * 根据type，code获取SystemDic
	 * 
	 * @param type
	 * @param name
	 * @return
	 */
	@Query(value = "from Dic where type = ?1 AND name = ?2")
	public Dic findByTypeAndName(String type, String name);

	/**
	 * 根据type获取SystemDic集合
	 * 
	 * @param type
	 * @return
	 */
	public List<Dic> findByType(String type);

	/**
	 * 根据id查询
	 * 
	 * @param id
	 * @return
	 */
//	public Dic findById(Long id);
	
	@Modifying
	@Transactional
	@Query("UPDATE Dic SET is_delete = 1 WHERE id = ?1")
	void delDicById(Long id);

	List<Dic> findAllByType(String equipTemplateIndustry);
}
