package com.yeejoin.amos.iot.business.dao.repository;

import java.util.Collection;
import java.util.List;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import com.yeejoin.amos.iot.business.entity.Equipment;

/**
 * 设备dao
 * 
 * @author as-youjun
 *
 */
@Repository("equipmentDao")
public interface IEquipmentDao extends BaseDao<Equipment, Long> {
	 @Query(value = "select * from t_equipment e where e.cfg_param_json->'$.sid' = ?1", nativeQuery = true)
	 Equipment findByCfgParamJson(String sid);
	 
	 @Query("SELECT COUNT(*) FROM Equipment WHERE isDelete=false AND (orgCode = ?1 OR orgCode LIKE ?2)")
	 long countByOrgCode(String orgCode, String likeOrgCode);
	 
	 long countByIsDeleteFalseAndIdIn(Collection<Long> ids);
	 
	 @Query("SELECT eqpTmplId FROM Equipment WHERE isDelete=false AND (orgCode = ?1 OR orgCode LIKE ?2)")
	 List<Long> findEqpTempIdsByOrgCode(String orgCode, String likeOrgCode);
	 
	 @Modifying(clearAutomatically = true)
	 @Transactional
	 @Query(value="update t_equipment set is_delete = 1 WHERE id = ?1",nativeQuery = true)
	 int delEquipmentById(Long id);
	 
	 @Query(value = "SELECT e.id as id,e.eqp_tmpl_id as eqp_tmpl_id,e.name as name,eq.model as model,eq.type as type,eq.industry_id as industryid FROM t_node as n INNER JOIN t_equipment as e  inner JOIN t_eqp_tmpl as eq on n.equipment_id = e.id and eq.id = e.eqp_tmpl_id where n.subgraph_id = ?1 and e.org_code = ?2 ORDER BY ?#{#pageable}",
			 countQuery ="SELECT count(e.id) FROM t_node as n INNER JOIN t_equipment as e  inner JOIN t_eqp_tmpl as eq on n.equipment_id = e.id and eq.id = e.eqp_tmpl_id where n.subgraph_id = ?1 and e.org_code = ?2" 
			 ,nativeQuery = true)
	 Page<Object> findEqpBySubgraph(String subgraphId, String orgCode,Pageable pageable);
	 
	/**
	 * 
	 * <pre>
	 * 根据eqpTmplId查询
	 * </pre>
	 *
	 * @param eqpTmplId
	 * @return
	 */
	List<Equipment> findByEqpTmplId(Long eqpTmplId);
	/**
	 * 
	 * <pre>
	 * 根据eqpTmplId查询设备数量
	 * </pre>
	 *
	 * @param eqpTmplId
	 * @return
	 */
	@Query("SELECT COUNT(*) FROM Equipment WHERE isDelete=false AND eqpTmplId = ?1 AND (orgCode = ?2 OR orgCode LIKE ?3)")
	long countByEqpTmplId(Long eqpTmplId, String orgCode, String orgCodeLike);
	
	/**
	 * 
	 * <pre>
	 * 条件查询设备
	 * </pre>
	 *
	 * @param name 设备名称
	 * @param type 设备模板类别
	 * @param industryId 设备模板行业
	 * @return
	 */
	@Query(value = "select e.* from t_equipment as e LEFT JOIN t_eqp_tmpl as ep on e.eqp_tmpl_id = ep.id where (e.name like ?1 or IFNULL(?1,'null')='null')"
			+ " and (ep.type = ?2 or IFNULL(?2,'null')='null')"
			+ " and (ep.industry_id = ?3 or IFNULL(?3,'null')='null')"
			+ " and e.is_delete = false"
			+ " and e.org_code = ?4"
			, nativeQuery = true)
	List<Equipment> getEquipmentByParam(String name,Integer type,Long industryId,String orgCode);
}
