package com.yeejoin.amos.iot.business.entity;

import javax.persistence.Column;
import javax.persistence.MappedSuperclass;
import javax.validation.constraints.NotNull;

import org.hibernate.validator.constraints.NotBlank;

import com.yeejoin.amos.iot.core.util.StringUtil;

/**
 * 基本属性
 * 
 * @author as-youjun
 *
 */
@MappedSuperclass
public class BasicAttribute extends BasicEntity {

	/**
	 * 
	 */
	private static final long serialVersionUID = 7046485114948164091L;

	/**
	 * 属性名称(英文、数字、下划线)
	 */
	private String name;

	/**
	 * 属性分类：1表示变量，2表示常量，3表示标签
	 */
	private Integer type;

	/**
	 * 显示名称
	 */
	private String displayName;

	/**
	 * 计量单位
	 */
	private String unit;

	/**
	 * 描述
	 */
	private String descr;

	/**
	 * 数据类型：1表示整型，2表示浮点型，3表示布尔型，4表示枚举，5表示字符串，6表示密码
	 */
	private Integer dataType;

	/**
	 * 字符串值
	 */
	private String valueStr;

	/**
	 * 数值型值
	 */
	private Double valueNumeric;

	/**
	 * 最小值
	 */
	private Double minValue;

	/**
	 * 最大值
	 */
	private Double maxValue;
	
	/**
	 * float/double类型精度
	 */
	private Integer precision;
	
	/**
	 * 枚举范围，使用逗号分隔
	 */
	private String enumRange;

	@Column(name = "name", length = 100, nullable = false)
	@NotNull(message = "属性名不能为空！")
	@NotBlank(message = "属性名不能为空字符串！")
	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	@Column(name = "type", nullable = false)
	public Integer getType() {
		return type;
	}

	public void setType(Integer type) {
		this.type = type;
	}

	@Column(name = "display_name", length = 100, nullable = false)
	@NotNull(message = "显示名称不能为空！")
	@NotBlank(message = "显示名称不能为空字符串！")
	public String getDisplayName() {
		return displayName;
	}

	public void setDisplayName(String displayName) {
		this.displayName = displayName;
	}

	@Column(name = "unit", length = 20)
	public String getUnit() {
		return unit;
	}

	public void setUnit(String unit) {
		this.unit = unit;
	}

	@Column(name = "descr", length = 512)
	public String getDescr() {
		if (StringUtil.isNotEmpty(descr)) {
			return descr.trim();
		}
		return null;
	}

	public void setDescr(String descr) {
		this.descr = descr;
	}

	@Column(name = "data_type", nullable = false)
	public Integer getDataType() {
		return dataType;
	}

	public void setDataType(Integer dataType) {
		this.dataType = dataType;
	}

	@Column(name = "value_str")
	public String getValueStr() {
		return valueStr;
	}

	public void setValueStr(String valueStr) {
		this.valueStr = valueStr;
	}

	@Column(name = "value_numeric")
	public Double getValueNumeric() {
		return valueNumeric;
	}

	public void setValueNumeric(Double valueNumeric) {
		this.valueNumeric = valueNumeric;
	}

	@Column(name = "min_value")
	public Double getMinValue() {
		return minValue;
	}

	public void setMinValue(Double minValue) {
		this.minValue = minValue;
	}

	@Column(name = "max_value")
	public Double getMaxValue() {
		return maxValue;
	}

	public void setMaxValue(Double maxValue) {
		this.maxValue = maxValue;
	}

	@Column(name="accuracy")
	public Integer getPrecision() {
		return precision;
	}

	public void setPrecision(Integer precision)
	{
		this.precision = precision;
	}

	@Column(name="enum_range",length=200)
	public String getEnumRange()
	{
		return enumRange;
	}

	public void setEnumRange(String enumRange)
	{
		this.enumRange = enumRange;
	}

	
}
