/**
 * 
 */
package com.yeejoin.amos.iot.business.entity;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.persistence.Transient;
import javax.validation.constraints.NotNull;

import org.hibernate.annotations.Fetch;
import org.hibernate.annotations.FetchMode;
import org.hibernate.annotations.Where;
import org.hibernate.validator.constraints.NotBlank;

import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.yeejoin.amos.iot.core.util.StringUtil;

/**
 * 能力
 * 
 * @author as-youjun
 *
 */
@Entity
@Table(name = "t_capability")
@JsonIgnoreProperties(value = { "eqpTmpl" })
@Where(clause="is_delete=0")//表示未删除的数据
public class Capability extends BasicEntity {

	/**
	 * 
	 */
	private static final long serialVersionUID = -2666096014364663424L;

	/**
	 * 名称(英文、数字、下划线)
	 */
	private String name;

	/**
	 * 能力分类：1表示采集；2表示控制；3表示诊断
	 */
	private Integer type;

	/**
	 * 描述
	 */
	private String descr;

	/**
	 * 协议id
	 */
	private Long protocolId;

	/**
	 * 计算公式
	 */
	private String formula;
	
	/**
	 * 监测方式：1表示监听，2表示采集
	 */
	private Integer monitorType = 1;

	/**
	 * 监测周期(秒)
	 */
	private Integer monitorPeriod = 30;

	/**
	 * 设备模板模板
	 */
	private EquipmentTemplate eqpTmpl;
	/**
	 * 设备是否启用，默认启用
	 */
	private Boolean isUser = true;


	public String getMongodb() {
		return mongodb;
	}

	public void setMongodb(String mongodb) {
		this.mongodb = mongodb;
	}

	private String mongodb;

	@Column(name = "name", length = 100, nullable = false)
	@NotNull(message = "设备能力名称不能为空！")
	@NotBlank(message = "设备能力名称不能为空！")
	public String getName() {
		if (StringUtil.isNotEmpty(name)) {
			return name.trim();
		}
		return null;
	}

	public void setName(String name) {
		this.name = name;
	}

	@Column(name = "type", nullable = false)
	public Integer getType() {
		return type;
	}

	public void setType(Integer type) {
		this.type = type;
	}

	@Column(name = "descr", length = 512)
	public String getDescr() {
		if (StringUtil.isNotEmpty(descr)) {
			return descr.trim();
		}
		return null;
	}

	public void setDescr(String descr) {
		this.descr = descr;
	}

	@Column(name = "protocol_id", nullable = false)
	public Long getProtocolId() {
		return protocolId;
	}

	public void setProtocolId(Long protocolId) {
		this.protocolId = protocolId;
	}

	@Column(name = "formula", length = 512)
	public String getFormula() {
		return formula;
	}

	public void setFormula(String formula) {
		this.formula = formula;
	}

	@ManyToOne(targetEntity = EquipmentTemplate.class, fetch = FetchType.LAZY)
	@Fetch(FetchMode.JOIN)
	@JoinColumn(name = "eqp_tmpl_id", referencedColumnName = "id", updatable = false, insertable = true)
	public EquipmentTemplate getEqpTmpl() {
		return eqpTmpl;
	}

	public void setEqpTmpl(EquipmentTemplate eqpTmpl) {
		this.eqpTmpl = eqpTmpl;
	}

	@Transient
	public Boolean getIsUser()
	{
		return isUser;
	}

	public void setIsUser(Boolean isUser)
	{
		this.isUser = isUser;
	}
	
	@Column(name = "monitor_type")
	public Integer getMonitorType() {
		return monitorType;
	}

	public void setMonitorType(Integer monitorType) {
		this.monitorType = monitorType;
	}

	@Column(name = "monitor_period")
	public Integer getMonitorPeriod() {
		return monitorPeriod;
	}

	public void setMonitorPeriod(Integer monitorPeriod) {
		this.monitorPeriod = monitorPeriod;
	}

}
