package com.yeejoin.amos.iot.business.entity;

import io.swagger.annotations.ApiModelProperty;

import java.io.Serializable;
import java.util.List;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.hibernate.annotations.GenericGenerator;
import org.hibernate.annotations.Where;

import com.yeejoin.amos.op.core.entity.BusinessEntity;
import com.yeejoin.amos.op.core.util.Constants;
/**
 * 
 * <pre>
 * drools规则实体
 * </pre>
 *
 * @author as-guowubin
 * @version $Id: DroolsRule.java, v 0.1 2018年4月20日 下午3:42:56 as-guowubin Exp $
 */
@Entity
@Table(name = "t_rm_droolsrule")
@Where(clause="is_delete=0")//表示未删除的数据
public class DroolsRule extends BusinessEntity implements Serializable
{

	private static final long serialVersionUID = -4230791185458128702L;

	@Id
    @GeneratedValue(generator = "uuid")
    @GenericGenerator(name = "uuid", strategy = "uuid2")
	@Column(name = "id", nullable = false, length = 36)
	private String id;
	@ApiModelProperty(notes = "规则名")
	private String name;
	@ApiModelProperty(notes = "drools规则")
	private String droolsRule;
	@ApiModelProperty(notes = "规则组id")
	private String groupId;
	@ApiModelProperty(notes = "所属规则id")
	private String ruleId;
	@ApiModelProperty(notes = "是否启用")
	private Boolean isUser;
	@Transient
	private String packageName;
	@Transient
	private List<String> importList;
	@Transient
	private List<String> conditions;
	@Transient
	private String actionClassName;
	@Transient
	private String bizId;
	//规则对象名（例：com.yeejoin.amos.iot.business.entity.Alarm）
	@Transient
	private String factClass;
	@Transient
	private String ruleStatus;//规则状态

	public String getName() {
		return this.name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getDroolsRule() {
		return this.droolsRule;
	}

	public void setDroolsRule(String droolsRule) {
		this.droolsRule = droolsRule;
	}

	public String getGroupId() {
		return this.groupId;
	}

	public void setGroupId(String groupId) {
		this.groupId = groupId;
	}

	public String getRuleId()
	{
		return ruleId;
	}

	public void setRuleId(String ruleId)
	{
		this.ruleId = ruleId;
	}

	public Boolean getIsUser()
	{
		return isUser;
	}

	public void setIsUser(Boolean isUser)
	{
		this.isUser = isUser;
	}

	public String getPackageName()
	{
		return packageName;
	}

	public void setPackageName(String packageName)
	{
		this.packageName = packageName;
	}

	public List<String> getImportList()
	{
		return importList;
	}

	public void setImportList(List<String> importList)
	{
		this.importList = importList;
	}

	public List<String> getConditions()
	{
		return conditions;
	}

	public void setConditions(List<String> conditions)
	{
		this.conditions = conditions;
	}

	public String getActionClassName()
	{
		return actionClassName;
	}

	public void setActionClassName(String actionClassName)
	{
		this.actionClassName = actionClassName;
	}
	
	public String getBizId()
	{
		return bizId;
	}

	public void setBizId(String bizId)
	{
		this.bizId = bizId;
	}

	public String getFactClass()
	{
		return factClass;
	}

	public void setFactClass(String factClass)
	{
		this.factClass = factClass;
	}

	public String getRuleStatus()
	{
		return ruleStatus;
	}

	public void setRuleStatus(String ruleStatus)
	{
		this.ruleStatus = ruleStatus;
	}

	@Override
	public String toString()
	{
		StringBuilder droolsRule = new StringBuilder("");
		StringBuilder droolsRuleAction = new StringBuilder(
				"List<RuleAIEntity> ruleEntitys = new ArrayList<RuleAIEntity>();"+Constants.NEW_LINE);
		droolsRule.append("package ").append(packageName).append(Constants.NEW_LINE);
		for (String importClass : importList)
		{
			droolsRule.append("import ").append(importClass).append(";"+Constants.NEW_LINE);
		}
		
		droolsRule.append("rule \"").append(name).append("\"" + Constants.NEW_LINE);
		droolsRule.append("salience \"0\" ").append(Constants.NEW_LINE);
		droolsRule.append("when" + Constants.NEW_LINE);
		/**
		 * 条件
		 */
		//类实例名,首字母小写
		String[] strings = getFactClass().split("\\.");
		String instanceName = strings[strings.length-1].toLowerCase();
	    droolsRule.append(instanceName).append(" : ").append(getFactClass()).append("(");
		if(getConditions() != null && getConditions().size()>0)
		{
			for (String condition : getConditions())
			{
				droolsRule.append("(").append(condition).append(")");
				droolsRule.append(Constants.RULE_CONDITION_AND);
			}
			droolsRule.delete(droolsRule.length()-Constants.RULE_CONDITION_AND.length(),droolsRule.length());
		}
		droolsRule.append(")"+Constants.NEW_LINE);
		droolsRuleAction.append("ruleEntitys.add(").append(instanceName).append(");"+Constants.NEW_LINE);
        /**
         * 动作
         */
		droolsRule.append("then" + Constants.NEW_LINE);
		// 获取动作的执行类 
		String actionClass =getActionClassName();
		// 创建一个工作，并插入动作队列中
		if (getBizId() != null && getRuleStatus() != null)
		{
			droolsRuleAction.append("RuleActionQueue.getInstance().add(new ")
			                .append(actionClass).append("(ruleEntitys, '").append(getBizId())
			                .append("', '").append(getRuleStatus()).append("'));");
		}else 
		{
			droolsRuleAction.append(
					"RuleActionQueue.getInstance().add(new "+ actionClass+"(ruleEntitys));");
		}
		droolsRule.append(droolsRuleAction);
		droolsRule.append(Constants.NEW_LINE +"end");
		return droolsRule.toString();
	}
	
	public String getId()
	{
		return id;
	}

	public void setId(String id)
	{
		this.id = id;
	}
}