/**
 * 
 */
package com.yeejoin.amos.iot.business.entity;

import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import javax.persistence.Table;
import javax.persistence.Transient;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;

import org.hibernate.annotations.Fetch;
import org.hibernate.annotations.FetchMode;
import org.hibernate.annotations.Where;
import org.hibernate.validator.constraints.NotBlank;

import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.yeejoin.amos.iot.business.dic.EquipTemplateIndustry;
import com.yeejoin.amos.iot.business.enums.EquipmentType;
import com.yeejoin.amos.iot.core.util.StringUtil;

/**
 * 设备模板
 * 
 * @author as-youjun
 *
 */
@Entity
@Table(name = "t_eqp_tmpl")
@JsonIgnoreProperties({ "hibernateLazyInitializer", "handler" })
@Where(clause="is_delete=0")//表示未删除的数据
public class EquipmentTemplate extends TimeEntity {

	/**
	 * 
	 */
	private static final long serialVersionUID = 5925551407041984662L;

	/**
	 * 名称
	 */
	private String name;

	/**
	 * 型号
	 */
	private String model;

	/**
	 * 设备类别：1表示传输设备，2表示传感设备，3表示控制设备,4其他
	 */
	private Integer type;

	/**
	 * 行业id
	 */
	private Long industryId;

	/**
	 * 所属行业
	 */
	private EquipTemplateIndustry equipTemplateIndustry;

	private String typeImg;

	/**
	 * 图片
	 */
	private String img;

	/**
	 * 描述
	 */
	private String descr;

	/**
	 * 可见性：0表示公共，1表示私有
	 */
	private Integer visibility;

	/**
	 * 用户id
	 */
	private String userId;

	/**
	 * 公司id
	 */
	private String orgCode;

	/**
	 * 视图路径
	 */
	private String viewPath;

	/**
	 * 设备模板序列号:模板类型（public\private）-行业（industryId）-设备类别（type）-自增编号
	 */
	private String serialNum;

	/**
	 * 设备模板属性
	 */
	private List<EqpTmplAttribute> attributes;

	/**
	 * 能力
	 */
	private List<Capability> capabilities;

	@Column(name = "name", length = 100, nullable = false)
	@NotBlank(message = "设备名称不能为空！")
	@NotNull(message = "设备名称不能为空！")
	@Size(min = 1, max = 100, message = "设备名称长度不能超过100")
	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	@Column(name = "model", length = 100)
	@NotBlank(message = "设备型号不能为空！")
	@NotNull(message = "设备型号不能为空！")
	@Size(min = 1, max = 100, message = "设备型号长度不能超过100！")
	public String getModel() {
		return model;
	}

	public void setModel(String model) {
		this.model = model;
	}

	@Column(name = "type", nullable = false)
	public Integer getType() {
		return type;
	}

	public void setType(Integer type) {
		this.type = type;
	}

	@Transient
	public String getTypeName()
	{
		return EquipmentType.getName(type);
	}

	@Column(name = "img", length = 100)
	public String getImg() {
		return img;
	}

	public void setImg(String img) {
		this.img = img;
	}

	@Column(name = "descr", length = 512)
	public String getDescr() {
		if (StringUtil.isNotEmpty(descr)) {
			return descr.trim();
		}
		return null;
	}

	public void setDescr(String descr) {
		this.descr = descr;
	}

	@Column(name = "visibility", nullable = false)
	public Integer getVisibility() {
		return visibility;
	}

	public void setVisibility(Integer visibility) {
		this.visibility = visibility;
	}

	@Column(name = "user_id")
	public String getUserId() {
		return userId;
	}

	public void setUserId(String userId) {
		this.userId = userId;
	}

	@OneToMany(mappedBy = "eqpTmpl", fetch = FetchType.LAZY, cascade = { CascadeType.ALL })
	@Where(clause="is_delete=0")//表示未删除的数据
	public List<EqpTmplAttribute> getAttributes() {
		return attributes;
	}

	public void setAttributes(List<EqpTmplAttribute> attributes) {
		this.attributes = attributes;
	}

	@OneToMany(mappedBy = "eqpTmpl", fetch = FetchType.LAZY, cascade = { CascadeType.ALL })
	@Where(clause="is_delete=0")//表示未删除的数据
	public List<Capability> getCapabilities() {
		return capabilities;
	}

	public void setCapabilities(List<Capability> capabilities) {
		this.capabilities = capabilities;
	}

	@Transient
	public String getTypeImg() {
		return typeImg;
	}

	public void setTypeImg(String typeImg) {
		this.typeImg = typeImg;
	}

	@Column(name = "view_path", length = 512)
	public String getViewPath() {
		return viewPath;
	}

	public void setViewPath(String viewPath) {
		this.viewPath = viewPath;
	}

	@Column(name = "org_code")
	public String getOrgCode() {
		return orgCode;
	}

	public void setOrgCode(String orgCode) {
		this.orgCode = orgCode;
	}

	@Column(name = "industry_id")
	public Long getIndustryId() {
		return industryId;
	}

	public void setIndustryId(Long industryId) {
		this.industryId = industryId;
	}

	@Column(name = "serial_num")
	public String getSerialNum()
	{
		return serialNum;
	}

	public void setSerialNum(String serialNum)
	{
		this.serialNum = serialNum;
	}

	
	@OneToOne(targetEntity = EquipTemplateIndustry.class, fetch = FetchType.LAZY)
	@Fetch(FetchMode.JOIN)
	@JoinColumn(name = "industry_id", referencedColumnName = "id", updatable = false, insertable = false)
	public EquipTemplateIndustry getEquipTemplateIndustry() {
		return equipTemplateIndustry;
	}

	public void setEquipTemplateIndustry(
			EquipTemplateIndustry equipTemplateIndustry) {
		this.equipTemplateIndustry = equipTemplateIndustry;
	}

}
