/**
 *
 */
package com.yeejoin.amos.iot.business.entity.mybatis;

import com.yeejoin.amos.iot.business.constants.IotConstant;
import net.sf.json.JSONObject;
import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;

import java.io.Serializable;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 设备、能力、接口、协议BO(业务对象)
 *
 * @author as-youjun
 */
public class EquipmentBo implements Serializable, Cloneable {

    /**
     * 日志记录器
     */
    private static final Logger logger = Logger
            .getLogger(EquipmentBo.class);

    /**
     * 设备id
     */
    private Long id;

    /**
     * 装备模板id
     */
    private Long eqpTmplId;

    /**
     * 设备名称
     */
    private String name;

    /**
     * 设备型号
     */
    private String model;

    /**
     * 设备类别：1表示采集设备，2表示传感设备，3表示控制设备
     */
    private Integer type;

    /**
     * 视图id
     */
    private Long graphId;

    /**
     * 节点id
     */
    private Long nodeId;

    /**
     * 能力
     */
    private List<CapabilityBo> capabilityBos;

    /**
     * 设备接口
     */
    private List<EqpTmplIntfcBo> eqpTmplIntfcs;

    /**
     * 设备模板属性
     */
    private List<EqpTmplAttributeBo> eqpTmplAttributes;

    /**
     * json格式参数值
     */
    private String paramJson;

    /**
     * 参数值(包含设备、能力、接口的所有参数)
     */
    private Map<String, Object> paramMap;

    /**
     * token
     */
    private String token;

    /**
     * 测试类型：connct表示接口测试；meric表示指标测试
     */
    private String flag;

    /**
     * @return the id
     */
    public Long getId() {
        return id;
    }

    /**
     * @param id the id to set
     */
    public void setId(Long id) {
        this.id = id;
    }

    /**
     * @return the eqpTmplId
     */
    public Long getEqpTmplId() {
        return eqpTmplId;
    }

    /**
     * @param eqpTmplId the eqpTmplId to set
     */
    public void setEqpTmplId(Long eqpTmplId) {
        this.eqpTmplId = eqpTmplId;
    }

    /**
     * @return the name
     */
    public String getName() {
        return name;
    }

    /**
     * @param name the name to set
     */
    public void setName(String name) {
        this.name = name;
    }

    /**
     * @return the model
     */
    public String getModel() {
        return model;
    }

    /**
     * @param model the model to set
     */
    public void setModel(String model) {
        this.model = model;
    }

    /**
     * @return the type
     */
    public Integer getType() {
        return type;
    }

    /**
     * @param type the type to set
     */
    public void setType(Integer type) {
        this.type = type;
    }

    /**
     * @return the graphId
     */
    public Long getGraphId() {
        return graphId;
    }

    /**
     * @param graphId the graphId to set
     */
    public void setGraphId(Long graphId) {
        this.graphId = graphId;
    }

    /**
     * @return the nodeId
     */
    public Long getNodeId() {
        return nodeId;
    }

    /**
     * @param nodeId the nodeId to set
     */
    public void setNodeId(Long nodeId) {
        this.nodeId = nodeId;
    }

    /**
     * @return the eqpTmplIntfcs
     */
    public List<EqpTmplIntfcBo> getEqpTmplIntfcs() {
        return eqpTmplIntfcs;
    }

    /**
     * @param eqpTmplIntfcs the eqpTmplIntfcs to set
     */
    public void setEqpTmplIntfcs(List<EqpTmplIntfcBo> eqpTmplIntfcs) {
        this.eqpTmplIntfcs = eqpTmplIntfcs;
    }

    public List<EqpTmplAttributeBo> getEqpTmplAttributes() {
        return eqpTmplAttributes;
    }

    public void setEqpTmplAttributes(List<EqpTmplAttributeBo> eqpTmplAttributes) {
        this.eqpTmplAttributes = eqpTmplAttributes;
    }

    /**
     * @param paramMap the paramMap to set
     */
    public void setParamMap(Map<String, Object> paramMap) {
        this.paramMap = paramMap;
    }

    /**
     * @return the capabilityBos
     */
    public List<CapabilityBo> getCapabilityBos() {
        return capabilityBos;
    }

    /**
     * @param capabilityBos the capabilityBos to set
     */
    public void setCapabilityBos(List<CapabilityBo> capabilityBos) {
        this.capabilityBos = capabilityBos;
    }

    /**
     * @return the token
     */
    public String getToken() {
        return token;
    }

    /**
     * @param token the token to set
     */
    public void setToken(String token) {
        this.token = token;
    }

    /**
     * @return the paramJson
     */
    public String getParamJson() {
        return paramJson;
    }

    /**
     * @param paramJson the paramJson to set
     */
    public void setParamJson(String paramJson) {
        this.paramJson = paramJson;
    }

    /**
     * @return the flag
     */
    public String getFlag() {
        return flag;
    }

    /**
     * @param flag the flag to set
     */
    public void setFlag(String flag) {
        this.flag = flag;
    }

    /**
     * @return the paramMap
     */
    @SuppressWarnings("unchecked")
    public Map<String, Object> getParamMap() {
        if (StringUtils.isNotEmpty(this.paramJson)) {
            Map<String, Class> classMap = new HashMap<String, Class>();
            classMap.put(IotConstant.EquipmentConstant.KEY_SELCPBLIDS, List.class);
            classMap.put(IotConstant.EquipmentConstant.KEY_SELINTFCIDS, List.class);
            classMap.put(IotConstant.EquipmentConstant.QUOTASELECTED, Map.class);
            paramMap = (Map<String, Object>) JSONObject.toBean(
                    JSONObject.fromObject(this.paramJson), Map.class, classMap);
        }
        return paramMap;
    }

    @Override
    public Object clone() {
        EquipmentBo newEquipmentBo = null;
        try {
            newEquipmentBo = (EquipmentBo) super.clone();
        } catch (CloneNotSupportedException e) {
            logger.error(e);
        }
        return newEquipmentBo;
    }

    @Override
    public String toString() {
        return new StringBuilder().append(",id:").append(id)
                .append(",eqpTmplId:").append(eqpTmplId).append(",paramJson:")
                .append(paramJson).toString();
    }

}
