/**
 *
 */
package com.yeejoin.amos.iot.business.polling;

import com.yeejoin.amos.iot.business.dic.ScriptLang;
import com.yeejoin.amos.iot.business.entity.Dic;
import com.yeejoin.amos.iot.business.entity.mybatis.CapabilityBo;
import com.yeejoin.amos.iot.business.service.intfc.IDicService;
import com.yeejoin.amos.iot.business.service.intfc.IProtocolService;
import com.yeejoin.amos.iot.business.vo.ProtocolVo;
import com.yeejoin.amos.iot.context.IotContext;
import com.yeejoin.amos.iot.entity.ParseResult;
import com.yeejoin.amos.iot.script.IScriptService;
import org.apache.log4j.Logger;

import java.util.Map;

/**
 * 执行单位基类
 *
 * @author as-youjun
 */
public abstract class BaseExecuteUnit implements Runnable {

    /**
     * 日志记录器
     */
    private static final Logger logger = Logger
            .getLogger(BaseExecuteUnit.class);

    /**
     * 解析协议数据
     *
     * @param capabilityBo
     * @param originals
     * @return
     */
    protected ParseResult parseData(CapabilityBo capabilityBo,
                                    Object[] originals) {
        ParseResult parseResult = null;
        IProtocolService protocolService = (IProtocolService) IotContext
                .getInstance().getBean("protocolService");
        IDicService dicService = (IDicService) IotContext.getInstance()
                .getBean("dicService");
        IScriptService groovyScriptService = (IScriptService) IotContext
                .getInstance().getBean("groovyScriptService");
        IScriptService javaScriptService = (IScriptService) IotContext
                .getInstance().getBean("javaScriptService");
        ProtocolVo protocolVo = protocolService.findById(capabilityBo
                .getProtocolId());
        Dic dic = dicService.getById(String.valueOf(protocolVo
                .getDicIdScriptLang()));

        // 调用groovy/javascript代码解析协议数据
        switch (dic.getCode()) {
            case ScriptLang.GROOVY:
                // groovy解析协议数据
                parseResult = groovyScriptService.analyze(
                        protocolVo.getScriptCode(), originals);
                break;
            case ScriptLang.JAVASCRIPT:
                // javascript解析协议数据
                parseResult = javaScriptService.analyze(protocolVo.getScriptCode(),
                        originals);
                break;
            case ScriptLang.PYTHON:
                // python解析协议数据---暂未实现
                break;
            case ScriptLang.LUA:
                // lua解析协议数据---暂未实现
                break;
            default:
                // 不执行任何业务
                break;
        }

        return parseResult;
    }

    /**
     * 设置参数值
     *
     * @param dataType
     * @param paramName
     * @param paramValue
     * @param paramMap
     */
    protected void fillParamValue(int dataType, String paramName,
                                  Object paramValue, Map<String, Object> paramMap) {
        try {
            switch (dataType) {
                case 1:// 整型
                    paramMap.put(paramName, Integer.valueOf(paramValue.toString()));
                    break;
                case 2:// 浮点型
                    paramMap.put(paramName, Double.valueOf(paramValue.toString()));
                    break;
                case 3:// 布尔型
                    paramMap.put(paramName, Boolean.valueOf(paramValue.toString()));
                    break;
                case 4:// 枚举
                    paramMap.put(paramName, paramValue.toString());
                    break;
                case 5:// 字符串
                    paramMap.put(paramName, paramValue.toString());
                    break;
                case 6:// 密码
                    paramMap.put(paramName, paramValue.toString());
                    break;

                default:
                    break;
            }
        } catch (Exception e) {
            logger.error(e);
        }
    }

}
