/**
 *
 */
package com.yeejoin.amos.iot.business.polling;

import com.yeejoin.amos.iot.business.constants.IotConstant;
import com.yeejoin.amos.iot.business.entity.IntfcAttribute;
import com.yeejoin.amos.iot.business.entity.mybatis.*;
import com.yeejoin.amos.iot.business.enums.CapabilityAttributeType;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.collections.MapUtils;
import org.apache.log4j.Logger;

import java.util.*;

/**
 * 监测工具类
 *
 * @author as-youjun
 */
public class MonitorUtil {

    /**
     * 日志记录器
     */
    private static final Logger logger = Logger.getLogger(MonitorUtil.class);

    /**
     * 设备接口填充到EquipmentBo
     *
     * @param eqpTmplIntfcBos
     * @param equipmentBo
     */
    @SuppressWarnings("unchecked")
    public static void fillEqpTmplIntfc2EquipmentBo(
            List<EqpTmplIntfcBo> eqpTmplIntfcBos, EquipmentBo equipmentBo) {
        if (CollectionUtils.isEmpty(eqpTmplIntfcBos)) {
            return;
        }
        List<Long> intfcIds = new ArrayList<Long>();
        List<EqpTmplIntfcBo> ibos = new ArrayList<EqpTmplIntfcBo>();
        for (EqpTmplIntfcBo eqpTmplIntfcBo : eqpTmplIntfcBos) {
            Object selTmplIntfcIdobj = equipmentBo.getParamMap().get(
                    IotConstant.EquipmentConstant.KEY_SELINTFCIDS);
            boolean selIntfc = false;// 是否选择接口
            if (selTmplIntfcIdobj == null) {
                selIntfc = true;
            } else {
                try {
                    List<Long> selTmplIntfcIds = (List<Long>) selTmplIntfcIdobj;
                    if (CollectionUtils.isEmpty(selTmplIntfcIds)) {
                        selIntfc = true;
                    } else if (selTmplIntfcIds.contains(eqpTmplIntfcBo
                            .getIntfcId())) {
                        selIntfc = true;
                    }
                } catch (Exception e) {
                    logger.error("transform param error! " + e);
                }
            }
            //
            if (selIntfc) {// 页面上勾选此接口
                if (!intfcIds.contains(eqpTmplIntfcBo.getIntfcId())) {
                    ibos.add(eqpTmplIntfcBo);
                    intfcIds.add(eqpTmplIntfcBo.getIntfcId());
                }
                break;
            }
        }
        equipmentBo.setEqpTmplIntfcs(ibos);
    }

    /**
     * CapabilityBo填充到EquipmentBo
     *
     * @param capabilityBos
     * @param equipmentBo
     */
    @SuppressWarnings("unchecked")
    public static void fillCapabilityBo2EquipmentBo(
            List<CapabilityBo> capabilityBos, EquipmentBo equipmentBo) {
        List<CapabilityBo> cbos = new ArrayList<CapabilityBo>();
        for (CapabilityBo capabilityBo : capabilityBos) {
            Object selCpblIdObj = equipmentBo.getParamMap().get(
                    IotConstant.EquipmentConstant.KEY_SELCPBLIDS);
            boolean selCpbl = false;// 是否选择能力
            if (selCpblIdObj == null) {
                selCpbl = true;
            } else {
                try {
                    List<Long> selCpblIds = (List<Long>) selCpblIdObj;
                    if (CollectionUtils.isEmpty(selCpblIds)) {
                        selCpbl = true;
                    } else if (selCpblIds.contains(capabilityBo.getId())) {
                        selCpbl = true;
                    }
                } catch (Exception e) {
                    logger.error("transform param error! " + e);
                }
            }
            if (selCpbl) {// 页面上勾选此能力
                cbos.add(capabilityBo);
                break;
            }
        }
        equipmentBo.setCapabilityBos(cbos);
    }

    /**
     * 接口属性填充到接口
     *
     * @param intfcAttributes
     * @param intfcBos
     */
    public static void fillIntfcAttr2Intfc(
            List<IntfcAttribute> intfcAttributes, List<IntfcBo> intfcBos) {
        if (CollectionUtils.isNotEmpty(intfcAttributes) && CollectionUtils.isNotEmpty(intfcBos)) {
            intfcBos.forEach(intfcBo -> {
                intfcAttributes.forEach(intfcAttr -> {
                    if (intfcAttr.getIntfcId().equals(intfcBo.getId())) {
                        if (intfcBo.getIntfcAttrs() == null) {
                            intfcBo.setIntfcAttrs(new ArrayList<IntfcAttribute>());
                        }
                        IntfcAttribute newIntfcAttribute = (IntfcAttribute) intfcAttr.clone();
                        intfcBo.getIntfcAttrs().add(newIntfcAttribute);
                    }
                });
            });
        } else {
            logger.debug("intfcBos is empty or intfcAttributes is empty!");
        }
    }

    /**
     * 能力属性填充到能力
     *
     * @param cpblAttrs
     * @param capabilityBos
     */
    public static void fillCpblAttr2Capablity(
            List<CapacityPropertyDataVo> cpblAttrs,
            List<CapabilityBo> capabilityBos, Map<String, List<Object>> cpblAttrMap) {
        if (CollectionUtils.isEmpty(cpblAttrs)) {
            return;
        }
        if (CollectionUtils.isEmpty(capabilityBos)) {
            return;
        }
        if (MapUtils.isEmpty(cpblAttrMap)) {
            return;
        }
        //1.获取已选择的能力id和能力属性id
        Set<Long> cpblIds = new HashSet<>();
        capabilityBos.forEach(capabilityBo -> {
            cpblIds.add(capabilityBo.getId());
        });
        Map<Long, Set<Long>> cpblAttrIdMap = new HashMap<>();
        cpblAttrMap.forEach((key, cpblAttrIdStrs) -> {
            Long cpblId = Long.valueOf(key);
            if (cpblAttrIdMap.get(cpblId) == null) {
                cpblAttrIdMap.put(cpblId, new HashSet<>());
            }
            if (CollectionUtils.isNotEmpty(cpblAttrIdStrs)) {
                cpblAttrIdStrs.forEach(cpblAttrIdStr -> {
                    Long cpblAttrId = Long.valueOf(cpblAttrIdStr.toString());
                    cpblAttrIdMap.get(cpblId).add(cpblAttrId);
                });
            } else {
                logger.debug("cpblAttrIdStrs is empty! cpblId:" + key);
            }
        });
        if (MapUtils.isEmpty(cpblAttrIdMap)) {
            return;
        }
        //2.按能力获取能力属性
        Map<Long, List<CapacityPropertyDataVo>> map = new HashMap<>();
        cpblAttrs.forEach(item -> {
            if (cpblIds.contains(item.getCpblId())) {
                if (map.get(item.getCpblId()) == null) {
                    map.put(item.getCpblId(), new ArrayList<>());
                }
                if (CapabilityAttributeType.CONSTANT.getValue() == item.getType().intValue()) {
                    //常量标识输入参数,不需要过滤
                    map.get(item.getCpblId()).add(item);
                } else {
                    if (CollectionUtils.isNotEmpty(cpblAttrIdMap.get(item.getCpblId()))) {
                        if (cpblAttrIdMap.get(item.getCpblId()).contains(item.getId())) {
                            map.get(item.getCpblId()).add(item);
                        }
                    } else {
                        logger.debug("cpblAttrIdMap.get(item.getCpblId()) is empty! cpblId:" + item.getCpblId());
                    }
                }
            }
        });
        //3.设置能力属性
        for (CapabilityBo bo : capabilityBos) {
            List<CapacityPropertyDataVo> newCpdVos = new ArrayList<>();
            if (CollectionUtils.isNotEmpty(map.get(bo.getId()))) {
                map.get(bo.getId()).forEach(capacityPropertyDataVo -> {
                    CapacityPropertyDataVo newCpdVo = (CapacityPropertyDataVo) capacityPropertyDataVo.clone();
                    newCpdVos.add(newCpdVo);
                });
            }
            bo.setCpblAttrs(newCpdVos);
        }
    }

    /**
     * IntfcBo填充到CapabilityBo
     *
     * @param intfcBos
     * @param capabilityBos
     */
    public static void fillIntfcBo2CapabilityBo(List<IntfcBo> intfcBos,
                                                List<CapabilityBo> capabilityBos) {
        if (CollectionUtils.isNotEmpty(capabilityBos) && CollectionUtils.isNotEmpty(intfcBos)) {
            Map<Long, List<IntfcBo>> intfcBoMap = new HashMap<>();
            intfcBos.forEach(intfcBo -> {
                if (intfcBoMap.get(intfcBo.getCpblId()) == null) {
                    intfcBoMap.put(intfcBo.getCpblId(), new ArrayList<>());
                }
                intfcBoMap.get(intfcBo.getCpblId()).add(intfcBo);
            });
            capabilityBos.forEach(capabilityBo -> {
                List<IntfcBo> ibos = intfcBoMap.get(capabilityBo.getId());
                if (CollectionUtils.isNotEmpty(ibos)) {
                    List<IntfcBo> newIntfcBos = new ArrayList<>(ibos.size());
                    ibos.forEach(ibo -> {
                        IntfcBo intfcBo = (IntfcBo) ibo.clone();//浅拷贝
                        newIntfcBos.add(intfcBo);
                    });
                    capabilityBo.setIntfcBos(newIntfcBos);
                }
            });
        } else {
            logger.debug("capabilityBos is empty or intfcBos is empty!");
        }
    }

    /**
     * CapabilityBo填充到EquipmentBo
     *
     * @param capabilityBos
     * @param equipmentBos
     */
    public static void fillCapabilityBo2EquipmentBo(List<CapabilityBo> capabilityBos,
                                                    List<EquipmentBo> equipmentBos) {
        if (CollectionUtils.isEmpty(equipmentBos) || CollectionUtils.isEmpty(capabilityBos)) {
            return;
        }
        equipmentBos.forEach(equipmentBo -> {
            Map<String, Object> paramMap = equipmentBo.getParamMap();
            if (MapUtils.isNotEmpty(paramMap)) {
                List<String> selCpblIdStrs = (List<String>) paramMap.get(IotConstant.EquipmentConstant.KEY_SELCPBLIDS);
                if (CollectionUtils.isNotEmpty(selCpblIdStrs)) {
                    Set<Long> selCboIds = new HashSet<>();//已选择的能力id集合
                    for (String selCpblIdStr : selCpblIdStrs) {
                        selCboIds.add(Long.valueOf(selCpblIdStr));
                    }
                    List<CapabilityBo> selCpblBos = new ArrayList<>();
                    capabilityBos.forEach(capabilityBo -> {
                        if (selCboIds.contains(capabilityBo.getId())) {
                            CapabilityBo newCbo = (CapabilityBo) capabilityBo.clone();
                            selCpblBos.add(newCbo);
                        }
                    });
                    equipmentBo.setCapabilityBos(selCpblBos);
                } else {
                    logger.debug("selCpblIdStrs is empty!");
                }
            } else {
                logger.debug("paramMap is empty!");
            }
        });
    }

    /**
     * 设备接口填充到EquipmentBo
     *
     * @param eqpTmplIntfcBos
     * @param equipmentBos
     */
    public static void fillEqpTmplIntfc2EquipmentBo(
            List<EqpTmplIntfcBo> eqpTmplIntfcBos, List<EquipmentBo> equipmentBos) {
        Map<Long, List<EqpTmplIntfcBo>> eqpTmplIntfcMap = new HashMap<>();
        eqpTmplIntfcBos.forEach(eqpTmplIntfc -> {
            if (eqpTmplIntfcMap.get(eqpTmplIntfc.getEqpTmplId()) == null) {
                eqpTmplIntfcMap.put(eqpTmplIntfc.getEqpTmplId(),
                        new ArrayList<>());
            }
            eqpTmplIntfcMap.get(eqpTmplIntfc.getEqpTmplId()).add(eqpTmplIntfc);
        });
        for (EquipmentBo equipmentBo : equipmentBos) {
            List<EqpTmplIntfcBo> allEtiBos = eqpTmplIntfcMap.get(equipmentBo
                    .getEqpTmplId());
            List<EqpTmplIntfcBo> selEtibos = new ArrayList<>();
            Map<String, Object> paramMap = equipmentBo.getParamMap();
            if (MapUtils.isNotEmpty(paramMap)) {
                List<String> selIntfcIdStrs = (List<String>) paramMap.get(IotConstant.EquipmentConstant.KEY_SELINTFCIDS);
                if (CollectionUtils.isNotEmpty(selIntfcIdStrs) && CollectionUtils.isNotEmpty(allEtiBos)) {
                    Set<Long> selIntfcIds = new HashSet<>();
                    selIntfcIdStrs.forEach(selIntfcIdStr -> {
                        selIntfcIds.add(Long.valueOf(selIntfcIdStr));
                    });
                    for (EqpTmplIntfcBo eqpTmplIntfcBo : allEtiBos) {
                        if (selIntfcIds.contains(eqpTmplIntfcBo.getIntfcId())) {
                            EqpTmplIntfcBo newEqpTmplIntfcBo = (EqpTmplIntfcBo) eqpTmplIntfcBo.clone();
                            selEtibos.add(newEqpTmplIntfcBo);
                        }
                    }
                } else {
                    logger.debug("allEtiBos or selIntfcBoIdStrs is empty!");
                }
            } else {
                logger.debug("paramMap is empty!");
            }
            equipmentBo.setEqpTmplIntfcs(selEtibos);
        }
    }

    /**
     * 设备模板属性填充到EquipmentBo
     *
     * @param eqpTmplAttributes
     * @param equipmentBos
     */
    public static void fillEqpTmplAttribute2EquipmentBo(
            List<EqpTmplAttributeBo> eqpTmplAttributes,
            List<EquipmentBo> equipmentBos) {
        Map<Long, List<EqpTmplAttributeBo>> eqpTmplAttrMap = new HashMap<>();
        eqpTmplAttributes.forEach(eqpTmplAttr -> {
            if (eqpTmplAttrMap.get(eqpTmplAttr.getEqpTmplId()) == null) {
                eqpTmplAttrMap.put(eqpTmplAttr.getEqpTmplId(),
                        new ArrayList<>());
            }
            eqpTmplAttrMap.get(eqpTmplAttr.getEqpTmplId()).add(eqpTmplAttr);
        });
        for (EquipmentBo equipmentBo : equipmentBos) {
            List<EqpTmplAttributeBo> newEtabos = new ArrayList<>();
            if (CollectionUtils.isNotEmpty(eqpTmplAttrMap.get(equipmentBo
                    .getEqpTmplId()))) {
                eqpTmplAttrMap.get(equipmentBo
                        .getEqpTmplId()).forEach(eqpTmplAttributeBo -> {
                    EqpTmplAttributeBo newEqpTmplAttributeBo = (EqpTmplAttributeBo) eqpTmplAttributeBo.clone();
                    newEtabos.add(newEqpTmplAttributeBo);
                });
            }
            equipmentBo.setEqpTmplAttributes(newEtabos);
        }
    }

    /**
     * intfcBo填充到eqpTmplIntfcBo
     *
     * @param intfcBos
     * @param eqpTmplIntfcBos
     */
    public static void fillIntfcBo2EqpTmplIntfcBo(List<IntfcBo> intfcBos, List<EqpTmplIntfcBo> eqpTmplIntfcBos) {
        if (CollectionUtils.isNotEmpty(eqpTmplIntfcBos) && CollectionUtils.isNotEmpty(intfcBos)) {
            eqpTmplIntfcBos.forEach(eqpTmplIntfcBo -> {
                for (IntfcBo intfcBo : intfcBos) {
                    if (intfcBo.getEqpTmplId().equals(eqpTmplIntfcBo.getEqpTmplId()) && intfcBo.getId().equals(eqpTmplIntfcBo.getIntfcId())) {
                        eqpTmplIntfcBo.setCommId(intfcBo.getCommId());
                        eqpTmplIntfcBo.setCommMode(intfcBo.getCommMode());
                        eqpTmplIntfcBo.setCommName(intfcBo.getCommName());
                        List<IntfcAttribute> newIntfcAttrs = new ArrayList<>();
                        if (CollectionUtils.isNotEmpty(intfcBo.getIntfcAttrs())) {
                            intfcBo.getIntfcAttrs().forEach(intfcAttribute -> {
                                IntfcAttribute newIntfcAttribute = (IntfcAttribute) intfcAttribute.clone();
                                newIntfcAttrs.add(newIntfcAttribute);
                            });
                        }
                        eqpTmplIntfcBo.setIntfcAttrs(newIntfcAttrs);
                        break;
                    }
                }
            });
        } else {
            logger.debug("eqpTmplIntfcBos is empty or intfcBos is empty!");
        }
    }

    /**
     * 监测是否选择该接口
     *
     * @param intfcBo
     * @param eqpTmplIntfcBos
     * @return
     */
    public static boolean isSelIntfc(IntfcBo intfcBo, List<EqpTmplIntfcBo> eqpTmplIntfcBos) {
        boolean isSelIntfc = false;
        if (CollectionUtils.isEmpty(eqpTmplIntfcBos)) {
            return isSelIntfc;
        }
        for (EqpTmplIntfcBo eqpTmplIntfcBo : eqpTmplIntfcBos) {
            if (intfcBo.getId().longValue() == eqpTmplIntfcBo.getIntfcId().longValue()) {
                isSelIntfc = true;
                break;
            }
        }
        return isSelIntfc;
    }

    /**
     * 构建指标采集结果
     *
     * @param cpblDataMap
     * @param cpblAttrs
     * @return
     */
    public static Map<String, Object> buildCpblAttrValue(Map<String, Object> cpblDataMap,
                                                         List<CapacityPropertyDataVo> cpblAttrs) {
        Map<String, Object> resultMap = new HashMap<>();
        if (MapUtils.isEmpty(cpblDataMap)) {
            return resultMap;
        }
        cpblAttrs.forEach(item -> {
            if (item.getType().intValue() != CapabilityAttributeType.CONSTANT
                    .getValue()) {
                Object objValue = cpblDataMap.get(item.getName());
                String attrValue = objValue == null ? "" : objValue.toString();
                resultMap.put(item.getName(), attrValue);
            }
        });
        return resultMap;
    }

}