/**
 *
 */
package com.yeejoin.amos.iot.business.polling;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.*;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;
import org.springframework.transaction.annotation.Transactional;

import com.yeejoin.amos.iot.business.constants.IotConstant;
import com.yeejoin.amos.iot.business.dao.repository.IEqpTmplPropertyDao;
import com.yeejoin.amos.iot.business.dao.repository.IEquipmentDao;
import com.yeejoin.amos.iot.business.dao.repository.IIntfcAttributeDao;
import com.yeejoin.amos.iot.business.entity.EqpTmplAttribute;
import com.yeejoin.amos.iot.business.entity.Equipment;
import com.yeejoin.amos.iot.business.entity.IntfcAttribute;
import com.yeejoin.amos.iot.business.entity.MetricData;
import com.yeejoin.amos.iot.business.entity.MetricDataRo;
import com.yeejoin.amos.iot.business.entity.mybatis.CapabilityBo;
import com.yeejoin.amos.iot.business.entity.mybatis.CapacityPropertyDataVo;
import com.yeejoin.amos.iot.business.entity.mybatis.EqpTmplAttributeBo;
import com.yeejoin.amos.iot.business.entity.mybatis.EqpTmplIntfcBo;
import com.yeejoin.amos.iot.business.entity.mybatis.EquipmentBo;
import com.yeejoin.amos.iot.business.entity.mybatis.IntfcBo;
import com.yeejoin.amos.iot.business.enums.AttributeType;
import com.yeejoin.amos.iot.business.enums.IntfcDirection;
import com.yeejoin.amos.iot.business.queue.AlarmDataObj;
import com.yeejoin.amos.iot.business.queue.AlarmDataQueue;
import com.yeejoin.amos.iot.business.service.intfc.IMetricDataService;
import com.yeejoin.amos.iot.context.IotContext;
import com.yeejoin.amos.iot.entity.ParseResult;
import com.yeejoin.amos.iot.mongodb.service.intfc.IEquipMetricDataService;
import com.yeejoin.amos.iot.netty.vehicle.IVehicleDataService;
import com.yeejoin.amos.iot.rule.engine.service.IRuleEngineService;
import com.yeejoin.amos.iot.rule.engine.service.impl.RuleEngineServiceImpl;

import net.sf.json.JSONObject;
import org.springframework.util.ObjectUtils;

/**
 * 轮询执行单元
 *
 * @author as-youjun
 */
public class PollingExecuteUnit extends BaseExecuteUnit {

    /**
     * 日志记录器
     */
    private static final Logger logger = Logger
            .getLogger(PollingExecuteUnit.class);

    /**
     * 监测对象
     */
    private EquipmentBo equipmentBo;
    IRuleEngineService ruleEngineService = (IRuleEngineService) IotContext.getInstance().getBean(RuleEngineServiceImpl.class);
    IVehicleDataService vehicleDataService = (IVehicleDataService) IotContext.getInstance().getBean("vehicleDataService");
    IEquipMetricDataService equipMetricDataService = (IEquipMetricDataService) IotContext
            .getInstance().getBean("equipMetricDataService");
    IMetricDataService metricDataService = (IMetricDataService) IotContext
            .getInstance().getBean("metricDataService");
    IEqpTmplPropertyDao iEqpTmplPropertyDao = (IEqpTmplPropertyDao) IotContext
            .getInstance().getBean(IEqpTmplPropertyDao.class);
    /**
     * 构造方法
     *
     * @param equipmentBo
     */
    public PollingExecuteUnit(EquipmentBo equipmentBo) {
        this.equipmentBo = equipmentBo;
    }

    /**
     * 执行单个资源监控业务
     */
    @Override
    public void run() {


        if (CollectionUtils.isEmpty(equipmentBo.getCapabilityBos())) {
            logger.debug("capability is empty!");
            return;
        }
        Date exeTime = new Date();

        for (CapabilityBo capabilityBo : equipmentBo.getCapabilityBos()) {
            if (CollectionUtils.isEmpty(capabilityBo.getIntfcBos())) {
                continue;
            }
            if (CollectionUtils.isEmpty(capabilityBo.getCpblAttrs())) {
                continue;
            }
            try {
                // 1.采集一个能力数据
                Map<String, Object> originalCpblDataMap = fetchCapabilityData(
                        equipmentBo.getEqpTmplAttributes(),
                        equipmentBo.getEqpTmplIntfcs(), capabilityBo,
                        equipmentBo.getParamMap());
                // 2.组装数据
                if (MapUtils.isEmpty(originalCpblDataMap)) {
                    continue;
                }
                Map<String, Object> cpblDataMap = MonitorUtil.buildCpblAttrValue(originalCpblDataMap, capabilityBo.getCpblAttrs());
                if (MapUtils.isEmpty(cpblDataMap)) {
                    continue;
                }
                // 3.组装一个能力数据
                MetricData metricData = buildMetricData(capabilityBo,
                        cpblDataMap, exeTime);

                // 4.保存数据
                saveMetricData(metricData);
                // 5.向前端推送数据
                sendWebSocketMessage(metricData);
                // 6.告警处理
                raiseAalrm(metricData);
                // 7.规则执行
                triggerRuleTask(metricData);

                originalCpblDataMap = null;
            } catch (Exception e) {
                logger.error(e);
            } finally {
            	System.gc();
    		}
        }
        equipmentBo = null;
    }
    
    public void triggerRuleTask(MetricData metricData) {
    	List list = new ArrayList();
    	List<Map<String, Object>> propertys = new ArrayList<>();
    	list.add(metricData);
        IEquipmentDao equipmentDao = (IEquipmentDao) IotContext
                .getInstance().getBean("equipmentDao");
    	MetricDataRo metricDataRo = new MetricDataRo(list);
        Equipment equipment = equipmentDao.findById(metricData.getEqpId()).get();
        metricDataRo.setEqpId(metricData.getEqpId());
        metricDataRo.setSubgraphId(metricData.getSubgraphId());

        metricDataRo.setOrgCode(equipment.getOrgCode());
        metricDataRo.setCpblId(metricData.getCpblId());
        metricDataRo.setEqpName(equipment.getName());
        metricDataRo.setAuthOrg(equipment.getAuthOrg());

        Map<String, Object> configs = equipment.getCfgParamMap();
        configs.keySet().forEach(item -> {
            if (item.startsWith("equip_")) {
                Map<String, Object> tempProperty = new HashMap<>();
                String idStr = item.substring(6, item.length());
                tempProperty.put("id", idStr);
                tempProperty.put("valueStr", configs.get(item));
                EqpTmplAttribute eqpTmplAttribute = iEqpTmplPropertyDao.findById(Long.valueOf(idStr)).get();
                if (eqpTmplAttribute != null) {
                    tempProperty.put("displayName", eqpTmplAttribute.getDisplayName());
                    propertys.add(tempProperty);
                }
            }
        });
        metricDataRo.setEqpConfigMap(propertys);
        ruleEngineService.fireRule(metricDataRo);

        logger.info(String.format("polling executeUnit ===> [%s]", metricDataRo.toString()));

//        String strAuthOrg = equipment.getAuthOrg();
//        if (!ObjectUtils.isEmpty(strAuthOrg)) {
//            String str = strAuthOrg.replaceAll(" +","");// 去空格
//            String strSub = str.substring(1,str.length()-1);
//            String strArray[] = strSub.split(",");
//            List<String> rowIdList = Arrays.asList(strArray);
//            for (String org : rowIdList) {
//                if ("null".equals(org) || ObjectUtils.isEmpty(org)) {
//                    continue;
//                }
//                metricDataRo.setOrgCode(org);
//                ruleEngineService.fireRule(metricDataRo);
//            }
//
//        }
    }

    /**
     * 向前端推送websocket消息
     *
     * @param metricData
     */
    public void sendWebSocketMessage(MetricData metricData) {
        try {
            
            vehicleDataService.sendWebSocketMessage(metricData);
        } catch (Exception e) {
            logger.error(e);
        }
    }

    /**
     * 保存指标数据
     *
     * @param metricData
     */
    @Transactional
    public void saveMetricData(MetricData metricData) {
        
        // 1.保存到mysql
        metricDataService.saveMetricData(metricData);
        // 2.保存到mongodb
        equipMetricDataService.saveMetricData2MongoDb(metricData);
    }

    /**
     * 组装一个能力数据
     *
     * @param capabilityBo
     * @param cpblDataMap
     * @param exeTime
     * @return
     */
    private MetricData buildMetricData(CapabilityBo capabilityBo,
                                       Map<String, Object> cpblDataMap, Date exeTime) {
        MetricData metricData = new MetricData();
        metricData.setEqpId(equipmentBo.getId());
        metricData.setEqpTmplId(equipmentBo.getEqpTmplId());
        metricData.setExeTime(exeTime);
        metricData.setPeriod(capabilityBo.getMonitorPeriod());
        metricData.setCpblId(capabilityBo.getId());
        metricData.setIsDelete(false);
        metricData.setMetricKey(capabilityBo.getId() + "-"
                + capabilityBo.getName());
        metricData.setMetricJson(JSONObject.fromObject(cpblDataMap).toString());
        metricData.setSubgraphId(equipmentBo.getGraphId());
        return metricData;
    }

    /**
     * 获取一个能力数据
     *
     * @param eqpTmplAttributes
     * @param eqpTmplIntfcBos
     * @param capabilityBo
     * @param eqpParamMap
     * @return
     */
    @SuppressWarnings("unchecked")
    private Map<String, Object> fetchCapabilityData(
            List<EqpTmplAttributeBo> eqpTmplAttributes,
            List<EqpTmplIntfcBo> eqpTmplIntfcBos, CapabilityBo capabilityBo,
            Map<String, Object> eqpParamMap) {
        Map<String, Object> cpblDataMap = null;
        int intfcSize = capabilityBo.getIntfcBos().size();// 接口数量
        Object[] originals = new Object[intfcSize];// 存放每一个接口采集的结果数据
        IntfcBo intfcBo = null;
        // 1.如果是上行接口，表示是监听方式，直接返回
        if (intfcSize > 0) {
            intfcBo = capabilityBo.getIntfcBos().get(0);
            // 如果是上行接口，表示是监听方式，直接返回！
            EqpTmplIntfcBo eqpTmplIntfcBo = getEqpTmplIntfc(eqpTmplIntfcBos,
                    intfcBo.getId());
            if (eqpTmplIntfcBo == null) {
            	originals = null;
                logger.debug("eqpTmplIntfcBo is null!");
                return cpblDataMap;
            }
            if (eqpTmplIntfcBo.getIntfcDirection().intValue() == IntfcDirection.UP
                    .getValue()) {
            	originals = null;
                return cpblDataMap;
            }
        }
        // 2.使用反射调用接口采集数据
        for (int i = 0; i < intfcSize; i++) {
            intfcBo = capabilityBo.getIntfcBos().get(i);
            //监测用户是否选择该接口
            boolean isSelIntfc = MonitorUtil.isSelIntfc(intfcBo, eqpTmplIntfcBos);
            if (!isSelIntfc) {
                originals[i] = null;
                continue;
            }
            Map<String, Object> paramMap = buildIntfcParamMap(
                    eqpTmplAttributes, intfcBo, capabilityBo.getCpblAttrs(),
                    eqpParamMap);
            paramMap.put(IotConstant.EquipmentConstant.EQPID, equipmentBo.getId());
            PollingResult pollingResult = invoke(intfcBo, paramMap);
            if (pollingResult.getStatus() == IotConstant.SUCCESS) {
                originals[i] = pollingResult.getResult();
            } else {
                originals[i] = pollingResult.getResult();
            }
        }
        // 3.调用脚本代码解析协议数据
        ParseResult parseResult = parseData(capabilityBo, originals);
        if (parseResult.getStatus() == IotConstant.SUCCESS) {
            try {
                String resultJson = (String) parseResult.getResult();
                cpblDataMap = (Map<String, Object>) JSONObject.toBean(
                        JSONObject.fromObject(resultJson), Map.class);
            } catch (Exception e) {
                logger.error("parseResult error!" + e);
            }
        } else {
            logger.error("parseResult error! " + parseResult.getErrorMsg());
        }
        originals = null;
        parseResult = null;
        return cpblDataMap;
    }

    /**
     * 获取EqpTmplIntfcBo
     *
     * @param eqpTmplIntfcBos
     * @param intfcId
     * @return
     */
    private EqpTmplIntfcBo getEqpTmplIntfc(
            List<EqpTmplIntfcBo> eqpTmplIntfcBos, Long intfcId) {
        EqpTmplIntfcBo eqpTmplIntfcBo = null;
        for (EqpTmplIntfcBo etib : eqpTmplIntfcBos) {
            if (intfcId.equals(etib.getIntfcId())) {
                eqpTmplIntfcBo = etib;
                break;
            }
        }
        return eqpTmplIntfcBo;
    }

    /**
     * 使用反射调用接口采集数据
     *
     * @param intfcBo
     * @param paramMap
     * @return
     */
    private PollingResult invoke(IntfcBo intfcBo, Map<String, Object> paramMap) {
        if (StringUtils.isEmpty(intfcBo.getCommName())) {
            logger.warn("CommMode is empty! intfc id=" + intfcBo.getId());
            return null;
        }

        PollingResult pollingResult = null;
        String pkg = "com.yeejoin.amos.iot."
                + intfcBo.getCommName().toLowerCase();
        String firstChar = intfcBo.getCommName().substring(0, 1).toUpperCase();
        String otherChars = intfcBo.getCommName()
                .substring(1, intfcBo.getCommName().length()).toLowerCase();
        String className = pkg + "." + firstChar + otherChars
                + "CollectService";
        try {
            Class<?> c = Class.forName(className);
            Object obj = c.newInstance();
            // 第一个参数写的是方法名,第二个\第三个\...写的是方法参数列表中参数的类型
            Method method = c.getMethod(intfcBo.getName(), Map.class);
            // invoke是执行该方法,并携带参数值
            pollingResult = (PollingResult) method.invoke(obj,
                    new Object[]{paramMap});
            obj = null;
            method = null;  
        } catch (ClassNotFoundException | InstantiationException
                | IllegalAccessException | NoSuchMethodException
                | SecurityException | IllegalArgumentException
                | InvocationTargetException e) {
        	e.printStackTrace();
            logger.error(e);
        }
        className = null;
        pkg = null;
        return pollingResult;
    }

    /**
     * 构建接口参数
     *
     * @param eqpTmplAttributes
     * @param intfcBo
     * @param cpblAttrs
     * @param eqpParamMap
     * @return
     */
    private Map<String, Object> buildIntfcParamMap(
            List<EqpTmplAttributeBo> eqpTmplAttributes, IntfcBo intfcBo,
            List<CapacityPropertyDataVo> cpblAttrs,
            Map<String, Object> eqpParamMap) {
        Map<String, Object> paramMap = new HashMap<>();
        // 1.参数来自设备模板
        fillEqpTmplAttrParam(eqpTmplAttributes, eqpParamMap, paramMap);
        // 2.参数来自接口自身
        fillIntfcAttrParam(intfcBo, eqpParamMap, paramMap);
        // 3.参数来自能力属性
        fillCpblAttrParam(cpblAttrs, paramMap);
        return paramMap;
    }

    /**
     * 参数来自能力属性
     *
     * @param cpblAttrs
     * @param paramMap
     */
    private void fillCpblAttrParam(List<CapacityPropertyDataVo> cpblAttrs,
                                   Map<String, Object> paramMap) {
        if (CollectionUtils.isNotEmpty(cpblAttrs)) {
            cpblAttrs.forEach(cpblAttr -> {
                if (cpblAttr.getType().intValue() == AttributeType.CONSTANT
                        .getValue()) {
                    // 设置参数值
                    fillParamValue(cpblAttr.getDataType(), cpblAttr.getName(),
                            cpblAttr.getValueStr(), paramMap);
                }
            });
        }
    }

    /**
     * 参数来自接口自身
     *
     * @param intfcBo
     * @param eqpParamMap
     * @param paramMap
     */
    private void fillIntfcAttrParam(IntfcBo intfcBo,
                                    Map<String, Object> eqpParamMap, Map<String, Object> paramMap) {
        Object paramValue = null;
        String paramKey = null;
        IIntfcAttributeDao intfcAttributeDao = (IIntfcAttributeDao) IotContext
                .getInstance().getBean("intfcAttributeDao");
        List<Long> intfcIds = new ArrayList<Long>();
        intfcIds.add(intfcBo.getId());
        List<IntfcAttribute> intfcAttributes = intfcAttributeDao
                .findAllByIntfcIdIn(intfcIds);
        if (CollectionUtils.isNotEmpty(intfcAttributes)) {
            for (IntfcAttribute intfcAttribute : intfcAttributes) {
                paramKey = IotConstant.EquipmentConstant.KEY_PREFIX_INTFC
                        + intfcBo.getId()
                        + IotConstant.EquipmentConstant.UNDERLINE
                        + intfcAttribute.getId();
                paramValue = eqpParamMap.get(paramKey);
                if (paramValue == null) {
                    paramValue = intfcAttribute.getValueStr();
                }
                // 设置参数值
                fillParamValue(intfcAttribute.getDataType(),
                        intfcAttribute.getName(), paramValue, paramMap);
            }
        }
    }

    /**
     * 参数来自设备模板
     *
     * @param eqpTmplAttributes
     * @param eqpParamMap
     * @param paramMap
     */
    private void fillEqpTmplAttrParam(
            List<EqpTmplAttributeBo> eqpTmplAttributes,
            Map<String, Object> eqpParamMap, Map<String, Object> paramMap) {
        Object paramValue = null;
        String paramKey = null;
        if (CollectionUtils.isNotEmpty(eqpTmplAttributes)) {
            for (EqpTmplAttributeBo eqpTmplAttributeBo : eqpTmplAttributes) {
                paramKey = IotConstant.EquipmentConstant.KEY_PREFIX_EQUIP
                        + eqpTmplAttributeBo.getId();
                paramValue = eqpParamMap.get(paramKey);
                if (paramValue == null) {
                    continue;
                }
                // 设置参数值
                fillParamValue(eqpTmplAttributeBo.getDataType(),
                        eqpTmplAttributeBo.getName(), paramValue, paramMap);
            }
        }
    }

    /**
     * 告警处理
     *
     * @param metricData
     */
    @SuppressWarnings("unchecked")
    public void raiseAalrm(MetricData metricData) {
        Map<String, Object> paramMap = (Map<String, Object>) JSONObject.toBean(
                JSONObject.fromObject(metricData.getMetricJson()), Map.class);
        // 状态告警
        if (paramMap.keySet().contains("status")) {
            AlarmDataObj alarmDataObj = new AlarmDataObj(metricData);
            alarmDataObj.setType(IotConstant.EQUIPMENT_STATUS);
            AlarmDataQueue.getInstance().add(alarmDataObj);
        }
    }

}
