package com.yeejoin.amos.iot.business.polling;

import java.util.List;

import org.apache.commons.collections.CollectionUtils;
import org.apache.log4j.Logger;

import com.yeejoin.amos.iot.business.entity.PollingJob;
import com.yeejoin.amos.iot.business.entity.mybatis.EquipmentBo;
import com.yeejoin.amos.iot.business.enums.MonitorType;
import com.yeejoin.amos.iot.business.param.QueryMonitorEqpParam;
import com.yeejoin.amos.iot.business.service.intfc.IMonitorService;
import com.yeejoin.amos.iot.context.IotContext;
import com.yeejoin.amos.iot.core.threadpool.AmosThreadPool;
import com.yeejoin.amos.iot.exception.YeeException;

/**
 * 每一个周期job执行者
 *
 * @author as-youjun
 */
public class PollingScheduledExecutor implements Runnable {

    /**
     * 日志记录器
     */
    private static final Logger logger = Logger
            .getLogger(PollingScheduledExecutor.class);

    /**
     * 定时job
     */
    private PollingJob pollingJob;

    IMonitorService monitorService = null;
    /**
     * 构造方法
     *
     * @param pollingJob
     */
    public PollingScheduledExecutor(PollingJob pollingJob) {
        this.pollingJob = pollingJob;
        monitorService = (IMonitorService) IotContext.getInstance().getBean("monitorService");
    }

    /**
     * 调度一个周期pollingjob
     */
    @Override
    public void run() {
        try {
            // 1.查询监测设备
            QueryMonitorEqpParam queryParam = new QueryMonitorEqpParam();
            queryParam.setEnableMonitor(true);
            queryParam.setMonitorType(MonitorType.COLLECT.getValue());
            queryParam.setMonitorPeriod(pollingJob.getPeriod());
            List<EquipmentBo> equipmentBos = monitorService
                    .findMonitorEquipment(queryParam);
            if (CollectionUtils.isEmpty(equipmentBos)) {
                logger.info("monitoring equipmentBos is empty! jobName:"
                        + pollingJob.getName());
                return;
            }
            // 2.开启线程执行job
            equipmentBos.forEach(equipmentBo -> {
                PollingExecuteUnit pollingExecuteUnit = new PollingExecuteUnit(
                        equipmentBo);
                AmosThreadPool.getInstance().execute(pollingExecuteUnit);
            });
            equipmentBos.clear();
            equipmentBos = null;
            queryParam = null;
        } catch (Exception e) {
            logger.error(e);
            throw new YeeException("running polling job error! " + e, 1000030);
        }  finally {
        	System.gc();
		}
    }

}
