/**
 *
 */
package com.yeejoin.amos.iot.business.polling.test;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import net.sf.json.JSONObject;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;

import com.yeejoin.amos.iot.business.constants.IotConstant;
import com.yeejoin.amos.iot.business.entity.IntfcAttribute;
import com.yeejoin.amos.iot.business.entity.MetricData;
import com.yeejoin.amos.iot.business.entity.mybatis.CapabilityBo;
import com.yeejoin.amos.iot.business.entity.mybatis.CapacityPropertyDataVo;
import com.yeejoin.amos.iot.business.entity.mybatis.EqpTmplAttributeBo;
import com.yeejoin.amos.iot.business.entity.mybatis.EqpTmplIntfcBo;
import com.yeejoin.amos.iot.business.entity.mybatis.EquipmentBo;
import com.yeejoin.amos.iot.business.entity.mybatis.IntfcBo;
import com.yeejoin.amos.iot.business.enums.AttributeType;
import com.yeejoin.amos.iot.business.enums.IntfcDirection;
import com.yeejoin.amos.iot.business.polling.BaseExecuteUnit;
import com.yeejoin.amos.iot.business.polling.PollingResult;
import com.yeejoin.amos.iot.entity.ParseResult;

/**
 * 指标采集测试执行单元
 *
 * @author as-youjun
 */
public class TestCpblExecuteUnit extends BaseExecuteUnit {

    private CapabilityBo capabilityBo;

    private TestEqpInputData testEqpInputData;

    /**
     * 日志记录器
     */
    private static final Logger logger = Logger
            .getLogger(TestCpblExecuteUnit.class);

    /**
     * 构造方法
     *
     * @param capabilityBo
     * @param testEqpInputData
     */
    public TestCpblExecuteUnit(CapabilityBo capabilityBo,
                               TestEqpInputData testEqpInputData) {
        this.capabilityBo = capabilityBo;
        this.testEqpInputData = testEqpInputData;
    }

    /**
     * 获取一个能力数据
     *
     * @param eqpTmplAttributes
     * @param eqpTmplIntfcBos
     * @param capabilityBo
     * @param eqpParamMap
     * @return
     */
    @SuppressWarnings("unchecked")
    private Map<String, Object> fetchCapabilityData(
            List<EqpTmplAttributeBo> eqpTmplAttributes,
            List<EqpTmplIntfcBo> eqpTmplIntfcBos, CapabilityBo capabilityBo,
            Map<String, Object> eqpParamMap) {
        Map<String, Object> cpblDataMap = null;
        int intfcSize = capabilityBo.getIntfcBos().size();// 接口数量
        Object[] originals = new Object[intfcSize];// 存放每一个接口采集的结果数据
        IntfcBo intfcBo = null;
        // 1.如果是上行接口，表示是监听方式，直接返回
        if (intfcSize > 0) {
            intfcBo = capabilityBo.getIntfcBos().get(0);
            // 如果是上行接口，表示是监听方式，直接返回！
            EqpTmplIntfcBo eqpTmplIntfcBo = getEqpTmplIntfc(eqpTmplIntfcBos,
                    intfcBo.getId());
            if (eqpTmplIntfcBo.getIntfcDirection().intValue() == IntfcDirection.UP
                    .getValue()) {
                return cpblDataMap;
            }
        }
        // 2.使用反射调用接口采集数据
        for (int i = 0; i < intfcSize; i++) {
            intfcBo = capabilityBo.getIntfcBos().get(i);
            Map<String, Object> paramMap = buildIntfcParamMap(
                    eqpTmplAttributes, intfcBo, capabilityBo.getCpblAttrs(),
                    eqpParamMap);
            PollingResult pollingResult = invoke(intfcBo, paramMap);
            if(pollingResult != null){
                originals[i] = pollingResult.getResult();
            }
        }
        // 3.调用脚本代码解析协议数据
        ParseResult parseResult = parseData(capabilityBo, originals);
        if (parseResult.getStatus() == IotConstant.SUCCESS) {
            try {
                String resultJson = (String) parseResult.getResult();
                cpblDataMap = (Map<String, Object>) JSONObject.toBean(
                        JSONObject.fromObject(resultJson), Map.class);
            } catch (Exception e) {
                logger.error("test parseResult error!" + e);
            }
        } else {
            logger.error("test  parseResult error! "
                    + parseResult.getErrorMsg());
        }

        return cpblDataMap;
    }

    /**
     * 获取EqpTmplIntfcBo
     *
     * @param eqpTmplIntfcBos
     * @param intfcId
     * @return
     */
    private EqpTmplIntfcBo getEqpTmplIntfc(
            List<EqpTmplIntfcBo> eqpTmplIntfcBos, Long intfcId) {
        EqpTmplIntfcBo eqpTmplIntfcBo = null;
        for (EqpTmplIntfcBo etib : eqpTmplIntfcBos) {
            if (intfcId.equals(etib.getIntfcId())) {
                eqpTmplIntfcBo = etib;
                break;
            }
        }
        return eqpTmplIntfcBo;
    }

    /**
     * 使用反射调用接口采集数据
     *
     * @param intfcBo
     * @param paramMap
     * @return
     */
    private PollingResult invoke(IntfcBo intfcBo, Map<String, Object> paramMap) {
        if (StringUtils.isEmpty(intfcBo.getCommName())) {
            logger.warn("CommMode is empty! intfc id=" + intfcBo.getId());
            return null;
        }

        PollingResult pollingResult = null;
        String pkg = "com.yeejoin.amos.iot."
                + intfcBo.getCommName().toLowerCase();
        String firstChar = intfcBo.getCommName().substring(0, 1).toUpperCase();
        String otherChars = intfcBo.getCommName()
                .substring(1, intfcBo.getCommName().length()).toLowerCase();
        String className = pkg + "." + firstChar + otherChars
                + "CollectService";
        try {
            Class<?> c = Class.forName(className);
            Object obj = c.newInstance();
            // 第一个参数写的是方法名,第二个\第三个\...写的是方法参数列表中参数的类型
            Method method = c.getMethod(intfcBo.getName(), Map.class);
            // invoke是执行该方法,并携带参数值
            pollingResult = (PollingResult) method.invoke(obj,
                    new Object[]{paramMap});
        } catch (ClassNotFoundException | InstantiationException
                | IllegalAccessException | NoSuchMethodException
                | SecurityException | IllegalArgumentException
                | InvocationTargetException e) {
            logger.error(e);
        }

        return pollingResult;
    }

    /**
     * 构建接口参数
     *
     * @param eqpTmplAttributes
     * @param intfcBo
     * @param cpblAttrs
     * @param eqpParamMap
     * @return
     */
    private Map<String, Object> buildIntfcParamMap(
            List<EqpTmplAttributeBo> eqpTmplAttributes, IntfcBo intfcBo,
            List<CapacityPropertyDataVo> cpblAttrs,
            Map<String, Object> eqpParamMap) {
        Map<String, Object> paramMap = new HashMap<>();
        // 1.参数来自设备模板
        fillEqpTmplAttrParam(eqpTmplAttributes, eqpParamMap, paramMap);
        // 2.参数来自接口自身
        fillIntfcAttrParam(intfcBo, eqpParamMap, paramMap);
        // 3.参数来自能力属性
        fillCpblAttrParam(cpblAttrs, paramMap);
        return paramMap;
    }

    /**
     * 参数来自能力属性
     *
     * @param cpblAttrs
     * @param paramMap
     */
    private void fillCpblAttrParam(List<CapacityPropertyDataVo> cpblAttrs,
                                   Map<String, Object> paramMap) {
        if (CollectionUtils.isNotEmpty(cpblAttrs)) {
            cpblAttrs.forEach(cpblAttr -> {
                if (cpblAttr.getType().intValue() == AttributeType.CONSTANT
                        .getValue()) {
                    // 设置参数值
                    fillParamValue(cpblAttr.getDataType(), cpblAttr.getName(),
                            cpblAttr.getValueStr(), paramMap);
                }
            });
        }
    }

    /**
     * 参数来自接口自身
     *
     * @param intfcBo
     * @param eqpParamMap
     * @param paramMap
     */
    private void fillIntfcAttrParam(IntfcBo intfcBo,
                                    Map<String, Object> eqpParamMap, Map<String, Object> paramMap) {
        Object paramValue = null;
        String paramKey = null;
        List<IntfcAttribute> intfcAttributes = intfcBo.getIntfcAttrs();
        if (CollectionUtils.isNotEmpty(intfcAttributes)) {
            for (IntfcAttribute intfcAttribute : intfcAttributes) {
                paramKey = IotConstant.EquipmentConstant.KEY_PREFIX_INTFC
                        + intfcBo.getId()
                        + IotConstant.EquipmentConstant.UNDERLINE
                        + intfcAttribute.getId();
                paramValue = eqpParamMap.get(paramKey);
                if (paramValue == null) {
                    paramValue = intfcAttribute.getValueStr();
                }
                // 设置参数值
                fillParamValue(intfcAttribute.getDataType(),
                        intfcAttribute.getName(), paramValue, paramMap);
            }
        }
    }

    /**
     * 参数来自设备模板
     *
     * @param eqpTmplAttributes
     * @param eqpParamMap
     * @param paramMap
     */
    private void fillEqpTmplAttrParam(
            List<EqpTmplAttributeBo> eqpTmplAttributes,
            Map<String, Object> eqpParamMap, Map<String, Object> paramMap) {
        Object paramValue = null;
        String paramKey = null;
        if (CollectionUtils.isNotEmpty(eqpTmplAttributes)) {
            for (EqpTmplAttributeBo eqpTmplAttributeBo : eqpTmplAttributes) {
                paramKey = IotConstant.EquipmentConstant.KEY_PREFIX_EQUIP
                        + eqpTmplAttributeBo.getId();
                paramValue = eqpParamMap.get(paramKey);
                if (paramValue == null) {
                    continue;
                }
                // 设置参数值
                fillParamValue(eqpTmplAttributeBo.getDataType(),
                        eqpTmplAttributeBo.getName(), paramValue, paramMap);
            }
        }
    }

    /**
     * 执行测试
     */
    @Override
    public void run() {
        try {
            // 1.采集一个能力数据
            Map<String, Object> cpblDataMap = fetchCapabilityData(
                    testEqpInputData.getEqpTmplAttributes(),
                    testEqpInputData.getEqpTmplIntfcs(), capabilityBo,
                    testEqpInputData.getPropertyMap());
            // 2.组装数据
            if (MapUtils.isEmpty(cpblDataMap)) {
                TestCapablility testCapablility = new TestCapablility();
                testCapablility.setId(capabilityBo.getId());
                testCapablility.setName(capabilityBo.getName());
                List<TestCpblAttr> attributes = TestUtil.buildDefaultTestCpblAttr(
                		capabilityBo.getCpblAttrs());
                testCapablility.setAttributes(attributes);
                TestEquipmentCache.getInstance().finishOneCpbl(testCapablility);
            } else {
                TestCapablility testCapablility = new TestCapablility();
                testCapablility.setId(capabilityBo.getId());
                testCapablility.setName(capabilityBo.getName());
                List<TestCpblAttr> attributes = TestUtil
                        .buildTestCpblAttr(cpblDataMap,
                                capabilityBo.getCpblAttrs());
                testCapablility.setAttributes(attributes);
                TestEquipmentCache.getInstance().finishOneCpbl(testCapablility);
            }

        } catch (Exception e) {
            logger.error(e);
            TestCapablility testCapablility = new TestCapablility();
            testCapablility.setId(capabilityBo.getId());
            testCapablility.setName(capabilityBo.getName());
            TestEquipmentCache.getInstance().finishOneCpbl(testCapablility);
        }
    }

}
