package com.yeejoin.amos.iot.business.polling.test;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.util.HashMap;
import java.util.Map;

import org.apache.log4j.Logger;

import freemarker.template.Configuration;
import freemarker.template.Template;

/**
 * 测试设备数据导出工具类
 * 
 * @author tianbo
 *
 */
public class TestEqpDataExportUtil {
	
	/**
     * 日志记录器
     */
    private static final Logger logger = Logger
            .getLogger(TestResultStoreUtil.class);
	
	private static Configuration conf = null;
	private static Map<String, Template> allTemplates = null;
	
	static {
		conf = new Configuration(Configuration.VERSION_2_3_23);
		conf.setDefaultEncoding("UTF-8");
		conf.setClassForTemplateLoading(TestEqpDataExportUtil.class, "/template");
		allTemplates = new HashMap<>();
		try {
			allTemplates.put("doc", conf.getTemplate("test.ftl"));
		} catch (IOException e) {
			logger.error(e.getMessage(), e);
			throw new RuntimeException(e);
		}
	}
	
	/**
	 * 导出设备测试结果，生成Word文档
	 * 
	 * @param resultMap 测试结果数据
	 * @param fileType 导出文件类型
	 */
	public static File exportTestEqpResult(Map<Object, Object> resultMap, String fileType) {

		File file = null;
		// 调用createDoc方法生成Word文档
		file = createDoc(resultMap, fileType);
		return file;
	}
	
	private static File createDoc(Map<Object, Object> dataMap, String fileType) {
		String name = "template.doc";
		File f = new File(name);
		Template t = allTemplates.get(fileType);
		try {
			// 这个地方不能使用FileWriter因为需要指定编码类型否则生成的Word文档会因为有无法识别的编码而无法打开
			Writer w = new OutputStreamWriter(new FileOutputStream(f), "utf-8");
			t.process(dataMap, w);
			w.close();
		} catch (Exception ex) {
			logger.error(ex.getMessage(), ex);
			throw new RuntimeException(ex);
		}
		return f;
	}
}
