/**
 *
 */
package com.yeejoin.amos.iot.business.polling.test;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.ReentrantLock;

import net.sf.json.JSONObject;

import org.apache.commons.collections.CollectionUtils;
import org.apache.log4j.Logger;

import com.yeejoin.amos.iot.business.constants.IotConstant;
import com.yeejoin.amos.iot.business.entity.mybatis.CapabilityBo;
import com.yeejoin.amos.iot.business.entity.mybatis.EqpTmplIntfcBo;
import com.yeejoin.amos.iot.context.IotContext;
import com.yeejoin.amos.iot.socket.websocket.CommonMessage;
import com.yeejoin.amos.iot.socket.websocket.WebsocketService;
import com.yeejoin.amos.op.core.util.ApplicationConfig;

/**
 * @author as-youjun
 */
public class TestEquipmentCache {

    /**
     * 日志记录器
     */
    private static final Logger logger = Logger
            .getLogger(TestEquipmentCache.class);

    /**
     * 单例
     */
    private static TestEquipmentCache instance;

    /**
     * 是否正在测试
     */
    private boolean isTesting = false;

    /**
     * 锁
     */
    private ReentrantLock lock = new ReentrantLock();

    /**
     * 接口总数
     */
    private int totalIntfcCount = 0;

    /**
     * 指标总数
     */
    private int totalCpblCount = 0;

    /**
     * 已完成接口id
     */
    private Set<Long> finishedIntfcIds = new HashSet<Long>();

    /**
     * 已完成指标id
     */
    private Set<Long> finishedCpblIds = new HashSet<Long>();

    /**
     * 测试设备数据
     */
    private TestEqpInputData testEqpInputData;

    /**
     * 时间：记录最后一次修改时间
     */
    private long updateTime = 0;

    /**
     * 执行服务
     */
    private ScheduledExecutorService scheduledService;

    /**
     * 构造方法
     */
    private TestEquipmentCache() {
		logger.debug(IotConstant.CONSTRUCTOR);    }

    /**
     * 获取单例
     *
     * @return
     */
    public static TestEquipmentCache getInstance() {
        if (instance == null) {
            synchronized (TestEquipmentCache.class) {
                if (instance == null) {
                    instance = new TestEquipmentCache();
                }
            }
        }
        return instance;
    }

    /**
     * 是否正在测试
     *
     * @return
     */
    public boolean isTesting() {
        return isTesting;
    }

    /**
     * 开始测试
     *
     * @param intfcCount
     * @param cpblCount
     * @param testEqpInputData
     */
    public void startTest(int intfcCount, int cpblCount, TestEqpInputData testEqpInputData) {
        lock.lock();
        try {

            this.isTesting = true;
            this.totalIntfcCount = intfcCount;
            this.totalCpblCount = cpblCount;
            this.testEqpInputData = testEqpInputData;
            this.updateTime = System.currentTimeMillis();

            // 调度超时监测
            scheduledService = Executors
                    .newSingleThreadScheduledExecutor();
            long initialDelay = 1;
            long period = 2;
            // 从现在开始1秒钟之后，每隔2秒钟执行一次job
            scheduledService.scheduleAtFixedRate(new CheckChange(),
                    initialDelay, period, TimeUnit.SECONDS);

        } catch (Exception e) {
            logger.error("startTest error! " + e);
        } finally {
            lock.unlock();
        }
    }

    /**
     * @return the testEqpInputData
     */
    public TestEqpInputData getTestEqpInputData() {
        return testEqpInputData;
    }

    /**
     * 发送消息
     *
     * @param message
     */
    private void sendMessage(CommonMessage message) {
        try {
            WebsocketService websocketService = (WebsocketService) IotContext
                    .getInstance().getBean("websocketService");
            websocketService.sendMessage(message);
        } catch (Exception e) {
            logger.error(e);
        }
    }

    /**
     * 缓存清零
     */
    public void clean() {
        this.testEqpInputData = null;
        this.isTesting = false;
        this.totalCpblCount = 0;
        this.totalIntfcCount = 0;
        this.finishedCpblIds.clear();
        this.finishedIntfcIds.clear();
        this.updateTime = 0;
        if (scheduledService != null) {
            try {
                scheduledService.shutdown();
                if (!scheduledService.awaitTermination(100, TimeUnit.MILLISECONDS)) {
                    scheduledService.shutdownNow();
                }
            } catch (InterruptedException e) {
                logger.error("awaitTermination interrupted: " + e);
                scheduledService.shutdownNow();
            } finally {
                scheduledService = null;
            }
        }
    }

    /**
     * 测试完一个接口
     *
     * @param testIntfc
     */
    public void finishOneIntfc(TestIntfc testIntfc) {
        lock.lock();
        try {
            // 是否需要发送接口测试消息
            boolean isSendMessage = this.testEqpInputData != null
                    && IotConstant.EquipmentConstant.CONNECT
                    .equals(this.testEqpInputData.getFlag())
                    && (!this.finishedIntfcIds.contains(testIntfc.getId()));
            if (isSendMessage) {
                this.finishedIntfcIds.add(testIntfc.getId());
                int finishedCount = this.finishedIntfcIds.size();
                Double percentage = Double.valueOf(finishedCount)
                        / Double.valueOf(totalIntfcCount) * 100.0;
                int percent = percentage.intValue();
                // 发送一个接口测试完成消息
                if (percent >= 100) {
                    // 所有接口测试完成
                    percent = 100;
                }
                testIntfc.setPercent(percent);
                CommonMessage message = new CommonMessage();
                message.setKey(IotConstant.EquipmentConstant.TEST_EQUIPMENT
                        + IotConstant.SHARP + testEqpInputData.getToken());
                Map<String, TestIntfc> dataMap = new HashMap<String, TestIntfc>();
                dataMap.put(IotConstant.EquipmentConstant.CONNECTDATA,
                        testIntfc);
                message.setJsonData(JSONObject.fromObject(dataMap).toString());
                sendMessage(message);             
                // 保存一次连接测试结果到Map中
                TestResultStoreUtil.getInstance().getResultMap().put(IotConstant.EquipmentConstant.TEST_EQUIPMENT
                        + IotConstant.SHARP + "connect" + finishedCount, dataMap);
                
                if (percent >= 100) {
                    // 缓存清零
                    clean();
                }
                this.updateTime = System.currentTimeMillis();
            }
        } catch (Exception e) {
            logger.error("finishAIntfc error! " + e);
        } finally {
            lock.unlock();
        }
    }

    /**
     * 测试完成一个指标
     *
     * @param testCapablility
     */
    public void finishOneCpbl(TestCapablility testCapablility) {
        lock.lock();
        try {
            // 是否需要发送指标测试消息
            boolean isSendMessage = this.testEqpInputData != null
                    && IotConstant.EquipmentConstant.METRIC
                    .equals(this.testEqpInputData.getFlag())
                    && (!this.finishedCpblIds.contains(testCapablility.getId()));
            if (isSendMessage) {
                this.finishedCpblIds.add(testCapablility.getId());
                int finishedCount = this.finishedCpblIds.size();
                Double percentage = Double.valueOf(finishedCount)
                        / Double.valueOf(totalCpblCount) * 100.0;
                int percent = percentage.intValue();
                // 发送一个指标测试完成消息
                if (percent >= 100) {
                    // 所有指标测试完成
                    percent = 100;
                }
                testCapablility.setPercent(percent);
                CommonMessage message = new CommonMessage();
                message.setKey(IotConstant.EquipmentConstant.TEST_EQUIPMENT
                        + IotConstant.SHARP + testEqpInputData.getToken());
                Map<String, TestCapablility> dataMap = new HashMap<String, TestCapablility>();
                dataMap.put(IotConstant.EquipmentConstant.METRICDATA,
                        testCapablility);
                message.setJsonData(JSONObject.fromObject(dataMap).toString());
                
                // 保存一次性能测试结果到Map中
                TestResultStoreUtil.getInstance().getResultMap().put(IotConstant.EquipmentConstant.TEST_EQUIPMENT
                        + IotConstant.SHARP + "metric" + finishedCount, dataMap);
                sendMessage(message);
                if (percent >= 100) {
                    // 缓存清零
                    clean();
                }
                this.updateTime = System.currentTimeMillis();
            }
        } catch (Exception e) {
            logger.error("finishAIntfc error! " + e);
        } finally {
            lock.unlock();
        }
    }

    /**
     * 定期检测设备测试接口调用/指标采集是否成功：如果超过预定义时长没有任何变化，则认为接口调用不通，测试结束
     *
     * @author as-youjun
     */
    class CheckChange implements Runnable {

        @Override
        public void run() {
            if (!isTesting) {
                return;
            }
            long currentTime = System.currentTimeMillis();
            ApplicationConfig applicationConfig = (ApplicationConfig) IotContext
                    .getInstance().getBean("applicationConfig");
            long timeSpan = currentTime - updateTime;
            String testTimeOut = applicationConfig.getParamValueStr("vehicle.test.timeout");
            long maxSpan = Integer.valueOf(testTimeOut) * 1000;
            if (timeSpan > maxSpan) {
                if (!isTesting) {
                    return;
                }

                // 接口调用超时失败
                boolean hasEqpTmplIntfcs = testEqpInputData != null
                        && CollectionUtils.isNotEmpty(testEqpInputData
                        .getEqpTmplIntfcs());
                if (hasEqpTmplIntfcs) {
                    for (EqpTmplIntfcBo eqpTmplIntfcBo : testEqpInputData
                            .getEqpTmplIntfcs()) {
                        if (!isTesting) {
                            break;
                        }
                        if (!finishedIntfcIds.contains(eqpTmplIntfcBo
                                .getIntfcId())) {
                            TestIntfc testIntfc = new TestIntfc();
                            testIntfc.setId(eqpTmplIntfcBo.getIntfcId());
                            testIntfc.setName(eqpTmplIntfcBo
                                    .getIntfcDisplayName());
                            testIntfc.setResult("接口调用失败！原因：超时！");
                            logger.debug("接口调用失败！原因：超时！ intfcId:"
                                    + eqpTmplIntfcBo.getIntfcId()
                                    + ",intfcName:"
                                    + eqpTmplIntfcBo.getIntfcDisplayName());
                            finishOneIntfc(testIntfc);
                        }
                    }
                } else {
                    logger.debug("equipmentBo is null or equipmentBo.getEqpTmplIntfcs() is empty!");
                }
                // 指标获取失败
                boolean hasCpblBos = testEqpInputData != null
                        && CollectionUtils.isNotEmpty(testEqpInputData
                        .getCapabilityBos());
                if (hasCpblBos) {
                    for (CapabilityBo capabilityBo : testEqpInputData
                            .getCapabilityBos()) {
                        if (!isTesting) {
                            break;
                        }
                        if (!finishedCpblIds.contains(capabilityBo.getId())) {
                            TestCapablility testCapablility = new TestCapablility();
                            testCapablility.setId(capabilityBo.getId());
                            testCapablility.setName(capabilityBo.getName());
                            finishOneCpbl(testCapablility);
                        }
                    }
                } else {
                    logger.debug("testEqpInputData is null or equipmentBo.getCapabilityBos() is empty!");
                }
            }
        }

    }

}
