/**
 *
 */
package com.yeejoin.amos.iot.business.polling.test;

import com.yeejoin.amos.iot.business.constants.IotConstant;
import com.yeejoin.amos.iot.business.controller.param.TestEqpParam;
import com.yeejoin.amos.iot.business.dao.mapper.DeviceMapper;
import com.yeejoin.amos.iot.business.dao.mapper.MonitorMapper;
import com.yeejoin.amos.iot.business.dao.repository.*;
import com.yeejoin.amos.iot.business.entity.IntfcAttribute;
import com.yeejoin.amos.iot.business.entity.mybatis.*;
import com.yeejoin.amos.iot.business.enums.CapabilityType;
import com.yeejoin.amos.iot.business.enums.IntfcDirection;
import com.yeejoin.amos.iot.business.enums.MonitorType;
import com.yeejoin.amos.iot.business.param.QueryCapabilityParam;
import com.yeejoin.amos.iot.business.param.QueryEqpTmplAttrParam;
import com.yeejoin.amos.iot.business.param.QueryEqpTmplIntfcParam;
import com.yeejoin.amos.iot.business.param.QueryIntfcParam;
import com.yeejoin.amos.iot.business.polling.MonitorUtil;
import com.yeejoin.amos.iot.core.threadpool.AmosThreadPool;
import com.yeejoin.amos.iot.exception.YeeException;
import org.apache.commons.collections.CollectionUtils;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 测试设备service实现类
 *
 * @author as-youjun
 */
@Service("testEquipmentService")
public class TestEquipmentService {

    /**
     * 日志记录器
     */
    private static final Logger logger = Logger
            .getLogger(TestEquipmentService.class);

    @Autowired
    private MonitorMapper monitorMapper;

    @Autowired
    private DeviceMapper deviceMapper;

    @Autowired
    private IIntfcAttributeDao intfcAttributeDao;

    @Autowired
    private IMetricDataDao metricDataDao;

    @Autowired
    private IEquipmentDao iEquipmentDao;

    @Autowired
    private IIntfcDao intfcDao;

    @Autowired
    private ICapabilityDao iCapabilityDao;

    /**
     * 测试设备
     *
     * @param testEqpParam
     * @param token
     * @param flag
     */
    public void startTest(TestEqpParam testEqpParam, String token, String flag) {
        if (TestEquipmentCache.getInstance().isTesting()) {
            logger.warn("正在测试，请稍后！");
            throw new YeeException("正在测试，请稍后！", 1030001);
        }
        TestEqpInputData testEqpInputData = buildTestEqpInputData(testEqpParam, token, flag);
        if (CollectionUtils.isEmpty(testEqpInputData.getEqpTmplIntfcs())) {
            logger.warn("接口为空,请选择接口！");
            throw new YeeException("接口为空,请选择接口！", 1030002);
        }
        if (CollectionUtils.isEmpty(testEqpInputData.getCapabilityBos())) {
            logger.warn("指标为空,请选择指标！");
            throw new YeeException("指标为空,请选择指标！", 1030003);
        }
        
        // 将设备名称及设备模板ID添加到测试数据结果Map中
        TestResultStoreUtil.getInstance().getResultMap().put("equipmentName", testEqpParam.getBasicInfo().getName());
        TestResultStoreUtil.getInstance().getResultMap().put("eqpTemplateID", testEqpParam.getBasicInfo().getEqpTmplId());
        
        TestEquipmentCache.getInstance().startTest(
                testEqpInputData.getEqpTmplIntfcs().size(),
                testEqpInputData.getCapabilityBos().size(), testEqpInputData);
        // 执行测试业务
        test(testEqpInputData);
    }

    /**
     * 构建TestEqpInputData
     *
     * @param testEqpParam
     * @param token
     * @param flag
     * @return
     */
    public TestEqpInputData buildTestEqpInputData(TestEqpParam testEqpParam, String token, String flag) {
        TestEqpInputData testEqpInputData = new TestEqpInputData();
        testEqpInputData.setPropertyMap(testEqpParam.getPropertyMap());
        testEqpInputData.setFlag(flag);
        testEqpInputData.setToken(token);
        // 2.查询设备能力
        List<Long> eqpTmplIds = new ArrayList<>();
        eqpTmplIds.add(testEqpParam.getBasicInfo().getEqpTmplId());
        QueryCapabilityParam queryCapabilityParam = new QueryCapabilityParam();
        queryCapabilityParam.setCpblType(CapabilityType.COLLECT.getValue());
        queryCapabilityParam.setEqpTmplIds(eqpTmplIds);
        List<CapabilityBo> capabilityBos = monitorMapper
                .findCpblByEqpTmplIds(queryCapabilityParam);
        // 3.查询设备能力接口
        QueryIntfcParam queryIntfcParam = new QueryIntfcParam();
        queryIntfcParam.setCpblType(CapabilityType.COLLECT.getValue());
        queryIntfcParam.setEqpTmplIds(eqpTmplIds);
        List<IntfcBo> intfcBos = monitorMapper
                .findIntfcOfCpblByEqpTmplIds(queryIntfcParam);
        if (CollectionUtils.isNotEmpty(capabilityBos)) {
            // CapabilityBo填充到EquipmentBo
            TestMonitorUtil
                    .fillCapabilityBo2TestEqpInputData(capabilityBos, testEqpInputData);
        }
        List<IntfcAttribute> intfcAttributes = null;
        if (CollectionUtils.isNotEmpty(testEqpInputData.getCapabilityBos())
                && CollectionUtils.isNotEmpty(intfcBos)) {
            // IntfcBo填充到CapabilityBo
            MonitorUtil.fillIntfcBo2CapabilityBo(intfcBos, testEqpInputData.getCapabilityBos());
            // 接口属性填充到接口
            List<Long> intfcIds = new ArrayList<Long>();
            intfcBos.forEach(item -> {
                intfcIds.add(item.getId());
            });
            intfcAttributes = intfcAttributeDao
                    .findAllByIntfcIdIn(intfcIds);
            for (CapabilityBo capabilityBo : testEqpInputData.getCapabilityBos()) {
                MonitorUtil.fillIntfcAttr2Intfc(intfcAttributes, capabilityBo.getIntfcBos());
            }
        }
        // 4.查询设备接口
        QueryEqpTmplIntfcParam queryEqpTmplIntfcParam = new QueryEqpTmplIntfcParam();
        queryEqpTmplIntfcParam.setEqpTmplIds(eqpTmplIds);
        List<EqpTmplIntfcBo> eqpTmplIntfcBos = monitorMapper
                .findEqpTmplIntfcByEqpTmplIds(queryEqpTmplIntfcParam);
        TestMonitorUtil.fillEqpTmplIntfc2TestEqpInputData(eqpTmplIntfcBos, testEqpInputData);
        if (CollectionUtils.isNotEmpty(testEqpInputData.getEqpTmplIntfcs())) {
            MonitorUtil.fillIntfcBo2EqpTmplIntfcBo(intfcBos, testEqpInputData.getEqpTmplIntfcs());
            TestMonitorUtil.fillIntfcAttr2EqpTmplIntfcBo(intfcAttributes, testEqpInputData.getEqpTmplIntfcs());
        }
        // 5.查询设备模板属性
        QueryEqpTmplAttrParam queryEqpTmplAttrParam = new QueryEqpTmplAttrParam();
        queryEqpTmplAttrParam.setEqpTmplIds(eqpTmplIds);
        List<EqpTmplAttributeBo> eqpTmplAttributes = monitorMapper
                .findEqpTmplAttrByEqpTmplIds(queryEqpTmplAttrParam);
        testEqpInputData.setEqpTmplAttributes(eqpTmplAttributes);
        // 6.查询能力属性
        if (CollectionUtils.isNotEmpty(testEqpInputData.getCapabilityBos())) {
            List<Long> cpblIds = new ArrayList<Long>();
            testEqpInputData.getCapabilityBos().forEach(cbo -> {
                cpblIds.add(cbo.getId());
            });
            List<CapacityPropertyDataVo> cpblAttrs = deviceMapper
                    .getCapacityPropertyByCpblIds(cpblIds);
            Map<String, List<Object>> cpblAttrMap = (Map<String, List<Object>>) testEqpParam
                    .getPropertyMap().get(IotConstant.EquipmentConstant.QUOTASELECTED);
            MonitorUtil.fillCpblAttr2Capablity(cpblAttrs, testEqpInputData.getCapabilityBos(), cpblAttrMap);
        }
        return testEqpInputData;
    }

    /**
     * 测试设备
     *
     * @param testEqpInputData
     */
    private void test(TestEqpInputData testEqpInputData) {
        // 1.开启线程执行job
        //1.1测试接口
        if (IotConstant.EquipmentConstant.CONNECT.equals(testEqpInputData.getFlag())) {
            for (EqpTmplIntfcBo eqpTmplIntfcBo : testEqpInputData.getEqpTmplIntfcs()) {
                if (eqpTmplIntfcBo.getIntfcDirection().intValue() == IntfcDirection.DOWN
                        .getValue()) {
                    TestIntfcExecuteUnit executeUnit = new TestIntfcExecuteUnit(eqpTmplIntfcBo, testEqpInputData);
                    AmosThreadPool.getInstance().execute(executeUnit);
                }
            }
        }
        //1.2测试指标
        if (IotConstant.EquipmentConstant.METRIC.equals(testEqpInputData.getFlag())) {
            for (CapabilityBo capabilityBo : testEqpInputData.getCapabilityBos()) {
                if (CollectionUtils.isEmpty(capabilityBo.getIntfcBos())) {
                    TestCapablility testCapablility = new TestCapablility();
                    testCapablility.setId(capabilityBo.getId());
                    testCapablility.setName(capabilityBo.getName());
                    TestEquipmentCache.getInstance().finishOneCpbl(testCapablility);
                    continue;
                }
                if (capabilityBo.getMonitorType().intValue() == MonitorType.COLLECT
                        .getValue()) {// 采集
                    TestCpblExecuteUnit executeUnit = new TestCpblExecuteUnit(
                            capabilityBo, testEqpInputData);
                    AmosThreadPool.getInstance().execute(executeUnit);
                }
            }
        }
    }

}
