/**
 *
 */
package com.yeejoin.amos.iot.business.polling.test;

import com.yeejoin.amos.iot.business.constants.IotConstant;
import com.yeejoin.amos.iot.business.entity.IntfcAttribute;
import com.yeejoin.amos.iot.business.entity.mybatis.EqpTmplAttributeBo;
import com.yeejoin.amos.iot.business.entity.mybatis.EqpTmplIntfcBo;
import com.yeejoin.amos.iot.business.polling.BaseExecuteUnit;
import com.yeejoin.amos.iot.business.polling.PollingResult;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 接口/通道测试执行单元
 *
 * @author as-youjun
 */
public class TestIntfcExecuteUnit extends BaseExecuteUnit {

    private EqpTmplIntfcBo eqpTmplIntfcBo;

    private TestEqpInputData testEqpInputData;

    /**
     * 日志记录器
     */
    private static final Logger logger = Logger
            .getLogger(TestIntfcExecuteUnit.class);

    /**
     * 构造方法
     *
     * @param eqpTmplIntfcBo
     * @param testEqpInputData
     */
    public TestIntfcExecuteUnit(EqpTmplIntfcBo eqpTmplIntfcBo,
                                TestEqpInputData testEqpInputData) {
        this.eqpTmplIntfcBo = eqpTmplIntfcBo;
        this.testEqpInputData = testEqpInputData;
    }

    /**
     * 使用反射调用接口采集数据
     *
     * @param eqpTmplIntfcBo
     * @param paramMap
     * @return
     */
    private PollingResult invoke(EqpTmplIntfcBo eqpTmplIntfcBo, Map<String, Object> paramMap) {
        if (StringUtils.isEmpty(eqpTmplIntfcBo.getCommName())) {
            logger.warn("CommMode is empty! intfc id=" + eqpTmplIntfcBo.getIntfcId());
            return null;
        }

        PollingResult pollingResult = null;
        String pkg = "com.yeejoin.amos.iot."
                + eqpTmplIntfcBo.getCommName().toLowerCase();
        String firstChar = eqpTmplIntfcBo.getCommName().substring(0, 1).toUpperCase();
        String otherChars = eqpTmplIntfcBo.getCommName()
                .substring(1, eqpTmplIntfcBo.getCommName().length()).toLowerCase();
        String className = pkg + "." + firstChar + otherChars
                + "CollectService";
        try {
            Class<?> c = Class.forName(className);
            Object obj = c.newInstance();
            // 第一个参数写的是方法名,第二个\第三个\...写的是方法参数列表中参数的类型
            Method method = c.getMethod(eqpTmplIntfcBo.getIntfcName(), Map.class);
            // invoke是执行该方法,并携带参数值
            pollingResult = (PollingResult) method.invoke(obj,
                    new Object[]{paramMap});
        } catch (ClassNotFoundException | InstantiationException
                | IllegalAccessException | NoSuchMethodException
                | SecurityException | IllegalArgumentException
                | InvocationTargetException e) {
            logger.error(e);
        }

        return pollingResult;
    }

    /**
     * 构建接口参数
     *
     * @param eqpTmplAttributes
     * @param eqpTmplIntfcBo
     * @param eqpParamMap
     * @return
     */
    private Map<String, Object> buildIntfcParamMap(
            List<EqpTmplAttributeBo> eqpTmplAttributes, EqpTmplIntfcBo eqpTmplIntfcBo,
            Map<String, Object> eqpParamMap) {
        Map<String, Object> paramMap = new HashMap<>();
        // 1.参数来自设备模板
        fillEqpTmplAttrParam(eqpTmplAttributes, eqpParamMap, paramMap);
        // 2.参数来自接口自身
        fillIntfcAttrParam(eqpTmplIntfcBo, eqpParamMap, paramMap);
        return paramMap;
    }

    /**
     * 参数来自接口自身
     *
     * @param eqpTmplIntfcBo
     * @param eqpParamMap
     * @param paramMap
     */
    private void fillIntfcAttrParam(EqpTmplIntfcBo eqpTmplIntfcBo,
                                    Map<String, Object> eqpParamMap, Map<String, Object> paramMap) {
        Object paramValue = null;
        String paramKey = null;
        List<IntfcAttribute> intfcAttributes = eqpTmplIntfcBo.getIntfcAttrs();
        if (CollectionUtils.isNotEmpty(intfcAttributes)) {
            for (IntfcAttribute intfcAttribute : intfcAttributes) {
                paramKey = IotConstant.EquipmentConstant.KEY_PREFIX_INTFC
                        + eqpTmplIntfcBo.getIntfcId()
                        + IotConstant.EquipmentConstant.UNDERLINE
                        + intfcAttribute.getId();
                paramValue = eqpParamMap.get(paramKey);
                if (paramValue == null) {
                    paramValue = intfcAttribute.getValueStr();
                }
                // 设置参数值
                fillParamValue(intfcAttribute.getDataType(),
                        intfcAttribute.getName(), paramValue, paramMap);
            }
        }
    }

    /**
     * 参数来自设备模板
     *
     * @param eqpTmplAttributes
     * @param eqpParamMap
     * @param paramMap
     */
    private void fillEqpTmplAttrParam(
            List<EqpTmplAttributeBo> eqpTmplAttributes,
            Map<String, Object> eqpParamMap, Map<String, Object> paramMap) {
        Object paramValue = null;
        String paramKey = null;
        if (CollectionUtils.isNotEmpty(eqpTmplAttributes)) {
            for (EqpTmplAttributeBo eqpTmplAttributeBo : eqpTmplAttributes) {
                paramKey = IotConstant.EquipmentConstant.KEY_PREFIX_EQUIP
                        + eqpTmplAttributeBo.getId();
                paramValue = eqpParamMap.get(paramKey);
                if (paramValue == null) {
                    continue;
                }
                // 设置参数值
                fillParamValue(eqpTmplAttributeBo.getDataType(),
                        eqpTmplAttributeBo.getName(), paramValue, paramMap);
            }
        }
    }

    /**
     * 执行测试
     */
    @Override
    public void run() {
        //测试接口
        try {
            Map<String, Object> paramMap = buildIntfcParamMap(
                    testEqpInputData.getEqpTmplAttributes(), eqpTmplIntfcBo,
                    testEqpInputData.getPropertyMap());
            Logger.getLogger(this.getClass()).debug(
                    "before invoke: intfcId="
                            + eqpTmplIntfcBo.getIntfcId() + ",intfcName="
                            + eqpTmplIntfcBo.getIntfcName());
            PollingResult pollingResult = invoke(eqpTmplIntfcBo, paramMap);
            if (pollingResult == null) {
                TestIntfc testIntfc = new TestIntfc();
                testIntfc.setId(eqpTmplIntfcBo.getIntfcId());
                testIntfc.setName(eqpTmplIntfcBo.getIntfcName());
                testIntfc.setResult("接口调用失败！原因：通信模式为空！");
                TestEquipmentCache.getInstance().finishOneIntfc(testIntfc);
            } else if (pollingResult.getStatus() == IotConstant.FAILURE) {
                TestIntfc testIntfc = new TestIntfc();
                testIntfc.setId(eqpTmplIntfcBo.getIntfcId());
                testIntfc.setName(eqpTmplIntfcBo.getIntfcName());
                testIntfc.setResult("接口调用失败！原因" + pollingResult.getErrorMsg());
                TestEquipmentCache.getInstance().finishOneIntfc(testIntfc);
            } else {
                TestIntfc testIntfc = new TestIntfc();
                testIntfc.setId(eqpTmplIntfcBo.getIntfcId());
                testIntfc.setName(eqpTmplIntfcBo.getIntfcName());
                testIntfc.setResult("ok");
                TestEquipmentCache.getInstance().finishOneIntfc(testIntfc);
            }
        } catch (Exception e) {
            logger.error(e);
            TestIntfc testIntfc = new TestIntfc();
            testIntfc.setId(eqpTmplIntfcBo.getIntfcId());
            testIntfc.setName(eqpTmplIntfcBo.getIntfcName());
            testIntfc.setResult("接口调用失败！原因" + e.getMessage());
            TestEquipmentCache.getInstance().finishOneIntfc(testIntfc);
        }
    }

}