package com.yeejoin.amos.iot.business.polling.test;

import com.yeejoin.amos.iot.business.constants.IotConstant;
import com.yeejoin.amos.iot.business.entity.IntfcAttribute;
import com.yeejoin.amos.iot.business.entity.mybatis.CapabilityBo;
import com.yeejoin.amos.iot.business.entity.mybatis.EqpTmplIntfcBo;
import org.apache.commons.collections.CollectionUtils;
import org.apache.log4j.Logger;

import java.util.ArrayList;
import java.util.List;

/**
 * 设备测试工具类
 */
public class TestMonitorUtil {

    /**
     * 日志记录器
     */
    private static final Logger logger = Logger.getLogger(TestMonitorUtil.class);

    /**
     * CapabilityBo填充到TestEqpInputData
     *
     * @param capabilityBos
     * @param testEqpInputData
     */
    @SuppressWarnings("unchecked")
    public static void fillCapabilityBo2TestEqpInputData(
            List<CapabilityBo> capabilityBos, TestEqpInputData testEqpInputData) {
        List<CapabilityBo> cbos = new ArrayList<CapabilityBo>();
        for (CapabilityBo capabilityBo : capabilityBos) {
            Object selCpblIdObj = testEqpInputData.getPropertyMap().get(
                    IotConstant.EquipmentConstant.KEY_SELCPBLIDS);
            boolean selCpbl = false;// 是否选择能力
            if (selCpblIdObj == null) {
                selCpbl = true;
            } else {
                try {
                    List<Object> selCpblIdObjs = (List<Object>) selCpblIdObj;
                    if (CollectionUtils.isEmpty(selCpblIdObjs)) {
                        selCpbl = true;
                    } else {
                        for (Object obj : selCpblIdObjs) {
                            Long selCpblId = Long.valueOf(obj.toString());
                            if (selCpblId.longValue() == capabilityBo.getId().longValue()) {
                                selCpbl = true;
                                break;
                            }
                        }
                    }
                } catch (Exception e) {
                    logger.error("transform param error! " + e);
                }
            }
            if (selCpbl) {// 页面上勾选此能力
                CapabilityBo newCapabilityBo = (CapabilityBo) capabilityBo.clone();
                cbos.add(newCapabilityBo);
            }
        }
        testEqpInputData.setCapabilityBos(cbos);
    }

    /**
     * 设备接口填充到TestEqpInputData
     *
     * @param eqpTmplIntfcBos
     * @param testEqpInputData
     */
    @SuppressWarnings("unchecked")
    public static void fillEqpTmplIntfc2TestEqpInputData(
            List<EqpTmplIntfcBo> eqpTmplIntfcBos, TestEqpInputData testEqpInputData) {
        if (CollectionUtils.isEmpty(eqpTmplIntfcBos)) {
            return;
        }
        List<Long> intfcIds = new ArrayList<Long>();
        List<EqpTmplIntfcBo> ibos = new ArrayList<EqpTmplIntfcBo>();
        for (EqpTmplIntfcBo eqpTmplIntfcBo : eqpTmplIntfcBos) {
            Object selTmplIntfcIdobj = testEqpInputData.getPropertyMap().get(
                    IotConstant.EquipmentConstant.KEY_SELINTFCIDS);
            boolean selIntfc = false;// 是否选择接口
            if (selTmplIntfcIdobj == null) {
                selIntfc = false;
            } else {
                try {
                    List<Object> selTmplIntfcIds = (List<Object>) selTmplIntfcIdobj;
                    if (CollectionUtils.isEmpty(selTmplIntfcIds)) {
                        selIntfc = false;
                    } else {
                        for (Object obj : selTmplIntfcIds) {
                            Long selTmplIntfcId = Long.valueOf(obj.toString());
                            if (selTmplIntfcId.longValue() == eqpTmplIntfcBo.getIntfcId().longValue()) {
                                selIntfc = true;
                                break;
                            }
                        }
                    }
                } catch (Exception e) {
                    logger.error("transform param error! " + e);
                }
            }
            //
            if (selIntfc) {// 页面上勾选此接口
                if (!intfcIds.contains(eqpTmplIntfcBo.getIntfcId())) {
                    EqpTmplIntfcBo newEqpTmplIntfcBo = (EqpTmplIntfcBo) eqpTmplIntfcBo.clone();
                    ibos.add(newEqpTmplIntfcBo);
                    intfcIds.add(eqpTmplIntfcBo.getIntfcId());
                }
            }
        }
        testEqpInputData.setEqpTmplIntfcs(ibos);
    }

    /**
     * 接口属性填充到设备接口
     *
     * @param intfcAttributes
     * @param eqpTmplIntfcBos
     */
    public static void fillIntfcAttr2EqpTmplIntfcBo(
            List<IntfcAttribute> intfcAttributes, List<EqpTmplIntfcBo> eqpTmplIntfcBos) {
        if (CollectionUtils.isNotEmpty(intfcAttributes) && CollectionUtils.isNotEmpty(eqpTmplIntfcBos)) {
            eqpTmplIntfcBos.forEach(eqpTmplIntfcBo -> {
                intfcAttributes.forEach(intfcAttr -> {
                    if (intfcAttr.getIntfcId().equals(eqpTmplIntfcBo.getIntfcId())) {
                        if (eqpTmplIntfcBo.getIntfcAttrs() == null) {
                            eqpTmplIntfcBo.setIntfcAttrs(new ArrayList<IntfcAttribute>());
                        }
                        IntfcAttribute newIntfcAttribute = (IntfcAttribute) intfcAttr.clone();
                        eqpTmplIntfcBo.getIntfcAttrs().add(newIntfcAttribute);
                    }
                });
            });
        } else {
            logger.debug("intfcBos is empty or intfcAttributes is empty!");
        }
    }

}
