/**
 *
 */
package com.yeejoin.amos.iot.business.polling.test;

import com.yeejoin.amos.iot.business.constants.IotConstant;
import com.yeejoin.amos.iot.business.entity.mybatis.CapacityPropertyDataVo;
import com.yeejoin.amos.iot.business.entity.mybatis.EqpTmplAttributeBo;
import com.yeejoin.amos.iot.business.enums.CapabilityAttributeType;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.collections.MapUtils;
import org.apache.log4j.Logger;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 测试工具类
 *
 * @author as-youjun
 */
public class TestUtil {

    /**
     * 日志记录器
     */
    private static final Logger logger = Logger.getLogger(TestUtil.class);

    /**
     * 构建指标采集结果
     *
     * @param cpblDataMap
     * @param cpblAttrs
     * @return
     */
    public static List<TestCpblAttr> buildTestCpblAttr(Map<String, Object> cpblDataMap,
                                                       List<CapacityPropertyDataVo> cpblAttrs) {
        List<TestCpblAttr> testCpblAttrs = new ArrayList<TestCpblAttr>();
        if (MapUtils.isEmpty(cpblDataMap)) {
            return testCpblAttrs;
        }
        cpblAttrs.forEach(item -> {
            if (item.getType().intValue() != CapabilityAttributeType.CONSTANT
                    .getValue()) {
                Object objValue = cpblDataMap.get(item.getName());
                String attrValue = objValue == null ? "" : objValue.toString();
                TestCpblAttr testCpblAttr = new TestCpblAttr();
                testCpblAttr.setId(item.getId());
                testCpblAttr.setName(item.getDisplayName());
                testCpblAttr.setValue(attrValue);
                testCpblAttrs.add(testCpblAttr);
            }
        });
        return testCpblAttrs;
    }

    /**
     * 获取设备参数
     *
     * @param testEqpInputData
     * @return
     */
    public static Map<String, Object> getEqpParamValueMap(
            TestEqpInputData testEqpInputData) {
        Map<String, Object> eqpParmaValueMap = new HashMap<String, Object>();
        List<EqpTmplAttributeBo> eqpTmplAttributes = testEqpInputData
                .getEqpTmplAttributes();
        Map<String, Object> valueMap = testEqpInputData.getPropertyMap();
        if (CollectionUtils.isEmpty(eqpTmplAttributes)) {
            return eqpParmaValueMap;
        }
        if (MapUtils.isEmpty(valueMap)) {
            return eqpParmaValueMap;
        }
        String key = null;
        Object value = null;
        for (EqpTmplAttributeBo eqpTmplAttributeBo : eqpTmplAttributes) {
            key = IotConstant.EquipmentConstant.KEY_PREFIX_EQUIP
                    + eqpTmplAttributeBo.getId();
            value = valueMap.get(key);
            eqpParmaValueMap.put(eqpTmplAttributeBo.getName(), value);
        }
        return eqpParmaValueMap;
    }

    /**
     * 当因为测试条件不正确导致测试结果为空时，构建默认指标采集结果
     * 
     * @param cpblAttrs	能力属性值列表
     * @return
     */
	public static List<TestCpblAttr> buildDefaultTestCpblAttr(List<CapacityPropertyDataVo> cpblAttrs) {
		List<TestCpblAttr> testCpblAttrs = new ArrayList<TestCpblAttr>();

		cpblAttrs.forEach(item -> {
			if (item.getType().intValue() != CapabilityAttributeType.CONSTANT.getValue()) {
				String attrValue = "--";	// 使用默认字符串（"--"）来标识该能力属性没有进行正常测试
				TestCpblAttr testCpblAttr = new TestCpblAttr();
				testCpblAttr.setId(item.getId());
				testCpblAttr.setName(item.getDisplayName());
				testCpblAttr.setValue(attrValue);
				
				testCpblAttrs.add(testCpblAttr);
				logger.debug("testCpblAttr: " + testCpblAttr);
			}
		});
		return testCpblAttrs;
	}
	
}
