/**
 * 
 */
package com.yeejoin.amos.iot.business.queue;

import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.LinkedBlockingQueue;

import com.yeejoin.amos.iot.business.queue.handler.*;
import org.apache.log4j.Logger;

import com.yeejoin.amos.iot.business.constants.IotConstant;

/**
 * 构建告警队列
 * 
 * @author AS-lichenglong
 *
 */
public class AlarmDataQueue {

	/**
	 * 单例
	 */
	private static AlarmDataQueue instance;
	/**
	 * 队列
	 */
	private static final BlockingQueue<AlarmDataObj> blockingQueue = new LinkedBlockingQueue<AlarmDataObj>();

	/**
	 * 控制消费者线程运行状态
	 */
	private static boolean isRunning = true;
	private Map<String, IDataAlarm> iDataAlarmMap = new HashMap<>();

	/**
	 * 构造方法
	 */
	private AlarmDataQueue() {
		iDataAlarmMap.put(IotConstant.METRIC, new MetricDataAlarmHandler());
		iDataAlarmMap.put(IotConstant.CHANNEL, new ChannelDataAlarmHandler());
		iDataAlarmMap.put(IotConstant.FRAME_EXECPTION,
				new FrameDataAlarmHandler());
		iDataAlarmMap.put(IotConstant.EQUIPMENT_STATUS,
				new EquimentStatusDataAlarmHandler());
		iDataAlarmMap.put(IotConstant.EQUIPMENT_CONNECTION,
				new EqpConnAlarmHandler());
	}

	public Map<String, IDataAlarm> getiDataAlarmMap() {
		return iDataAlarmMap;
	}

	public void setiDataAlarmMap(Map<String, IDataAlarm> iDataAlarmMap) {
		this.iDataAlarmMap = iDataAlarmMap;
	}

	/**
	 * 获取单例
	 * 
	 * @return
	 */
	public static AlarmDataQueue getInstance() {
		if (instance == null) {
			synchronized (AlarmDataQueue.class) {
				if (instance == null) {
					instance = new AlarmDataQueue();
				}
			}
		}
		return instance;
	}

	/**
	 * 添加数据
	 * 
	 * @param data
	 */
	public void add(AlarmDataObj data) {
		while (blockingQueue.add(data)) {
			break;
		}
	}

	public void start() {
		Thread thread = new Thread(new VehicleDataConsumer(),
				"vehicle data consumer thread");
		thread.start();
	}

	/**
	 * 能力属性消费者线程
	 * 
	 *
	 */
	class VehicleDataConsumer implements Runnable {

		/**
		 * 1.从队列中取能力属性；2.构建告警；
		 */
		@Override
		public void run() {
			while (isRunning) {
				// // 1.从队列中取出帧
				AlarmDataObj data = null;
				try {
					data = blockingQueue.take();
				} catch (InterruptedException e) {
					Logger.getLogger(this.getClass()).error(e);
				}
				// 2.调用业务接口；
				if (data == null) {
					Logger.getLogger(this.getClass()).warn("MetricData is empty!");
				} else {
					try {
						Object key = data.getType();
						IDataAlarm da = iDataAlarmMap.get(key.toString());
						da.handData(data);
					} catch (Exception e) {
						// TODO: handle exception
						e.printStackTrace();
						Logger.getLogger(this.getClass()).error(
								"Data is error!");
					}
				}

			}
		}
	}

}
