package com.yeejoin.amos.iot.business.queue.handler;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import net.sf.json.JSONObject;

import org.apache.commons.collections.CollectionUtils;

import com.yeejoin.amos.iot.business.constants.IotConstant;
import com.yeejoin.amos.iot.business.dao.mapper.DeviceMapper;
import com.yeejoin.amos.iot.business.dao.mapper.MonitorMapper;
import com.yeejoin.amos.iot.business.entity.Equipment;
import com.yeejoin.amos.iot.business.entity.mybatis.EqpGraphNodeBo;
import com.yeejoin.amos.iot.business.entity.mybatis.EqpTmplAttributeBo;
import com.yeejoin.amos.iot.business.param.QueryEqpTmplAttrParam;
import com.yeejoin.amos.iot.business.queue.AlarmDataObj;
import com.yeejoin.amos.iot.business.service.intfc.IAlarmService;
import com.yeejoin.amos.iot.business.service.intfc.IDeviceService;
import com.yeejoin.amos.iot.business.service.intfc.IEqpCpblPropertyService;
import com.yeejoin.amos.iot.context.IotContext;

public abstract class AbstractDataAlarm implements IDataAlarm {

    protected DeviceMapper deviceMapper;

    protected IAlarmService alarmService;

    protected IDeviceService deviceService;

    protected MonitorMapper monitorMapper;

    protected IEqpCpblPropertyService eqpCpblPropertyService;

    @Override
    public void handData(AlarmDataObj alarmDataObj) {
        //
    }

    protected void registerService() {
        deviceMapper = (DeviceMapper) IotContext.getInstance().getBean(
                "deviceMapper");
        alarmService = (IAlarmService) IotContext.getInstance().getBean(
                "alarmService");
        deviceService = (IDeviceService) IotContext.getInstance().getBean(
                "deviceService");
        monitorMapper = (MonitorMapper) IotContext.getInstance().getBean(
                "monitorMapper");
        eqpCpblPropertyService = (IEqpCpblPropertyService) IotContext.getInstance()
        		.getBean("EqpCpblPropertyService");
    }

    /**
     * 根据设备id查询EqpGraphNodeBo
     *
     * @param eqpId
     * @return
     */
    protected EqpGraphNodeBo findEqpGraphNodeByEqpId(Long eqpId) {
        EqpGraphNodeBo eqpGraphNodeBo = null;
        List<EqpGraphNodeBo> eqpGraphNodeBos = monitorMapper
                .findEqpGraphNodeByEqpId(eqpId);
        if (CollectionUtils.isNotEmpty(eqpGraphNodeBos)) {
            eqpGraphNodeBo = eqpGraphNodeBos.get(0);
        }
        return eqpGraphNodeBo;
    }

    /**
     * 获取设备属性集合
     *
     * @param equipment
     * @return
     */
    @SuppressWarnings("unchecked")
    protected Map<String, Object> getEqpAttrMap(Equipment equipment) {
        Map<String, Object> attrMap = new HashMap<>();
        List<Long> eqpTmplIds = new ArrayList<>();
        eqpTmplIds.add(equipment.getEqpTmplId());
        QueryEqpTmplAttrParam queryEqpTmplAttrParam = new QueryEqpTmplAttrParam();
        queryEqpTmplAttrParam.setEqpTmplIds(eqpTmplIds);
        List<EqpTmplAttributeBo> eqpTmplAttributes = monitorMapper
                .findEqpTmplAttrByEqpTmplIds(queryEqpTmplAttrParam);
        if (CollectionUtils.isNotEmpty(eqpTmplAttributes)) {
            Map<String, Object> paramMap = (Map<String, Object>) JSONObject
                    .toBean(JSONObject.fromObject(equipment), Map.class);
            String key = null;
            Object value = null;
            for (EqpTmplAttributeBo eqpTmplAttributeBo : eqpTmplAttributes) {
                key = IotConstant.EquipmentConstant.KEY_PREFIX_EQUIP
                        + eqpTmplAttributeBo.getId();
                value = paramMap.get(key);
                if (value != null) {
                    attrMap.put(eqpTmplAttributeBo.getName(), value);
                }
            }
        }
        return attrMap;
    }

}
