package com.yeejoin.amos.iot.business.queue.handler;

import com.yeejoin.amos.iot.business.constants.IotConstant;
import com.yeejoin.amos.iot.business.entity.Alarm;
import com.yeejoin.amos.iot.business.entity.Equipment;
import com.yeejoin.amos.iot.business.entity.MetricData;
import com.yeejoin.amos.iot.business.entity.mybatis.EqpGraphNodeBo;
import com.yeejoin.amos.iot.business.enums.AlarmLevel;
import com.yeejoin.amos.iot.business.enums.AlarmStatus;
import com.yeejoin.amos.iot.business.enums.AlarmType;
import com.yeejoin.amos.iot.business.queue.AlarmDataObj;
import net.sf.json.JSONObject;
import org.apache.log4j.Logger;

import java.util.Date;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * 设备状态告警处理
 *
 * @author as-lichenglong
 */
public class EquimentStatusDataAlarmHandler extends AbstractDataAlarm implements
        IDataAlarm {

    private Map<String, Boolean> lastAlarmOfPortMap = new ConcurrentHashMap<String, Boolean>();

    public EquimentStatusDataAlarmHandler() {
        registerService();
    }

    @Override
    public void handData(AlarmDataObj alarmDataObj) {
        Object obj = alarmDataObj.getDataObj();
        MetricData data = (MetricData) obj;
        String json = data.getMetricJson();
        @SuppressWarnings("unchecked")
        Map<String, Object> sourceDataMap = (Map<String, Object>) JSONObject
                .toBean(JSONObject.fromObject(json), Map.class);
        if (sourceDataMap.keySet().contains("status")) {
            Object value = sourceDataMap.get("status");
            if (value == null) {
            } else {
                Equipment equipment = deviceService.findEquipmentById(data
                        .getEqpId());
                compareThreshold(value, equipment);
            }
        }
    }

    // 比较门限
    private void compareThreshold(Object value, Equipment equipment) {
        // 从缓存中取出上次的告警标志
        String equFlag = new StringBuilder().append(equipment.getId())
                .append("#").append(equipment.getEqpTmplId()).toString();
        Boolean lastAlarmFlag = lastAlarmOfPortMap.get(equFlag);
        if (null == lastAlarmFlag) {

            // 判断上次是否产生告警
            lastAlarmFlag = false;
            lastAlarmOfPortMap.put(equFlag, lastAlarmFlag);
        }
        try {
            boolean isOver = value.equals(IotConstant.OPEN);
            if (isOver) {
                // 产生清除告警
                if (lastAlarmFlag) {
                    buildClearAlarm(value, equipment);
                    lastAlarmFlag = false;
                    lastAlarmOfPortMap.put(equFlag, lastAlarmFlag);
                }
            } else {
                // 产生新告警
                buildAlarm(value, equipment);
                lastAlarmFlag = true;
                lastAlarmOfPortMap.put(equFlag, lastAlarmFlag);
            }
        } catch (Exception e) {
            Logger.getLogger(this.getClass()).error(e);
            Logger.getLogger(this.getClass()).error(
                    "equipment status data is not  threshold" + value);
        }
    }

    // 构建新告警
    private void buildAlarm(Object value, Equipment equipment) {
        if (equipment == null) {
            Logger.getLogger(this.getClass()).warn(
                    "没有添加设备，无法构建告警，设备状态 equipment:" + equipment);
            return;
        }
        EqpGraphNodeBo eqpGraphNodeBo = findEqpGraphNodeByEqpId(equipment
                .getId());
        if (eqpGraphNodeBo == null) {
            Logger.getLogger(this.getClass()).warn(
                    "没有视图，无法构建告警，设备状态 equipment:" + equipment);
            return;
        }
        Alarm alarm = new Alarm();
        alarm.setName(equipment.getName());
        Map<String, Object> attrMap = getEqpAttrMap(equipment);
        String ip = attrMap.get(IotConstant.IP) == null ? "" : attrMap.get(
                IotConstant.IP).toString();
        alarm.setDescr(ip + "设备掉线！");
        alarm.setType(AlarmType.DOWN.getValue());
        alarm.setLevel(AlarmLevel.CRITICAL.getValue());
        alarm.setStatus(AlarmStatus.RAISE.getValue());
        alarm.setRaiseTime(new Date());
        alarm.setCount(1);
        alarm.setEqpId(equipment.getId());
        alarm.setOrgCode(equipment.getOrgCode());
        alarm.setSubgraphId(eqpGraphNodeBo.getSubGraphId());
        alarmService.raise(alarm, false);
    }

    // 构建清除告警
    private void buildClearAlarm(Object value, Equipment equipment) {
        Alarm alarm = new Alarm();
        alarm.setName(equipment.getName());
        alarm.setType(AlarmType.DOWN.getValue());
        alarm.setLevel(AlarmLevel.CRITICAL.getValue());
        alarm.setStatus(AlarmStatus.RAISE.getValue());
        alarm.setClearTime(new Date());
        alarm.setEqpId(equipment.getId());
        alarm.setOrgCode(equipment.getOrgCode());
        alarmService.raise(alarm, true);
    }

}
