package com.yeejoin.amos.iot.business.queue.handler;

import java.util.Date;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import org.apache.log4j.Logger;

import com.yeejoin.amos.iot.business.constants.IotConstant;
import com.yeejoin.amos.iot.business.entity.Alarm;
import com.yeejoin.amos.iot.business.entity.Equipment;
import com.yeejoin.amos.iot.business.entity.mybatis.EqpGraphNodeBo;
import com.yeejoin.amos.iot.business.enums.AlarmLevel;
import com.yeejoin.amos.iot.business.enums.AlarmStatus;
import com.yeejoin.amos.iot.business.enums.AlarmType;
import com.yeejoin.amos.iot.business.queue.AlarmDataObj;
import com.yeejoin.amos.iot.business.queue.ChannelAlarmData;

/**
 * 采集异常告警处理
 * 
 * @author as-lichenglong
 *
 */

public class FrameDataAlarmHandler extends AbstractDataAlarm implements
		IDataAlarm {
	private Map<String, Boolean> lastAlarmOfPortMap = new ConcurrentHashMap<String, Boolean>();

	public FrameDataAlarmHandler() {
		registerService();
	}

	@Override
	public void handData(AlarmDataObj alarmDataObj) {
		Object obj = alarmDataObj.getDataObj();
		ChannelAlarmData channelAlarmData = (ChannelAlarmData) obj;

		String type = channelAlarmData.getType();
		String sId = channelAlarmData.getsId();
		Boolean lastAlarmFlag = lastAlarmOfPortMap.get(sId);
		if (null == lastAlarmFlag) {
			// 判断上次是否产生告警
			lastAlarmFlag = false;
			lastAlarmOfPortMap.put(sId, lastAlarmFlag);
		}
		if (type.equals(IotConstant.UNNORMAL)) {
			buildAlarm(channelAlarmData);
			lastAlarmFlag = true;
		} else if (type.equals(IotConstant.NORMAL) && lastAlarmFlag) {
			buildClearAlarm(channelAlarmData);
			lastAlarmFlag = false;
		} else {
			Logger.getLogger(this.getClass()).info("type is not" + type);
		}
		lastAlarmOfPortMap.put(sId, lastAlarmFlag);
	}

	// 构建新告警
	private void buildAlarm(ChannelAlarmData channelAlarmData) {
		String sid = channelAlarmData.getsId();
		Equipment equipment = deviceService.findByCfgParamJson(sid);
		if (equipment == null) {
			Logger.getLogger(this.getClass()).warn(
					"没有配置设备，无法构建告警，设备equipment:" + equipment);
			return;
		}
		EqpGraphNodeBo eqpGraphNodeBo = findEqpGraphNodeByEqpId(equipment
				.getId());
		if (eqpGraphNodeBo == null) {
			Logger.getLogger(this.getClass()).warn(
					"没有视图，无法构建告警，设备equipment:" + equipment);
			return;
		}
		Alarm alarm = new Alarm();
		alarm.setName(equipment.getName());
		alarm.setDescr(sid + "设备数据采集异常！");
		alarm.setType(AlarmType.EXCEPTION.getValue());
		alarm.setLevel(AlarmLevel.MAJOR.getValue());
		alarm.setStatus(AlarmStatus.RAISE.getValue());
		alarm.setRaiseTime(new Date());
		alarm.setCount(1);
		alarm.setEqpId(equipment.getId());
		alarm.setOrgCode(equipment.getOrgCode());
		alarm.setSubgraphId(eqpGraphNodeBo.getSubGraphId());
		alarmService.raise(alarm, false);
	}

	// 构建清除告警
	private void buildClearAlarm(ChannelAlarmData channelAlarmData) {
		String sid = channelAlarmData.getsId();
		Equipment equipment = deviceService.findByCfgParamJson(sid);
		if (equipment == null) {
			Logger.getLogger(this.getClass()).warn(
					"没有配置设备，无法构建告警，设备equipment:" + equipment);
			return;
		}
		Alarm alarm = new Alarm();
		alarm.setName(equipment.getName());
		alarm.setType(AlarmType.EXCEPTION.getValue());
		alarm.setLevel(AlarmLevel.MAJOR.getValue());
		alarm.setStatus(AlarmStatus.RAISE.getValue());
		alarm.setClearTime(new Date());
		alarm.setEqpId(equipment.getId());
		alarm.setOrgCode(equipment.getOrgCode());
		alarmService.raise(alarm, true);
	}
}
