package com.yeejoin.amos.iot.business.queue.handler;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;

import com.yeejoin.amos.iot.business.entity.Alarm;
import com.yeejoin.amos.iot.business.entity.EqpCpblProperty;
import com.yeejoin.amos.iot.business.entity.Equipment;
import com.yeejoin.amos.iot.business.entity.MetricData;
import com.yeejoin.amos.iot.business.entity.mybatis.CapacityPropertyDataVo;
import com.yeejoin.amos.iot.business.entity.mybatis.EqpGraphNodeBo;
import com.yeejoin.amos.iot.business.enums.AlarmLevel;
import com.yeejoin.amos.iot.business.enums.AlarmStatus;
import com.yeejoin.amos.iot.business.enums.AlarmType;
import com.yeejoin.amos.iot.business.enums.DataType;
import com.yeejoin.amos.iot.business.queue.AlarmDataObj;

import net.sf.json.JSONObject;
import org.springframework.util.ObjectUtils;

/**
 * 指标越界告警处理
 * 
 * @author as-lichenglong
 *
 */
public class MetricDataAlarmHandler extends AbstractDataAlarm implements
		IDataAlarm {
	/**
     * 日志记录器
     */
    private static final Logger log = Logger.getLogger(MetricDataAlarmHandler.class);
    
	private Map<String, Boolean> lastAlarmOfPortMap = new ConcurrentHashMap<String, Boolean>();

	public MetricDataAlarmHandler() {
		registerService();
	}

	@Override
	public void handData(AlarmDataObj alarmDataObj) {
		Object obj = alarmDataObj.getDataObj();
		MetricData data = (MetricData) obj;
//		Logger.getLogger(this.getClass())
//				.debug("MetricData:" + data.toString());
		String json = data.getMetricJson();
		List<CapacityPropertyDataVo> capabilityAttributes = deviceMapper
				.getCapacityPropertyById(data.getCpblId());
		@SuppressWarnings("unchecked")
		Map<String, Object> sourceDataMap = (Map<String, Object>) JSONObject
				.toBean(JSONObject.fromObject(json), Map.class);
		//首先获取设备能力下私有的门限配置
		List<EqpCpblProperty> eqpCpblProps = eqpCpblPropertyService
				.findByEqpIdAndCpblId(data.getEqpId(), data.getCpblId());
		//key为能力属性
		Map<Long, EqpCpblProperty> eqpCpblPropMap = new HashMap<Long, EqpCpblProperty>();
		for (EqpCpblProperty eqpCpblProperty : eqpCpblProps)
		{
			eqpCpblPropMap.put(eqpCpblProperty.getCpblPropertyId(), eqpCpblProperty);
		}
		//进行门限判断
		if (ObjectUtils.isEmpty(sourceDataMap)) {
			return;
		}
		for (String key : sourceDataMap.keySet())
		{
			Object value = sourceDataMap.get(key);
			for (CapacityPropertyDataVo capacityPropertyDataVo : capabilityAttributes) {
				String name = capacityPropertyDataVo.getName();
				if (key.equals(name) && !ObjectUtils.isEmpty(capacityPropertyDataVo)) {
					int type = capacityPropertyDataVo.getDataType();
					// 进行上下限的比较，在构建告警
					if (DataType.VARIABLE.getValue() == type
							|| DataType.FLOAT.getValue() == type) {
//						System.err.println("key--->" + key);
						Equipment equipment = deviceService
								.findEquipmentById(data.getEqpId());
						compareThreshold(value, capacityPropertyDataVo,
								equipment, eqpCpblPropMap);
					}
				} else {
//					log.debug("key is not" + key);
				}
			}
		}
//		System.out.println(capabilityAttributes);
	}

	// 比较门限
	private void compareThreshold(Object value, CapacityPropertyDataVo capacityPropertyDataVo, 
			Equipment equipment, Map<Long, EqpCpblProperty> eqpCpblPropMap) {
		// 从缓存中取出上次的告警标志
		String equFlag = new StringBuilder().append(equipment.getId())
				.append("#").append(equipment.getEqpTmplId()).toString();
		Boolean lastAlarmFlag = lastAlarmOfPortMap.get(equFlag);
		if (null == lastAlarmFlag) {

			// 判断上次是否产生告警
			lastAlarmFlag = false;
			lastAlarmOfPortMap.put(equFlag, lastAlarmFlag);
		}
		try {
			if (!((value instanceof Integer) || (value instanceof Float) || (value instanceof Double) || StringUtils.isNumeric(value.toString()))) {
				log.error("data type is not match" + value + "euipment id and qeptepId =" + equFlag);
				return;
			}

			double currentValue = Double.valueOf(value.toString());
			Double maxValue = capacityPropertyDataVo.getMaxValue();
			Double minValue = capacityPropertyDataVo.getMinValue();
			if (maxValue == null && minValue == null) {
				return;
			}

			//首先获取设备能力下私有的门限配置,如果没有则通过设备模板中的配置进行判断
			EqpCpblProperty eqpCpblProp= eqpCpblPropMap.get(capacityPropertyDataVo.getId());
			if (eqpCpblProp != null)
			{
				maxValue = eqpCpblProp.getMaxValue();
				minValue = eqpCpblProp.getMinValue();
			}
			boolean isOver = true;
			if (maxValue != null &&currentValue >  maxValue) {
				isOver = false;
			} else if (minValue != null && currentValue <  minValue) {
				isOver = false;
			} else {
				isOver = true;
			}
//			boolean isOver = maxValue > currentValue && minValue < currentValue;
			if (isOver) {
				// 产生清除告警
				if (lastAlarmFlag) {
					buildClearAlarm(capacityPropertyDataVo, currentValue,
							maxValue, minValue, equipment);
					lastAlarmFlag = false;
					lastAlarmOfPortMap.put(equFlag, lastAlarmFlag);
				}
			} else {
				// 产生新告警
				buildAlarm(capacityPropertyDataVo, currentValue, maxValue,
						minValue, equipment);
				lastAlarmFlag = true;
				lastAlarmOfPortMap.put(equFlag, lastAlarmFlag);
			}
		} catch (NumberFormatException ne) {
			ne.printStackTrace();
			Logger.getLogger(this.getClass()).debug(
					"value numberFormatException" + value);
		} catch (Exception e) {
			log.error(e.getMessage(),e);
		}
	}

	// 构建新告警
	private void buildAlarm(CapacityPropertyDataVo capacityPropertyDataVo,
			double currentValue, double maxValue, double minValue,
			Equipment equipment) {
		if (equipment == null) {
			Logger.getLogger(this.getClass()).warn(
					"没有添加设备，无法构建告警，设备equipment:" + equipment);
			return;
		}
		EqpGraphNodeBo eqpGraphNodeBo = findEqpGraphNodeByEqpId(equipment
				.getId());
		if (eqpGraphNodeBo == null) {
			Logger.getLogger(this.getClass()).warn(
					"没有视图，无法构建告警，设备equipment:" + equipment);
			return;
		}
		Alarm alarm = new Alarm();
		alarm.setName(equipment.getName() + "-"
				+ capacityPropertyDataVo.getCpblName() + "-"
				+ capacityPropertyDataVo.getDisplayName());
		alarm.setDescr("当前值：" + currentValue + "，最大值：" + maxValue + "，最小值："
				+ minValue + "，当前值不在量程范围内！");
		alarm.setType(AlarmType.OVER.getValue());
		alarm.setLevel(AlarmLevel.MINOR.getValue());
		alarm.setStatus(AlarmStatus.RAISE.getValue());
		alarm.setRaiseTime(new Date());
		alarm.setCount(1);
		alarm.setCpblAttrId(capacityPropertyDataVo.getId());
		alarm.setEqpId(equipment.getId());
		alarm.setOrgCode(equipment.getOrgCode());
		alarm.setSubgraphId(eqpGraphNodeBo.getSubGraphId());
		alarmService.raise(alarm, false);
	}

	// 构建清除告警
	private void buildClearAlarm(CapacityPropertyDataVo capacityPropertyDataVo,
			double currentValue, double maxValue, double minValue,
			Equipment equipment) {
		if (equipment == null) {
			Logger.getLogger(this.getClass()).warn(
					"没有添加设备，无法构建告警，设备equipment:" + equipment);
			return;
		}
		Alarm alarm = new Alarm();
		alarm.setName(equipment.getName() + "-"
				+ capacityPropertyDataVo.getCpblName() + "-"
				+ capacityPropertyDataVo.getDisplayName());
		alarm.setType(AlarmType.OVER.getValue());
		alarm.setLevel(AlarmLevel.MINOR.getValue());
		alarm.setStatus(AlarmStatus.RAISE.getValue());
		alarm.setClearTime(new Date());
		alarm.setCpblAttrId(capacityPropertyDataVo.getId());
		alarm.setEqpId(equipment.getId());
		alarm.setOrgCode(equipment.getOrgCode());
		alarm.setClearInfo("当前值：" + currentValue + "，最大值：" + maxValue + "，最小值："
				+ minValue + "，当前值已经恢复在量程范围内！");
		alarmService.raise(alarm, true);
	}
}
