package com.yeejoin.amos.iot.business.service.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.serializer.SerializeConfig;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.alibaba.fastjson.serializer.SimpleDateFormatSerializer;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.yeejoin.amos.iot.business.constants.IotConstant;
import com.yeejoin.amos.iot.business.dao.mapper.AlarmMapper;
import com.yeejoin.amos.iot.business.dao.mapper.CapacityAttrMapper;
import com.yeejoin.amos.iot.business.dao.mapper.EqpTmplAttrMapper;
import com.yeejoin.amos.iot.business.dao.repository.IAlarmDao;
import com.yeejoin.amos.iot.business.dao.repository.IEquipmentDao;
import com.yeejoin.amos.iot.business.entity.Alarm;
import com.yeejoin.amos.iot.business.entity.Equipment;
import com.yeejoin.amos.iot.business.entity.mybatis.AlarmDataVo;
import com.yeejoin.amos.iot.business.entity.mybatis.EqpTmplAttributeBo;
import com.yeejoin.amos.iot.business.enums.AlarmClearType;
import com.yeejoin.amos.iot.business.enums.AlarmLevel;
import com.yeejoin.amos.iot.business.enums.AlarmStatus;
import com.yeejoin.amos.iot.business.param.DeleteAlarmParam;
import com.yeejoin.amos.iot.business.remote.RemoteSecurityService;
import com.yeejoin.amos.iot.business.service.intfc.IAlarmService;
import com.yeejoin.amos.iot.core.common.request.CommonPageable;
import com.yeejoin.amos.iot.core.enums.QueryOperatorEnum;
import com.yeejoin.amos.iot.core.remote.services.RemoteOperationService;
import com.yeejoin.amos.iot.core.util.DateUtil;
import com.yeejoin.amos.iot.core.util.query.BaseQuerySpecification;
import com.yeejoin.amos.iot.rule.engine.service.IRuleEngineService;
import com.yeejoin.amos.iot.socket.websocket.alarm.IAlarmSocketService;
import com.yeejoin.amos.iot.socket.websocket.service.IWebSocketService;
import com.yeejoin.amos.op.core.common.query.DaoCriteria;
import com.yeejoin.amos.op.core.entity.Resource;
import com.yeejoin.amos.op.core.entity.handler.EntityChangeHandler;
import com.yeejoin.amos.op.core.jpa.GenericManagerImpl;
import com.yeejoin.amos.op.core.jpa.IBaseRepository;

/**
 * 告警service实现类
 *
 * @author as-youjun
 */
@Service("alarmService")
public class AlarmServiceImpl 
//extends GenericManagerImpl<Alarm, Long> 
implements IAlarmService {

    private Logger log = Logger.getLogger(AlarmServiceImpl.class);
    
    @Autowired
    RemoteSecurityService userController;

    @Autowired
    private AlarmMapper alarmMapper;

    @Autowired
    private IAlarmDao alarmDao;

    @Autowired
    private IAlarmSocketService alarmSocketService;

    @Autowired
    private IRuleEngineService ruleEngineService;

    @Autowired
    private RemoteOperationService remoteOperationService;

    @Autowired
    private IEquipmentDao equipmentDao;

    @Autowired
    private EqpTmplAttrMapper eqpTmplAttrMapper;

    @Autowired
    private CapacityAttrMapper capacityAttrMapper;

    @Autowired
    private IWebSocketService webSocketService;

//    public AlarmServiceImpl(IBaseRepository<Alarm, Long> repository) {
//        super(repository);
//        EntityChangeHandler.addModel(this);
//    }

    /**
     * 分页带条件查询告警数据
     */
    @Override
    public Map<String, Object> findAllByParam(List<DaoCriteria> criterias, CommonPageable pageable) {
//		criterias.add(new DaoCriteria("userId", QueryOperatorEnum.EQUAL.getName(),
//				Long.parseLong(CurrentAuthentication.getAuthenticatedUserId())));
    	BaseQuerySpecification<Alarm> spec = new BaseQuerySpecification(criterias);
        Page<Alarm> page = alarmDao.findAll(spec, pageable);
        List<Alarm> alarmList = page.getContent();

        List<AlarmDataVo> alarmDataVoList = new ArrayList<AlarmDataVo>();
        for (Alarm alarm : alarmList) {
            AlarmDataVo alarmDataVo = new AlarmDataVo();
            alarmDataVo.setClearInfo(alarm.getClearInfo());
            alarmDataVo.setClearTime(alarm.getClearTime());
            alarmDataVo.setClearType(alarm.getClearType());
            alarmDataVo.setClearUserId(alarm.getClearUserId());
            alarmDataVo.setCount(alarm.getCount());
            alarmDataVo.setCpblAttrId(alarm.getCpblAttrId());
            alarmDataVo.setDescr(alarm.getDescr());
            alarmDataVo.setEqpId(alarm.getEqpId());
            alarmDataVo.setId(alarm.getId());
            alarmDataVo.setLevel(alarm.getLevel());
            alarmDataVo.setName(alarm.getName());
            alarmDataVo.setRaiseTime(alarm.getRaiseTime());
            alarmDataVo.setRecord(alarm.getRecord());
            alarmDataVo.setRemark(alarm.getRemark());
            alarmDataVo.setStatus(alarm.getStatus());
            alarmDataVo.setType(alarm.getType());
            alarmDataVo.setUpdateTime(alarm.getUpdateTime());
            alarmDataVo.setOrgCode(alarm.getOrgCode());
            String durationDate = null;
            if (alarm.getClearTime() == null) {
                durationDate = DateUtil.getTimeDifference(alarm.getRaiseTime(), new Date());
            } else {
                durationDate = DateUtil.getTimeDifference(alarm.getRaiseTime(), alarm.getClearTime());
            }
            alarmDataVo.setDurationDate(durationDate);
            alarmDataVoList.add(alarmDataVo);
        }

        Map<String, Object> map = new HashMap<>();
        map.put("content", alarmDataVoList);
        map.put("totalElements", page.getTotalElements());
        return map;
    }

    @Override
    public void raise(Alarm alarm, boolean isClear) {
        // TODO Auto-generated method stub
//        log.info("Alarm---->" + alarm.toString());
//        System.err.println("Alarm---->" + alarm.toString());
        List<Alarm> alarms = new ArrayList<Alarm>();

        List<DaoCriteria> listCriteria = new ArrayList<DaoCriteria>();
        DaoCriteria typeCriteria = new DaoCriteria();
        typeCriteria.setOperator(QueryOperatorEnum.EQUAL.getName());
        typeCriteria.setPropertyName("type");
        typeCriteria.setValue(alarm.getType());
        listCriteria.add(typeCriteria);

        if (alarm.getSecondaryType() != null) {
            DaoCriteria secondaryTypeCriteria = new DaoCriteria();
            secondaryTypeCriteria.setOperator(QueryOperatorEnum.EQUAL.getName());
            secondaryTypeCriteria.setPropertyName("secondaryType");
            secondaryTypeCriteria.setValue(alarm.getSecondaryType());
            listCriteria.add(secondaryTypeCriteria);
        }

        DaoCriteria resourceTypeCriteria = new DaoCriteria();
        resourceTypeCriteria.setOperator(QueryOperatorEnum.EQUAL.getName());
        resourceTypeCriteria.setPropertyName("eqpId");
        resourceTypeCriteria.setValue(alarm.getEqpId());
        listCriteria.add(resourceTypeCriteria);

        DaoCriteria statusCriteria = new DaoCriteria();
        statusCriteria.setOperator(QueryOperatorEnum.EQUAL.getName());
        statusCriteria.setPropertyName("status");
        statusCriteria.setValue(AlarmStatus.RAISE.getValue());
        listCriteria.add(statusCriteria);

        DaoCriteria codeCriteria = new DaoCriteria();
        codeCriteria.setOperator(QueryOperatorEnum.EQUAL.getName());
        codeCriteria.setPropertyName("orgCode");
        codeCriteria.setValue(alarm.getOrgCode());
        listCriteria.add(codeCriteria);

        DaoCriteria nameCriteria = new DaoCriteria();
        nameCriteria.setOperator(QueryOperatorEnum.EQUAL.getName());
        nameCriteria.setPropertyName("name");
        nameCriteria.setValue(alarm.getName());
        listCriteria.add(nameCriteria);
        Alarm currentAlarm = null;
        if (isClear) {//是清除告警
        	BaseQuerySpecification<Alarm> spec = new BaseQuerySpecification(listCriteria);
            alarms = (List<Alarm>) alarmDao.findAll(spec);
            if (null != alarms && alarms.size() > 0) {
                currentAlarm = alarms.get(0);
                currentAlarm.setStatus(AlarmStatus.HISTORY.getValue());
                currentAlarm.setClearInfo(alarm.getClearInfo());
                currentAlarm.setClearType(AlarmClearType.AUTO.getValue());
                currentAlarm.setClearUserId(IotConstant.ADMIN_ID);
                currentAlarm.setClearTime(new Date());
                currentAlarm.setRemark("告警已处理");
                alarmDao.save(currentAlarm);
                sendAlarm(currentAlarm);
                //触发告警规则
                ruleEngineService.fireRule(alarm);
            } else {
                log.info(" alarms is NULL");
                return;
            }
        } else //不是清除告警
        {	
        	BaseQuerySpecification<Alarm> spec = new BaseQuerySpecification(listCriteria);
            alarms = (List<Alarm>) alarmDao.findAll(spec);
            //是一件存在的告警
            if (null != alarms && alarms.size() > 0) {
                if (alarms.size() == 1) {
                    currentAlarm = alarms.get(0);
                    currentAlarm.setUpdateTime(new Date());
                    currentAlarm.setCount(currentAlarm.getCount() + 1);
                    currentAlarm.setSubgraphId(alarm.getSubgraphId());
                    currentAlarm.setDescr(alarm.getDescr());
                    alarmDao.save(currentAlarm);
                } else {
                    log.info("double alarms:size=" + alarms.size());
                    return;
                }
            } else {//新发告警
                alarm.setStatus(AlarmStatus.RAISE.getValue());

                alarmDao.save(alarm);
                sendAlarm(alarm);
            }
            //触发告警规则
            ruleEngineService.fireRule(alarm);
            ObjectMapper MAPPER = new ObjectMapper();
            try {
                String json = MAPPER.writeValueAsString(alarm);
                JSONObject jsonObject = JSONObject.parseObject(json);

                JSONObject equipmentAttrs = getAttrByEqpTmplId(alarm.getEqpId());
                jsonObject.put("equipmentAttrs", equipmentAttrs);

                HashMap<String, Object> message = new HashMap<>();
                HashMap<String, String> filterMap = new HashMap<>();
                filterMap.put("subData", "equipmentData");

                message.put("message", jsonObject.toJSONString());
                message.put("path", "metricData");
                message.put("filter", filterMap);
                try {
                    webSocketService.sendMetricDataMessage(message);
                } catch (Exception e) {
                    e.printStackTrace();
                }
            } catch (JsonProcessingException e) {
                e.printStackTrace();
            }

        }
    }

    private JSONObject getAttrByEqpTmplId(Long equipmentId) {
        Equipment equipment = equipmentDao.findById(equipmentId).get();
        Map<String, Object> cfgParamMap = equipment.getCfgParamMap();
        List<EqpTmplAttributeBo> positionBos = eqpTmplAttrMapper.listByEqpTmplId(equipment.getEqpTmplId());
        JSONObject attr = new JSONObject();
        positionBos.forEach(p -> {
            String eqipId = "equip_" + p.getId();
            attr.put(p.getName(), cfgParamMap.get(eqipId));
        });
        return attr;
    }

    private void sendAlarm(Alarm alarm) {
        SerializeConfig mapping = new SerializeConfig();
        mapping.put(Date.class, new SimpleDateFormatSerializer(IotConstant.DATE_TIME_STR_YMDHMS));
        String jsonData = JSONObject.toJSONString(alarm, mapping, SerializerFeature.WriteDateUseDateFormat);
        alarmSocketService.sendAlarmDataMessage(jsonData, IotConstant.ALARM_SOCKET);
        //向运维发送告警工单
        remoteOperationService.onAlarm(alarm);
    }

    @Override
    public void deleteAlarm(DeleteAlarmParam requests) {
        Map<String, Object> map = new HashMap<>();
        List<Long> idsList = requests.getAlarmIds();
        String ids = "";
        int i = 1;
        for (Long id : idsList) {
            ids = ids + id.toString();
            if (i < idsList.size()) {
                ids = ids + ",";
            }
            i++;
        }
        map.put("alarmIds", ids);
        map.put("status", requests.getStatus());
        if (requests.getStatus() == 2) {
            map.put("remark", requests.getRemark());
            map.put("clearTime", new Date());
            map.put("clearUser", userController.getUserId());
            map.put("clearType", 1);
            alarmMapper.eliminateAlarmByIds(map);
        } else if (requests.getStatus() == 3) {
            alarmMapper.deleteAlarmByIds(map);
        }
    }

    @Override
    public Map<String, Object> queryPresentAlarmStatic(List<DaoCriteria> daoCriterias) {
        Map<String, Object> result = new HashMap<>();
        DaoCriteria leaveCriteria = new DaoCriteria("level", QueryOperatorEnum.EQUAL.getName(), AlarmLevel.CRITICAL.getValue());
        daoCriterias.add(leaveCriteria);
        //一级告警数量
        BaseQuerySpecification<Alarm> spec1 = new BaseQuerySpecification<>(daoCriterias);
        long criticalTotal = alarmDao.count(spec1);
        result.put("critical", criticalTotal);
        //二级告警数量
        leaveCriteria.setValue(AlarmLevel.MAJOR.getValue());
        BaseQuerySpecification<Alarm> spec2 = new BaseQuerySpecification<>(daoCriterias);
        long majorTotal = alarmDao.count(spec2);
        result.put("major", majorTotal);
        //三级告警数量
        leaveCriteria.setValue(AlarmLevel.MINOR.getValue());
        BaseQuerySpecification<Alarm> spec3 = new BaseQuerySpecification<>(daoCriterias);
        long minorTotal = alarmDao.count(spec3);
        result.put("minor", minorTotal);
        return result;
    }

    @Override
    public List<Alarm> getAlarmsByEqpId(Long id) {
        return alarmDao.fetchAlarmsByRqpId(id);
    }

    @Override
    public Map<String, Object> alarmStaticByStatus(String orgCode) {
        return alarmMapper.alarmStaticByStatus(orgCode);
    }

    @Override
    public List queryAlarmByEquipAttr(HashMap<String, Object> attrName) {
        return alarmMapper.queryAlarmByEquipAttr(attrName);
    }

//    @Override
    public void fireResourceReomoved(Resource resource) {
        if (resource instanceof Equipment) {
            Equipment equipment = (Equipment) resource;
            alarmDao.delAlarmByEqpId(equipment.getId());
			/*
			List<Alarm> alarms = getAlarmsByEqpId(equipment.getId());
			if (alarms != null)
			{
				alarms.forEach(i->alarmDao.delAlarmById(i.getId()));
			}*/
        }
    }

}
