package com.yeejoin.amos.iot.business.service.impl;

import com.yeejoin.amos.iot.business.dao.repository.ICapabilityPropertyDao;
import com.yeejoin.amos.iot.business.entity.Capability;
import com.yeejoin.amos.iot.business.entity.CapabilityAttribute;
import com.yeejoin.amos.iot.business.enums.CapabilityAttributeType;
import com.yeejoin.amos.iot.business.service.intfc.ICapacityPropertyService;
import com.yeejoin.amos.op.core.entity.Resource;
import com.yeejoin.amos.op.core.entity.handler.EntityChangeHandler;
import com.yeejoin.amos.op.core.jpa.GenericManagerImpl;
import com.yeejoin.amos.op.core.jpa.IBaseRepository;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;

import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;
import java.util.ArrayList;
import java.util.List;

/**
 * <pre>
 * 能力属性
 * </pre>
 *
 * @author as-chenjiajun
 * @version $Id: CapacityPropertyServiceImpl.java, v 0.1 2018年5月29日 下午5:55:43 as-chenjiajun Exp $
 */
@Service
public class CapacityPropertyServiceImpl 
implements ICapacityPropertyService 
{

    @Autowired
    private ICapabilityPropertyDao iCapabilityPropertyDao;

    @Override
    public void deleteCapacityProperty(Long id) {
        iCapabilityPropertyDao.delCapabilityAttributeById(id);
    }

//    @Override
    public void fireResourceReomoved(Resource resource) {

        if (resource instanceof Capability) {
            Capability capability = (Capability) resource;
            Specification<CapabilityAttribute> capaAttrSpec = new Specification<CapabilityAttribute>() {
                public Predicate toPredicate(Root<CapabilityAttribute> root, CriteriaQuery<?> query,
                                             CriteriaBuilder cb) {
                    List<Predicate> list = new ArrayList<Predicate>();
                    Predicate id = cb.equal(root.get("capability").as(Capability.class), capability);
                    list.add(id);
                    Predicate[] p = new Predicate[list.size()];
                    return cb.and(list.toArray(p));
                }
            };
            List<CapabilityAttribute> capabilityAttributes = iCapabilityPropertyDao.findAll(capaAttrSpec);
            if (capabilityAttributes != null) {
                capabilityAttributes.forEach(a -> iCapabilityPropertyDao.delCapabilityAttributeById(a.getId()));
            }
        }

    }

    @Override
    public List<CapabilityAttribute> findByCapabilityId(Long cpId) {
        return iCapabilityPropertyDao.findByCapabilityId(cpId);
    }

    @Override
    public List<CapabilityAttribute> getInfoByIds(boolean isContainConstants, Long... ids) {
        List<CapabilityAttribute> capabilityAttributes = iCapabilityPropertyDao.getInfoByIds(ids);
        if (isContainConstants) {
            return capabilityAttributes;
        }
        List<CapabilityAttribute> cas = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(capabilityAttributes)) {
            capabilityAttributes.forEach(capabilityAttribute -> {
                if (capabilityAttribute.getType().intValue() != CapabilityAttributeType.CONSTANT.getValue()) {
                    cas.add(capabilityAttribute);
                }
            });
        }
        return cas;
    }

}