package com.yeejoin.amos.iot.business.service.impl;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.stream.Collectors;

import com.yeejoin.amos.feign.privilege.Privilege;
import org.apache.commons.collections.CollectionUtils;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.alibaba.fastjson.JSON;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.yeejoin.amos.iot.business.dao.mapper.Capacity_Intfc_protocolMapper;
import com.yeejoin.amos.iot.business.dao.mapper.DeviceMapper;
import com.yeejoin.amos.iot.business.dao.mapper.EqpTmplAttrMapper;
import com.yeejoin.amos.iot.business.dao.mapper.EqpTmplMapper;
import com.yeejoin.amos.iot.business.dao.mapper.MonitorMapper;
import com.yeejoin.amos.iot.business.dao.mapper.StatisticMapper;
import com.yeejoin.amos.iot.business.dao.repository.ICapabilityDao;
import com.yeejoin.amos.iot.business.dao.repository.ICapabilityInterfaceDao;
import com.yeejoin.amos.iot.business.dao.repository.ICapabilityPropertyDao;
import com.yeejoin.amos.iot.business.dao.repository.IDicDao;
import com.yeejoin.amos.iot.business.dao.repository.IEqpCpblDao;
import com.yeejoin.amos.iot.business.dao.repository.IEqpTmplInterfaceDao;
import com.yeejoin.amos.iot.business.dao.repository.IEqpTmplPropertyDao;
import com.yeejoin.amos.iot.business.dao.repository.IEquipmentDao;
import com.yeejoin.amos.iot.business.dao.repository.IEquipmentTemplateDao;
import com.yeejoin.amos.iot.business.dao.repository.IIntfcAttributeDao;
import com.yeejoin.amos.iot.business.dao.repository.IIntfcDao;
import com.yeejoin.amos.iot.business.dao.repository.INodeDao;
import com.yeejoin.amos.iot.business.dao.repository.IProtocolDao;
import com.yeejoin.amos.iot.business.dao.repository.ISubgraohDao;
import com.yeejoin.amos.iot.business.entity.Capability;
import com.yeejoin.amos.iot.business.entity.CapabilityAttribute;
import com.yeejoin.amos.iot.business.entity.CapabilitylInterface;
import com.yeejoin.amos.iot.business.entity.Dic;
import com.yeejoin.amos.iot.business.entity.EqpTmplAttribute;
import com.yeejoin.amos.iot.business.entity.EqpTmplInterface;
import com.yeejoin.amos.iot.business.entity.Equipment;
import com.yeejoin.amos.iot.business.entity.EquipmentCapability;
import com.yeejoin.amos.iot.business.entity.EquipmentTemplate;
import com.yeejoin.amos.iot.business.entity.Intfc;
import com.yeejoin.amos.iot.business.entity.IntfcAttribute;
import com.yeejoin.amos.iot.business.entity.Node;
import com.yeejoin.amos.iot.business.entity.Protocol;
import com.yeejoin.amos.iot.business.entity.Subgraph;
import com.yeejoin.amos.iot.business.entity.mybatis.CapacityInterfaceDataVo;
import com.yeejoin.amos.iot.business.entity.mybatis.CapacityPropertyDataVo;
import com.yeejoin.amos.iot.business.entity.mybatis.DeviceInterfaceDataVo;
import com.yeejoin.amos.iot.business.entity.mybatis.EqpCount;
import com.yeejoin.amos.iot.business.entity.mybatis.EqpTmplAttributeBo;
import com.yeejoin.amos.iot.business.entity.mybatis.EquipmentBo;
import com.yeejoin.amos.iot.business.entity.mybatis.EquipmentTemplateVo;
import com.yeejoin.amos.iot.business.entity.mybatis.EquipmentVo;
import com.yeejoin.amos.iot.business.enums.EquipTempIndustryType;
import com.yeejoin.amos.iot.business.enums.EquipmentType;
import com.yeejoin.amos.iot.business.param.CapacityPropertyParam;
import com.yeejoin.amos.iot.business.param.DevicePageParam;
import com.yeejoin.amos.iot.business.param.EqpTemplPageParam;
import com.yeejoin.amos.iot.business.param.EquipmentCapacityParam;
import com.yeejoin.amos.iot.business.param.EquipmentInterfaceParam;
import com.yeejoin.amos.iot.business.param.EquipmentParam;
import com.yeejoin.amos.iot.business.param.EquipmentPropertyParam;
import com.yeejoin.amos.iot.business.param.QueryEqpCountParam;
import com.yeejoin.amos.iot.business.param.TopoNodeParam;
import com.yeejoin.amos.iot.business.remote.RemoteSecurityService;
import com.yeejoin.amos.iot.business.service.intfc.IDeviceService;
import com.yeejoin.amos.iot.core.common.response.CommonPage;
import com.yeejoin.amos.iot.core.enums.QueryOperatorEnum;
import com.yeejoin.amos.iot.core.util.DateUtil;
import com.yeejoin.amos.iot.core.util.ImgFileServer;
import com.yeejoin.amos.iot.core.util.StringUtil;
import com.yeejoin.amos.iot.core.util.query.BaseQuerySpecification;
import com.yeejoin.amos.iot.core.util.query.QuerySpecificationHelper;
import com.yeejoin.amos.op.core.common.query.DaoCriteria;
import com.yeejoin.amos.op.core.entity.handler.EntityChangeHandler;
import com.yeejoin.amos.op.core.jpa.GenericManagerImpl;
import com.yeejoin.amos.op.core.jpa.IBaseRepository;

import net.sf.json.JSONObject;
import org.springframework.util.ObjectUtils;

/**
 * <pre>
 * 设备操作Service
 * </pre>
 *
 * @author as-chenjiajun
 * @version $Id: DeviceServiceImpl.java, v 0.1 2018年1月29日 下午5:01:13
 * as-chenjiajun Exp $
 */
@SuppressWarnings("unused")
@Service("deviceService")
public class DeviceServiceImpl 
implements IDeviceService {
//    public DeviceServiceImpl(
//            IBaseRepository<EquipmentTemplate, Long> repository) {
//        super(repository);
//    }
    
    @Autowired
    RemoteSecurityService userController;

    @Autowired
    private IEquipmentTemplateDao iEquipmentTemplateDao;

    @Autowired
    private IEquipmentDao iEquipmentDao;

    @Autowired
    private IEqpTmplPropertyDao iEqpTmplPropertyDao;

    @Autowired
    private IEqpTmplInterfaceDao iEqpTmplInterfaceDao;

    @Autowired
    private ICapabilityDao iCapabilityDao;

    @Autowired
    private ICapabilityInterfaceDao iCapabilitylInterfaceDao;

    @Autowired
    private Capacity_Intfc_protocolMapper capacity_Intfc_protocolMapper;

    @Autowired
    private ICapabilityPropertyDao iCapabilityPropertyDao;

    @Autowired
    private DeviceMapper deviceMapper;

    @Autowired
    private ImgFileServer imgFileServer;

    @Autowired
    private IProtocolDao iProtocolDao;

    @Autowired
    private MonitorMapper processMapper;

    @Autowired
    private IDicDao dicDao;

    @Autowired
    private INodeDao iNodeDao;

    @Autowired
    private ISubgraohDao iSubgraohDao;

    @Autowired
    private IEqpCpblDao iEqpCpblDao;

    @Autowired
    private IIntfcDao iIntfcDao;

    @Autowired
    private IIntfcAttributeDao iIntfcAttributeDao;

    @Autowired
    private EqpTmplAttrMapper eqpTmplAttrMapper;

    @Autowired
    private EqpTmplMapper eqpTmplMapper;

    @Autowired
    private StatisticMapper statisticMapper;

    @Transactional
    @Override
    public Long saveEquipment(EquipmentParam param) {
        // 保存图片（如果有）
        if (checkNameConflict(param)) {
            return 0L;
        }
        if (param.getBinaryImg() != null) {
            imgFileServer.saveDeviceFile(param.getBinaryImg(), param.getImgName(), "device", "ISO-8859-1");
        }
        // 保存设备基本信息
        EquipmentTemplate equipmentTemplate = new EquipmentTemplate();
        equipmentTemplate.setCreateTime(DateUtil.getCurrentDate());
        equipmentTemplate.setImg(param.getImgName());
        equipmentTemplate.setName(param.getName());
        equipmentTemplate.setType(param.getType());
        equipmentTemplate.setModel(param.getModel());
        equipmentTemplate.setDescr(param.getDesc());
        equipmentTemplate.setVisibility(param.getVisibility());
        //考虑到前端暂时没有该字段的录入功能，因此使用imgName，后期有了对应功能后再修改
        equipmentTemplate.setViewPath(param.getImgName());
//		equipmentTemplate.setViewPath(param.getViewPath());
        equipmentTemplate.setIndustryId(param.getIndustryId());
        equipmentTemplate.setOrgCode(param.getOrgCode());
        if (equipmentTemplate.getUserId() == null) {
            equipmentTemplate.setUserId(userController.getUserId());
        }
        equipmentTemplate = iEquipmentTemplateDao.save(equipmentTemplate);
        Long eqpTmplId = equipmentTemplate.getId();
        // 保存设备接口信息
        List<EquipmentInterfaceParam> equipmentInterfaceParams = param.getEquipmentInterfaceParams();
        Map<String, Long> idRelationMap = null;
        if (equipmentInterfaceParams != null) {
            idRelationMap = saveEquipmentInterface(eqpTmplId, equipmentInterfaceParams);
        }
        // 添加设备能力信息
        List<EquipmentCapacityParam> equipmentCapacityParams = param.getEquipmentCapacityParams();
        if (equipmentCapacityParams != null) {
            List<Capability> capabilities = new ArrayList<Capability>();
            for (EquipmentCapacityParam equipmentCapacityParam : equipmentCapacityParams) {
                Capability tempCapability = parseToEquipmentCapability(equipmentTemplate, equipmentCapacityParam);
                tempCapability = iCapabilityDao.save(tempCapability);
                if (tempCapability == null) {
                    Logger.getLogger(this.getClass()).error("save tempCapability failed!");
                    continue;
                }
                // 能力接口
                Long cpblId = tempCapability.getId();
                capabilityIntfc(equipmentCapacityParam, cpblId, idRelationMap);
                // 能力属性
                capacityPropertysSave(equipmentCapacityParam, tempCapability);
                capabilities.add(tempCapability);
            }
            equipmentTemplate.setCapabilities(capabilities);
        }
        // 添加设备属性信息
        List<EquipmentPropertyParam> equipmentPropertyParams = param.getEquipmentPropertyParams();
        if (equipmentPropertyParams != null) {
            List<EqpTmplAttribute> eqpTmplAttributes = new ArrayList<EqpTmplAttribute>();
            for (EquipmentPropertyParam equipmentPropertyParam : equipmentPropertyParams) {
                eqpTmplAttributes.add(parseToEquipmentProperty(equipmentTemplate, equipmentPropertyParam));
            }
            equipmentTemplate.setAttributes(eqpTmplAttributes);
        }
        equipmentTemplate = iEquipmentTemplateDao.save(equipmentTemplate);
        return equipmentTemplate == null ? null : equipmentTemplate.getId();
    }

    /**
     * <pre>
     * 检查是否名称已存在
     * </pre>
     *
     * @param param
     * @return
     */
    private boolean checkNameConflict(EquipmentParam param) {
        List<?> results = iEquipmentTemplateDao.findAll(QuerySpecificationHelper.getDuplicationNameEqpTemplate(param));
        if (results != null && results.size() > 0) {
            return true;
        } else {
            return false;
        }
    }

    private Map<String, Long> saveEquipmentInterface(Long eqpTmplId,
                                                     List<EquipmentInterfaceParam> equipmentInterfaceParams) {
        Map<String, Long> idRelationMap = new HashMap<String, Long>();
        for (EquipmentInterfaceParam equipmentInterfaceParam : equipmentInterfaceParams) {
            EqpTmplInterface eqpTmplInterface = new EqpTmplInterface();
            eqpTmplInterface.setEqpTmplId(eqpTmplId);
            eqpTmplInterface.setIntfcDirection(equipmentInterfaceParam.getIntfcDirection());
            eqpTmplInterface.setIntfcId(equipmentInterfaceParam.getIntfcId());
            eqpTmplInterface = iEqpTmplInterfaceDao.save(eqpTmplInterface);
            idRelationMap.put(eqpTmplInterface.getIntfcId() + "-" + eqpTmplInterface.getIntfcDirection(),
                    eqpTmplInterface.getId());
        }
        return idRelationMap;
    }

    public Capability parseToEquipmentCapability(EquipmentTemplate equipmentTemplate,
                                                 EquipmentCapacityParam equipmentCapacityParam) {
        Capability capability = null;
        if (equipmentCapacityParam.getId() != null) {
            capability = iCapabilityDao.getOne(equipmentCapacityParam.getId());
        } else {
            capability = new Capability();
        }
        capability.setEqpTmpl(equipmentTemplate);
        capability.setDescr(equipmentCapacityParam.getDescr());
        capability.setName(equipmentCapacityParam.getName());
        capability.setType(equipmentCapacityParam.getType());
        capability.setProtocolId(equipmentCapacityParam.getProtocolId());
        capability.setFormula(equipmentCapacityParam.getFormula());
        capability.setMonitorType(equipmentCapacityParam.getMonitortype());
        capability.setMonitorPeriod(equipmentCapacityParam.getPeriod());
        capability.setMongodb(equipmentCapacityParam.getMongodb());
        return capability;
    }

    public EqpTmplAttribute parseToEquipmentProperty(EquipmentTemplate equipmentTemplate,
                                                     EquipmentPropertyParam param) {
        EqpTmplAttribute eqpTmplAttribute = new EqpTmplAttribute();
        eqpTmplAttribute.setDataType(param.getDataType());
        eqpTmplAttribute.setDescr(param.getDescr());
        eqpTmplAttribute.setDisplayName(param.getDisplayName());
        eqpTmplAttribute.setMaxValue(param.getMaxValue());
        eqpTmplAttribute.setMinValue(param.getMinValue());
        eqpTmplAttribute.setName(param.getName());
        eqpTmplAttribute.setType(param.getType());
        eqpTmplAttribute.setUnit(param.getUnit());
        if (param.getValueStr() != null) {
            eqpTmplAttribute.setValueStr(param.getValueStr());
        }
        if (param.getEnumRange() != null) {
            eqpTmplAttribute.setEnumRange(param.getEnumRange());
        }
        if (param.getPrecision() != null) {
            eqpTmplAttribute.setPrecision(param.getPrecision());
        }
        eqpTmplAttribute.setEqpTmpl(equipmentTemplate);
        return eqpTmplAttribute;
    }

    /**
     * <pre>
     * 设备属性新增 / 编辑使用
     * </pre>
     *
     * @param param
     * @return
     */
    public EqpTmplAttribute paramparseToEquipmentProperty(EquipmentPropertyParam param) {
        EqpTmplAttribute eqpTmplAttribute = null;
        if (param.getId() == null) {
            eqpTmplAttribute = new EqpTmplAttribute();
        } else {
            eqpTmplAttribute = iEqpTmplPropertyDao.findById(param.getId()).get();
        }
        if (null != param.getDataType()) {
            eqpTmplAttribute.setDataType(param.getDataType());
        }
        eqpTmplAttribute.setDescr(param.getDescr());
        if (StringUtil.isNotEmpty(param.getDisplayName())) {
            eqpTmplAttribute.setDisplayName(param.getDisplayName());
        }
        if (null != param.getMaxValue()) {
            eqpTmplAttribute.setMaxValue(param.getMaxValue());
        }
        if (null != param.getMinValue()) {
            eqpTmplAttribute.setMinValue(param.getMinValue());
        }
        if (StringUtil.isNotEmpty(param.getName())) {
            eqpTmplAttribute.setName(param.getName());
        }

        if (null != param.getType()) {
            eqpTmplAttribute.setType(param.getType());
        }

        if (StringUtil.isNotEmpty(param.getUnit())) {
            eqpTmplAttribute.setUnit(param.getUnit());
        }
        if (param.getValueStr() != null) {
            eqpTmplAttribute.setValueStr(param.getValueStr());
        }
        if (param.getEnumRange() != null) {
            eqpTmplAttribute.setEnumRange(param.getEnumRange());
        }
        if (param.getPrecision() != null) {
            eqpTmplAttribute.setPrecision(param.getPrecision());
        }

        if (null != param.getIsRequired()) {
            eqpTmplAttribute.setIsRequired(param.getIsRequired());
        }

        EquipmentTemplate equipmentTemplate = iEquipmentTemplateDao.findById(param.getEquipmentId()).get();
        eqpTmplAttribute.setEqpTmpl(equipmentTemplate);
        return eqpTmplAttribute;
    }

    // 设备能力基本信息编辑使用
    public Capability paramparseToEquipmentCapability(EquipmentCapacityParam equipmentCapacityParam) {
        Capability capability = iCapabilityDao.findById(equipmentCapacityParam.getId()).get();
        EquipmentTemplate equipmentTemplate = iEquipmentTemplateDao.findById(equipmentCapacityParam.getEquipmentId()).get();
        capability.setEqpTmpl(equipmentTemplate);
        capability.setDescr(equipmentCapacityParam.getDescr());
        capability.setName(equipmentCapacityParam.getName());
        capability.setType(equipmentCapacityParam.getType());
        capability.setMongodb(equipmentCapacityParam.getMongodb());
        return capability;
    }

    // 能力接口保存
    public void capabilityIntfc(EquipmentCapacityParam equipmentCapacityParam, Long cpblId,
                                Map<String, Long> idRelationMap) {
        if (idRelationMap == null) {
            idRelationMap = new HashMap<String, Long>();
            List<DeviceInterfaceDataVo> eqpTmplInterfaces = deviceMapper
                    .getDeviceInterfaceById(equipmentCapacityParam.getEquipmentId());
            for (DeviceInterfaceDataVo eqpTmplInterface : eqpTmplInterfaces) {
                idRelationMap.put(eqpTmplInterface.getIntfcId() + "-" + eqpTmplInterface.getIntfcDirection(),
                        eqpTmplInterface.getDevIntfcID());
            }
        }
        // 先删除原有数据
        List<CapacityInterfaceDataVo> capacityInterfaceDataVos = deviceMapper.getCapacityInterfaceByCapId(cpblId);
        if (CollectionUtils.isNotEmpty(capacityInterfaceDataVos)) {
            for (CapacityInterfaceDataVo capacityInterfaceDataVo : capacityInterfaceDataVos) {
                Long capaIntId = capacityInterfaceDataVo.getCapaIntId();
                iCapabilitylInterfaceDao.delCapabilitylInterfaceById(capaIntId);
            }
        }
        // 添加新数据
        List<String> intfcIds = equipmentCapacityParam.getIntfcIds();
        if (CollectionUtils.isNotEmpty(intfcIds)) {
            for (String intfcId : intfcIds) {
                Long empIntfcId = idRelationMap.get(intfcId);
                CapabilitylInterface capabilitylInterface = new CapabilitylInterface();
                capabilitylInterface.setIntfcId(Long.valueOf(intfcId.split("-")[0]));
                capabilitylInterface.setCpblId(cpblId);
                capabilitylInterface.setEmpTmplIntfcId(empIntfcId);
                iCapabilitylInterfaceDao.save(capabilitylInterface);
            }
        }
    }

    // 设备能力和属性关系保存
    public void capacityPropertysSave(EquipmentCapacityParam equipmentCapacityParam, Capability tempCapability) {
        List<CapacityPropertyParam> capacityPropertyParams = equipmentCapacityParam.getCapacityPropertyParams();
        if (capacityPropertyParams != null) {
            for (CapacityPropertyParam capacityPropertyParam : capacityPropertyParams) {
                CapabilityAttribute capabilityAttribute = new CapabilityAttribute();
                capabilityAttribute.setCapability(tempCapability);
                capabilityAttribute.setDataType(capacityPropertyParam.getDataType());
                capabilityAttribute.setDescr(capacityPropertyParam.getDescr());
                capabilityAttribute.setDisplayName(capacityPropertyParam.getDisplayName());
                capabilityAttribute.setMaxValue(capacityPropertyParam.getMaxValue());
                capabilityAttribute.setMinValue(capacityPropertyParam.getMinValue());
                capabilityAttribute.setName(capacityPropertyParam.getName());
                capabilityAttribute.setType(capacityPropertyParam.getType());
                capabilityAttribute.setUnit(capacityPropertyParam.getUnit());
                if (capacityPropertyParam.getValueStr() != null) {
                    capabilityAttribute.setValueStr(capacityPropertyParam.getValueStr());
                }
                if (capacityPropertyParam.getEnumRange() != null) {
                    capabilityAttribute.setEnumRange(capacityPropertyParam.getEnumRange());
                }
                if (capacityPropertyParam.getPrecision() != null) {
                    capabilityAttribute.setPrecision(capacityPropertyParam.getPrecision());
                }
                iCapabilityPropertyDao.save(capabilityAttribute);
            }
        }
    }

    /**
     * 设备基本信息编辑
     *
     * @param param
     * @see com.yeejoin.amos.iot.business.service.intfc.IDeviceService#(com.yeejoin.amos.iot.business.entity.EquipmentTemplate)
     */
    @Override
    public Long updateDeviceBasicInfo(EquipmentParam param) {
        if (checkNameConflict(param)) {
            return -1L;
        }
        EquipmentTemplate equipmentTemplate = null;
        if (param.getId() == null) {
            equipmentTemplate = new EquipmentTemplate();
        } else {
            equipmentTemplate = iEquipmentTemplateDao.findById(param.getId()).get();
        }
        equipmentTemplate.setUpdateTime(DateUtil.getCurrentDate());
        if (StringUtil.isNotEmpty(param.getType())) {
            equipmentTemplate.setType(param.getType());
        }
        if (StringUtil.isNotEmpty(param.getModel())) {
            equipmentTemplate.setModel(param.getModel());
        }
        if (StringUtil.isNotEmpty(param.getViewPath())) {
            equipmentTemplate.setViewPath(param.getViewPath());
            ;
        }
        if (StringUtil.isNotEmpty(param.getName())) {
            equipmentTemplate.setName(param.getName());
        }
        if (StringUtil.isNotEmpty(param.getIndustryId())) {
            equipmentTemplate.setIndustryId(param.getIndustryId());
        }
        equipmentTemplate.setDescr(param.getDesc());
        // 保存图片（如果有变化）
        if (param.getBinaryImg() != null) {
            imgFileServer.saveDeviceFile(param.getBinaryImg(), param.getImgName(), "device", "ISO-8859-1");
            equipmentTemplate.setImg(param.getImgName());
            //考虑到前端暂时没有该字段的录入功能，因此使用imgName，后期有了对应功能后再修改
            equipmentTemplate.setViewPath(param.getImgName());
//			equipmentTemplate.setViewPath(param.getViewPath());
        }
        iEquipmentTemplateDao.saveAndFlush(equipmentTemplate);
        return 0L;
    }

    /**
     * 设备属性编辑
     *
     * @param param
     * @see com.yeejoin.amos.iot.business.service.intfc.IDeviceService#deviceAttributeUpdate(com.yeejoin.amos.iot.business.param.EquipmentPropertyParam)
     */
    @Override
    public void deviceAttributeUpdate(EquipmentPropertyParam param) {
        EqpTmplAttribute eqpTmplAttribute = paramparseToEquipmentProperty(param);
        iEqpTmplPropertyDao.saveAndFlush(eqpTmplAttribute);
    }

    /**
     * 设备接口编辑/新增
     *
     * @param param
     * @see com.yeejoin.amos.iot.business.service.intfc.IDeviceService#deviceInterfaceUpdate(com.yeejoin.amos.iot.business.param.EquipmentInterfaceParam)
     */
    @Override
    public void deviceInterfaceUpdate(EquipmentInterfaceParam param) {
        EqpTmplInterface eqpTmplInterface = null;
        if (param.getId() == null) {
            eqpTmplInterface = new EqpTmplInterface();
            eqpTmplInterface.setEqpTmplId(param.getDeviceId());
        } else {
            eqpTmplInterface = iEqpTmplInterfaceDao.findById(param.getId()).get();
        }
        eqpTmplInterface.setIntfcId(param.getIntfcId());
        eqpTmplInterface.setIntfcDirection(param.getIntfcDirection());
        iEqpTmplInterfaceDao.saveAndFlush(eqpTmplInterface);
    }

    @Override
    public CommonPage<EquipmentTemplateVo> pageDeviceData(DevicePageParam param) {
        if (param.getUserId() == null) {
            param.setUserId(userController.getUserId());
        }
        long total = deviceMapper.countDeviceData(param);
        List<EquipmentTemplateVo> content = deviceMapper.pageDeviceData(param).stream().map(equipmentTemplate -> {
            if (equipmentTemplate.getType() == 1) {
                equipmentTemplate.setTypeImg("acquisition_equipment.png");
            } else if (equipmentTemplate.getType() == 2) {
                equipmentTemplate.setTypeImg("sensing_equipment.png");
            } else {
                equipmentTemplate.setTypeImg("control_equipment.png");
            }
            return equipmentTemplate;
        }).collect(Collectors.toList());
        CommonPage<EquipmentTemplateVo> result = new CommonPage<>(content, param, total);
        return result;
    }

    /**
     * 设备能力编辑
     *
     * @param param
     * @see com.yeejoin.amos.iot.business.service.intfc.IDeviceService#deviceCapacityUpdate(com.yeejoin.amos.iot.business.param.EquipmentCapacityParam)
     */
    @Override
    public void deviceCapacityUpdate(EquipmentCapacityParam param) {
        Capability capability = paramparseToEquipmentCapability(param);
        // 设备能力基本信息编辑
        iCapabilityDao.saveAndFlush(capability);
        // 先删除之前能力的接口、协议、和属性
        Long cpblId = param.getId();
        capacity_Intfc_protocolMapper.delCaIntfProRelation(cpblId);
        // 设备能力接口和协议编辑
        capabilityIntfc(param, cpblId, null);
        // 设备能力属性编辑
        capacityPropertysSave(param, capability);
    }

    /**
     * 能力接口编辑/新增
     *
     * @param param
     * @see com.yeejoin.amos.iot.business.service.intfc.IDeviceService#capacityIntfUpdate(com.yeejoin.amos.iot.business.entity.CapabilitylInterface)
     */
    @Override
    public void capacityIntfUpdate(CapabilitylInterface param) {
        iCapabilitylInterfaceDao.saveAndFlush(param);
    }

    /**
     * 能力属性编辑/新增
     *
     * @see com.yeejoin.amos.iot.business.service.intfc.IDeviceService#capacityPropertyUpdate(com.yeejoin.amos.iot.business.param.CapacityPropertyParam)
     */
    @Override
    public void capacityPropertyUpdate(CapacityPropertyParam capacityPropertyParam) {
        CapabilityAttribute capabilityAttribute = null;
        if (capacityPropertyParam.getCpblId() != null) {
            capabilityAttribute = new CapabilityAttribute();
            Capability capability = iCapabilityDao.findById(capacityPropertyParam.getCpblId()).get();
            capabilityAttribute.setCapability(capability);
        } else {
            capabilityAttribute = iCapabilityPropertyDao.findById(capacityPropertyParam.getId()).get();
        }
        capabilityAttribute.setDataType(capacityPropertyParam.getDataType());
        capabilityAttribute.setDescr(capacityPropertyParam.getDescr());
        capabilityAttribute.setDisplayName(capacityPropertyParam.getDisplayName());
        capabilityAttribute.setMaxValue(capacityPropertyParam.getMaxValue());
        capabilityAttribute.setMinValue(capacityPropertyParam.getMinValue());
        capabilityAttribute.setName(capacityPropertyParam.getName());
        capabilityAttribute.setType(capacityPropertyParam.getType());
        capabilityAttribute.setUnit(capacityPropertyParam.getUnit());
        if (capacityPropertyParam.getValueStr() != null) {
            capabilityAttribute.setValueStr(capacityPropertyParam.getValueStr());
        }
        if (capacityPropertyParam.getEnumRange() != null) {
            capabilityAttribute.setEnumRange(capacityPropertyParam.getEnumRange());
        }
        if (capacityPropertyParam.getPrecision() != null) {
            capabilityAttribute.setPrecision(capacityPropertyParam.getPrecision());
        }
        iCapabilityPropertyDao.saveAndFlush(capabilityAttribute);
    }

    @Override
    public EquipmentTemplate getDeviceById(Long id) {
        return iEquipmentTemplateDao.findById(id).get();
    }

    @Override
    public List<DeviceInterfaceDataVo> getDeviceInterfaceById(Long id) {
        List<DeviceInterfaceDataVo> interfaceList = deviceMapper.getDeviceInterfaceById(id);
        return interfaceList == null ? new ArrayList<DeviceInterfaceDataVo>() : interfaceList;
    }

    //查询设备类型
    @Override
    public List<EquipmentTemplateVo> getEqpTmpModelByOrgCode(String orgCode) {
        List<EquipmentTemplateVo> eqpTempList = deviceMapper.getEqpTmpModelByOrgCode(orgCode);
        return eqpTempList == null ? new ArrayList<EquipmentTemplateVo>() : eqpTempList;
    }


    @Override
    public List<CapacityInterfaceDataVo> getCapacityInterfaceById(Long id) {
        List<CapacityInterfaceDataVo> capacityInterfaceDataVos = deviceMapper.getCapacityInterfaceByCapId(id);
        return capacityInterfaceDataVos == null ? new ArrayList<CapacityInterfaceDataVo>() : capacityInterfaceDataVos;
    }

    @Override
    public List<CapacityPropertyDataVo> getCapacityPropertyById(Long id) {
        List<CapacityPropertyDataVo> capabilityAttributes = deviceMapper.getCapacityPropertyById(id);
        return capabilityAttributes == null ? new ArrayList<CapacityPropertyDataVo>() : capabilityAttributes;
    }

    @Override
    public void addDeviceCapacity(EquipmentCapacityParam equipmentCapacityParam) {
        EquipmentTemplate eqpTmpl = iEquipmentTemplateDao.getOne(equipmentCapacityParam.getEquipmentId());
        Capability tempCapability = parseToEquipmentCapability(eqpTmpl, equipmentCapacityParam);
        tempCapability = iCapabilityDao.save(tempCapability);
        if (tempCapability == null) {
            Logger.getLogger(this.getClass()).error("save tempCapability failed!");
        }
        // 能力接口
        Long cpblId = tempCapability.getId();
        capabilityIntfc(equipmentCapacityParam, cpblId, null);
    }

    @Override
    public void deleteDevice(Long id) {
        EquipmentTemplate device = iEquipmentTemplateDao.findById(id).get();
        if (device == null) {
            return;
        }
        EntityChangeHandler.fireResourceReomoved(device);
        // 删除设备
        iEquipmentTemplateDao.delEquipmentTemplateById(device.getId());
    }

    @SuppressWarnings("unchecked")
    @Override
    public List<Equipment> getRealDeviceList(List<DaoCriteria> criterias) {
//		List<DaoCriteria> criterias = new ArrayList<DaoCriteria>();
//		criterias.add(new DaoCriteria("userId", QueryOperatorEnum.EQUAL.getName(),
//				Long.parseLong(CurrentAuthentication.getAuthenticatedUserId())));
        BaseQuerySpecification<Equipment> spec = new BaseQuerySpecification<>(criterias);
        List<Equipment> equipmentList = iEquipmentDao.findAll(spec);
        for (Equipment equipment : equipmentList) {
            equipment.setCfgParamJson("");
            Subgraph subgraph = processMapper.getSubgraphByEqbId(equipment.getId());
            Node node = processMapper.getNodeByEqbId(equipment.getId());
            if (StringUtil.isNotEmpty(subgraph) && StringUtil.isNotEmpty(node)) {
                String config = node.getConfig();
                Map<String, Object> map = (Map<String, Object>) JSON.parse(config);
                equipment.setName(map.get("label") + "-" + subgraph.getName());
            }

        }
        return equipmentList;
    }

    @Override
    public void updateDeviceBasicInfo(TopoNodeParam param) {
        EquipmentTemplate equipmentTemplate = null;
        if (param.getId() == null) {
            equipmentTemplate = new EquipmentTemplate();
        } else {
            equipmentTemplate = iEquipmentTemplateDao.findById(param.getId()).get();
        }
        equipmentTemplate.setUpdateTime(DateUtil.getCurrentDate());
        if (StringUtil.isNotEmpty(param.getName())) {
            equipmentTemplate.setName(param.getName());
        }
        iEquipmentTemplateDao.saveAndFlush(equipmentTemplate);
    }

    @Override
    public String getProtocolNameById(Long id) {
        Capability capability = iCapabilityDao.getOne(id);
        Protocol protocol = iProtocolDao.getOne(capability.getProtocolId());
        return protocol.getName();
    }

    @Override
    public Equipment findEquipmentById(Long eqpId) {
        // TODO Auto-generated method stub
        return iEquipmentDao.findById(eqpId).get();
    }

    @Override
    public Equipment findByCfgParamJson(String sid) {

        return iEquipmentDao.findByCfgParamJson(sid);
    }

    @Override
    public Map<String, Object> equipTempTypeStatic(List<DaoCriteria> daoCriterias) {
        Map<String, Object> result = new HashMap<>();
        daoCriterias.add(new DaoCriteria("isDelete", QueryOperatorEnum.EQUAL.getName(), false));
        DaoCriteria criteria = new DaoCriteria();
        criteria.setPropertyName("eqpTmplId");
        criteria.setOperator(QueryOperatorEnum.IN.getName());
        daoCriterias.add(criteria);
        for (EquipmentType equipmentType : EquipmentType.values()) {
            List<Long> eqpTmplIds = iEquipmentTemplateDao.findIdByType(equipmentType.getValue());
            long count = 0l;
            if (!eqpTmplIds.isEmpty()) {
                criteria.setValue(eqpTmplIds);
                BaseQuerySpecification<Equipment> spec = new BaseQuerySpecification<>(daoCriterias);
                count = iEquipmentDao.count(spec);
            }
            result.put(equipmentType.getType(), count);
        }
        return result;
    }

    @Override
    public Map<String, Object> equipTempIndustryStatic(List<DaoCriteria> daoCriterias) {
        Map<String, Object> result = new HashMap<>();
        daoCriterias.add(new DaoCriteria("isDelete", QueryOperatorEnum.EQUAL.getName(), false));
        DaoCriteria criteria = new DaoCriteria();
        criteria.setPropertyName("eqpTmplId");
        criteria.setOperator(QueryOperatorEnum.IN.getName());
        daoCriterias.add(criteria);
        List<Dic> dics = dicDao.findByType("EquipTemplateIndustry");
        for (Dic dic : dics) {
            List<Long> eqpTmplIds = iEquipmentTemplateDao.findIdByIndustryId(dic.getId());
            long count = 0l;
            if (!eqpTmplIds.isEmpty()) {
                criteria.setValue(eqpTmplIds);
                BaseQuerySpecification<Equipment> spec = new BaseQuerySpecification<>(daoCriterias);
                count = iEquipmentDao.count(spec);
            }
            EquipTempIndustryType industryType = EquipTempIndustryType.getEnumByCode(dic.getCode());
            result.put(industryType.getTypeName(), count);
        }
        return result;
    }

    @Override
    public Map<String, Object> getDeviceCount(List<DaoCriteria> daoCriteria) {
        List<DaoCriteria> pubCriterias = new ArrayList<>();
        List<DaoCriteria> priCriterias = new ArrayList<>();
        for (DaoCriteria criteria : daoCriteria) {
            priCriterias.add(criteria);
        }
        pubCriterias.add(new DaoCriteria("visibility", QueryOperatorEnum.EQUAL.getName(), 0));
        priCriterias.add(new DaoCriteria("visibility", QueryOperatorEnum.EQUAL.getName(), 1));
//		BaseQuerySpecification<EquipmentTemplate> spec = new BaseQuerySpecification<>(daoCriteria);
        BaseQuerySpecification<EquipmentTemplate> pubSpec = new BaseQuerySpecification<>(pubCriterias);
        BaseQuerySpecification<EquipmentTemplate> priSpec = new BaseQuerySpecification<>(priCriterias);
//		Long count = iEquipmentTemplateDao.count(spec);
        Long pubCount = iEquipmentTemplateDao.count(pubSpec);
        Long priCount = iEquipmentTemplateDao.count(priSpec);
        Map<String, Object> map = new HashMap<>();
        map.put("count", pubCount + priCount);
        map.put("pubCount", pubCount);
        map.put("priCount", priCount);
        return map;
    }

    @Override
    public Page<Object> findEqpBySubgraph(String subgraphId, String orgCode, Pageable page) {
        // TODO Auto-generated method stub
        return iEquipmentDao.findEqpBySubgraph(subgraphId, orgCode, page);
    }

    @Override

    public Map<String, Object> getRealDeviceData(EquipmentParam param) {
        //获取查看视图所属单位下（不包括子单位）的符合过滤条件的设备
        String name = param.getName() == null ? null : "%" + param.getName() + "%";
        List<Equipment> equipmentList = iEquipmentDao.getEquipmentByParam(name,
                param.getType(), param.getIndustryId(), param.getOrgCode());
        // 获取查看视图所属单位下（不包括子单位）的所有视图下已添加的设备节点
        List<DaoCriteria> criterias = new ArrayList<DaoCriteria>();
        DaoCriteria compDaoCriteria = new DaoCriteria();
        compDaoCriteria.setPropertyName("orgCode");
        compDaoCriteria.setOperator(QueryOperatorEnum.EQUAL.getName());
        compDaoCriteria.setValue(param.getOrgCode());
        criterias.add(compDaoCriteria);
        BaseQuerySpecification<Subgraph> oldsSpec = new BaseQuerySpecification<>(criterias);
        List<Subgraph> subgraphList = iSubgraohDao.findAll(oldsSpec);
        List<Long> eqpIdLists = new ArrayList<Long>();
        for (Subgraph subgraph : subgraphList) {
            List<Node> nodeList = iNodeDao.findNodBysubgraphId(subgraph.getId());
            if (nodeList.size() > 0) {
                for (Node node : nodeList) {
                    Long eqId = node.getEquipmentId();
                    if (eqId == null) {
                        continue;
                    }
                    eqpIdLists.add(eqId);
                }
            }
        }
        //过滤出未添加到视图中的设备
        List<EquipmentTemplateVo> noUseList = new ArrayList<>();
        List<EquipmentTemplateVo> useList = new ArrayList<>();
        Boolean flag = false;
        for (Equipment equipment : equipmentList) {
            flag = false;
            //判断设备是否已添加至视图中
            if (eqpIdLists.contains(equipment.getId())) {
                flag = true;
            }
            Long eqpTmplId = equipment.getEqpTmplId();
            EquipmentTemplate equipmentTemplate = iEquipmentTemplateDao.findById(eqpTmplId).get();
            if (equipmentTemplate == null) {
                continue;
            }
            EquipmentTemplateVo equipmentTemplateVo = new EquipmentTemplateVo();
            equipmentTemplateVo.setId(equipment.getId());
            equipmentTemplateVo.setName(equipment.getName());
            equipmentTemplateVo.setCreateTime(equipment.getCreateTime());
            equipmentTemplateVo.setDescr(equipment.getDescr());
            equipmentTemplateVo.setImg(equipmentTemplate.getImg());
            equipmentTemplateVo.setIndustryCode(equipmentTemplate.getEquipTemplateIndustry().getCode());
            equipmentTemplateVo.setIndustryName(equipmentTemplate.getEquipTemplateIndustry().getName());
            equipmentTemplateVo.setIsDelete(equipment.getIsDelete());
            equipmentTemplateVo.setModel(equipmentTemplate.getModel());
            equipmentTemplateVo.setOrgCode(equipment.getOrgCode());
            equipmentTemplateVo.setType(equipmentTemplate.getType());
            equipmentTemplateVo.setUpdateTime(equipment.getUpdateTime());
            equipmentTemplateVo.setUserId(equipment.getUserId());
            equipmentTemplateVo.setViewPath(equipmentTemplate.getViewPath());
            equipmentTemplateVo.setVisibility(equipmentTemplate.getVisibility());
            equipmentTemplateVo.setCfgParamJson(equipment.getCfgParamJson());
            equipmentTemplateVo.setEqpTmplId(eqpTmplId);
            if (equipmentTemplate.getType() == 1) {
                equipmentTemplateVo.setTypeImg("acquisition_equipment.png");
            } else if (equipmentTemplate.getType() == 2) {
                equipmentTemplateVo.setTypeImg("sensing_equipment.png");
            } else {
                equipmentTemplateVo.setTypeImg("control_equipment.png");
            }
            //对设备进行分类
            if (flag) {
                useList.add(equipmentTemplateVo);
            } else {
                noUseList.add(equipmentTemplateVo);
            }

        }
        Map<String, Object> map = new HashMap<>();
        map.put("noUseList", noUseList);
        map.put("useList", useList);
        return map;
    }

    @Override
    public void editEquipmentInfo(EquipmentParam equipment) {
        Equipment dbEquipment = iEquipmentDao.findById(equipment.getId()).get();
        dbEquipment.setName(equipment.getName());
        dbEquipment.setDescr(equipment.getDesc());
        iEquipmentDao.save(dbEquipment);
    }

    @Override
    public List<EquipmentBo> findEqpBySubgraphId(Long subgraphId, String orgCode) {
        // TODO Auto-generated method stub
        return deviceMapper.findEqpBySubgraphId(subgraphId, orgCode);
    }

    /**
     * 查询设备数量
     *
     * @param param
     * @return
     */
    @Override
    public Map<Long, Long> findEqpCount(QueryEqpCountParam param) {
        Map<Long, Long> eqpCountMap = new HashMap<>();
        List<EqpCount> eqpCounts = deviceMapper.findEqpCount(param);
        if (CollectionUtils.isNotEmpty(eqpCounts)) {
            eqpCounts.forEach(eqpCount -> {
                eqpCountMap.put(eqpCount.getEqoTmplId(), eqpCount.getEqpCount());
            });
        }
        //eqpCounts中不包含数量为0的设备信息,因此需要特殊处理
        param.getEqpTmplIds().forEach(eqpId -> {
            if (!eqpCountMap.keySet().contains(eqpId)) {
                eqpCountMap.put(eqpId, 0L);
            }
        });
        return eqpCountMap;
    }

    @SuppressWarnings("unchecked")
    @Override
    public void editEquipmentConfigInfo(Map<String, Object> param) {
        Long eqpId = Long.valueOf(param.get("id").toString());
        param.remove("id");
        if (eqpId != null) {
            if (param.containsKey("task")) {
                Map<String, Object> equipmentCapabilityMap = (Map<String, Object>) (param.get("task"));
                EquipmentCapability equipmentCapability = iEqpCpblDao.findById(Long.valueOf(equipmentCapabilityMap.get("id").toString())).get();
                equipmentCapability.setMonitorPeriod(Integer.valueOf(equipmentCapabilityMap.get("monitorPeriod").toString()));
                param.remove("task");
                iEqpCpblDao.save(equipmentCapability);
            }
            Equipment equipment = iEquipmentDao.findById(eqpId).get();
            Map<String, Object> oldParam = equipment.getCfgParamMap();
            //直接覆盖原有值
            for (Entry<String, Object> entry : param.entrySet()) {
                oldParam.put(entry.getKey(), entry.getValue());
            }
            equipment.setCfgParamJson(JSONObject.fromObject(oldParam).toString());
            iEquipmentDao.saveAndFlush(equipment);
        }
    }

    @SuppressWarnings("unchecked")
    @Override
    public Map<String, Object> fetchTopoDevice(Long id) {
        Map<String, Object> result = new HashMap<>();
        //设备、设备属性
        Equipment equipment = iEquipmentDao.findById(id).get();
        Map<String, Object> configs = equipment.getCfgParamMap();
        List<Map<String, Object>> propertys = new ArrayList<>();
        List<Map<String, Object>> cpblInfoList = new ArrayList<>();
        List<String> selIds = (List<String>) configs.get("selCpblIds");
        List<Long> cpblAttrIdList = new ArrayList<>();
        Map<String, Object> quotaSelected = (Map<String, Object>) configs.get("quotaSelected");
        for (String key : quotaSelected.keySet()) {
            Object object = quotaSelected.get(key);
            if (object != null) {
                List<Integer> intList = (List<Integer>) object;
                for (Integer integer : intList) {
                    cpblAttrIdList.add(Long.valueOf(integer));
                }
            }
        }
        configs.keySet().forEach(item -> {
            if (item.startsWith("equip_")) {
                Map<String, Object> tempProperty = new HashMap<>();
                String idStr = item.substring(6, item.length());
                tempProperty.put("id", idStr);
                tempProperty.put("valueStr", configs.get(item));
                EqpTmplAttribute eqpTmplAttribute = iEqpTmplPropertyDao.findById(Long.valueOf(idStr)).get();
                if (eqpTmplAttribute != null) {
                    tempProperty.put("displayName", eqpTmplAttribute.getDisplayName());
                    propertys.add(tempProperty);
                }
            }
        });
        result.put("eqp", equipment);
        result.put("device", iEquipmentTemplateDao.getOne(equipment.getEqpTmplId()));
        result.put("eqpProperty", propertys);
        //能力、能力属性、能力的协议、task配置、接口、接口参数
        //1构建一个<key,<key,value>>的map，里面包括所有选中的能力属性
        EquipmentTemplate equipmentTemplate = iEquipmentTemplateDao.findById(equipment.getEqpTmplId()).get();
        List<Capability> capabilityList = equipmentTemplate.getCapabilities();
        for (Capability capability : capabilityList) {
            Map<String, Map<String, Object>> cpblPropertys = new HashMap<>();
            //只处理选择的能力
            if (selIds != null && selIds.contains(capability.getId().toString())) {
                Map<String, Object> cpblInfo = new HashMap<>();
                //基本信息
                cpblInfo.put("cpbl", capability);
                //属性
//				List<CapacityPropertyDataVo> capabilityAttList = getCapacityPropertyById(capability.getId());
//				for (CapacityPropertyDataVo capacityPropertyDataVo : capabilityAttList)
//				{
//					//只处理选择的能力属性
//					if (cpblAttrIdList.contains(capacityPropertyDataVo.getId()))
//					{
//						String defaultValue =  capacityPropertyDataVo.getValueStr();
//						Map<String, Object> tempProperty = new HashMap<>();
//						tempProperty.put("id", capacityPropertyDataVo.getId());
//						tempProperty.put("value", defaultValue);
//						tempProperty.put("displayName", capacityPropertyDataVo.getDisplayName());
//						cpblPropertys.put(capacityPropertyDataVo.getId().toString(),tempProperty);
//					}
//				}
//				//2已经配置好的能力属性，直接覆盖默认值
//				configs.keySet().forEach(item -> {
//					if (item.startsWith("cpbl_") && !item.startsWith("cpbl_intfc_"))
//					{
//						Map<String, Object> tempProperty = new HashMap<>();
//						String[] cpblKeys = item.split("_");
//						//只处理当前能力的属性
//						if (cpblKeys[1].equals(capability.getId().toString()))
//						{							
//							String cpblKey = cpblKeys[2];
//							tempProperty.put("id", cpblKey);
//							tempProperty.put("value", configs.get(item));
//							CapabilityAttribute capabilityAttribute = iCapabilityPropertyDao.findOne(Long.valueOf(cpblKey));
//							if (capabilityAttribute != null)
//							{
//								tempProperty.put("displayName", capabilityAttribute.getDisplayName());
//								cpblPropertys.put(cpblKey,tempProperty);
//							}
//						}
//					}
//				});
//				cpblInfo.put("property", cpblPropertys.values());
                //协议
                Long protocolId = capability.getProtocolId();
                cpblInfo.put("protocol", iProtocolDao.findById(protocolId).get());
                //task配置
                List<EquipmentCapability> equipmentCapabilities = iEqpCpblDao.findByEpbIdAndCpblId(equipment.getId(), capability.getId());
                if (equipmentCapabilities != null && equipmentCapabilities.size() == 1) {
                    cpblInfo.put("task", equipmentCapabilities.get(0));
                }
                //接口
                List<CapacityInterfaceDataVo> capacityInterfaceDataVos = getCapacityInterfaceById(capability.getId());
                if (capacityInterfaceDataVos.size() == 1) {
                    Intfc intfc = iIntfcDao.findById(capacityInterfaceDataVos.get(0).getIntfcId()).get();
                    cpblInfo.put("intfc", intfc);
                    //同能力属性，第一步取模板数据，第二部覆盖设备里配置的值
                    Map<String, Map<String, Object>> intfcPropertys = new HashMap<>();
                    List<IntfcAttribute> intfcAttributes = iIntfcAttributeDao.getPropertysById(intfc.getId());
                    for (IntfcAttribute intfcAttribute : intfcAttributes) {
                        String defaultValue = intfcAttribute.getValueStr();
                        Map<String, Object> tempProperty = new HashMap<>();
                        tempProperty.put("id", intfcAttribute.getId());
                        tempProperty.put("value", defaultValue);
                        tempProperty.put("displayName", intfcAttribute.getDisplayName());
                        intfcPropertys.put(intfcAttribute.getId().toString(), tempProperty);
                    }
                    for (String item : configs.keySet()) {
                        if (item.startsWith("cpbl_intfc_")) {
                            Map<String, Object> tempProperty = new HashMap<>();
                            String[] cpblKeys = item.split("_");
                            //只处理当前能力当前接口的属性
                            if (cpblKeys[2].equals(capability.getId().toString()) && cpblKeys[3].equals(intfc.getId().toString())) {
                                String intfcPropertyKey = cpblKeys[4];
                                //如果是已配置数据，修改value值
                                if (intfcPropertys.containsKey(intfcPropertyKey)) {
                                    Map<String, Object> tempIntfcProperty = intfcPropertys.get(intfcPropertyKey);
                                    tempIntfcProperty.put("value", configs.get(item));
                                }
                            }
                        }
                    }
                    cpblInfo.put("intfcProperty", intfcPropertys.values());
                }
                cpblInfoList.add(cpblInfo);
            }
        }
        result.put("cpbl", cpblInfoList);
        return result;
    }

    @Override
    public void addEditEquipmentImg(EquipmentParam param) {
        imgFileServer.saveIconFile(param.getBinaryImg(), param.getImgName(), "device", "ISO-8859-1");
        Equipment equipment = iEquipmentDao.findById(param.getId()).get();
        if (equipment != null) {
            equipment.setLogo(param.getImgName());
            iEquipmentDao.save(equipment);
        }
    }

    //查询设备信息
    @Override
    public CommonPage<EquipmentVo> pageDeviceDataByMybatis(EqpTemplPageParam requestParam) {
        long total = deviceMapper.countDeviceDataByMybatis(requestParam);
        List<EquipmentVo> content = deviceMapper.pageDeviceDataByMybatis(requestParam);
        if (!CollectionUtils.isEmpty(content)) {
            List<String> needAttr = requestParam.getNeedAttr();
            if (needAttr != null && needAttr.size() > 0) {
                List<Long> eqpTmplIds = Lists.transform(content, EquipmentVo::getEqpTmplId);
                for (String key : needAttr) {
                    setAttrObj(key, eqpTmplIds, content);
                }
            }
        }
        return new CommonPage<>(content, requestParam, total);
    }

    @Override
    public List<Map> qryEqpCountByTemplAttrs(String orgCode, List<String> attrName) {
        return deviceMapper.qryEqpCountByTemplAttrs(orgCode, attrName);
    }

    @Override
    public List<Map<String, Object>> statisticDeviceByModel(String orgCode) {
        List<Map<String, Object>> list = statisticMapper.statisticDeviceGroupByModel(orgCode);
        list.forEach(map -> {
            if (map.get("eqpTmplId") != null && map.get("eqpIds") != null) {
                Long eqpTmplId = Long.valueOf(map.get("eqpTmplId").toString());
                String ids = map.get("eqpIds").toString();
                List<Long> eqpIds = Arrays.stream(ids.split(",")).map(s -> Long.parseLong(s.trim())).collect(Collectors.toList());
                if (eqpIds.size() > 0) {
                    map.put("alarmTotal", statisticMapper.statisticDeviceHaveAlarmByEqpTmplId(eqpTmplId, eqpIds));
                } else {
                    map.put("alarmTotal", 0);
                }
            }
        });
        return list;
    }

    @Override
    public Map<String, Object> statisticDeviceByAlarmLevel(String orgCode) {
        return statisticMapper.statisticDeviceByAlarmLevel(orgCode);
    }

    @Override
    public List<EquipmentVo> listByOrgCodeAndModel(String orgCode, String model) {
        List<EquipmentVo> list = Lists.newArrayList();
        List<EquipmentTemplateVo> templates = eqpTmplMapper.getByModel(model);
        if (templates != null) {
            for (EquipmentTemplateVo template : templates) {
                List<EquipmentVo> tmplEquipments = deviceMapper.listByOrgCodeAndTmplId(null, template.getId());
                tmplEquipments = tmplEquipments.stream()
                        .filter((EquipmentVo s) ->
                        {
                            String[] strArray = s.getAuthOrg();
                            if (ObjectUtils.isEmpty(strArray)) {
                                strArray = new String[0];
                            }
                            List<String> rowIdList = Arrays.asList(strArray);
                            List<String> orgList = new ArrayList<>();
                            orgList.addAll(rowIdList);
                            orgList.add(s.getOrgCode());
                            for (String org : orgList) {
                                if (orgCode.equals(org)) {
                                    return true;
                                }
                            }
                            return false;
                        })
                        .collect(Collectors.toList());
                List<EqpTmplAttributeBo> eqpAttrs = eqpTmplAttrMapper.listByEqpTmplId(template.getId());
                if (!CollectionUtils.isEmpty(eqpAttrs)) {
                    if (!CollectionUtils.isEmpty(tmplEquipments)) {
                        tmplEquipments.forEach(equipmentVo -> {
                            eqpAttrs.forEach(eqpAttr -> {
                                String eqipId = "equip_" + eqpAttr.getId();
                                Map<String, Object> cfgParamMap = equipmentVo.getCfgParamMap();
                                equipmentVo.getCfgParamJsonObject().put(eqpAttr.getName(), cfgParamMap.get(eqipId));
                            });
                        });
                    }
                }
                list.addAll(tmplEquipments);
            }

        }
        return list;
    }

    @Override
    public List<EquipmentVo> listDeviceAndDeviceCpblAttr(Long equipmentId) {
        return deviceMapper.listDeviceAndDeviceCpblAttr(equipmentId);
    }

    private void setAttrObj(String key, List<Long> eqpTmplIds, List<EquipmentVo> cotent) {
        List<EqpTmplAttributeBo> positionBos = eqpTmplAttrMapper.listByNameAndEqpTmplIds(key, eqpTmplIds);
        if (!CollectionUtils.isEmpty(positionBos)) {
            Map<Long, EqpTmplAttributeBo> positionMap = Maps.uniqueIndex(positionBos, EqpTmplAttributeBo::getEqpTmplId);
            for (EquipmentVo e : cotent) {
                String eqipId = "equip_" + positionMap.get(e.getEqpTmplId()).getId();
                Map<String, Object> cfgParamMap = e.getCfgParamMap();
                e.getCfgParamJsonObject().put(key, cfgParamMap.get(eqipId));
            }
        }
    }

}
