package com.yeejoin.amos.iot.business.service.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.yeejoin.amos.iot.business.dao.mapper.DeviceMapper;
import com.yeejoin.amos.iot.business.dao.repository.IEquipmentDao;
import com.yeejoin.amos.iot.business.dao.repository.IEquipmentTemplateDao;
import com.yeejoin.amos.iot.business.entity.Equipment;
import com.yeejoin.amos.iot.business.entity.EquipmentTemplate;
import com.yeejoin.amos.iot.business.entity.mybatis.CapacityPropertyDataVo;
import com.yeejoin.amos.iot.business.param.QueryMetricDataParam;
import com.yeejoin.amos.iot.business.service.intfc.IHistoryDataService;
import com.yeejoin.amos.iot.core.util.StringUtil;
import com.yeejoin.amos.iot.mongodb.dao.repository.EquipMetricDataRepository;
import com.yeejoin.amos.iot.mongodb.entity.CapabilityData;
import com.yeejoin.amos.iot.mongodb.entity.EqpMetricData;

/**
 * 
 * <pre>
 * 设备历史数据
 * </pre>
 *
 * @author as-chenjiajun
 * @version $Id: HistoryDataServiceImpl.java, v 0.1 2018年3月22日 上午9:56:46 as-chenjiajun Exp $
 */
@Service("historyDataService")
public class HistoryDataServiceImpl implements IHistoryDataService
{
	@Autowired
	private EquipMetricDataRepository equipMetricDataRepository;
	
	@Autowired
	private DeviceMapper deviceMapper;
	
	@Autowired
	private IEquipmentTemplateDao iEquipmentTemplateDao;

	@Autowired
	private IEquipmentDao iEquipmentDao;

	@Override
	public Map<String, List<Map<String, Object>>> findByParam(QueryMetricDataParam param)
	{
		List<EqpMetricData> eqpMetricDatas = equipMetricDataRepository.findByParam(param);
		Map<String, List<Map<String, Object>>> resultMap = new HashMap<String, List<Map<String, Object>>>();
		//增加缓存
		Map<Long, List<CapacityPropertyDataVo>> capabilityAttributesMap = new HashMap<Long, List<CapacityPropertyDataVo>>();
		for (EqpMetricData eqpMetricData : eqpMetricDatas)
		{
			CapabilityData capabilityData = eqpMetricData.getCapabilityData();
			if (capabilityData != null)
			{				
				if (capabilityData.getCpblId() != null)
				{
					Long cpblId = capabilityData.getCpblId();
					if (capabilityAttributesMap.get(cpblId) == null)
					{
						capabilityAttributesMap.put(cpblId, deviceMapper.getCapacityPropertyById(cpblId));
					}
				}
			}
		}
		//数据格式转换
		for (EqpMetricData eqpMetricData : eqpMetricDatas)
		{
			CapabilityData capabilityData = eqpMetricData.getCapabilityData();
			if (capabilityData != null)
			{				
				Long cpblId = capabilityData.getCpblId();
				String cpblName = capabilityData.getCpblName();
				Date exeTime = capabilityData.getExeTime();
				Map<String, Object> dataMap = capabilityData.getMetricDataMap();
				List<CapacityPropertyDataVo> capabilityAttributes = capabilityAttributesMap.get(cpblId);
				dataMap = DeviceServiceHelper.buildCpblAttrData(dataMap, capabilityAttributes);
				for (Entry<String, Object> data : dataMap.entrySet())
				{
					Object value = data.getValue();
					if (StringUtil.isNumeric(value.toString()))
					{
						String dataKey = new StringBuilder().append(cpblId)
								.append("-").append(cpblName).append("-")
								.append(data.getKey()).toString();
						List<Map<String, Object>> itemList = null;
						if (resultMap.get(dataKey) == null)
						{
							itemList = new ArrayList<Map<String, Object>>();
							resultMap.put(dataKey, new ArrayList<Map<String, Object>>());
						} else {
							itemList = resultMap.get(dataKey);
						}
						Map<String, Object> itemMap = new HashMap<String, Object>();
						itemMap.put("exeTime", exeTime);
						itemMap.put("value", Double.parseDouble(value.toString()));
						itemList.add(itemMap);
						resultMap.put(dataKey, itemList);
					}
				}
			}
		}
		return resultMap;
	}

	@Override
	public EquipmentTemplate getDeviceById(Long id)
	{
		Equipment equipment = iEquipmentDao.getOne(id);
		if (equipment == null)
		{
			return null;
		}
		else
		{
			return iEquipmentTemplateDao.getOne(equipment.getEqpTmplId());
		}
	}
	
}
