package com.yeejoin.amos.iot.business.service.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.yeejoin.amos.iot.business.dao.repository.IIntfcAttributeDao;
import com.yeejoin.amos.iot.business.dao.repository.IIntfcDao;
import com.yeejoin.amos.iot.business.entity.Intfc;
import com.yeejoin.amos.iot.business.entity.IntfcAttribute;
import com.yeejoin.amos.iot.business.exceptions.IntfcException;
import com.yeejoin.amos.iot.business.remote.RemoteSecurityService;
import com.yeejoin.amos.iot.business.service.intfc.IIntfcService;
import com.yeejoin.amos.iot.core.enums.QueryOperatorEnum;
import com.yeejoin.amos.iot.core.util.StringUtil;
import com.yeejoin.amos.iot.core.util.query.BaseQuerySpecification;
import com.yeejoin.amos.op.core.common.query.DaoCriteria;
import com.yeejoin.amos.op.core.entity.handler.EntityChangeHandler;
import com.yeejoin.amos.op.core.jpa.GenericManagerImpl;
import com.yeejoin.amos.op.core.jpa.IBaseRepository;

/**
 * 
 * <pre>
 * 设备接口数据Service实现
 * </pre>
 *
 * @author as-shibaobao
 * @version $Id: IntfcServiceImpl.java, v 0.1 2018年1月26日 上午11:11:35 as-shibaobao
 *          Exp $
 */
@Service("intfcService")
public class IntfcServiceImpl 
implements IIntfcService {

//	public IntfcServiceImpl(IBaseRepository<Intfc, Long> repository)
//	{
//		super(repository);
//	}

	@Autowired
	RemoteSecurityService userController;
	
	@Autowired
	private IIntfcDao intfcDao;

	@Autowired
	private IIntfcAttributeDao intfcAttributeDao;

	@Transactional
	@Override
	public Intfc saveBasicInfoAndAttrs(Intfc intfc) throws IntfcException {
		List<IntfcAttribute> intfcAttrs = intfc.getAttributes();
		Intfc basic = null;
		if (intfc.getUserId() == null) {
			intfc.setUserId(userController.getUserId());
		}
		if (intfc.getDicIdCommMode() == null || intfc.getDicIdCommMode() <= 0) {
			throw new IntfcException("通讯方式异常！");
		}

		if (!StringUtil.isNotEmpty(intfc.getId())) {
			intfc.setAttributes(null);
			intfc.setCreateTime(new Date());

			basic = intfcDao.saveAndFlush(intfc);
		} else if (intfc.getId() > 0) {
			Intfc exist = findOne(intfc.getId());

			exist.setDicIdCommMode(intfc.getDicIdCommMode());
			exist.setRemark(intfc.getRemark());
			exist.setName(intfc.getName());
			exist.setUpdateTime(new Date());

			basic = intfcDao.saveAndFlush(exist);
		}

		if (StringUtil.isNotEmpty(intfcAttrs) && intfcAttrs.size() > 0) {
			for (IntfcAttribute attr : intfcAttrs) {
				attr.setIntfcId(basic.getId());
			}

			intfcAttrs = intfcAttributeDao.saveAll(intfcAttrs);

			basic.setAttributes(intfcAttrs);
		}

		return basic;
	}

	@Override
	public Page<IntfcAttribute> queryIntfcAttributeList(List<DaoCriteria> criterias, Pageable pageable) {
		BaseQuerySpecification<IntfcAttribute> spec = new BaseQuerySpecification<>(criterias);
		return intfcAttributeDao.findAll(spec, pageable);
	}

	@Override
	public IntfcAttribute editAttribute(IntfcAttribute intfcAttribute) throws IntfcException {
		return intfcAttributeDao.saveAndFlush(intfcAttribute);
	}

	@Override
	public void delIntfc(Long id) throws IntfcException {
		EntityChangeHandler.fireResourceReomoved(findOne(id));
		intfcDao.delIntfcById(id);
	}

	@Override
	public Map<String, Long> countIntfcInfo(List<DaoCriteria> daoCriteria)
	{
		List<DaoCriteria> pubCriterias = new ArrayList<>();
		List<DaoCriteria> priCriterias = new ArrayList<>();
		for (DaoCriteria criteria : daoCriteria)
		{
			priCriterias.add(criteria);
		}
		pubCriterias.add(new DaoCriteria("visibility",
				QueryOperatorEnum.EQUAL.getName(), 0));
		priCriterias.add(new DaoCriteria("visibility",
				QueryOperatorEnum.EQUAL.getName(), 1));
		BaseQuerySpecification<Intfc> pubSpec = new BaseQuerySpecification<>(
				pubCriterias);
		BaseQuerySpecification<Intfc> priSpec = new BaseQuerySpecification<>(
				priCriterias);
		Long pubCount = intfcDao.count(pubSpec);
		Long priCount = intfcDao.count(priSpec);
		Map<String, Long> map = new HashMap<>();
		map.put("count", pubCount + priCount);
		map.put("pubCount", pubCount);
		map.put("priCount", priCount);
		return map;
	}
	
	@Override
	@Transactional
	public List<Intfc> findAll(List<DaoCriteria> criterias) {
//		if(criterias.size() != 0){
//			for (DaoCriteria daoCriteria : criterias) {
//				if("visibility".equals(daoCriteria.getPropertyName()) && "1".equals(daoCriteria.getValue())){
//					criterias.add(new DaoCriteria("userId", QueryOperatorEnum.EQUAL.getName(),
//							Long.parseLong(CurrentAuthentication.getAuthenticatedUserId())));
//					break;
//				}
//			}
//		}
		BaseQuerySpecification<Intfc> spec = new BaseQuerySpecification<>(criterias);
		return intfcDao.findAll(spec);
	}

	@Override
	public Intfc findOne(Long intfcId) {
		return intfcDao.getOne(intfcId);
	}
}
