/**
 *
 */
package com.yeejoin.amos.iot.business.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.yeejoin.amos.iot.business.constants.IotConstant;
import com.yeejoin.amos.iot.business.dao.mapper.CapacityAttrMapper;
import com.yeejoin.amos.iot.business.dao.mapper.DeviceMapper;
import com.yeejoin.amos.iot.business.dao.mapper.MetricDataMapper;
import com.yeejoin.amos.iot.business.dao.repository.IAlarmDao;
import com.yeejoin.amos.iot.business.dao.repository.ICapabilityDao;
import com.yeejoin.amos.iot.business.dao.repository.ICapabilityPropertyDao;
import com.yeejoin.amos.iot.business.dao.repository.IEquipmentDao;
import com.yeejoin.amos.iot.business.dao.repository.IMetricDataDao;
import com.yeejoin.amos.iot.business.entity.Alarm;
import com.yeejoin.amos.iot.business.entity.Capability;
import com.yeejoin.amos.iot.business.entity.Equipment;
import com.yeejoin.amos.iot.business.entity.MetricData;
import com.yeejoin.amos.iot.business.entity.mybatis.CapacityAttrBo;
import com.yeejoin.amos.iot.business.entity.mybatis.CapacityPropertyDataVo;
import com.yeejoin.amos.iot.business.entity.mybatis.MetricDataBo;
import com.yeejoin.amos.iot.business.entity.mybatis.MetricDataVo;
import com.yeejoin.amos.iot.business.param.InputColumn;
import com.yeejoin.amos.iot.business.param.MetricDataPageParam;
import com.yeejoin.amos.iot.business.service.intfc.IMetricDataService;
import com.yeejoin.amos.iot.core.common.response.CommonPage;
import com.yeejoin.amos.iot.core.util.query.BaseQuerySpecification;
import com.yeejoin.amos.iot.socket.websocket.vo.RealTimeMetricDataVo;
import com.yeejoin.amos.op.core.common.query.DaoCriteria;
import com.yeejoin.amos.op.core.jpa.GenericManagerImpl;
import com.yeejoin.amos.op.core.jpa.IBaseRepository;
import com.yeejoin.amos.op.core.repository.NativeRespository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Random;

/**
 * 指标数据service实现类
 *
 * @author as-youjun
 *
 */
@Service("metricDataService")
public class MetricDataServiceImpl 
        implements IMetricDataService {

    @Autowired
    private NativeRespository nativeRespository;

    @Autowired
    private MetricDataMapper metricDataMapper;

    @Autowired
    private CapacityAttrMapper capacityAttrMapper;

    @Autowired
    private IMetricDataDao metricDataDao;

    @Autowired
    private ICapabilityDao iCapabilityDao;

    @Autowired
    private ICapabilityPropertyDao iCapabilityPropertyDao;

    @Autowired
    private DeviceMapper deviceMapper;

    @Autowired
    private IEquipmentDao iEquipmentDao;

    @Autowired
    private IAlarmDao alarmDao;

    //演示数据
    private static final List<String> alarmTypes = Arrays.asList("A", "B", "C", "AB", "AC", "BC", "ABC");

    /**
     * 分页查询指标数据
     */
    @Override
    public CommonPage<MetricDataVo> pageMetricData(MetricDataPageParam pageParam) {
        long totalNum = metricDataMapper.countMetricData();
        List<MetricDataVo> metricDataVos = metricDataMapper
                .pageMetricData(pageParam);
        CommonPage<MetricDataVo> page = new CommonPage<MetricDataVo>(
                metricDataVos, pageParam, totalNum);
        return page;
    }

    /**
     * 保存指标数据
     */
    @Transactional
    @Override
    public void saveMetricData(MetricData metricData) {
        // 保存到mysql,指标数据只存储最后一条上报数据

        MetricData metricDataInstance = metricDataDao.findLastDataByEqpIdAndCapId(metricData.getEqpId(), metricData.getCpblId());
        if (metricDataInstance != null) {
            metricDataInstance.setExeTime(metricData.getExeTime());
            metricDataInstance.setMetricKey(metricData.getMetricKey());
            metricDataInstance.setMetricJson(metricData.getMetricJson());
            metricDataInstance.setCreateTime(new Date());
            metricDataDao.save(metricDataInstance);
        } else {
            metricData.setCreateTime(new Date());
            metricDataDao.save(metricData);
        }

    }

    @Override
    public Map<String, Object> queryEquipQuotaData(Long equipId, String[] quotaKey, String userName) {
        Equipment equipment = iEquipmentDao.findById(equipId).get();
        Map<String, Object> resultMap = new HashMap<>();
        if (null == equipment) {
            return resultMap;
        }
        Map<Long, List<RealTimeMetricDataVo>> realTimeMetricDataVoMap = new HashMap<>();
        List<RealTimeMetricDataVo> realTimeMetricDataVoList = new ArrayList<>();
        Map<Long, List<Long>> cpblAttrIdArr = new HashMap<>();
        for (String quotaStr : quotaKey) {
            if (!quotaStr.contains("-")) {
                List<Long> cpIds = iCapabilityPropertyDao.findIdByCapabilityId(Long.valueOf(quotaStr));
                cpblAttrIdArr.put(Long.valueOf(quotaStr), cpIds);
            } else {
                String[] quotaArr = quotaStr.split("-");
                Long key = Long.valueOf(quotaArr[0]);
                Long value = Long.valueOf(quotaArr[1]);
                if (cpblAttrIdArr.containsKey(key)) {
                    cpblAttrIdArr.get(key).add(value);
                } else {
                    List<Long> valueList = new ArrayList<>();
                    valueList.add(value);
                    cpblAttrIdArr.put(key, valueList);
                }
            }
        }
        if (cpblAttrIdArr.isEmpty()) {
            return resultMap;
        }
        for (Entry<Long, List<Long>> entry : cpblAttrIdArr.entrySet()) {
            Capability capability = iCapabilityDao.findById(entry.getKey()).get();
            RealTimeMetricDataVo realTimeMetricDataVo = new RealTimeMetricDataVo();
            realTimeMetricDataVo.setEqpId(equipId);
            realTimeMetricDataVo.setCpblId(entry.getKey());
            realTimeMetricDataVo.setCpblName(capability.getName());
            realTimeMetricDataVo.setTime(new Date());

            Map<String, Object> map = new HashMap<>();
            map.put("eqpId", equipId);
            map.put("cpblId", entry.getKey());
            Map<String, String> paramMap = new HashMap<>();
            MetricData metricData = metricDataMapper.getRealTimeData(map);
            List<CapacityPropertyDataVo> capabilityAttributes = deviceMapper.getCapacityPropertyByCpblProIds(entry.getValue());
            if (metricData != null && metricData.getMetricJson() != null && metricData.getMetricJson() != "") {
                paramMap = DeviceServiceHelper.buildCpblAttrData(metricData.getMetricJson(), capabilityAttributes);
            } else {
                List<Long> cpblAttrIds = cpblAttrIdArr.get(entry.getKey());
                for (Long cpblAttrId : cpblAttrIds) {
                    for (CapacityPropertyDataVo capacityPropertyDataVo : capabilityAttributes) {
                        if (cpblAttrId.equals(capacityPropertyDataVo.getId())) {
                            if (userName.equals("yanshi"))//演示数据
                            {
                                paramMap.put(
                                        capacityPropertyDataVo.getDisplayName(),
                                        String.valueOf(new BigDecimal(Math.random())
                                                .setScale(2,
                                                        BigDecimal.ROUND_HALF_UP)
                                                .doubleValue()));
                            } else {
                                //正常处理
                                paramMap.put(capacityPropertyDataVo.getDisplayName(), "");
                            }
                            break;
                        }
                    }
                }
            }
            realTimeMetricDataVo.setMetricDataMap(paramMap);
            realTimeMetricDataVoList.add(realTimeMetricDataVo);
            realTimeMetricDataVoMap.put(equipId, realTimeMetricDataVoList);
        }
        resultMap.put("metricData", realTimeMetricDataVoMap);
        resultMap.put("isWarning", equipment.getIsWarning());
        if (userName.equals("yanshi"))//演示数据
        {
            resultMap.put("imgStatusValue", alarmTypes.get(new Random().nextInt(alarmTypes.size())));
        }
        //计算告警最高等级
        List<Alarm> alarms = alarmDao.fetchAlarmsByRqpId(equipment.getId());
        if (alarms != null) {
            if (alarms.isEmpty()) {
                //如果没有查询到告警，强制设置isWarning为false
                resultMap.put("isWarning", false);
            } else {
                Integer maxLevel = alarms.stream().mapToInt(i -> i.getLevel()).summaryStatistics().getMin();
                resultMap.put("maxLevel", maxLevel);
            }
        }
        return resultMap;
    }

    @Override
    public List<MetricData> findAllMetricData(List<DaoCriteria> daoCriterias) {
    	BaseQuerySpecification<MetricData> spec = new BaseQuerySpecification<>(daoCriterias);
        return metricDataDao.findAll(spec);
    }

    @Override
    public List<MetricDataBo> findAllMetricDataByEquipId(long id) {
        List<MetricDataBo> list = metricDataMapper.findAllMetricDataByEquipId(id);
        list.forEach(metricDataBo -> {
            List<CapacityAttrBo> capacityAttrBos = capacityAttrMapper.listByCapacityId(metricDataBo.getCpblId());
            JSONObject metricJsonObj = JSONObject.parseObject(metricDataBo.getMetricJson());
            JSONArray jsonArray = new JSONArray();
            capacityAttrBos.forEach(capacityAttrBo -> {
                String value = metricJsonObj.getString(capacityAttrBo.getName());
                if (value != null) {
                    JSONObject json = new JSONObject();
                    json.put("name", capacityAttrBo.getDisplayName());
                    json.put("key", capacityAttrBo.getName());
                    json.put("value", value);
                    jsonArray.add(json);
                }
            });
            metricDataBo.setMetricList(jsonArray);
        });
        return list;
    }

    public Object getEchartStatisData(List<InputColumn> columnInfo) {
        StringBuilder w_sb = new StringBuilder();
        StringBuilder m_sb = new StringBuilder();
        StringBuilder t_sb = new StringBuilder();
        Map<String, List<Object>> jsonResult = new HashMap<>();
        String x = null;
        for (InputColumn inputColumn : columnInfo) {
            if (inputColumn.getAreaCode().contains(IotConstant.DIMENSION)) {// x轴
                x = inputColumn.getName();
                String x_key = inputColumn.getAliasName();
                Long x_eqpId = Long.parseLong(inputColumn.getDsId());
                w_sb.append("t.").append(x).append(" AS ").append(x).append("_dimension_").append(" , ");
                m_sb.append("metric_json->>'$.").append(x).append("' AS ").append(x).append(" , ");
                t_sb.append("eqp_id = ").append(x_eqpId).append(" OR ");
                jsonResult.put(x_key, new ArrayList<Object>());
            } else if (inputColumn.getAreaCode().contains(IotConstant.MEASURE)) {// y
                String y = inputColumn.getName();
                String y_key = inputColumn.getAliasName();
                String type = inputColumn.getType();
                Long y_eqpId = Long.parseLong(inputColumn.getDsId());
                w_sb.append(getType(type)).append("(t.").append(y).append(") AS ").append(y).append("_measure_").append(getType(type)).append(" , ");
                m_sb.append("metric_json->>'$.").append(y).append("' AS ").append(y).append(" , ");
                t_sb.append("eqp_id = ").append(y_eqpId).append(" OR ");
                jsonResult.put(y_key, new ArrayList<Object>());
            }
        }
        String sql = new StringBuilder().append("SELECT ")
                .append(w_sb.substring(0, w_sb.length() - 2))
                .append(" FROM (SELECT eqp_id,")
                .append(m_sb.substring(0, m_sb.length() - 2)).append(" FROM t_metric_data) t ")
                .append(" where ").append(t_sb.substring(0, t_sb.length() - 3)).append(" GROUP BY t.").append(x)
                .toString();
        List<?> list = nativeRespository.executeQuerysToMap(sql);
        for (Object object : list) {
            @SuppressWarnings("unchecked")
            Map<String, Object> map = (Map<String, Object>) object;
            for (String key : map.keySet()) {
                jsonResult.get(key).add(map.get(key));
            }
        }
        return JSON.toJSON(jsonResult);
    }

    public Object getEchartPStatisData(List<InputColumn> columnInfo) {
        StringBuilder w_sb = new StringBuilder();
        StringBuilder m_sb = new StringBuilder();
        StringBuilder t_sb = new StringBuilder();
        Map<String, List<Object>> jsonResult = new HashMap<>();
        String x = null;
        for (InputColumn inputColumn : columnInfo) {
            if (inputColumn.getAreaCode().contains(IotConstant.DIMENSION)) {// x轴
                x = inputColumn.getName();
                String x_key = inputColumn.getAliasName();
                Long x_eqpId = Long.parseLong(inputColumn.getDsId());
                w_sb.append("t.").append(x).append(" AS ").append(x).append("_dimension_").append(" , ");
                m_sb.append("metric_json->>'$.").append(x).append("' AS ").append(x).append(" , ");
                t_sb.append("eqp_id = ").append(x_eqpId).append(" OR ");
                jsonResult.put(x_key, new ArrayList<Object>());
            } else if (inputColumn.getAreaCode().contains(IotConstant.MEASURE)) {// y
                String y = inputColumn.getName();
                String y_key = inputColumn.getAliasName();
                String type = inputColumn.getType();
                Long y_eqpId = Long.parseLong(inputColumn.getDsId());
                w_sb.append(getType(type)).append("(t.").append(y).append(") AS ").append(y).append("_measure_").append(getType(type)).append(" , ");
                m_sb.append("metric_json->>'$.").append(y).append("' AS ").append(y).append(" , ");
                t_sb.append("eqp_id = ").append(y_eqpId).append(" OR ");
                jsonResult.put(y_key, new ArrayList<Object>());
            }
        }
        String sql = new StringBuilder().append("SELECT ")
                .append(w_sb.substring(0, w_sb.length() - 2))
                .append(" FROM (SELECT eqp_id,")
                .append(m_sb.substring(0, m_sb.length() - 2)).append(" FROM t_metric_data) t ")
                .append(" where ").append(t_sb.substring(0, t_sb.length() - 3)).append(" GROUP BY t.").append(x)
                .toString();
        List<?> list = nativeRespository.executeQuerysToMap(sql);
        for (Object object : list) {
            @SuppressWarnings("unchecked")
            Map<String, Object> map = (Map<String, Object>) object;
            Map<String, Object> infoMap = new HashMap<String, Object>();
            for (String key : map.keySet()) {
                if (key.contains(IotConstant.DIMENSION)) {
                    String xr = map.get(key) == null ? "" : map.get(key).toString();
                    infoMap.put("name", xr);
                    jsonResult.get(key).add(map.get(key));
                } else if (key.contains(IotConstant.MEASURE)) {
                    String yr = map.get(key) == null ? "" : map.get(key).toString();
                    infoMap.put("value", yr);
                    jsonResult.get(key).add(infoMap);
                }
            }
        }
        return JSON.toJSON(jsonResult);
    }

    public String getType(String type) {
        switch (type) {
            case "STRING":
                return "COUNT";
            case "DATE":
                return "COUNT";
            case "VARIABLE":
                return "SUM";
            case "FLOAT":
                return "SUM";
            case "BOOLEAN":
                return "COUNT";
            case "ENUMERATION":
                return "COUNT";
            case "PASSWORD":
                return "COUNT";
            default:
                return "SUM";
        }
    }

	@Override
	public void save(MetricData metricData) {
		metricDataDao.save(metricData);
	}

}
