/**
 *
 */
package com.yeejoin.amos.iot.business.service.impl;

import com.yeejoin.amos.iot.business.constants.IotConstant;
import com.yeejoin.amos.iot.business.dao.mapper.DeviceMapper;
import com.yeejoin.amos.iot.business.dao.mapper.MonitorMapper;
import com.yeejoin.amos.iot.business.dao.repository.IEquipmentDao;
import com.yeejoin.amos.iot.business.dao.repository.IEquipmentTemplateDao;
import com.yeejoin.amos.iot.business.dao.repository.IIntfcAttributeDao;
import com.yeejoin.amos.iot.business.dao.repository.IListenCommDao;
import com.yeejoin.amos.iot.business.entity.IntfcAttribute;
import com.yeejoin.amos.iot.business.entity.ListenComm;
import com.yeejoin.amos.iot.business.entity.mybatis.*;
import com.yeejoin.amos.iot.business.enums.CapabilityType;
import com.yeejoin.amos.iot.business.enums.MonitorType;
import com.yeejoin.amos.iot.business.param.*;
import com.yeejoin.amos.iot.business.polling.MonitorUtil;
import com.yeejoin.amos.iot.business.service.intfc.IMonitorService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.collections.MapUtils;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 监测服务实现类
 *
 * @author as-youjun
 */
@Service("monitorService")
public class MonitorServiceImpl implements IMonitorService {

    /**
     * 日志记录器
     */
    private static final Logger logger = Logger
            .getLogger(MonitorServiceImpl.class);

    @Autowired
    private MonitorMapper monitorMapper;

    @Autowired
    private DeviceMapper deviceMapper;

    @Autowired
    private IIntfcAttributeDao intfcAttributeDao;

    @Autowired
    private IListenCommDao listenCommDao;

    /**
     * 查询监测设备数量
     */
    @Override
    public long countMonitorEquipment(QueryMonitorEqpParam param) {
        long eqpCount = monitorMapper.countMonitorEquipment(param);
        return eqpCount;
    }

    /**
     * 查询监测设备
     */
    @Override
    public List<EquipmentBo> findMonitorEquipment(QueryMonitorEqpParam param) {
        // 1.查询设备数据
        List<EquipmentBo> equipmentBos = monitorMapper.findMonitorEquipment(param);
        if (CollectionUtils.isEmpty(equipmentBos)) {
            return equipmentBos;
        }
        // 2.查询设备能力
        List<Long> eqpTmplIds = new ArrayList<>();
        equipmentBos.forEach(eqpb -> {
            if (!eqpTmplIds.contains(eqpb.getEqpTmplId())) {
                eqpTmplIds.add(eqpb.getEqpTmplId());
            }
        });
        QueryCapabilityParam queryCapabilityParam = new QueryCapabilityParam();
        queryCapabilityParam.setCpblType(CapabilityType.COLLECT.getValue());
        queryCapabilityParam.setEqpTmplIds(eqpTmplIds);
        List<CapabilityBo> capabilityBos = monitorMapper
                .findCpblByEqpTmplIds(queryCapabilityParam);
        if (CollectionUtils.isNotEmpty(capabilityBos)) {
            // CapabilityBo填充到EquipmentBo
            MonitorUtil.fillCapabilityBo2EquipmentBo(capabilityBos,
                    equipmentBos);
        } else {
            return equipmentBos;
        }
        // 3.查询设备能力接口
        QueryIntfcParam queryIntfcParam = new QueryIntfcParam();
        queryIntfcParam.setCpblType(CapabilityType.COLLECT.getValue());
        queryIntfcParam.setEqpTmplIds(eqpTmplIds);
        List<IntfcBo> intfcBos = monitorMapper
                .findIntfcOfCpblByEqpTmplIds(queryIntfcParam);
        if (CollectionUtils.isNotEmpty(intfcBos)) {
            // IntfcBo填充到CapabilityBo
            equipmentBos.forEach(equipmentBo -> {
                MonitorUtil.fillIntfcBo2CapabilityBo(intfcBos, equipmentBo.getCapabilityBos());
            });
            // 接口属性填充到接口
            List<Long> intfcIds = new ArrayList<Long>();
            intfcBos.forEach(item -> {
                intfcIds.add(item.getId());
            });
            List<IntfcAttribute> intfcAttributes = intfcAttributeDao
                    .findAllByIntfcIdIn(intfcIds);
            if (CollectionUtils.isNotEmpty(intfcAttributes)) {
                equipmentBos.forEach(equipmentBo -> {
                    if (CollectionUtils.isNotEmpty(equipmentBo.getCapabilityBos())) {
                        equipmentBo.getCapabilityBos().forEach(capabilityBo -> {
                            MonitorUtil.fillIntfcAttr2Intfc(intfcAttributes, capabilityBo.getIntfcBos());
                        });
                    }
                });
            } else {
                logger.debug("intfcAttributes is empty!");
            }
            //能力属性填充到能力
            fillCpblAttr2Cpbl(equipmentBos);
        }
        // 4.查询设备接口
        QueryEqpTmplIntfcParam queryEqpTmplIntfcParam = new QueryEqpTmplIntfcParam();
        queryEqpTmplIntfcParam.setEqpTmplIds(eqpTmplIds);
        List<EqpTmplIntfcBo> eqpTmplIntfcBos = monitorMapper
                .findEqpTmplIntfcByEqpTmplIds(queryEqpTmplIntfcParam);
        if (CollectionUtils.isNotEmpty(eqpTmplIntfcBos)) {
            //设备模板接口填充到设备
            MonitorUtil.fillEqpTmplIntfc2EquipmentBo(eqpTmplIntfcBos, equipmentBos);
            //接口填充到设备模板接口
            equipmentBos.forEach(equipmentBo -> {
                MonitorUtil.fillIntfcBo2EqpTmplIntfcBo(intfcBos, equipmentBo.getEqpTmplIntfcs());
            });
        } else {
            logger.debug("eqpTmplIntfcBos is empty!");
        }
        // 5.查询设备模板属性
        QueryEqpTmplAttrParam queryEqpTmplAttrParam = new QueryEqpTmplAttrParam();
        queryEqpTmplAttrParam.setEqpTmplIds(eqpTmplIds);
        List<EqpTmplAttributeBo> eqpTmplAttributes = monitorMapper
                .findEqpTmplAttrByEqpTmplIds(queryEqpTmplAttrParam);
        if (CollectionUtils.isNotEmpty(eqpTmplAttributes)) {
            MonitorUtil.fillEqpTmplAttribute2EquipmentBo(eqpTmplAttributes,
                    equipmentBos);
        } else {
            logger.debug("eqpTmplAttributes is empty!");
        }
        return equipmentBos;
    }

    /**
     * 能力属性填充到能力
     *
     * @param equipmentBos
     */
    private void fillCpblAttr2Cpbl(List<EquipmentBo> equipmentBos) {
        if (CollectionUtils.isEmpty(equipmentBos)) {
            return;
        }
        //1.获取批量设备的能力属性
        List<Long> cpblIds = new ArrayList<>();
        equipmentBos.forEach(ebo -> {
            if (CollectionUtils.isNotEmpty(ebo.getCapabilityBos())) {
                ebo.getCapabilityBos().forEach(cbo -> {
                    if (!cpblIds.contains(cbo.getId())) {
                        cpblIds.add(cbo.getId());
                    }
                });
            } else {
                logger.debug("ebo.getCapabilityBos() is empty when equipmentBoId=" + ebo.getId());
            }
        });
        if (CollectionUtils.isEmpty(cpblIds)) {
            return;
        }
        List<CapacityPropertyDataVo> cpblAttrs = deviceMapper
                .getCapacityPropertyByCpblIds(cpblIds);
        if (CollectionUtils.isEmpty(cpblAttrs)) {
            return;
        }
        //2.填充设备能力属性
        equipmentBos.forEach(ebo -> {
            if (CollectionUtils.isNotEmpty(ebo.getCapabilityBos())) {
                Map<String, List<Object>> cpblAttrMap = (Map<String, List<Object>>) ebo
                        .getParamMap().get(IotConstant.EquipmentConstant.QUOTASELECTED);
                if (MapUtils.isNotEmpty(cpblAttrMap)) {
                    MonitorUtil.fillCpblAttr2Capablity(cpblAttrs, ebo.getCapabilityBos(), cpblAttrMap);
                } else {
                    logger.debug("cpblAttrMap is empty!");
                }
            } else {
                logger.debug("ebo.getCapabilityBos() is empty when equipmentBoId=" + ebo.getId());
            }
        });
    }

    /**
     * 根据设备id查询EqpGraphNodeBo
     *
     * @param eqpId
     * @return
     */
    @Override
    public EqpGraphNodeBo findEqpGraphNodeByEqpId(Long eqpId) {
        EqpGraphNodeBo eqpGraphNodeBo = null;
        List<EqpGraphNodeBo> eqpGraphNodeBos = monitorMapper
                .findEqpGraphNodeByEqpId(eqpId);
        if (CollectionUtils.isNotEmpty(eqpGraphNodeBos)) {
            eqpGraphNodeBo = eqpGraphNodeBos.get(0);
        }
        return eqpGraphNodeBo;
    }

    /**
     * 查询tcp端口信息
     *
     * @param param
     * @return
     */
    @Override
    public List<ListenPortBo> findTcpPortBo(QueryMonitorEqpParam param) {
        // 1.查询设备数据
        List<EquipmentBo> equipmentBos = monitorMapper
                .findMonitorEquipment(param);
        List<ListenPortBo> tcpPortBos = new ArrayList<>();
        if (CollectionUtils.isEmpty(equipmentBos)) {
            return tcpPortBos;
        }
        List<ListenComm> listenComms =
                listenCommDao.findAll();
        if (CollectionUtils.isEmpty(listenComms)) {
            return tcpPortBos;
        }
        Set<Long> listenCommIds = new HashSet<>();
        listenComms.forEach(listenComm -> {
            listenCommIds.add(listenComm.getCommDicId());
        });
        // 2.查询设备能力接口
        List<Long> eqpTmplIds = new ArrayList<>();
        equipmentBos.forEach(eqpb -> {
            if (!eqpTmplIds.contains(eqpb.getEqpTmplId())) {
                eqpTmplIds.add(eqpb.getEqpTmplId());
            }
        });
        QueryIntfcParam queryIntfcParam = new QueryIntfcParam();
        queryIntfcParam.setCpblType(CapabilityType.COLLECT.getValue());
        queryIntfcParam.setEqpTmplIds(eqpTmplIds);
        List<IntfcBo> intfcBos = monitorMapper
                .findIntfcOfCpblByEqpTmplIds(queryIntfcParam);
        if (CollectionUtils.isNotEmpty(intfcBos)) {
            // 接口属性填充到接口
            List<Long> intfcIds = new ArrayList<Long>();
            intfcBos.forEach(item -> {
                intfcIds.add(item.getId());
            });
            List<IntfcAttribute> intfcAttributes = intfcAttributeDao
                    .findAllByIntfcIdIn(intfcIds);
            MonitorUtil.fillIntfcAttr2Intfc(intfcAttributes, intfcBos);
        }
        // 3.查询设备接口
        QueryEqpTmplIntfcParam queryEqpTmplIntfcParam = new QueryEqpTmplIntfcParam();
        queryEqpTmplIntfcParam.setEqpTmplIds(eqpTmplIds);
        List<EqpTmplIntfcBo> eqpTmplIntfcBos = monitorMapper
                .findEqpTmplIntfcByEqpTmplIds(queryEqpTmplIntfcParam);
        MonitorUtil.fillEqpTmplIntfc2EquipmentBo(eqpTmplIntfcBos, equipmentBos);
        MonitorUtil.fillIntfcBo2EqpTmplIntfcBo(intfcBos, eqpTmplIntfcBos);
        equipmentBos.forEach(equipmentBo -> {
            if (CollectionUtils.isNotEmpty(eqpTmplIntfcBos)) {
                for (EqpTmplIntfcBo eqpTmplIntfcBo : eqpTmplIntfcBos) {
                    if (!listenCommIds.contains(eqpTmplIntfcBo.getCommId())) {
                        continue;
                    }
                    if (CollectionUtils.isEmpty(eqpTmplIntfcBo.getIntfcAttrs())) {
                        continue;
                    }
                    for (IntfcAttribute intfcAttribute : eqpTmplIntfcBo.getIntfcAttrs()) {
                        Map<String, Object> paramMap = equipmentBo.getParamMap();
                        if (MapUtils.isNotEmpty(paramMap)) {
                            Object obj = paramMap.get(IotConstant.EquipmentConstant.KEY_PREFIX_INTFC + intfcAttribute.getIntfcId() + IotConstant.EquipmentConstant.UNDERLINE + intfcAttribute.getId());
                            if (obj != null) {
                                ListenPortBo tcpPortBo = new ListenPortBo();
                                tcpPortBo.setEqpTmplId(eqpTmplIntfcBo.getEqpTmplId());
                                tcpPortBo.setIntfcAttrId(intfcAttribute.getId());
                                tcpPortBo.setIntfcAttrName(intfcAttribute.getName());
                                tcpPortBo.setIntfcId(intfcAttribute.getIntfcId());
                                int port = Integer.valueOf(obj.toString());
                                tcpPortBo.setPort(port);
                                tcpPortBos.add(tcpPortBo);
                            }
                        }
                    }
                }
            } else {
                logger.debug("eqpTmplIntfcBos is empty!");
            }
        });

        return tcpPortBos;
    }

//	@Override
//	public List<ListenPortBo> findAllTcpPortBo(QueryMonitorEqpParam param) {
//	        // 1.查询设备数据
//	        List<EquipmentBo> equipmentBos = monitorMapper.findMonitorEquipmentPorts(param);	       
//	        List<ListenPortBo> tcpPortBos = new ArrayList<>();
//	        if (CollectionUtils.isEmpty(equipmentBos)) {
//	            return tcpPortBos;
//	        }
//	        equipmentBos.forEach(equipmentBo -> {
//	            Map<String, Object> paramMap = equipmentBo.getParamMap();
//	            if (MapUtils.isNotEmpty(paramMap)) {
//	                for (Map.Entry<String, Object> entry : paramMap.entrySet()) {
//	                    String key =entry.getKey();
//	                    Object obj =entry.getValue();
//	                    if((key.startsWith(IotConstant.EquipmentConstant.KEY_PREFIX_CPBL+IotConstant.EquipmentConstant.KEY_PREFIX_INTFC))&&obj != null){
//		                    ListenPortBo tcpPortBo = new ListenPortBo();
//		                    int port = Integer.valueOf(obj.toString());
//		                    tcpPortBo.setPort(port);
//		                    tcpPortBo.setEqpTmplId(equipmentBo.getEqpTmplId());
//		                    tcpPortBos.add(tcpPortBo);
//	                    }
//                   	}
//
//	            }
//	        });
//
//	        return tcpPortBos;
//	}

	@Override
	public List<ListenPortBo> findAllTcpPortBo(QueryMonitorEqpParam param) {
	        // 1.查询设备数据
	        List<EquipmentBo> equipmentBos = monitorMapper.findMonitorEquipmentPorts(param);	       
	        List<ListenPortBo> tcpPortBos = new ArrayList<>();
	        if (CollectionUtils.isEmpty(equipmentBos)) {
	            return tcpPortBos;
	        }
	        List<ListenComm> listenComms =listenCommDao.findAll();
	        if (CollectionUtils.isEmpty(listenComms)) {
	            return tcpPortBos;
	        }
	        Set<Long> listenCommIds = new HashSet<>();
	        listenComms.forEach(listenComm -> {
	            listenCommIds.add(listenComm.getCommDicId());
	        });
	        // 2.查询设备能力接口
	        List<Long> eqpTmplIds = new ArrayList<>();
	        equipmentBos.forEach(eqpb -> {
	            if (!eqpTmplIds.contains(eqpb.getEqpTmplId())) {
	                eqpTmplIds.add(eqpb.getEqpTmplId());
	            }
	        });
	        QueryIntfcParam queryIntfcParam = new QueryIntfcParam();
	        queryIntfcParam.setCpblType(CapabilityType.COLLECT.getValue());
	        queryIntfcParam.setEqpTmplIds(eqpTmplIds);
	        List<IntfcBo> intfcBos = monitorMapper.findIntfcOfCpblByEqpTmplIds(queryIntfcParam);
	        if (CollectionUtils.isNotEmpty(intfcBos)) {
	            // 接口属性填充到接口
	            List<Long> intfcIds = new ArrayList<Long>();
	            intfcBos.forEach(item -> {
	                intfcIds.add(item.getId());
	            });
	            List<IntfcAttribute> intfcAttributes = intfcAttributeDao
	                    .findAllByIntfcIdIn(intfcIds);
	            MonitorUtil.fillIntfcAttr2Intfc(intfcAttributes, intfcBos);
	        }
	        // 3.查询设备接口
	        QueryEqpTmplIntfcParam queryEqpTmplIntfcParam = new QueryEqpTmplIntfcParam();
	        queryEqpTmplIntfcParam.setEqpTmplIds(eqpTmplIds);
	        List<EqpTmplIntfcBo> eqpTmplIntfcBos = monitorMapper.findEqpTmplIntfcByEqpTmplIds(queryEqpTmplIntfcParam);
	        MonitorUtil.fillEqpTmplIntfc2EquipmentBo(eqpTmplIntfcBos, equipmentBos);
	        MonitorUtil.fillIntfcBo2EqpTmplIntfcBo(intfcBos, eqpTmplIntfcBos);
	        equipmentBos.forEach(equipmentBo -> {
	            if (CollectionUtils.isNotEmpty(eqpTmplIntfcBos)) {
	                for (EqpTmplIntfcBo eqpTmplIntfcBo : eqpTmplIntfcBos) {
	                    if (!listenCommIds.contains(eqpTmplIntfcBo.getCommId())) {
	                        continue;
	                    }
	                    if (CollectionUtils.isEmpty(eqpTmplIntfcBo.getIntfcAttrs())) {
	                        continue;
	                    }
	                    for (IntfcAttribute intfcAttribute : eqpTmplIntfcBo.getIntfcAttrs()) {
	                        Map<String, Object> paramMap = equipmentBo.getParamMap();
	                        for (Map.Entry<String, Object> entry : paramMap.entrySet()) {
			                    String key =entry.getKey();
			                    Object obj =entry.getValue();
			                    if((key.startsWith(IotConstant.EquipmentConstant.KEY_PREFIX_CPBL+IotConstant.EquipmentConstant.KEY_PREFIX_INTFC))&&(key.endsWith(""+intfcAttribute.getIntfcId()+IotConstant.EquipmentConstant.UNDERLINE+intfcAttribute.getId()))&&obj != null){
			                    	ListenPortBo tcpPortBo = new ListenPortBo();
			                    	tcpPortBo.setEqpTmplId(eqpTmplIntfcBo.getEqpTmplId());
	                                tcpPortBo.setIntfcAttrId(intfcAttribute.getId());
	                                tcpPortBo.setIntfcAttrName(intfcAttribute.getName());
	                                tcpPortBo.setIntfcId(intfcAttribute.getIntfcId());
	                                tcpPortBo.setImpType(eqpTmplIntfcBo.getCommName());
	                                int port = Integer.valueOf(obj.toString());
	                                tcpPortBo.setPort(port);
	                                tcpPortBos.add(tcpPortBo);
			                    }
	                        }
	                    }
	              }
			} else {
				logger.debug("eqpTmplIntfcBos is empty!");
			}
	        });

	        return tcpPortBos;
	    
	}
	
	@Override
	public List<EquipmentBo> findMonitorEquipmentDet(QueryMonitorEqpParam queryParam) {
		// 1.查询设备数据
        queryParam.setMonitorType(MonitorType.LISTENING.getValue());
        List<EquipmentBo> equipmentBos = monitorMapper.findMonitorEquipmentDet(queryParam);;
        if (CollectionUtils.isEmpty(equipmentBos)) {
            return equipmentBos;
        }
        //2.查询设备模板属性
        List<Long> eqpTmplIds = new ArrayList<>();
        equipmentBos.forEach(eqpb -> {
            if (!eqpTmplIds.contains(eqpb.getEqpTmplId())) {
                eqpTmplIds.add(eqpb.getEqpTmplId());
            }
        });
        QueryEqpTmplAttrParam queryEqpTmplAttrParam = new QueryEqpTmplAttrParam();
        queryEqpTmplAttrParam.setEqpTmplIds(eqpTmplIds);
        List<EqpTmplAttributeBo> eqpTmplAttributes = monitorMapper.findEqpTmplAttrByEqpTmplIds(queryEqpTmplAttrParam);
        if (CollectionUtils.isNotEmpty(eqpTmplAttributes)) {
            MonitorUtil.fillEqpTmplAttribute2EquipmentBo(eqpTmplAttributes,equipmentBos);
        } else {
            logger.debug("eqpTmplAttributes is empty!");
        }
        return equipmentBos;
	}

}
