package com.yeejoin.amos.iot.business.service.impl;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.UUID;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.yeejoin.amos.iot.business.dao.repository.INodeVoDao;
import com.yeejoin.amos.iot.business.entity.DroolsRule;
import com.yeejoin.amos.iot.business.entity.LinkVo;
import com.yeejoin.amos.iot.business.entity.NodeVo;
import com.yeejoin.amos.iot.business.entity.Rule;
import com.yeejoin.amos.iot.business.entity.RuleDataSource;
import com.yeejoin.amos.iot.business.entity.RuleEmail;
import com.yeejoin.amos.iot.business.entity.RuleGroup;
import com.yeejoin.amos.iot.business.entity.RuleMqtt;
import com.yeejoin.amos.iot.business.entity.RuleTcp;
import com.yeejoin.amos.iot.business.enums.RuleNodeType;
import com.yeejoin.amos.iot.business.param.NodeVoParam;
import com.yeejoin.amos.iot.business.param.RuleConfigParam;
import com.yeejoin.amos.iot.business.param.RuleDataParam;
import com.yeejoin.amos.iot.business.service.intfc.IDroolsRuleService;
import com.yeejoin.amos.iot.business.service.intfc.ILinkVoService;
import com.yeejoin.amos.iot.business.service.intfc.INodeVoService;
import com.yeejoin.amos.iot.business.service.intfc.IRuleDataSourceService;
import com.yeejoin.amos.iot.business.service.intfc.IRuleEmailService;
import com.yeejoin.amos.iot.business.service.intfc.IRuleGroupService;
import com.yeejoin.amos.iot.business.service.intfc.IRuleMqttService;
import com.yeejoin.amos.iot.business.service.intfc.IRuleService;
import com.yeejoin.amos.iot.business.service.intfc.IRuleTcpService;
import com.yeejoin.amos.iot.rule.engine.service.IRuleEngineService;
import com.yeejoin.amos.iot.rule.engine.service.IRuleInterpreterService;
import com.yeejoin.amos.op.core.entity.Resource;
import com.yeejoin.amos.op.core.entity.handler.EntityChangeHandler;
import com.yeejoin.amos.op.core.jpa.GenericManagerImpl;
import com.yeejoin.amos.op.core.jpa.IBaseRepository;

import net.sf.json.JSONObject;

/**
 * 
 * <pre>
 * 视图节点元素service实现
 * </pre>
 *
 * @author as-chenjiajun
 * @version $Id: NodeVoServiceImpl.java, v 0.1 2018年4月10日 下午3:56:39 as-chenjiajun Exp $
 */
@Service("nodeVoService")
public class NodeVoServiceImpl 
implements INodeVoService
{

	@Autowired
	private IRuleService IRuleService;
	
	@Autowired
	private ILinkVoService iLinkVoService;
	
	@Autowired
	private IRuleDataSourceService iRuleDataSourceService;
	
	@Autowired
	private IRuleEmailService iRuleEmailService;
	
	@Autowired
	private IRuleInterpreterService ruleInterpreterService;
	
	@Autowired
	private IRuleGroupService ruleGroupService;
	
	@Autowired
	private IRuleEngineService ruleEngineService;
	
	@Autowired
	private IDroolsRuleService droolsRuleService;
	
	@Autowired
	private IRuleTcpService iRuleTcpService;
	
	@Autowired
	private IRuleMqttService iRuleMqttService;
	
	@Autowired
	private INodeVoDao iNodeVoDao;
	
	@Transactional
	@Override
	public void addEditRuleNodeAndLinks(RuleDataParam ruleData)
	{
		//先删除老数据
		List<LinkVo> oldLinkVos = iLinkVoService.findByRuleId(ruleData.getId());
		for (LinkVo linkVo : oldLinkVos)
		{
			iLinkVoService.delLinkVoById(linkVo.getId());
		}
		List<LinkVo> newLinkVos = new ArrayList<LinkVo>();
		//保存新数据 
		ruleData.getLinkVoList().forEach(link -> {
			link.setRuleId(ruleData.getId());
			link.setId(UUID.randomUUID().toString());
			iLinkVoService.save(link);
			newLinkVos.add(link);
		});
		List<NodeVo> newNodeVos = new ArrayList<NodeVo>();
		ruleData.getNodeVoList().forEach(node -> {
			NodeVo tempNode = iNodeVoDao.getOne(node.getId());
			tempNode = tempNode == null ? new NodeVo() : tempNode;
			tempNode.setId(node.getId());
			tempNode.setX(node.getConfig().getStyle().getLeft());
			tempNode.setY(node.getConfig().getStyle().getTop());
			iNodeVoDao.save(tempNode);
			newNodeVos.add(tempNode);
		});
		//进行规则解析及保存
		List<DroolsRule> oldDroolsRules = droolsRuleService.findByRuleId(ruleData.getId());
		for (DroolsRule droolsRule : oldDroolsRules)
		{
			droolsRuleService.delDroolsRuleById(droolsRule.getId());
		}
		Rule rule = IRuleService.findOne(ruleData.getId());
		List<DroolsRule> newDroolsRules = ruleInterpreterService.parseRule(newNodeVos, newLinkVos, rule);
		if(newDroolsRules.size() > 0) {
			droolsRuleService.saveAll(newDroolsRules);
		}
		if (rule.getIsUser())//如果规则为启用状态，则更新规则引擎
		{
			RuleGroup ruleGroup = ruleGroupService.findOne("1");
			List<DroolsRule> droolsRules = droolsRuleService.findByGroupIdAndIsUser(ruleGroup.getId(), true);
			ruleEngineService.updateRule(ruleGroup, droolsRules);
		}
	}
	
	@Override
	public List<NodeVo> findByRuleId(String id)
	{
		List<NodeVo> nodeVos =  iNodeVoDao.findByRuleId(id);
		return nodeVos == null ? Collections.emptyList() : nodeVos;
	}
	
	@Override
	public List<NodeVo> findByKey(String id)
	{
		List<NodeVo> nodeVos =  iNodeVoDao.findByViewKey(id);
		return nodeVos == null ? Collections.emptyList() : nodeVos;
	}

	@Transactional
	@Override
	public void deleteNode(String id)
	{
		NodeVo nodeVo = iNodeVoDao.getOne(id);
		//删除节点
		EntityChangeHandler.fireResourceReomoved(nodeVo);
		iNodeVoDao.delNodeVoById(id);
	}

	@Override
	public void editNodeName(NodeVo nodeVo)
	{
		String name = nodeVo.getDisplayName();
		nodeVo = findOne(nodeVo.getId());
		nodeVo.setDisplayName(name);
		String key = nodeVo.getViewKey();
		if (key != null)
		{
			if (RuleNodeType.DATANODE_CPBL.equals(nodeVo.getNodeType())
					|| RuleNodeType.DATANODE_DEVICE.equals(nodeVo.getNodeType())
					|| RuleNodeType.DATANODE_TASK.equals(nodeVo.getNodeType())
					|| RuleNodeType.DATANODE_VIEW.equals(nodeVo.getNodeType()))
			{
				RuleDataSource ruleDataSource = iRuleDataSourceService.findOne(key);
				ruleDataSource.setName(name);
				ruleDataSource.setDisplayName(name);
				ruleDataSource = iRuleDataSourceService.save(ruleDataSource);
			} else if (RuleNodeType.OUTPUT_EMAIL.equals(nodeVo.getNodeType()))
			{
				RuleEmail ruleEmail = iRuleEmailService.findOne(key);
				ruleEmail.setName(name);
				ruleEmail.setDisplayName(name);
				ruleEmail = iRuleEmailService.save(ruleEmail);
			} else if (RuleNodeType.OUTPUT_TCP.equals(nodeVo.getNodeType()))
			{
				RuleTcp ruleTcp = iRuleTcpService.findOne(key);
				ruleTcp.setName(name);
				ruleTcp.setDisplayName(name);
				ruleTcp = iRuleTcpService.save(ruleTcp);
				
			} else if (RuleNodeType.OUTPUT_MQTT.equals(nodeVo.getNodeType()))
			{
				RuleMqtt ruleMqtt = iRuleMqttService.findOne(key);
				ruleMqtt.setName(name);
				ruleMqtt.setDisplayName(name);
				ruleMqtt = iRuleMqttService.save(ruleMqtt);
			}
		}
		iNodeVoDao.save(nodeVo);
	}

	@Override
	public void addNode(NodeVoParam node)
	{
		NodeVo tempNode = new NodeVo();
		tempNode.setId(node.getId());
		tempNode.setRuleId(node.getRuleId());
		tempNode.setDisplayName(node.getConfig().getLabel());
		tempNode.setX(node.getConfig().getStyle().getLeft());
		tempNode.setY(node.getConfig().getStyle().getTop());
		tempNode.setRemark(node.getConfig().getRuleNodeId());
		tempNode.setType(node.getConfig().getType());
		tempNode.setNodeType(node.getConfig().getNodeType());
		iNodeVoDao.save(tempNode);
	}

	@Transactional
	@Override
	public void editNode(RuleConfigParam param)
	{
		NodeVo nodeVo = findOne(param.getNodeId());
		String key = nodeVo.getViewKey();
		if (key == null)
		{
			if (RuleNodeType.DATANODE_CPBL.equals(param.getNodeType())
					|| RuleNodeType.DATANODE_DEVICE.equals(param.getNodeType())
					|| RuleNodeType.DATANODE_TASK.equals(param.getNodeType())
					|| RuleNodeType.DATANODE_VIEW.equals(param.getNodeType()))
			{
				RuleDataSource ruleDataSource = (RuleDataSource)JSONObject.toBean(JSONObject.fromObject(param.getConfigInfo()),RuleDataSource.class);
				ruleDataSource.setName(param.getLabel());
				ruleDataSource.setDisplayName(param.getLabel());
				ruleDataSource = iRuleDataSourceService.save(ruleDataSource);
				nodeVo.setViewKey(ruleDataSource.getId());
			} else if (RuleNodeType.OUTPUT_EMAIL.equals(param.getNodeType()))
			{
				RuleEmail ruleEmail = (RuleEmail)JSONObject.toBean(JSONObject.fromObject(param.getConfigInfo()),RuleEmail.class);
				ruleEmail.setName(param.getLabel());
				ruleEmail.setDisplayName(param.getLabel());
				ruleEmail = iRuleEmailService.save(ruleEmail);
				nodeVo.setViewKey(ruleEmail.getId());
			} else if (RuleNodeType.OUTPUT_TCP.equals(param.getNodeType()))
			{
				RuleTcp ruleTcp = (RuleTcp)JSONObject.toBean(JSONObject.fromObject(param.getConfigInfo()),RuleTcp.class);
				ruleTcp.setName(param.getLabel());
				ruleTcp.setDisplayName(param.getLabel());
				ruleTcp = iRuleTcpService.save(ruleTcp);
				nodeVo.setViewKey(ruleTcp.getId());
			} else if (RuleNodeType.OUTPUT_MQTT.equals(param.getNodeType()))
			{
				RuleMqtt ruleMqtt = (RuleMqtt)JSONObject.toBean(JSONObject.fromObject(param.getConfigInfo()),RuleMqtt.class);
				ruleMqtt.setName(param.getLabel());
				ruleMqtt.setDisplayName(param.getLabel());
				ruleMqtt = iRuleMqttService.save(ruleMqtt);
				nodeVo.setViewKey(ruleMqtt.getId());
			}
		} else {
			if (RuleNodeType.DATANODE_CPBL.equals(param.getNodeType())
					|| RuleNodeType.DATANODE_DEVICE.equals(param.getNodeType())
					|| RuleNodeType.DATANODE_TASK.equals(param.getNodeType())
					|| RuleNodeType.DATANODE_VIEW.equals(param.getNodeType()))
			{
				RuleDataSource ruleDataSource = iRuleDataSourceService.findOne(key);
				RuleDataSource newRuleDataSource = (RuleDataSource)JSONObject.toBean(JSONObject.fromObject(param.getConfigInfo()),RuleDataSource.class);
				ruleDataSource.setName(param.getLabel());
				ruleDataSource.setDisplayName(param.getLabel());
				ruleDataSource.setUser_type(newRuleDataSource.getUser_type());
				if (newRuleDataSource.getView() != null)
				{
					ruleDataSource.setView(newRuleDataSource.getView());
				}
				if (newRuleDataSource.getEqpId() != null)
				{
					ruleDataSource.setEqpId(newRuleDataSource.getEqpId());
				}
				if (newRuleDataSource.getMonitorRuleId() != null)
				{
					ruleDataSource.setMonitorRuleId(newRuleDataSource.getMonitorRuleId());
				}
				ruleDataSource = iRuleDataSourceService.save(ruleDataSource);
			} else if (RuleNodeType.OUTPUT_EMAIL.equals(param.getNodeType()))
			{
				RuleEmail ruleEmail = iRuleEmailService.findOne(key);
				RuleEmail newRuleEmail = (RuleEmail)JSONObject.toBean(JSONObject.fromObject(param.getConfigInfo()),RuleEmail.class);
				ruleEmail.setName(param.getLabel());
				ruleEmail.setDisplayName(param.getLabel());
				ruleEmail.setToEmail(newRuleEmail.getToEmail());
				ruleEmail.setServer(newRuleEmail.getServer());
				ruleEmail.setPort(newRuleEmail.getPort());
				ruleEmail.setFromEmail(newRuleEmail.getFromEmail());
				ruleEmail.setFromPassword(newRuleEmail.getFromPassword());
				ruleEmail.setIsSSL(newRuleEmail.getIsSSL());
				ruleEmail = iRuleEmailService.save(ruleEmail);
			} else if (RuleNodeType.OUTPUT_TCP.equals(param.getNodeType()))
			{
				RuleTcp ruleTcp = iRuleTcpService.findOne(key);
				RuleTcp newRuleTcp = (RuleTcp)JSONObject.toBean(JSONObject.fromObject(param.getConfigInfo()),RuleTcp.class);
				ruleTcp.setName(param.getLabel());
				ruleTcp.setDisplayName(param.getLabel());
				ruleTcp.setPort(newRuleTcp.getPort());
				ruleTcp.setServerIp(newRuleTcp.getServerIp());
				ruleTcp = iRuleTcpService.save(ruleTcp);
				
			} else if (RuleNodeType.OUTPUT_MQTT.equals(param.getNodeType()))
			{
				RuleMqtt ruleMqtt = iRuleMqttService.findOne(key);
				RuleMqtt newRuleMqtt = (RuleMqtt)JSONObject.toBean(JSONObject.fromObject(param.getConfigInfo()),RuleMqtt.class);
				ruleMqtt.setName(param.getLabel());
				ruleMqtt.setDisplayName(param.getLabel());
				ruleMqtt.setPort(newRuleMqtt.getPort());
				ruleMqtt.setServerIp(newRuleMqtt.getServerIp());
				ruleMqtt.setTheme(newRuleMqtt.getTheme());
				ruleMqtt.setUserName(newRuleMqtt.getUserName());
				ruleMqtt.setPassword(newRuleMqtt.getPassword());
				ruleMqtt = iRuleMqttService.save(ruleMqtt);
			}
		}
		nodeVo.setDisplayName(param.getLabel());
		iNodeVoDao.save(nodeVo);
	}

	@Override
	public RuleConfigParam fetchNode(String id)
	{
		NodeVo nodeVo = findOne(id);
		RuleConfigParam ruleConfigParam = new RuleConfigParam();
		String key = nodeVo.getViewKey();
		if (key != null)
		{
			if (RuleNodeType.DATANODE_CPBL.equals(nodeVo.getNodeType())
				|| RuleNodeType.DATANODE_DEVICE.equals(nodeVo.getNodeType())
				|| RuleNodeType.DATANODE_TASK.equals(nodeVo.getNodeType())
				|| RuleNodeType.DATANODE_VIEW.equals(nodeVo.getNodeType()))
			{
				ruleConfigParam.setConfigInfo(iRuleDataSourceService.findOne(key));
			} else if (RuleNodeType.OUTPUT_EMAIL.equals(nodeVo.getNodeType())) {
				ruleConfigParam.setConfigInfo(iRuleEmailService.findOne(key));
			} else if (RuleNodeType.OUTPUT_TCP.equals(nodeVo.getNodeType())) {
				ruleConfigParam.setConfigInfo(iRuleTcpService.findOne(key));
			} else if (RuleNodeType.OUTPUT_MQTT.equals(nodeVo.getNodeType())) {
				ruleConfigParam.setConfigInfo(iRuleMqttService.findOne(key));
			}
		}
		ruleConfigParam.setLabel(nodeVo.getDisplayName());
		ruleConfigParam.setNodeId(nodeVo.getId());
		ruleConfigParam.setType(nodeVo.getType());
		ruleConfigParam.setNodeType(nodeVo.getNodeType());
		return ruleConfigParam;
	}
	
//	@Override
	public void fireResourceReomoved(Resource resource)
	{
		if (resource instanceof RuleDataSource)
		{
			RuleDataSource ruleDataSource = (RuleDataSource)resource;
			List<NodeVo> nodeVos = findByKey(ruleDataSource.getId());
			for (NodeVo nodeVo : nodeVos)
			{
				nodeVo.setViewKey(null);
				iNodeVoDao.save(nodeVo);
			}
		} else if (resource instanceof Rule)
		{
			Rule rule = (Rule)resource;
			findByRuleId(rule.getId()).forEach(nodeVo -> {
				deleteNode(nodeVo.getId());
			});
		}
	}

	@Override
	public NodeVo findOne(String id) {
		return iNodeVoDao.getOne(id);
	}
	
}
