package com.yeejoin.amos.iot.business.service.impl;

import java.util.List;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.yeejoin.amos.iot.business.dao.repository.IOrgImgRelationshipsDao;
import com.yeejoin.amos.iot.business.entity.OrgImgRelationships;
import com.yeejoin.amos.iot.business.param.ImgData;
import com.yeejoin.amos.iot.business.param.OrgImgParam;
import com.yeejoin.amos.iot.business.service.intfc.IOrgImgRelationshipsService;
import com.yeejoin.amos.iot.core.util.ImgFileServer;
import com.yeejoin.amos.iot.core.util.query.QuerySpecificationHelper;
import com.yeejoin.amos.op.core.jpa.GenericManagerImpl;
import com.yeejoin.amos.op.core.jpa.IBaseRepository;

/**
 * 
 * <pre>
 * Service
 * </pre>
 *
 * @author as-chenjiajun
 * @version $Id: OrgImgRelationshipsServiceImpl.java, v 0.1 2018年5月7日 下午1:58:15 as-chenjiajun Exp $
 */
@Service("OrgImgRelationshipsService")
public class OrgImgRelationshipsServiceImpl 
implements IOrgImgRelationshipsService
{
	@Autowired
	private ImgFileServer imgFileServer;
	
	@Autowired
	private IOrgImgRelationshipsService iOrgImgRelationshipsService;
	
	@Autowired
	private IOrgImgRelationshipsDao imgRelationshipsDao;


	@Transactional
	@Override
	public void editOrgImg(OrgImgParam param)
	{
		//删除旧数据
		List<OrgImgRelationships> relationships = imgRelationshipsDao.findAll(QuerySpecificationHelper.getOrgImgs(param));
		relationships.forEach(d->{
			String fileName = d.getImgUid()+ "." + d.getName().split("\\.")[1];
			imgFileServer.deleteFile(fileName);
			imgRelationshipsDao.delOrgImgRelationshipsById(d.getId());
		});
		//保存新数据
		List<ImgData> imgDatas = param.getImgFiles();
		if (imgDatas != null)
		{		
			imgDatas.forEach(i -> {
				imgFileServer.saveBackgroundFile(i);
				OrgImgRelationships orgImgRelationships = new OrgImgRelationships();
				orgImgRelationships.setOrgCode(param.getOrgCode());
				orgImgRelationships.setImgUid(i.getUid());
				orgImgRelationships.setName(i.getName());
				iOrgImgRelationshipsService.save(orgImgRelationships);
			});
		}
	}

	@Override
	public OrgImgParam fetchImgsEntity(String id)
	{
		OrgImgParam param = new OrgImgParam();
		param.setOrgCode(id);;
		List<OrgImgRelationships> relationships = imgRelationshipsDao.findAll(QuerySpecificationHelper.getOrgImgs(param));
		return new OrgImgParam(id,relationships.stream().map(s -> {
			ImgData imgData = new ImgData();
			imgData.setName(s.getName());
			imgData.setUid(s.getImgUid());
			imgData.setThumbUrl(imgFileServer.getBackgroundFile(s.getImgUid()+"."+s.getName().split("\\.")[1]));
			return imgData;
		}).collect(Collectors.toList()));
	}
	
	@Override
	public OrgImgParam fetchSimpleImgsEntity(String id)
	{
		OrgImgParam param = new OrgImgParam();
		param.setOrgCode(id);;
		List<OrgImgRelationships> relationships = imgRelationshipsDao.findAll(QuerySpecificationHelper.getOrgImgs(param));
		return new OrgImgParam(id,relationships.stream().map(s -> {
			ImgData imgData = new ImgData();
			imgData.setName(s.getName());
			imgData.setUid(s.getImgUid()+"."+s.getName().split("\\.")[1]);
			return imgData;
		}).collect(Collectors.toList()));
	}

	@Override
	public void save(OrgImgRelationships orgImgRelationships) {
		imgRelationshipsDao.save(orgImgRelationships);
	}

}
