package com.yeejoin.amos.iot.business.service.impl;

import com.yeejoin.amos.iot.business.controller.BaseController;
import com.yeejoin.amos.iot.business.dao.mapper.ProtocolMapper;
import com.yeejoin.amos.iot.business.dao.repository.IProtocolDao;
import com.yeejoin.amos.iot.business.entity.Protocol;
import com.yeejoin.amos.iot.business.exceptions.IntfcException;
import com.yeejoin.amos.iot.business.exceptions.ProtocolException;
import com.yeejoin.amos.iot.business.remote.RemoteSecurityService;
import com.yeejoin.amos.iot.business.service.intfc.IProtocolService;
import com.yeejoin.amos.iot.business.vo.ProtocolDicVo;
import com.yeejoin.amos.iot.business.vo.ProtocolVo;
import com.yeejoin.amos.iot.core.enums.QueryOperatorEnum;
import com.yeejoin.amos.iot.core.util.DateUtil;
import com.yeejoin.amos.iot.core.util.StringUtil;
import com.yeejoin.amos.iot.core.util.query.BaseQuerySpecification;
import com.yeejoin.amos.op.core.common.query.DaoCriteria;
import com.yeejoin.amos.op.core.jpa.GenericManagerImpl;
import com.yeejoin.amos.op.core.jpa.IBaseRepository;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <pre>
 * 协议service实现类
 * </pre>
 *
 * @author as-lixiangyang
 * @version $Id: ProtocolServiceImpl.java, v 0.1 2018年1月29日 下午1:32:32
 * as-lixiangyang Exp $
 */
@Service("protocolService")
public class ProtocolServiceImpl 
implements IProtocolService {

    
    @Autowired
    RemoteSecurityService userController;
    
    @Autowired
    private IProtocolDao protocolDao;

    @Autowired
    private ProtocolMapper protocolMapper;

    @Override
    public void saveProtocol(Protocol protocol) throws ProtocolException {
        if (protocol == null) {
            throw new ProtocolException("协议对象无效");
        }

        if (protocol.getId() == null) {// 新增协议
            if (!StringUtil.isNotEmpty(protocol.getDicIdScriptLang()) || protocol.getDicIdScriptLang() <= 0) {
                throw new ProtocolException("脚本语言类型异常");
            }
            if (protocol.getUserId() == null) {
                protocol.setUserId(userController.getUserId());
            }
            protocol.setCreateTime(DateUtil.getCurrentDate());
            protocolDao.saveAndFlush(protocol);
        } else {
            Protocol exist = protocolDao.getOne(protocol.getId());

            if (!StringUtil.isNotEmpty(protocol.getDicIdScriptLang())) {// 编辑基础信息
                protocol.setDicIdScriptLang(exist.getDicIdScriptLang());
                protocol.setScriptCode(exist.getScriptCode());
                protocol.setUpdateTime(new Date());
                protocol.setCreateTime(exist.getCreateTime());

                protocolDao.saveAndFlush(protocol);
            } else {// 编辑脚本信息
                exist.setDicIdScriptLang(protocol.getDicIdScriptLang());
                exist.setScriptCode(protocol.getScriptCode());
                exist.setUpdateTime(new Date());
                protocolDao.saveAndFlush(exist);
                protocol = exist;
            }
        }
    }

    @Override
    public List<Protocol> findAll(List<DaoCriteria> criterias) {
        if (criterias.size() != 0) {
            for (DaoCriteria daoCriteria : criterias) {
                if ("visibility".equals(daoCriteria.getPropertyName()) && "1".equals(daoCriteria.getValue())) {
                    criterias.add(new DaoCriteria("userId", QueryOperatorEnum.EQUAL.getName(),
                    		userController.getUserId()));
                    break;
                }
            }
        }
        BaseQuerySpecification<Protocol> spec = new BaseQuerySpecification<>(criterias);
        return protocolDao.findAll(spec);
    }

    @Override
    public Protocol queryById(Long id) {
        return protocolDao.getOne(id);
    }

    /**
     * 根据协议名称查询
     */
    @Override
    public ProtocolVo findById(Long id) {
        Protocol protocol = protocolDao.getOne(id);
        return ProtocolHelper.buildProtocolVo(protocol);
    }

    @Override
    public void delProtocol(Long id) throws IntfcException {
        protocolDao.delProtocolById(id);
    }

    @Override
    public Map<String, Long> countProtocolInfo(List<DaoCriteria> daoCriteria) {
        List<DaoCriteria> pubCriterias = new ArrayList<>();
        List<DaoCriteria> priCriterias = new ArrayList<>();
        for (DaoCriteria criteria : daoCriteria) {
            priCriterias.add(criteria);
        }
        pubCriterias.add(new DaoCriteria("visibility",
                QueryOperatorEnum.EQUAL.getName(), 0));
        priCriterias.add(new DaoCriteria("visibility",
                QueryOperatorEnum.EQUAL.getName(), 1));
        BaseQuerySpecification<Protocol> pubSpec = new BaseQuerySpecification<>(
                pubCriterias);
        BaseQuerySpecification<Protocol> priSpec = new BaseQuerySpecification<>(
                priCriterias);
        Long pubCount = protocolDao.count(pubSpec);
        Long priCount = protocolDao.count(priSpec);
        Map<String, Long> map = new HashMap<>();
        map.put("count", pubCount + priCount);
        map.put("pubCount", pubCount);
        map.put("priCount", priCount);
        return map;
    }

    @Override
    public ProtocolDicVo findProtocolDicVoByFrameType(int frameType) {
        return protocolMapper.findProtocolDicVoByFrameType(frameType);
    }

}
