package com.yeejoin.amos.iot.business.service.impl;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.yeejoin.amos.iot.business.dao.repository.IRuleDataSourceDao;
import com.yeejoin.amos.iot.business.entity.Node;
import com.yeejoin.amos.iot.business.entity.NodeVo;
import com.yeejoin.amos.iot.business.entity.RuleDataSource;
import com.yeejoin.amos.iot.business.enums.RuleNodeType;
import com.yeejoin.amos.iot.business.service.intfc.IRuleDataSourceService;
import com.yeejoin.amos.op.core.entity.Resource;
import com.yeejoin.amos.op.core.entity.handler.EntityChangeHandler;
import com.yeejoin.amos.op.core.jpa.GenericManagerImpl;

/**
 * 
 * <pre>
 * 规则视图中数据源service实现
 * </pre>
 *
 * @author as-chenjiajun
 * @version $Id: RuleDataSourceServiceImpl.java, v 0.1 2018年4月10日 下午3:58:45 as-chenjiajun Exp $
 */
@Service("ruleDataSourceService")
public class RuleDataSourceServiceImpl 
implements IRuleDataSourceService
{
	
	@Autowired
	IRuleDataSourceDao ruleDataSourceDao;
	

	@Override
	public List<RuleDataSource> findAllByEqpId(Long eqpId)
	{
		return ruleDataSourceDao.findAllByEqpId(eqpId);
	}

	@Override
	public void delRuleDataSourceById(String id)
	{
		EntityChangeHandler.fireResourceReomoved(findOne(id));
		ruleDataSourceDao.delRuleDataSourceById(id);
	}

//	@Override
	public void fireResourceReomoved(Resource resource)
	{
		if (resource instanceof Node)
		{
			Node node = (Node)resource;
			Long equipmentId = node.getEquipmentId();
			if (equipmentId != null) {
				//删除规则中的设备
				List<RuleDataSource> ruleDataSourceList = findAllByEqpId(equipmentId);
				for (RuleDataSource ruleDataSource : ruleDataSourceList)
				{
					delRuleDataSourceById(ruleDataSource.getId());
				}
			}
		} else if (resource instanceof NodeVo)
		{
			NodeVo nodeVo = (NodeVo)resource;
			if (RuleNodeType.DATANODE_DEVICE.equals(nodeVo.getType())
					|| RuleNodeType.DATANODE_TASK.equals(nodeVo.getType())
					|| RuleNodeType.DATANODE_VIEW.equals(nodeVo.getType()))
			{
								
				String key = nodeVo.getViewKey();
				if (key != null)
				{
					delRuleDataSourceById(key);
				}
			}
		}
	}

	@Override
	public RuleDataSource findOne(String id) {
		return ruleDataSourceDao.getOne(id);
	}

	@Override
	public RuleDataSource save(RuleDataSource ruleDataSource) {
		return ruleDataSourceDao.save(ruleDataSource);
	}
}
