package com.yeejoin.amos.iot.business.service.impl;

import java.util.*;
import java.util.Map.Entry;
import java.util.stream.Collectors;

import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSON;
import com.yeejoin.amos.iot.business.dao.mapper.DeviceMapper;
import com.yeejoin.amos.iot.business.dao.mapper.MetricDataMapper;
import com.yeejoin.amos.iot.business.dao.repository.IAlarmDao;
import com.yeejoin.amos.iot.business.dao.repository.ICapabilityDao;
import com.yeejoin.amos.iot.business.dao.repository.IEquipmentDao;
import com.yeejoin.amos.iot.business.dao.repository.IEquipmentTemplateDao;
import com.yeejoin.amos.iot.business.dao.repository.IIntfcAttributeDao;
import com.yeejoin.amos.iot.business.dao.repository.INodeDao;
import com.yeejoin.amos.iot.business.dao.repository.ISubgraohDao;
import com.yeejoin.amos.iot.business.entity.Alarm;
import com.yeejoin.amos.iot.business.entity.Capability;
import com.yeejoin.amos.iot.business.entity.Connection;
import com.yeejoin.amos.iot.business.entity.EqpTmplAttribute;
import com.yeejoin.amos.iot.business.entity.Equipment;
import com.yeejoin.amos.iot.business.entity.EquipmentTemplate;
import com.yeejoin.amos.iot.business.entity.IntfcAttribute;
import com.yeejoin.amos.iot.business.entity.MetricData;
import com.yeejoin.amos.iot.business.entity.Node;
import com.yeejoin.amos.iot.business.entity.Subgraph;
import com.yeejoin.amos.iot.business.entity.mybatis.CapacityInterfaceDataVo;
import com.yeejoin.amos.iot.business.entity.mybatis.CapacityPropertyDataVo;
import com.yeejoin.amos.iot.business.param.SubgraphParam;
import com.yeejoin.amos.iot.business.param.TopoNodeParam;
import com.yeejoin.amos.iot.business.remote.RemoteSecurityService;
import com.yeejoin.amos.iot.business.service.intfc.ITopoService;
import com.yeejoin.amos.iot.core.common.datastructure.TreeNode;
import com.yeejoin.amos.iot.core.common.request.CommonPageable;
import com.yeejoin.amos.iot.core.enums.QueryOperatorEnum;
import com.yeejoin.amos.iot.core.util.DateUtil;
import com.yeejoin.amos.iot.core.util.ImgFileServer;
import com.yeejoin.amos.iot.core.util.TreeUtil;
import com.yeejoin.amos.iot.core.util.query.BaseQuerySpecification;
import com.yeejoin.amos.iot.core.util.query.QuerySpecificationHelper;
import com.yeejoin.amos.iot.exception.YeeException;
import com.yeejoin.amos.iot.socket.websocket.vo.RealTimeMetricDataVo;
import com.yeejoin.amos.op.core.common.query.DaoCriteria;
import com.yeejoin.amos.op.core.entity.Resource;
import com.yeejoin.amos.op.core.entity.handler.EntityChangeHandler;
import com.yeejoin.amos.op.core.jpa.GenericManagerImpl;
import com.yeejoin.amos.op.core.jpa.IBaseRepository;

import net.sf.json.JSONObject;

/**
 * 
 * <pre>
 * topo service
 * </pre>
 *
 * @author as-chenjiajun
 * @version $Id: TopoServiceImpl.java, v 0.1 2018年3月8日 下午1:29:47 as-chenjiajun
 *          Exp $
 */
@Service("topoService")
public class TopoServiceImpl 
implements ITopoService {
	
	@Autowired
	RemoteSecurityService userController;

	@Autowired
	private ISubgraohDao iSubgraohDao;

	@Autowired
	private IEquipmentDao iEquipmentDao;

	@Autowired
	private MetricDataMapper metricDataMapper;

	@Autowired
	private DeviceMapper deviceMapper;

	@Autowired
	private ICapabilityDao iCapabilityDao;
	
	@Autowired
	private IEquipmentTemplateDao iEquipmentTemplateDao;
	
	@Autowired
	private IIntfcAttributeDao intfcAttributeDao;
	
	@Autowired
	private INodeDao iNodeDao;
	
	@Autowired
	private ImgFileServer imgFileServer;
	
	@Autowired
	private IAlarmDao alarmDao;
	
	@Override
	public Long addEditSubgraph(Subgraph subgraph,String orgCode) {
		// 设置外键数据
		Subgraph persistObj = null;
		if (subgraph.getId() != null) {
			// 重复性验证
			SubgraphParam param = new SubgraphParam();
			param.setName(subgraph.getName());
			param.setId(subgraph.getId());
			param.setOrgCode(orgCode);
			if (checkNameConflict(param)) {
				return -1L;
			}
			// 编辑
			persistObj = iSubgraohDao.getOne(subgraph.getId());
			persistObj.setName(subgraph.getName());
			if (subgraph.getImgName() != null)
			{
				persistObj.setImgName(subgraph.getImgName());
			}
			if (subgraph.getImgOpacity() != null)
			{
				persistObj.setImgOpacity(subgraph.getImgOpacity());
			}
			if (subgraph.getImgLayout() != null)
			{
				persistObj.setImgLayout(subgraph.getImgLayout());
			}
			subgraph = iSubgraohDao.save(persistObj);
		} else {
			// 重复性验证
			SubgraphParam param = new SubgraphParam();
			param.setName(subgraph.getName());
			param.setOrgCode(orgCode);
			if (checkNameConflict(param)) {
				return -1L;
			}
			// 新增，新增时添加默认GIOT节点
			subgraph.setUserId(userController.getUserId());
			subgraph = iSubgraohDao.save(subgraph);
			List<Node> newNodeList = new ArrayList<Node>();
			Node node = new Node();
			node.setSubgraph(subgraph);
			node.setCreateTime(DateUtil.getCurrentDate());
			Node rootGraphNode = iNodeDao.findById(1L).get();
			node.setConfig(rootGraphNode.getConfig());
			node.setNodeId(UUID.randomUUID().toString());
			newNodeList.add(node);
			subgraph.setNodes(newNodeList);
			//添加默认值
			subgraph.setImgName("default_1.png");
			subgraph.setImgOpacity(1F);
			subgraph.setImgLayout(1);
			subgraph = iSubgraohDao.save(subgraph);
		}
		return subgraph == null ? null : subgraph.getId();
	}
	
	/**
	 * 
	 * <pre>
	 * 检查是否名称已存在
	 * </pre>
	 *
	 * @param param
	 * @return
	 */
	private boolean checkNameConflict(SubgraphParam param)
	{
		Long count = iSubgraohDao.count(QuerySpecificationHelper.getDuplicationNameSubgraph(param));
		if (count > 0)
		{			
			return true;
		} else {
			return false;
		}
	}

	@Override
	public Long addEditSubgraphData(SubgraphParam subgraph) {
		Subgraph persistObj = iSubgraohDao.getOne(subgraph.getId());
		// 设置外键数据
		List<Connection> newConnectionList = subgraph.getConnections();
		if (newConnectionList != null) {
			List<Connection> persistConnectionList = persistObj.getConnections();
			persistConnectionList.clear();
			for (Connection connection : newConnectionList) {
				connection.setSubgraph(persistObj);
				connection.setCreateTime(DateUtil.getCurrentDate());
				connection.setId(null);
				persistConnectionList.add(connection);
			}
			persistObj.setConnections(persistConnectionList);
		}
		List<Node> newNodeList = subgraph.getNodes();
		if (newNodeList != null) {
			List<Node> persistNodeList = persistObj.getNodes();
			persistNodeList.clear();
			for (Node node : newNodeList) {
				// node记录equipment的id
				node.setSubgraph(persistObj);
				node.setCreateTime(DateUtil.getCurrentDate());
//				Map<String, Class<?>> classMap = new HashMap<String, Class<?>>();
//				classMap.put("data", EquipmentTemplateVo.class);
				@SuppressWarnings("unchecked")
				Map<String, Object> nodeInfo = (Map<String, Object>) JSON.parse(node.getConfig());
//				Map<String, Object> nodeInfo = (Map<String, Object>) JSONObject
//						.toBean(JSONObject.fromObject(node.getConfig()), Map.class, classMap);
				if (!nodeInfo.get("label").toString().equals("giot")) {
					Map<String, Object> equipmentTemplateVo = (Map<String, Object>) nodeInfo.get("data");
//					Long equipmentId = node.getEquipmentId();
//					Equipment equipment = null;
//					if (equipmentId != null) {
//						equipment = iEquipmentDao.getOne(equipmentId);
//					} else {
//						equipment = new Equipment();
//						equipment.setId(Long.valueOf(equipmentTemplateVo.get("id").toString()));
//						equipment.setCfgParamJson(equipmentTemplateVo.get("cfgParamJson").toString());
//					}
//					equipment.setName(nodeInfo.get("label").toString());
//					equipment.setDescr(nodeInfo.get("label").toString());
//					equipment.setUserId(Long.parseLong(CurrentAuthentication.getAuthenticatedUserId()));
//					equipment.setEqpTmplId(Long.valueOf(equipmentTemplateVo.get("id").toString()));
//					equipment.setCreateTime(DateUtil.getCurrentDate());
//					equipment.setOrgCode(subgraph.getOrgCode());
//					equipment = iEquipmentDao.save(equipment);
					if (equipmentTemplateVo.get("id") != null)
					{
						node.setEquipmentId(Long.valueOf(equipmentTemplateVo.get("id").toString()));
					}
				}
				node.setId(null);
				persistNodeList.add(node);
			}
			persistObj.setNodes(persistNodeList);
		}
		// 更新/保存
		persistObj = iSubgraohDao.save(persistObj);
		return persistObj == null ? null : persistObj.getId();
	}

	@Override
	public void delSubgraph(Long id) {
		Subgraph subgraph = iSubgraohDao.findById(id).get();
//		EntityChangeHandler.fireResourceReomoved(subgraph);
		iSubgraohDao.delSubgraphById(id);
	}

	@Override
	public List<TreeNode> getSubgraphsByCondition(SubgraphParam param) {
		List<Subgraph> subgraphs = getSubgraphByParam(param);
		// 转换成树结构
		List<TreeNode> treeNodes = new ArrayList<TreeNode>();
		for (Subgraph subgraph : subgraphs) {
			TreeNode treeNode = new TreeNode(subgraph.getCurrentKey(), subgraph.getName(), subgraph.getParentKey());
			treeNode.setSourceId(subgraph.getId());
			treeNodes.add(treeNode);
		}
		return TreeUtil.buildByRecursive(treeNodes, "0");
	}

	@Override
	public Subgraph getSubgraphById(Long id) {
		Subgraph subgraph = iSubgraohDao.findById(id).get();
		if (subgraph != null && subgraph.getNodes() != null)
		{			
			for ( Node node : subgraph.getNodes())
			{
				if (node.getEquipmentId() != null)
				{
//					Equipment equipment = iEquipmentDao.findById(node.getEquipmentId()).get();
					Optional<Equipment> optional = iEquipmentDao.findById(node.getEquipmentId());
					if (optional != null && optional.isPresent())
					{
						node.setEquipmentParam(optional.get().getCfgParamJson());
					}
				}
			}
		}
		return subgraph;
	}

	public List<Subgraph> getSubgraphByParam(SubgraphParam param) {
		Specification<Subgraph> subgraphSpec = new Specification<Subgraph>() {
			public Predicate toPredicate(Root<Subgraph> root, CriteriaQuery<?> query, CriteriaBuilder cb) {
				List<Predicate> list = new ArrayList<Predicate>();
				List<Predicate> predicateList = new ArrayList<Predicate>();
//				if (param.getUserId() != null) {
//					Predicate id = cb.equal(root.get("userId").as(Long.class), param.getUserId());
//					list.add(id);
//				}
				if(param.getIsDirector() != null ){
					Predicate orgCode = cb.equal(root.get("orgCode").as(String.class), param.getOrgCode());
					predicateList.add(orgCode);
					
					if(param.getIsDirector()){
						Predicate orgCode2 = cb.like(root.get("orgCode").as(String.class), param.getOrgCode()+"*%");
						predicateList.add(cb.or(orgCode2));
					}
				}	
				
				if (param.getName() != null) {
					Predicate id = cb.equal(root.get("name").as(String.class), param.getName());
					list.add(id);
				}
				if (param.getParentKey() != null) {
					Predicate id = cb.like(root.get("parentKey").as(String.class), param.getParentKey() + "%");
					list.add(id);
				}
				if (param.getCurrentKey() != null) {
					Predicate id = cb.equal(root.get("currentKey").as(String.class), param.getCurrentKey());
					list.add(id);
				}
				list.add(cb.or(predicateList.toArray(new Predicate[predicateList.size()])));
				Predicate[] p = new Predicate[list.size()];
				return cb.and(list.toArray(p));
			}
		};
		List<Subgraph> subgraphs = iSubgraohDao.findAll(subgraphSpec);
		return subgraphs == null ? new ArrayList<>() : subgraphs;
	}

	@Override
	public Long updateDeviceName(TopoNodeParam param) {
		Specification<Node> nodeSpec = new Specification<Node>() {
			public Predicate toPredicate(Root<Node> root, CriteriaQuery<?> query, CriteriaBuilder cb) {
				List<Predicate> list = new ArrayList<Predicate>();
				if (param.getNodeId() != null) {
					Predicate id = cb.equal(root.get("nodeId").as(String.class), param.getNodeId());
					list.add(id);
				}
				Predicate[] p = new Predicate[list.size()];
				return cb.and(list.toArray(p));
			}
		};
		List<Node> nodeList = iNodeDao.findAll(nodeSpec);
		for (Node node : nodeList) {
			Long equipmentId = node.getEquipmentId();
			Equipment equipment = iEquipmentDao.getOne(equipmentId);
			Equipment equipmentParam = new Equipment();
			equipmentParam.setName(param.getName());
			equipmentParam.setId(equipment.getId());
			if (checkNameConflict(equipmentParam))
			{
				return -1L;
			}
			// 更新拓扑信息
			node.setConfig(param.getConfig());
			iNodeDao.save(node);
			// 更新设备表信息
			if (equipment != null) {
				equipment.setName(param.getName());
				iEquipmentDao.save(equipment);
			}
		}
		return 0L;
	}
	
	/**
	 * 
	 * <pre>
	 * 检查是否名称已存在
	 * </pre>
	 *
	 * @param param
	 * @return
	 */
	private boolean checkNameConflict(Equipment param)
	{
		Long count = iEquipmentDao.count(QuerySpecificationHelper.getDuplicationNameEquipment(param));
		if (count > 0)
		{			
			return true;
		} else {
			return false;
		}
	}

	@Override
	public List<Equipment> getSubgraphEqpsByName(String name, String code) {
		// 重复性验证
		SubgraphParam param = new SubgraphParam();
		param.setName(name);
		param.setUserId(userController.getUserId());
		param.setOrgCode(code);
//		param.setIsDirector(director);
		List<Subgraph> subgraphs = getSubgraphByParam(param);
		if (subgraphs == null || subgraphs.size() > 1) {
			throw new YeeException("子图:" + name + "不唯一");
		}
		Subgraph subgraph = subgraphs.get(0);
		List<Equipment> equipments = new ArrayList<Equipment>();
		List<Node> nodes = subgraph.getNodes();
		for (Node node : nodes) {
			if (node.getEquipmentId() == null) {
				continue;
			}
			equipments.add(iEquipmentDao.getOne(node.getEquipmentId()));
		}
		return equipments;
	}

	@SuppressWarnings({ "unchecked" })
	@Override
	public Map<String,Object> getRealTimeData(Long id) {
		Equipment equipment = iEquipmentDao.findById(id).get();
		Map<String, Object> resultMap = new HashMap<>();
		List<RealTimeMetricDataVo> realTimeMetricDataVoList = new ArrayList<>();
		String cfgParamJson = equipment.getCfgParamJson();
		if (cfgParamJson == null || "".equals(cfgParamJson)) {
			return resultMap;
		}
		Map<String, Object> cfgParamMap = (Map<String, Object>) JSON.parse(cfgParamJson);
		List<String> selIds = (List<String>) cfgParamMap.get("selCpblIds");
		Map<String, Object> temp = (Map<String, Object>)cfgParamMap.get("quotaSelected");
		List<Long> cpblIdArr = temp.keySet().stream().map(i -> Long.valueOf(i)).collect(Collectors.toList());;
		Map<Long,List<String>> cpblAttrIdArr = new HashMap<>();
		for (Entry<String, Object> entry : temp.entrySet())
		{
			List<Integer> singleCpblAttrIds = (List<Integer>)entry.getValue();
			List<String> singleCpblAttrIdArr = singleCpblAttrIds.stream().map(i -> String.valueOf(i.intValue())).collect(Collectors.toList());
			cpblAttrIdArr.put(Long.valueOf(entry.getKey()),singleCpblAttrIdArr);
		}
		if (cpblIdArr == null || cpblIdArr.size() == 0) {
			return resultMap;
		}
		for (Long cpblId : cpblIdArr) {
			Capability capability = iCapabilityDao.findById(cpblId).get();
			RealTimeMetricDataVo realTimeMetricDataVo = new RealTimeMetricDataVo();
			realTimeMetricDataVo.setEqpId(id);
			realTimeMetricDataVo.setCpblId(cpblId);
			realTimeMetricDataVo.setCpblName(capability.getName());
//			realTimeMetricDataVo.setTime(new Date());

			Map<String, Object> map = new HashMap<>();
			map.put("eqpId", id);
			map.put("cpblId", cpblId);
			Map<String, String> paramMap = new HashMap<>();
			MetricData metricData = metricDataMapper.getRealTimeData(map);
			List<CapacityPropertyDataVo> capabilityAttributes = deviceMapper.getCapacityPropertyById(cpblId);
			if (metricData != null && metricData.getMetricJson() != null && metricData.getMetricJson() != "") {
				paramMap = DeviceServiceHelper.buildCpblAttrData(metricData.getMetricJson(), capabilityAttributes);
			} else {
				List<String> cpblAttrIds = cpblAttrIdArr.get(cpblId);
				for (String cpblAttrIdString : cpblAttrIds)
				{						
					Long cpblAttrId = Long.valueOf(cpblAttrIdString);
					for (CapacityPropertyDataVo capacityPropertyDataVo : capabilityAttributes) {
						if (cpblAttrId.equals(capacityPropertyDataVo.getId())) {
							if (capacityPropertyDataVo.getType() != 2 && selIds.contains(cpblId.toString()))
							{
								paramMap.put(capacityPropertyDataVo.getDisplayName(), "");
								break;
							}
						}
					}
				}
			}
			if (metricData != null) {
				realTimeMetricDataVo.setTime(metricData.getExeTime());
			}
			realTimeMetricDataVo.setMetricDataMap(paramMap);
			realTimeMetricDataVoList.add(realTimeMetricDataVo);
		}
		resultMap.put("realTimeMetricDataVoList", realTimeMetricDataVoList);
		resultMap.put("isWarning", equipment.getIsWarning());
		//计算告警最高等级
		List<Alarm> alarms = alarmDao.fetchAlarmsByRqpId(equipment.getId());
		if (alarms != null)
		{
			if (alarms.isEmpty())
			{
				//如果没有查询到告警，强制设置isWarning为false
				resultMap.put("isWarning", false);
			} else {				
				Integer maxLevel = alarms.stream().mapToInt(i -> i.getLevel()).summaryStatistics().getMin();
				resultMap.put("maxLevel", maxLevel);
			}
		}
		return resultMap;
	}
	
	/**
	 * 
	 * <pre>
	 * 获取设备指定设备模板的全部属性
	 * </pre>
	 *
	 * @param eqpTmplId
	 * @return
	 */
	private String getEmptyPropertyString(Long eqpTmplId)
	{
		EquipmentTemplate equipmentTemplate = iEquipmentTemplateDao.getOne(eqpTmplId);
		Map<String, String> propertyMap = new HashMap<String, String>();
		//设备属性
		for (EqpTmplAttribute eqpTmplAttribute : equipmentTemplate.getAttributes())
		{
			StringBuilder keyBuilder = new StringBuilder();
			keyBuilder.append("equip_").append(eqpTmplAttribute.getId());
			propertyMap.put(keyBuilder.toString(), eqpTmplAttribute.getValueStr() == null ? "" : eqpTmplAttribute.getValueStr());
		}
		for (Capability capability : equipmentTemplate.getCapabilities())
		{
			//能力属性
			List<CapacityPropertyDataVo> capacityPropertyDataVos = deviceMapper.getCapacityPropertyById(capability.getId());
			capacityPropertyDataVos = capacityPropertyDataVos == null ? new ArrayList<>() : capacityPropertyDataVos; 
			for (CapacityPropertyDataVo capacityPropertyDataVo : capacityPropertyDataVos)
			{
				StringBuilder keyBuilder = new StringBuilder();
				keyBuilder.append("cpbl_").append(capability.getId()).append("_").append(capacityPropertyDataVo.getId());
				propertyMap.put(keyBuilder.toString(), capacityPropertyDataVo.getValueStr() == null ? "" : capacityPropertyDataVo.getValueStr());
			}
			//接口属性
			List<CapacityInterfaceDataVo> capacityInterfaceDataVos = deviceMapper.getCapacityInterfaceByCapId(capability.getId());
			capacityInterfaceDataVos = capacityInterfaceDataVos == null ? new ArrayList<>() : capacityInterfaceDataVos; 
			for (CapacityInterfaceDataVo capacityInterfaceDataVo : capacityInterfaceDataVos)
			{
				List<DaoCriteria> criterias = new ArrayList<>();
				criterias.add(new DaoCriteria("intfcId", QueryOperatorEnum.EQUAL.getName(), capacityInterfaceDataVo.getIntfcId()));
				BaseQuerySpecification<IntfcAttribute> spec = new BaseQuerySpecification<>(criterias);
				List<IntfcAttribute> intfcAttributes = intfcAttributeDao.findAll(spec, new CommonPageable()).getContent();
				intfcAttributes = intfcAttributes == null ? new ArrayList<>() : intfcAttributes;
				for (IntfcAttribute intfcAttribute : intfcAttributes)
				{					
					StringBuilder keyBuilder = new StringBuilder();
					keyBuilder.append("cpbl_intfc_").append(capability.getId()).append("_").append(intfcAttribute.getIntfcId()).append("_").append(intfcAttribute.getId());
					propertyMap.put(keyBuilder.toString(), intfcAttribute.getValueStr() == null ? "" : intfcAttribute.getValueStr());
				}
			}
		}
		JSONObject jsonObject = JSONObject.fromObject(propertyMap);
		return jsonObject.toString();
	}

	@Override
	public List<Subgraph> getSubviewByOrgCode(String orgCode) {
		List<DaoCriteria> criterias = new ArrayList<>();
		criterias.add(new DaoCriteria("orgCode", QueryOperatorEnum.EQUAL.getName(),orgCode));
		criterias.add(new DaoCriteria("orgCode", QueryOperatorEnum.LIKE.getName(),orgCode+"*%"));
		BaseQuerySpecification<Subgraph> spec = new BaseQuerySpecification<>(criterias);
		return iSubgraohDao.findAll(spec);
	}

	@Override
	public void addEditSubgraphImg(Subgraph subgraph)
	{
		if (subgraph.getBinaryImg() != null) {
			imgFileServer.saveSubgraphFile(subgraph.getBinaryImg(), subgraph.getImgName(), "subgraph", "ISO-8859-1");
		}
		String imgName = subgraph.getImgName();
		subgraph = iSubgraohDao.findById(subgraph.getId()).get();
		subgraph.setImgName(imgName);
		iSubgraohDao.save(subgraph);
	}

	@Override
	public List<Equipment> getSubgraphEqpsByID(Long id)
	{
		Subgraph subgraph = iSubgraohDao.findById(id).get();
		List<Equipment> equipments = new ArrayList<Equipment>();
		List<Node> nodes = subgraph.getNodes();
		for (Node node : nodes) {
			if (node.getEquipmentId() == null) {
				continue;
			}
			equipments.add(iEquipmentDao.getOne(node.getEquipmentId()));
		}
		return equipments;
	}

//	@Override
	public void fireResourceReomoved(Resource resource)
	{
		if (resource instanceof Subgraph)
		{
			
		}
	}

	@Override
	public void oneOpenOrClose(Long id, Boolean enableMonitor) {
		iSubgraohDao.oneOpenOrClose(enableMonitor,id);
	}
}
