package com.yeejoin.amos.iot.business.service.intfc;

import com.yeejoin.amos.iot.business.entity.CapabilitylInterface;
import com.yeejoin.amos.iot.business.entity.Equipment;
import com.yeejoin.amos.iot.business.entity.EquipmentTemplate;
import com.yeejoin.amos.iot.business.entity.mybatis.CapacityInterfaceDataVo;
import com.yeejoin.amos.iot.business.entity.mybatis.CapacityPropertyDataVo;
import com.yeejoin.amos.iot.business.entity.mybatis.DeviceInterfaceDataVo;
import com.yeejoin.amos.iot.business.entity.mybatis.EquipmentBo;
import com.yeejoin.amos.iot.business.entity.mybatis.EquipmentTemplateVo;
import com.yeejoin.amos.iot.business.entity.mybatis.EquipmentVo;
import com.yeejoin.amos.iot.business.param.CapacityPropertyParam;
import com.yeejoin.amos.iot.business.param.DevicePageParam;
import com.yeejoin.amos.iot.business.param.EqpTemplPageParam;
import com.yeejoin.amos.iot.business.param.EquipmentCapacityParam;
import com.yeejoin.amos.iot.business.param.EquipmentInterfaceParam;
import com.yeejoin.amos.iot.business.param.EquipmentParam;
import com.yeejoin.amos.iot.business.param.EquipmentPropertyParam;
import com.yeejoin.amos.iot.business.param.QueryEqpCountParam;
import com.yeejoin.amos.iot.business.param.TopoNodeParam;
import com.yeejoin.amos.iot.core.common.response.CommonPage;
import com.yeejoin.amos.op.core.common.query.DaoCriteria;
import com.yeejoin.amos.op.core.jpa.IGenericManager;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;

import java.util.List;
import java.util.Map;

/**
 * <pre>
 * 设备操作接口
 * </pre>
 *
 * @author as-chenjiajun
 * @version $Id: IDeviceService.java, v 0.1 2018年1月29日 下午5:01:43 as-chenjiajun Exp $
 */
public interface IDeviceService 
{
    /**
     * <pre>
     * 保存设备属性
     * </pre>
     *
     * @param param
     * @return
     */
    Long saveEquipment(EquipmentParam param);

    /**
     * <pre>
     * 更新设备基本信息
     * </pre>
     *
     * @param param
     */
    Long updateDeviceBasicInfo(EquipmentParam param);

    /**
     * <pre>
     * 更新设备属性
     * </pre>
     *
     * @param param
     */
    void deviceAttributeUpdate(EquipmentPropertyParam param);

    /**
     * <pre>
     * 设备接口信息更新
     * </pre>
     *
     * @param param
     */
    void deviceInterfaceUpdate(EquipmentInterfaceParam param);

    /**
     * <pre>
     * 分页查询设备列表
     * </pre>
     *
     * @param param
     * @return
     */
    CommonPage<EquipmentTemplateVo> pageDeviceData(DevicePageParam param);

    /**
     * <pre>
     * 设备能力更新
     * </pre>
     *
     * @param param
     */
    void deviceCapacityUpdate(EquipmentCapacityParam param);

    /**
     * <pre>
     * 能力接口更新
     * </pre>
     *
     * @param param
     */
    void capacityIntfUpdate(CapabilitylInterface param);

    /**
     * <pre>
     * 能力属性更新
     * </pre>
     *
     * @param param
     */
    void capacityPropertyUpdate(CapacityPropertyParam param);

    /**
     * <pre>
     * 根据ID查询设备模板
     * </pre>
     *
     * @param id
     * @return
     */
    EquipmentTemplate getDeviceById(Long id);

    /**
     * <pre>
     * 根据设备ID获取设备接口关系表
     * </pre>
     *
     * @param id
     * @return
     */
    List<DeviceInterfaceDataVo> getDeviceInterfaceById(Long id);

    /**
     * <pre>
     * 获取能力接口
     * </pre>
     *
     * @param id
     * @return
     */
    List<CapacityInterfaceDataVo> getCapacityInterfaceById(Long id);

    /**
     * <pre>
     * 获取能力属性
     * </pre>
     *
     * @param id
     * @return
     */
    List<CapacityPropertyDataVo> getCapacityPropertyById(Long id);

    /**
     * <pre>
     * 新增设备能力
     * </pre>
     *
     * @param equipmentCapacityParam
     */
    void addDeviceCapacity(EquipmentCapacityParam equipmentCapacityParam);

    /**
     * <pre>
     * 删除设备
     * </pre>
     *
     * @param id
     */
    void deleteDevice(Long id);

    /**
     * <pre>
     * 查询真实设备数据
     * </pre>
     *
     * @return
     */
    List<Equipment> getRealDeviceList(List<DaoCriteria> criterias);

    /**
     * <pre>
     * 更新设备信息
     * </pre>
     *
     * @param param
     */
    void updateDeviceBasicInfo(TopoNodeParam param);

    /**
     * <pre>
     * 获取协议名称
     * </pre>
     *
     * @param id
     * @return
     */
    String getProtocolNameById(Long id);

    Equipment findEquipmentById(Long eqpId);

    /**
     * 根据sid查询设备
     *
     * @param sid
     * @return
     */
    Equipment findByCfgParamJson(String sid);

    List<Map> qryEqpCountByTemplAttrs(String orgCode, List<String> attrName);

    /**
     * 根据条件统计纳管分类类型
     *
     * @param daoCriterias
     * @return
     */
    Map<String, Object> equipTempTypeStatic(List<DaoCriteria> daoCriterias);

    /**
     * 设备原型分别统计
     *
     * @param daoCriterias
     * @return
     */
    Map<String, Object> equipTempIndustryStatic(List<DaoCriteria> daoCriterias);

    Map<String, Object> getRealDeviceData(EquipmentParam param);

    Map<String, Object> getDeviceCount(List<DaoCriteria> daoCriteria);

    Page<Object> findEqpBySubgraph(String subgraphId, String orgCode, Pageable page);

    /**
     * <pre>
     * 编辑设备信息
     * </pre>
     *
     * @param equipment
     */
    void editEquipmentInfo(EquipmentParam equipment);

    List<EquipmentBo> findEqpBySubgraphId(Long subgraphId, String orgCode);

    /**
     * 查询设备数量
     *
     * @param param
     * @return
     */
    Map<Long, Long> findEqpCount(QueryEqpCountParam param);

    /**
     * <pre>
     * 编辑设备json配置信息（单独）
     * </pre>
     *
     * @param param
     */
    void editEquipmentConfigInfo(Map<String, Object> param);

    /**
     * <pre>
     * 查询拓扑中设备所有配置信息
     * </pre>
     *
     * @param id
     * @return
     */
    Map<String, Object> fetchTopoDevice(Long id);

    /**
     * <pre>
     * 新增编辑设备图片
     * </pre>
     *
     * @param param
     */
    void addEditEquipmentImg(EquipmentParam param);

    /**
     * <pre>
     * 查询设备类型
     * </pre>
     */
    List<EquipmentTemplateVo> getEqpTmpModelByOrgCode(String orgCode);

    /**
     * <pre>
     * 分页查询设备类型下的所有设备（mybatis实现）
     * </pre>
     */
    CommonPage<EquipmentVo> pageDeviceDataByMybatis(EqpTemplPageParam requestParam);

    List<Map<String, Object>> statisticDeviceByModel(String orgCode);

    Map<String, Object> statisticDeviceByAlarmLevel(String orgCode);

    List<EquipmentVo> listByOrgCodeAndModel(String orgCode, String model);

    List<EquipmentVo> listDeviceAndDeviceCpblAttr(Long equipmentId);
}
