package com.yeejoin.amos.iot.business.service.intfc;

import java.util.List;
import java.util.Map;

import com.yeejoin.amos.iot.business.entity.Equipment;
import com.yeejoin.amos.iot.business.entity.Subgraph;
import com.yeejoin.amos.iot.business.param.SubgraphParam;
import com.yeejoin.amos.iot.business.param.TopoNodeParam;
import com.yeejoin.amos.iot.core.common.datastructure.TreeNode;
import com.yeejoin.amos.op.core.jpa.IGenericManager;

/**
 * 
 * <pre>
 * 拓扑模块方法
 * </pre>
 *
 * @author as-chenjiajun
 * @version $Id: ITopoService.java, v 0.1 2018年3月8日 下午1:30:09 as-chenjiajun Exp $
 */
public interface ITopoService
{
	/**
	 * 
	 * <pre>
	 * 删除子图
	 * </pre>
	 *
	 * @param id
	 */
	void delSubgraph(Long id);

	/**
	 * 
	 * <pre>
	 * 按条件获取子图集合
	 * </pre>
	 *
	 * @param param
	 * @return
	 */
	List<TreeNode> getSubgraphsByCondition(SubgraphParam param);

	/**
	 * 
	 * <pre>
	 * 根据ID获取子图
	 * </pre>
	 *
	 * @param id
	 * @return
	 */
	Subgraph getSubgraphById(Long id);
	
	/**
	 * 
	 * <pre>
	 * 根据ID获取子图
	 * </pre>
	 *
	 * @param id
	 * @return
	 */
	List<Subgraph> getSubviewByOrgCode(String orgCode);

	/**
	 * 
	 * <pre>
	 * 新增或编辑子图
	 * </pre>
	 *
	 * @param subgraph
	 * @return
	 * @throws Exception 
	 */
	Long addEditSubgraph(Subgraph subgraph,String orgCode);

	/**
	 * 
	 * <pre>
	 * 新增编辑子图数据（node和link）
	 * </pre>
	 *
	 * @param subgraph
	 * @return
	 */
	Long addEditSubgraphData(SubgraphParam subgraph);

	/**
	 * 
	 * <pre>
	 * 更新设备名称
	 * </pre>
	 *
	 * @param param
	 */
	Long updateDeviceName(TopoNodeParam param);

	/**
	 * 
	 * <pre>
	 * 获取设备最新的一条实时数据
	 * </pre>
	 *
	 * @param id
	 */
	Map<String,Object> getRealTimeData(Long id);

	/**
	 * 
	 * <pre>
	 * 查询指定子图下的设备
	 * </pre>
	 *
	 * @param name
	 * @param code 
	 * @return
	 */
	List<Equipment> getSubgraphEqpsByName(String name, String code);

	/**
	 * 
	 * <pre>
	 * 编辑子图背景图
	 * </pre>
	 *
	 * @param subgraph
	 */
	void addEditSubgraphImg(Subgraph subgraph);
	
	void oneOpenOrClose(Long id,Boolean enableMonitor);
	/**
	 * 
	 * <pre>
	 * 查询视图下设备
	 * </pre>
	 *
	 * @param id
	 * @param orgCode
	 * @param director
	 * @return
	 */
	List<Equipment> getSubgraphEqpsByID(Long id);
}
