package com.yeejoin.amos.iot.core.util;

import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Base64;
import java.util.Base64.Decoder;
import java.util.Base64.Encoder;

import javax.imageio.ImageIO;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import com.yeejoin.amos.iot.business.param.ImgData;

/**
 * 
 * <pre>
 * 图片文件服务
 * </pre>
 *
 * @author as-chenjiajun
 * @version $Id: FileOperateUtil.java, v 0.1 2018年1月29日 下午3:08:33 as-chenjiajun
 *          Exp $
 */
@Component
public class ImgFileServer
{
	@Value("${iot.imagesPath}")
	private String mImagesPath;
	
	private final Logger logger = Logger.getLogger(this.getClass());
	
	private static final String SEPARATOR = System.getProperty("file.separator");
	//实时监测背景图片路径
	private static final String BACKGROUND_FILE_PATH = "org";
	//设备ICON路径
	private static final String EQUIPMENT_ICON_FILE_PATH = "icon";
	//视图背景路径
	private static final String SUBGRAPH_FILE_PATH = "graph";
	
	
	
	final Decoder decoder = Base64.getDecoder();
	final Encoder encoder = Base64.getEncoder();
	
	public ImgFileServer()
	{
		
	}

	/**
	 * 
	 * <pre>
	 * 保存设备图片
	 * </pre>
	 *
	 * @param imageFileContent
	 * @param fileName
	 * @param type: device
	 * @return
	 */
	public boolean saveDeviceFile(String imageFileContent, String fileName,String type,String code)
	{
		InputStream input = null;
		try
		{
			File dir = new File(mImagesPath.replace("file:/", ""));
			boolean flg = true;
			if (!dir.exists())
			{
				dir.mkdirs();
			}
			byte bytes[] = imageFileContent.getBytes(code);
			int length = bytes.length;
			FileOutputStream stream = null;
			try
			{
				stream = new FileOutputStream(dir + SEPARATOR + fileName);
				stream.write(bytes, 0, length);
			}
			catch (Exception e)
			{
				logger.error(e.getMessage());
				flg = false;
			}
			finally
			{
				if (stream != null)
				{
					stream.close();
				}
			}
			return flg;
		}
		catch (IOException io)
		{
			logger.error(io.getMessage());
		}
		finally
		{
			if (input != null)
			{
				try
				{
					input.close();
				}
				catch (IOException e)
				{
					logger.error(e.getMessage());
				}
			}
		}
		return false;
	}

	/**
	 * 
	 * <pre>
	 * 保存背景图片
	 * </pre>
	 *
	 * @param imgData
	 * @return
	 */
	public boolean saveBackgroundFile(ImgData imgData)
	{
		InputStream input = null;
		try
		{
			File dir = new File(mImagesPath.replace("file:/", "")+BACKGROUND_FILE_PATH+SEPARATOR);
			boolean flg = true;
			if (!dir.exists())
			{
				dir.mkdirs();
			}
			FileOutputStream stream = null;
			byte[] decode = decoder.decode((imgData.getThumbUrl().split("base64,")[1]));
			try
			{
				stream = new FileOutputStream(dir + SEPARATOR+imgData.getUid() + "."+imgData.getName().split("\\.")[1]);
				stream.write(decode, 0, decode.length);
			}
			catch (Exception e)
			{
				logger.error(e.getMessage());
				flg = false;
			}
			finally
			{
				if (stream != null)
				{
					stream.close();
				}
			}
			return flg;
		}
		catch (IOException io)
		{
			logger.error(io.getMessage());
		}
		finally
		{
			if (input != null)
			{
				try
				{
					input.close();
				}
				catch (IOException e)
				{
					logger.error(e.getMessage());
				}
			}
		}
		return false;
	}
	
	/**
	 * 
	 * <pre>
	 * 获取背景图片
	 * </pre>
	 *
	 * @param fileName
	 * @return
	 */
	public String getBackgroundFile(String fileName)
	{
		InputStream input = null;
		try
		{
			File dir = new File(mImagesPath.replace("file:/", "")+BACKGROUND_FILE_PATH+SEPARATOR+fileName);
			if (!dir.exists())
			{
				return null;
			}
			ByteArrayOutputStream outputStream = null;
			String type = fileName.split("\\.")[1];
			BufferedImage bufferedImage = ImageIO.read(dir);
			outputStream = new ByteArrayOutputStream();
			ImageIO.write(bufferedImage, type, outputStream);
			return getBaseHead(type) + encoder.encodeToString(outputStream.toByteArray());
		}
		catch (Exception e)
		{
			logger.error(e.getMessage());
		}
		finally
		{
			if (input != null)
			{
				try
				{
					input.close();
				}
				catch (IOException e)
				{
					logger.error(e.getMessage());
				}
			}
		}
		return null;
	}
	
	/**
	 * 
	 * <pre>
	 * 获取文件头
	 * </pre>
	 *
	 * @param type
	 * @return
	 */
	private String getBaseHead(String type)
	{
		StringBuilder sBuilder = new StringBuilder();
		sBuilder.append("data:image/").append(type).append(";base64,");
		return sBuilder.toString();
	}

	/**
	 * 
	 * <pre>
	 * 删除文件
	 * </pre>
	 *
	 * @param fileName
	 */
	public void deleteFile(String fileName)
	{
		try
		{			
			Path dstPath = Paths.get(mImagesPath.replace("file:/", "")+BACKGROUND_FILE_PATH+SEPARATOR+fileName);
			if (dstPath.toFile().exists())
			{
				dstPath.toFile().delete();
			}
		}
		catch (Exception e)
		{
			logger.error(e.getMessage());
		}
	}
	
	/**
	 * 
	 * <pre>
	 * 保存设备图片
	 * </pre>
	 *
	 * @param imageFileContent
	 * @param fileName
	 * @param type: device
	 * @return
	 */
	public boolean saveIconFile(String imageFileContent, String fileName,String type,String code)
	{
		InputStream input = null;
		try
		{
			File dir = new File(mImagesPath.replace("file:/", "")+EQUIPMENT_ICON_FILE_PATH+SEPARATOR);
			boolean flg = true;
			if (!dir.exists())
			{
				dir.mkdirs();
			}
			byte bytes[] = imageFileContent.getBytes(code);
			int length = bytes.length;
			FileOutputStream stream = null;
			try
			{
				stream = new FileOutputStream(dir + SEPARATOR + fileName);
				stream.write(bytes, 0, length);
			}
			catch (Exception e)
			{
				logger.error(e.getMessage());
				flg = false;
			}
			finally
			{
				if (stream != null)
				{
					stream.close();
				}
			}
			return flg;
		}
		catch (Exception e)
		{
			logger.error(e.getMessage());
		}
		finally
		{
			if (input != null)
			{
				try
				{
					input.close();
				}
				catch (IOException e)
				{
					logger.error(e.getMessage());
				}
			}
		}
		return false;
	}

	public boolean saveSubgraphFile(String binaryImg, String imgName,
			String type, String code)
	{
		InputStream input = null;
		try
		{
			File dir = new File(mImagesPath.replace("file:/", "")+SUBGRAPH_FILE_PATH+SEPARATOR);
			boolean flg = true;
			if (!dir.exists())
			{
				dir.mkdirs();
			}
			byte bytes[] = binaryImg.getBytes(code);
			int length = bytes.length;
			FileOutputStream stream = null;
			try
			{
				stream = new FileOutputStream(dir + SEPARATOR + imgName);
				stream.write(bytes, 0, length);
			}
			catch (Exception e)
			{
				logger.error(e.getMessage());
				flg = false;
			}
			finally
			{
				if (stream != null)
				{
					stream.close();
				}
			}
			return flg;
		}
		catch (Exception e)
		{
			logger.error(e.getMessage());
		}
		finally
		{
			if (input != null)
			{
				try
				{
					input.close();
				}
				catch (IOException e)
				{
					logger.error(e.getMessage());
				}
			}
		}
		return false;
	}
}
