package com.yeejoin.amos.iot.core.util;

import java.net.URISyntaxException;
import java.util.concurrent.ConcurrentHashMap;

import com.yeejoin.amos.iot.rxtx.service.SerialDataQueue;
import net.sf.json.JSONObject;

import org.apache.log4j.Logger;
import org.fusesource.mqtt.client.FutureConnection;
import org.fusesource.mqtt.client.MQTT;
import org.fusesource.mqtt.client.QoS;

/**
 * 
 * <pre>
 * Mqtt发送工具类
 * </pre>
 *
 * @author as-guowubin
 * @version $Id: MqttPublisher.java, v 0.1 2018年5月7日 下午2:21:01 as-guowubin Exp $
 */
public class MqttPublisher
{
	private static final Logger log = Logger.getLogger(MqttPublisher.class);
	private final static boolean CLEAN_START = true;
	private final static short KEEP_ALIVE = 30;// 低耗网络，但是又需要及时获取数据，心跳30s
	public final static long RECONNECTION_ATTEMPT_MAX = 6;
	public final static long RECONNECTION_DELAY = 2000;

	public final static int SEND_BUFFER_SIZE = 2 * 1024 * 1024;// 发送最大缓冲为2M
	private FutureConnection connection = null;

	private static ConcurrentHashMap<String, MqttPublisher> cacheMap = new ConcurrentHashMap<>();
	private MqttPublisher(String user, String password, String host, String port){
		init(user, password, host, port);
	}

	public FutureConnection getConnection() {
		return connection;
	}

	private void init(String user, String password, String host, String port) {
		MQTT mqtt = new MQTT();
		try {
			// 设置服务端的ip
			mqtt.setHost("tcp://" + host + ":" + port);
			mqtt.setUserName(user);
			mqtt.setPassword(password);
			// 连接前清空会话信息
			mqtt.setCleanSession(CLEAN_START);
			// 设置重新连接的次数
			mqtt.setReconnectAttemptsMax(RECONNECTION_ATTEMPT_MAX);
			// 设置重连的间隔时间
			mqtt.setReconnectDelay(RECONNECTION_DELAY);
			// 设置心跳时间
			mqtt.setKeepAlive(KEEP_ALIVE);
			// 设置缓冲的大小
			mqtt.setSendBufferSize(SEND_BUFFER_SIZE);

			// 创建连接
			connection = mqtt.futureConnection();
			connection.connect();
		}
		catch (URISyntaxException e)
		{
			log.error("MqttAction执行失败：" + e.getMessage());
		}
		catch (Exception e)
		{
			log.error("MqttAction执行失败：" + e.getMessage());
		}
	}


	public static MqttPublisher getInstance(String user, String password, String host, String port) {
		MqttPublisher instance = cacheMap.get(user + password + host + port);
		if (instance == null) {
			synchronized (MqttPublisher.class) {
				if (instance == null) {
					instance = new MqttPublisher(user, password, host, port);
					cacheMap.put(user + password + host + port, instance);
				}
			}
		}
		return instance;
	}

	/**
	 * 
	 * <pre>
	 * Mqtt消息发送
	 * </pre>
	 *
	 * @param message 发送消息内容
	 * @param topic 消息主题
	 */
	public void publisher(Object message, String topic) throws Exception
	{
		connection.publish(topic, JSONObject.fromObject(message).toString().getBytes("UTF-8"),
				QoS.AT_LEAST_ONCE, false);
	}
}
