package com.yeejoin.amos.iot.core.util.query;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashSet;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.persistence.EntityManager;

import org.apache.commons.lang3.StringUtils;
import org.hibernate.Query;
import org.hibernate.Session;

import com.yeejoin.amos.iot.core.enums.QueryOperatorEnum;
import com.yeejoin.amos.op.core.common.query.DaoCriteria;

/**
 * Criteria助手类
 * 
 * @author as-youjun
 *
 */
public class CriteriaHelper {
	public static final int UNLIMITED_RESULTS = -1;

	public static <T> Object getFullQuery(Session session, Class<T> entityClass, int firstResult, int maxResult,
			List<DaoCriteria> daoCriteriaList) {
		HQLResult hQLResult = getHQLResult(daoCriteriaList);
		StringBuffer hql = new StringBuffer();
		hql.append(" from " + entityClass.getName() + " where 1=1  AND ");
		hql.append(hQLResult.hql);
		return hqlFind(session, hql.toString(), firstResult, maxResult, hQLResult.values);
	}

	private static <T> Map<String, List<DaoCriteria>> fenlei(List<DaoCriteria> criteraList) {
		Map<String, List<DaoCriteria>> criteraType = new LinkedHashMap<String, List<DaoCriteria>>();
		if (criteraList != null && criteraList.size() > 0) {
			for (DaoCriteria daoCriteria : criteraList) {
				List<DaoCriteria> temp = criteraType.get(daoCriteria.getPropertyName());
				if (temp == null) {
					temp = new ArrayList<DaoCriteria>();
					criteraType.put(daoCriteria.getPropertyName(), temp);
				}
				temp.add(daoCriteria);
			}
		}
		return criteraType;

	}

	public static <T> javax.persistence.Query getFullQuery(EntityManager entityManager, Class<T> entityClass,
			String sqlstr, List<DaoCriteria> daoCriteriaList, boolean naiveQuery) {
		StringBuffer Sql = new StringBuffer(sqlstr);
		StringBuffer orders = new StringBuffer();
		Sql.append(" where 1=1 ");
		List<DaoCriteria> newdaoCriteriaList = new ArrayList<DaoCriteria>();
		if (daoCriteriaList != null && daoCriteriaList.size() > 0) {
			for (DaoCriteria daoCriteria : daoCriteriaList) {
				String propertyName = daoCriteria.getPropertyName();
				String newpropertyName = String.valueOf(propertyName);
				if (newpropertyName.contains(".")) {
					newpropertyName = newpropertyName.replace(".", "_");
				}
				Object value = daoCriteria.getValue();
				String operator = daoCriteria.getOperator();
				newdaoCriteriaList.add(new DaoCriteria(newpropertyName, operator, value));
				if (StringUtils.isNotEmpty(propertyName)) {
					if (operator.equals(QueryOperatorEnum.EQUAL.getName())) {
						Sql.append(" and " + propertyName + " =:" + newpropertyName);// room.name
																						// like
																						// :room
					} else if (operator.equals(QueryOperatorEnum.LIKE.getName())) {
						Sql.append(" and " + propertyName + " like :" + newpropertyName);
					} else if (operator.equals(QueryOperatorEnum.NOT_IN.getName())) {
						Sql.append(" and " + propertyName + " not in (:" + newpropertyName + ")");
					} else if (operator.equals(QueryOperatorEnum.OR.getName())) {
						Sql.append(" and " + propertyName + " or (:" + newpropertyName + ")");
					} else if (operator.equals(QueryOperatorEnum.BIGGER.getName())) {
						Sql.append(" and " + propertyName + " > (:" + newpropertyName + ")");
					} else if (operator.equals(QueryOperatorEnum.BIGGER_EQUAL.getName())) {
						Sql.append(" and " + propertyName + " >= (:" + newpropertyName + ")");
					} else if (operator.equals(QueryOperatorEnum.LESS.getName())) {
						Sql.append(" and " + propertyName + " < (:" + newpropertyName + ")");
					} else if (operator.equals(QueryOperatorEnum.LESS_EQUAL.getName())) {
						Sql.append(" and " + propertyName + " <= (:" + newpropertyName + ")");
					} else if (operator.equals(QueryOperatorEnum.NOT_EQUAL.getName())) {
						Sql.append(" and " + propertyName + " != (:" + newpropertyName + ")");
					} else if (operator.equals(QueryOperatorEnum.IN.getName())) {
						Sql.append(" and " + propertyName + " in (:" + newpropertyName + ")");
					}
				} else {
					if (operator.equals(QueryOperatorEnum.ORDER_BY.getName())) {
						orders.append(QueryOperatorEnum.ORDER_BY.getName()).append(" ").append(value.toString());
					}
				}
			}
		}
		javax.persistence.Query query;
		if (naiveQuery) {
			query = entityManager.createNativeQuery(Sql.toString(), entityClass);
		} else {
			query = entityManager.createQuery(Sql.toString(), entityClass);
		}
		setParameterToQuery(query, newdaoCriteriaList);
		return query;
	}

	private static void setParameterToQuery(javax.persistence.Query q, List<DaoCriteria> criteraList) {

		Map<String, List<DaoCriteria>> params = fenlei(criteraList);
		if (params != null && !params.isEmpty()) {
			for (String key : params.keySet()) {
				if (StringUtils.isNotEmpty(key)) {
					if (params.get(key) instanceof List<?>) {
						List<DaoCriteria> collection = params.get(key);
						if (collection.size() == 1) {
							DaoCriteria dao = (DaoCriteria) collection.get(0);
							if (dao.getOperator().equals(QueryOperatorEnum.LIKE.getName())) {
								q.setParameter(key, dao.getValue());
							} else {
								q.setParameter(key, dao.getValue());
							}
						} else if (collection.size() > 1) {
							List<Object> paramList = new ArrayList<Object>();
							for (DaoCriteria object : collection) {
								paramList.add(object.getValue());
							}
							q.setParameter(key, paramList);
						}

					}
				}
			}
		}
	}

	public static String getFullSqlCriteria(String sqlstr, String gruopbystr, List<DaoCriteria> daoCriteriaList) {
		StringBuffer Sql = new StringBuffer(sqlstr);
		StringBuffer orders = new StringBuffer();
		Sql.append(" where 1=1 ");
		if (daoCriteriaList != null && daoCriteriaList.size() > 0) {
			for (DaoCriteria daoCriteria : daoCriteriaList) {
				String propertyName = daoCriteria.getPropertyName();
				Object value = daoCriteria.getValue();
				String operator = daoCriteria.getOperator();
				if (StringUtils.isNotEmpty(propertyName)) {
					if (operator.equals(QueryOperatorEnum.EQUAL.getName())) {
						Sql.append(" and " + propertyName + " =:" + propertyName);// room.name
																					// like
																					// :room
					} else if (operator.equals(QueryOperatorEnum.LIKE.getName())) {
						Sql.append(" and " + propertyName + " like :" + propertyName);
					} else if (operator.equals(QueryOperatorEnum.NOT_IN.getName())) {
						Sql.append(" and " + propertyName + " not in (:" + propertyName + ")");
					} else if (operator.equals(QueryOperatorEnum.OR.getName())) {
						Sql.append(" and " + propertyName + " or (:" + propertyName + ")");
					} else if (operator.equals(QueryOperatorEnum.BIGGER.getName())) {
						Sql.append(" and " + propertyName + " > (:" + propertyName + ")");
					} else if (operator.equals(QueryOperatorEnum.BIGGER_EQUAL.getName())) {
						Sql.append(" and " + propertyName + " >= (:" + propertyName + ")");
					} else if (operator.equals(QueryOperatorEnum.LESS.getName())) {
						Sql.append(" and " + propertyName + " < (:" + propertyName + ")");
					} else if (operator.equals(QueryOperatorEnum.LESS_EQUAL.getName())) {
						Sql.append(" and " + propertyName + " <= (:" + propertyName + ")");
					} else if (operator.equals(QueryOperatorEnum.NOT_EQUAL.getName())) {
						Sql.append(" and " + propertyName + " != (:" + propertyName + ")");
					} else if (operator.equals(QueryOperatorEnum.IN.getName())) {
						Sql.append(" and " + propertyName + " in (:" + propertyName + ")");
					}
				} else {
					if (operator.equals(QueryOperatorEnum.ORDER_BY.getName())) {
						orders.append(QueryOperatorEnum.ORDER_BY.getName()).append(" ").append(value.toString());
					}
				}
			}
		}
		Sql.append(gruopbystr);
		return Sql.toString();
	}

	public static void main(String[] args) {
		ArrayList<DaoCriteria> criterias = new ArrayList<DaoCriteria>();
		DaoCriteria criteria = new DaoCriteria();
		criteria.setPropertyName("deviceName");
		criteria.setOperator(QueryOperatorEnum.EQUAL.getName());
		criteria.setValue(null);
		criterias.add(criteria);

		criteria = new DaoCriteria();
		criteria.setPropertyName("deviceType");
		criteria.setOperator(QueryOperatorEnum.EQUAL.getName());
		criteria.setValue(2);
		criterias.add(criteria);
	}

	/**********************************************************************/
	public static class HQLResult {
		public String hql;
		public Object[] values;
	}

	public static HQLResult getHQLResult(List<DaoCriteria> criterias) {
		return getHQLResult(criterias, null, null, true);
	}

	public static HQLResult getHQLResult(List<DaoCriteria> criterias, String entityName, String[] orFields,
			boolean DateSPEnabled) {
		HashSet<String> orFieldsSet = new HashSet<String>();
		if (orFields != null) {
			for (String string : orFields) {
				orFieldsSet.add(string);
			}
		}
		Map<String, List<DaoCriteria>> map = fenlei(criterias);
		HQLResult result = getHQLResult(map, entityName, orFieldsSet, DateSPEnabled);
		return result;
	}

	private static HQLResult getHQLResult(Map<String, List<DaoCriteria>> map, String entityName, Set<String> orFields,
			boolean DateSPEnabled) {

		HQLResult result = new HQLResult();
		ArrayList<Object> values = new ArrayList<Object>();
		if (map.size() == 0) {
			result.hql = "1 = 1";
			result.values = new Object[] {};
			return result;
		}
		boolean and = false;
		StringBuilder str = new StringBuilder();
		List<DaoCriteria> orderByList = new ArrayList<DaoCriteria>();
		for (List<DaoCriteria> listDaoCriterias : map.values()) {
			boolean isOrderBy = false;
			for (int k = 0; k < listDaoCriterias.size(); k++) {
				DaoCriteria daoCriteria = (DaoCriteria) listDaoCriterias.get(k);
				String ope = daoCriteria.getOperator();
				QueryOperatorEnum temp = QueryOperatorEnum.getEnum(ope);
				if (temp == QueryOperatorEnum.ORDER_BY) {
					orderByList.add(daoCriteria);
					isOrderBy = true;
				}
			}
			if (isOrderBy) {
				continue;
			}
			if (and == true) {
				str.append(" and ");
			}
			str.append("(");
			for (int i = 0; i < listDaoCriterias.size(); i++) {
				DaoCriteria daoCriteria = (DaoCriteria) listDaoCriterias.get(i);
				String ope = daoCriteria.getOperator();
				String name = daoCriteria.getPropertyName();
				Object value = daoCriteria.getValue();
				String operator = getOperator(ope);
				QueryOperatorEnum temp = QueryOperatorEnum.getEnum(ope);
				if (i != 0) {
					if (DateSPEnabled) {
						if (!(value instanceof Date)) {
							str.append(" OR ");
						} else {
							str.append(" AND ");
						}
					} else {
						str.append(" OR ");
					}
				}
				if (value != null) {
					if (entityName == null) {
						str.append(name).append(" ");

					} else {
						str.append(entityName).append(".").append(name).append(" ");
					}

					if (temp == QueryOperatorEnum.LIKE) {
						str.append(operator).append(" ? ");
						values.add(value);
					} else if (temp == QueryOperatorEnum.IN) {
						str.append(" IN ").append(" ( ");
						if (value instanceof String[]) {
							String[] objs = (String[]) value;
							for (int m = 0; m < objs.length; m++) {
								if ((objs.length - 1) == m) {
									str.append("'").append(objs[m]).append("'");
								} else
									str.append("'").append(objs[m] + "' , ");
							}
						} else if (value instanceof Integer[]) {
							Integer[] objs = (Integer[]) value;
							for (int m = 0; m < objs.length; m++) {
								if ((objs.length - 1) == m) {
									str.append(objs[m]);
								} else
									str.append(objs[m] + " , ");
							}
						} else if (value instanceof ArrayList<?>) {
							ArrayList<?> objs = (ArrayList<?>) value;
							for (int m = 0; m < objs.size(); m++) {
								if ((objs.size() - 1) == m) {
									str.append(objs.get(m));
								} else
									str.append(objs.get(m) + " , ");
							}
						}
						str.append(" ) ");
					} else if (temp == QueryOperatorEnum.NOT_IN) {
						str.append(" NOT IN ").append(" ( ");
						if (value instanceof String[]) {
							String[] objs = (String[]) value;
							for (int m = 0; m < objs.length; m++) {
								if ((objs.length - 1) == m) {
									str.append("'").append(objs[m]).append("'");
								} else
									str.append("'").append(objs[m] + "' , ");
							}
						} else if (value instanceof Integer[]) {
							Integer[] objs = (Integer[]) value;
							for (int m = 0; m < objs.length; m++) {
								if ((objs.length - 1) == m) {
									str.append(objs[m]);
								} else
									str.append(objs[m] + " , ");
							}
						} else if (value instanceof List<?>) {
							ArrayList<?> objs = (ArrayList<?>) value;
							for (int m = 0; m < objs.size(); m++) {
								if ((objs.size() - 1) == m) {
									str.append(objs.get(m));
								} else
									str.append(objs.get(m) + " , ");
							}
						}
						str.append(" ) ");
					} else {
						str.append(operator).append(" ? ");
						values.add(value);
					}

				} else {
					if (temp == QueryOperatorEnum.EQUAL) {
						if (entityName == null) {
							str.append(name).append(" IS NULL ");
						} else {
							str.append(entityName).append(".").append(name).append(" IS NULL ");
						}
					} else if (temp == QueryOperatorEnum.IN) {
						str.append(" IN ").append(" ( ");
						if (value instanceof String[]) {
							String[] objs = (String[]) value;
							for (int m = 0; m < objs.length; m++) {
								if ((objs.length - 1) == m) {
									str.append("'").append(objs[m]).append("'");
								} else
									str.append("'").append(objs[m] + "' , ");
							}
						} else if (value instanceof Integer[]) {
							Integer[] objs = (Integer[]) value;
							for (int m = 0; m < objs.length; m++) {
								if ((objs.length - 1) == m) {
									str.append(objs[m]);
								} else
									str.append(objs[m] + " , ");
							}
						} else if (value instanceof ArrayList<?>) {
							ArrayList<?> objs = (ArrayList<?>) value;
							for (int m = 0; m < objs.size(); m++) {
								if ((objs.size() - 1) == m) {
									str.append(objs.get(m));
								} else
									str.append(objs.get(m) + " , ");
							}
						}
						str.append(" ) ");
					} else {
						if (entityName == null) {
							str.append(name).append(" IS NOT NULL ");
						} else {
							str.append(entityName).append(".").append(name).append(" IS NOT NULL ");
						}
					}
				}
			}
			str.append(")");
			and = true;
		}
		for (DaoCriteria daoCriteria : orderByList) {
			String name = daoCriteria.getPropertyName();
			Object value = daoCriteria.getValue();
			if (str.indexOf("ORDER BY") > -1) {
				str.append(",").append(name).append(" ").append(value);
			} else {
				str.append(" " + QueryOperatorEnum.ORDER_BY.getName()).append(" ").append(name).append(" ")
						.append(value);
			}
		}
		result.hql = str.toString();
		result.values = values.toArray();
		return result;
	}

	public static String getOperator(String operator) {
		return QueryOperatorEnum.getEnum(operator).getCondition();

	}

	public static Object hqlFind(Session session, String hql, int firstResult, int maxResult, Object[] values) {
		Query queryObject = session.createQuery(hql);
		if (maxResult != UNLIMITED_RESULTS) {
			queryObject.setFirstResult(firstResult);
			queryObject.setMaxResults(maxResult);
		}
		if (values != null) {
			for (int i = 0; i < values.length; i++) {
				queryObject.setParameter(i, values[i]);
			}
		}
		return queryObject.list();
	}

	/**
	 * 将驼峰字段转换为数据库字段
	 * 
	 * @param property
	 * @return
	 */
	public static String propertyToColumn(String property) {
		if (property == null || property.isEmpty()) {
			return "";
		}
		StringBuilder column = new StringBuilder();
		column.append(property.substring(0, 1).toLowerCase());
		for (int i = 1; i < property.length(); i++) {
			String s = property.substring(i, i + 1);
			// 在小写字母前添加下划线
			if (!Character.isDigit(s.charAt(0)) && s.equals(s.toUpperCase())) {
				column.append("_");
			}
			// 其他字符直接转成小写
			column.append(s.toLowerCase());
		}

		return column.toString();
	}
}