package com.yeejoin.amos.iot.core.util.query;

import java.util.ArrayList;
import java.util.List;
import java.util.function.Predicate;

import com.yeejoin.amos.op.core.common.query.DaoCriteria;

/**
 * 查询条件构造器
 * 
 * @author as-youjun
 *
 */
public class DaoCriteriaBuilder {

	private List<DaoCriteria> daoCriterias;

	private DaoCriteriaBuilder() {
		daoCriterias = new ArrayList<>();
	}

	private DaoCriteriaBuilder(List<DaoCriteria> daoCriteria) {
		daoCriterias = daoCriteria;
	}

	public static DaoCriteriaBuilder builder() {
		return new DaoCriteriaBuilder();
	}

	public static DaoCriteriaBuilder builder(List<DaoCriteria> daoCriterias) {
		return new DaoCriteriaBuilder(daoCriterias);
	}

	public DaoCriteriaBuilder append(String propertyName, String operator, Object value) {
		daoCriterias.add(new DaoCriteria(propertyName, operator, value));
		return this;
	}

	public DaoCriteriaBuilder append(Predicate<Boolean> predicate, String propertyName, String operator, Object value) {
		if (predicate.test(true)) {
			daoCriterias.add(new DaoCriteria(propertyName, operator, value));
		}
		return this;
	}

	public List<DaoCriteria> getConditionList() {
		return daoCriterias;
	}

	public <T> BaseQuerySpecification<T> toSpec() {
		return new BaseQuerySpecification<>(daoCriterias);
	}

}
