package com.yeejoin.amos.iot.core.util.query;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

import javax.persistence.criteria.Predicate;

import org.springframework.data.jpa.domain.Specification;

import com.yeejoin.amos.iot.business.entity.Equipment;
import com.yeejoin.amos.iot.business.entity.EquipmentTemplate;
import com.yeejoin.amos.iot.business.entity.LinkVo;
import com.yeejoin.amos.iot.business.entity.OrgImgRelationships;
import com.yeejoin.amos.iot.business.entity.Rule;
import com.yeejoin.amos.iot.business.entity.RuleNode;
import com.yeejoin.amos.iot.business.entity.Section;
import com.yeejoin.amos.iot.business.entity.Subgraph;
import com.yeejoin.amos.iot.business.param.EquipmentParam;
import com.yeejoin.amos.iot.business.param.OrgImgParam;
import com.yeejoin.amos.iot.business.param.RulePageParam;
import com.yeejoin.amos.iot.business.param.SectionParam;
import com.yeejoin.amos.iot.business.param.SubgraphParam;

/**
 * 
 * <pre>
 * 查询条件生成工具
 * </pre>
 *
 * @author as-chenjiajun
 * @version $Id: QuerySpecificationHelper.java, v 0.1 2018年4月10日 下午4:42:12 as-chenjiajun Exp $
 */
public class QuerySpecificationHelper
{
	public static Specification<Rule> getSpecification(RulePageParam request)
	{
		return (root, query, cb) -> {
			List<Predicate> list = new ArrayList<Predicate>();
			Optional<String> nameOp = Optional.ofNullable(request.getName());
			if (nameOp.isPresent())
			{
				Predicate name = cb.like(root.get("name").as(String.class), String.join(nameOp.get(), "%","%"));
				list.add(name);
			}
			Optional<String> orgCodeOp = Optional.ofNullable(request.getOrgCode());
			if (orgCodeOp.isPresent())
			{
				Predicate orgLke = cb.equal(root.get("orgCode").as(String.class), request.getOrgCode());
				Predicate orgEqual = cb.like(root.get("orgCode").as(String.class), request.getOrgCode()+"*%");
				Predicate org = cb.or(orgLke, orgEqual);
				list.add(org);
			}
			Predicate[] p = new Predicate[list.size()];
			return cb.and(list.toArray(p));
		};
	}
	
	/**
	 * 
	 * <pre>
	 * 根据NodeID获取相关联节点
	 * </pre>
	 *
	 * @param id
	 * @return
	 */
	public static Specification<LinkVo> getNodeLinks(String id)
	{
		return (root, query, cb) -> {
			List<Predicate> list = new ArrayList<Predicate>();
			if (id != null)
			{
				Predicate source = cb.equal(root.get("sourceId").as(String.class), id);
				list.add(source);
				Predicate target = cb.equal(root.get("targetId").as(String.class), id);
				list.add(target);
			}
			Predicate[] p = new Predicate[list.size()];
			return cb.or(list.toArray(p));
		};
	}
	
	/**
	 * 
	 * <pre>
	 * 根据NodeID获取相关联节点
	 * </pre>
	 *
	 * @param id
	 * @return
	 */
	public static Specification<RuleNode> getRuleNodes(RuleNode ruleNode)
	{
		return (root, query, cb) -> {
			List<Predicate> list = new ArrayList<Predicate>();
			if (ruleNode.getType() != null)
			{
				Predicate source = cb.equal(root.get("type").as(String.class), ruleNode.getType());
				list.add(source);
			}
			Predicate[] p = new Predicate[list.size()];
			return cb.and(list.toArray(p));
		};
	}
	/**
	 * 
	 * <pre>
	 * 根据NodeID获取相关联节点
	 * </pre>
	 *
	 * @param id
	 * @return
	 */
	public static Specification<OrgImgRelationships> getOrgImgs(OrgImgParam param)
	{
		return (root, query, cb) -> {
			List<Predicate> list = new ArrayList<Predicate>();
			if (param.getOrgCode() != null)
			{
				Predicate orgLke = cb.equal(root.get("orgCode").as(String.class), param.getOrgCode());
				Predicate orgEqual = cb.like(root.get("orgCode").as(String.class), param.getOrgCode()+"*%");
				Predicate org = cb.or(orgLke, orgEqual);
				list.add(org);
			}
			Predicate[] p = new Predicate[list.size()];
			return cb.and(list.toArray(p));
		};
	}
	
	public static Specification<EquipmentTemplate> getDuplicationNameEqpTemplate(EquipmentParam param)
	{
		return (root, query, cb) -> {
			List<Predicate> list = new ArrayList<Predicate>();
			if (param.getOrgCode() != null)
			{
				Predicate orgLke = cb.equal(root.get("orgCode").as(String.class), param.getOrgCode());
				Predicate orgEqual = cb.like(root.get("orgCode").as(String.class), param.getOrgCode()+"*%");
				Predicate org = cb.or(orgLke, orgEqual);
				list.add(org);
			}
			if (param.getName() != null)
			{
				Predicate name = cb.equal(root.get("name").as(String.class), param.getName());
				list.add(name);
			}
			if (param.getId() != null)
			{
				Predicate id = cb.notEqual(root.get("id").as(Long.class), param.getId());
				list.add(id);
			}
			Predicate[] p = new Predicate[list.size()];
			return cb.and(list.toArray(p));
		};
	}

	public static Specification<Equipment> getDuplicationNameEquipment(Equipment equipment)
	{
		return (root, query, cb) -> {
			List<Predicate> list = new ArrayList<Predicate>();
			if (equipment.getOrgCode() != null)
			{
				Predicate orgLke = cb.equal(root.get("orgCode").as(String.class), equipment.getOrgCode());
				Predicate orgEqual = cb.like(root.get("orgCode").as(String.class), equipment.getOrgCode()+"*%");
				Predicate org = cb.or(orgLke, orgEqual);
				list.add(org);
			}
			if (equipment.getName() != null)
			{
				Predicate name = cb.equal(root.get("name").as(String.class), equipment.getName());
				list.add(name);
			}
			if (equipment.getId() != null)
			{
				Predicate id = cb.notEqual(root.get("id").as(Long.class), equipment.getId());
				list.add(id);
			}
			Predicate[] p = new Predicate[list.size()];
			return cb.and(list.toArray(p));
		};
	}

	public static Specification<Subgraph> getDuplicationNameSubgraph(SubgraphParam param)
	{
		return (root, query, cb) -> {
			List<Predicate> list = new ArrayList<Predicate>();
			if (param.getOrgCode() != null)
			{
				Predicate orgLke = cb.equal(root.get("orgCode").as(String.class), param.getOrgCode());
				Predicate orgEqual = cb.like(root.get("orgCode").as(String.class), param.getOrgCode()+"*%");
				Predicate org = cb.or(orgLke, orgEqual);
				list.add(org);
			}
			if (param.getName() != null)
			{
				Predicate name = cb.equal(root.get("name").as(String.class), param.getName());
				list.add(name);
			}
			if (param.getId() != null)
			{
				Predicate id = cb.notEqual(root.get("id").as(Long.class), param.getId());
				list.add(id);
			}
			Predicate[] p = new Predicate[list.size()];
			return cb.and(list.toArray(p));
		};
	}

	public static Specification<Section> getSectionParam(SectionParam param)
	{
		return (root, query, cb) -> {
			List<Predicate> list = new ArrayList<Predicate>();
			if (param.getOrgCode() != null)
			{
				Predicate orgLke = cb.equal(root.get("orgCode").as(String.class), param.getOrgCode());
				Predicate orgEqual = cb.like(root.get("orgCode").as(String.class), param.getOrgCode()+"*%");
				Predicate org = cb.or(orgLke, orgEqual);
				list.add(org);
			}
			if (param.getName() != null)
			{
				Predicate name = cb.like(root.get("name").as(String.class), "%"+param.getName()+"%");
				list.add(name);
			}
			if (param.getIsRecommend() != null)
			{
				Predicate isRecommend = cb.equal(root.get("isRecommend").as(Boolean.class), param.getIsRecommend());
				list.add(isRecommend);
			}
			if (param.getType() != null)
			{
				Predicate type = cb.equal(root.get("type").as(Integer.class), param.getType());
				list.add(type);
			}
			if (param.getIsFree() != null)
			{
				Predicate isFree = cb.equal(root.get("isFree").as(Boolean.class), param.getIsFree());
				list.add(isFree);
			}
			if (param.getRateValue() != null)
			{
				Predicate rateValue = cb.equal(root.get("rateValue").as(Float.class), param.getRateValue());
				list.add(rateValue);
			}
			Predicate[] p = new Predicate[list.size()];
			return cb.and(list.toArray(p));
		};
	}
}
