package com.yeejoin.amos.iot.icmp;

import com.yeejoin.amos.iot.business.constants.IotConstant;
import com.yeejoin.amos.iot.business.dic.IotDic;
import com.yeejoin.amos.iot.business.polling.PollingResult;
import com.yeejoin.amos.iot.business.queue.AlarmDataObj;
import com.yeejoin.amos.iot.business.queue.AlarmDataQueue;
import com.yeejoin.amos.iot.business.queue.EqpConnData;
import org.apache.log4j.Logger;

import java.util.Map;

/**
 * imcp协议数据采集service
 *
 * @author as-youjun
 */
public class IcmpCollectService {

    /**
     * 日志记录器
     */
    private static final Logger logger = Logger.getLogger(IcmpCollectService.class);

    /**
     * 获取协议数据
     *
     * @param paramMap
     * @return
     */
    public PollingResult fetchIcmpData(Map<String, Object> paramMap) {
        PollingResult pollingResult = new PollingResult();

        boolean isIcmpConnected = false;//设备icmp是否连通
        String errorMessage = null;
        try {
            String hostName = (String) paramMap.get("ip");
            Integer count = Integer.valueOf(paramMap.get("count").toString());
            Integer packetTimeout = Integer.valueOf(paramMap.get("packetTimeout").toString());
            IcmpParam icmpParam = new IcmpParam();
            icmpParam.setHostName(hostName.trim());
            icmpParam.setCount(count);
            icmpParam.setPacketTimeout(packetTimeout);
            PingResult pingResult = PingUtil.executePing(icmpParam);
            if (pingResult == null) {
                isIcmpConnected = false;
                errorMessage = "：设备PING不通！";
            } else {
                pollingResult.setResult(pingResult);
                if (pingResult.getPacketLossRate() == null) {
                    isIcmpConnected = false;
                } else if (pingResult.getPacketLossRate().doubleValue() >= 100.0) {
                    isIcmpConnected = false;
                } else {
                    isIcmpConnected = true;
                }
            }
        } catch (Exception e) {
            errorMessage = e.getMessage();
            isIcmpConnected = false;
            logger.error("icmp fetch data error! " + e);
        }
        //设置ping结果
        if (isIcmpConnected) {//ping成功
            pollingResult.setStatus(IotConstant.SUCCESS);
        } else {//ping失败
            pollingResult.setStatus(IotConstant.FAILURE);
            pollingResult.setErrorMsg(errorMessage);
        }
        if (paramMap.get(IotConstant.EquipmentConstant.EQPID) == null) {
            //测试设备，无需产生告警
            logger.debug("test equipment, no need raise alarm!");
        } else {
            long eqpId = (Long) paramMap.get(IotConstant.EquipmentConstant.EQPID);
            raiseAlarm(eqpId, isIcmpConnected);
        }

        return pollingResult;
    }

    /**
     * 产生设备icmp连通性告警
     *
     * @param eqpId
     * @param isIcmpConnected
     */
    private void raiseAlarm(long eqpId, boolean isIcmpConnected) {
        EqpConnData eqpConnData = new EqpConnData();
        eqpConnData.setEqpId(eqpId);
        eqpConnData.setProtocolName(IotDic.ProtocolName.ICMP);
        eqpConnData.setConnected(isIcmpConnected);
        AlarmDataObj alarmDataObj = new AlarmDataObj(eqpConnData);
        alarmDataObj.setType(IotConstant.EQUIPMENT_CONNECTION);
        AlarmDataQueue.getInstance().add(alarmDataObj);
        logger.debug("设备icmp连通性,eqpId:" + eqpId + ",isIcmpConnected:" + isIcmpConnected);
    }

}
