package com.yeejoin.amos.iot.icmp;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

import org.apache.log4j.Logger;

import com.yeejoin.amos.iot.business.constants.IotConstant;

/**
 * ping工具类
 * 
 * @author as-youjun
 *
 */
public class PingUtil {

	/**
	 * linux系统ping
	 * 
	 * @param hostName
	 * @param count
	 * @return
	 */
	public static PingResult linuxPing(String hostName, int count) {
		try {
			Process pro = Runtime.getRuntime().exec(
					"ping " + hostName + " -c " + count);
			BufferedReader buf = new BufferedReader(new InputStreamReader(
					pro.getInputStream()));
			List<String> resultStringList = new ArrayList<String>();
			String line = null;
			while ((line = buf.readLine()) != null) {
				// 移除所有空行
				if (line.equals("")) {
					continue;
				}
				resultStringList.add(line);
			}

			if (!resultStringList.get(resultStringList.size() - 1)
					.contains("/")) {
				PingResult result = new PingResult();
				result.setAvgTime(IotConstant.FAIL_INT_VALUE);
				result.setMaxTime(IotConstant.FAIL_INT_VALUE);
				result.setPacketLossRate(100.0);
				result.setMinTime(IotConstant.FAIL_INT_VALUE);
				result.setIcmpPackSize(IotConstant.FAIL_INT_VALUE);
				return result;
			} else {
				PingResult result = new PingResult();
				// 拿到倒数第二行,解析丢包率
				String temp = resultStringList.get(resultStringList.size() - 2);
				String packetLossString = temp.split(",")[2];
				result.setPacketLossRate(Double.parseDouble(packetLossString
						.split("%")[0].trim()));
				// 拿到倒数第一行,解析rtt
				temp = resultStringList.get(resultStringList.size() - 1);
				String[] rtt = temp.split("=")[1].split("/");

				result.setMinTime((int) Float.parseFloat(rtt[0].trim()));
				result.setAvgTime((int) Float.parseFloat(rtt[1].trim()));
				result.setMaxTime((int) Float.parseFloat(rtt[2].trim()));
				result.setIcmpPackSize(Integer.parseInt(resultStringList.get(1)
						.split(" ")[0].trim()));
				return result;
			}
		} catch (Exception e) {
			Logger.getLogger(PingUtil.class.getSimpleName()).error(e);
			Logger.getLogger(PingUtil.class.getSimpleName()).error(
					e.getMessage());
		}
		return null;

	}

	/**
	 * 英文windows系统ping
	 * 
	 * @param hostName
	 * @param count
	 * @param packetTimeout
	 * @return
	 */
	public static PingResult windowPingEn(String hostName, int count,
			int packetTimeout) {
		try {
			StringBuilder sb = new StringBuilder();
			sb.append("ping ").append(hostName).append(" -n ").append(count)
					.append(" -w ").append(packetTimeout);
			Process pro = Runtime.getRuntime().exec(sb.toString());
			BufferedReader buf = new BufferedReader(new InputStreamReader(
					pro.getInputStream()));
			List<String> resultStringList = new ArrayList<String>();
			String line = null;

			boolean isUnreachable = false;
			while ((line = buf.readLine()) != null) {
				if (line.contains("unreachable")) {
					isUnreachable = true;
					break;
				}
				// 移除所有空行
				if (line.equals("")) {
					continue;
				}
				resultStringList.add(line);
			}
			buf.close();
			if (resultStringList.get(resultStringList.size() - 1).contains("(")
					|| isUnreachable) {
				// 未ping 通 ,返回类似 Packets: Sent = 4, Received = 0, Lost = 4 (100%
				// loss),
				PingResult result = new PingResult();
				result.setAvgTime(IotConstant.FAIL_INT_VALUE);
				result.setMaxTime(IotConstant.FAIL_INT_VALUE);
				result.setPacketLossRate(100.0);
				result.setMinTime(IotConstant.FAIL_INT_VALUE);
				result.setIcmpPackSize(IotConstant.FAIL_INT_VALUE);
				return result;
			} else {
				// 可ping 通 返回类似 如下
				// Ping statistics for 192.168.0.1:
				// Packets: Sent = 4, Received = 4, Lost = 0 (0% loss),
				// Approximate round trip times in milli-seconds:
				// Minimum = 1ms, Maximum = 2ms, Average = 1ms

				// 正在 Ping 10.240.2.123 具有 32 字节的数据:
				// 来自 10.240.2.123 的回复: 字节=32 时间<1ms TTL=61
				// 来自 10.240.2.123 的回复: 字节=32 时间<1ms TTL=61
				// 来自 10.240.2.123 的回复: 字节=32 时间<1ms TTL=61
				// 来自 10.240.2.123 的回复: 字节=32 时间<1ms TTL=61
				//
				// 10.240.2.123 的 Ping 统计信息:
				// 数据包: 已发送 = 4，已接收 = 4，丢失 = 0 (0% 丢失)，
				// 往返行程的估计时间(以毫秒为单位):
				// 最短 = 0ms，最长 = 0ms，平均 = 0ms

				PingResult result = new PingResult();
				int size = resultStringList.size();
				String temp = resultStringList.get(size - 1);// 获取最后一行,解析延迟,
				if (!temp.contains(",")) {
					temp = temp.replace("ms", "ms,");
				}
				String[] rtt = temp.split(",");
				String rttTemp = rtt[0].split("=")[1];
				result.setMinTime(Integer.parseInt(rttTemp.substring(0,
						rttTemp.length() - 2).trim()));
				rttTemp = rtt[1].split("=")[1];
				result.setMaxTime(Integer.parseInt(rttTemp.substring(0,
						rttTemp.length() - 2).trim()));
				rttTemp = rtt[2].split("=")[1];
				result.setAvgTime(Integer.parseInt(rttTemp.substring(0,
						rttTemp.length() - 2).trim()));
				// 获取 倒数第三行,解析丢包
				result.setPacketLossRate(Double.parseDouble(resultStringList
						.get(size - 3).split("\\(")[1].split("%")[0].trim()));
				String packSize = resultStringList.get(3);
				if (!packSize.contains("Request timed out.")) {
					result.setIcmpPackSize(Integer.parseInt(resultStringList
							.get(size - 6).split("=")[1].split(" ")[0].trim()));
				} else {
					result.setIcmpPackSize(0);
				}
				return result;
			}
		} catch (Exception e) {
			Logger.getLogger(PingUtil.class.getSimpleName()).error(
					e.getMessage());
		}
		return null;
	}

	/**
	 * 中文windows系统ping
	 * 
	 * @param hostName
	 * @param count
	 * @param packetTimeout
	 * @return
	 * @throws IOException
	 */
	public static PingResult windowPingZh(String hostName, int count,
			int packetTimeout) throws IOException {
		StringBuilder sb = new StringBuilder();
		sb.append("ping ").append(hostName).append(" -n ").append(count)
				.append(" -w ").append(packetTimeout);
		Process pro = Runtime.getRuntime().exec(sb.toString());
		BufferedReader buf = new BufferedReader(new InputStreamReader(
				pro.getInputStream(), "gbk"));
		List<String> resultStringList = new ArrayList<String>();
		String line = null;
		boolean isUnreachable = false;
		while ((line = buf.readLine()) != null) {
			if (line.contains("无法访问")) {
				isUnreachable = true;
				break;
			}
			// 移除所有空行
			if (line.equals("")) {
				continue;
			}
			resultStringList.add(line);
		}
		if (isUnreachable
				|| resultStringList.get(resultStringList.size() - 1).contains(
						"(")) {
			// 未ping 通 ,返回类似 Packets: Sent = 4, Received = 0, Lost = 4 (100%
			// loss),
			PingResult result = new PingResult();
			result.setAvgTime(IotConstant.FAIL_INT_VALUE);
			result.setMaxTime(IotConstant.FAIL_INT_VALUE);
//			Logger.getLogger(PingUtil.class).warn(
//					"Ping lost: isUnreachable" + isUnreachable
//							+ ";resultStringList:" + resultStringList);
			result.setPacketLossRate(100.0);
			result.setMinTime(IotConstant.FAIL_INT_VALUE);
			result.setIcmpPackSize(IotConstant.FAIL_INT_VALUE);
			return result;
		} else {
//			Logger.getLogger(PingUtil.class).warn(
//					"Ping lost: isUnreachable" + isUnreachable
//							+ ";resultStringList:" + resultStringList);
			// 可ping 通 返回类似 如下
			// 正在 Ping 10.240.2.123 具有 32 字节的数据:
			// 来自 10.240.2.123 的回复: 字节=32 时间<1ms TTL=61
			// 来自 10.240.2.123 的回复: 字节=32 时间<1ms TTL=61
			// 来自 10.240.2.123 的回复: 字节=32 时间<1ms TTL=61
			// 来自 10.240.2.123 的回复: 字节=32 时间<1ms TTL=61
			//
			// 10.240.2.123 的 Ping 统计信息:
			// 数据包: 已发送 = 4，已接收 = 4，丢失 = 0 (0% 丢失)，
			// 往返行程的估计时间(以毫秒为单位):
			// 最短 = 0ms，最长 = 0ms，平均 = 0ms
			PingResult result = new PingResult();
			int size = resultStringList.size();
			String temp = resultStringList.get(size - 1);// 获取最后一行,解析延迟,
			// System.out.println("~~~~~~~~~~~"+temp);
			// 由于某些系统的字符格式原因，如vista，返回的格式不被识别，呈现：口口口=1ms口口口=2ms口口口口0ms之类的，进行特殊处理
			if (!temp.contains("，")) {
				temp = temp.replace("ms", "ms，");
			}
			String[] rtt = temp.split("，");
			String rttTemp = rtt[0].split("=")[1];
			result.setMinTime(Integer.parseInt(rttTemp.substring(0,
					rttTemp.length() - 2).trim()));
			rttTemp = rtt[1].split("=")[1];
			result.setMaxTime(Integer.parseInt(rttTemp.substring(0,
					rttTemp.length() - 2).trim()));
			rttTemp = rtt[2].split("=")[1];
			result.setAvgTime(Integer.parseInt(rttTemp.substring(0,
					rttTemp.length() - 2).trim()));

//			Logger.getLogger(PingUtil.class).warn(
//					"resultStringList:" + resultStringList);
			// 获取 倒数第三行,解析丢包
			result.setPacketLossRate(Double.parseDouble(resultStringList.get(
					size - 3).split("\\(")[1].split("%")[0].trim()));
			int length = resultStringList.get(0).split(" ").length;
			if (length == 6) {
				result.setIcmpPackSize(Integer.parseInt(resultStringList.get(0)
						.split(" ")[4]));
			} else {
				result.setIcmpPackSize(0);
			}
			// result.setIcmpPackSize(Integer.parseInt(resultStringList.get(
			// size - 7).split("<")[0].split("=")[1].split(" ")[0].trim()));
			return result;
		}
	}

	/**
	 * Ping检测
	 * 
	 * @param icmpParam
	 * @return
	 */
	public static PingResult executePing(IcmpParam icmpParam) {
		String hostName = icmpParam.getHostName();
		int count = icmpParam.getCount();
		int packetTimeout = icmpParam.getPacketTimeout();
		String osName = null;
		try {
			osName = System.getProperty("os.name").toLowerCase();
			if (osName.startsWith("linux")) {
				return linuxPing(hostName, count);
			} else if (osName.startsWith("windows")) {
				if (Locale.getDefault().equals(Locale.CHINA)
						|| Locale.getDefault().equals(Locale.CHINESE)) {
					if (!osName.equals("windows 2003")
							&& !osName.equals("windows xp")) {
						return windowPingZh(hostName, count, packetTimeout);
					} else {
						return windowPingEn(hostName, count, packetTimeout);
					}
				} else {
					return windowPingEn(hostName, count, packetTimeout);
				}
			} else {
				Logger.getLogger(PingUtil.class).error(
						hostName + ":os.name:" + osName
								+ ",PingResult will return null.");
				return null;
			}

		} catch (Exception ex) {
			Logger.getLogger(PingUtil.class).error(
					"ping error:host " + hostName);
			Logger.getLogger(PingUtil.class).error(ex);
			Logger.getLogger(PingUtil.class).error(
					hostName + ":os.name:" + osName
							+ ",PingResult will return null.");
		}
		return null;
	}

}
