/**
 * 
 */
package com.yeejoin.amos.iot.mongodb.dao.repository;

import java.util.Date;
import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.bson.Document;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.BasicQuery;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.data.mongodb.core.query.TextCriteria;
import org.springframework.data.mongodb.core.query.TextQuery;
import org.springframework.data.mongodb.repository.MongoRepository;
import org.springframework.stereotype.Repository;

import com.mongodb.BasicDBList;
import com.mongodb.BasicDBObject;
import com.mongodb.DBObject;
import com.yeejoin.amos.iot.business.param.QueryMetricDataParam;
import com.yeejoin.amos.iot.context.IotContext;
import com.yeejoin.amos.iot.mongodb.entity.EqpMetricData;

/**
 * 设备指标数据Repository接口
 * 
 * @author as-youjun
 *
 */
@Repository
public interface EquipMetricDataRepository extends MongoRepository<EqpMetricData, String> {

	/**
	 * 根据参数查询指标数据
	 * 
	 * @param param
	 * @return
	 */
	default List<EqpMetricData> findByParam(QueryMetricDataParam param) {
		MongoTemplate mongoTemplate = (MongoTemplate) IotContext.getInstance().getBean("mongoTemplate");
		Query query = new Query();
		//添加ID条件查询
		if (param.getEqpId() != null)
		{
			query.addCriteria(Criteria.where("eqpId").is(param.getEqpId()));
		}
		if (param.getCpblId() != null)
		{
			query.addCriteria(Criteria.where("capabilityData.cpblId").is(param.getCpblId()));
		}	
		if (param.getEqpType() != null) {
			query.addCriteria(Criteria.where("eqpType").is(param.getEqpType()));
		}
		if (StringUtils.isNotEmpty(param.getEquipName())) {
			query.addCriteria(Criteria.where("equipName").regex(".*" + param.getEquipName() + ".*"));
		}
		if (StringUtils.isNotEmpty(param.getCpblName())) {
			query.addCriteria(Criteria.where("capabilityData.cpblName").regex(".*" + param.getCpblName() + ".*"));
		}
		if (param.getCpblType() != null) {
			query.addCriteria(Criteria.where("capabilityData.cpblType").is(param.getCpblType()));
		}
		Date startTime = param.getStartTime();
		Date endTime = param.getEndTime();
		if (startTime != null && endTime == null) {
			query.addCriteria(Criteria.where("capabilityData.exeTime").gte(param.getStartTime()));

		} else if (startTime == null && endTime != null)
		{
			query.addCriteria(Criteria.where("capabilityData.exeTime").lte(param.getEndTime()));
		} else if (startTime != null && endTime != null)
		{
			query.addCriteria(Criteria.where("capabilityData.exeTime").lte(param.getEndTime()).gte(param.getStartTime()));
		}
		List<EqpMetricData> equipMetricDatas = mongoTemplate.find(query, EqpMetricData.class);
		return equipMetricDatas;
	}

	/**
	 * 分页查询指标数据
	 * 
	 * @param param
	 * @return
	 */
	default List<EqpMetricData> pageByParam(QueryMetricDataParam param) {
		MongoTemplate mongoTemplate = (MongoTemplate) IotContext.getInstance().getBean("mongoTemplate");
		Query query = new Query();
		if (param.getEqpType() != null) {
			query.addCriteria(Criteria.where("eqpType").is(param.getEqpType()));
		}
		if (StringUtils.isNotEmpty(param.getEquipName())) {
			query.addCriteria(Criteria.where("equipName").regex(".*" + param.getEquipName() + ".*"));
		}
		if (StringUtils.isNotEmpty(param.getCpblName())) {
			query.addCriteria(Criteria.where("capabilityData.cpblName").regex(".*" + param.getCpblName() + ".*"));
		}
		if (param.getCpblType() != null) {
			query.addCriteria(Criteria.where("capabilityData.cpblType").is(param.getCpblType()));
		}
		if (param.getStartTime() != null) {
			query.addCriteria(Criteria.where("capabilityData.exeTime").gte(param.getStartTime()));

		}
		if (param.getEndTime() != null) {
			query.addCriteria(Criteria.where("capabilityData.exeTime").lte(param.getEndTime()));
		}
		query.with(new Sort(Sort.Direction.ASC, "eqpId", "capabilityData.cpblId", "capabilityData.exeTime"));
		final Pageable pageableRequest = new PageRequest(param.getPage(), param.getSize());
		query.with(pageableRequest);
		List<EqpMetricData> equipMetricDatas = mongoTemplate.find(query, EqpMetricData.class, "EquipMetricData");
		return equipMetricDatas;
	}

	/**
	 * 测试BasicQuery
	 * 
	 * @param param
	 * @return
	 */
	default List<EqpMetricData> testBasicQuery(QueryMetricDataParam param) {
		MongoTemplate mongoTemplate = (MongoTemplate) IotContext.getInstance().getBean("mongoTemplate");
		Query query = null;
		if ((param.getEqpType() != null) && (param.getCpblType() != null)) {
			// 1.查询条件
			BasicDBList basicDBList = new BasicDBList();
			basicDBList.add(new BasicDBObject("eqpType", param.getEqpType()));
			basicDBList.add(new BasicDBObject("capabilityData.cpblType", param.getCpblType()));
			Document queryObject = new Document();
			//DBObject queryObject = new BasicDBObject();
			queryObject.put("$or", basicDBList);
			// 2.返回字段
			/**
			 * BasicQuery查询语句可以指定返回字段，构造函数 BasicQuery(DBObject queryObject,
			 * DBObject fieldsObject) fieldsObject 这个字段可以指定返回字段
			 * fieldsObject.put(key,value) key：字段 value： 说明： 1或者true表示返回字段
			 * 0或者false表示不返回该字段 _id:默认就是1，没指定返回该字段时，默认会返回，除非设置为0是，就不会返回该字段。
			 * 指定返回字段，有时文档字段多并数据大时，我们指定返回我们需要的字段，这样既节省传输数据量，减少了内存消耗，提高了性能，在数据大时，性能很明显的。
			 */
			Document fieldsObject = new Document();
			//BasicDBObject fieldsObject = new BasicDBObject();
			fieldsObject.put("eqpId", 1);
			fieldsObject.put("equipName", 1);
			fieldsObject.put("eqpType", 1);
			fieldsObject.put("capabilityData.cpblId", 1);
			fieldsObject.put("capabilityData.cpblName", 1);
			fieldsObject.put("capabilityData.cpblType", 1);
			fieldsObject.put("capabilityData.specificEquipData.sourceId", 1);
			fieldsObject.put("capabilityData.metricData.sourceID", 1);
			fieldsObject.put("capabilityData.metricData.formatTime", 1);
			fieldsObject.put("capabilityData.metricData.status", 1);
			// fieldsObject.put("capabilityData.metricData.latitude", 1);
			// fieldsObject.put("capabilityData.metricData.longitude", 1);
			query = new BasicQuery(queryObject, fieldsObject);
			query.with(new Sort(Sort.Direction.ASC, "eqpId", "capabilityData.cpblId",
					"capabilityData.metricData.exeTime"));
			final Pageable pageableRequest = new PageRequest(param.getPage(), param.getSize());
			query.with(pageableRequest);
			List<EqpMetricData> equipMetricDatas = mongoTemplate.find(query, EqpMetricData.class,
					"EquipMetricData");
			return equipMetricDatas;
		}
		return null;
	}

	/**
	 * 测试testTextQuery
	 * 
	 * @param param
	 * @return
	 */
	default List<EqpMetricData> testTextQuery(QueryMetricDataParam param) {
		MongoTemplate mongoTemplate = (MongoTemplate) IotContext.getInstance().getBean("mongoTemplate");
		TextCriteria criteria = TextCriteria.forDefaultLanguage().matchingAny(param.getEquipName());
		Query query = new TextQuery(criteria);
		// query.with(new Sort(Sort.Direction.ASC, "eqpId",
		// "capabilityData.cpblId", "capabilityData.metricData.exeTime"));
		// final Pageable pageableRequest = new PageRequest(param.getPage(),
		// param.getSize());
		// query.with(pageableRequest);
		List<EqpMetricData> equipMetricDatas = mongoTemplate.find(query, EqpMetricData.class, "EquipMetricData");
		return equipMetricDatas;
	}

}
