/**
 * 
 */
package com.yeejoin.amos.iot.mongodb.entity;

import org.springframework.data.annotation.Id;
import org.springframework.data.mongodb.core.index.CompoundIndex;
import org.springframework.data.mongodb.core.index.CompoundIndexes;
import org.springframework.data.mongodb.core.index.Indexed;
import org.springframework.data.mongodb.core.mapping.Document;

import java.util.HashMap;
import java.util.Map;

/**
 * 设备(设备、数据库、中间件等)指标数据
 * 
 * @author as-youjun
 *
 */
@Document(collection = "EqpMetricData")
@CompoundIndexes({ @CompoundIndex(name = "idx_emd_eqp_type_name", def = "{'eqpType': 1, 'eqpName': 1}"),
		@CompoundIndex(name = "idx_emd_eqp_cpbl_metric_time", def = "{'eqpType': 1, 'eqpName': 1, 'capabilityData.cpblType':1, 'capabilityData.cpblName':1, 'capabilityData.exeTime':1}")
		,@CompoundIndex(name = "idx_eqpId_cpblId_exeTime", def = "{'eqpId':1,'capabilityData.cpblId':1,'capabilityData.exeTime':1}")
})
public class Alarm {

	/**
	 * id
	 */
	@Id
	// @Indexed(unique = true)
	public String id;

	// 以下是设备的基本信息

	/**
	 * 设备id
	 */
	private Long eqpId;

	/**
	 * 设备名称
	 */
	@Indexed
	private String eqpName;

	/**
	 * 设备描述
	 */
	private String eqpDescr;

	/**
	 * 用户id
	 */
	private String userId;

	/**
	 * 设备模板id
	 */
	private Long eqpTmplId;

	/**
	 * 设备模板名称
	 */
	private String eqpTmplName;

	/**
	 * 设备型号
	 */
	private String eqpModel;

	/**
	 * 设备类别：1表示采集设备，2表示传感设备，3表示控制设备
	 */
	@Indexed
	private Integer eqpType;

	/**
	 * 描述
	 */
	private String eqpTmpldescr;

	/**
	 * 设备模板可见性：0表示公共，1表示私有
	 */
	private Integer visibility;

	// 以下是设备特有数据
	/**
	 * 设备特有数据(value为基本类型的封装类)
	 */
	private Map<String, Object> specEqpDataMap = new HashMap<>();

	/**
	 * 能力数据
	 */
	private CapabilityData capabilityData;

	/**
	 * @return the id
	 */
	public String getId() {
		return id;
	}

	/**
	 * @param id
	 *            the id to set
	 */
	public void setId(String id) {
		this.id = id;
	}

	/**
	 * @return the eqpId
	 */
	public Long getEqpId() {
		return eqpId;
	}

	/**
	 * @param eqpId
	 *            the eqpId to set
	 */
	public void setEqpId(Long eqpId) {
		this.eqpId = eqpId;
	}

	/**
	 * @return the eqpName
	 */
	public String getEqpName() {
		return eqpName;
	}

	/**
	 * @param eqpName
	 *            the eqpName to set
	 */
	public void setEqpName(String eqpName) {
		this.eqpName = eqpName;
	}

	/**
	 * @return the eqpDescr
	 */
	public String getEqpDescr() {
		return eqpDescr;
	}

	/**
	 * @param eqpDescr
	 *            the eqpDescr to set
	 */
	public void setEqpDescr(String eqpDescr) {
		this.eqpDescr = eqpDescr;
	}

	/**
	 * @return the userId
	 */
	public String getUserId() {
		return userId;
	}

	/**
	 * @param userId
	 *            the userId to set
	 */
	public void setUserId(String userId) {
		this.userId = userId;
	}

	/**
	 * @return the eqpTmplId
	 */
	public Long getEqpTmplId() {
		return eqpTmplId;
	}

	/**
	 * @param eqpTmplId
	 *            the eqpTmplId to set
	 */
	public void setEqpTmplId(Long eqpTmplId) {
		this.eqpTmplId = eqpTmplId;
	}

	/**
	 * @return the eqpTmplName
	 */
	public String getEqpTmplName() {
		return eqpTmplName;
	}

	/**
	 * @param eqpTmplName
	 *            the eqpTmplName to set
	 */
	public void setEqpTmplName(String eqpTmplName) {
		this.eqpTmplName = eqpTmplName;
	}

	/**
	 * @return the eqpModel
	 */
	public String getEqpModel() {
		return eqpModel;
	}

	/**
	 * @param eqpModel
	 *            the eqpModel to set
	 */
	public void setEqpModel(String eqpModel) {
		this.eqpModel = eqpModel;
	}

	/**
	 * @return the eqpType
	 */
	public Integer getEqpType() {
		return eqpType;
	}

	/**
	 * @param eqpType
	 *            the eqpType to set
	 */
	public void setEqpType(Integer eqpType) {
		this.eqpType = eqpType;
	}

	/**
	 * @return the eqpTmpldescr
	 */
	public String getEqpTmpldescr() {
		return eqpTmpldescr;
	}

	/**
	 * @param eqpTmpldescr
	 *            the eqpTmpldescr to set
	 */
	public void setEqpTmpldescr(String eqpTmpldescr) {
		this.eqpTmpldescr = eqpTmpldescr;
	}

	/**
	 * @return the visibility
	 */
	public Integer getVisibility() {
		return visibility;
	}

	/**
	 * @param visibility
	 *            the visibility to set
	 */
	public void setVisibility(Integer visibility) {
		this.visibility = visibility;
	}

	/**
	 * @return the specEqpDataMap
	 */
	public Map<String, Object> getSpecEqpDataMap() {
		return specEqpDataMap;
	}

	/**
	 * @param specEqpDataMap
	 *            the specEqpDataMap to set
	 */
	public void setSpecEqpDataMap(Map<String, Object> specEqpDataMap) {
		this.specEqpDataMap = specEqpDataMap;
	}

	/**
	 * @return the capabilityData
	 */
	public CapabilityData getCapabilityData() {
		return capabilityData;
	}

	/**
	 * @param capabilityData
	 *            the capabilityData to set
	 */
	public void setCapabilityData(CapabilityData capabilityData) {
		this.capabilityData = capabilityData;
	}

}
