/**
 *
 */
package com.yeejoin.amos.iot.mongodb.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.yeejoin.amos.iot.business.dao.mapper.CapacityAttrMapper;
import com.yeejoin.amos.iot.business.dao.mapper.EqpTmplAttrMapper;
import com.yeejoin.amos.iot.business.dao.repository.ICapabilityDao;
import com.yeejoin.amos.iot.business.dao.repository.IEquipmentDao;
import com.yeejoin.amos.iot.business.dao.repository.IEquipmentTemplateDao;
import com.yeejoin.amos.iot.business.entity.Capability;
import com.yeejoin.amos.iot.business.entity.Equipment;
import com.yeejoin.amos.iot.business.entity.EquipmentTemplate;
import com.yeejoin.amos.iot.business.entity.MetricData;
import com.yeejoin.amos.iot.business.entity.mybatis.CapacityAttrBo;
import com.yeejoin.amos.iot.business.entity.mybatis.EqpTmplAttributeBo;
import com.yeejoin.amos.iot.business.param.QueryMetricDataParam;
import com.yeejoin.amos.iot.mongodb.dao.repository.EquipMetricDataRepository;
import com.yeejoin.amos.iot.mongodb.entity.CapabilityData;
import com.yeejoin.amos.iot.mongodb.entity.EqpMetricData;
import com.yeejoin.amos.iot.mongodb.service.intfc.IEquipMetricDataService;
import com.yeejoin.amos.iot.netty.vehicle.VechicleServiceHandler;
import com.yeejoin.amos.iot.socket.websocket.service.IWebSocketService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.ObjectUtils;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * 设备指标数据Service实现类
 *
 * @author as-youjun
 *
 */
@Service("equipMetricDataService")
public class EquipMetricDataServiceImpl implements IEquipMetricDataService {

    @Autowired
    private EquipMetricDataRepository equipMetricDataRepository;

    @Autowired
    private IEquipmentTemplateDao equipmentTemplateDao;

    @Autowired
    private IEquipmentDao equipmentDao;

    @Autowired
    private ICapabilityDao capabilityDao;

    @Autowired
    private MongoTemplate mongoTemplate;

    @Autowired
    private IWebSocketService webSocketService;

    @Autowired
    private EqpTmplAttrMapper eqpTmplAttrMapper;

    @Autowired
    private CapacityAttrMapper capacityAttrMapper;

    private ExecutorService threadPool = Executors.newCachedThreadPool();

    /**
     * 保存数据
     */
    @Transactional
    @Override
    public void save(EqpMetricData equipMetricData) {
        equipMetricDataRepository.save(equipMetricData);
    }


    /**
     * 根据参数查询指标数据
     */
    @Override
    public List<EqpMetricData> findByParam(QueryMetricDataParam param) {
        return equipMetricDataRepository.findByParam(param);
    }

    /**
     * 指标数据保存到mongodb
     */
    @Transactional
    public void saveMetricData2MongoDb(MetricData metricData, String sourceId) {
        // 1.设备模板数据
        EquipmentTemplate equipmentTemplate = equipmentTemplateDao
                .findById(metricData.getEqpTmplId()).get();
        // 2.设备数据
        Equipment equipment = equipmentDao.findById(metricData.getEqpId()).get();
        // 3.能力数据
        Capability capability = capabilityDao.findById(metricData.getCpblId()).get();
        CapabilityData capabilityData = VechicleServiceHandler
                .buildCapabilityData(capability, metricData);
        EqpMetricData equipMetricData = VechicleServiceHandler
                .buildEqpMetricData(metricData, equipmentTemplate, equipment,
                        sourceId, capabilityData);
        // 4.保存到mongodb
        save(equipMetricData);

        // 5.保存到能治mongodb collection中
        if (!ObjectUtils.isEmpty(capability.getMongodb())) {
            if (equipMetricData != null && equipMetricData.getCapabilityData() != null
                    && equipMetricData.getCapabilityData().getMetricDataMap() != null) {
                if (!ObjectUtils.isEmpty(equipMetricData)) {
                    JSONObject jsonObject = new JSONObject();
                    jsonObject.put("eqpId", equipMetricData.getEqpId());
                    jsonObject.put("eqpName", equipMetricData.getEqpName());
                    jsonObject.put("eqpDescr", equipMetricData.getEqpDescr());
                    jsonObject.put("userId", equipMetricData.getUserId());
                    jsonObject.put("eqpTmplId", equipMetricData.getEqpTmplId());
                    jsonObject.put("eqpTmplName", equipMetricData.getEqpTmplName());
                    jsonObject.put("eqpModel", equipMetricData.getEqpModel());
                    jsonObject.put("eqpType", equipMetricData.getEqpType());
                    jsonObject.put("eqpTmpldescr", equipMetricData.getEqpTmpldescr());
                    jsonObject.put("visibility", equipMetricData.getVisibility());
                    jsonObject.put("orgCode", equipment.getOrgCode());
                    jsonObject.put("authOrg", equipment.getAuthOrg());
                    Map<String, Object> specEqpDataMap = equipMetricData.getSpecEqpDataMap();
                    if (specEqpDataMap != null) {
                        for (Map.Entry<String, Object> entry : specEqpDataMap.entrySet()) {
                            jsonObject.put(entry.getKey(), entry.getValue());
                        }
                    }
                    CapabilityData saveCapabilityData = equipMetricData.getCapabilityData();
                    if (saveCapabilityData != null) {
                        jsonObject.put("cpblDescr", saveCapabilityData.getCpblDescr());
                        jsonObject.put("cpblId", saveCapabilityData.getCpblId());
                        jsonObject.put("cpblName", saveCapabilityData.getCpblName());

                        jsonObject.put("cpblType", saveCapabilityData.getCpblType());
                        jsonObject.put("exeTime", saveCapabilityData.getExeTime());
                        jsonObject.put("protocolId", saveCapabilityData.getProtocolId());
                        Map<String, Object> metricDataMap = saveCapabilityData.getMetricDataMap();
                        if (metricDataMap != null) {
                            for (Map.Entry<String, Object> entry : metricDataMap.entrySet()) {
                                jsonObject.put(entry.getKey(), entry.getValue());
                            }
                        }
                    }
                    JSONObject equipmentAttrs = getAttrByEqpTmplId(equipment);
                    jsonObject.put("equipmentAttrs", equipmentAttrs);
                    mongoTemplate.save(jsonObject, capability.getMongodb());
                    List<CapacityAttrBo> capacityAttrBos = capacityAttrMapper.listByCapacityId(metricData.getCpblId());
                    jsonObject.put("capacityAttrList", capacityAttrBos);
//                    handleAlarmData(capability,jsonObject,equipmentAttrs,saveCapabilityData.getMetricDataMap());
                    threadPool.execute(new EquipMetricDataServiceImpl.SendMongoDBMessageTask(capability.getMongodb(), jsonObject));
                }
            }
        }
    }

    private JSONObject getAttrByEqpTmplId(Equipment equipment) {
        Map<String, Object> cfgParamMap = equipment.getCfgParamMap();
        List<EqpTmplAttributeBo> positionBos = eqpTmplAttrMapper.listByEqpTmplId(equipment.getEqpTmplId());
        JSONObject attr = new JSONObject();
        positionBos.forEach(p -> {
            String eqipId = "equip_" + p.getId();
            attr.put(p.getName(), cfgParamMap.get(eqipId));
        });
        return attr;
    }

    class SendMongoDBMessageTask implements Runnable {

        private JSONObject jsonObject;
        private String filter;
        private JSONObject attr;

        public SendMongoDBMessageTask(String filter, JSONObject jsonObject) {
            this.jsonObject = jsonObject;
            this.filter = filter;
        }

        @Override
        public void run() {
            HashMap<String, Object> message = new HashMap<>();
            HashMap<String, String> filterMap = new HashMap<>();
            filterMap.put("subData", this.filter);

            message.put("message", jsonObject.toJSONString());
            message.put("path", "metricData");
            message.put("filter", filterMap);
            try {
                webSocketService.sendMetricDataMessage(message);
            } catch (Exception e) {
                e.printStackTrace();
            }

        }
    }

    /**
     * 指标数据保存到mongodb
     *
     * @param metricData
     */
    @Transactional
    public void saveMetricData2MongoDb(MetricData metricData) {
        // 1.设备模板数据
        EquipmentTemplate equipmentTemplate = equipmentTemplateDao
                .findById(metricData.getEqpTmplId()).get();
        // 2.设备数据
        Equipment equipment = equipmentDao.findById(metricData.getEqpId()).get();
        // 3.能力数据
        Capability capability = capabilityDao.findById(metricData.getCpblId()).get();
        CapabilityData capabilityData = VechicleServiceHandler
                .buildCapabilityData(capability, metricData);
        EqpMetricData equipMetricData = VechicleServiceHandler
                .buildEqpMetricData(metricData, equipmentTemplate, equipment,
                        null, capabilityData);
        // 4.保存到mongodb
        save(equipMetricData);
    }

}
