package com.yeejoin.amos.iot.netty;

import com.yeejoin.amos.iot.business.constants.IotConstant;
import com.yeejoin.amos.iot.business.dao.mapper.MonitorMapper;
import com.yeejoin.amos.iot.business.dic.IotDic;
import com.yeejoin.amos.iot.business.entity.mybatis.EquipmentBo;
import com.yeejoin.amos.iot.business.enums.MonitorType;
import com.yeejoin.amos.iot.business.param.QueryMonitorEqpParam;
import com.yeejoin.amos.iot.business.queue.AlarmDataObj;
import com.yeejoin.amos.iot.business.queue.AlarmDataQueue;
import com.yeejoin.amos.iot.business.queue.EqpConnData;
import com.yeejoin.amos.op.core.util.ApplicationConfig;
import org.apache.commons.collections.CollectionUtils;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Timer;
import java.util.TimerTask;

/**
 * 设备连通性监测
 */
@Service
public class EqpConnCheckService {


    /**
     * 日志记录器
     */
    private static final Logger logger = Logger.getLogger(EqpConnCheckService.class);

    @Autowired
    private MonitorMapper monitorMapper;

    @Autowired
    private ApplicationConfig applicationConfig;

    /**
     * 初始延迟initialDelay秒，每隔fixedRate秒
     * 检测设备连通性
     */
    public void start() {
        // 一个线程管理pollingjob
        Timer timer = new Timer();
        long initialDelay = Long.valueOf(applicationConfig.getParamValueStr("report.equipment.initialDelay")).longValue();
        long fixedRate = Long.valueOf(applicationConfig.getParamValueStr("report.equipment.fixedRate")).longValue();
        timer.schedule(new EqpConnCheckTask(), initialDelay * 1000, fixedRate * 1000);
    }

    /**
     * 产生设备连通性告警
     *
     * @param eqpId
     * @param isEqpConnected
     */
    private void raiseAlarm(long eqpId, boolean isEqpConnected) {
        EqpConnData eqpConnData = new EqpConnData();
        eqpConnData.setEqpId(eqpId);
        eqpConnData.setProtocolName(IotDic.ProtocolName.TCP);
        eqpConnData.setConnected(isEqpConnected);
        AlarmDataObj alarmDataObj = new AlarmDataObj(eqpConnData);
        alarmDataObj.setType(IotConstant.EQUIPMENT_CONNECTION);
        AlarmDataQueue.getInstance().add(alarmDataObj);
    }

    /**
     * 设备连通性监测任务
     */
    class EqpConnCheckTask extends TimerTask {

        @Override
        public void run() {
            //获取待检测设备
            QueryMonitorEqpParam queryParam = new QueryMonitorEqpParam();
            queryParam.setEnableMonitor(true);
            queryParam.setMonitorType(MonitorType.LISTENING.getValue());
            queryParam.setShortPro(applicationConfig.getParamValueStr("param.protocol.not.check"));
            List<EquipmentBo> equipmentBos = monitorMapper.findLongConMonitorEqp(queryParam);
            if (CollectionUtils.isEmpty(equipmentBos)) {
                return;
            }
            equipmentBos.forEach(equipmentBo -> {
                boolean isEqpConnected = ReceiveEqpCache.getInstance().isEqpConnected(equipmentBo.getId());
                raiseAlarm(equipmentBo.getId(), isEqpConnected);
            });
        }
    }

}