/**
 *
 */
package com.yeejoin.amos.iot.netty;

import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map.Entry;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.locks.ReentrantLock;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;

import org.apache.commons.collections.CollectionUtils;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import com.yeejoin.amos.iot.business.dao.mapper.MonitorMapper;
import com.yeejoin.amos.iot.business.dao.repository.IListenCommDao;
import com.yeejoin.amos.iot.business.entity.mybatis.ListenPortBo;
import com.yeejoin.amos.iot.business.enums.MonitorType;
import com.yeejoin.amos.iot.business.param.QueryListenPortParam;
import com.yeejoin.amos.iot.business.param.QueryMonitorEqpParam;
import com.yeejoin.amos.iot.business.service.intfc.IMonitorService;
import com.yeejoin.amos.iot.netty.customer.CustomerTcpServer;
import com.yeejoin.amos.iot.netty.http.HttpServer;
import com.yeejoin.amos.iot.netty.mqtt.server.MqttServer;
import com.yeejoin.amos.op.core.util.ApplicationConfig;

/**
 * netty服务器service
 *
 * @author as-youjun
 */
@Component("nettyServerService")
@Scope("singleton")
public class NettyServerService {

    /**
     * 日志记录器
     */
    private static final Logger logger = Logger.getLogger(NettyServerService.class);

    @Autowired
	private ApplicationConfig applicationConfig;

    @Autowired
    private MonitorMapper monitorMapper;

    @Autowired
    private IMonitorService monitorService;

    @Autowired
    private IListenCommDao listenCommDao;

    /**
     * 缓存tcpserver
     */
    private ConcurrentHashMap<Integer, TcpServer> tcpServerMap = new ConcurrentHashMap<>();
    
    /**
     * 缓存httpserver
     */
    private ConcurrentHashMap<Integer, HttpServer> httpServerMap = new ConcurrentHashMap<>();
    
    /**
     * 缓存mqttserver
     */
    private ConcurrentHashMap<Integer, MqttServer> mqttServerMap = new ConcurrentHashMap<>();

    /**
     * 可重入锁
     */
    private ReentrantLock lock = new ReentrantLock();
    
    @Autowired
    private CustomerTcpServer customerTcpServer;

    /**
     * 启动netty服务器
     **/
    @PostConstruct
	public void start() {
		List<ListenPortBo> listenPorts = getAllListenPorts();
		listenPorts.forEach(ListenPortBo -> {
			String type = ListenPortBo.getImpType();
			int port = ListenPortBo.getPort();
			startPortByType(type,port);//启动监听		
		});
		customerTcpServer.setPort(30000);
		customerTcpServer.setApplicationConfig(applicationConfig);
		customerTcpServer.start();
	}
/**
 * 启动监听
 * @param type
 * @param port
 */
	private void startPortByType(String type, int port) {
		if (type.toUpperCase().equals("TCP")) {
			if (tcpServerMap.get(port) == null) {
				TcpServer tcpServer = new TcpServer(applicationConfig, port);
				tcpServerMap.put(port, tcpServer);
				tcpServerMap.get(port).start();
			}
		} else if (type.toUpperCase().equals("HTTP")) {
			if (httpServerMap.get(port) == null) {
				HttpServer httpServer = new HttpServer(applicationConfig, port);
				httpServerMap.put(port, httpServer);
				httpServerMap.get(port).start();
			}
		} else if (type.toUpperCase().equals("MQTT")) {
			if (mqttServerMap.get(port) == null) {
				MqttServer mqttServer = new MqttServer(applicationConfig, port);
				mqttServerMap.put(port, mqttServer);
				mqttServerMap.get(port).start();
			}
		} else {
			logger.error("Only support MQTT,HTTP,TCP");
		}	
		
	}

	/**
     * 获取监听端口,待测试完成后修改名称删除
     *
     * @return
     */
//    private Set<Integer> getListenPorts() {
//        Set<Integer> ports = new HashSet<>();
//        //1.从设备模板中取监听端口
//        List<ListenComm> listenComms =
//                listenCommDao.findAll();
//        if (CollectionUtils.isEmpty(listenComms)) {
//            logger.warn("listenComms is empty!");
//            return ports;
//        }
//        List<Long> commDicIds = new ArrayList<>();
//        listenComms.forEach(listenComm -> {
//            commDicIds.add(listenComm.getCommDicId());
//        });
//        QueryListenPortParam param = new QueryListenPortParam();
//        param.setDicIds(commDicIds);      
//        List<ListenPortBo> listenPortBosFromEqpTmpl = monitorMapper.findListenPortBo(param);
//        if (CollectionUtils.isEmpty(listenPortBosFromEqpTmpl)) {
//            logger.info("listenPortBosFromEqpTmpl is empty!");
//        } else {
//            listenPortBosFromEqpTmpl.forEach(tcpPortBo -> {
//                ports.add(tcpPortBo.getPort());
//            });
//        }
//        //2.从真实设备取监听端口
//        QueryMonitorEqpParam queryParam = new QueryMonitorEqpParam();
//        queryParam.setEnableMonitor(true);
//        queryParam.setMonitorType(MonitorType.LISTENING.getValue());
//        List<ListenPortBo> listenPortBosFromEquipment = monitorService.findTcpPortBo(queryParam);
//        if (CollectionUtils.isEmpty(listenPortBosFromEquipment)) {
//				logger.debug("listenPortBosFromEquipment is empty!");
//        } else {
//            listenPortBosFromEquipment.forEach(tcpPortBo -> {
//                ports.add(tcpPortBo.getPort());
//            });
//        }
//        return ports;
//    }

    /**
     * 获取监听端口
     *
     * @param eqpIds
     * @return
     */
//    private Set<Integer> getListenPorts(List<Long> eqpIds) {
//        Set<Integer> ports = new HashSet<>();
//        //从真实设备取监听端口
//        QueryMonitorEqpParam queryParam = new QueryMonitorEqpParam();
//        queryParam.setEnableMonitor(true);
//        queryParam.setMonitorType(MonitorType.LISTENING.getValue());
//        queryParam.setEqpIds(eqpIds);
//        List<ListenPortBo> listenPortBosFromEquipment = monitorService.findTcpPortBo(queryParam);
//        if (CollectionUtils.isEmpty(listenPortBosFromEquipment)) {
//            logger.debug("listenPortBosFromEquipment is empty!");
//        } else {
//            listenPortBosFromEquipment.forEach(tcpPortBo -> {
//                ports.add(tcpPortBo.getPort());
//            });
//        }
//        return ports;
//    }    
    /**
     * 获取监听端口
     *
     * @param eqpIds
     * @return
     */
    private List<ListenPortBo> getListenPorts(List<Long> eqpIds) {
        //从真实设备取监听端口
        QueryMonitorEqpParam queryParam = new QueryMonitorEqpParam();
        queryParam.setEnableMonitor(true);
        queryParam.setMonitorType(MonitorType.LISTENING.getValue());
        queryParam.setEqpIds(eqpIds);
        List<ListenPortBo> listenPortBosFromEquipment = monitorService.findAllTcpPortBo(queryParam);
        return listenPortBosFromEquipment;
    } 
	
    /**
     * 获取所有监听端口
     */
    private List<ListenPortBo> getAllListenPorts() {
    	//1.从设备模板中取监听端口
         QueryListenPortParam param = new QueryListenPortParam();       
         List<ListenPortBo> listenPortBosFromEqpTmpl = monitorMapper.findTempListenPortBo(param);
         //2.从真实设备取监听端口
         QueryMonitorEqpParam queryParam = new QueryMonitorEqpParam();
         queryParam.setEnableMonitor(true);
         queryParam.setMonitorType(MonitorType.LISTENING.getValue());         
         List<ListenPortBo> listenPortBosFromEquipment = monitorService.findAllTcpPortBo(queryParam); 
         //3.合并查询结果集
         listenPortBosFromEqpTmpl.addAll(listenPortBosFromEquipment);
         return listenPortBosFromEqpTmpl;
	}
    /**
     * 退出系统之前执行
     */
    @PreDestroy
    public void stop() {
        this.tcpServerMap.values().forEach(tcpServer -> {
            try {
                tcpServer.stop();
            } catch (Exception e) {
                logger.error(e);
            }
        });
        this.httpServerMap.values().forEach(httpServer -> {
            try {
            	httpServer.stop();
            } catch (Exception e) {
                logger.error(e);
            }
        });
        this.mqttServerMap.values().forEach(mqttServer -> {
            try {
            	mqttServer.stop();
            } catch (Exception e) {
                logger.error(e);
            }
        });
    }

    /**
     * 全量更新netty服务:当监听端口有变化时调用此接口
     */
    public void update() {
        Thread updateTcpServerThread = new Thread(() -> {
            lock.lock();
            updateAll();
            lock.unlock();
        });
        updateTcpServerThread.setName("updateServerThreadWithAll");
        updateTcpServerThread.start();
    }

    /**
     * 全量更新
     */
//    private void updateAll() {
//        Set<Integer> newPorts = getListenPorts();
//        //1.如果旧端口被删除了,则停止相应tcp服务
//        Set<Integer> removedProts = new HashSet<>();
//        this.tcpServerMap.keySet().forEach(tcpPort -> {
//            if (!newPorts.contains(tcpPort)) {
//                try {
//                    tcpServerMap.get(tcpPort).stop();
//                } catch (Exception e) {
//                    logger.error(e);
//                } finally {
//                    removedProts.add(tcpPort);
//                }
//            }
//        });
//        //2.删除已停止的tcp服务对象
//        if (CollectionUtils.isNotEmpty(removedProts)) {
//            removedProts.forEach(removedProt -> {
//                tcpServerMap.remove(removedProt);
//            });
//        }
//        //3.运行新TCP端口服务
//        newPorts.forEach(port -> {
//            if (tcpServerMap.get(port) == null) {
//                TcpServer tcpServer = new TcpServer(applicationConfig, port);
//                tcpServerMap.put(port, tcpServer);
//                try {
//                    tcpServerMap.get(port).start();
//                } catch (Exception e) {
//                    logger.error(e);
//                }
//            }
//        });
//    }

    /**
     * 全量更新
     */
    private void updateAll() {
    	List<ListenPortBo> listenPorts = getAllListenPorts();
    	HashMap<String,Set<Integer>> portMap = new HashMap<String,Set<Integer>>();
    	Set<Integer> tcpProts = new HashSet<>();
    	Set<Integer> httpProts = new HashSet<>();
    	Set<Integer> mqttProts = new HashSet<>();  
    	//1.准备端口 	
    	listenPorts.forEach(ListenPortBo -> {
			String type = ListenPortBo.getImpType();
			int port = ListenPortBo.getPort();
			if(type.toUpperCase().equals("TCP")){
				tcpProts.add(port);
			}else if(type.toUpperCase().equals("HTTP")){
				httpProts.add(port);
			}else if(type.toUpperCase().equals("MQTT")){
				mqttProts.add(port);
			}	
		});	
    	portMap.put("TCP", tcpProts);
    	portMap.put("HTTP", httpProts);
    	portMap.put("MQTT", mqttProts);
    	//2.更新端口
    	reviseOldPortByType(portMap);
    }
    /**
     * 停止端口按照不同；类型
     * @param portMap
     */
   private void reviseOldPortByType(HashMap<String, Set<Integer>> portMap) {
		for (Entry<String, Set<Integer>> entry : portMap.entrySet()) {
			String type = entry.getKey();
			Set<Integer> newPorts = entry.getValue();
			Set<Integer> removedProts = new HashSet<>();
			if (type.toUpperCase().equals("TCP")) {
				this.tcpServerMap.keySet().forEach(tcpPort -> {
					if (!newPorts.contains(tcpPort)) {
						try {
							tcpServerMap.get(tcpPort).stop();
						} catch (Exception e) {
							logger.error(e);
						} finally {
							removedProts.add(tcpPort);
						}
					}
				});
				// 2.删除已停止的tcp服务对象
				if (CollectionUtils.isNotEmpty(removedProts)) {
					removedProts.forEach(removedProt -> {
						tcpServerMap.remove(removedProt);
					});
				}
				// 3.运行新TCP端口服务
				newPorts.forEach(port -> {
					if (tcpServerMap.get(port) == null) {
						TcpServer tcpServer = new TcpServer(applicationConfig, port);
						tcpServerMap.put(port, tcpServer);
						try {
							tcpServerMap.get(port).start();
						} catch (Exception e) {
							logger.error(e);
						}
					}
				});
			} else if (type.toUpperCase().equals("HTTP")) {
				removedProts.clear();
				this.httpServerMap.keySet().forEach(httpPort -> {
					if (!newPorts.contains(httpPort)) {
						try {
							httpServerMap.get(httpPort).stop();
						} catch (Exception e) {
							logger.error(e);
						} finally {
							removedProts.add(httpPort);
						}
					}
				});
				// 2.删除已停止的http服务对象
				if (CollectionUtils.isNotEmpty(removedProts)) {
					removedProts.forEach(removedProt -> {
						httpServerMap.remove(removedProt);
					});
				}
				// 3.运行新HTTP端口服务
				newPorts.forEach(port -> {
					if (httpServerMap.get(port) == null) {
						HttpServer httpServer = new HttpServer(applicationConfig, port);
						httpServerMap.put(port, httpServer);
						try {
							httpServerMap.get(port).start();
						} catch (Exception e) {
							logger.error(e);
						}
					}
				});
			} else if (type.toUpperCase().equals("MQTT")) {
				removedProts.clear();
				this.mqttServerMap.keySet().forEach(mqttPort -> {
					if (!newPorts.contains(mqttPort)) {
						try {
							mqttServerMap.get(mqttPort).stop();
						} catch (Exception e) {
							logger.error(e);
						} finally {
							removedProts.add(mqttPort);
						}
					}
				});
				// 2.删除已停止的mqtt服务对象
				if (CollectionUtils.isNotEmpty(removedProts)) {
					removedProts.forEach(removedProt -> {
						mqttServerMap.remove(removedProt);
					});
				}
				// 3.运行新mqtt端口服务
				newPorts.forEach(port -> {
					if (mqttServerMap.get(port) == null) {
						MqttServer mqttServer = new MqttServer(applicationConfig, port);
						mqttServerMap.put(port, mqttServer);
						try {
							mqttServerMap.get(port).start();
						} catch (Exception e) {
							logger.error(e);
						}
					}
				});
			}
		}
	   	
	}
	/**
     * 部分更新netty服务:当监听端口有变化时调用此接口:新增或者修改设备
     */
    public void update(List<Long> eqpIds) {
        Thread updateTcpServerThread = new Thread(() -> {
            lock.lock();
            List<ListenPortBo> listenPorts = getListenPorts(eqpIds);
            listenPorts.forEach(ListenPortBo -> {
    			String type = ListenPortBo.getImpType();
    			int port = ListenPortBo.getPort();
    			startPortByType(type,port);//启动监听		
    		});
            lock.unlock();
        });
        updateTcpServerThread.setName("updateServerThreadWithAddOrEdit");
        updateTcpServerThread.start();
    }

    /**
     * 删除设备时更新netty服务
     *
     * @param eqpIds
     */
    public void delete(List<Long> eqpIds) {
        eqpIds.forEach(eqpId -> {
        });
        Thread updateTcpServerThread = new Thread(() -> {
            /***
             * 删除设备时：被删除设备的端口号，可能被其他设备使用，因此不能简单地删除相应端口服务；必须全库监测已删除设备端口号
             * 在是否被其他设备使用；这样做与全量更新在性能上没有区别，因此采用全量更新方式实现删除逻辑
             */
            lock.lock();
            updateAll();
            lock.unlock();
        });
        updateTcpServerThread.setName("updateServerThreadWithDelete");
        updateTcpServerThread.start();
    }

}
