package com.yeejoin.amos.iot.netty;

import com.yeejoin.amos.iot.context.IotContext;
import com.yeejoin.amos.op.core.util.ApplicationConfig;
import org.apache.log4j.Logger;

import java.util.Date;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * 设备上报缓存:缓存设备唯一标识
 */
public class ReceiveEqpCache {

    /**
     * 日志记录器
     */
    private static final Logger logger = Logger.getLogger(ReceiveEqpCache.class);

    /**
     * 单例
     */
    private volatile static ReceiveEqpCache instance;

    /***
     * 设备id集合:缓存采集到的设备id
     */
    private Map<Long, Long> eqpIdMap = new ConcurrentHashMap<>();

    private ReceiveEqpCache() {
        //
    }

    /**
     * 获取单例
     *
     * @return
     */
    public static ReceiveEqpCache getInstance() {
        if (instance == null) {
            synchronized (ReceiveEqpCache.class) {
                if (instance == null) {
                    instance = new ReceiveEqpCache();
                }
            }
        }
        return instance;
    }

    /**
     * start
     */
    public void start() {
        //
    }

    /**
     * 缓存设备id
     *
     * @param eqpId
     */
    public void put(Long eqpId) {
        if (eqpId == null) {
			logger.debug("eqpId is null!");
            return;
        }
        eqpIdMap.put(eqpId, new Date().getTime());
    }

    /**
     * 检测设备连通性
     *
     * @param eqpId
     * @return
     */
    public boolean isEqpConnected(Long eqpId) {
        if (eqpId == null) {
            return false;
        }
        boolean isEqpConnected = false;
        if (eqpIdMap.get(eqpId) == null) {
			logger.debug("eqpIdMap.get(eqpId) = null!");
            return isEqpConnected;
        }
        long timeSpan = new Date().getTime() - eqpIdMap.get(eqpId).longValue();
        ApplicationConfig applicationConfig = (ApplicationConfig) IotContext.getInstance().getBean("applicationConfig");
        long fixedRate = Long.valueOf(applicationConfig.getParamValueStr("report.equipment.fixedRate")).longValue();
        long timeSpanLimit = fixedRate * 1000 + 30000;
        if (timeSpan <= timeSpanLimit) {
            isEqpConnected = true;
        } else {
            eqpIdMap.remove(eqpId);
        }
        return isEqpConnected;
    }

}
