package com.yeejoin.amos.iot.netty.customer;

import java.net.InetSocketAddress;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;

import com.yeejoin.amos.iot.context.IotContext;
import com.yeejoin.amos.iot.netty.customer.entity.Wapper;
import com.yeejoin.amos.iot.netty.customer.invoke.CustomerFrameQueue;
import com.yeejoin.amos.iot.netty.customer.session.SessionManager;

import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.ChannelInboundHandlerAdapter;


public class CustomerHandler  extends ChannelInboundHandlerAdapter{
	
	@Autowired
	SessionManager sessionManager;
	
	public CustomerHandler() {
		sessionManager = (SessionManager) IotContext.getInstance().getBean(SessionManager.class);
	}

	@Override
	public void channelRead(ChannelHandlerContext ctx, Object msg) throws Exception {		
		super.channelRead(ctx, msg);
		if (msg instanceof Wapper) {
			Wapper wapper = (Wapper) msg;
			receive(wapper);
			InetSocketAddress socket =  (InetSocketAddress) ctx.channel().remoteAddress();
			String clientIp = socket.toString();
			if (!sessionManager.containsSession(clientIp)) {
				sessionManager.updateTimeStamp(clientIp, ctx);
			}
			else {
				sessionManager.connect(ctx, clientIp);
			}	
		}
	}
	
	public void receive(Wapper wapper) {
		CustomerFrameQueue.getInstance().add(wapper);
	}

	/**
	 * 发生异常调用
	 */
	@Override
	public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) throws Exception {		
		super.exceptionCaught(ctx, cause);
		InetSocketAddress socket =  (InetSocketAddress) ctx.channel().remoteAddress();
		String clientIp = socket.toString();
		Logger.getLogger(this.getClass()).debug(clientIp + " is disconnected in exceptionCaught");
		if (sessionManager.containsSession(clientIp)) {
			sessionManager.removeSession(clientIp);
		}
	}



	/**
	 * channel激活调用
	 */
	@Override
	public void channelActive(ChannelHandlerContext ctx) throws Exception {
		super.channelActive(ctx);
		
		InetSocketAddress socket =  (InetSocketAddress) ctx.channel().remoteAddress();
		String clientIp = socket.toString();
		Logger.getLogger(this.getClass()).debug(clientIp + " is connected in channelActive");
		if (!sessionManager.containsSession(clientIp)) {
			sessionManager.updateTimeStamp(clientIp, ctx);
		}
		else {
			sessionManager.connect(ctx, clientIp);
		}
	}


	/**
	 * channel结束生命周期调用
	 */
	@Override
	public void channelInactive(ChannelHandlerContext ctx) throws Exception {
		super.channelInactive(ctx);
		InetSocketAddress socket =  (InetSocketAddress) ctx.channel().remoteAddress();
		String clientIp = socket.toString();
		Logger.getLogger(this.getClass()).debug(clientIp + " is disconnected in channelInactive");
		if (sessionManager.containsSession(clientIp)) {
			sessionManager.removeSession(clientIp);
		}
	}

	
}
