package com.yeejoin.amos.iot.netty.customer.coder;

import java.util.List;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;

import com.alibaba.fastjson.JSONObject;
import com.yeejoin.amos.iot.context.IotContext;
import com.yeejoin.amos.iot.netty.customer.entity.IntfcProtocol;
import com.yeejoin.amos.iot.netty.customer.entity.Wapper;
import com.yeejoin.amos.iot.netty.customer.utils.ByteConvert;
import com.yeejoin.amos.iot.netty.customer.utils.Constant;

import groovy.lang.Script;
import io.netty.buffer.ByteBuf;
import io.netty.channel.ChannelHandlerContext;
import io.netty.handler.codec.ByteToMessageDecoder;
import io.netty.handler.codec.TooLongFrameException;


public class CustomerDecoder  extends ByteToMessageDecoder{

    private final static int MIN_LENGTH = 5; // 有人协议帧最小长度

    /**
     * 最大帧长度
     */
    private final int maxFrameLength;

    public CustomerDecoder() {
        this(2048); // 2K
    }
	
    public CustomerDecoder(int maxFrameLength) {
        this.maxFrameLength = maxFrameLength;
        protocolManager = (ProtocolManager) IotContext.getInstance().getBean(ProtocolManager.class);
    }

	@Autowired
	ProtocolManager protocolManager;
	
	@Override
	protected void decode(ChannelHandlerContext ctx, ByteBuf in, List<Object> out) throws Exception {
		if (in.readableBytes() > 0) {
            if (in.readableBytes() > maxFrameLength) {
                in.skipBytes(in.readableBytes());
                throw new TooLongFrameException(
                        "buffer length exceeds " + maxFrameLength + ": " + in.readableBytes() + " bytes discarded");
            }
            // 1.获取bytebuf起始位和可读长度
            int idx = in.readerIndex();
            int readableLength = in.readableBytes();
            
            // 2.获取协议解析格式
            byte[] dst = null;
            if (readableLength < MIN_LENGTH) {
            	dst = new byte[readableLength];
            } else {
            	dst = new byte[MIN_LENGTH];
            }
            in.getBytes(idx, dst, 0, dst.length);           
            IntfcProtocol intfcProtocol = protocolManager.getIntfcProtocol(dst);
            if (intfcProtocol == null) {
            	in.skipBytes(1);
            	return;
            }          
            Script groovyScript = intfcProtocol.getScript();
            if (groovyScript == null) {
            	Logger.getLogger(this.getClass()).error("protocol decode result error, groovyScript is not defined!" );
            	in.skipBytes(1);
            	return;
            }
            byte[] src = new byte[readableLength];
            in.getBytes(idx, src, 0, src.length); 
            Object[] args = new Object[]{src};

            Wapper wapper = null;
            try {       
                Object result = groovyScript.invokeMethod("decode", args);
                if (result == null) {
                	return;
                }
                wapper  = JSONObject.parseObject(result.toString(), Wapper.class);          	
            } catch (Exception e) {
            	in.readerIndex(readableLength);
                Logger.getLogger(this.getClass()).error("protocol decode byte error!" + e);
                return;
            }

            if (!wapper.isVerification()) {
            	if (wapper.getTail() == null || wapper.getTail().length <= 0) {
            		in.skipBytes(1);
                	return;
            	}
            	in.readerIndex(wapper.getLength());
            	return;
            }
            src = new byte[wapper.getLength()];
            in.getBytes(idx, src, 0, wapper.getLength());
            wapper.setBody(src);
            JSONObject oLength = intfcProtocol.getProtocalFormat().getJSONObject("head");
            int length = oLength.getInteger("length");
            dst = new byte[length];
            in.getBytes(idx, dst, 0, dst.length);     
            wapper.setHead(dst);
           
            out.add(wapper);
            in.readerIndex(wapper.getLength());
         /*   // 3.根据协议解析格式解析报文
            if (format == null) {
            	in.skipBytes(1);
            	return;
            }
//            Wapper wapper = new Wapper();
            
            int transfeType = format.getInteger("transfeType");
            
            JSONObject formatLength = format.getJSONObject("length");
            int startField = formatLength.getInteger("startByte");
            int fieldLength = formatLength.getInteger("length");
            String propertyType = formatLength.getString("propertyType");
            
            JSONObject formatHead= format.getJSONObject("head");
            int headLength = formatHead.getInteger("length");
            
            JSONObject formatVerif= format.getJSONObject("verification");
            int VerifLength = formatVerif.getInteger("length");
            String VerifType = formatVerif.getString("propertyType");
            
            JSONObject formatTail = format.getJSONObject("tail");
            int tailLength = formatTail.getInteger("length");
            
            dst = new byte[headLength];
            in.getBytes(idx, dst, 0, headLength);
            wapper.setHead(dst);
            dst = new byte[fieldLength];
            in.getBytes(idx + startField + headLength - 1, dst, 0, fieldLength);
            wapper.setLength(dst);
            
            byte[] body = null;
            if (Constant.CUSTOMER_LENGTH_PROPERTY_MESSAGE.equals(propertyType)) {         	
            	int bodyLength = getLength(transfeType, dst);
            	if (bodyLength > readableLength) {
            		return;
            	}
            	body = new byte[bodyLength];
            	in.getBytes(idx + headLength, body, 0, bodyLength);
            	wapper.setBody(body);
            	if (VerifLength > 0) {
            		byte[] verif = new byte[VerifLength];
            		in.getBytes(idx + startField + bodyLength, verif, 0, VerifLength);
            		wapper.setVerification(verif);
            	}
            	if (tailLength > 0) {
            		byte[] tail = new byte[tailLength];
            		in.getBytes(idx + startField + bodyLength + VerifLength, tail, 0, tailLength);
            		wapper.setTail(tail);
            	}
            	
            	// 4.校验数据报文
            	byte[] message = new byte[idx + bodyLength];
            	in.getBytes(idx, message, 0, message.length);
            	if (check(wapper.getVerification(), message, transfeType, VerifType)) {
            		out.add(wapper);
            	}
            	in.readerIndex(idx + bodyLength + VerifLength + tailLength);
            }
            else if (Constant.CUSTOMER_LENGTH_PROPERTY_BODY.equals(propertyType)){
            	int bodyLength = getLength(transfeType, dst);
            	if (bodyLength > readableLength) {
            		return;
            	}
            	body = new byte[bodyLength];
            	
                JSONObject formatBody = format.getJSONObject("body");
                int startBody = formatBody.getInteger("startByte");
            	
            	in.getBytes(idx + headLength + startBody, body, 0, bodyLength);
            	wapper.setBody(body);
            	if (VerifLength > 0) {
            		byte[] verif = new byte[VerifLength];
            		in.getBytes(idx + headLength + startBody + bodyLength - VerifLength, verif, 0, VerifLength);
            		wapper.setVerification(verif);
            	}
            	if (tailLength > 0) {
            		byte[] tail = new byte[tailLength];
            		in.getBytes(idx + startBody + bodyLength + VerifLength, tail, 0, tailLength);
            		wapper.setTail(tail); 
            	}
            	
            	// 4.校验数据报文
            	byte[] message = new byte[idx + headLength +startBody + body.length - VerifLength];
            	in.getBytes(idx, message, 0, message.length);
            	if (check(wapper.getVerification(), message, transfeType, VerifType)) {
            		out.add(wapper);
            	}
            	in.readerIndex(idx + bodyLength + VerifLength + tailLength + startBody);
            }
            else {
            	in.skipBytes(1);
            	return;
            }*/
        }
		
	}

	private boolean check(byte[] verification, byte[] message, int type, String verType) {
		int ver = getLength(type, verification);
		if ("crc16".equals(verType)) {
			if (ver == crc16(message)) {
				return true;
			}
		}
		else if ("cs".equals(verType)) {
			if (ver == cs(message, verification.length)) {
				return true;
			}
		}
		else if ("xor".equals(verType)) {
			if (ver == xor(message, verification.length)) {
				return true;
			}
		}
		
		return false;
	}

	private int crc16(byte[] msg) {
        int CRC = 0x0000ffff;
        int POLYNOMIAL = 0x0000a001;

        int i, j;
        for (i = 0; i < msg.length; i++) {
            CRC ^= ((int) msg[i] & 0x000000ff);
            for (j = 0; j < 8; j++) {
                if ((CRC & 0x00000001) != 0) {
                    CRC >>= 1;
                    CRC ^= POLYNOMIAL;
                } else {
                    CRC >>= 1;
                }
            }
        }
		return CRC;
	}
	
	private int cs(byte[] msg, int length) {
		int CS = 0;
		for (byte b : msg) {
			CS += b;
		}
		int trans = 4 - length;
		CS = (0xffffffff >> trans) & CS;
		return CS;
	}
	
	private int xor(byte[] msg, int length) {
		int XOR = 0;
		for (byte b : msg) {
			XOR ^= b;
		}
		int trans = 4 - length;
		XOR = (0xffffffff >> trans) & XOR;
		return XOR;
	}
	
	private int getLength(int type, byte[] src) {
		 if (Constant.NET_MESSAGE_FORMAT_BIG == type) {
			 return ByteConvert.Bytes2Int_BE(src);
		 }
		 else  if (Constant.NET_MESSAGE_FORMAT_lITTLE == type) {
			 return ByteConvert.Bytes2Int_LE(src);
		 }
		return 0;
	}
	
}
