package com.yeejoin.amos.iot.netty.customer.coder;

import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import javax.annotation.PostConstruct;

import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;
import org.bouncycastle.pqc.math.linearalgebra.ByteUtils;
import org.codehaus.groovy.control.CompilationFailedException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;
import org.springframework.util.ObjectUtils;

import com.alibaba.fastjson.JSONObject;
import com.yeejoin.amos.iot.business.entity.IntfcProtocolVo;
import com.yeejoin.amos.iot.business.service.intfc.IInfcProtocolService;
import com.yeejoin.amos.iot.netty.customer.entity.IntfcProtocol;

import groovy.lang.GroovyShell;
import groovy.lang.Script;

@Component
@Scope("singleton")
public class ProtocolManager {
	
	private static LinkedList<JSONObject> protocalFormats = new LinkedList<>();
	private static ConcurrentHashMap<String, IntfcProtocol> protocals = new ConcurrentHashMap<>();
    /**
     * 缓存通过GroovyShell的parse方法生成的Script实例，避免相同脚本内容生成多个Script引起加载类内存溢出
     * 每次parse都会生成一个新的Script，而一个Script相当于一个新类
     */
    private final static Map<String, Script> scriptClassCach = new ConcurrentHashMap<>();
	
	@Autowired
	IInfcProtocolService infcProtocolService;
	
    @PostConstruct
	public void start() {
    	List<IntfcProtocolVo> list = infcProtocolService.getAllInfcProtocol();
    	  	
    	for (IntfcProtocolVo vo: list) {
    		IntfcProtocol intfcProtocol = new IntfcProtocol();
    		JSONObject formate = JSONObject.parseObject(vo.getProtocalFormat());
    		JSONObject head = formate.getJSONObject("head");
    		String propertyValue = head.getString("propertyValue");
    		
    		intfcProtocol.setProtocalFormat(formate);
    		
    		String scriptText = vo.getDecodeScript();
    		if (!StringUtils.isNotEmpty(scriptText)) {
    			protocals.put(propertyValue, intfcProtocol);
    			continue;
    		}
    		try {
				Script groovyScript = findOrCompile(scriptText);
				intfcProtocol.setScript(groovyScript);
				protocals.put(propertyValue, intfcProtocol);
			} catch (Exception e) {
				Logger.getLogger(this.getClass()).error("parse protocol data error!" + e);
			}
    	}
	}
    
    /**
     * 从缓存中获取已有Script或者编译新的Script
     *
     * @param scriptText
     * @return
     * @throws CompilationFailedException
     * @throws InstantiationException
     * @throws IllegalAccessException
     */
    private static Script findOrCompile(String scriptText) throws CompilationFailedException, InstantiationException, IllegalAccessException {
        String scriptTextHashCode = String.valueOf(scriptText.hashCode());
        if (!scriptClassCach.containsKey(scriptTextHashCode)) {
            GroovyShell shell = new GroovyShell();
            Script groovyScript = shell.parse(scriptText).getClass().newInstance();
            scriptClassCach.put(scriptTextHashCode, groovyScript);
        }
        return scriptClassCach.get(scriptTextHashCode);
    }
    
    /**
     * 判断接口是否支撑该协议头
     * @param sync
     * @return
     */
    public static boolean containHead(byte[] sync)
    {
    	boolean isOk = false;
    	
    	if (ObjectUtils.isEmpty(sync)) {
    		return isOk;
    	}
    	
    	for (JSONObject protocal : protocalFormats) {
    		JSONObject oLength = protocal.getJSONObject("head");
    		String propertyValue = oLength.getString("propertyValue");
    		int length = oLength.getInteger("length");
    		
    		byte[] head = new byte[length];
    		if (sync.length < length) {
    			continue;
    		}
    		
    		System.arraycopy(sync, 0, head, 0, length);
    		
    		String strHead = ByteUtils.toHexString(head);
    		if (propertyValue.equals(strHead)) {
    			isOk = true;
    			return isOk;
    		}
    	}
 
    	return isOk;
    }
    
    
    /**
     * 获取报文解析格式
     * @param sync
     * @return
     */
    public static IntfcProtocol getIntfcProtocol(byte[] sync) {
    	if (ObjectUtils.isEmpty(sync)) {
    		return null;
    	}
    	//遍历map中的键 
    	for (Map.Entry<String, IntfcProtocol> entry : protocals.entrySet()) { 
    		JSONObject format = entry.getValue().getProtocalFormat().getJSONObject("head");
    		String propertyValue = format.getString("propertyValue");
    		int length = format.getInteger("length");
    		
    		byte[] head = new byte[length];
    		if (sync.length < length) {
    			continue;
    		}
    		System.arraycopy(sync, 0, head, 0, length);
    		
    		String strHead = ByteUtils.toHexString(head);
    		if (propertyValue.equals(strHead)) {
    			return entry.getValue();
    		}
    	} 
    	
    	return null;
    }
}
