package com.yeejoin.amos.iot.netty.customer.invoke;

import java.util.concurrent.BlockingQueue;
import java.util.concurrent.LinkedBlockingQueue;

import org.apache.log4j.Logger;

import com.yeejoin.amos.iot.business.constants.IotConstant;
import com.yeejoin.amos.iot.context.IotContext;
import com.yeejoin.amos.iot.netty.customer.entity.CustomerData;
import com.yeejoin.amos.iot.netty.customer.service.intfc.ICustomerDataService;
import com.yeejoin.amos.iot.netty.vehicle.dic.VehicleDic.CommonDic;

public class CustomerDataQueue {
	/**
	 * 单例
	 */
	private static CustomerDataQueue instance;

	/**
	 * 帧队列
	 */
	private static final BlockingQueue<CustomerData> blockingQueue = new LinkedBlockingQueue<CustomerData>();

	/**
	 * 控制帧消费者线程运行状态
	 */
	private static boolean isRunning = true;

	/**
	 * 构造方法
	 */
	private CustomerDataQueue() {
		Logger.getLogger(CommonDic.class).debug(IotConstant.CONSTRUCTOR);
	}

	/**
	 * 获取单例
	 * 
	 * @return
	 */
	public static CustomerDataQueue getInstance() {
		if (instance == null) {
			synchronized (CustomerDataQueue.class) {
				if (instance == null) {
					instance = new CustomerDataQueue();
				}
			}
		}
		return instance;
	}

	/**
	 * 添加数据
	 * 
	 * @param data
	 */
	public void add(CustomerData data) {
		while (blockingQueue.add(data)) {
			break;
		}
	}

	public void start() {
		Thread thread = new Thread(new VehicleDataConsumer(),
				"customer data consumer thread");
		thread.start();
	}

	/**
	 * 帧消费者线程
	 * 
	 * @author as-youjun
	 *
	 */
	class VehicleDataConsumer implements Runnable {

		/**
		 * 1.从队列中取出帧；2.调用业务接口；
		 */
		@Override
		public void run() {
			while (isRunning) {
				// 1.从队列中取出帧
				CustomerData data = null;
				try {
					data = blockingQueue.take();
				} catch (InterruptedException e) {
					Logger.getLogger(this.getClass()).error(e);
				}
				// 2.调用业务接口；
				if (data == null) {
					Logger.getLogger(this.getClass()).warn(
							"customerData is empty!");
				} else {
					ICustomerDataService customerDataService = (ICustomerDataService) IotContext
							.getInstance().getBean("customerDataService");
					boolean result = customerDataService.process(data);
					if (!result) {
						// 失败数据入库
						Logger.getLogger(this.getClass()).warn("failure...!");
					}
				}
			}
		}
	}
}
