package com.yeejoin.amos.iot.netty.customer.invoke;

import java.util.concurrent.ConcurrentHashMap;

import org.apache.log4j.Logger;

import com.yeejoin.amos.iot.business.constants.IotConstant;
import com.yeejoin.amos.iot.business.queue.AlarmDataObj;
import com.yeejoin.amos.iot.business.queue.AlarmDataQueue;
import com.yeejoin.amos.iot.business.queue.ChannelAlarmData;
import com.yeejoin.amos.iot.netty.vehicle.VehicleChannelCache;
import com.yeejoin.amos.iot.netty.vehicle.VehicleFrameCache;

public class CustomerFrameCache {
	/**
	 * 单例
	 */
	private static CustomerFrameCache instance;
	/**
	 * 记录正常和异常报文：<frameType,sid>
	 */
	private ConcurrentHashMap<String, String> customerDeviceMap = new ConcurrentHashMap<>();
	private static final String NA = "NA";

	/**
	 * 构造方法
	 */
	private CustomerFrameCache() {
		Logger.getLogger(VehicleFrameCache.class).debug(IotConstant.CONSTRUCTOR);
	}

	/**
	 * 获取单例
	 * 
	 * @return
	 */
	public static CustomerFrameCache getInstance() {
		if (instance == null) {
			synchronized (VehicleChannelCache.class) {
				if (instance == null) {
					instance = new CustomerFrameCache();
				}
			}
		}
		return instance;
	}
	/**
	 * 正常报文
	 * @param sId
	 * @param frame
	 */
	public void normalFrame(String sId,String frameType,Object type) {
		String key= type == null?(sId+"#"+frameType+NA):(sId+"#"+frameType+type);
		String f = customerDeviceMap.get(key);
		if(null != f){
			//缓存新的通道
			customerDeviceMap.put(key,sId);
			ChannelAlarmData channelAlarmData = new ChannelAlarmData();
			channelAlarmData.setsId(sId);
			channelAlarmData.setType(IotConstant.NORMAL);
			AlarmDataObj alarmDataObj = new AlarmDataObj(channelAlarmData);
			alarmDataObj.setType(IotConstant.FRAME_EXECPTION);
			//设备通道建立清除告警
			AlarmDataQueue.getInstance().add(alarmDataObj);
			customerDeviceMap.remove(key);
		}else{
			Logger.getLogger(VehicleChannelCache.class).info("normal  sId:"+sId);
		}
	}
	/**
	 * 异常报文
	 * @param sId
	 * @param frame
	 */
	public void unnormalFrame(String sId,String frameType,Object type){
		    String key= type == null?(sId+"#"+frameType+NA):(sId+"#"+frameType+type);
		    customerDeviceMap.put(key,sId);
			ChannelAlarmData channelAlarmData = new ChannelAlarmData();
			channelAlarmData.setsId(sId);
			channelAlarmData.setType(IotConstant.UNNORMAL);
			AlarmDataObj alarmDataObj = new AlarmDataObj(channelAlarmData);
			alarmDataObj.setType(IotConstant.FRAME_EXECPTION);
			AlarmDataQueue.getInstance().add(alarmDataObj);
	}
}
