package com.yeejoin.amos.iot.netty.customer.invoke;

import java.util.concurrent.BlockingQueue;
import java.util.concurrent.LinkedBlockingQueue;

import org.apache.log4j.Logger;

import com.yeejoin.amos.iot.business.constants.IotConstant;
import com.yeejoin.amos.iot.context.IotContext;
import com.yeejoin.amos.iot.netty.customer.entity.Wapper;
import com.yeejoin.amos.iot.netty.customer.service.intfc.ICustomerFrameService;
import com.yeejoin.amos.iot.netty.vehicle.VehicleFrameQueue;
import com.yeejoin.amos.iot.netty.vehicle.dic.VehicleDic.CommonDic;

public class CustomerFrameQueue {
	/**
	 * 单例
	 */
	private static CustomerFrameQueue instance;

	/**
	 * 帧队列
	 */
	private static final BlockingQueue<Wapper> blockingQueue = new LinkedBlockingQueue<Wapper>();

	/**
	 * 控制帧消费者线程运行状态
	 */
	private static boolean isRunning = true;

	/**
	 * 构造方法
	 */
	private CustomerFrameQueue() {
		Logger.getLogger(CommonDic.class).debug(IotConstant.CONSTRUCTOR);
	}

	/**
	 * 获取单例
	 * 
	 * @return
	 */
	public static CustomerFrameQueue getInstance() {
		if (instance == null) {
			synchronized (VehicleFrameQueue.class) {
				if (instance == null) {
					instance = new CustomerFrameQueue();
				}
			}
		}
		return instance;
	}

	/**
	 * 添加数据
	 * 
	 * @param frame
	 */
	public void add(Wapper frame) {
		while (blockingQueue.add(frame)) {
			break;
		}
	}

	public void start() {
		Thread thread = new Thread(new CustomerFrameConsumer(), "Customer frame thread");
		thread.start();
	}

	/**
	 * 帧消费者线程
	 *
	 */
	class CustomerFrameConsumer implements Runnable {

		/**
		 * 1.从队列中取出帧；2.调用业务接口；
		 */
		@Override
		public void run() {
			while (isRunning) {
				// 1.从队列中取出帧
				Wapper frame = null;
				try {
					frame = blockingQueue.take();
				} catch (InterruptedException e) {
					Logger.getLogger(this.getClass()).error(e);
				}
				// 2.调用业务接口；
				if (frame == null) {
					Logger.getLogger(this.getClass()).warn("frame is empty!");
				} else {
					try {
						ICustomerFrameService customerFrameService = (ICustomerFrameService) IotContext.getInstance()
								.getBean(("customerFrameService"));
						customerFrameService.parseData(frame);
					} catch (Exception e) {
						Logger.getLogger(this.getClass()).error(e);
						// 失败处理
					}
				}
			}
		}

	}
}
