package com.yeejoin.amos.iot.netty.customer.service.impl;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.collections.CollectionUtils;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.yeejoin.amos.iot.business.constants.IotConstant;
import com.yeejoin.amos.iot.business.dao.mapper.Capacity_Intfc_protocolMapper;
import com.yeejoin.amos.iot.business.dao.mapper.DeviceMapper;
import com.yeejoin.amos.iot.business.dao.mapper.MonitorMapper;
import com.yeejoin.amos.iot.business.dao.repository.IAlarmDao;
import com.yeejoin.amos.iot.business.dao.repository.ICapabilityDao;
import com.yeejoin.amos.iot.business.dao.repository.ICapabilityPropertyDao;
import com.yeejoin.amos.iot.business.dao.repository.IEqpTmplPropertyDao;
import com.yeejoin.amos.iot.business.dao.repository.IEquipmentDao;
import com.yeejoin.amos.iot.business.entity.Alarm;
import com.yeejoin.amos.iot.business.entity.Capability;
import com.yeejoin.amos.iot.business.entity.CapabilityAttribute;
import com.yeejoin.amos.iot.business.entity.Equipment;
import com.yeejoin.amos.iot.business.entity.MetricData;
import com.yeejoin.amos.iot.business.entity.MetricDataRo;
import com.yeejoin.amos.iot.business.entity.mybatis.CapacityPropertyDataVo;
import com.yeejoin.amos.iot.business.entity.mybatis.EqpNodeSubGraphBo;
import com.yeejoin.amos.iot.business.entity.mybatis.MetricDataVo;
import com.yeejoin.amos.iot.business.queue.AlarmDataObj;
import com.yeejoin.amos.iot.business.queue.AlarmDataQueue;
import com.yeejoin.amos.iot.business.service.impl.DeviceServiceHelper;
import com.yeejoin.amos.iot.business.service.intfc.IMetricDataService;
import com.yeejoin.amos.iot.core.util.DateJsonValueProcessor;
import com.yeejoin.amos.iot.mongodb.service.intfc.IEquipMetricDataService;
import com.yeejoin.amos.iot.netty.ReceiveEqpCache;
import com.yeejoin.amos.iot.netty.customer.entity.CustomerData;
import com.yeejoin.amos.iot.netty.customer.service.intfc.ICustomerDataService;
import com.yeejoin.amos.iot.rule.engine.service.IRuleEngineService;
import com.yeejoin.amos.iot.socket.websocket.service.IWebSocketService;
import com.yeejoin.amos.iot.socket.websocket.vo.RealTimeMetricDataVo;

import net.sf.json.JsonConfig;

@Service("customerDataService")
public class CustomerDataServiceImpl implements ICustomerDataService{

	  /**
     * 日志记录器
     */
    private static final Logger logger = Logger
            .getLogger(CustomerDataServiceImpl.class);

    @Autowired
    private MonitorMapper processMapper;


    @Autowired
    private IMetricDataService metricDataService;

    
    @Autowired
    IEqpTmplPropertyDao eqpTmplPropertyDao;


    @Autowired
    private IEquipmentDao equipmentDao;

    @Autowired
    private ICapabilityDao capabilityDao;
    @Autowired
    private IAlarmDao alarmDao;
    @Autowired
    private ICapabilityPropertyDao capabilityPropertyDao;

    @Autowired
    private IEquipMetricDataService equipMetricDataService;

    @Autowired
    private IWebSocketService webSocketService;
    @Autowired
    private IWebSocketService webSocketMorphicService;
    @Autowired
    private DeviceMapper deviceMapper;

    @Autowired
    private IRuleEngineService ruleEngineService;
    @Autowired
    private Capacity_Intfc_protocolMapper capacity_Intfc_protocolMapper;
    
    /**
     * 线程池
     */
    private ExecutorService threadPool = Executors.newCachedThreadPool();
	
    /**
     * 数据处理逻辑：1.关联设备/设备模板/能力/接口；2.数据入库；3.产生告警
     */
	@Override
	public boolean process(CustomerData customerData) {

		// 1.根据报文帧头， 命令码，设备编码，获取获取摸版能力属性列表
		List<MetricDataVo> metrices = capacity_Intfc_protocolMapper.findMetricDatas(customerData);
		if (metrices == null || metrices.size() <= 0) {
			return false;
		}
		MetricData metricData = new MetricData();
		try {
			BeanUtils.copyProperties(metricData, metrices.get(0));
		} catch (IllegalAccessException e1) {
			logger.error("get metricData error!" + e1.getMessage());
		} catch (InvocationTargetException e1) {
			logger.error("get metricData error!" + e1.getMessage());
		}
		
		// 2.根据能力属性组装设备指标数据
		JSONObject json = JSONObject.parseObject(customerData.getFrameJson());
		JSONObject resultJson = new JSONObject();
		for (MetricDataVo data: metrices) {
			if (json.containsKey(data.getMetricName())) {
				resultJson.put(data.getMetricName(), json.get(data.getMetricName()));
			}
		}
		if (resultJson.isEmpty()) {
			return true;
		}
		
		metricData.setMetricJson(resultJson.toJSONString());
		metricData.setExeTime(customerData.getTime());
        //缓存上报设备id,以用于设备连通性检测
        ReceiveEqpCache.getInstance().put(metricData.getEqpId());
        // 3.保存指标数据到mysql
        try {
            metricDataService.saveMetricData(metricData);
        } catch (Exception e) {
            logger.error("saveMetricData error!" + e.getMessage());
           
        }
         // 4.指标数据保存到mongodb

        try {
            equipMetricDataService.saveMetricData2MongoDb(metricData, customerData.getSourceID());
        } catch (Exception e) {
            logger.error("saveMetricData mongodb error!" +e.getMessage());
        }

        // 5.产生告警(告警由规则引擎实现...)
        threadPool.execute(new RaiseAlarmTask(metricData));
        // 6.向前段推送设备指标数据websocket消息
        threadPool.execute(new SendMetricMessageTask(metricData));
        // 7.向前端组态组件推送数据
        threadPool.execute(new SendMorphiMessageTask(metricData));
        // 8.触发性能数据规则
        threadPool.execute(new TriggerRuleTask(metricData));
		return true;
	}


	/**
     * 向前端推送websocket消息
     *
     * @param metricData
     */
    public void sendWebSocketMessage(MetricData metricData) {
        Capability capability = capabilityDao.findById(metricData.getCpblId()).get();
        RealTimeMetricDataVo realTimeMetricDataVo = buildRealTimeMetricDataVo(
                metricData, capability);
        Map<String, Class<?>> map = new HashMap<String, Class<?>>();
        map.put("time", Date.class);
        JsonConfig jsonConfig = new JsonConfig();
        jsonConfig.registerJsonValueProcessor(Date.class,
                new DateJsonValueProcessor(IotConstant.DATE_TIME_STR_YMDHMS));
        String jsonData = net.sf.json.JSONObject.fromObject(realTimeMetricDataVo,
                jsonConfig).toString();
        List<EqpNodeSubGraphBo> eqpNodeSubGraphBos = processMapper
                .findEqpNodeGraphByEqpId(metricData.getEqpId());
        if (CollectionUtils.isEmpty(eqpNodeSubGraphBos)) {
            logger.warn("eqpNodeSubGraphBos is empty! no need send message!");
            return;
        }
        List<Long> subGraphIds = new ArrayList<Long>();
        eqpNodeSubGraphBos.forEach(eqpNodeSubGraphBo -> {
            if (!subGraphIds.contains(eqpNodeSubGraphBo.getSubGraphId())) {
                subGraphIds.add(eqpNodeSubGraphBo.getSubGraphId());
            }
        });
        webSocketService.sendMetricDataMessage(subGraphIds, jsonData,
                IotConstant.METRIC_SOCKET);
    }
    
    /**
     * 构造RealTimeMetricDataVo
     *
     * @param metricData
     * @param capability
     * @return
     */
    private RealTimeMetricDataVo buildRealTimeMetricDataVo(
            MetricData metricData, Capability capability) {
        RealTimeMetricDataVo realTimeMetricDataVo = new RealTimeMetricDataVo();
        realTimeMetricDataVo.setEqpId(metricData.getEqpId());
        realTimeMetricDataVo.setCpblId(metricData.getCpblId());
        realTimeMetricDataVo.setTime(metricData.getExeTime());
        realTimeMetricDataVo.setCpblName(capability.getName());
        List<CapacityPropertyDataVo> capabilityAttributes = deviceMapper
                .getCapacityPropertyById(metricData.getCpblId());
        realTimeMetricDataVo.setMetricDataMap(DeviceServiceHelper
                .buildCpblAttrData(metricData.getMetricJson(),
                        capabilityAttributes));
        return realTimeMetricDataVo;
    }

	  /**
     * 处理告警
     *
     * @param metricData
     */
    public void raiseAlarm(MetricData metricData) {
        AlarmDataObj alarmDataObj = new AlarmDataObj(metricData);
        alarmDataObj.setType(IotConstant.METRIC);
        AlarmDataQueue.getInstance().add(alarmDataObj);
        logger.info("指标越界告警源数据,sid:" + metricData.getEqpId());
    }

    /**
     * 向前端组态推送websocket消息
     *
     * @param metricData
     */
    public void sendMorphicWebSocketMessage(MetricData metricData) {
        Equipment equipment = equipmentDao.findById(metricData.getEqpId()).get();
        Map<String, Object> resultMap = new HashMap<>();
        Map<String, List<RealTimeMetricDataVo>> realTimeMetricDataVoMap = new HashMap<>();
        List<RealTimeMetricDataVo> realTimeMetricDataVoList = new ArrayList<>();
        Capability capability = capabilityDao.findById(metricData.getCpblId()).get();
        RealTimeMetricDataVo realTimeMetricDataVo = new RealTimeMetricDataVo();
        realTimeMetricDataVo.setEqpId(metricData.getEqpId());
        realTimeMetricDataVo.setCpblId(metricData.getCpblId());
        realTimeMetricDataVo.setCpblName(capability.getName());
        realTimeMetricDataVo.setTime(new Date());

        Map<String, Object> map = new HashMap<>();
        map.put("eqpId", metricData.getEqpId());
        map.put("cpblId", metricData.getCpblId());
        Map<String, String> paramMap = new HashMap<>();
        List<CapabilityAttribute> capabilityAttributes = capabilityPropertyDao
                .findByCapabilityId(metricData.getCpblId());
        if (metricData != null && metricData.getMetricJson() != null && metricData.getMetricJson() != "") {
            paramMap = DeviceServiceHelper.buildCpblAttrDatas(metricData.getMetricJson(), capabilityAttributes);
        } else {
            for (CapabilityAttribute capacityPropertyDataVo : capabilityAttributes) {
                paramMap.put(capacityPropertyDataVo.getDisplayName(), "");
            }
        }
        realTimeMetricDataVo.setMetricDataMap(paramMap);
        realTimeMetricDataVoList.add(realTimeMetricDataVo);
        realTimeMetricDataVoMap.put(metricData.getEqpId().toString(), realTimeMetricDataVoList);

        resultMap.put("metricData", realTimeMetricDataVoMap);
        resultMap.put("isWarning", equipment.getIsWarning());
        // 计算告警最高等级
        List<Alarm> alarms = alarmDao.fetchAlarmsByRqpId(equipment.getId());
        if (alarms != null) {
            Integer maxLevel = alarms.stream().mapToInt(i -> i.getLevel()).summaryStatistics().getMin();
            resultMap.put("maxLevel", maxLevel);
        }
        String jsonData = JSON.toJSONString(resultMap, SerializerFeature.WriteMapNullValue);
//        System.out.println("组态：" + jsonData);
        //发送结果
        webSocketMorphicService.sendMetricDataMessage(jsonData, IotConstant.MORPHIC_SOCKET);
    }
	
	   /**
     * 告警处理
     */
    class RaiseAlarmTask implements Runnable {

        private MetricData metricDatas;

        public RaiseAlarmTask(MetricData metricDatas) {
            this.metricDatas = metricDatas;
        }

        @Override
        public void run() {
            // 产生告警(告警由规则引擎实现...)
            raiseAlarm(metricDatas);
        }
    }

    /**
     * 指标数据推送到前端
     */
    class SendMetricMessageTask implements Runnable {

        private MetricData metricDatas;

        public SendMetricMessageTask(MetricData metricDatas) {
            this.metricDatas = metricDatas;
        }

        @Override
        public void run() {
            // 向前段推送设备指标数据websocket消息
            try {
                sendWebSocketMessage(metricDatas);
            } catch (Exception e) {
                logger.error("sendWebSocketMessage error!" + e);
            }

        }
    }

    /**
     * 向前端组态组件推送数据
     */
    class SendMorphiMessageTask implements Runnable {

        private MetricData metricDatas;

        public SendMorphiMessageTask(MetricData metricDatas) {
            this.metricDatas = metricDatas;
        }

        @Override
        public void run() {
            //向前端组态组件推送数据
            try {
                sendMorphicWebSocketMessage(metricDatas);
            } catch (Exception e) {
                logger.error("sendMorphicWebSocketMessage error!" + e);
            }
        }
    }

    /**
     * 规则执行
     */
    class TriggerRuleTask implements Runnable {

        private MetricData metricDatas;

        public TriggerRuleTask(MetricData metricDatas) {
            this.metricDatas = metricDatas;
        }

        @Override
        public void run() {
            try {
            	List<MetricData> list = new ArrayList<>();
            	list.add(metricDatas);
                MetricDataRo metricDataRo = new MetricDataRo(list);
                Equipment equipment = equipmentDao.findById(metricDatas.getEqpId()).get();
                metricDataRo.setEqpId(metricDatas.getEqpId());
                metricDataRo.setSubgraphId(metricDatas.getSubgraphId());
                metricDataRo.setOrgCode(equipment.getOrgCode());
                ruleEngineService.fireRule(metricDataRo);
            } catch (Exception e) {
                logger.error(e);
            }
        }
    }
}
