package com.yeejoin.amos.iot.netty.customer.service.impl;

import java.util.Date;

import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;
import org.bouncycastle.pqc.math.linearalgebra.ByteUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

import com.yeejoin.amos.iot.business.constants.IotConstant;
import com.yeejoin.amos.iot.business.dao.repository.IFrameProtocolDao;
import com.yeejoin.amos.iot.business.dic.ScriptLang;
import com.yeejoin.amos.iot.business.entity.Dic;
import com.yeejoin.amos.iot.business.entity.FrameProtocol;
import com.yeejoin.amos.iot.business.service.intfc.IDicService;
import com.yeejoin.amos.iot.business.service.intfc.IProtocolService;
import com.yeejoin.amos.iot.business.vo.ProtocolVo;
import com.yeejoin.amos.iot.core.util.DateUtil;
import com.yeejoin.amos.iot.entity.ParseResult;
import com.yeejoin.amos.iot.netty.customer.entity.CustomerData;
import com.yeejoin.amos.iot.netty.customer.entity.Wapper;
import com.yeejoin.amos.iot.netty.customer.invoke.CustomerDataQueue;
import com.yeejoin.amos.iot.netty.customer.invoke.CustomerFrameCache;
import com.yeejoin.amos.iot.netty.customer.service.intfc.ICustomerFrameService;
import com.yeejoin.amos.iot.netty.customer.utils.ByteConvert;
import com.yeejoin.amos.iot.script.IScriptService;

import net.sf.json.JSONObject;

@Service("customerFrameService")
public class CustomerFrameServiceImpl implements ICustomerFrameService {
    @Autowired
    private IProtocolService protocolService;

    @Autowired
    private IDicService dicService;

    @Autowired
    private IScriptService groovyScriptService;

    @Autowired
    private IScriptService javaScriptService;
    @Autowired
    private IFrameProtocolDao frameProtocolDao;
	
	@Override
	public void parseData(Wapper wapper) {
        if (wapper == null) {
            return;
        }
        CustomerData customerData = new CustomerData();
        String sid = wapper.getSid();
        String type = wapper.getType();

        if (!StringUtils.isNotEmpty(sid) && !StringUtils.isNotEmpty(type)) {
        	return;
        }
        
        // 2.调用groovy/javascript代码解析帧体中的数据
        FrameProtocol frameProtocol = frameProtocolDao.findByFrameTypeAndCommand(ByteConvert.Bytes2Int_BE(wapper.getHead()), type);
        if(frameProtocol == null){
            return;
        }
        Long protocolId = frameProtocol.getProtocolId();
        if (ObjectUtils.isEmpty(protocolId)) {
            return;
        }
        customerData.setCommand(protocolId);
        
        ProtocolVo protocolVo = protocolService.findById(protocolId);
        Dic dic = dicService.getById(String.valueOf(protocolVo.getDicIdScriptLang()));
        // 调用groovy/javascript代码解析协议数据
        Object[] args = new Object[]{wapper.getBody(), sid};
        ParseResult parseResult = null;
        switch (dic.getCode()) {
            case ScriptLang.GROOVY:
                // groovy解析协议数据
                parseResult = groovyScriptService.analyze(protocolVo.getScriptCode(), args);
                break;
            case ScriptLang.JAVASCRIPT:
                // javascript解析协议数据
                parseResult = javaScriptService.analyze(protocolVo.getScriptCode(), args);
                break;
            case ScriptLang.PYTHON:
                // python解析协议数据---暂未实现
                break;
            case ScriptLang.LUA:
                // lua解析协议数据---暂未实现
                break;
            default:
                // 不执行任何业务
                break;
        }
        String resultJson = null;

        try {
            resultJson = JSONObject.fromObject(parseResult.getResult()).toString();
        } catch (Exception e) {
            Logger.getLogger(this.getClass()).error("protocol analye result error!" + e);
            return;
        }
        if (resultJson == null) {
        	return;
        }
        if ((resultJson != null) && resultJson.contains("errorMessage")) {
        	CustomerFrameCache.getInstance().unnormalFrame(sid, ByteUtils.toHexString(wapper.getHead()), type);
            return;
        }
        
        if (StringUtils.isNotEmpty(resultJson)) {
            JSONObject json = JSONObject.fromObject(resultJson);
            
            customerData.setSync(ByteUtils.toHexString(wapper.getHead()));           
            customerData.setSourceID(json.getString("sid"));
            customerData.setFrameJson(json.toString());
            Date now = new Date();
            Date exeTime = null;
            try {
                if (StringUtils.isEmpty(json.getString("formatTime"))) {
                    exeTime = now;// 当前时间
                } else {
                    exeTime = DateUtil.str2Date(json.getString("formatTime"), IotConstant.DATE_TIME_STR_YMDHMS);
                }
            } catch (Exception e) {
                exeTime = now;// 当前时间
            }
            if (exeTime == null) {
                exeTime = now;
            }
            customerData.setTime(exeTime);
            boolean isValid = isFrameValid(json);
            // 解析结果数据放入队列
            if (isValid) {
            	CustomerDataQueue.getInstance().add(customerData);
                CustomerFrameCache.getInstance().normalFrame(sid, ByteUtils.toHexString(wapper.getHead()), type);

            } else {
                Logger.getLogger(this.getClass()).warn("isValid=fale,:" + resultJson);
            }
        }
	}
	
    /**
     * 监测帧数据是否有效
     *
     * @param json
     * @return
     */
    private boolean isFrameValid(JSONObject json) {
        boolean isValid = true;
        try {
            String status = (String) json.get("status");
            if ("信息不可用".equals(status)) {
                isValid = false;
            }
        } catch (Exception e) {
            Logger.getLogger(this.getClass()).error(e);
        }
        return isValid;
    }

}
