package com.yeejoin.amos.iot.netty.customer.session;

import java.util.Date;
import java.util.Iterator;
import java.util.concurrent.ConcurrentHashMap;

import javax.annotation.PostConstruct;

import org.apache.log4j.Logger;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import io.netty.channel.ChannelHandlerContext;

@Component
@Scope("singleton")
public class SessionManager {

	private ConcurrentHashMap<String, Session> sessions = new ConcurrentHashMap<>();
	private int MAX_INTERVAL = 1000 * 60 * 5;  //五分钟
	private boolean running = false;
    @PostConstruct
	public void start() {
    	sessionManagerThread.start();
	}
	
    /**
     * session管理线程，当长时间没有任何数据上报时断开连接
     */
    Thread sessionManagerThread = new Thread(new Runnable() {
		
		@Override
		public void run() {
			running = true;
			while(running) {
				for (Iterator<ConcurrentHashMap.Entry<String, Session>> it = sessions.entrySet().iterator(); it.hasNext();){
					ConcurrentHashMap.Entry<String, Session> item = it.next();
				    Date date = new Date();
				    long time = date.getTime();
				    // 判断上次连接和当前时间的间隔
				    if ((time - item.getValue().getRecordTime()) > MAX_INTERVAL) {
				    	Logger.getLogger(this.getClass()).warn(item.getKey() + " session timeout connect close! ");
				    	item.getValue().closeConnect();
				    	it.remove();
				    }
				}
				try {
					Thread.sleep(1000* 30);
				} catch (InterruptedException e) {
					running = false;
					e.printStackTrace();
				}
			}
			running = false;
		}
	});
    
    /**
     * 创建设备连接session
     * @param sessionId
     * @param ctx
     */
	public void createSession(String sessionId, ChannelHandlerContext ctx) {
		if (!sessions.containsKey(sessionId)) {
			Session session = new Session(sessionId, ctx);
			sessions.put(sessionId, session);
			session.refreshRecordTime();
			Logger.getLogger(this.getClass()).debug(sessionId + " session created ");
		}
	}
	
	public void connect(ChannelHandlerContext ctx, String sessionId) {
		createSession(sessionId, ctx);
	
	}
	/**
	 * 更新上次数据连接时间，如果session为空，则创建session
	 * @param sessionId
	 * @param ctx
	 */
	public void updateTimeStamp(String sessionId, ChannelHandlerContext ctx) {
		Session session = sessions.get(sessionId);
		if (session != null) {
			session.refreshRecordTime();
		}
		else {
			connect(ctx, sessionId);
		}
	}
	
	/**
	 * 移除session
	 * @param sessionId
	 */
	public void removeSession(String sessionId) {
		sessions.remove(sessionId);
	}
	
	public boolean containsSession(String sessionId) {
		return sessions.containsKey(sessionId);
	}
	
	public Session getSessionById(String sessionId) {
		return sessions.get(sessionId);
	}

	
}
