package com.yeejoin.amos.iot.netty.gateway;

import java.util.List;

import io.netty.buffer.ByteBuf;
import io.netty.channel.ChannelHandlerContext;
import io.netty.handler.codec.ByteToMessageDecoder;
import io.netty.handler.codec.TooLongFrameException;

/**
 * 有人协议解码
 * 
 * @author as-mafeipeng
 *
 */
public class UsrDecoder extends ByteToMessageDecoder {

    private final static byte START = (byte) 0x7B; // 帧起始符
    private final static byte END = (byte) 0x7B; // 帧头结束符

    private final static int MIN_LENGTH = 16; // 有人协议帧最小长度

    /**
     * 最大帧长度
     */
    private final int maxFrameLength;

    public UsrDecoder() {
        this(2048); // 2K
    }

    public UsrDecoder(int maxFrameLength) {
        this.maxFrameLength = maxFrameLength;
    }

    @Override
    protected void decode(ChannelHandlerContext ctx, ByteBuf in, List<Object> out) throws Exception {
        if (in.readableBytes() > 0) {
            if (in.readableBytes() > maxFrameLength) {
                // buffer size exceeded maxFrameLength; discarding the complete buffer.
                in.skipBytes(in.readableBytes());
                throw new TooLongFrameException(
                        "buffer length exceeds " + maxFrameLength + ": " + in.readableBytes() + " bytes discarded");
            }
            int idx = in.readerIndex();
            //find start 0x7B
            if (in.getByte(idx) == START) {
                //is buffer enough
                if (in.readableBytes() < MIN_LENGTH) {
                    return;
                }
                //length of frame
                int length = in.getUnsignedShort(idx + 2);
                if (in.readableBytes() < length) {
                    return;
                }
                //find end 0x7B
                if (in.getByte(idx + length - 1) == END) {
                    byte[] frame = extractObject(ctx, in, length);
                    in.readerIndex(idx + length);
                    out.add(frame);
                } else {
                    in.skipBytes(1);
                }
            } else {
                in.skipBytes(1);
            }
        }
    }

    /**
     * 获取一个完整的帧
     * 
     * @param ctx
     * @param in
     * @param length
     * @return
     */
    private byte[] extractObject(ChannelHandlerContext ctx, ByteBuf in, int length) {
        byte[] b = new byte[length];
        in.getBytes(in.readerIndex(), b);
        return b;
    }

}
