package com.yeejoin.amos.iot.netty.gateway;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import io.netty.channel.ChannelHandlerContext;

/**
 * 车库网关通道管理器
 * @author mafeipeng
 *
 */
public class UsrGatewayManager {
    /**
     * 通道管理器
     * key:网关序列号
     * value:通道
     */
    private static Map<String, ChannelHandlerContext> id2ctx = new ConcurrentHashMap<>();
    /**
     * 通道管理器
     * key:通道
     * value:网关序列号
     */
    private static Map<ChannelHandlerContext, String> ctx2id = new ConcurrentHashMap<>();

    /**
     * 记录通道和网关序列号关系
     * 如果新通道对应的网关在管理器中存在通道记录，则关闭旧通道，记录新通道；如果没有记录，则记录网关和通道的关系
     * @param id
     * @param ctx
     */
    public static void record(String id, ChannelHandlerContext ctx) {
        //如果该网关已有对应通道，则关闭旧的，记录新的
        ChannelHandlerContext curCtx = id2ctx.get(id);
        id2ctx.put(id, ctx);
        ctx2id.put(ctx, id);
        if (curCtx != null && curCtx != ctx) {
            curCtx.close();
        }
    }
    
    /**
     * 根据网关序号和通道信息移除通道和网关的记录
     * @param id
     * @param ctx
     */
    public static void remove(String id, ChannelHandlerContext ctx) {
        id2ctx.remove(id, ctx);
        ctx2id.remove(ctx);
    }
    
    /**
     * 根据通道信息，一处通道和网关记录
     * @param ctx
     */
    public static void remove(ChannelHandlerContext ctx) {
        String id = getId(ctx);
        if (getCtx(id) == ctx) {
            remove(id, ctx);
        } else {
            ctx2id.remove(ctx);
        }
    }
    
    /**
     * 通过通道获得网关序列号
     * @param ctx
     * @return
     */
    public static String getId(ChannelHandlerContext ctx) {
        return ctx2id.get(ctx);
    }
    
    /**
     * 通过网关序列号获取通道
     * @param id
     * @return
     */
    public static ChannelHandlerContext getCtx(String id) {
        return id2ctx.get(id);
    }
}
