package com.yeejoin.amos.iot.netty.gateway;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import net.sf.json.JSONArray;
import net.sf.json.JSONObject;

/**
 * 通过配置文件启动采集任务
 * 目前该方法被通过swagger调用接口添加任务替代
 * @author
 *
 */
@Service("usrSamplerTaskStarter")
public class UsrSamplerTaskStarter {

    @Autowired
    private UsrSamplerTaskConfig usrSamplerTaskConfig;
    
    public void start() {
        final ScheduledExecutorService service = Executors.newSingleThreadScheduledExecutor();
        Logger.getLogger(this.getClass()).debug("usrSamplerTaskConfig.TaskParam = "+usrSamplerTaskConfig.getTaskParam());
        JSONArray jsonArray = JSONArray.fromObject(usrSamplerTaskConfig.getTaskParam());
        for (Object object : jsonArray) {
            JSONObject jsonObject = (JSONObject) object;
            String gaId = jsonObject.getString("gaId");
            JSONArray commands = jsonObject.getJSONArray("commands");
            if (gaId != null && commands != null && commands.size() != 0) {
                //总周期=采集间隔*命令数
                int period = usrSamplerTaskConfig.getPeriod() * commands.size();
                for (int i = 0; i < commands.size(); i++) {
                    //按顺序启动任务，每个任务延迟启动时间=采集间隔*命令序号
                    int delay = usrSamplerTaskConfig.getPeriod() * i;
                    service.scheduleAtFixedRate(new UsrSamplerScheduleTask(gaId, commands.getString(i)), delay, period, TimeUnit.SECONDS);
                }
            }
        }
    }
    
/*    *//**
     * 网关序号
     * @return
     *//*
    public String genGaId(String id) {
        if (id == null) return null;
        if (id.length() < 11) {
            StringBuilder sb = new StringBuilder();
            sb.append(id);
            for (int i = 0; i < 11-id.length(); i++) {
                sb.append("0");
            }
            return sb.toString();
        }
        if (id.length() > 11) return id.substring(0, 11);
        return id;
    }*/
    
    private byte[] slipEscape(byte[] data) {
        List<Byte> list = new ArrayList<Byte>();
        list.add((byte)0xC0);
        for (int i = 0; i < data.length; i++) {
            if (data[i] == 0xDB) {
                list.add((byte)0xDB);
                list.add((byte)0xDD);
            } else if (data[i] == 0xC0) {
                list.add((byte)0xDB);
                list.add((byte)0xDC);
            } else {
                list.add(data[i]);
            }
        }
        list.add((byte)0xC0);
        byte[] result = new byte[list.size()];
        for (int i = 0; i < result.length; i++) {
            result[i] = list.get(i);
        }
        return result;
    }
}
