package com.yeejoin.amos.iot.netty.gateway;

import io.netty.bootstrap.ServerBootstrap;
import io.netty.channel.ChannelFuture;
import io.netty.channel.ChannelOption;
import io.netty.channel.EventLoopGroup;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.nio.NioServerSocketChannel;

import org.apache.log4j.Logger;

import com.yeejoin.amos.iot.exception.YeeException;
import com.yeejoin.amos.op.core.util.ApplicationConfig;

/**
 * 车库网关（有人网关）接入服务
 * @author mafeipeng
 *
 */
public class UsrTcpServer {

    /**
     * boss线程池
     */
    private EventLoopGroup bossLoopGroup;

    /**
     * worker线程池
     */
    private EventLoopGroup workLoogGroup;

    /**
     * ChannelFuture
     */
    private ChannelFuture vehicleChannelFuture;

    /**
     * server配置
     */
    private ApplicationConfig applicationConfig;



    /**
     * 监听端口
     */
    private int port;

    public UsrTcpServer(ApplicationConfig applicationConfig) {
        this.applicationConfig = applicationConfig;
        String tcpPortGateway = "8040";
        //String tcpPortGateway = applicationConfig.getParamValueStr("tcp.port.gateway");
        this.port = Integer.valueOf(tcpPortGateway);
    }
    
    public UsrTcpServer(ApplicationConfig applicationConfig, int port) {
        this.applicationConfig = applicationConfig;
        this.port = port;
    }

    /**
     * 运行tcp服务
     */
    public void start() {
        try {
            // 创建一个serverbootstrap实例
            ServerBootstrap serverBootstrap = new ServerBootstrap();
            // BOSS线程池
            bossLoopGroup = new NioEventLoopGroup(1);
            // CPU个数
            int processorsNumber = Runtime.getRuntime().availableProcessors();
            // worker线程池
            workLoogGroup = new NioEventLoopGroup(processorsNumber * 2);
            // 指定 boss线程池、worker线程池
            serverBootstrap.group(bossLoopGroup, workLoogGroup);
            // 指定使用NIO传输Channel
            serverBootstrap.channel(NioServerSocketChannel.class);

            // 设置option
            /***
             * BACKLOG用于构造服务端套接字ServerSocket对象，标识当服务器请求处理线程全满时，用于临时存放已完成三次握手的请求的队列的最大长度。
             * 如果未设置或所设置的值小于1，Java将使用默认值50。
             */
            //String backlog = applicationConfig.getParamValueStr("so.backlog");
            String backlog = "128";
            serverBootstrap.option(ChannelOption.SO_BACKLOG, Integer.valueOf(backlog));
            serverBootstrap.childOption(ChannelOption.SO_KEEPALIVE, true);
            // 设置连接超时时间30秒
            serverBootstrap.option(ChannelOption.CONNECT_TIMEOUT_MILLIS, 30000);
            // 设置ChannelHandler
            serverBootstrap.childHandler(new UsrChannellInitalizer());
            // 异步的绑定服务器,sync()一直等到绑定完成:绑定端口
            vehicleChannelFuture = serverBootstrap.bind(port).sync();
        } catch (Exception e) {
            Logger.getLogger(this.getClass()).error(e);
            throw new YeeException("run gateway netty server error!", e, 1004001);
        }
    }

    /**
     * 停止tcp服务
     */
    public void stop() {
        try {
            vehicleChannelFuture.channel().closeFuture().sync();
        } catch (Exception e) {
            Logger.getLogger(this.getClass()).error(e);
        } finally {
            bossLoopGroup.shutdownGracefully();
            workLoogGroup.shutdownGracefully();
        }
    }
}
