package com.yeejoin.amos.iot.netty.gateway.controller;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.yeejoin.amos.iot.business.controller.BaseController;
import com.yeejoin.amos.iot.core.util.StringUtil;
import com.yeejoin.amos.iot.netty.gateway.intfc.IGatewayService;
import com.yeejoin.amos.iot.netty.gateway.param.UsrGatewaySampleTaskParam;
import com.yeejoin.amos.op.core.common.response.CommonResponse;
import com.yeejoin.amos.op.core.util.CommonResponseUtil;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;

/**
 * 有人车库网关任务添加接口脚手架
 * @author mafeipeng
 *
 */
@RestController
@RequestMapping("/usrGateway")
@Api(value = "usrGateway", tags = { "有人网关管理" })
public class UsrGatewayController extends BaseController {

    @Autowired
    private IGatewayService gatewayService;
    
    @ApiOperation(httpMethod = "POST", value = "添加有人网关采集任务", notes = "添加有人网关采集任务")
    @RequestMapping(value = "/addSampleTask", produces = "application/json;charset=UTF-8", method = RequestMethod.POST)
    public CommonResponse addSampleTask(@ApiParam(value = "添加有人网关采集任务", required = true) @RequestBody UsrGatewaySampleTaskParam request) {
        try {
            if (request.getGatewayId() == null || request.getGatewayId().equals("")) {
                return CommonResponseUtil.failure("网关序列号不能为空");
            }
            if (request.getInterval() <= 0) {
                return CommonResponseUtil.failure("采集间隔不能为负或0");
            }
            if (request.getCommands() == null || request.getCommands().isEmpty()) {
                return CommonResponseUtil.failure("采集命令不能为空");
            }
            gatewayService.addTask(request);
            return CommonResponseUtil.success();
        } catch (Exception e) {
            Logger.getLogger(getClass()).error(e);
            return CommonResponseUtil.failure();
        }
    }
}
