package com.yeejoin.amos.iot.netty.http;


import java.util.Map;

import org.apache.log4j.Logger;

import com.yeejoin.amos.iot.context.IotContext;
import com.yeejoin.amos.iot.core.util.StringUtil;
import com.yeejoin.amos.op.core.util.ApplicationConfig;

import io.netty.buffer.ByteBuf;
import io.netty.buffer.Unpooled;
import io.netty.channel.ChannelFutureListener;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.ChannelInboundHandlerAdapter;
import io.netty.handler.codec.http.DefaultFullHttpResponse;
import io.netty.handler.codec.http.FullHttpRequest;
import io.netty.handler.codec.http.FullHttpResponse;
import io.netty.handler.codec.http.HttpMethod;
import io.netty.handler.codec.http.HttpResponseStatus;
import io.netty.handler.codec.http.HttpVersion;
import io.netty.util.CharsetUtil;
import net.sf.json.JSONObject;

public class HttpChannelHandler extends ChannelInboundHandlerAdapter {
	
	private String result="";
		 
	@Override
	public void channelRead(ChannelHandlerContext ctx, Object msg) throws Exception {		
		//1.校验请求报文是否合法
		if (!(msg instanceof FullHttpRequest)) {
			result = "未知请求!";
			send(ctx, result, HttpResponseStatus.BAD_REQUEST);
			return;
		}
        ApplicationConfig applicationConfig = (ApplicationConfig) IotContext.getInstance().getBean("applicationConfig");
		String uri = applicationConfig.getParamValueStr("report.http.uri");
		FullHttpRequest httpRequest = (FullHttpRequest)msg;
		try {
			String path = httpRequest.getUri();			
			HttpMethod method=httpRequest.getMethod();
			if (!uri.equalsIgnoreCase(path)) {
				result = "非法请求!";
				send(ctx, result, HttpResponseStatus.BAD_REQUEST);
				return;
			}
			if (!HttpMethod.POST.equals(method)) {
				result = "错误的请求方式!";
				send(ctx, result, HttpResponseStatus.BAD_REQUEST);
				return;
			}			
			ByteBuf buf = httpRequest.content();
			String json= buf.toString(CharsetUtil.UTF_8);
			Map<String, Object> valueMap = (Map<String, Object>) JSONObject.toBean(JSONObject.fromObject(json),Map.class);
			String sid =String.valueOf(valueMap.get("sourceId"));
			if(!StringUtil.isNotEmpty(sid)){
				result = "非法请求!";
				send(ctx, result, HttpResponseStatus.BAD_REQUEST);
				return;
			}
			//2.数据存放队列
			byte[]  frame = json.getBytes();	
			HttpFrameQueue.getInstance().add(frame);
			result = "succese!";
			send(ctx, result, HttpResponseStatus.OK);
		} catch (Exception e) {
			Logger.getLogger(this.getClass()).error(e);
		}finally {
			//释放请求
			httpRequest.release();
		}
		
	}

	@Override
	public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) {
		cause.printStackTrace();
		ctx.close();
	}

	
	/**
	 * 发送的返回值
	 * @param ctx	  返回
	 * @param context 消息
	 * @param status 状态
	 */
	private void send(ChannelHandlerContext ctx, String context,HttpResponseStatus status) {
		FullHttpResponse response = new DefaultFullHttpResponse(HttpVersion.HTTP_1_1, status, Unpooled.copiedBuffer(context, CharsetUtil.UTF_8));
		response.headers().set("content-type", "text/plain;charset=UTF-8");
		ctx.writeAndFlush(response).addListener(ChannelFutureListener.CLOSE);
	}
	
	/*
	 * 建立连接时，返回消息
	 */
	@Override
	public void channelActive(ChannelHandlerContext ctx) throws Exception {
		super.channelActive(ctx);
	}
	
}
