package com.yeejoin.amos.iot.netty.http;

import com.yeejoin.amos.iot.business.constants.IotConstant;
import com.yeejoin.amos.iot.context.IotContext;
import com.yeejoin.amos.iot.netty.vehicle.dic.VehicleDic.CommonDic;
import org.apache.log4j.Logger;

import java.util.concurrent.BlockingQueue;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.LinkedBlockingQueue;

public class HttpFrameQueue {

    /**
     * 单例
     */
    private static HttpFrameQueue instance;

    /**
     * 帧队列
     */
    private static final BlockingQueue<byte[]> blockingQueue = new LinkedBlockingQueue<byte[]>();

    /**
     * 控制帧消费者线程运行状态
     */
    private static boolean isRunning = true;

    //线程池大小
    private static final int poolSize = 10;

    //固定线程池
    private static ExecutorService executorService = Executors.newFixedThreadPool(poolSize);

    /**
     * 构造方法
     */
    private HttpFrameQueue() {
        Logger.getLogger(CommonDic.class).debug(IotConstant.CONSTRUCTOR);
    }

    /**
     * 获取单例
     *
     * @return
     */
    public static HttpFrameQueue getInstance() {
        if (instance == null) {
            synchronized (HttpFrameQueue.class) {
                if (instance == null) {
                    instance = new HttpFrameQueue();
                }
            }
        }
        return instance;
    }

    /**
     * 添加数据
     *
     * @param frame
     */
    public void add(byte[] frame) {
        while (blockingQueue.add(frame)) {
            Logger.getLogger(this.getClass()).debug("add a frame into blockingQueue successful!");
            break;
        }
    }

    public void start() {
        for (int i = 0; i < poolSize; i++) {
            executorService.execute(new HttpFrameConsumer());
        }
    }

    /**
     * 帧消费者线程
     *
     * @author as-youjun
     */
    class HttpFrameConsumer implements Runnable {

        /**
         * 1.从队列中取出帧；2.调用业务接口；
         */
        @Override
        public void run() {
            while (isRunning) {
                // 1.从队列中取出帧
                byte[] frame = null;
                try {
                    frame = blockingQueue.take();
                } catch (InterruptedException e) {
                    Logger.getLogger(this.getClass()).error(e);
                }
                // 2.调用业务接口；
                if (frame == null) {
                    Logger.getLogger(this.getClass()).warn("frame is empty!");
                } else {
                    try {
                        IHttpFrameService httpFrameService = (IHttpFrameService) IotContext.getInstance().getBean(("httpFrameService"));
                        httpFrameService.parseData(frame);
                        frame = null;
                    } catch (Exception e) {
                        Logger.getLogger(this.getClass()).error(e);
                        // 失败处理
                    }
                }
            }
        }

    }


}

