package com.yeejoin.amos.iot.netty.mqtt;

import java.util.concurrent.BlockingQueue;
import java.util.concurrent.LinkedBlockingQueue;

import org.apache.log4j.Logger;

import com.yeejoin.amos.iot.business.constants.IotConstant;
import com.yeejoin.amos.iot.context.IotContext;
import com.yeejoin.amos.iot.netty.vehicle.dic.VehicleDic.CommonDic;

public class MqttFrameQueue {

	/**
	 * 单例
	 */
	private static MqttFrameQueue instance;

	/**
	 * 帧队列
	 */
	private static final BlockingQueue<byte[]> blockingQueue = new LinkedBlockingQueue<byte[]>();

	/**
	 * 控制帧消费者线程运行状态
	 */
	private static boolean isRunning = true;

	/**
	 * 构造方法
	 */
	private MqttFrameQueue() {
		Logger.getLogger(CommonDic.class).debug(IotConstant.CONSTRUCTOR);
	}

	/**
	 * 获取单例
	 * 
	 * @return
	 */
	public static MqttFrameQueue getInstance() {
		if (instance == null) {
			synchronized (MqttFrameQueue.class) {
				if (instance == null) {
					instance = new MqttFrameQueue();
				}
			}
		}
		return instance;
	}

	/**
	 * 添加数据
	 * 
	 * @param frame
	 */
	public void add(byte[] frame) {
		while (blockingQueue.add(frame)) {
			break;
		}
	}

	public void start() {
		Thread thread = new Thread(new MqttFrameConsumer(), "mqtt frame consumer thread");
		thread.start();
	}

	/**
	 * 帧消费者线程
	 * 
	 * @author as-youjun
	 *
	 */
	class MqttFrameConsumer implements Runnable {

		/**
		 * 1.从队列中取出帧；2.调用业务接口；
		 */
		@Override
		public void run() {
			while (isRunning) {
				// 1.从队列中取出帧
				byte[] frame = null;
				try {
					frame = blockingQueue.take();
				} catch (InterruptedException e) {
					Logger.getLogger(this.getClass()).error(e);
				}
				// 2.调用业务接口；
				if (frame == null) {
					Logger.getLogger(this.getClass()).warn("frame is empty!");
				} else {
					try {
						IMqttFrameService mqttFrameService = (IMqttFrameService) IotContext.getInstance().getBean(("mqttFrameService"));
						mqttFrameService.parseData(frame);
						frame = null;
					} catch (Exception e) {
						Logger.getLogger(this.getClass()).error(e);
						// 失败处理
					}
				}
			}
		}

	}



}

