package com.yeejoin.amos.iot.netty.mqtt.handler;

import io.netty.channel.ChannelFutureListener;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.ChannelInboundHandlerAdapter;
import io.netty.handler.timeout.ReadTimeoutException;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import org.meqantt.message.ConnAckMessage;
import org.meqantt.message.ConnAckMessage.ConnectionStatus;
import org.meqantt.message.DisconnectMessage;
import org.meqantt.message.Message;
import org.meqantt.message.Message.Type;
import org.meqantt.message.PingRespMessage;

import com.yeejoin.amos.iot.netty.mqtt.processor.ConnectProcesser;
import com.yeejoin.amos.iot.netty.mqtt.processor.DisConnectProcesser;
import com.yeejoin.amos.iot.netty.mqtt.processor.PingReqProcesser;
import com.yeejoin.amos.iot.netty.mqtt.processor.Processer;
import com.yeejoin.amos.iot.netty.mqtt.processor.PublishProcesser;
import com.yeejoin.amos.iot.netty.mqtt.processor.SubscribeProcesser;
import com.yeejoin.amos.iot.netty.mqtt.processor.UnsubscribeProcesser;

public class MqttMessageHandler extends ChannelInboundHandlerAdapter {
	private static PingRespMessage PINGRESP = new PingRespMessage();

	private static final Map<Message.Type, Processer> processers;
	static {
		Map<Message.Type, Processer> map = new HashMap<Message.Type, Processer>(6);

		map.put(Type.CONNECT, new ConnectProcesser());
		map.put(Type.PUBLISH, new PublishProcesser());
		map.put(Type.SUBSCRIBE, new SubscribeProcesser());
		map.put(Type.UNSUBSCRIBE, new UnsubscribeProcesser());
		map.put(Type.PINGREQ, new PingReqProcesser());
		map.put(Type.DISCONNECT, new DisConnectProcesser());

		processers = Collections.unmodifiableMap(map);
	}

	@Override
	public void exceptionCaught(ChannelHandlerContext ctx, Throwable e)
			throws Exception {
		try {
			if (e.getCause() instanceof ReadTimeoutException) {
				ctx.write(PINGRESP).addListener(
						ChannelFutureListener.CLOSE_ON_FAILURE);
			} else {
				ctx.channel().close();
			}
		} catch (Throwable t) {
			t.printStackTrace();
			ctx.channel().close();
		}

		e.printStackTrace();
	}

	@Override
	public void channelRead(ChannelHandlerContext ctx, Object obj)
			throws Exception {
		Message msg = (Message) obj;
		// 根据不同的消息类型选择对应的消息处理器
		Processer p = processers.get(msg.getType());
		if (p == null) {
			return;
		}
		Message rmsg = p.proc(msg, ctx);
		if (rmsg == null) {
			return;
		}

		if (rmsg instanceof ConnAckMessage
				&& ((ConnAckMessage) rmsg).getStatus() != ConnectionStatus.ACCEPTED) {
			ctx.write(rmsg).addListener(ChannelFutureListener.CLOSE);
		} else if (rmsg instanceof DisconnectMessage) {
			ctx.write(rmsg).addListener(ChannelFutureListener.CLOSE);
		} else {
			ctx.write(rmsg).addListener(ChannelFutureListener.CLOSE_ON_FAILURE);
		}
	}

	@Override
	public void channelReadComplete(ChannelHandlerContext ctx) throws Exception {
		ctx.flush();
	}
}