/**
 * 
 */
package com.yeejoin.amos.iot.netty.vehicle;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import net.sf.json.JSONObject;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.collections.MapUtils;
import org.apache.log4j.Logger;

import com.yeejoin.amos.iot.business.constants.IotConstant;
import com.yeejoin.amos.iot.business.entity.mybatis.EqpTmplAttributeBo;
import com.yeejoin.amos.iot.business.entity.mybatis.EquipmentBo;

/**
 * 自研协议采集service基类
 * 
 * @author as-youjun
 *
 */
public abstract class BaseVehicleDataService {

	/**
	 * 日志记录器
	 */
	private static final Logger logger = Logger
			.getLogger(BaseVehicleDataService.class);

	/**
	 * 获取设备参数
	 * 
	 * @param equipmentBos
	 * @return
	 */
	@SuppressWarnings("unchecked")
	protected Map<Long, Map<String, Object>> getEqpParamValueMap(
			List<EquipmentBo> equipmentBos) {
		Map<Long, Map<String, Object>> eqpParmaValueMap = new HashMap<Long, Map<String, Object>>();
		List<Long> eqpTmplIds = new ArrayList<Long>();
		equipmentBos.forEach(item -> {
			if (!eqpTmplIds.contains(item.getEqpTmplId())) {
				eqpTmplIds.add(item.getEqpTmplId());
			}
		});
		String key = null;
		Object value = null;
		for (EquipmentBo equipmentBo : equipmentBos) {
			Map<String, Object> valueMap = (Map<String, Object>) JSONObject.toBean(JSONObject.fromObject(equipmentBo.getParamJson()),Map.class);
			if (MapUtils.isEmpty(valueMap)) {
				logger.debug("valueMap is empty!");
				continue;
			}
			List<EqpTmplAttributeBo> eqpTmplAttributes = equipmentBo.getEqpTmplAttributes();
			if (CollectionUtils.isEmpty(eqpTmplAttributes)) {
				logger.debug("eqpTmplAttributes is empty! equipId="+ equipmentBo.getId() + ",equipName="+ equipmentBo.getName());
				continue;
			}
			for (EqpTmplAttributeBo eqpTmplAttributeBo : eqpTmplAttributes) {
				if (eqpParmaValueMap.get(equipmentBo.getId()) == null) {
					eqpParmaValueMap.put(equipmentBo.getId(),
							new HashMap<String, Object>());
				}
				key = IotConstant.EquipmentConstant.KEY_PREFIX_EQUIP+ eqpTmplAttributeBo.getId();
				value = valueMap.get(key);
				eqpParmaValueMap.get(equipmentBo.getId()).put(eqpTmplAttributeBo.getName(), value);
			}
		}
		return eqpParmaValueMap;
	}

	/**
	 * 根据sourceId获取监测设备EquipmentBo
	 * 
	 * @param sourceId
	 * @param equipmentBos
	 * @param eqpParmaValueMap
	 * @return
	 */
	protected List<EquipmentBo> getMebosBySourceId(String sourceId,
			List<EquipmentBo> equipmentBos,
			Map<Long, Map<String, Object>> eqpParmaValueMap) {
		List<EquipmentBo> monitorEquipmentBos = new ArrayList<EquipmentBo>();
		String sId = null;
		Map<String, Object> tempMap = null;
		try {
			for (EquipmentBo ebo : equipmentBos) {
				tempMap = eqpParmaValueMap.get(ebo.getId());
				if ((tempMap != null) && (tempMap.get("sourceId") != null)) {
					sId = (String) tempMap.get("sourceId");
					logger.debug("getMebosBySourceId sourceId:" + sId);
					if (sId.equalsIgnoreCase(sourceId)) {
						logger.debug("ebo:" + ebo.toString());
						monitorEquipmentBos.add(ebo);
					}
				}
			}
		} catch (Exception e) {
			logger.error("getMebosBySourceId: " + e);
		}
		return monitorEquipmentBos;
	}

}
