package com.yeejoin.amos.iot.netty.vehicle;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.TimeZone;

import org.apache.log4j.Logger;

import com.yeejoin.amos.iot.business.constants.IotConstant;

/**
 * <pre>
 * 报文解析和构建公共方法
 * </pre>
 *
 * @author as-chennan
 * @version MessageCommonUtil.java v0.1
 * @time 2017-1-6 上午11:22:47
 */
public class MessageCommonUtil {

	/**
	 * byte转化为二进制的位数
	 */
	public static final int byteLowLength = 8;

	public static final int radix_2 = 2;

	public static final int radix_8 = 8;

	public static final int radix_16 = 16;
	/**
	 * 1个byte所能表示的最大16进制数
	 */
	public static final int maxbyte = 0xff;

	public static final String HEX_CHARACTER = "0123456789ABCDEF";
	/**
	 * Requires the output to be formatted as an integer in base sixteen, The
	 * length is 4,if its not 4 then append "0" at High posstion
	 */
	public static final String FORMATE_HEX_4 = "%04x";

	/**
	 * 2字节的16进制数长度
	 */
	public static final int HEX_2byte_LENGTH = 4;

	private MessageCommonUtil() {
		Logger.getLogger(this.getClass()).debug(IotConstant.CONSTRUCTOR);
	}

	/**
	 * 
	 * <pre>
	 * 两位反序 
	 * 参数为16进制数,转化为16进制数返回
	 * EXP1:sourceHex="AB12",targetLength=6 则source--》00AB12--》12AB00
	 * EXP2:sourceHex="AB12",targetLength=4 则source--》12AB
	 * EXP3:sourceHex="AB120",targetLength=4 则source--》AB12-->12AB （该方式舍弃低位，误差较大）
	 * </pre>
	 *
	 * @param sourceHex
	 *            需要转化的16进制数
	 * @param targetLength
	 *            目标长度
	 * @return
	 */
	public static String reverseByTwo(String sourceHex, int targetLength) {
		int sourLen = sourceHex.length();
		if (sourLen < targetLength) {
			for (int i = sourLen; i < targetLength; i++) {
				sourceHex = "0" + sourceHex;
			}
		}
		sourLen = sourceHex.length();
		String[] temp = new String[sourLen / 2];
		int j = 0;
		for (int i = 0; i < sourceHex.length() - 1; i++) {
			if (i % 2 == 0) {
				String tempS = sourceHex.substring(i, i + 2);
				temp[j] = tempS;
				j++;
			}
		}
		int leng = temp.length;
		String result = "";
		for (int i = leng - 1; i >= 0; i--) {
			result = new StringBuilder(result).append(temp[i]).toString();
		}
		return result;
	}

	/**
	 * <pre>
	 * 两位反序 
	 * 参数为16进制数,转化为低位在前高位在后16进制数返回
	 * EXP：AABB--》BBAA
	 * 要求：参数长度为偶数,负责将会舍弃低位，如：AAB--》AA
	 * </pre>
	 * 
	 * @param sourceHex
	 *            AABB--》BBAA
	 */
	public static String reverse16ByTwo(String sourceHex) {
		StringBuffer sb = new StringBuffer();
		int len = sourceHex.length();
		String[] temp = new String[len / 2];
		int j = 0;
		for (int i = 0; i < sourceHex.length() - 1; i++) {
			if (i % 2 == 0) {
				String tempS = sourceHex.substring(i, i + 2);
				temp[j] = tempS;
				j++;
			}
		}
		int leng = temp.length;
		for (int i = leng - 1; i >= 0; i--) {
			if (temp[i].length() < 2) {
				sb.append(0).append(temp[i]);
			} else {
				sb.append(temp[i]);
			}
		}
		return sb.toString().toUpperCase();
	}

	/**
	 * 
	 * <pre>
	 * 两位反序
	 * 将16进制字符串转化成指定字节数，反序
	 * EXP:sourceHex="AABB",byteCounts=4;则：AABB--》0000AABB--》BBAA0000
	 * </pre>
	 *
	 * @param sourceHex
	 * @param byteCounts
	 * @return
	 */
	public static String reverseByTwoWithByteCounts(String sourceHex, int byteCounts) {
		return reverseByTwo(sourceHex, byteCounts * 2);
	}

	/**
	 * 
	 * <pre>
	 * 两位反序 
	 * 参数为16进制数,转化为16进制数返回，不进行补位
	 *	比如：6012-->1260
	 * </pre>
	 * 
	 * @param sourceHex
	 * @param dataLen
	 *            字节
	 * @return
	 */
	public static String reverseByTwoNoZero(String sourceHex, int dataLen) {
		int len = sourceHex.length();
		if (len < (dataLen * 2)) {
			for (int i = len; i < 12; i++) {
				sourceHex = sourceHex + "0";
			}
		}
		len = sourceHex.length();
		String[] temp = new String[len / 2];
		int j = 0;
		for (int i = 0; i < sourceHex.length() - 1; i++) {
			if (i % 2 == 0) {
				String s = sourceHex.substring(i, i + 2);
				temp[j] = s;
				j++;
			}
		}
		int leng = temp.length;
		String result = "";
		for (int i = leng - 1; i >= 0; i--) {
			result = new StringBuilder(result).append(temp[i]).toString();
		}
		return result;
	}

	/**
	 * 
	 * <pre>
	 * 计算校验码
	 * desc:各字节转化为int类型，然后相加，再转化为16进制字符串取最后两位即的校验位
	 * exp：AABBCC--》[AA,BB,CC]--》个字节专为int进行相加:170+187+204=561--》转为16进制：231--》去最后两位31，即的校验码
	 * </pre>
	 * 
	 * @param sourceHex
	 * @return
	 */
	public static String calculateCheck(String sourceHex) {
		int len = sourceHex.length();
		String[] temp = new String[len / 2];
		int j = 0;
		for (int i = 0; i < sourceHex.length() - 1; i++) {
			if (i % 2 == 0) {
				String tempS = sourceHex.substring(i, i + 2);
				temp[j] = tempS;
				j++;
			}
		}
		int count = 0;
		for (String string : temp) {
			count += Integer.parseInt(string, 16);
		}
		String result = Integer.toHexString(count);
		return result.substring(result.length() - 2, result.length()).toUpperCase();
	}

	/**
	 * 
	 * <pre>
	 * 计算校验码
	 * desc:将给定数组，按照规则计算校验码. 数组各项和，取二进制后8位反码 + 1 
	 * exp：AABBCC--》[AA,BB,CC]--》个字节专为int进行相加:170+187+204=561--》转为16进制：231--》去最后两位31，即的校验码
	 * </pre>
	 * 
	 * @param sourceHex
	 * @return
	 */
	public static String calculateCheck(byte[] array) {
		long temp = 0;
		for (byte b : array) {
			temp += (b & 0xff);
		}

		String binary = Long.toBinaryString(~temp);
		binary = binary.substring(binary.length() - 8);
		temp = Long.valueOf(binary, radix_2) + 1;

		return Long.toHexString(temp);
	}

	/**
	 * 
	 * <pre>
	 * 每两位分割成char
	 * 
	 * 参数为16进制数 返回16进制数char数组
	 * </pre>
	 * 
	 * 79 31 35 30 31 2D 30 31 46 30 30 30 32 79 01 00 27
	 * 
	 * @param str
	 * @return
	 */
	public static byte[] splitByTwo(String str) {
		List<Byte> byList = new ArrayList<Byte>();
		for (int i = 0; i < str.length() - 1; i++) {
			if (i % 2 == 0) {
				String s = str.substring(i, i + 2);
				int number = Integer.parseInt(s, radix_16);
				byList.add((byte) number);
			}
		}
		byte[] by = new byte[byList.size()];
		for (int i = 0; i < by.length; i++) {
			by[i] = byList.get(i);
		}
		return by;
	}

	/**
	 * 
	 * <pre>
	 *	将byte数组中的负数转化为相应的16进制
	 *	过程：245--11110101--取反--00001010--（+1）--00001011==-11
	 *	逆过程：
	 * </pre>
	 *
	 * @param number
	 *            -256至255
	 * @return
	 */
	public static String transforNegativeNumber(int number) {
		// -11 -- 11 -- 1011 --
		// 1、取绝对值
		int positiveNumber = Math.abs(number);
		// 2、减1
		positiveNumber = positiveNumber - 1;
		// 3、转化为2进制
		String tempBinary = Integer.toBinaryString(positiveNumber);
		if (tempBinary.length() > 8) {
			return null;
		}
		// 4、确保8位
		boolean is = tempBinary.length() % byteLowLength == 0 ? true : false;
		if (!is) {
			StringBuilder sb = new StringBuilder(tempBinary);
			int len = byteLowLength - (tempBinary.length() % byteLowLength);
			for (int i = 0; i < len; i++) {
				tempBinary = sb.reverse().append("0").reverse().toString();
			}
		}
		// 5、取反 00001010
		char[] charTemps = tempBinary.toCharArray();
		StringBuffer result = new StringBuffer();
		for (char tempChar : charTemps) {
			if (String.valueOf(tempChar).equals("0")) {
				result.append("1");
			} else if (String.valueOf(tempChar).equals("1")) {
				result.append("0");
			}
		}
		// 6、新的2进制数 11110101,转化为16进制即可
		return Integer.toHexString(Integer.parseInt(result.toString(), radix_2)).toUpperCase();
	}

	/**
	 * 
	 * <pre>
	 *	将byte数组中的负数转化为相应的int正数
	 *	过程：245--11110101--取反--00001010--（+1）--00001011==-11
	 *	逆过程：
	 * </pre>
	 *
	 * @param number
	 * @return
	 */
	public static int transforNumber(int number) {
		// -11 -- 11 -- 1011 --
		// 1、取绝对值
		int positiveNumber = Math.abs(number);
		// 2、减1
		positiveNumber = positiveNumber - 1;
		// 3、转化为2进制
		String tempBinary = Integer.toBinaryString(positiveNumber);
		if (tempBinary.length() > 8) {
			return -1;
		}
		// 4、确保8位
		boolean is = tempBinary.length() % byteLowLength == 0 ? true : false;
		if (!is) {
			StringBuilder sb = new StringBuilder(tempBinary);
			int len = byteLowLength - (tempBinary.length() % byteLowLength);
			for (int i = 0; i < len; i++) {
				tempBinary = sb.reverse().append("0").reverse().toString();
			}
		}
		// 5、取反 00001010
		char[] charTemps = tempBinary.toCharArray();
		StringBuffer result = new StringBuffer();
		for (char tempChar : charTemps) {
			if (String.valueOf(tempChar).equals("0")) {
				result.append("1");
			} else if (String.valueOf(tempChar).equals("1")) {
				result.append("0");
			}
		}
		// 6、新的2进制数 11110101,转化为10进制即可
		return Integer.parseInt(result.toString(), radix_2);
	}

	/**
	 * 
	 * <pre>
	 * 两16进制相减返回16进制
	 * </pre>
	 *
	 * @return
	 */
	public static String reduceByRadix_16(String start_16, String end_16) {
		int one = Integer.parseInt(start_16, radix_16);
		int two = Integer.parseInt(end_16, radix_16);
		int resultValue = one - two;
		String result = Integer.toHexString(resultValue).toUpperCase();
		if (result.length() < 2) {
			result = "0" + result;
		}
		return result;
	}

	/**
	 * 
	 * <pre>
	 * 获取报文中指定模块数据，返回指定模块新byte数组
	 * EXP：报文如下：[90,107,124,126,16,00,00,00,02,00,00,00,177,04,00,00]
	 * 要获取“177,04,00,00”,则 prifix = 12; partLength = 4;代表(i=12;i<16;i++)
	 * </pre>
	 *
	 * @param result
	 *            报文
	 * @param startPos
	 *            指定开始(数组下标从0开始)
	 * @param partLength
	 *            指定模块长度
	 * @return
	 */
	public static byte[] getAnyPartMessage(byte[] result, int startPos, int partLength) {
		byte[] resultByte = new byte[partLength];

		System.arraycopy(result, startPos, resultByte, 0, partLength);

		/*
		 * for (int i = startPos; i < startPos + partLength; i++) { resultByte[i
		 * - startPos] = result[i]; }
		 */
		return resultByte;
	}

	/**
	 * 
	 * <pre>
	 * 按照自定义格式 format，构建数据
	 * </pre>
	 * 
	 * xxxxxx.xx
	 * 
	 * @param format
	 * @param data
	 * @return
	 */
	public static Double dataFormat(Double format, String data) {
		Long valueTen = Long.valueOf(data);
		Double value = valueTen / format;
		return value;
	}

	/**
	 * 
	 * <pre>
	 * 按照自定义格式 format，构建数据
	 * </pre>
	 * 
	 * xxxxxx.xx
	 * 
	 * @param format
	 * @param data
	 * @return
	 */
	public static Double dataPrecision(Double precision, String dateString) {
		Double value = null;
		Long valueTen = Long.valueOf(dateString);
		if (valueTen != null) {
			Double values = valueTen / precision;
			int length = String.valueOf(valueTen).length();
			if (values > 1) {
				value = valueTen % (Math.pow(10, (length - 1)) * 8);
				if (value == 0.0) {
					value = 0.0;
				} else if (value < valueTen) {
					value = -value / precision;
				} else {
					value = value / precision;
				}
			} else {
				value = values;
			}
		}
		return value;

	}

	/**
	 * 
	 * <pre>
	 * 
	 * </pre>
	 *
	 * @param data
	 * @return
	 */
	public static Date dateFormat(String data, int dataLen) {
		Calendar calendar = Calendar.getInstance();
		if (dataLen == 5) {
			int year = Integer.parseInt("20" + data.substring(0, 2));
			int month = Integer.parseInt(data.substring(2, 4));
			int day = Integer.parseInt(data.substring(4, 6));
			int hour = Integer.parseInt(data.substring(6, 8));
			int minute = Integer.parseInt(data.substring(8, 10));
			calendar.set(year, month - 1, day, hour, minute);
		} else if (3 == dataLen) {
			int year = calendar.get(Calendar.YEAR);
			int month = calendar.get(Calendar.MONTH) + 1;
			int day = calendar.get(Calendar.DAY_OF_MONTH);
			int hour = Integer.parseInt(data.substring(0, 2));
			int minute = Integer.parseInt(data.substring(2, 4));
			int second = Integer.parseInt(data.substring(4, 6));
			calendar.set(year, month - 1, day, hour, minute, second);
		} else if (4 == dataLen) {
			int year = calendar.get(Calendar.YEAR);
			int month = Integer.parseInt(data.substring(0, 2));
			int day = Integer.parseInt(data.substring(2, 4));
			int hour = Integer.parseInt(data.substring(4, 6));
			int minute = Integer.parseInt(data.substring(6, 8));
			calendar.set(year, month - 1, day, hour, minute);
		}
		Date dateTime = new Date(calendar.getTimeInMillis());
		return dateTime;
	}

	/**
	 * 
	 * <pre>
	 * yyyy - mmmm - dddd - hhhh - mmmm - ssss
	 * </pre>
	 *
	 * @param data
	 * @param dataLen
	 * @return
	 */
	public static Date dateFormatbyymdhms(String data, int dataLen) {
		Calendar calendar = Calendar.getInstance();
		if (dataLen == 12) {
			int year = Integer.parseInt(data.substring(0, 4), 16);
			int month = Integer.parseInt(data.substring(4, 8), 16);
			int day = Integer.parseInt(data.substring(8, 12), 16);
			int hour = Integer.parseInt(data.substring(12, 16), 16);
			int minute = Integer.parseInt(data.substring(16, 20), 16);
			int second = Integer.parseInt(data.substring(20, 24), 16);
			calendar.set(year, month - 1, day, hour, minute, second);
		}
		Date dateTime = new Date(calendar.getTimeInMillis());
		return dateTime;
	}

	/**
	 * 
	 * <pre>
	 * 字符串转16进制
	 * </pre>
	 *
	 * @param str
	 * @return
	 */
	public static String transStringToAscill(String str) {
		StringBuffer sb = new StringBuffer();
		// 字符串转换为十六进制
		for (int i = 0; i < str.length(); i++) {
			int ch = (int) str.charAt(i);
			String value = Integer.toHexString(ch);
			if (value.length() == 1)// 补全8位
			{
				value = "0" + value;
			}
			sb.append(value);
		}
		return sb.toString().toUpperCase();
	}

	/**
	 * 
	 * <pre>
	 * 将byte数组转化为16进制
	 * </pre>
	 * 
	 * @param b
	 *            byte数组
	 * @return
	 */
	public static String bytes2HexString(byte[] b) {
		byte[] hex = HEX_CHARACTER.getBytes();
		byte[] buff = new byte[2 * b.length];
		for (int i = 0; i < b.length; i++) {
			buff[2 * i] = hex[(b[i] >> 4) & 0x0f];
			buff[2 * i + 1] = hex[b[i] & 0x0f];
		}

		return new String(buff);
	}

	public static String bytes2HexStringWithSpace(byte[] b) {
		byte[] hex = HEX_CHARACTER.getBytes();
		byte[] buff = new byte[2 * b.length];
		for (int i = 0; i < b.length; i++) {
			buff[2 * i] = hex[(b[i] >> 4) & 0x0f];
			buff[2 * i + 1] = hex[b[i] & 0x0f];
		}

		String result = new String(buff);
		StringBuffer sBuffer = new StringBuffer();
		for (int i = 0; i < result.length() / 2; i++) {
			sBuffer.append(result.substring(2 * i, 2 * (i + 1))).append(" ");
		}
		return sBuffer.toString();
	}

	public static String bytes2HexStringWithDelimiter(byte[] b, String delimiter) {
		byte[] hex = HEX_CHARACTER.getBytes();
		byte[] buff = new byte[2 * b.length];
		for (int i = 0; i < b.length; i++) {
			buff[2 * i] = hex[(b[i] >> 4) & 0x0f];
			buff[2 * i + 1] = hex[b[i] & 0x0f];
		}

		String result = new String(buff);
		StringBuffer sBuffer = new StringBuffer();
		for (int i = 0; i < result.length() / 2; i++) {
			sBuffer.append(result.substring(2 * i, 2 * (i + 1)));
			if (i != (result.length() / 2 - 1)) {
				sBuffer.append(delimiter);
			}
		}
		return sBuffer.toString();
	}

	/**
	 * 
	 * <pre>
	 * 16进制的字符串表示转成字节数组
	 * </pre>
	 * 
	 * @param hexString
	 *            16进制格式的字符串
	 * @return 转换后的字节数组
	 */
	public static byte[] hexStr2ByteArray(String hexString) {
		if (!hexString.isEmpty()) {
			hexString = hexString.toLowerCase();
			final byte[] byteArray = new byte[hexString.length() / 2];
			int k = 0;
			for (int i = 0; i < byteArray.length; i++) {
				/**
				 * 因为是16进制，最多只会占用4位，转换成字节需要两个16进制的字符，高位在先将hex 转换成byte "&"
				 * 操作为了防止负数的自动扩展 hex转换成byte 其实只占用了4位，然后把高位进行右移四位 然后“|”操作 低四位
				 * 就能得到 两个 16进制数转换成一个byte.
				 */
				byte high = (byte) (Character.digit(hexString.charAt(k), 16) & maxbyte);
				byte low = (byte) (Character.digit(hexString.charAt(k + 1), 16) & maxbyte);
				byteArray[i] = (byte) (high << 4 | low);
				k += 2;
			}
			return byteArray;
		}
		return null;
	}

	/**
	 * <pre>
	 * 16进制字符串转换成byte数组
	 * </pre>
	 * 
	 * @param 16进制字符串
	 * @return 转换后的byte数组
	 */
	public static byte[] hex2Byte(String hex) {
		char[] hex2char = hex.toCharArray();
		byte[] bytes = new byte[hex.length() / 2];
		int temp;
		for (int i = 0; i < bytes.length; i++) {
			/**
			 * 其实和hexStr2ByteArray(...)方法是一样的 multiple 16 就是右移4位
			 * 这样就成了高4位了,然后和低四位相加， 相当于 位操作"|"相加后的数字 进行 位 "&" 操作 防止负数的自动扩展. {0xff
			 * byte最大表示数}
			 */
			temp = HEX_CHARACTER.indexOf(hex2char[2 * i]) * 16;
			temp += HEX_CHARACTER.indexOf(hex2char[2 * i + 1]);
			bytes[i] = (byte) (temp & maxbyte);
		}
		return bytes;
	}

	/**
	 * 
	 * <pre>
	 *  将10进制的CRC转化为16进制CRC,同时低位在前高位在后
	 *	exp:44678-->AE86-->86AE
	 * </pre>
	 *
	 * @param temp
	 *            参数为10进制数
	 * @return 16进制数,长度4
	 */
	public static String calculateCRC16WithLow_High(int temp) {
		String crcResult = String.format(FORMATE_HEX_4, temp);
		return reverse16ByTwo(crcResult);
	}

	/**
	 * 
	 * <pre>
	 *  将10进制数转化为长度为4的16进制数,不足4位则在高位补0
	 *	exp:11-->B-->000B
	 * </pre>
	 *
	 * @param temp
	 *            参数为10进制数
	 * @return 16进制数,长度4
	 */
	public static String toHexWith4Length(int temp) {
		String crcResult = String.format(FORMATE_HEX_4, temp);
		return crcResult.toUpperCase();
	}

	/**
	 * 
	 * <pre>
	 * 将10进制数转化为长度为length的16进制数,不足length位则在高位补0
	 *	exp:temp=11,length=4-->B-->000B
	 *	exp:temp=11,length=1-->B
	 *注意：当参数temp为负数时，长度默认8
	 * </pre>
	 *
	 * @param temp
	 *            10进制int数(正数byte范围内可指定长度)
	 * @param length
	 *            转化后的16进制字符串长度
	 * @return
	 */
	public static String toHexWithAppointLength(int temp, int appointLength) {
		String formater = new StringBuffer().append("%0").append(appointLength).append("x").toString();
		String crcResult = String.format(formater, temp);
		return crcResult.toUpperCase();
	}

	/**
	 * 
	 * <pre>
	 * 将16进制数转化为doube
	 * </pre>
	 *
	 * @param dataHex
	 * @param multiple
	 *            除的倍数
	 * @return
	 */
	public static double hexDataFormat2Double(String dataHex, int multiple) {
		/**
		 * 将long型转化为double，确保除以multiple时，保存精度
		 */
		double tempLong = Long.valueOf(dataHex, radix_16);
		double result = tempLong / multiple;
		return result;
	}

	/**
	 * 
	 * <pre>
	 * 将16进制字符串转化为2进制
	 * </pre>
	 *
	 * @param dataHex
	 * @return
	 */
	public static String convertHex2Binary(String dataHex) {
		int tempInt = Integer.parseInt(dataHex, radix_16);
		return Integer.toBinaryString(tempInt);
	}

	/**
	 * 
	 * <pre>
	 * 将16进制字符串转化为2进制,不足长度，则在高位补0
	 * </pre>
	 *
	 * @param dataHex
	 * @param pointLength
	 * @return
	 */
	public static String convertHex2Binary(String dataHex, int pointLength) {
		String tempHex = convertHex2Binary(dataHex);
		int length = tempHex.length();
		if (length == pointLength) {
			return tempHex;
		} else if (length < pointLength) {
			// 反序,高位补“0”
			StringBuffer sb = new StringBuffer().append(tempHex).reverse();
			for (int i = 0; i < pointLength - length; i++) {
				sb.append("0");
			}
			return sb.reverse().toString();// 还原顺序
		} else {
			return tempHex.substring(length - pointLength);
		}
	}

	/**
	 * 
	 * <pre>
	 * 获取世纪秒，转为16进制，字节长度为4
	 * </pre>
	 *
	 * @return
	 */
	public static String getCurrentTimestmap() {
		// 设置时区
		TimeZone tz = TimeZone.getTimeZone("ETC/GMT-8");
		TimeZone.setDefault(tz);
		long timestmap = System.currentTimeMillis() / 1000;
		return String.format("%08x", timestmap).toUpperCase();
	}

	/**
	 * 
	 * <pre>
	 * 获取备用位，返回16进制
	 * exp: byteCounts=3,则-->000000
	 *  	byteCounts=4,则-->00000000
	 * </pre>
	 *
	 * @param byteCounts
	 *            大于0
	 * @return
	 */
	public static String getReserve1Bylength(int byteCounts) {
		StringBuffer sb = new StringBuffer();
		for (int i = 0; i < byteCounts; i++) {
			sb.append("00");
		}
		return sb.toString();
	}

	/**
	 * 
	 * <pre>
	 * 将16进制转化为int类型
	 * </pre>
	 *
	 * @param dataHex
	 * @return
	 */
	public static int convertHex2Integer(String dataHex) {
		return Integer.parseInt(dataHex, radix_16);
	}

	/**
	 * 从一个byte[]数组中截取一部分
	 * 
	 * @param src
	 * @param begin
	 * @param count
	 * @return
	 */
	public static byte[] subBytes(byte[] src, int begin, int count) {
		byte[] bs = new byte[count];
		for (int i = begin; i < begin + count; i++)
			bs[i - begin] = src[i];
		return bs;
	}

	/**
	 * 把两个字节转换为一个无符号的整型
	 * 
	 * @param bl
	 *            低位字节
	 * @param bh
	 *            高位字节
	 * @return 转换后的整型值
	 */
	public static int convertTwoBytes2Uint16(byte bl, byte bh) {
		return (bl & 0x00FF) | ((bh << 8) & 0xFF00);
	}

	/**
	 * 把两个字节转换为一个有符号的整型
	 * 
	 * @param bl
	 *            低位字节
	 * @param bh
	 *            高位字节
	 * @return 转换后的整型值
	 */
	public static int convertTwoBytes2Int16(byte bl, byte bh) {
		int temp = (bl & 0x00FF) | ((bh << 8) & 0xFF00);
		if (temp > 32767) {
			temp -= 65536;
		}
		return temp;
	}

	/**
	 * 
	 * <pre>
	 * 把字节数组从指定的位置取3个字节转换成无符号整型
	 * </pre>
	 *
	 * @param values
	 * @return
	 */
	public static int getUnsignedMedium(byte[] values, int pos) {
		return (values[pos] & 0xff) << 16 | (values[pos + 1] & 0xff) << 8 | values[pos + 2] & 0xff;
	}

	/**
	 * 
	 * <pre>
	 * 把4个字节的数组按小端模式转换成Float
	 * </pre>
	 *
	 * @param values
	 * @return
	 */
	public static float convert4bytes2Float(byte[] values) {
		int temp = 0;
		for (int i = 0; i < values.length; i++) {
			temp |= (values[i] & 0xff) << i * 8;

		}
		return Float.intBitsToFloat(temp);
	}

	/**
	 * 
	 * <pre>
	 * 把4个字节的数组按小端模式转换成有符号的int32
	 * </pre>
	 *
	 * @param values
	 * @return
	 */
	public static int convert4bytes2Int32InLittleEnd(byte[] values) {
		int temp = 0;
		temp += ((values[0] & 0xff)) + ((values[1] & 0xff) << 8) + ((values[2] & 0xff) << 16)
				+ (((values[3] & 0xff) << 24) & 0xff000000);
		return temp;
	}

	/**
	 * 
	 * <pre>
	 * 把4个字节的数组按小端模式转换成无符号的uint32
	 * java中int无法覆盖uint32的取值范围，所以用long来表示uint32的值
	 * </pre>
	 *
	 * @param values
	 * @return
	 */
	public static long convert4bytes2Uint32InLittleEnd(byte[] values) {
		long temp = 0;
		temp += ((values[0] & 0xff)) + ((values[1] & 0xff) << 8) + ((values[2] & 0xff) << 16)
				+ (((values[3] & 0xff) << 24) & 0xff000000L);
		return temp;
	}

	/**
	 * 把一个字节转换成无符号数以整型表达
	 * 
	 * @param b
	 *            要转换的字节
	 * @return 返回整型的无符号数
	 */
	public static int convertByte2Unsigned(byte b) {
		return Byte.toUnsignedInt(b);
	}

	/**
	 * 把一个整型以十六进制的方式转换为小端模式的字节数组
	 * 
	 * @param intValue
	 *            需要转换的整数
	 * @return 转换后的以小端模式表示大小为2个的字节数组
	 */
	public static byte[] convertInt2TwoBytesInLittleEnd(int intValue) {
		byte[] byteArrayToInt = new byte[2];
		byteArrayToInt[0] = (byte) ((intValue) & 0xFF);
		byteArrayToInt[1] = (byte) ((intValue >> 8) & 0xFF);
		return byteArrayToInt;
	}

	/**
	 * 把一个整型以十六进制的方式转换为大端模式的字节数组
	 * 
	 * @param intValue
	 *            需要转换的整数
	 * @return 转换后的以大端模式表示大小为2个的字节数组
	 */
	public static byte[] convertInt2TwoBytesInBigEnd(int intValue) {
		byte[] byteArrayToInt = convertInt2TwoBytesInLittleEnd(intValue);
		return converse(byteArrayToInt);
	}

	/**
	 * 把一个整型以十六进制的方式转换为小端模式的字节数组
	 * 
	 * @param intValue
	 *            需要转换的整数
	 * @return 转换后的以小端模式表示大小为4个的字节数组
	 */
	public static byte[] convertInt2FourBytesInLittleEnd(int intValue) {
		byte[] byteArrayToInt = new byte[4];
		byteArrayToInt[0] = (byte) ((intValue) & 0xFF);
		byteArrayToInt[1] = (byte) ((intValue >> 8) & 0xFF);
		byteArrayToInt[2] = (byte) ((intValue >> 16) & 0xFF);
		byteArrayToInt[3] = (byte) ((intValue >> 24) & 0xFF);
		return byteArrayToInt;
	}

	/**
	 * 
	 * <pre>
	 * 异或校验（前3个字节求异或与最后一个自己比较）
	 * </pre>
	 *
	 * @param bytes
	 * @return
	 */
	public static boolean cxorc(byte[] bytes) {
		if (bytes.length < 4) {
			return false;
		}
		return (bytes[0] ^ bytes[1] ^ bytes[2]) == bytes[3];
	}

	/**
	 * 
	 * <pre>
	 * 反转byte数组
	 * </pre>
	 *
	 * @param bytes
	 * @return
	 */
	private static byte[] converse(byte[] bytes) {
		byte[] temp = new byte[bytes.length];
		for (int i = 0; i < temp.length; i++) {
			temp[i] = bytes[bytes.length - i - 1];
		}
		return temp;
	}

}
