/**
 * 
 */
package com.yeejoin.amos.iot.netty.vehicle;

import java.util.List;
import java.util.Map;

import net.sf.json.JSONObject;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.collections.MapUtils;
import org.apache.log4j.Logger;

import com.yeejoin.amos.iot.business.entity.Capability;
import com.yeejoin.amos.iot.business.entity.Equipment;
import com.yeejoin.amos.iot.business.entity.EquipmentTemplate;
import com.yeejoin.amos.iot.business.entity.MetricData;
import com.yeejoin.amos.iot.business.entity.mybatis.CapacityPropertyDataVo;
import com.yeejoin.amos.iot.business.enums.AttributeType;
import com.yeejoin.amos.iot.mongodb.entity.CapabilityData;
import com.yeejoin.amos.iot.mongodb.entity.EqpMetricData;

/**
 * 自研协议service助手类
 * 
 * @author as-youjun
 *
 */
public class VechicleServiceHandler {

	/**
	 * 日志记录器
	 */
	private static final Logger logger = Logger
			.getLogger(VechicleServiceHandler.class);

	/**
	 * 获取属性名对应的值
	 * 
	 * @param cpblAttrs
	 * @param attrName
	 * @return
	 */
	public static Integer getAttrValueInt(
			List<CapacityPropertyDataVo> cpblAttrs, String attrName) {
		Integer attrValue = null;
		if (CollectionUtils.isEmpty(cpblAttrs)) {
			return attrValue;
		}
		try {
			for (CapacityPropertyDataVo cpblAttr : cpblAttrs) {
//				logger.debug("cpblId:" + cpblAttr.getId() + ",cpblName:"
//						+ cpblAttr.getName());
				if (attrName.equals(cpblAttr.getName())
						&& (cpblAttr.getType().intValue() == AttributeType.CONSTANT
								.getValue())) {
					attrValue = Integer.valueOf(cpblAttr.getValueStr());
//					logger.debug(attrName + ":" + attrValue);
				}
			}
		} catch (Exception e) {
			logger.error("getAttrValue error! " + e);
		}
		return attrValue;
	}

	/**
	 * 构建数据对象EqpMetricData
	 * 
	 * @param metricData
	 * @param equipmentTemplate
	 * @param equipment
	 * @param sourceId
	 * @param capabilityData
	 * @return
	 */
	public static EqpMetricData buildEqpMetricData(MetricData metricData,
			EquipmentTemplate equipmentTemplate, Equipment equipment,
			String sourceId, CapabilityData capabilityData) {
		EqpMetricData equipMetricData = new EqpMetricData();
		equipMetricData.setEqpTmplId(metricData.getEqpTmplId());
		equipMetricData.setEqpModel(equipmentTemplate.getModel());
		equipMetricData.setEqpTmpldescr(equipmentTemplate.getDescr());
		equipMetricData.setEqpTmplName(equipmentTemplate.getName());
		equipMetricData.setEqpType(equipmentTemplate.getType());
		equipMetricData.setVisibility(equipmentTemplate.getVisibility());
		equipMetricData.setEqpId(metricData.getEqpId());
		equipMetricData.setEqpName(equipment.getName());
		equipMetricData.setEqpDescr(equipment.getDescr());
		equipMetricData.setUserId(equipment.getUserId());
		if (sourceId != null && MapUtils.isNotEmpty(equipment.getCfgParamMap())) {
			equipMetricData.getSpecEqpDataMap().put("sourceId", sourceId);
		}
		equipMetricData.setCapabilityData(capabilityData);
		return equipMetricData;
	}

	/**
	 * 构建数据对象CapabilityData
	 * 
	 * @param capability
	 * @param metricData
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public static CapabilityData buildCapabilityData(Capability capability,
			MetricData metricData) {
		CapabilityData capabilityData = new CapabilityData();
		capabilityData.setCpblId(capability.getId());
		capabilityData.setCpblName(capability.getName());
		capabilityData.setCpblDescr(capability.getDescr());
		capabilityData.setCpblType(capability.getType());
		capabilityData.setProtocolId(capability.getProtocolId());
		capabilityData.setExeTime(metricData.getExeTime());
		// 指标数据
		Map<String, Object> metricDataMap = (Map<String, Object>) JSONObject
				.toBean(JSONObject.fromObject(metricData.getMetricJson()),
						Map.class);
		capabilityData.setMetricDataMap(metricDataMap);
		return capabilityData;
	}

}
