/**
 * 
 */
package com.yeejoin.amos.iot.netty.vehicle;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;

import com.yeejoin.amos.iot.business.constants.IotConstant;

/**
 * 车载终端上报数据字节数组缓存
 * 
 * @author as-youjun
 *
 */
public class VehicleByteArrayBuffer {

	/**
	 * 单例
	 */
	private static VehicleByteArrayBuffer instance;

	/**
	 * 字节数组数据缓存
	 */
	private byte[] bytes;

	/**
	 * 构造方法
	 */
	private VehicleByteArrayBuffer() {
		Logger.getLogger(VehicleByteArrayBuffer.class).debug(IotConstant.CONSTRUCTOR);
	}

	/**
	 * 获取单例
	 * 
	 * @return
	 */
	public static VehicleByteArrayBuffer getInstance() {
		if (instance == null) {
			synchronized (VehicleByteArrayBuffer.class) {
				if (instance == null) {
					instance = new VehicleByteArrayBuffer();
				}
			}
		}
		return instance;
	}

	/**
	 * 获取帧数据
	 * 
	 * @param sourceBytes
	 * @return
	 */
	public List<byte[]> parseFrames(byte[] sourceBytes) {
		if (sourceBytes == null) {
			return null;
		}
		// sourceBytes追加到this.bytes
		if (this.bytes == null) {
			this.bytes = sourceBytes;
		} else {
			byte[] data = new byte[this.bytes.length + sourceBytes.length];
			System.arraycopy(this.bytes, 0, data, 0, this.bytes.length);
			System.arraycopy(sourceBytes, 0, data, this.bytes.length, sourceBytes.length);
			this.bytes = data;
		}
		List<byte[]> dataList = new ArrayList<byte[]>();
		boolean isNeedContinue = true;// 是否继续获取一帧数据
		while (isNeedContinue) {
			byte[] frameBytes = parseOneFrame();
			if (frameBytes == null) {
				isNeedContinue = false;
			} else {
				dataList.add(frameBytes);
			}
		}
		return dataList;
	}

	/**
	 * 获取一帧数据
	 * 
	 * @return
	 */
	public byte[] parseOneFrame() {
		if (this.bytes == null) {
			Logger.getLogger(this.getClass()).debug("bytes is null!");
			return null;
		}
		if (this.bytes.length <= (VehicleFrame.HEAD_LENGTH + VehicleFrame.TAIL_LENGTH)) {
			return null;
		}
		// 帧体长度
		int lengthOfBody = lengthOfBody(this.bytes);
		int frameLength = VehicleFrame.HEAD_LENGTH + lengthOfBody + VehicleFrame.TAIL_LENGTH;
		if (this.bytes.length < frameLength) {
			// 长度小于一帧数据长度，缓存数据，留给下一次处理
			return null;
		} else if (this.bytes.length == frameLength) {
			// 长度等于一帧数据长度
			byte[] frameBytes = new byte[this.bytes.length];
			System.arraycopy(this.bytes, 0, frameBytes, 0, this.bytes.length);
			this.bytes = null;
			return frameBytes;
		} else {
			// 长度大于一帧数据长度
			byte[] frameBytes = new byte[frameLength];
			System.arraycopy(this.bytes, 0, frameBytes, 0, frameLength);
			// 需要继续缓存的字节数据
			byte[] bufferBytes = new byte[this.bytes.length - frameLength];
			System.arraycopy(this.bytes, frameLength, bufferBytes, 0, this.bytes.length - frameLength);
			this.bytes = bufferBytes;
			return frameBytes;
		}
	}

	/**
	 * 帧体长度
	 * 
	 * @param bytes
	 * @return
	 */
	private int lengthOfBody(byte[] bytes) {
		return MessageCommonUtil.convertTwoBytes2Uint16(bytes[2], bytes[1]);
	}

}
