/**
 * 
 */
package com.yeejoin.amos.iot.netty.vehicle;

import java.util.Map.Entry;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;

import org.apache.log4j.Logger;

import com.yeejoin.amos.iot.business.constants.IotConstant;
import com.yeejoin.amos.iot.business.queue.AlarmDataObj;
import com.yeejoin.amos.iot.business.queue.AlarmDataQueue;
import com.yeejoin.amos.iot.business.queue.ChannelAlarmData;
import com.yeejoin.amos.iot.netty.vehicle.entity.VehicleDevice;

import io.netty.channel.Channel;

/**
 * 通道缓存
 * 
 * @author as-youjun
 *
 */
public class VehicleChannelCache {

	/**
	 * 单例
	 */
	private static VehicleChannelCache instance;

	/**
	 * 记录通道和sid：<通道,sid>
	 */
	private ConcurrentHashMap<Channel, String> channelSidMap = new ConcurrentHashMap<>();

	/**
	 * 记录消防物联网设备标识、时间和连接状况数据：<sid,VehicleDevice>
	 */
	private ConcurrentHashMap<String, VehicleDevice> vehicleDeviceMap = new ConcurrentHashMap<>();

	/**
	 * 构造方法
	 */
	private VehicleChannelCache() {
		Logger.getLogger(VehicleChannelCache.class).debug(IotConstant.CONSTRUCTOR);
	}

	/**
	 * 获取单例
	 * 
	 * @return
	 */
	public static VehicleChannelCache getInstance() {
		if (instance == null) {
			synchronized (VehicleChannelCache.class) {
				if (instance == null) {
					instance = new VehicleChannelCache();
				}
			}
		}
		return instance;
	}
	public void channelRegister(Channel channel, String sId) {
		String str = channelSidMap.get(channel);
		if(null == str){
			//缓存新的通道
			channelSidMap.put(channel, sId);
			ChannelAlarmData channelAlarmData = new ChannelAlarmData();
			channelAlarmData.setsId(sId);
			channelAlarmData.setType(IotConstant.REGISTER);
			AlarmDataObj alarmDataObj = new AlarmDataObj(channelAlarmData);
			alarmDataObj.setType(IotConstant.CHANNEL);
			//设备通道建立清除告警
			AlarmDataQueue.getInstance().add(alarmDataObj);
		}else{
			Logger.getLogger(VehicleChannelCache.class).info("register  sId:"+sId);
		}
	}
	public void channelUnregister(Channel channel){
		String sId = channelSidMap.get(channel);
		if(null != sId){
			ChannelAlarmData channelAlarmData = new ChannelAlarmData();
			channelAlarmData.setsId(sId);
			channelAlarmData.setType(IotConstant.UNREGISTER);
			AlarmDataObj alarmDataObj = new AlarmDataObj(channelAlarmData);
			alarmDataObj.setType(IotConstant.CHANNEL);
			AlarmDataQueue.getInstance().add(alarmDataObj);
			//清除旧的通道
			channelSidMap.remove(channel);
		}else{
			Logger.getLogger(VehicleChannelCache.class).info("unregister  sId:"+sId);
		}
	}
	/**
	 * 添加通道
	 * 
	 * @param channel
	 * @param sid
	 */
	public void addChannel(Channel channel, String sid) {
		// 1.获取与关闭旧连接
		Channel oldChannel = getChannel(sid);
		try {
			channelSidMap.remove(oldChannel);
			oldChannel.closeFuture().sync();
		} catch (Exception e) {
			Logger.getLogger(this.getClass()).error(e);
		}
		// 2.添加通道
		channelSidMap.put(channel, sid);
	}

	/**
	 * 获取通道
	 * 
	 * @param sid
	 * @return
	 */
	public Channel getChannel(String sid) {
		Channel channel = null;
		Set<Entry<Channel, String>> channels = channelSidMap.entrySet();
		for (Entry<Channel, String> entry : channels) {
			if (entry.getValue().equals(sid)) {
				channel = entry.getKey();
				break;
			}
		}
		return channel;
	}

	/**
	 * 获取sid
	 * 
	 * @param channel
	 * @return
	 */
	public String getSid(Channel channel) {
		return channelSidMap.get(channel);
	}

	/**
	 * 移除通道
	 * 
	 * @param channel
	 */
	public void removeChannel(Channel channel) {
		channelSidMap.remove(channel);
	}

	/**
	 * 移除通道
	 * 
	 * @param sid
	 */
	public void removeChannel(String sid) {
		Channel channel = getChannel(sid);
		if (channel != null) {
			channelSidMap.remove(channel);
		}
	}

	/**
	 * 缓存中添加消防物联网设备
	 * 
	 * @param vehicleDevice
	 */
	public void addVehicleDevice(VehicleDevice vehicleDevice) {
		vehicleDeviceMap.put(vehicleDevice.getSourceId(), vehicleDevice);
	}

	/**
	 * 缓存中移除消防物联网设备
	 * 
	 * @param sid
	 */
	public void removeVehicleDevice(String sid) {
		vehicleDeviceMap.remove(sid);
	}

}
