/**
 * 
 */
package com.yeejoin.amos.iot.netty.vehicle;

import java.util.concurrent.BlockingQueue;
import java.util.concurrent.LinkedBlockingQueue;

import org.apache.log4j.Logger;

import com.yeejoin.amos.iot.business.constants.IotConstant;
import com.yeejoin.amos.iot.context.IotContext;
import com.yeejoin.amos.iot.netty.vehicle.dic.VehicleDic.CommonDic;

/**
 * 车载终端数据队列： 1.车载终端监听器接收并解析协议数据，并封装成帧； 2.监听器将一帧数据VehicleData存入队列；
 * 3.从队列取出一帧数据，并调执行后续业务逻辑；
 * 
 * @author as-youjun
 *
 */
public class VehicleDataQueue {

	/**
	 * 单例
	 */
	private static VehicleDataQueue instance;

	/**
	 * 帧队列
	 */
	private static final BlockingQueue<VehicleData> blockingQueue = new LinkedBlockingQueue<VehicleData>();

	/**
	 * 控制帧消费者线程运行状态
	 */
	private static boolean isRunning = true;

	/**
	 * 构造方法
	 */
	private VehicleDataQueue() {
		Logger.getLogger(CommonDic.class).debug(IotConstant.CONSTRUCTOR);
	}

	/**
	 * 获取单例
	 * 
	 * @return
	 */
	public static VehicleDataQueue getInstance() {
		if (instance == null) {
			synchronized (VehicleDataQueue.class) {
				if (instance == null) {
					instance = new VehicleDataQueue();
				}
			}
		}
		return instance;
	}

	/**
	 * 添加数据
	 * 
	 * @param data
	 */
	public void add(VehicleData data) {
		while (blockingQueue.add(data)) {
			break;
		}
	}

	public void start() {
		Thread thread = new Thread(new VehicleDataConsumer(),
				"vehicle data consumer thread");
		thread.start();
	}

	/**
	 * 帧消费者线程
	 * 
	 * @author as-youjun
	 *
	 */
	class VehicleDataConsumer implements Runnable {

		/**
		 * 1.从队列中取出帧；2.调用业务接口；
		 */
		@Override
		public void run() {
			while (isRunning) {
				// 1.从队列中取出帧
				VehicleData data = null;
				try {
					data = blockingQueue.take();
				} catch (InterruptedException e) {
					Logger.getLogger(this.getClass()).error(e);
				}
				// 2.调用业务接口；
				if (data == null) {
					Logger.getLogger(this.getClass()).warn(
							"VehicleData is empty!");
				} else {
					IVehicleDataService vehicleDataService = (IVehicleDataService) IotContext
							.getInstance().getBean("vehicleDataService");
					boolean result = vehicleDataService.process(data);
					data = null;
					if (!result) {
						// 失败数据入库
						Logger.getLogger(this.getClass()).warn("failure...!");
					}
				}
			}
		}

	}

}
