/**
 *
 */
package com.yeejoin.amos.iot.netty.vehicle;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.yeejoin.amos.iot.business.constants.IotConstant;
import com.yeejoin.amos.iot.business.dao.mapper.DeviceMapper;
import com.yeejoin.amos.iot.business.dao.mapper.MonitorMapper;
import com.yeejoin.amos.iot.business.dao.repository.*;
import com.yeejoin.amos.iot.business.entity.*;
import com.yeejoin.amos.iot.business.entity.mybatis.*;
import com.yeejoin.amos.iot.business.enums.CapabilityType;
import com.yeejoin.amos.iot.business.enums.MonitorType;
import com.yeejoin.amos.iot.business.param.QueryCapabilityParam;
import com.yeejoin.amos.iot.business.param.QueryEqpTmplIntfcParam;
import com.yeejoin.amos.iot.business.param.QueryIntfcParam;
import com.yeejoin.amos.iot.business.param.QueryMonitorEqpParam;
import com.yeejoin.amos.iot.business.polling.MonitorUtil;
import com.yeejoin.amos.iot.business.queue.AlarmDataObj;
import com.yeejoin.amos.iot.business.queue.AlarmDataQueue;
import com.yeejoin.amos.iot.business.service.impl.DeviceServiceHelper;
import com.yeejoin.amos.iot.business.service.intfc.IMetricDataService;
import com.yeejoin.amos.iot.business.service.intfc.IMonitorService;
import com.yeejoin.amos.iot.context.IotContext;
import com.yeejoin.amos.iot.core.util.DateJsonValueProcessor;
import com.yeejoin.amos.iot.mongodb.service.intfc.IEquipMetricDataService;
import com.yeejoin.amos.iot.netty.ReceiveEqpCache;
import com.yeejoin.amos.iot.rule.engine.service.IRuleEngineService;
import com.yeejoin.amos.iot.socket.websocket.service.IWebSocketService;
import com.yeejoin.amos.iot.socket.websocket.vo.RealTimeMetricDataVo;
import net.sf.json.JSONObject;
import net.sf.json.JsonConfig;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * 车载终端数据service实现类
 *
 * @author as-youjun
 */
@Service("vehicleDataService")
public class VehicleDataServiceImpl extends BaseVehicleDataService implements
        IVehicleDataService {

    /**
     * 日志记录器
     */
    private static final Logger logger = Logger
            .getLogger(VehicleDataServiceImpl.class);

    @Autowired
    private MonitorMapper processMapper;
    
    @Autowired
    private MonitorMapper monitorMapper;

    @Autowired
    private IMetricDataService metricDataService;

    @Autowired
    private IEquipmentTemplateDao equipmentTemplateDao;
    
    @Autowired
    private IIntfcAttributeDao intfcAttributeDao;

    @Autowired
    private IEquipmentDao equipmentDao;

    @Autowired
    private ICapabilityDao capabilityDao;
    @Autowired
    private IAlarmDao alarmDao;
    @Autowired
    private ICapabilityPropertyDao capabilityPropertyDao;

    @Autowired
    private IEquipMetricDataService equipMetricDataService;

    @Autowired
    private IWebSocketService webSocketService;
    @Autowired
    private IWebSocketService webSocketMorphicService;
    @Autowired
    private DeviceMapper deviceMapper;

    @Autowired
    private IMonitorService monitorService;

    @Autowired
    private IRuleEngineService ruleEngineService;

    @Autowired
    IEqpTmplPropertyDao iEqpTmplPropertyDao;

    /**
     * 线程池
     */
    private ExecutorService threadPool = Executors.newFixedThreadPool(10);

    /**
     * 车载终端数据处理逻辑：1.关联设备/设备模板/能力/接口；2.数据入库；3.产生告警
     */
    @Override
    public boolean process(VehicleData vehicleData) {
        /****
         * 1.首先保证指标数据解析和指标数据入库同步执行;
         * 2.告警/消息推送/组态业务/规则执行等采用异步方式实现,各自模块自行保证业务正确性/效率等
         */
        boolean isUsable = checkVehicleData(vehicleData);
        if (!isUsable) {
            return false;
        }
        boolean result = false;
        List<MetricData> metricDatas = null;
        try {
            // 1.构造指标数据
            metricDatas = buildMetricData(vehicleData);
            result = true;
        } catch (Exception e) {
            logger.error(e);
            result = false;
        }
        if (CollectionUtils.isEmpty(metricDatas)) {
            return result;
        } 
        //2.指标数据保存到mysql
        metricDatas.forEach(metricData -> {
            //缓存上报设备id,以用于设备连通性检测
            ReceiveEqpCache.getInstance().put(metricData.getEqpId());
            // 保存指标数据
            try {
                metricDataService.saveMetricData(metricData);
            } catch (Exception e) {
                logger.error("saveMetricData error!" + e.getMessage());
               
            }
        });
        //3.指标数据保存到mongodb
        metricDatas.forEach(metricData -> {
            try {
                equipMetricDataService.saveMetricData2MongoDb(metricData,vehicleData.getSourceID());
            } catch (Exception e) {
                logger.error("saveMetricData mongodb error!" +e.getMessage());
            }
        });
        //4.产生告警(告警由规则引擎实现...)
        threadPool.execute(new RaiseAlarmTask(metricDatas));
        //5.向前段推送设备指标数据websocket消息
        threadPool.execute(new SendMetricMessageTask(metricDatas));
        //6.向前端组态组件推送数据
        threadPool.execute(new SendMorphiMessageTask(metricDatas));
        // 7.触发性能数据规则
        threadPool.execute(new TriggerRuleTask(metricDatas));
        return result;
    }

    /**
     * 监测VehicleData数据格式是否正确
     *
     * @param vehicleData
     * @return
     */
    private boolean checkVehicleData(VehicleData vehicleData) {
        if (StringUtils.isEmpty(vehicleData.getSourceID())) {
            logger.warn("vehicleData.sourceID is empty!");
            return false;
        }
        return true;
    }

    /**
     * 向前端推送websocket消息
     *
     * @param metricData
     */
    public void sendWebSocketMessage(MetricData metricData) {
        Capability capability = capabilityDao.findById(metricData.getCpblId()).get();
        RealTimeMetricDataVo realTimeMetricDataVo = buildRealTimeMetricDataVo(
                metricData, capability);
        Map<String, Class<?>> map = new HashMap<String, Class<?>>();
        map.put("time", Date.class);
        JsonConfig jsonConfig = new JsonConfig();
        jsonConfig.registerJsonValueProcessor(Date.class,
                new DateJsonValueProcessor(IotConstant.DATE_TIME_STR_YMDHMS));
        String jsonData = JSONObject.fromObject(realTimeMetricDataVo,
                jsonConfig).toString();
        List<EqpNodeSubGraphBo> eqpNodeSubGraphBos = processMapper
                .findEqpNodeGraphByEqpId(metricData.getEqpId());
        if (CollectionUtils.isEmpty(eqpNodeSubGraphBos)) {
            logger.warn("eqpNodeSubGraphBos is empty! no need send message!");
            return;
        }
        List<Long> subGraphIds = new ArrayList<Long>();
        eqpNodeSubGraphBos.forEach(eqpNodeSubGraphBo -> {
            if (!subGraphIds.contains(eqpNodeSubGraphBo.getSubGraphId())) {
                subGraphIds.add(eqpNodeSubGraphBo.getSubGraphId());
            }
        });
        webSocketService.sendMetricDataMessage(subGraphIds, jsonData,
                IotConstant.METRIC_SOCKET);
    }

    /**
     * 构造RealTimeMetricDataVo
     *
     * @param metricData
     * @param capability
     * @return
     */
    private RealTimeMetricDataVo buildRealTimeMetricDataVo(
            MetricData metricData, Capability capability) {
        RealTimeMetricDataVo realTimeMetricDataVo = new RealTimeMetricDataVo();
        realTimeMetricDataVo.setEqpId(metricData.getEqpId());
        realTimeMetricDataVo.setCpblId(metricData.getCpblId());
        realTimeMetricDataVo.setTime(metricData.getExeTime());
        realTimeMetricDataVo.setCpblName(capability.getName());
        List<CapacityPropertyDataVo> capabilityAttributes = deviceMapper
                .getCapacityPropertyById(metricData.getCpblId());
        realTimeMetricDataVo.setMetricDataMap(DeviceServiceHelper
                .buildCpblAttrData(metricData.getMetricJson(),
                        capabilityAttributes));
        return realTimeMetricDataVo;
    }

    /**
     * 构造指标数据
     *
     * @param vehicleData
     * @return
     */
    private List<MetricData> buildMetricData(VehicleData vehicleData) {
        List<MetricData> metricDatas = new ArrayList<>();
        // 1.查询运行中的车载物联网装备
        QueryMonitorEqpParam queryParam = new QueryMonitorEqpParam();
        queryParam.setEnableMonitor(true);
        queryParam.setMonitorType(MonitorType.LISTENING.getValue());
        List<EquipmentBo> equipmentBos = monitorService.findMonitorEquipmentDet(queryParam);
        if (CollectionUtils.isEmpty(equipmentBos)) {
            return metricDatas;
        }
        // 2.构造指标数据
        metricDatas = buildMetricData(vehicleData, equipmentBos);
        return metricDatas;
    }

    /**
     * 构造指标数据,待测试完成后删除
     *
     * @param vehicleData
     * @param equipmentBos
     * @return
     */
//    private List<MetricData> buildMetricData(VehicleData vehicleData,
//                                             List<EquipmentBo> equipmentBos) {
//        List<MetricData> metricDatas = new ArrayList<>();
//        if (StringUtils.isEmpty(vehicleData.getSourceID())) {
//            return metricDatas;
//        }
//        // 1.获取设备标识
//        long startTime4 = System.currentTimeMillis();
//        Map<Long, Map<String, Object>> eqpParmaValueMap = getEqpParamValueMap(equipmentBos);
//        long startTime5 = System.currentTimeMillis();
//        // 2.根据sourceId获取设备
//        List<EquipmentBo> mebos = getMebosBySourceId(vehicleData.getSourceID(),
//                equipmentBos, eqpParmaValueMap);
//        long startTime6 = System.currentTimeMillis();
//        if (CollectionUtils.isEmpty(mebos)) {
//            logger.warn("monitor EquipmentBos is empty!");
//            return metricDatas;
//        }
//        for (EquipmentBo mebo : mebos) {
//            // 3.遍历能力
//            List<CapabilityBo> capabilityBos = mebo.getCapabilityBos();
//            if (CollectionUtils.isEmpty(capabilityBos)) {
//                return metricDatas;
//            }
//            for (CapabilityBo capabilityBo : capabilityBos) {
//                MetricData metricData = buildSingleMetricData(vehicleData,
//                        mebo, capabilityBo);
//                if (metricData != null) {
//                    metricDatas.add(metricData);
//                } else {
//                    logger.debug("metricData is null! capabilityBo.id:"
//                            + capabilityBo.getId() + ",capabilityBo.name:"
//                            + capabilityBo.getName());
//                }
//            }
//        }
//        return metricDatas;
//    }
    
    /**
     * 匹配设备、填充能力接口、构造指标数据
     *
     * @param vehicleData
     * @param equipmentBos
     * @return
     */
    private List<MetricData> buildMetricData(VehicleData vehicleData,
                                             List<EquipmentBo> equipmentBos) {
        List<MetricData> metricDatas = new ArrayList<>();
        if (StringUtils.isEmpty(vehicleData.getSourceID())) {
            return metricDatas;
        }
        // 1.获取设备标识
        Map<Long, Map<String, Object>> eqpParmaValueMap = getEqpParamValueMap(equipmentBos);
        // 2.根据sourceId获取设备
        List<EquipmentBo> mebos = getMebosBySourceId(vehicleData.getSourceID(), equipmentBos, eqpParmaValueMap);
        if (CollectionUtils.isEmpty(mebos)) {
            return metricDatas;
        }
        // 3.查询设备能力
        List<Long> eqpTmplIds = new ArrayList<>();
        mebos.forEach(eqpb -> {
            if (!eqpTmplIds.contains(eqpb.getEqpTmplId())) {
                eqpTmplIds.add(eqpb.getEqpTmplId());
            }
        });
        QueryCapabilityParam queryCapabilityParam = new QueryCapabilityParam();
        queryCapabilityParam.setCpblType(CapabilityType.COLLECT.getValue());
        queryCapabilityParam.setEqpTmplIds(eqpTmplIds);
        List<CapabilityBo> capabilityBos = monitorMapper.findCpblByEqpTmplIds(queryCapabilityParam);
        if (CollectionUtils.isNotEmpty(capabilityBos)) {
            // CapabilityBo填充到EquipmentBo
            MonitorUtil.fillCapabilityBo2EquipmentBo(capabilityBos, mebos);
        } else {
            return metricDatas;
        }
        // 4.查询设备能力接口
        QueryIntfcParam queryIntfcParam = new QueryIntfcParam();
        queryIntfcParam.setCpblType(CapabilityType.COLLECT.getValue());
        queryIntfcParam.setEqpTmplIds(eqpTmplIds);
        List<IntfcBo> intfcBos = monitorMapper.findIntfcOfCpblByEqpTmplIds(queryIntfcParam);
        if (CollectionUtils.isNotEmpty(intfcBos)) {
            // IntfcBo填充到CapabilityBo
            mebos.forEach(equipmentBo -> {
                MonitorUtil.fillIntfcBo2CapabilityBo(intfcBos, equipmentBo.getCapabilityBos());
            });
            // 接口属性填充到接口
            List<Long> intfcIds = new ArrayList<Long>();
            intfcBos.forEach(item -> {
                intfcIds.add(item.getId());
            });
            List<IntfcAttribute> intfcAttributes = intfcAttributeDao.findAllByIntfcIdIn(intfcIds);   
            if (CollectionUtils.isNotEmpty(intfcAttributes)) {
                mebos.forEach(equipmentBo -> {
                    if (CollectionUtils.isNotEmpty(equipmentBo.getCapabilityBos())) {
                        equipmentBo.getCapabilityBos().forEach(capabilityBo -> {
                            MonitorUtil.fillIntfcAttr2Intfc(intfcAttributes, capabilityBo.getIntfcBos());
                        });
                    }
                });
            } else {
                logger.debug("intfcAttributes is empty!");
            }
            //能力属性填充到能力
            fillCpblAttr2Cpbl(mebos);
        }
        // 5.查询设备接口
        QueryEqpTmplIntfcParam queryEqpTmplIntfcParam = new QueryEqpTmplIntfcParam();
        queryEqpTmplIntfcParam.setEqpTmplIds(eqpTmplIds);
        List<EqpTmplIntfcBo> eqpTmplIntfcBos = monitorMapper.findEqpTmplIntfcByEqpTmplIds(queryEqpTmplIntfcParam);
        if (CollectionUtils.isNotEmpty(eqpTmplIntfcBos)) {
            //设备模板接口填充到设备
            MonitorUtil.fillEqpTmplIntfc2EquipmentBo(eqpTmplIntfcBos, mebos);
            //接口填充到设备模板接口
            mebos.forEach(equipmentBo -> {
                MonitorUtil.fillIntfcBo2EqpTmplIntfcBo(intfcBos, equipmentBo.getEqpTmplIntfcs());
            });
        } else {
            logger.debug("eqpTmplIntfcBos is empty!");
        }
        //6.构造指标
        for (EquipmentBo mebo : mebos) {
            // 6.1.遍历能力
            List<CapabilityBo> capabilityBos1 = mebo.getCapabilityBos();
            if (CollectionUtils.isEmpty(capabilityBos1)) {
                return metricDatas;
            }
            for (CapabilityBo capabilityBo : capabilityBos1) {
                MetricData metricData = buildSingleMetricData(vehicleData,
                        mebo, capabilityBo);
                if (metricData != null) {
                    metricDatas.add(metricData);
                } else {
                    logger.debug("==metricData is null==! capabilityBo.id:"
                            + capabilityBo.getId() + ",capabilityBo.name:"
                            + capabilityBo.getName());
                }
            }
        }
        return metricDatas;
    }
    /**
     * 能力属性填充到能力
     *
     * @param equipmentBos
     */
    private void fillCpblAttr2Cpbl(List<EquipmentBo> equipmentBos) {
        if (CollectionUtils.isEmpty(equipmentBos)) {
            return;
        }
        //1.获取批量设备的能力属性
        List<Long> cpblIds = new ArrayList<>();
        equipmentBos.forEach(ebo -> {
            if (CollectionUtils.isNotEmpty(ebo.getCapabilityBos())) {
                ebo.getCapabilityBos().forEach(cbo -> {
                    if (!cpblIds.contains(cbo.getId())) {
                        cpblIds.add(cbo.getId());
                    }
                });
            } else {
                logger.debug("ebo.getCapabilityBos() is empty when equipmentBoId=" + ebo.getId());
            }
        });
        if (CollectionUtils.isEmpty(cpblIds)) {
            return;
        }
        List<CapacityPropertyDataVo> cpblAttrs = deviceMapper
                .getCapacityPropertyByCpblIds(cpblIds);
        if (CollectionUtils.isEmpty(cpblAttrs)) {
            return;
        }
        //2.填充设备能力属性
        equipmentBos.forEach(ebo -> {
            if (CollectionUtils.isNotEmpty(ebo.getCapabilityBos())) {
                Map<String, List<Object>> cpblAttrMap = (Map<String, List<Object>>) ebo
                        .getParamMap().get(IotConstant.EquipmentConstant.QUOTASELECTED);
                if (MapUtils.isNotEmpty(cpblAttrMap)) {
                    MonitorUtil.fillCpblAttr2Capablity(cpblAttrs, ebo.getCapabilityBos(), cpblAttrMap);
                } else {
                    logger.debug("cpblAttrMap is empty!");
                }
            } else {
                logger.debug("ebo.getCapabilityBos() is empty when equipmentBoId=" + ebo.getId());
            }
        });
    }
    /**
     * 构建单个指标数据
     *
     * @param vehicleData
     * @param equipmentBo
     * @param capabilityBo
     * @return
     */
    @SuppressWarnings("unchecked")
    private MetricData buildSingleMetricData(VehicleData vehicleData,
                                             EquipmentBo equipmentBo, CapabilityBo capabilityBo) {
        MetricData metricData = null;     
        if (CollectionUtils.isEmpty(capabilityBo.getCpblAttrs())) {
            return metricData;
        }
        // 帧数据
        Map<String, Object> frameMap = (Map<String, Object>) JSONObject.toBean(
                JSONObject.fromObject(vehicleData.getFrameJson()), Map.class);
        // 是否是设备的指标数据
        boolean isEqpMetricData = false;
        boolean isFrameTypeEqual = false;
        boolean isLableTypeEqual = false;
        Integer frameType = VechicleServiceHandler.getAttrValueInt(
                capabilityBo.getCpblAttrs(),
                IotConstant.EquipmentConstant.FRAMETYPE);// 帧类型
        if ((frameType != null)
                && (Integer.valueOf(vehicleData.getFrameType())
                .equals(frameType))) {
            isFrameTypeEqual = true;
        }
        Integer labelType = VechicleServiceHandler.getAttrValueInt(
                capabilityBo.getCpblAttrs(),
                IotConstant.EquipmentConstant.LABELTYPE);// 标签类型
        if ((labelType != null)
                && (vehicleData.getLabelType() != null)
                && (Integer.valueOf(vehicleData.getLabelType())
                .equals(labelType))) {
            isLableTypeEqual = true;
        }
        if (vehicleData.getLabelType() == null) {
            // 标签类型为null，只需匹配frameType
            isEqpMetricData = isFrameTypeEqual;
        } else {
            // 需匹配frameType、labelType
            isEqpMetricData = isFrameTypeEqual && isLableTypeEqual;
        }
        if (isEqpMetricData) {
            metricData = buildSingleMetricData(equipmentBo, vehicleData, capabilityBo, frameMap);
        }
        return metricData;
    }

    /**
     * 构建单个指标数据
     *
     * @param equipmentBo
     * @param vehicleData
     * @param capabilityBo
     * @param frameMap
     * @return
     */
    @SuppressWarnings("unchecked")
    private MetricData buildSingleMetricData(EquipmentBo equipmentBo,
                                             VehicleData vehicleData, CapabilityBo capabilityBo,
                                             Map<String, Object> frameMap) {
        MetricData metricData = new MetricData();
        metricData.setExeTime(vehicleData.getTime());
        metricData.setEqpId(equipmentBo.getId());
        metricData.setEqpTmplId(equipmentBo.getEqpTmplId());
        metricData.setCpblId(capabilityBo.getId());
        metricData.setIsDelete(false);
        metricData.setPeriod(0);
        metricData.setCreateTime(new Date());
        metricData.setSubgraphId(equipmentBo.getGraphId());
        metricData.setMetricKey(capabilityBo.getId() + IotConstant.BAR+ capabilityBo.getName());
        // ids的数据是集合，特殊处理
        try {
            if (frameMap.get("ids") != null) {
                List<Object> ids = (List<Object>) frameMap.get("ids");
                String idStr = "";
                for (Object s : ids) {
                    idStr += s.toString() + ",";
                }
                if (idStr.contains(",")) {
                    idStr = idStr.substring(0, idStr.length() - 1);
                }
                frameMap.put("labelStr", idStr);
                // frameMap.remove("ids");
            }
        } catch (Exception e) {
            logger.error("buildSingleMetricData error! " + e);
        }
        metricData.setMetricJson(JSONObject.fromObject(frameMap).toString());
        return metricData;
    }

    /**
     * 处理告警
     *
     * @param metricData
     */
    public void raiseAlarm(MetricData metricData) {
        AlarmDataObj alarmDataObj = new AlarmDataObj(metricData);
        alarmDataObj.setType(IotConstant.METRIC);
        AlarmDataQueue.getInstance().add(alarmDataObj);
        logger.info("指标越界告警源数据,sid:" + metricData.getEqpId());
    }

    /**
     * 向前端组态推送websocket消息
     *
     * @param metricData
     */
    public void sendMorphicWebSocketMessage(MetricData metricData) {
        Equipment equipment = equipmentDao.findById(metricData.getEqpId()).get();
        Map<String, Object> resultMap = new HashMap<>();
        Map<String, List<RealTimeMetricDataVo>> realTimeMetricDataVoMap = new HashMap<>();
        List<RealTimeMetricDataVo> realTimeMetricDataVoList = new ArrayList<>();
        Capability capability = capabilityDao.findById(metricData.getCpblId()).get();
        RealTimeMetricDataVo realTimeMetricDataVo = new RealTimeMetricDataVo();
        realTimeMetricDataVo.setEqpId(metricData.getEqpId());
        realTimeMetricDataVo.setCpblId(metricData.getCpblId());
        realTimeMetricDataVo.setCpblName(capability.getName());
        realTimeMetricDataVo.setTime(new Date());

        Map<String, Object> map = new HashMap<>();
        map.put("eqpId", metricData.getEqpId());
        map.put("cpblId", metricData.getCpblId());
        Map<String, String> paramMap = new HashMap<>();
        List<CapabilityAttribute> capabilityAttributes = capabilityPropertyDao
                .findByCapabilityId(metricData.getCpblId());
        if (metricData != null && metricData.getMetricJson() != null && metricData.getMetricJson() != "") {
            paramMap = DeviceServiceHelper.buildCpblAttrDatas(metricData.getMetricJson(), capabilityAttributes);
        } else {
            for (CapabilityAttribute capacityPropertyDataVo : capabilityAttributes) {
                paramMap.put(capacityPropertyDataVo.getDisplayName(), "");
            }
        }
        realTimeMetricDataVo.setMetricDataMap(paramMap);
        realTimeMetricDataVoList.add(realTimeMetricDataVo);
        realTimeMetricDataVoMap.put(metricData.getEqpId().toString(), realTimeMetricDataVoList);

        resultMap.put("metricData", realTimeMetricDataVoMap);
        resultMap.put("isWarning", equipment.getIsWarning());
        // 计算告警最高等级
        List<Alarm> alarms = alarmDao.fetchAlarmsByRqpId(equipment.getId());
        if (alarms != null) {
            Integer maxLevel = alarms.stream().mapToInt(i -> i.getLevel()).summaryStatistics().getMin();
            resultMap.put("maxLevel", maxLevel);
        }
        String jsonData = JSON.toJSONString(resultMap, SerializerFeature.WriteMapNullValue);
//        System.out.println("组态：" + jsonData);
        //发送结果
        webSocketMorphicService.sendMetricDataMessage(jsonData, IotConstant.MORPHIC_SOCKET);
    }

    /**
     * 告警处理
     */
    class RaiseAlarmTask implements Runnable {

        private List<MetricData> metricDatas;

        public RaiseAlarmTask(List<MetricData> metricDatas) {
            this.metricDatas = metricDatas;
        }

        @Override
        public void run() {
            if (CollectionUtils.isNotEmpty(metricDatas)) {
                for (MetricData metricData : metricDatas) {
                    // 产生告警(告警由规则引擎实现...)
                    raiseAlarm(metricData);
                }
            }
        }
    }

    /**
     * 指标数据推送到前端
     */
    class SendMetricMessageTask implements Runnable {

        private List<MetricData> metricDatas;

        public SendMetricMessageTask(List<MetricData> metricDatas) {
            this.metricDatas = metricDatas;
        }

        @Override
        public void run() {
            if (CollectionUtils.isNotEmpty(metricDatas)) {
                for (MetricData metricData : metricDatas) {
                    // 向前段推送设备指标数据websocket消息
                    try {
                        sendWebSocketMessage(metricData);
                    } catch (Exception e) {
                        logger.error("sendWebSocketMessage error!" + e);
                    }
                }
            }
        }
    }

    /**
     * 向前端组态组件推送数据
     */
    class SendMorphiMessageTask implements Runnable {

        private List<MetricData> metricDatas;

        public SendMorphiMessageTask(List<MetricData> metricDatas) {
            this.metricDatas = metricDatas;
        }

        @Override
        public void run() {
            if (CollectionUtils.isNotEmpty(metricDatas)) {
                for (MetricData metricData : metricDatas) {
                    //向前端组态组件推送数据
                    try {
                        sendMorphicWebSocketMessage(metricData);
                    } catch (Exception e) {
                        e.printStackTrace();
                        logger.error("sendMorphicWebSocketMessage error!" + e);
                    }
                }
            }
        }
    }

    /**
     * 规则执行
     */
    class TriggerRuleTask implements Runnable {

        private List<MetricData> metricDatas;

        public TriggerRuleTask(List<MetricData> metricDatas) {
            this.metricDatas = metricDatas;
        }

        @Override
        public void run() {
            if (CollectionUtils.isNotEmpty(metricDatas)) {
                try {
                    MetricDataRo metricDataRo = new MetricDataRo(metricDatas);
                    MetricData metricData = metricDatas.get(0);
                    Equipment equipment = equipmentDao.findById(metricData.getEqpId()).get();
                    metricDataRo.setEqpId(metricData.getEqpId());
                    metricDataRo.setSubgraphId(metricData.getSubgraphId());
                    metricDataRo.setOrgCode(equipment.getOrgCode());
                    metricDataRo.setAuthOrg(equipment.getAuthOrg());
                    metricDataRo.setEqpName(equipment.getName());
                    List<Map<String, Object>> propertys = new ArrayList<>();
                    Map<String, Object> configs = equipment.getCfgParamMap();
                    configs.keySet().forEach(item -> {
                        if (item.startsWith("equip_")) {
                            Map<String, Object> tempProperty = new HashMap<>();
                            String idStr = item.substring(6, item.length());
                            tempProperty.put("id", idStr);
                            tempProperty.put("valueStr", configs.get(item));
                            EqpTmplAttribute eqpTmplAttribute = iEqpTmplPropertyDao.findById(Long.valueOf(idStr)).get();
                            if (eqpTmplAttribute != null) {
                                tempProperty.put("displayName", eqpTmplAttribute.getDisplayName());
                                propertys.add(tempProperty);
                            }
                        }
                    });
                    metricDataRo.setEqpConfigMap(propertys);

                    ruleEngineService.fireRule(metricDataRo);
                } catch (Exception e) {
                    logger.error(e);
                }
            }
        }
    }

}