package com.yeejoin.amos.iot.netty.vehicle;

import org.apache.log4j.Logger;

import com.yeejoin.amos.iot.business.constants.IotConstant;

/**
 * 车载终端帧
 * 
 * @author as-youjun
 *
 */
public class VehicleFrame {

	/**
	 * 帧头大小
	 */
	public final static int HEAD_LENGTH = 4;

	/**
	 * 帧尾大小
	 */
	public final static int TAIL_LENGTH = 3;

	/**
	 * 开始标识
	 */
	public final static byte START = (byte) 0xAF;

	/**
	 * 结束标识
	 */
	public final static byte END = (byte) 0xFF;

	/**
	 * 帧头
	 */
	public byte[] header = null;

	/**
	 * 帧体
	 */
	public byte[] body = null;

	/**
	 * 帧尾
	 */
	public byte[] tail = null;

	public VehicleFrame() {
		Logger.getLogger(this.getClass()).debug(IotConstant.CONSTRUCTOR);
	}

	public VehicleFrame(byte[] frame) {
		this.header = new byte[HEAD_LENGTH];
		this.body = new byte[frame.length - HEAD_LENGTH - TAIL_LENGTH];
		this.tail = new byte[TAIL_LENGTH];
		System.arraycopy(frame, 0, this.header, 0, this.header.length);
		System.arraycopy(frame, HEAD_LENGTH, this.body, 0, this.body.length);
		System.arraycopy(frame, frame.length - TAIL_LENGTH, this.tail, 0, this.tail.length);
	}

	public VehicleFrame(byte[] header, byte[] body, byte[] tail) {
		this.header = header;
		this.body = body;
		this.tail = tail;
	}

	/**
	 * 获取carCode消防车编码
	 * 
	 * @return
	 */
	public String getSid() {
		String sid = null;
		if (body != null && body.length > 2) {
			sid = MessageCommonUtil.bytes2HexString(MessageCommonUtil.getAnyPartMessage(body, 0, 2));
		}
		// 实际模块序列号为长度为6的十六进制字符串（如："000001"），两个字节的id转换成十六进制字符串后需要补00来满足次此格式
		return "00" + sid;
	}

	public static VehicleFrame createFrameByBody(byte[] body) {
		if (body == null) {
			return null;
		}
		byte[] header = createHeader(body);
		byte[] tail = createTail(header, body);
		VehicleFrame frame = new VehicleFrame(header, body, tail);
		return frame;
	}

	public static VehicleFrame createFrameByBodyWhithoutType(byte[] body) {
		if (body == null) {
			return null;
		}

		byte[] header = createHeader(body);
		byte[] tail = createTail(header, body);
		return new VehicleFrame(header, body, tail);
	}

	public static byte[] createHeader(byte[] body) {
		byte[] length = MessageCommonUtil.convertInt2TwoBytesInBigEnd(body.length);
		byte[] header = new byte[HEAD_LENGTH];
		header[0] = START;
		header[1] = length[0];
		header[2] = length[1];
		header[3] = START;
		return header;
	}

	public static byte[] createTail(byte[] header, byte[] body) {
		byte[] crcData = new byte[header.length + body.length];
		System.arraycopy(header, 0, crcData, 0, header.length);
		System.arraycopy(body, 0, crcData, header.length, body.length);
		byte[] crc = MessageCommonUtil.hexStr2ByteArray(ComputeCRCUtil.calcCrc16(crcData));
		byte[] tail = new byte[TAIL_LENGTH];
		tail[0] = crc[1];
		tail[1] = crc[0];
		tail[2] = END;
		return tail;
	}

	/**
	 * 帧字节数组
	 * 
	 * @return
	 */
	public byte[] getFrame() {
		byte[] framebytes = new byte[header.length + body.length + tail.length];
		System.arraycopy(header, 0, framebytes, 0, header.length);
		System.arraycopy(body, 0, framebytes, header.length, body.length);
		System.arraycopy(tail, 0, framebytes, header.length + body.length, tail.length);
		return framebytes;
	}

	/**
	 * 获取数据
	 * 
	 * @return
	 */
	public byte[] data() {
		return MessageCommonUtil.getAnyPartMessage(body, 3, body.length - 3);
	}

	/**
	 * 帧总大小
	 * 
	 * @return
	 */
	public int size() {
		return header.length + body.length + tail.length;
	}

	/**
	 * 帧类型
	 * 
	 * @return
	 */
	public byte type() {
		return body[2];
	}

}
