/**
 *
 */
package com.yeejoin.amos.iot.netty.vehicle;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.concurrent.BlockingQueue;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.LinkedBlockingQueue;
import com.yeejoin.amos.iot.context.IotContext;

/**
 * 帧队列：缓存上报数据帧
 *
 * @author as-youjun
 *
 */
public class VehicleFrameQueue {

    private static Logger logger = Logger.getLogger(VehicleFrameQueue.class);

//    @Autowired
//    private IVehicleFrameService iVehicleFrameService;

    /**
     * 单例
     */
    private static VehicleFrameQueue instance;

    /**
     * 帧队列
     */
    private static final BlockingQueue<byte[]> blockingQueue = new LinkedBlockingQueue<byte[]>();

    /**
     * 控制帧消费者线程运行状态
     */
    private static boolean isRunning = true;

    //线程池大小
    private static final int poolSize = 10;

    //固定线程池
    private static ExecutorService executorService = Executors.newFixedThreadPool(poolSize);

    /**
     * 构造方法
     */
    private VehicleFrameQueue() {
    }

    /**
     * 获取单例
     *
     * @return
     */
    public static VehicleFrameQueue getInstance() {
        if (instance == null) {
            synchronized (VehicleFrameQueue.class) {
                if (instance == null) {
                    instance = new VehicleFrameQueue();
                }
            }
        }
        return instance;
    }

    /**
     * 添加数据
     *
     * @param frame
     */
    public void add(byte[] frame) {
        while (blockingQueue.add(frame)) {
            break;
        }
    }

    public void start() {
        for (int i = 0; i < poolSize; i++) {
            executorService.execute(new VehicleFrameConsumer());
        }
    }

    /**
     * 帧消费者线程
     *
     * @author as-youjun
     *
     */
    class VehicleFrameConsumer implements Runnable {

        /**
         * 1.从队列中取出帧；2.调用业务接口；
         */
        @Override
        public void run() {
            while (isRunning) {
                // 1.从队列中取出帧
                byte[] frame = null;
                try {
                    frame = blockingQueue.take();
                } catch (InterruptedException e) {
                    logger.error("take frame happened error", e);
                }
                // 2.调用业务接口；
                if (frame == null) {
                    logger.warn("frame is empty!");
                } else {
                    try {
                        IVehicleFrameService vehicleFrameService = (IVehicleFrameService) IotContext.getInstance().getBean(("vehicleFrameService"));
                        vehicleFrameService.parseData(frame);
                        frame = null;
                    } catch (Exception e) {
                        logger.error("parse frame happened error", e);
                        // 失败处理
                    }
                }
            }
        }

    }

}
