/**
 *
 */
package com.yeejoin.amos.iot.netty.vehicle;

import com.yeejoin.amos.iot.business.constants.IotConstant;
import com.yeejoin.amos.iot.business.dao.repository.IFrameProtocolDao;
import com.yeejoin.amos.iot.business.dic.ScriptLang;
import com.yeejoin.amos.iot.business.service.intfc.IDicService;
import com.yeejoin.amos.iot.business.service.intfc.IProtocolService;
import com.yeejoin.amos.iot.business.vo.ProtocolDicVo;
import com.yeejoin.amos.iot.core.util.DateUtil;
import com.yeejoin.amos.iot.entity.ParseResult;
import com.yeejoin.amos.iot.netty.vehicle.test.TestVehicleDataQueue;
import com.yeejoin.amos.iot.script.IScriptService;
import net.sf.json.JSONObject;
import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;

/**
 * 帧服务实现类
 *
 * @author as-youjun
 */
@Service("vehicleFrameService")
public class VehicleFrameServiceImpl implements IVehicleFrameService {

    @Autowired
    private IProtocolService protocolService;

    @Autowired
    private IDicService dicService;

    @Autowired
    private IScriptService groovyScriptService;

    @Autowired
    private IScriptService javaScriptService;

    @Autowired
    private IFrameProtocolDao frameProtocolDao;

    /**
     * 解析协议数据
     */
    @Override
    public void parseData(byte[] frameBytes) {
        if (frameBytes == null) {
            return;
        }
        VehicleFrame frame = new VehicleFrame(frameBytes);
        frameBytes = null;
        // 如果是心跳帧，则不做任务处理
        if (frame.type() == VehicleFrameType.HEART_BEAT) {
            return;
        }

        String sid = frame.getSid();
        int frameType = frame.type();
        // 2.调用groovy/javascript代码解析帧体中的数据
//        FrameProtocol frameProtocol = frameProtocolDao.findByFrameType(frameType);
//        if (frameProtocol == null) {
//            return;
//        }
//        Long protocolId = frameProtocol.getProtocolId();
//        if (protocolId == null) {
//            return;
//        }
//        ProtocolVo protocolVo = protocolService.findById(protocolId);
//        Dic dic = dicService.getById(String.valueOf(protocolVo.getDicIdScriptLang()));
        ProtocolDicVo protocolDicVo = protocolService.findProtocolDicVoByFrameType(frameType);
        // 调用groovy/javascript代码解析协议数据
        Object[] args = new Object[]{frame.data(), sid};
        ParseResult parseResult = null;
        switch (protocolDicVo.getCode()) {
            case ScriptLang.GROOVY:
                // groovy解析协议数据
                parseResult = groovyScriptService.analyze(protocolDicVo.getScriptCode(), args);
                break;
            case ScriptLang.JAVASCRIPT:
                // javascript解析协议数据
                parseResult = javaScriptService.analyze(protocolDicVo.getScriptCode(), args);
                break;
            case ScriptLang.PYTHON:
                // python解析协议数据---暂未实现
                break;
            case ScriptLang.LUA:
                // lua解析协议数据---暂未实现
                break;
            default:
                // 不执行任何业务
                break;
        }
        if (parseResult == null) {
            return;
        }
        JSONObject resultJsonObj = JSONObject.fromObject(parseResult.getResult());
        if (resultJsonObj == null) {
            return;
        }
        if (resultJsonObj.containsKey("errorMessage")) {
            Object type = resultJsonObj.get("type");
            VehicleFrameCache.getInstance().unnormalFrame(sid, String.valueOf(frameType), type);
            return;
        }
        VehicleData vehicleData = new VehicleData();
        vehicleData.setSourceID(resultJsonObj.getString("sourceId"));
        vehicleData.setFrameType(frame.type());
        Object type = resultJsonObj.get("type");
        if (type != null) {
            if (type instanceof Byte) {
                vehicleData.setLabelType((Byte) type);
            } else if (type instanceof Integer) {
                Integer v = (Integer) type;
                vehicleData.setLabelType(v.byteValue());
            }
        }
        vehicleData.setFrameJson(resultJsonObj.toString());
        Date now = new Date();
        Date exeTime;
        try {
            if (StringUtils.isEmpty(resultJsonObj.getString("formatTime"))) {
                exeTime = now;// 当前时间
            } else {
                exeTime = DateUtil.str2Date(resultJsonObj.getString("formatTime"), IotConstant.DATE_TIME_STR_YMDHMS);
            }
        } catch (Exception e) {
            exeTime = now;// 当前时间
        }
        if (exeTime == null) {
            exeTime = now;
        }
        vehicleData.setTime(exeTime);
        boolean isValid = isFrameValid(resultJsonObj);
        // 解析结果数据放入队列
        if (isValid) {
            VehicleDataQueue.getInstance().add(vehicleData);
            VehicleFrameCache.getInstance().normalFrame(sid, String.valueOf(frameType), type);
            //放入设备测试队列
            TestVehicleDataQueue.getInstance().add(vehicleData);
        } else {
            Logger.getLogger(this.getClass()).warn("isValid=fale,:" + resultJsonObj);
        }
    }

    /**
     * 监测帧数据是否有效
     *
     * @param json
     * @return
     */
    private boolean isFrameValid(JSONObject json) {
        boolean isValid = true;
        try {
            String status = (String) json.get("status");
            if ("信息不可用".equals(status)) {
                isValid = false;
            }
        } catch (Exception e) {
            Logger.getLogger(this.getClass()).error(e);
        }
        return isValid;
    }
}
