package com.yeejoin.amos.iot.netty.vehicle;

import java.util.HashMap;
import java.util.Map;

import org.apache.log4j.Logger;

import com.yeejoin.amos.iot.business.constants.IotConstant;

/**
 * <pre>
 * 帧类型
 * </pre>
 *
 * @author as-mafeipeng
 * @version $Id: FrameType.java, v 0.1 2017年6月17日 下午6:03:11 as-mafeipeng Exp $
 */
public class VehicleFrameType {

    private VehicleFrameType() {
        Logger.getLogger(this.getClass()).debug(IotConstant.CONSTRUCTOR);
    }

    // 采集器心跳
    public final static byte HEART_BEAT = (byte) 0x7F;

    // 从采集器到服务器主动上报
    public final static byte SAMPLER_ACTIVE_REPORT_GPS = (byte) 0x01; // 采集器主动上报GPS数据帧
    public final static byte SAMPLER_ACTIVE_REPORT_LORA = (byte) 0x02; // 采集器主动上报LoRa数据帧
    public final static byte SAMPLER_ACTIVE_REPORT_POWER_INFO = (byte) 0x03; // 采集器主动上报电源数据帧

    // 从平板到服务器主动上报
    public final static byte TABLET_ACTIVE_REPORT_GPS = (byte) 0x77; // 平板主动上报GPS数据帧
    public final static byte TABLET_ACTIVE_REPORT_POWER_INFO = (byte) 0x78; // 平板主动上报电源数据帧
    public final static byte TABLET_ACTIVE_REPORT_COMMON_LABEL = (byte) 0x70; // 平板主动上报新普通标签数据帧（丁长波方案）
    public final static byte TABLET_ACTIVE_REPORT_OBD = (byte) 0x71; // 平板主动上报OBD数据帧（丁长波方案）
    public final static byte TABLET_ACTIVE_REPORT_TPMS = (byte) 0x72; // 平板主动上报胎压数据帧（丁长波方案）
    public final static byte TABLET_ACTIVE_REPORT_LIQUID_LEVEL = (byte) 0x73; // 平板主动上报上装液位数据帧（丁长波方案）

    // 服务器查询参数（请求/响应）
    // 控制方向从服务器到采集器
    // 以上报SAMPLER_ACTIVE_REPORT_GPS和SAMPLER_ACTIVE_REPORT_LORA的形式响应
    public final static byte SAMPLER_QUERY_CAR_INFO = (byte) 0x81; // 车辆信息查询帧
    public final static byte SAMPLER_QUERY_PROGRAME_VERSION = (byte) 0x8A; // 查询采集器程序版本帧

    // 服务器远程控制(发送/确认)
    // 控制方向从服务器到采集器
    public final static byte SAMPLER_REMOTE_CONTROL_RESTART = (byte) 0x82; // 远程控制采集器重启帧
    public final static byte SAMPLER_REMOTE_CONTROL_SHUTDOWN = (byte) 0x83; // 远程控制采集器关机帧

    // 服务器设置参数(发送/确认)
    // 控制方向从服务器到采集器
    public final static byte SAMPLER_CONFIGURE_REPORT_GPS_INTERVAL = (byte) 0x84; // 配置采集器GPS上报间隔帧
    public final static byte SAMPLER_CONFIGURE_REPORT_LORA_INTERVAL = (byte) 0x85; // 配置采集器Lora上报间隔帧

    // 服务器远程升级程序(发送/确认)
    // 控制方向从服务器到采集器
    public final static byte SAMPLER_REMOTE_UPDATE_PROGRAME_START = (byte) 0x86; // 采集器远程升级程序开始帧
    public final static byte SAMPLER_REMOTE_UPDATE_PROGRAME_FILE_NUMBER = (byte) 0x87; // 采集器远程升级程序数据数量帧
    public final static byte SAMPLER_REMOTE_UPDATE_PROGRAME_FILE = (byte) 0x88; // 采集器远程升级程序数据帧

    // 采集器确认帧
    // 控制方向从采集器到服务器
    public final static byte REMOTE_CONTROL_RESTART_RES = (byte) 0x82; // 远程控制采集器重启确认帧
    public final static byte REMOTE_CONTROL_SHUTDOWN_RES = (byte) 0x83; // 远程控制采集器关机确认帧
    public final static byte CONFIGURE_REPORT_GPS_INTERVAL_RES = (byte) 0x84; // 配置采集器GPS上报确认帧
    public final static byte CONFIGURE_REPORT_LORA_INTERVAL_RES = (byte) 0x85; // 配置采集器Lora上报间隔帧
    public final static byte QUERY_PROGRAME_VERSION_RES = (byte) 0x0A; // 配置采集器Lora上报间隔帧
    public final static byte SAMPLER_REMOTE_UPDATE_PROGRAME_START_RES = (byte) 0x06; // 采集器远程升级程序开始确认帧
    public final static byte SAMPLER_REMOTE_UPDATE_PROGRAME_FILE_NUMBER_RES = (byte) 0x07; // 采集器远程升级程序数据数量确认帧
    public final static byte SAMPLER_REMOTE_UPDATE_PROGRAME_FILE_RES = (byte) 0x08; // 采集器远程升级程序数据确认帧
    public final static byte SAMPLER_REMOTE_UPDATE_PROGRAME_END = (byte) 0x09; // 采集器远程升级程序完成帧

    //气体灭火临时方案使用帧类型
    public final static byte GASEOUS_FIRE_SUPPRESSION = (byte) 0xF1;

}