/**
 *
 */
package com.yeejoin.amos.iot.netty.vehicle.test;

import com.yeejoin.amos.iot.business.constants.IotConstant;
import com.yeejoin.amos.iot.business.enums.IntfcDirection;
import com.yeejoin.amos.iot.business.polling.test.TestEqpInputData;
import com.yeejoin.amos.iot.business.polling.test.TestEquipmentCache;
import com.yeejoin.amos.iot.business.polling.test.TestIntfc;
import com.yeejoin.amos.iot.business.polling.test.TestUtil;
import com.yeejoin.amos.iot.context.IotContext;
import com.yeejoin.amos.iot.netty.vehicle.VehicleData;
import com.yeejoin.amos.iot.netty.vehicle.dic.VehicleDic.CommonDic;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;

import java.util.Map;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.LinkedBlockingQueue;

/**
 * 自研协议在线测试队列
 *
 * @author as-youjun
 */
public class TestVehicleDataQueue {

    /**
     * 日志记录器
     */
    private static final Logger logger = Logger
            .getLogger(TestVehicleDataQueue.class);

    /**
     * 单例
     */
    private static TestVehicleDataQueue instance;

    /**
     * 帧队列
     */
    private static final BlockingQueue<VehicleData> blockingQueue = new LinkedBlockingQueue<VehicleData>();

    /**
     * 控制帧消费者线程运行状态
     */
    private static boolean isRunning = true;

    /**
     * 构造方法
     */
    private TestVehicleDataQueue() {
        Logger.getLogger(CommonDic.class).debug(IotConstant.CONSTRUCTOR);
    }

    /**
     * 获取单例
     *
     * @return
     */
    public static TestVehicleDataQueue getInstance() {
        if (instance == null) {
            synchronized (TestVehicleDataQueue.class) {
                if (instance == null) {
                    instance = new TestVehicleDataQueue();
                }
            }
        }
        return instance;
    }

    /**
     * 添加数据
     *
     * @param data
     */
    public void add(VehicleData data) {
        while (blockingQueue.add(data)) {
            break;
        }
    }

    public void start() {
        Thread thread = new Thread(new TestVehicleDataConsumer(),
                "test vehicle data consumer thread");
        thread.start();
    }

    /**
     * 帧消费者线程
     *
     * @author as-youjun
     */
    class TestVehicleDataConsumer implements Runnable {

        /**
         * 1.从队列中取出帧；2.调用业务接口；
         */
        @Override
        public void run() {
            while (isRunning) {
                // 1.从队列中取出帧
                VehicleData data = null;
                try {
                    data = blockingQueue.take();
                } catch (InterruptedException e) {
                    logger.error(e);
                }
                // 2.协议数据后续逻辑；
                if (data == null) {
                    logger.warn("test VehicleData is empty!");
                } else {
                    TestEqpInputData testEqpInputData = TestEquipmentCache.getInstance()
                            .getTestEqpInputData();
                    if (testEqpInputData != null) {
                        // 3.接口调用成功
                        if (IotConstant.EquipmentConstant.CONNECT.equals(testEqpInputData.getFlag())) {
                            finishIntfcsCall(testEqpInputData, data.getSourceID());
                        }
                        // 4.解析指标数据
                        if (IotConstant.EquipmentConstant.METRIC.equals(testEqpInputData.getFlag())) {
                            TestVehicleDataService testVehicleDataService = (TestVehicleDataService) IotContext
                                    .getInstance()
                                    .getBean("testVehicleDataService");
                            testVehicleDataService.process(data);
                        }
                    } else {
                        logger.debug("test monitor equipmentBo is null!");
                    }
                }
            }
        }

    }

    /**
     * 接口调用成功
     *
     * @param testEqpInputData
     * @param sourceID
     */
    private void finishIntfcsCall(TestEqpInputData testEqpInputData, String sourceID) {
        // 1.获取设备标识
        Map<String, Object> eqpParmaValueMap = TestUtil
                .getEqpParamValueMap(testEqpInputData);
        String sourceIdFromEquip = (String) eqpParmaValueMap.get(IotConstant.EquipmentConstant.SOURCEID);
        if (StringUtils.isEmpty(sourceIdFromEquip)) {
            return;
        }
        if (sourceID.equals(sourceIdFromEquip)) {
            if (CollectionUtils.isEmpty(testEqpInputData.getEqpTmplIntfcs())) {
                return;
            }
            testEqpInputData
                    .getEqpTmplIntfcs()
                    .forEach(
                            eqpTmplIntfc -> {
                                if (eqpTmplIntfc.getIntfcDirection().intValue() == IntfcDirection.UP
                                        .getValue()) {
                                    // 上行接口:接口调用成功
                                    TestIntfc testIntfc = new TestIntfc();
                                    testIntfc.setId(eqpTmplIntfc.getIntfcId());
                                    testIntfc.setName(eqpTmplIntfc
                                            .getIntfcName());
                                    testIntfc.setResult("ok");
                                    TestEquipmentCache.getInstance()
                                            .finishOneIntfc(testIntfc);
                                }
                            });
        }
    }

}
