/**
 *
 */
package com.yeejoin.amos.iot.netty.vehicle.test;

import com.yeejoin.amos.iot.business.constants.IotConstant;
import com.yeejoin.amos.iot.business.dao.mapper.MonitorMapper;
import com.yeejoin.amos.iot.business.dao.repository.ICapabilityDao;
import com.yeejoin.amos.iot.business.entity.mybatis.CapabilityBo;
import com.yeejoin.amos.iot.business.entity.mybatis.CapacityPropertyDataVo;
import com.yeejoin.amos.iot.business.enums.AttributeType;
import com.yeejoin.amos.iot.business.polling.test.*;
import com.yeejoin.amos.iot.business.service.intfc.IMonitorService;
import com.yeejoin.amos.iot.netty.vehicle.VehicleData;
import net.sf.json.JSONObject;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 测试数据service
 *
 * @author as-youjun
 */
@Service("testVehicleDataService")
public class TestVehicleDataService {

    /**
     * 日志记录器
     */
    private static final Logger logger = Logger
            .getLogger(TestVehicleDataService.class);

    @Autowired
    private MonitorMapper processMapper;

    @Autowired
    private ICapabilityDao capabilityDao;

    @Autowired
    private IMonitorService monitorService;

    /**
     * 测试指标采集
     */
    public void process(VehicleData vehicleData) {
        boolean isUsable = checkVehicleData(vehicleData);
        if (!isUsable) {
            logger.debug("sourceId is empty!");
            return;
        }
        // 测试指标采集
        testMetric(vehicleData);
    }

    /**
     * 监测VehicleData数据格式是否正确
     *
     * @param vehicleData
     * @return
     */
    private boolean checkVehicleData(VehicleData vehicleData) {
        if (StringUtils.isEmpty(vehicleData.getSourceID())) {
            Logger.getLogger(this.getClass()).warn(
                    "vehicleData.sourceID is empty!");
            return false;
        }
        return true;
    }

    /**
     * 构造指标数据
     *
     * @param vehicleData
     * @return
     */
    private void testMetric(VehicleData vehicleData) {
        // 1.从缓存中获取测试设备
        TestEqpInputData testEqpInputData = TestEquipmentCache.getInstance()
                .getTestEqpInputData();
        if (testEqpInputData == null) {
            return;
        }

        // 2.获取设备标识
        if (StringUtils.isEmpty(vehicleData.getSourceID())) {
            return;
        }
        Map<String, Object> eqpParmaValueMap = TestUtil
                .getEqpParamValueMap(testEqpInputData);
        String sourceIdFromEquip = (String) eqpParmaValueMap.get("sourceId");
        if (StringUtils.isEmpty(sourceIdFromEquip)) {
            return;
        }
        //3.比对设备标识
        if (!sourceIdFromEquip.equals(vehicleData.getSourceID())) {
            return;
        }
        // 4.遍历能力
        List<CapabilityBo> capabilityBos = testEqpInputData.getCapabilityBos();
        if (CollectionUtils.isEmpty(capabilityBos)) {
            Logger.getLogger(this.getClass()).debug("test capabilityBos is empty!");
            return;
        }
        for (CapabilityBo capabilityBo : capabilityBos) {
            /***
             * 一个vehicleData对象可能包括多个能力数据，因此需要组个拆解
             */
            Map<String, Object> cpblDataMap = buildSingleMetricData(vehicleData,
                    capabilityBo);
            if (MapUtils.isNotEmpty(cpblDataMap)) {
                // 指标采集成功
                TestCapablility testCapablility = new TestCapablility();
                testCapablility.setId(capabilityBo.getId());
                testCapablility.setName(capabilityBo.getName());
                List<TestCpblAttr> attributes = TestUtil
                        .buildTestCpblAttr(cpblDataMap,
                                capabilityBo.getCpblAttrs());
                testCapablility.setAttributes(attributes);
                TestEquipmentCache.getInstance().finishOneCpbl(testCapablility);
            }
        }
    }

    /**
     * 构建单个指标数据
     *
     * @param vehicleData
     * @param capabilityBo
     * @return
     */
    @SuppressWarnings("unchecked")
    private Map<String, Object> buildSingleMetricData(VehicleData vehicleData, CapabilityBo capabilityBo) {
        Map<String, Object> cpblDataMap = new HashMap<>();
        if (CollectionUtils.isEmpty(capabilityBo.getCpblAttrs())) {
            return cpblDataMap;
        }
        // 帧数据
        Map<String, Object> frameMap = (Map<String, Object>) JSONObject.toBean(
                JSONObject.fromObject(vehicleData.getFrameJson()), Map.class);
        // 是否是设备的指标数据
        boolean isEqpMetricData = false;
        boolean isFrameTypeEqual = false;
        boolean isLableTypeEqual = false;
        Integer frameType = getAttrValueInt(capabilityBo.getCpblAttrs(),
                IotConstant.EquipmentConstant.FRAMETYPE);// 帧类型
        if ((frameType != null)
                && (Integer.valueOf(vehicleData.getFrameType())
                .equals(frameType))) {
            isFrameTypeEqual = true;
        }
        Integer labelType = getAttrValueInt(capabilityBo.getCpblAttrs(),
                IotConstant.EquipmentConstant.LABELTYPE);// 标签类型
        if ((labelType != null)
                && (vehicleData.getLabelType() != null)
                && (Integer.valueOf(vehicleData.getLabelType())
                .equals(labelType))) {
            isLableTypeEqual = true;
        }
        if (vehicleData.getLabelType() == null) {
            // 标签类型为null，只需匹配frameType
            isEqpMetricData = isFrameTypeEqual;
            Logger.getLogger(this.getClass()).debug(
                    "test isEqpMetricData enter labeType is null");
        } else {
            // 需匹配frameType、labelType
            isEqpMetricData = isFrameTypeEqual && isLableTypeEqual;
            Logger.getLogger(this.getClass()).debug(
                    "test isEqpMetricData enter labeType is not null");
        }
        if (isEqpMetricData) {
            cpblDataMap = buildSingleMetricData(frameMap);
        }
        return cpblDataMap;
    }

    /**
     * 获取属性名对应的值
     *
     * @param cpblAttrs
     * @param attrName
     * @return
     */
    private Integer getAttrValueInt(List<CapacityPropertyDataVo> cpblAttrs,
                                    String attrName) {
        Integer attrValue = null;
        if (CollectionUtils.isEmpty(cpblAttrs)) {
            return attrValue;
        }
        try {
            for (CapacityPropertyDataVo cpblAttr : cpblAttrs) {
                if (attrName.equals(cpblAttr.getName())
                        && (cpblAttr.getType().intValue() == AttributeType.CONSTANT
                        .getValue())) {
                    attrValue = Integer.valueOf(cpblAttr.getValueStr());
                    Logger.getLogger(this.getClass()).debug(
                            attrName + ":" + attrValue);
                }
            }
        } catch (Exception e) {
            Logger.getLogger(this.getClass()).error("test getAttrValue error! " + e);
        }
        return attrValue;
    }

    /**
     * 构建单个指标数据
     *
     * @param frameMap
     * @return
     */
    @SuppressWarnings("unchecked")
    private Map<String, Object> buildSingleMetricData(
            Map<String, Object> frameMap) {
        // ids的数据是集合，特殊处理
        try {
            if (frameMap.get("ids") != null) {
                List<Object> ids = (List<Object>) frameMap.get("ids");
                String idStr = "";
                for (Object s : ids) {
                    idStr += s.toString() + ",";
                }
                if (idStr.contains(",")) {
                    idStr = idStr.substring(0, idStr.length() - 1);
                }
                frameMap.put("labelStr", idStr);
                // frameMap.remove("ids");
            }
        } catch (Exception e) {
            Logger.getLogger(this.getClass()).error( "test buildSingleMetricData erroe! " + e);
        }
        return frameMap;
    }

}
