package com.yeejoin.amos.iot.rule.cmd;

import java.util.ArrayList;
import java.util.List;

import javax.annotation.PostConstruct;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * <pre>
 * 命令上下文
 * </pre>
 * 
 * @author HK
 * @version $Id: RuleCmdContext.java, v 0.1 2017年12月25日 下午7:23:56 HK Exp $
 */
@Component
public class RuleCmdContext
{
	/**
	 * 命令调用器
	 */
	protected CommandInterceptor commandInvoker;

	/**
	 * 命令拦截器
	 */
	protected List<CommandInterceptor> commandInterceptors;

	/**
	 * 命令执行器
	 */
	protected CommandExecutor commandExecutor;

	/**
	 * 命令执行上下文
	 */
	@Autowired
	protected CommandContext commandContext;

	private static RuleCmdContext instance;

	public static RuleCmdContext getInstance()
	{
		if (instance == null)
		{
			synchronized (RuleCmdContext.class)
			{
				if (instance == null)
				{
					instance = new RuleCmdContext();
				}
			}
		}
		return instance;
	}

	public CommandInterceptor getCommandInvoker()
	{
		return commandInvoker;
	}

	public List<CommandInterceptor> getCommandInterceptors()
	{
		return commandInterceptors;
	}

	public CommandExecutor getCommandExecutor()
	{
		return commandExecutor;
	}

	/**
	 * 
	 * <pre>
	 * 初始化方法
	 * </pre>
	 *
	 */
	@PostConstruct
	public void init()
	{
		initCommandInvoker();
		initCommandInterceptors();
		initCommandExecutor();
		instance = this;
	}

	/**
	 * 
	 * <pre>
	 * 初始化命令执行器
	 * </pre>
	 *
	 */
	private void initCommandExecutor()
	{
		if (commandExecutor == null)
		{
			CommandInterceptor first = initInterceptorChain(
					commandInterceptors);
			commandExecutor = new CommandExecutorImpl(first);
		}
	}

	/**
	 * 
	 * <pre>
	 * 初始化命令拦截链
	 * </pre>
	 *
	 * @param chain
	 * @return
	 */
	private CommandInterceptor initInterceptorChain(
			List<CommandInterceptor> chain)
	{
		if (chain == null || chain.isEmpty())
		{
			return null;
		}
		for (int i = 0; i < chain.size() - 1; i++)
		{
			chain.get(i).setNext(chain.get(i + 1));
		}
		return chain.get(0);
	}

	/**
	 * 
	 * <pre>
	 * 初始化命令拦截器
	 * </pre>
	 *
	 */
	private void initCommandInterceptors()
	{
		if (commandInterceptors == null)
		{
			commandInterceptors = new ArrayList<CommandInterceptor>();
//			commandInterceptors.add(new LogInterceptor());
			commandInterceptors.add(commandInvoker);
		}
	}

	/**
	 * 
	 * <pre>
	 * 初始化命令调用器
	 * </pre>
	 *
	 */
	private void initCommandInvoker()
	{
		if (commandInvoker == null)
		{
			commandInvoker = new CommandInvoker();
		}
	}

	/**
	 * 
	 * <pre>
	 * 获取上下文
	 * </pre>
	 *
	 * @return
	 */
	protected CommandContext getCommandContext()
	{
		return this.commandContext;
	}
}
