package com.yeejoin.amos.iot.rule.engine.action;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;

import net.sf.json.JSONObject;

import org.apache.log4j.Logger;
import org.springframework.data.jpa.domain.Specification;

import com.yeejoin.amos.iot.business.entity.Alarm;
import com.yeejoin.amos.iot.business.entity.Capability;
import com.yeejoin.amos.iot.business.entity.CapabilityAttribute;
import com.yeejoin.amos.iot.business.entity.Equipment;
import com.yeejoin.amos.iot.business.entity.MetricData;
import com.yeejoin.amos.iot.business.entity.MetricDataRo;
import com.yeejoin.amos.iot.business.entity.RuleEmail;
import com.yeejoin.amos.iot.business.enums.AlarmLevel;
import com.yeejoin.amos.iot.business.enums.AlarmStatus;
import com.yeejoin.amos.iot.business.enums.AlarmType;
import com.yeejoin.amos.iot.rule.cmd.CommandContext;
import com.yeejoin.amos.iot.rule.resource.RuleAIEntity;
import com.yeejoin.amos.op.core.util.Constants;
import com.yeejoin.amos.op.core.util.DateUtil;
import com.yeejoin.amos.op.core.util.MailUtil;

/**
 * 
 * <pre>
 * Email输出动作
 * </pre>
 *
 * @author as-guowubin
 * @version $Id: EmailAction.java, v 0.1 2018年4月25日 下午4:00:48 as-guowubin Exp $
 */
public class EmailAction extends CmdAction
{
	private static final Logger log = Logger.getLogger(EmailAction.class);

	public String emailId;
	public String ruleStatus;
	
	public EmailAction(List<RuleAIEntity> ruleEntities)
	{
		super(ruleEntities);
	}
	
	public EmailAction(List<RuleAIEntity> ruleEntities, String emailId, String ruleStatus)
	{		
		super(ruleEntities);
		this.emailId = emailId;
		this.ruleStatus =ruleStatus;
	}

	@SuppressWarnings("unchecked")
	@Override
	public Object execute(CommandContext commandContext)
	{
		try
		{
			for (RuleAIEntity ruleAIEntity : ruleEntities)
			{
				RuleEmail ruleEmail = commandContext.getRuleEmailService().findOne(emailId);
				StringBuilder context = new StringBuilder();
				StringBuilder title = new StringBuilder();
				if (ruleAIEntity instanceof Alarm)
				{
					Alarm alarm = (Alarm) ruleAIEntity;
					Date  endDate = new Date(); 
					if (alarm.getStatus() == AlarmStatus.HISTORY.getValue())//清除告警
					{
						endDate = alarm.getClearTime();
					}
					//构建邮件标题和内容
					title.append("【告警消息】")
					     .append(alarm.getName()).append("-")
						 .append(AlarmType.getName(alarm.getType())).append("-")
						 .append(alarm.getDescr()).append("-")
						 .append(DateUtil.getDateTimeFormat(alarm.getRaiseTime()));
					String record = alarm.getRecord();
					if (record == null)
					{
						record = "无";
					}
					context.append("告警源：").append(alarm.getName()).append(Constants.NEW_LINE)
					       .append("告警级别：").append(AlarmLevel.getName(alarm.getLevel())).append(Constants.NEW_LINE)
						   .append("状态：").append(AlarmStatus.getName(alarm.getStatus())).append(Constants.NEW_LINE)
						   .append("告警类型：").append(AlarmType.getName(alarm.getType())).append(Constants.NEW_LINE)
						   .append("持续时间：").append(DateUtil.getDatePoorToSec(endDate, alarm.getRaiseTime())).append(Constants.NEW_LINE)
						   .append("告警次数：").append(alarm.getCount()).append(Constants.NEW_LINE)
						   .append("描述：").append(alarm.getDescr()).append(Constants.NEW_LINE)
						   .append("活动记录：").append(record);
				}else 
					if (ruleAIEntity instanceof MetricDataRo)
					{
						MetricDataRo metricDataRo = (MetricDataRo)ruleAIEntity;
						List<MetricData> metricDatas = metricDataRo.getMetricDatas();
						if (metricDatas == null || metricDatas.size() == 0)
						{
							return null;
						}
						MetricData metricData = metricDatas.get(0);
						Equipment equipment = commandContext.getDeviceService().findEquipmentById(metricData.getEqpId());
						//构建邮件标题
						title.append("【采集数据】")
						     .append(equipment.getName()).append("-")
						     .append(DateUtil.getDateTimeFormat(metricData.getExeTime()));
						//构建邮件内容
						context.append("数据源：").append(equipment.getName()).append("      ")
					       .append("采集时间：").append(DateUtil.getDateTimeFormat(metricData.getExeTime())).append(Constants.NEW_LINE).append(Constants.NEW_LINE);
						int num = 1;
						for (MetricData metric : metricDatas)
						{
							Capability  capability = commandContext.getCapabilityService().findOne(metric.getCpblId());
							context.append("能力").append(num).append("：").append(capability.getName()).append(Constants.NEW_LINE)
						           .append("指标数据：").append(Constants.NEW_LINE);
							//构建指标
							Specification<CapabilityAttribute> capaAttrSpec = new Specification<CapabilityAttribute>() {
									public Predicate toPredicate(Root<CapabilityAttribute> root, CriteriaQuery<?> query,
											CriteriaBuilder cb) {
										List<Predicate> list = new ArrayList<Predicate>();
										Predicate id = cb.equal(root.get("capability").as(Capability.class), capability);
										list.add(id);
										Predicate[] p = new Predicate[list.size()];
										return cb.and(list.toArray(p));
									}
								};
							List<CapabilityAttribute> capabilityAttrs = commandContext.getCapabilityAttrService().findAll(capaAttrSpec);
							Map<String, String> attrNameMap = new HashMap<String, String>();
							for (CapabilityAttribute capabilityAttribute : capabilityAttrs)
							{
								attrNameMap.put(capabilityAttribute.getName(), capabilityAttribute.getDisplayName());
							}
							String metricJson = metric.getMetricJson();
							Map<String, Object> sourceDataMap = (Map<String, Object>) JSONObject.toBean(JSONObject.fromObject(metricJson),
							          Map.class);
							for (String key : sourceDataMap.keySet()) 
							{
								Object value = sourceDataMap.get(key);
								String displayName = attrNameMap.get(key);
								if (displayName != null)
								{
									context.append(displayName).append("：").append(value).append("，");
								}
							}
							context.append(Constants.NEW_LINE);
							
						}
					}
				title.append("（").append(ruleStatus).append("）");
				String host =ruleEmail.getServer();
				String port =ruleEmail.getPort();
				String sendder =ruleEmail.getFromEmail();
				String pass =ruleEmail.getFromPassword();
				String[] to = {ruleEmail.getToEmail()};
				MailUtil.sendMail(context.toString(), title.toString(), host, port, sendder, pass, to, ruleEmail.getIsSSL());
			}
		}
		catch (Exception e)
		{
			log.error("EmailAction执行失败：" + e.getMessage());
		}
		return null;
	}

}
