package com.yeejoin.amos.iot.rule.engine.action;

import java.io.BufferedWriter;
import java.io.OutputStreamWriter;
import java.net.Socket;
import java.util.List;

import org.apache.log4j.Logger;

import com.yeejoin.amos.iot.business.entity.MetricData;
import com.yeejoin.amos.iot.business.entity.MetricDataRo;
import com.yeejoin.amos.iot.business.entity.RuleTcp;
import com.yeejoin.amos.iot.rule.cmd.CommandContext;
import com.yeejoin.amos.iot.rule.resource.RuleAIEntity;

import net.sf.json.JSONObject;

/**
 * 
 * <pre>
 * tcp动作
 * 提供TCP输出的选择。 可以连接到远程TCP端口， 接受传入连接。
 * </pre>
 *
 * @author as-guowubin
 * @version $Id: TcpAction.java, v 0.1 2018年5月3日 上午10:54:47 as-guowubin Exp $
 */
public class TcpAction extends CmdAction
{
	private static final Logger log = Logger.getLogger(TcpAction.class);

	public String tcpId;
	public String ruleStatus;
	
	public TcpAction(List<RuleAIEntity> ruleEntities)
	{
		super(ruleEntities);
	}
	
	public TcpAction(List<RuleAIEntity> ruleEntities, String tcpId, String ruleStatus)
	{		
		super(ruleEntities);
		this.tcpId = tcpId;
		this.ruleStatus = ruleStatus;
	}

	@Override
	public Object execute(CommandContext commandContext)
	{
		try
		{
			for (RuleAIEntity ruleAIEntity : ruleEntities)
			{
				RuleTcp ruleTcp = commandContext.getRuleTcpService().findOne(tcpId);
				ruleAIEntity.setRuleStatus(ruleStatus);
			    //与服务端建立连接    
			    Socket client = new Socket(ruleTcp.getServerIp(), Integer.valueOf(ruleTcp.getPort()));    
			    //建立连接后就可以往服务端写数据了    
			    BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(client.getOutputStream()));    
			    if (ruleAIEntity instanceof MetricDataRo)
				{
					MetricDataRo metricDataRo = (MetricDataRo)ruleAIEntity;
					List<MetricData> metricDatas = metricDataRo.getMetricDatas();
					if (metricDatas != null && metricDatas.size() > 0)
					{
						MetricData metricData = metricDatas.get(0);
						writer.write(JSONObject.fromObject(metricData.getMetricJson()).toString());    
					}
				}else 
				{
					writer.write(JSONObject.fromObject(ruleAIEntity).toString());    
				}
			    writer.flush();//写完后要记得flush    
			    writer.close();    
			    client.close(); 
			}
		}
		catch (Exception e)
		{
			log.error("TcpAction执行失败：" + e.getMessage());
		}
		return null;
	}

}
