package com.yeejoin.amos.iot.rule.engine.action.queue;

import java.util.concurrent.LinkedBlockingDeque;

import com.yeejoin.amos.iot.rule.cmd.Command;
import com.yeejoin.amos.iot.rule.cmd.RuleCmdContext;
import com.yeejoin.amos.op.core.thread.AbstractThread;
/**
 * 
 * <pre>
 * 规则动作执行队列
 * </pre>
 *
 * @author as-guowubin
 * @version $Id: RuleActionQueue.java, v 0.1 2018年4月23日 下午4:28:25 as-guowubin Exp $
 */
public class RuleActionQueue extends AbstractThread
{

	private LinkedBlockingDeque<Object> ruleActionQueue = new LinkedBlockingDeque<Object>();
	private static RuleActionQueue instance;

	public static RuleActionQueue getInstance()
	{
		if (instance == null)
		{
			synchronized (RuleActionQueue.class)
			{
				if (instance == null)
				{
					instance = new RuleActionQueue();
					Thread t = new Thread(instance);
					t.start();
				}
			}
		}
		return instance;
	}

	private RuleActionQueue()
	{
	}

	/**
	 * 加入队列
	 * @param action
	 * @return
	 * @see com.yeejoin.amos.op.core.thread.AbstractThread#doBeforeNotify(java.lang.Object)
	 */
	@Override
	public boolean doBeforeNotify(Object action)
	{
		try
		{
			ruleActionQueue.put(action);
		}
		catch (InterruptedException e)
		{
			ruleActionQueue.add(action);
		}
		return true;
	}

	/**
	 * 队列取值执行
	 * @return
	 * @see com.yeejoin.amos.op.core.thread.AbstractThread#doBeforeWait()
	 */
	@SuppressWarnings({ "unchecked", "rawtypes" })
	@Override
	public boolean doBeforeWait()
	{
		boolean isWait = false;
		if (!ruleActionQueue.isEmpty())
		{
			try
			{
				Object action = ruleActionQueue.take();
				if (action instanceof Command)
				{
					Command command = (Command) action;
					RuleCmdContext.getInstance().getCommandExecutor().execute(command);
				}
				else
				{
					System.out.println(action.toString());
				}
			}
			catch (InterruptedException e)
			{
				Object action = ruleActionQueue.poll();
				if (action instanceof Command)
				{
					Command command = (Command) action;
					RuleCmdContext.getInstance().getCommandExecutor().execute(command);
				}
				else
				{
					System.out.println(action.toString());
				}
			}
		}
		else
		{
			isWait = true;
		}
		return isWait;
	}

}