
package com.yeejoin.amos.iot.rule.engine.cfg;

import java.io.IOException;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnMissingBean;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import com.yeejoin.amos.iot.rule.log.ConsoleLogHandler;
import com.yeejoin.amos.iot.rule.log.DatabaseLogHandler;
import com.yeejoin.amos.iot.rule.log.LogHandler;

/**
 * 
 * <pre>
 * spring boot 自动配置
 * </pre>
 *
 * @author HK
 * @version $Id: SpringRuleEngineAutoConfiguration.java, v 0.1 2018年1月12日
 *          上午10:35:06 HK Exp $
 */
@Configuration
public class SpringRuleEngineAutoConfiguration
{
	@Autowired(required = false)
	private RuleEngineConfigurer ruleEngineConfigurer;

	@Autowired
	private DatabaseLogHandler databaseLogHandler;

	@Bean
	@ConditionalOnMissingBean
	public SpringRuleEngineConfiguration springRuleEngineConfiguration()
			throws IOException
	{
		SpringRuleEngineConfiguration configuration = new SpringRuleEngineConfiguration();

		// log
		List<LogHandler> logHandlers = configuration.getLogHandlers();
		logHandlers.add(databaseLogHandler);
		logHandlers.add(new ConsoleLogHandler());
		configuration.setLogHandlers(logHandlers);

		// 自定义
		if (ruleEngineConfigurer != null)
		{
			ruleEngineConfigurer.configure(configuration);
		}
		configuration.init();
		return configuration;
	}

}