package com.yeejoin.amos.iot.rule.engine.service.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.yeejoin.amos.feign.privilege.model.AgencyUserModel;
import com.yeejoin.amos.iot.business.entity.DroolsRule;
import com.yeejoin.amos.iot.business.entity.LinkVo;
import com.yeejoin.amos.iot.business.entity.NodeVo;
import com.yeejoin.amos.iot.business.entity.Rule;
import com.yeejoin.amos.iot.business.entity.RuleDataSource;
import com.yeejoin.amos.iot.business.entity.RuleEmail;
import com.yeejoin.amos.iot.business.entity.RuleMqtt;
import com.yeejoin.amos.iot.business.entity.RuleNode;
import com.yeejoin.amos.iot.business.entity.RuleTcp;
import com.yeejoin.amos.iot.business.enums.RuleNodeType;
import com.yeejoin.amos.iot.business.enums.RuleStatus;
import com.yeejoin.amos.iot.business.remote.RemoteSecurityService;
import com.yeejoin.amos.iot.business.service.intfc.IRuleDataSourceService;
import com.yeejoin.amos.iot.business.service.intfc.IRuleEmailService;
import com.yeejoin.amos.iot.business.service.intfc.IRuleMqttService;
import com.yeejoin.amos.iot.business.service.intfc.IRuleNodeService;
import com.yeejoin.amos.iot.business.service.intfc.IRuleTcpService;
import com.yeejoin.amos.iot.rule.engine.service.IRuleInterpreterService;

@Service("RuleInterpreterService")
public class RuleInterpreterServiceImpl implements IRuleInterpreterService
{
	
	@Autowired
	RemoteSecurityService userController;

	@Autowired
	private IRuleEmailService ruleEmailService;
	
	@Autowired
	private IRuleMqttService ruleMqttService;
	
	@Autowired
	private IRuleTcpService ruleTcpService;
	
	@Autowired
	private IRuleDataSourceService ruleDataSourceService;
	
	@Autowired
	private IRuleNodeService ruleNodeService;
	
	private static final String alarmFactName = "com.yeejoin.amos.iot.business.entity.Alarm";
	private static final String metricFactName = "com.yeejoin.amos.iot.business.entity.MetricDataRo";
	
	/**
	 * 根据规则配置视图解析为drools规则
	 * @param nodes
	 * @param Links
	 * @return DroolsRule
	 */
	public List<DroolsRule> parseRule(List<NodeVo> nodes, List<LinkVo> Links, Rule rule) 
	{
		//key为targetId，value为sourceId集合
		Map<String, List<String>> linkMap = new HashMap<String, List<String>>();
		for (LinkVo linkVo : Links)
		{
			List<String> sourceIdList = linkMap.get(linkVo.getTargetId());
			if (sourceIdList == null)
			{
				sourceIdList = new ArrayList<String>();
			}
			sourceIdList.add(linkVo.getSourceId());
			linkMap.put(linkVo.getTargetId(), sourceIdList);
		}
		//输出节点，根据输出节点创建droolsRule
		Map<String, NodeVo> outputNodeMap = new HashMap<String, NodeVo>();
		//key存对应的nodeId
		Map<String, NodeVo> nodeMap = new HashMap<String, NodeVo>();
		for (NodeVo nodeVo : nodes)
		{ 
			if (nodeVo.getNodeType().equals(RuleNodeType.OUTPUT_EMAIL) && nodeVo.getViewKey() != null)//email动作
			{
				RuleEmail ruleEmail = ruleEmailService.findOne(nodeVo.getViewKey());
				if (ruleEmail != null)//如果动作信息未配置，则不进行规则创建
				{
					outputNodeMap.put(ruleEmail.getId(), nodeVo);
				}
			}else 
				if (nodeVo.getNodeType().equals(RuleNodeType.OUTPUT_MQTT) && nodeVo.getViewKey() != null)//mqtt动作
				{
					RuleMqtt ruleMqtt = ruleMqttService.findOne(nodeVo.getViewKey());
					if (ruleMqtt != null)//如果动作信息未配置，则不进行规则创建
					{
						outputNodeMap.put(ruleMqtt.getId(), nodeVo);
					}
				}else 
					if (nodeVo.getNodeType().equals(RuleNodeType.OUTPUT_TCP) && nodeVo.getViewKey() != null)//tcp动作
					{
						RuleTcp ruleTcp = ruleTcpService.findOne(nodeVo.getViewKey());
						if (ruleTcp != null)//如果动作信息未配置，则不进行规则创建
						{
							outputNodeMap.put(ruleTcp.getId(), nodeVo);
						}
					}
			nodeMap.put(nodeVo.getId(), nodeVo);
		}
		/**
		 * 构建droolsRule
		 */
		List<DroolsRule> droolsRules = new ArrayList<DroolsRule>();
		List<String> nameList = new ArrayList<String>();//保存drools名称，保证名称不能重复
		for (String bizId : outputNodeMap.keySet())
		{
			NodeVo nodeVo = outputNodeMap.get(bizId);
			//key为一个规则的条件集合，value为条件对应的规则对象名（例：com.yeejoin.amos.iot.business.entity.Alarm）
			Map<List<String>, String> conditionMap = new HashMap<List<String>, String>();
			List<String> sourceIdList = linkMap.get(nodeVo.getId());
			if (sourceIdList != null)
			{
				for (String sourceId : sourceIdList)
				{
					//默认规则对象为性能数据
					String factName = metricFactName;
					List<String> condition = new ArrayList<String>();
					buildConditionList(sourceId, linkMap, nodeMap, condition, factName, conditionMap);
				}
			}
			for (List<String> conditionInfo : conditionMap.keySet())
			{
				String fact = conditionMap.get(conditionInfo);
				if (fact.equals(alarmFactName) && conditionInfo.size() < 2)//告警节点需要和数据源联合使用
				{
					continue;
				}
				//获取drools规则默认值
				DroolsRule droolsRule = getDefaultDrools(nameList);
				droolsRule.setBizId(bizId);
				droolsRule.setRuleId(nodeVo.getRuleId());
				droolsRule.setFactClass(fact);
				RuleNode ruleNode = ruleNodeService.findOne(nodeVo.getRemark());
				droolsRule.setActionClassName(ruleNode.getValue());
				droolsRule.setConditions(conditionInfo);
				droolsRule.setGroupId("1");
				droolsRule.setIsUser(rule.getIsUser());
				droolsRule.setRuleStatus(RuleStatus.getName(Integer.valueOf(rule.getStatus())));
				droolsRule.setDroolsRule(droolsRule.toString());
				droolsRules.add(droolsRule);
			}
		}
		return droolsRules;
	}

	/**
	 * 
	 * <pre>
	 * 一直到线路的第一个节点，则为为一个条件
	 * </pre>
	 *
	 * @param sourceId
	 * @param linkMap
	 * @param nodeMap
	 * @param condition
	 * @param factName
	 * @param conditionMap
	 */
	private void buildConditionList(String sourceId, Map<String, List<String>> linkMap ,
			Map<String, NodeVo> nodeMap, List<String> condition, String factName,
			Map<List<String>, String> conditionMap)
	{
		NodeVo nodeVo = nodeMap.get(sourceId);
		if (nodeVo == null) {
			return;
		}
		RuleNode ruleNode = ruleNodeService.findOne(nodeVo.getRemark());
		//判断规则对象，默认为性能数据，如果条件中包含告警节点则为告警数据
		if (ruleNode.getType().startsWith("alarm"))
		{
			factName = alarmFactName;
		}
		//构建条件集合
		if (ruleNode.getType().equals(RuleNodeType.DATANODE_CPBL)) {
			RuleDataSource ruleDataSource = getRuleDataSource(nodeVo.getViewKey());
			if (ruleDataSource == null)
			{
				return;
			}
			condition.add(ruleNode.getValue().replace("cpbl_id", String.valueOf(ruleDataSource.getCpblId())).replace("eqp_id", String.valueOf(ruleDataSource.getEqpId())));
		} else
			if (ruleNode.getType().equals(RuleNodeType.DATANODE_DEVICE))
			{
				RuleDataSource ruleDataSource = getRuleDataSource(nodeVo.getViewKey());
				if (ruleDataSource == null)
				{
					return;
				}
				condition.add(ruleNode.getValue().replace("eqp_id", String.valueOf(ruleDataSource.getEqpId())));
			}else
				if (ruleNode.getType().equals(RuleNodeType.DATANODE_TASK))
				{
					RuleDataSource ruleDataSource = getRuleDataSource(nodeVo.getViewKey());
					if (ruleDataSource == null)
					{
						return;
					}
					condition.add(ruleNode.getValue().replace("taskId", String.valueOf(ruleDataSource.getMonitorRuleId())));
				}else
					if (ruleNode.getType().equals(RuleNodeType.DATANODE_VIEW))
					{
						RuleDataSource ruleDataSource = getRuleDataSource(nodeVo.getViewKey());
						if (ruleDataSource == null)
						{
							return;
						}
						condition.add(ruleNode.getValue().replace("viewId", ruleDataSource.getView()));
					}else
						if (ruleNode.getType().equals(RuleNodeType.DATANODE_CURUSER) ||
								ruleNode.getType().equals(RuleNodeType.DATANODE_SUBUSER))
						{
							AgencyUserModel currUser = userController.getUserInfo();
							condition.add(ruleNode.getValue().replace("currUser", currUser.getAgencyCode()));
						}else
							{
								condition.add(ruleNode.getValue());
							}
		List<String> sourceIdList = linkMap.get(sourceId);
		if (sourceIdList != null)
		{
			for (String string : sourceIdList)
			{
				//克隆条件
				List<String> copyCondition = new ArrayList<String>();
				for (String c : condition)
				{
					copyCondition.add(c);
				}
				buildConditionList(string, linkMap, nodeMap, copyCondition, factName, conditionMap);
			}
		}else 
			if (sourceIdList == null && condition.size()>0)
			{
				//克隆条件
				List<String> copyCondition = new ArrayList<String>();
				for (String string : condition)
				{
					copyCondition.add(string);
				}
				conditionMap.put(copyCondition,factName);
			}
	}
	
	private RuleDataSource getRuleDataSource(String ruleDataSourceId)
	{
		RuleDataSource ruleDataSource = null;
		if (ruleDataSourceId != null)
		{
			ruleDataSource = ruleDataSourceService.findOne(ruleDataSourceId);
		}
		return ruleDataSource;
	}
	
	private DroolsRule getDefaultDrools(List<String> nameList) 
	{
		DroolsRule droolsRule = new DroolsRule();
		droolsRule.setPackageName("com.yeejoin.amos.iot.business.entity");
		List<String> importList = new ArrayList<>();
		importList.add("java.util.*");
		// 加入日志记录的包引用
		importList.add("com.yeejoin.amos.iot.rule.engine.action.*");
		importList.add("com.yeejoin.amos.iot.rule.resource.*");
		importList.add("com.yeejoin.amos.iot.business.entity.*");
		importList.add("com.yeejoin.amos.iot.rule.engine.action.queue.RuleActionQueue");
		droolsRule.setImportList(importList);
		String name = getRulename(nameList)+"_"+userController.getUserId();
		nameList.add(name);
		droolsRule.setName(name);
		return droolsRule;
	}
	
	private String getRulename(List<String> nameList)
	{
		String name = String.valueOf(System.currentTimeMillis());
		//循环直至找到未存在的name
		while (nameList.contains(name))
		{
			name = String.valueOf(Long.valueOf(name) +1) ;
		}
		nameList.add(name);
		return name;
	}
	

}