package com.yeejoin.amos.iot.rule.log;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.ConcurrentLinkedQueue;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.atomic.AtomicLong;

import org.apache.log4j.Logger;

import com.yeejoin.amos.op.core.entity.Resource;


/**
 * 
 * <pre>
 * 带有缓存区的日志记录
 * </pre>
 *
 * @author HK
 * @version $Id: BufferLogCollector.java, v 0.1 2018年1月19日 下午5:40:46 HK Exp $
 */
public class BufferLogCollector
{
	private static final Logger log = Logger
			.getLogger(BufferLogCollector.class);

	private static final List<LogHandler> logHandlers = new ArrayList<>();	
	private static final ConcurrentLinkedQueue<Resource> buffers =  new  ConcurrentLinkedQueue<>();
	private static final HandleLogRunnable handleLogRunnable = new HandleLogRunnable();
	private static final ExecutorService executorService = Executors.newSingleThreadExecutor();
	private static final LogSaver LOG_SAVER = new LogSaver();
	private static final long logBufferMax = 100;
	private static final AtomicLong count = new AtomicLong();
	
	public static void addLogHandler(List<LogHandler> logHandler){
		logHandlers.addAll(logHandler);
	}
	
	 /**
     * 等待任务处理完毕
     * 然后关闭线程池
     */
    public static void close(){
    	log.info("等待任务处理完毕");
        if(shoudHandle()){
            LOG_SAVER.save();
        }
        log.info("关闭日志处理线程池");
        executorService.shutdown();
    }
    /**
     * 将日志加入缓冲区
     * @param <T>
     * @param t 
     */
    public static <T extends Resource> void collect(T t){
        buffers.add(t);
        //判断缓冲区是否达到限制
        if(buffers.size() > logBufferMax){
        	log.info("缓冲区已达到限制数："+logBufferMax+" ，处理日志");
            handleLog();
        }
    }
    
    /**
     * 提交日志之后立即返回，不会阻塞调用线程
     */
    public static void handleLog(){
    	log.info("处理缓冲区中的日志");
        if(shoudHandle()){
            executorService.submit(handleLogRunnable);
        }
    }
    private static boolean shoudHandle(){
        if(logHandlers.isEmpty()){    
        	log.error("未找到任何LogHandler");
            return false;
        }
        int len=buffers.size();
        if(len==0){
        	log.info("没有日志需要保存");
            return false;
        }
        log.info("需要处理缓冲区中的日志");
        return true;
    }
    /**
     * 处理日志线程，异步使用
     */
    private static class HandleLogRunnable implements Runnable{
        @Override
        public void run() {
            LOG_SAVER.save();
        }    
    }
    /**
     * 处理日志类，同步使用
     */
    private static class LogSaver{
        public void save(){
            int len=buffers.size();
            log.info("开始处理缓冲区中的"+len+" 个日志对象");
            List<Resource> list=new ArrayList<>(len);
            for(int i=0;i<len;i++){
                list.add(buffers.remove());            
            }        
            //把日志交给LogHandler处理
            logHandlers.forEach(logHandler -> {
                logHandler.handle(list);
            });
            //加速垃圾回收
            list.clear();
            log.info("缓冲区中的日志处理完毕");
        }
    }
}
