package com.yeejoin.amos.iot.rxtx;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.annotation.PostConstruct;

import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

import com.yeejoin.amos.iot.business.dao.repository.IIntfcAttributeDao;
import com.yeejoin.amos.iot.business.dic.CommunityProtocol;
import com.yeejoin.amos.iot.business.entity.Intfc;
import com.yeejoin.amos.iot.business.entity.IntfcAttribute;
import com.yeejoin.amos.iot.business.service.intfc.IInterfacePropertyService;
import com.yeejoin.amos.iot.business.service.intfc.IIntfcService;
import com.yeejoin.amos.iot.core.enums.QueryOperatorEnum;
import com.yeejoin.amos.iot.rxtx.reader.AnyDataReader;
import com.yeejoin.amos.iot.rxtx.service.SerialFrameQueue;
import com.yeejoin.amos.op.core.common.query.DaoCriteria;

import gnu.io.SerialPort;

@Service("serialStringService")
@Scope("singleton")
public class SerialStringService {


    private static Logger logger = Logger.getLogger(SerialStringService.class);

    private static List<SerialContext> allContext = new ArrayList<>();
    @Autowired
    private IIntfcService intfcService;
    @Autowired
    private IInterfacePropertyService iInterfacePropertyService;
    @Autowired
    private IIntfcAttributeDao iIntfcAttributeDao;
    private static ArrayList<Intfc> notInitIntfcs = new ArrayList<Intfc>();

    String pattern = "COM\\d{1,}$";

    Pattern r = Pattern.compile(pattern);

    @PostConstruct
    public void start() {
        List<DaoCriteria> daoCriterias = getSerialParams();
        List<Intfc> serialIntfcList = intfcService.findAll(daoCriterias);
        new Thread(new Runnable() {
            @Override
            public void run() {
                initSerial(serialIntfcList);
            }
        }).start();
    }

    public void initSerial(List<Intfc> serialIntfcList) {
    	logger.error("initSerial size===》" + serialIntfcList.size());
        for (Intfc intfc : serialIntfcList) {
            List<IntfcAttribute> intfcAttributs = iIntfcAttributeDao.getPropertysById(intfc.getId());
            HashMap<String, String> param = new HashMap<>();
            for (IntfcAttribute attribute : intfcAttributs) {
                switch (attribute.getName()) {
                    case "port":
                        param.put("portName", attribute.getValueStr());
                        break;
                    case "baudRate":
                        param.put("baudRate", attribute.getValueStr());
                        break;
                    case "dataBits":
                        param.put("dataBits", attribute.getValueStr());
                        break;
                    case "stopBits":
                        param.put("stopBits", attribute.getValueStr());
                        break;
                    case "parity":
                        param.put("parity", attribute.getValueStr());
                        break;
                        default:
                            logger.debug("unknow param " + attribute.getName());
                }
            }

            Matcher m = r.matcher(param.get("portName"));
            if (ObjectUtils.isEmpty(param.get("portName")) ||
                    ObjectUtils.isEmpty(param.get("baudRate")) ||
                    !StringUtils.isNumeric(param.get("baudRate")) ||
                    !m.matches()) {
                logger.error("ObjectUtils.isEmpty(param");
                continue;
            }
            
            if (!SerialUtils.commNamesIsExist(param.get("portName"))) {
                notInitIntfcs.add(intfc);
                logger.error("SerialUtils.commNamesIsExist ===》" + param.get("portName"));
                continue;
            }
            try {
                connect(param);
            } catch (Exception e) {
                e.printStackTrace();
                notInitIntfcs.add(intfc);
            }
        }
    }

    private void connect(HashMap<String, String> param) throws Exception {
        if (ObjectUtils.isEmpty(param.get("dataBits")) || !StringUtils.isNumeric(param.get("dataBits"))) {
            param.put("dataBits", SerialPort.DATABITS_8 +"");
        }
        if (ObjectUtils.isEmpty(param.get("stopBits")) || !StringUtils.isNumeric(param.get("stopBits"))) {
            param.put("stopBits", SerialPort.STOPBITS_1 +"");
        }
        if (ObjectUtils.isEmpty(param.get("parity")) || !StringUtils.isNumeric(param.get("parity"))) {
            param.put("parity", SerialPort.PARITY_NONE +"");
        }
        SerialPort connect = SerialUtils.connect(param.get("portName"),
                Integer.parseInt(param.get("baudRate")),
                Integer.parseInt(param.get("dataBits")),
                Integer.parseInt(param.get("stopBits")),
                Integer.parseInt(param.get("parity")));
        SerialContext serialContext = SerialContext.build(connect);
        serialContext.setSerialReader(new AnyDataReader(serialContext));
        serialContext.setSerialPortEventListener(new DefaultSerialDataListener(serialContext));
        serialContext.setSerialByteDataProcessor(SerialFrameQueue.getInstance()::add);
    }

    public List<DaoCriteria> getSerialParams() {
        List<DaoCriteria> daoCriterias = new ArrayList<DaoCriteria>();
        DaoCriteria visDaoCriteria = new DaoCriteria();
        visDaoCriteria.setPropertyName("visibility");
        visDaoCriteria.setOperator(QueryOperatorEnum.EQUAL.getName());
        visDaoCriteria.setValue(0);
        daoCriterias.add(visDaoCriteria);
        DaoCriteria modeDaoCriteria = new DaoCriteria();
        modeDaoCriteria.setPropertyName("dicIdCommMode");
        modeDaoCriteria.setOperator(QueryOperatorEnum.EQUAL.getName());
        modeDaoCriteria.setValue(CommunityProtocol.SERIAL);
        daoCriterias.add(modeDaoCriteria);
        return daoCriterias;
    }
}
