package com.yeejoin.amos.iot.rxtx.service;

import com.yeejoin.amos.iot.context.IotContext;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.Map;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.LinkedBlockingQueue;

public class SerialDataQueue {
    private static Logger logger = Logger.getLogger(SerialDataQueue.class);

    /**
     * 单例
     */
    private static SerialDataQueue instance;

    /**
     * 帧队列
     */
    private static final BlockingQueue<Map> blockingQueue = new LinkedBlockingQueue<Map>();

    /**
     * 控制帧消费者线程运行状态
     */
    private static boolean isRunning = true;

    //线程池大小
    private static final int poolSize = 2;

    //固定线程池
    private static ExecutorService executorService = Executors.newFixedThreadPool(poolSize);

    ISerialDataService iSerialDataService = (ISerialDataService) IotContext.getInstance().getBean("iSerialDataService");
    
    /**
     * 构造方法
     */
    private SerialDataQueue() {
        start();
    }

    /**
     * 获取单例
     *
     * @return
     */
    public static SerialDataQueue getInstance() {
        if (instance == null) {
            synchronized (SerialDataQueue.class) {
                if (instance == null) {
                    instance = new SerialDataQueue();
                }
            }
        }
        return instance;
    }

    /**
     * 添加数据
     *
     * @param data
     */
    public void add(Map data) {
        logger.debug("add map  queue"+data.toString());
    	blockingQueue.add(data);
    }

    private void start() {
        for (int i = 0; i < poolSize; i++) {
            executorService.execute(new SerialDataQueue.SerialDataConsumer());
        }
    }

    /**
     * 帧消费者线程
     *
     * @author as-youjun
     *
     */
    class SerialDataConsumer implements Runnable {

        /**
         * 1.从队列中取出帧；2.调用业务接口；
         */
        @Override
        public void run() {
            while (isRunning) {
                // 1.从队列中取出帧
                Map data = null;
                try {
                    data = blockingQueue.take();
                } catch (InterruptedException e) {
                    logger.error("take frame happened error", e);
                }
                // 2.调用业务接口；
                if (data == null) {
                    logger.warn("frame is empty!");
                } else {
                    try {
                        logger.debug("process data  queue"+data.toString());
                        iSerialDataService.process(data);
                        data.clear();
                        data = null;
                        
                    } catch (Exception e) {
                        logger.error("parse frame happened error", e);
                        // 失败处理
                    } finally {
                    	System.gc();
					}
                }
            }
        }
    }
}
