package com.yeejoin.amos.iot.rxtx.service;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import javax.annotation.Resource;

import com.yeejoin.amos.iot.business.dao.repository.*;
import com.yeejoin.amos.iot.business.entity.*;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.collections.MapUtils;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.yeejoin.amos.iot.business.constants.IotConstant;
import com.yeejoin.amos.iot.business.dao.mapper.DeviceMapper;
import com.yeejoin.amos.iot.business.dao.mapper.MonitorMapper;
import com.yeejoin.amos.iot.business.entity.mybatis.CapabilityBo;
import com.yeejoin.amos.iot.business.entity.mybatis.CapacityPropertyDataVo;
import com.yeejoin.amos.iot.business.entity.mybatis.EqpNodeSubGraphBo;
import com.yeejoin.amos.iot.business.entity.mybatis.EqpTmplAttributeBo;
import com.yeejoin.amos.iot.business.entity.mybatis.EqpTmplIntfcBo;
import com.yeejoin.amos.iot.business.entity.mybatis.EquipmentBo;
import com.yeejoin.amos.iot.business.entity.mybatis.IntfcBo;
import com.yeejoin.amos.iot.business.enums.CapabilityType;
import com.yeejoin.amos.iot.business.enums.MonitorType;
import com.yeejoin.amos.iot.business.param.QueryCapabilityParam;
import com.yeejoin.amos.iot.business.param.QueryEqpTmplIntfcParam;
import com.yeejoin.amos.iot.business.param.QueryIntfcParam;
import com.yeejoin.amos.iot.business.param.QueryMonitorEqpParam;
import com.yeejoin.amos.iot.business.polling.MonitorUtil;
import com.yeejoin.amos.iot.business.queue.AlarmDataObj;
import com.yeejoin.amos.iot.business.queue.AlarmDataQueue;
import com.yeejoin.amos.iot.business.service.impl.DeviceServiceHelper;
import com.yeejoin.amos.iot.business.service.intfc.IEqpTmplPropertyService;
import com.yeejoin.amos.iot.business.service.intfc.IMetricDataService;
import com.yeejoin.amos.iot.business.service.intfc.IMonitorService;
import com.yeejoin.amos.iot.context.RedisUtil;
import com.yeejoin.amos.iot.core.util.DateJsonValueProcessor;
import com.yeejoin.amos.iot.mongodb.service.intfc.IEquipMetricDataService;
import com.yeejoin.amos.iot.netty.ReceiveEqpCache;
import com.yeejoin.amos.iot.rule.engine.service.IRuleEngineService;
import com.yeejoin.amos.iot.socket.websocket.service.IWebSocketService;
import com.yeejoin.amos.iot.socket.websocket.vo.RealTimeMetricDataVo;

import net.sf.json.JSONObject;
import net.sf.json.JsonConfig;

@Service("iSerialDataService")
@Scope("singleton")
public class SerialDataServiceImpl implements ISerialDataService{
    private static Logger logger = Logger.getLogger(SerialDataServiceImpl.class);

    @Resource
    private MonitorMapper processMapper;

    @Resource
    private MonitorMapper monitorMapper;

    @Autowired
    private IMetricDataService metricDataService;

    @Autowired
    private IEquipmentTemplateDao equipmentTemplateDao;

    @Autowired
    private IIntfcAttributeDao intfcAttributeDao;

    @Autowired
    private IEquipmentDao equipmentDao;

    @Autowired
    private ICapabilityDao capabilityDao;
    @Autowired
    private IAlarmDao alarmDao;
    @Autowired
    private ICapabilityPropertyDao capabilityPropertyDao;

    @Autowired
    private IEquipMetricDataService equipMetricDataService;

    @Autowired
    private IWebSocketService webSocketService;
    @Autowired
    private IWebSocketService webSocketMorphicService;

    @Resource
    private DeviceMapper deviceMapper;
    @Autowired
    private IMonitorService monitorService;

    @Autowired
    private IRuleEngineService ruleEngineService;

    @Autowired
    protected RedisUtil redisUtil;

    @Autowired
    IEqpTmplPropertyDao iEqpTmplPropertyDao;

    /**
     * 线程池
     */
    private ExecutorService threadPool = Executors.newFixedThreadPool(10);

    @Override
    public boolean process(Map serialData) {
        boolean result = false;
        List<MetricData> metricDatas = null;
        try {
            // 1.构造指标数据
            metricDatas = buildMetricData(serialData);
            result = true;
        } catch (Exception e) {
            logger.error(e);
            result = false;
        }
        logger.error("data build>>>" + metricDatas.toString());
        if (CollectionUtils.isEmpty(metricDatas)) {
            return result;
        }
        //2.指标数据保存到mysql
        metricDatas.forEach(metricData -> {
            //缓存上报设备id,以用于设备连通性检测
            ReceiveEqpCache.getInstance().put(metricData.getEqpId());
            // 保存指标数据
            try {
                if (!ObjectUtils.isEmpty(metricData.getMetricJson()) && !"null".equals(metricData.getMetricJson())) {
                    metricDataService.saveMetricData(metricData);
                }
            } catch (Exception e) {
                logger.error("saveMetricData error!" + e.getMessage());

            }
        });
        logger.error("data mysql save>>>");
        //3.指标数据保存到mongodb
        metricDatas.forEach(metricData -> {
            try {
                equipMetricDataService.saveMetricData2MongoDb(metricData,serialData.get("sourceId").toString());
            } catch (Exception e) {
                logger.error("saveMetricData mongodb error!" +e.getMessage());
            }
        });
        logger.error("data mongodb save>>>");
        //4.产生告警(告警由规则引擎实现...)
        threadPool.execute(new SerialDataServiceImpl.RaiseAlarmTask(metricDatas));
        //5.向前段推送设备指标数据websocket消息
        threadPool.execute(new SerialDataServiceImpl.SendMetricMessageTask(metricDatas));
//        //6.向前端组态组件推送数据
        threadPool.execute(new SerialDataServiceImpl.SendMorphiMessageTask(metricDatas));
        // 7.触发性能数据规则
        threadPool.execute(new SerialDataServiceImpl.TriggerRuleTask(metricDatas));

        return result;
    }

    private List<MetricData> buildMetricData(Map serialData) {
        List<MetricData> metricDatas = new ArrayList<>();
        // 1.查询运行中的设备
        QueryMonitorEqpParam queryParam = new QueryMonitorEqpParam();
        queryParam.setEnableMonitor(true);
        queryParam.setMonitorType(MonitorType.LISTENING.getValue());
        List<EquipmentBo> equipmentBos = monitorService.findMonitorEquipmentDet(queryParam);
        if (CollectionUtils.isEmpty(equipmentBos)) {
            logger.error("equipmentBos is null>>>" );
            return metricDatas;
        }
        // 2.构造指标数据
        metricDatas = buildMetricData(serialData, equipmentBos);
        return metricDatas;
    }

    private List<MetricData> buildMetricData(Map serialData, List<EquipmentBo> equipmentBos) {
        List<MetricData> metricDatas = new ArrayList<>();
        if (ObjectUtils.isEmpty(serialData.get("sourceId"))) {
            return metricDatas;
        }
        // 1.获取设备标识
        Map<Long, Map<String, Object>> eqpParmaValueMap = getEqpParamValueMap(equipmentBos);
        // 2.根据sourceId获取设备aV
        List<EquipmentBo> mebos = getMebosBySourceId(serialData.get("sourceId").toString(), equipmentBos, eqpParmaValueMap);
        if (CollectionUtils.isEmpty(mebos)) {
            logger.error("mebos is null>>>" );
            return metricDatas;
        }
        // 3.查询设备能力
        List<Long> eqpTmplIds = new ArrayList<>();
        mebos.forEach(eqpb -> {
            if (!eqpTmplIds.contains(eqpb.getEqpTmplId())) {
                eqpTmplIds.add(eqpb.getEqpTmplId());
            }
        });
        QueryCapabilityParam queryCapabilityParam = new QueryCapabilityParam();
        queryCapabilityParam.setCpblType(CapabilityType.COLLECT.getValue());
        queryCapabilityParam.setEqpTmplIds(eqpTmplIds);
        List<CapabilityBo> capabilityBos = monitorMapper.findCpblByEqpTmplIds(queryCapabilityParam);
        if (CollectionUtils.isNotEmpty(capabilityBos)) {
            // CapabilityBo填充到EquipmentBo
            MonitorUtil.fillCapabilityBo2EquipmentBo(capabilityBos, mebos);
        } else {
            return metricDatas;
        }
        // 4.查询设备能力接口
        QueryIntfcParam queryIntfcParam = new QueryIntfcParam();
        queryIntfcParam.setCpblType(CapabilityType.COLLECT.getValue());
        queryIntfcParam.setEqpTmplIds(eqpTmplIds);
        List<IntfcBo> intfcBos = monitorMapper.findIntfcOfCpblByEqpTmplIds(queryIntfcParam);
        if (CollectionUtils.isNotEmpty(intfcBos)) {
            // IntfcBo填充到CapabilityBo
            mebos.forEach(equipmentBo -> {
                MonitorUtil.fillIntfcBo2CapabilityBo(intfcBos, equipmentBo.getCapabilityBos());
            });
            // 接口属性填充到接口
            List<Long> intfcIds = new ArrayList<Long>();
            intfcBos.forEach(item -> {
                intfcIds.add(item.getId());
            });
            List<IntfcAttribute> intfcAttributes = intfcAttributeDao.findAllByIntfcIdIn(intfcIds);
            if (CollectionUtils.isNotEmpty(intfcAttributes)) {
                mebos.forEach(equipmentBo -> {
                    if (CollectionUtils.isNotEmpty(equipmentBo.getCapabilityBos())) {
                        equipmentBo.getCapabilityBos().forEach(capabilityBo -> {
                            MonitorUtil.fillIntfcAttr2Intfc(intfcAttributes, capabilityBo.getIntfcBos());
                        });
                    }
                });
            } else {
                logger.debug("intfcAttributes is empty!");
            }
            //能力属性填充到能力
            fillCpblAttr2Cpbl(mebos);
        }
        // 5.查询设备接口
        QueryEqpTmplIntfcParam queryEqpTmplIntfcParam = new QueryEqpTmplIntfcParam();
        queryEqpTmplIntfcParam.setEqpTmplIds(eqpTmplIds);
        List<EqpTmplIntfcBo> eqpTmplIntfcBos = monitorMapper.findEqpTmplIntfcByEqpTmplIds(queryEqpTmplIntfcParam);
        if (CollectionUtils.isNotEmpty(eqpTmplIntfcBos)) {
            //设备模板接口填充到设备
            MonitorUtil.fillEqpTmplIntfc2EquipmentBo(eqpTmplIntfcBos, mebos);
            //接口填充到设备模板接口
            mebos.forEach(equipmentBo -> {
                MonitorUtil.fillIntfcBo2EqpTmplIntfcBo(intfcBos, equipmentBo.getEqpTmplIntfcs());
            });
        } else {
            logger.debug("eqpTmplIntfcBos is empty!");
        }
        //6.构造指标
        for (EquipmentBo mebo : mebos) {
            // 6.1.遍历能力
            List<CapabilityBo> capabilityBos1 = mebo.getCapabilityBos();
            if (CollectionUtils.isEmpty(capabilityBos1)) {
                logger.error("metricDatas is null>>>" );
                return metricDatas;
            }
            for (CapabilityBo capabilityBo : capabilityBos1) {

                MetricData metricData = buildSingleMetricData(serialData,
                        mebo, capabilityBo);
                if (metricData != null) {
                    metricDatas.add(metricData);
                } else {
                    logger.debug("==metricData is null==! capabilityBo.id:"
                            + capabilityBo.getId() + ",capabilityBo.name:"
                            + capabilityBo.getName());
                }
            }
        }
        return metricDatas;
    }

    private MetricData buildSingleMetricData(Map serialData, EquipmentBo mebo, CapabilityBo capabilityBo) {
        MetricData metricData = null;
        // 是否是设备的指标数据
        boolean isLableTypeEqual = false;
        List<CapabilityAttribute> attrs = capabilityPropertyDao.findByCapabilityId(capabilityBo.getId());
        if (ObjectUtils.isEmpty(attrs)) {
            return metricData;
        }
        String labelType = null;
        for (CapabilityAttribute attr : attrs) {
            if ("labelType".equals(attr.getName())) {
                labelType = attr.getValueStr();
            }
        }
        if (ObjectUtils.isEmpty(labelType)) {
            return metricData;
        }
        Map<String, Object> frameMap = (Map<String, Object>)((Map<String, Object>) serialData.get("businessInfo")).get(labelType);
        metricData = buildSingleMetricData(mebo, serialData, capabilityBo, frameMap);
        return metricData;
    }

    /**
     * 构建单个指标数据
     *
     * @param equipmentBo
     * @param serialData
     * @param capabilityBo
     * @param frameMap
     * @return
     */
    @SuppressWarnings("unchecked")
    private MetricData buildSingleMetricData(EquipmentBo equipmentBo,
                                             Map serialData, CapabilityBo capabilityBo,
                                             Map<String, Object> frameMap) {
        MetricData metricData = new MetricData();
        String collectTime = serialData.get("collectTime").toString();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

        try {
            metricData.setExeTime(sdf.parse(collectTime));
        } catch (Exception e) {
            SimpleDateFormat sdfhms = new SimpleDateFormat("HH:mm:ss");
            try {
                metricData.setExeTime(sdfhms.parse(collectTime));
            } catch (Exception el) {
                SimpleDateFormat sdfymd = new SimpleDateFormat("yyyy-MM-dd");
                try {
                    metricData.setExeTime(sdfymd.parse(collectTime));
                } catch (Exception ell) {
//                    ell.printStackTrace();
                }
            }
        }
        if (ObjectUtils.isEmpty(metricData.getExeTime())) {
            metricData.setExeTime(new Date());
        }
        metricData.setEqpId(equipmentBo.getId());
        metricData.setEqpTmplId(equipmentBo.getEqpTmplId());
        metricData.setCpblId(capabilityBo.getId());
        metricData.setIsDelete(false);
        metricData.setPeriod(0);
        metricData.setCreateTime(new Date());
        metricData.setSubgraphId(equipmentBo.getGraphId());
        metricData.setMetricKey(capabilityBo.getId() + IotConstant.BAR+ capabilityBo.getName());
        metricData.setMetricJson(JSONObject.fromObject(frameMap).toString());
        return metricData;
    }


    private void fillCpblAttr2Cpbl(List<EquipmentBo> mebos) {
    }

    private Map<Long, Map<String, Object>> getEqpParamValueMap(List<EquipmentBo> equipmentBos) {
        Map<Long, Map<String, Object>> eqpParmaValueMap = new HashMap<Long, Map<String, Object>>();
        List<Long> eqpTmplIds = new ArrayList<Long>();
        equipmentBos.forEach(item -> {
            if (!eqpTmplIds.contains(item.getEqpTmplId())) {
                eqpTmplIds.add(item.getEqpTmplId());
            }
        });
        String key = null;
        Object value = null;
        for (EquipmentBo equipmentBo : equipmentBos) {
            Map<String, Object> valueMap = (Map<String, Object>) JSONObject.toBean(JSONObject.fromObject(equipmentBo.getParamJson()),Map.class);
            if (MapUtils.isEmpty(valueMap)) {
                logger.debug("valueMap is empty!");
                continue;
            }
            List<EqpTmplAttributeBo> eqpTmplAttributes = equipmentBo.getEqpTmplAttributes();
            if (CollectionUtils.isEmpty(eqpTmplAttributes)) {
                logger.debug("eqpTmplAttributes is empty! equipId="+ equipmentBo.getId() + ",equipName="+ equipmentBo.getName());
                continue;
            }
            for (EqpTmplAttributeBo eqpTmplAttributeBo : eqpTmplAttributes) {
                if (eqpParmaValueMap.get(equipmentBo.getId()) == null) {
                    eqpParmaValueMap.put(equipmentBo.getId(),
                            new HashMap<String, Object>());
                }
                key = IotConstant.EquipmentConstant.KEY_PREFIX_EQUIP+ eqpTmplAttributeBo.getId();
                value = valueMap.get(key);
                eqpParmaValueMap.get(equipmentBo.getId()).put(eqpTmplAttributeBo.getName(), value);
            }
        }
        return eqpParmaValueMap;
    }

    /**
     * 根据sourceId获取监测设备EquipmentBo
     *
     * @param sourceId
     * @param equipmentBos
     * @param eqpParmaValueMap
     * @return
     */
    protected List<EquipmentBo> getMebosBySourceId(String sourceId,
                                                   List<EquipmentBo> equipmentBos,
                                                   Map<Long, Map<String, Object>> eqpParmaValueMap) {
        List<EquipmentBo> monitorEquipmentBos = new ArrayList<EquipmentBo>();
        String sId = null;
        Map<String, Object> tempMap = null;
        try {
            for (EquipmentBo ebo : equipmentBos) {
                tempMap = eqpParmaValueMap.get(ebo.getId());
                if ((tempMap != null) && (tempMap.get("sourceId") != null)) {
                    sId = (String) tempMap.get("sourceId");
//                    logger.debug("getMebosBySourceId sourceId:" + sId);
                    if (sId.equalsIgnoreCase(sourceId)) {
                        logger.debug("ebo:" + ebo.toString());
                        monitorEquipmentBos.add(ebo);
                    }
                }
            }
        } catch (Exception e) {
            logger.error("getMebosBySourceId: " + e);
        }
        return monitorEquipmentBos;
    }

    /**
     * 处理告警
     *
     * @param metricData
     */
    public void raiseAlarm(MetricData metricData) {
        AlarmDataObj alarmDataObj = new AlarmDataObj(metricData);
        alarmDataObj.setType(IotConstant.METRIC);
        AlarmDataQueue.getInstance().add(alarmDataObj);
        logger.info("指标越界告警源数据,sid:" + metricData.getEqpId());
    }

    /**
     * 向前端组态推送websocket消息
     *
     * @param metricData
     */
    public void sendMorphicWebSocketMessage(MetricData metricData) {
        Equipment equipment = equipmentDao.findById(metricData.getEqpId()).get();
        Map<String, Object> resultMap = new HashMap<>();
        Map<String, List<RealTimeMetricDataVo>> realTimeMetricDataVoMap = new HashMap<>();
        List<RealTimeMetricDataVo> realTimeMetricDataVoList = new ArrayList<>();
        Capability capability = capabilityDao.findById(metricData.getCpblId()).get();
        RealTimeMetricDataVo realTimeMetricDataVo = new RealTimeMetricDataVo();
        realTimeMetricDataVo.setEqpId(metricData.getEqpId());
        realTimeMetricDataVo.setCpblId(metricData.getCpblId());
        realTimeMetricDataVo.setCpblName(capability.getName());
        realTimeMetricDataVo.setTime(new Date());

        Map<String, Object> map = new HashMap<>();
        map.put("eqpId", metricData.getEqpId());
        map.put("cpblId", metricData.getCpblId());
        Map<String, String> paramMap = new HashMap<>();
        logger.info("metricData>>" + metricData.getCpblId());
        List<CapabilityAttribute> capabilityAttributes = capabilityPropertyDao
                .findByCapabilityId(metricData.getCpblId());
        if (metricData != null && metricData.getMetricJson() != null && metricData.getMetricJson() != "" && !"null".equals(metricData.getMetricJson())) {
            paramMap = DeviceServiceHelper.buildCpblAttrDatas(metricData.getMetricJson(), capabilityAttributes);
        } else {
            for (CapabilityAttribute capacityPropertyDataVo : capabilityAttributes) {
                paramMap.put(capacityPropertyDataVo.getDisplayName(), "");
            }
        }
        realTimeMetricDataVo.setMetricDataMap(paramMap);
        realTimeMetricDataVoList.add(realTimeMetricDataVo);
        realTimeMetricDataVoMap.put(metricData.getEqpId().toString(), realTimeMetricDataVoList);

        resultMap.put("metricData", realTimeMetricDataVoMap);
        resultMap.put("isWarning", equipment.getIsWarning());
        logger.info("equipment>>" + equipment.getId());
        // 计算告警最高等级
        List<Alarm> alarms = alarmDao.fetchAlarmsByRqpId(equipment.getId());
        if (alarms != null) {
            Integer maxLevel = alarms.stream().mapToInt(i -> i.getLevel()).summaryStatistics().getMin();
            resultMap.put("maxLevel", maxLevel);
        }
        String jsonData = JSON.toJSONString(resultMap, SerializerFeature.WriteMapNullValue);
//        System.out.println("组态：" + jsonData);
        //发送结果
        logger.info("sendMetricDataMessage1" + jsonData);
        webSocketService.sendMetricDataMessage(jsonData, IotConstant.MORPHIC_SOCKET);
        try{
            List<Map> tas = eqpTmplPropertyService.findAllByTempId(metricData.getEqpTmplId());
            logger.info("tas>>>>>" + tas.toString());
            HashMap<String, String> filter = new HashMap<>();
            HashMap<String, Object> message = new HashMap<>();
            for (Map attr: tas) {
                if (ObjectUtils.isEmpty(equipment.getCfgParamMap().get("equip_"+attr.get("id")))) {
                    continue;
                }
                filter.put(attr.get("name").toString(), equipment.getCfgParamMap().get("equip_"+attr.get("id")).toString());
            }
            logger.info("equipment++++++++" + equipment.toString());
            filter.put("orgCode", equipment.getOrgCode());
            filter.put("capability", metricData.getCpblId().toString());
            filter.put("subData", "generate");
            resultMap.put("equipment", equipment);
            resultMap.put("equipmentProperty", tas);
            jsonData = JSON.toJSONString(resultMap, SerializerFeature.WriteMapNullValue);
            message.put("message", jsonData);
            message.put("path", "metricData");
            message.put("filter", filter);
            logger.info("sendMetricDataMessage2" + jsonData);
            webSocketService.sendMetricDataMessage(message);
        } catch (Exception e) {
            e.printStackTrace();
            logger.info("sendMetricDataMessage3" + e.toString());
        }
    }


    @Autowired
    private IEqpTmplPropertyService eqpTmplPropertyService;
    /**
     * 告警处理
     */
    class RaiseAlarmTask implements Runnable {

        private List<MetricData> metricDatas;

        public RaiseAlarmTask(List<MetricData> metricDatas) {
            this.metricDatas = metricDatas;
        }

        @Override
        public void run() {
            if (CollectionUtils.isNotEmpty(metricDatas)) {
                for (MetricData metricData : metricDatas) {
                    // 产生告警(告警由规则引擎实现...)
                    raiseAlarm(metricData);
                }
            }
        }
    }

    /**
     * 指标数据推送到前端
     */
    class SendMetricMessageTask implements Runnable {

        private List<MetricData> metricDatas;

        public SendMetricMessageTask(List<MetricData> metricDatas) {
            this.metricDatas = metricDatas;
        }

        @Override
        public void run() {
            if (CollectionUtils.isNotEmpty(metricDatas)) {
                for (MetricData metricData : metricDatas) {
                    // 向前段推送设备指标数据websocket消息
                    try {
                        sendWebSocketMessage(metricData);
                    } catch (Exception e) {
                        e.printStackTrace();
                        logger.error("sendWebSocketMessage error!" + e);
                    }
                }
            }
        }
    }

    /**
     * 向前端组态组件推送数据
     */
    class SendMorphiMessageTask implements Runnable {

        private List<MetricData> metricDatas;

        public SendMorphiMessageTask(List<MetricData> metricDatas) {
            this.metricDatas = metricDatas;
        }

        @Override
        public void run() {
            if (CollectionUtils.isNotEmpty(metricDatas)) {
                for (MetricData metricData : metricDatas) {
                    logger.error("metricData>>>" + metricData);
                    //向前端组态组件推送数据
                    try {
                        sendMorphicWebSocketMessage(metricData);
                    } catch (Exception e) {
                        e.printStackTrace();
                        logger.error("sendMorphicWebSocketMessage error!" + e);
                    }
                }
            }
        }
    }

    /**
     * 规则执行
     */
    class TriggerRuleTask implements Runnable {

        private List<MetricData> metricDatas;

        public TriggerRuleTask(List<MetricData> metricDatas) {
            this.metricDatas = metricDatas;
        }

        @Override
        public void run() {
            if (CollectionUtils.isNotEmpty(metricDatas)) {
                try {
                        MetricData metricData = metricDatas.get(0);
                        Equipment equipment = equipmentDao.findById(metricData.getEqpId()).get();

                        for (MetricData d : metricDatas) {
                            List<MetricData> mList = new ArrayList<>();
                            List<Map<String, Object>> propertys = new ArrayList<>();
                            mList.add(d);
                            MetricDataRo metricDataRo = new MetricDataRo(mList);
                            metricDataRo.setEqpId(metricData.getEqpId());
                            metricDataRo.setSubgraphId(metricData.getSubgraphId());
                            metricDataRo.setOrgCode(equipment.getOrgCode());
                            metricDataRo.setAuthOrg(equipment.getAuthOrg());
                            metricDataRo.setEqpName(equipment.getName());
                            Map<String, Object> configs = equipment.getCfgParamMap();
                            configs.keySet().forEach(item -> {
                                if (item.startsWith("equip_")) {
                                    Map<String, Object> tempProperty = new HashMap<>();
                                    String idStr = item.substring(6, item.length());
                                    tempProperty.put("id", idStr);
                                    tempProperty.put("valueStr", configs.get(item));
                                    EqpTmplAttribute eqpTmplAttribute = iEqpTmplPropertyDao.findById(Long.valueOf(idStr)).get();
                                    if (eqpTmplAttribute != null) {
                                        tempProperty.put("displayName", eqpTmplAttribute.getDisplayName());
                                        propertys.add(tempProperty);
                                    }
                                }
                            });
                            metricDataRo.setEqpConfigMap(propertys);

                            metricDataRo.setCpblId(d.getCpblId());
                            logger.error(metricDataRo.toString());
                            ruleEngineService.fireRule(metricDataRo);
                        }
                } catch (Exception e) {
                    logger.error(e);
                }
            }
        }
    }

    /**
     * 向前端推送websocket消息
     *
     * @param metricData
     */
    public void sendWebSocketMessage(MetricData metricData) {
        logger.error("entrn websocket>>>" + metricData.toString());
        Capability capability = capabilityDao.findById(metricData.getCpblId()).get();
        RealTimeMetricDataVo realTimeMetricDataVo = buildRealTimeMetricDataVo(
                metricData, capability);
        Map<String, Class<?>> map = new HashMap<String, Class<?>>();
        map.put("time", Date.class);
        realTimeMetricDataVo.setTime(new Date());
        JsonConfig jsonConfig = new JsonConfig();
        jsonConfig.registerJsonValueProcessor(Date.class,
                new DateJsonValueProcessor(IotConstant.DATE_TIME_STR_YMDHMS));
        String jsonData = JSONObject.fromObject(realTimeMetricDataVo,
                jsonConfig).toString();
        List<EqpNodeSubGraphBo> eqpNodeSubGraphBos = processMapper
                .findEqpNodeGraphByEqpId(metricData.getEqpId());
        if (CollectionUtils.isEmpty(eqpNodeSubGraphBos)) {
            logger.warn("eqpNodeSubGraphBos is empty! no need send message!");
            return;
        }
        List<Long> subGraphIds = new ArrayList<Long>();
        eqpNodeSubGraphBos.forEach(eqpNodeSubGraphBo -> {
            if (!subGraphIds.contains(eqpNodeSubGraphBo.getSubGraphId())) {
                subGraphIds.add(eqpNodeSubGraphBo.getSubGraphId());
            }
        });
        logger.error("send metricData websocket>>>" +subGraphIds );
        logger.error("send metricData websocket next>>>" +jsonData );
        webSocketService.sendMetricDataMessage(subGraphIds, jsonData,
                IotConstant.METRIC_SOCKET);
    }

    /**
     * 构造RealTimeMetricDataVo
     *
     * @param metricData
     * @param capability
     * @return
     */
    private RealTimeMetricDataVo buildRealTimeMetricDataVo(
            MetricData metricData, Capability capability) {
        RealTimeMetricDataVo realTimeMetricDataVo = new RealTimeMetricDataVo();
        realTimeMetricDataVo.setEqpId(metricData.getEqpId());
        realTimeMetricDataVo.setCpblId(metricData.getCpblId());
        realTimeMetricDataVo.setTime(metricData.getExeTime());
        realTimeMetricDataVo.setCpblName(capability.getName());
        List<CapacityPropertyDataVo> capabilityAttributes = deviceMapper
                .getCapacityPropertyById(metricData.getCpblId());
        if (metricData !=null && metricData.getMetricJson() != null && !"".equals(metricData.getMetricJson())
         && !"null".equals(metricData.getMetricJson())) {
            realTimeMetricDataVo.setMetricDataMap(DeviceServiceHelper
                    .buildCpblAttrData(metricData.getMetricJson(),
                            capabilityAttributes));
        }

        return realTimeMetricDataVo;
    }
}
