package com.yeejoin.amos.iot.rxtx.service;

import com.yeejoin.amos.iot.context.IotContext;
import org.apache.log4j.Logger;

import java.util.concurrent.BlockingQueue;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.LinkedBlockingQueue;

public class SerialFrameQueue {
    private static Logger logger = Logger.getLogger(SerialFrameQueue.class);

    /**
     * 单例
     */
    private static SerialFrameQueue instance;

    /**
     * 帧队列
     */
    private static final BlockingQueue<byte[]> blockingQueue = new LinkedBlockingQueue<byte[]>();

    /**
     * 控制帧消费者线程运行状态
     */
    private static boolean isRunning = true;

    //线程池大小
    private static final int poolSize = 2;

    //固定线程池
    private static ExecutorService executorService = Executors.newFixedThreadPool(poolSize);

    ISerialFrameService iSerialFrameService = (ISerialFrameService) IotContext.getInstance().getBean("iSerialFrameService");
    /**
     * 构造方法
     */
    private SerialFrameQueue() {
        start();
    }

    /**
     * 获取单例
     *
     * @return
     */
    public static SerialFrameQueue getInstance() {
        if (instance == null) {
            synchronized (SerialFrameQueue.class) {
                if (instance == null) {
                    instance = new SerialFrameQueue();
                }
            }
        }
        return instance;
    }

    /**
     * 添加数据
     *
     * @param frame
     */
    public void add(byte[] frame) {
        blockingQueue.add(frame);
    }

    private void start() {
        for (int i = 0; i < poolSize; i++) {
            executorService.execute(new SerialFrameQueue.SerialFrameConsumer());
        }
    }

    /**
     * 帧消费者线程
     *
     * @author as-youjun
     *
     */
    class SerialFrameConsumer implements Runnable {

        /**
         * 1.从队列中取出帧；2.调用业务接口；
         */
        @Override
        public void run() {
            while (isRunning) {
                // 1.从队列中取出帧
                byte[] frame = null;
                try {
                    frame = blockingQueue.take();
                } catch (InterruptedException e) {
                    logger.error("take frame happened error", e);
                }
                // 2.调用业务接口；
                if (frame == null) {
                    logger.warn("frame is empty!");
                } else {
                    try {
                        iSerialFrameService.parseData(frame);
                        frame = null;
                    } catch (Exception e) {
                        logger.error("parse frame happened error", e);
                    }
                }
            }
        }
    }
}
