/**
 *
 */
package com.yeejoin.amos.iot.script;

import groovy.lang.GroovyShell;
import groovy.lang.Script;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import org.apache.log4j.Logger;
import org.codehaus.groovy.control.CompilationFailedException;
import org.springframework.stereotype.Service;

import com.yeejoin.amos.iot.business.constants.IotConstant;
import com.yeejoin.amos.iot.entity.ParseResult;

/**
 * groovy动态脚本service实现类
 *
 * @author as-youjun
 */
@Service("groovyScriptService")
public class GroovyScriptServiceImpl implements IScriptService {
    /**
     * 缓存通过GroovyShell的parse方法生成的Script实例，避免相同脚本内容生成多个Script引起加载类内存溢出
     * 每次parse都会生成一个新的Script，而一个Script相当于一个新类
     */
    private final static Map<String, Script> scriptClassCach = new ConcurrentHashMap<>();

    /**
     * 解析协议数据
     */
    @Override
    public ParseResult analyze(final String scriptText, final Object[] args) {
        ParseResult parseResult = new ParseResult();
        if (scriptText == null) {
            parseResult.setStatus(IotConstant.FAILURE);
            parseResult.setErrorMsg("脚本内容不能为空");
            return parseResult;
        }

        try {
            Script groovyScript = findOrCompile(scriptText);
            Object result = groovyScript.invokeMethod(PARSE_DATA_METHOD_NAME,
                    args);
            parseResult.setStatus(IotConstant.SUCCESS);
            parseResult.setResult(result);
        } catch (Exception e) {
			Logger.getLogger(this.getClass()).error(
                    "parse protocol data error!" + e);
            parseResult.setStatus(IotConstant.FAILURE);
            parseResult.setErrorMsg(e.getMessage());
        }
        return parseResult;
    }

    /**
     * 从缓存中获取已有Script或者编译新的Script
     *
     * @param scriptText
     * @return
     * @throws CompilationFailedException
     * @throws InstantiationException
     * @throws IllegalAccessException
     */
    private Script findOrCompile(String scriptText) throws CompilationFailedException, InstantiationException, IllegalAccessException {
        String scriptTextHashCode = String.valueOf(scriptText.hashCode());
        if (!scriptClassCach.containsKey(scriptTextHashCode)) {
            GroovyShell shell = new GroovyShell();
            Script groovyScript = shell.parse(scriptText).getClass().newInstance();
            scriptClassCach.put(scriptTextHashCode, groovyScript);
        }
        return scriptClassCach.get(scriptTextHashCode);
    }

    /**
     * 解析协议数据
     */
/*	@Override
	public ParseResult analyze(String scriptText, Object[] args) {
		ParseResult parseResult = new ParseResult();
		try {
			GroovyShell shell = new GroovyShell();
			Script groovyScript = shell.parse(scriptText);
			Object result = groovyScript.invokeMethod(PARSE_DATA_METHOD_NAME,
					args);
			parseResult.setStatus(IotConstant.SUCCESS);
			parseResult.setResult(result);
		} catch (Exception e) {
			Logger.getLogger(this.getClass()).error(
					"parse protocol data error!" + e);
			parseResult.setStatus(IotConstant.FAILURE);
			parseResult.setErrorMsg(e.getMessage());
		}
		return parseResult;
	}*/

}
