/**
 *
 */
package com.yeejoin.amos.iot.snmp;

import com.adventnet.snmp.snmp2.SnmpVarBind;
import com.yeejoin.amos.iot.business.constants.IotConstant;
import com.yeejoin.amos.iot.business.dic.IotDic;
import com.yeejoin.amos.iot.business.polling.PollingResult;
import com.yeejoin.amos.iot.business.queue.AlarmDataObj;
import com.yeejoin.amos.iot.business.queue.AlarmDataQueue;
import com.yeejoin.amos.iot.business.queue.EqpConnData;
import org.apache.commons.lang.ArrayUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;

import java.util.HashMap;
import java.util.Map;

/**
 * snmp数据采集/控制service
 *
 * @author as-youjun
 */
public class SnmpCollectService {

    /**
     * 日志记录器
     */
    private static final Logger logger = Logger
            .getLogger(SnmpCollectService.class);

    /**
     * 获取协议数据
     *
     * @param paramMap
     * @return
     */
    public PollingResult fetchSnmpData(Map<String, Object> paramMap) {
        PollingResult pollingResult = new PollingResult();
        //snmp是否连通
        boolean isSnmpConnected = false;
        String errorMessage = null;
        try {
            SnmpParam snmpParam = new SnmpParam();
            snmpParam.setIp((String) paramMap.get("ip"));
            snmpParam.setPort(Integer.valueOf(paramMap.get("port").toString()));
            snmpParam.setRdComm((String) paramMap.get("rdCom"));
            if (paramMap.get("version") != null) {
                snmpParam.setVersion((Integer) paramMap.get("version"));
            } else {
                snmpParam.setVersion(SnmpVersion.V2C.getValue());
            }
            if (paramMap.get("timeout") != null) {
                snmpParam.setTimeout((Integer) paramMap.get("timeout"));
            } else {
                snmpParam.setTimeout(3000);
            }
            if (paramMap.get("trytimes") != null) {
                snmpParam.setTryTimes((Integer) paramMap.get("trytimes"));
            } else {
                snmpParam.setTryTimes(3);
            }
            if (paramMap.get("maxRows") != null) {
                snmpParam.setMaxRows((Integer) paramMap.get("maxRows"));
            } else {
                snmpParam.setMaxRows(1000);
            }
            String[] oids = null;
            String defalultOid = ".1.3.6.1.2.1.1.2";
            if (paramMap.get("oids") == null) {
                // 默认oid：.1.3.6.1.2.1.1.2
                oids = new String[]{defalultOid};
            } else {
                String oidStr = (String) paramMap.get("oids");
                if (StringUtils.isEmpty(oidStr)) {
                    oids = new String[]{defalultOid};
                } else {
                    oids = oidStr.split(",");
                }
            }
            snmpParam.setOids(oids);
            SnmpVarBind[][] snmpvarbindss = SnmpOpDevice
                    .snmpGetAllVariableBindings(snmpParam);
            if (ArrayUtils.isEmpty(snmpvarbindss)) {
                errorMessage = "：设备SNMP不通！";
                isSnmpConnected = false;
            } else {
                isSnmpConnected = true;
                pollingResult.setStatus(IotConstant.SUCCESS);
                Map<String, String> valueMap = new HashMap<>();
                for (SnmpVarBind[] svbs : snmpvarbindss) {
                    if (ArrayUtils.isEmpty(svbs)) {
                        continue;
                    }
                    for (SnmpVarBind svb : svbs) {
                        valueMap.put(svb.getObjectID().toString(), svb.getVariable().toString());
                    }
                }
                pollingResult.setResult(valueMap);
            }
            snmpvarbindss = null;
        } catch (Exception e) {
            isSnmpConnected = false;
            errorMessage = e.getMessage();
            Logger.getLogger(this.getClass()).error(e);
        }
        //设置snmp结果
        if (isSnmpConnected) {//snmp获取数据成功
            pollingResult.setStatus(IotConstant.SUCCESS);
        } else {//snmp获取数据失败
            pollingResult.setStatus(IotConstant.FAILURE);
            pollingResult.setErrorMsg(errorMessage);
        }
        if (paramMap.get(IotConstant.EquipmentConstant.EQPID) == null) {
            //测试设备，无需产生告警
        } else {
            long eqpId = (Long) paramMap.get(IotConstant.EquipmentConstant.EQPID);
            raiseAlarm(eqpId, isSnmpConnected);
        }

        return pollingResult;
    }

    /**
     * 产生设备snmp连通性告警
     *
     * @param eqpId
     * @param isSnmpConnected
     */
    private void raiseAlarm(long eqpId, boolean isSnmpConnected) {
        EqpConnData eqpConnData = new EqpConnData();
        eqpConnData.setEqpId(eqpId);
        eqpConnData.setProtocolName(IotDic.ProtocolName.SNMP);
        eqpConnData.setConnected(isSnmpConnected);
        AlarmDataObj alarmDataObj = new AlarmDataObj(eqpConnData);
        alarmDataObj.setType(IotConstant.EQUIPMENT_CONNECTION);
        AlarmDataQueue.getInstance().add(alarmDataObj);
    }

}
