
//字符串转换成字节数组
function stringToByte(str) {  
    var bytes = new Array();  
    var len, c;  
    len = str.length;  
    for(var i = 0; i < len; i++) {  
        c = str.charCodeAt(i);  
        if(c >= 0x010000 && c <= 0x10FFFF) {  
            bytes.push(((c >> 18) & 0x07) | 0xF0);  
            bytes.push(((c >> 12) & 0x3F) | 0x80);  
            bytes.push(((c >> 6) & 0x3F) | 0x80);  
            bytes.push((c & 0x3F) | 0x80);  
        } else if(c >= 0x000800 && c <= 0x00FFFF) {  
            bytes.push(((c >> 12) & 0x0F) | 0xE0);  
            bytes.push(((c >> 6) & 0x3F) | 0x80);  
            bytes.push((c & 0x3F) | 0x80);  
        } else if(c >= 0x000080 && c <= 0x0007FF) {  
            bytes.push(((c >> 6) & 0x1F) | 0xC0);  
            bytes.push((c & 0x3F) | 0x80);  
        } else {  
            bytes.push(c & 0xFF);  
        }  
    }  
    return bytes;  
}

//字节数组转换成字符串
function byteToString(arr) {  
    if(typeof arr === 'string') {  
        return arr;  
    }  
    var str = '',  
        _arr = arr;  
    for(var i = 0; i < _arr.length; i++) {  
        var one = _arr[i].toString(2),  
            v = one.match(/^1+?(?=0)/);  
        if(v && one.length == 8) {  
            var bytesLength = v[0].length;  
            var store = _arr[i].toString(2).slice(7 - bytesLength);  
            for(var st = 1; st < bytesLength; st++) {  
                store += _arr[st + i].toString(2).slice(2);  
            }  
            str += String.fromCharCode(parseInt(store, 2));  
            i += bytesLength - 1;  
        } else {  
            str += String.fromCharCode(_arr[i]);  
        }  
    }  
    return str;  
}

//从字节数组中解析GPS数据
function parseData(arr, sourceId){
	var errorMessage = "";
	//时间
	var time;
	//状态
	var status = "";
	//维度
	var latitude;
	//北半球或南半球指示器（N/S）
	var indicatorNS = "";
	//经度
	var longitude;
	//东半球或西半球指示器（E/W）
	var indicatorEW = "";
	//对地速度
	var speed;
    //对地方向
	var direction;
	//日期
	var dayTime;
    //磁极变量
	var poleNum;
	//总和检查码
	var checkNum;
	//原始字符串信息
	var sourceString;
	//原始bytes信息
	var sourceBytes;
	try {
		var str = byteToString(arr);
		str = str.substr(0, str.length-3);
		sourceString = str;
		var sBytes = stringToByte(str);
		sourceBytes = bytes2HexStringWithDelimiter(sBytes, ',');
		var array = str.split(',');
		time = transTime(array[1], array[9]);
		status = transStatus(array[2]);
		latitude = transLatitude(array[3], array[4]);
		indicatorNS = array[4];
		longitude = transLongitude(array[5], array[5]);
		indicatorEW = array[6];
		speed = array[7];
		direction = array[8];
		dayTime = array[9];
		poleNum = array[10];
		checkNum = array[12];
	} catch (e) {
		errorMessage = e;
	}
	var gpsData = {
			sourceID:sourceId,
			formatTime:time,
			status:status,
			latitude:latitude,
			indicatorNS:indicatorNS,
			longitude:longitude,
			indicatorEW:indicatorEW,
			speed:speed,
			direction:direction,
			dayTime:dayTime,
			poleNum:poleNum,
			checkNum:checkNum,
			sourceString:sourceString,
			sourceBytes:sourceBytes,
			errorMessage:errorMessage
		};
	return JSON.stringify(gpsData);
}

//解析时间
function transTime(timeStr, dayStr){
	if("" == timeStr || "" == dayStr){
		 return "空";
	}
	var year = parseInt('20' + dayStr.substr(4, 2));
	var month = parseInt(dayStr.substr(2, 2)) - 1;
	var day = parseInt(dayStr.substr(0, 2));
	var hours = parseInt(timeStr.substr(0, 2));
	var minutes = parseInt(timeStr.substr(2, 2));
	var seconds = parseInt(timeStr.substr(4, 2));
	var milliseconds = 0;
	var time = new Date(year, month, day, hours, minutes, seconds, milliseconds);
	//time.setHours(time.getHours() + 8);
	return dateFtt("yyyy-MM-dd hh:mm:ss", time);
}

/**************************************时间格式化处理************************************/
function dateFtt(fmt,date){  
  var o = {   
    "M+" : date.getMonth()+1,                 //月份   
    "d+" : date.getDate(),                    //日   
    "h+" : date.getHours(),                   //小时   
    "m+" : date.getMinutes(),                 //分   
    "s+" : date.getSeconds(),                 //秒   
    "q+" : Math.floor((date.getMonth()+3)/3), //季度   
    "S"  : date.getMilliseconds()             //毫秒   
  };   
  if(/(y+)/.test(fmt)){
    fmt=fmt.replace(RegExp.$1, (date.getFullYear()+"").substr(4 - RegExp.$1.length));   
  }
  for(var k in o){
	  if(new RegExp("("+ k +")").test(fmt)){
		  fmt = fmt.replace(RegExp.$1, (RegExp.$1.length==1) ? (o[k]) : (("00"+ o[k]).substr((""+ o[k]).length))); 
	  }   
  }
  return fmt;   
}

//解析时间
function transStatus(statusStr){
	var status = "";
	if ('A' == statusStr) {
		status = "信息可用";
	} else if("V" == statusStr){
		status = "信息不可用";
	}
	return status;
}

//解析维度
function transLatitude(latitude, indicator){
	if ('' == latitude) {
		return "未知";
	}
	var x = parseFloat(latitude.substring(0, 2));
	var y = parseFloat(latitude.substring(2, latitude.length));
	var locate = x + y / 60;
	if ("S" == indicator) {
	    locate = 0 - locate;
	}
	return locate;
}

//解析经度
function transLongitude(longitude, indicator){
	if ('' == longitude) {
		return "未知";
	}
	var x = parseFloat(longitude.substring(0, 3));
	var y = parseFloat(longitude.substring(3, longitude.length));
	var locate = x + y / 60;
	if ("W" == indicator) {
	    locate = 360 - locate;
	}
	return locate;
}

//字节数组转换成16进制字符串
function bytes2HexStringWithDelimiter(b, delimiter){
	var hex = stringToByte("0123456789ABCDEF");
	var buff = new Array(2 * b.length);
	for (var i = 0; i < b.length; i++){
		buff[2 * i] = hex[(b[i] >> 4) & 0x0f];
		buff[2 * i + 1] = hex[b[i] & 0x0f];
	}
	var result = byteToString(buff);
	var str = "";
	for (var i = 0; i < result.length/2; i++){
		str += result.substring(2*i, 2*(i+1));
		if (i != (result.length/2 - 1)){
			str += delimiter;
		}
	}
	return str;
}
