package com.yeejoin.amos;

import com.yeejoin.amos.iot.business.polling.PollingJobStarter;
import com.yeejoin.amos.iot.business.queue.AlarmDataQueue;
import com.yeejoin.amos.iot.context.IotContext;
import com.yeejoin.amos.iot.netty.ReceiveEqpCache;
import com.yeejoin.amos.iot.netty.gateway.UsrTcpServer;
import com.yeejoin.amos.iot.netty.mqtt.MqttFrameQueue;
import com.yeejoin.amos.iot.netty.vehicle.VehicleDataQueue;
import com.yeejoin.amos.iot.netty.vehicle.VehicleFrameQueue;
import com.yeejoin.amos.iot.netty.vehicle.test.TestVehicleDataQueue;
import com.yeejoin.amos.op.core.util.ApplicationConfig;
import org.springframework.beans.BeansException;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.boot.web.servlet.ServletComponentScan;
import org.springframework.cloud.client.discovery.EnableDiscoveryClient;
import org.springframework.cloud.openfeign.EnableFeignClients;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.core.annotation.Order;
import org.springframework.core.env.Environment;
import org.springframework.data.jpa.repository.config.EnableJpaAuditing;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.transaction.annotation.EnableTransactionManagement;
import org.springframework.web.cors.CorsConfiguration;
import org.springframework.web.cors.UrlBasedCorsConfigurationSource;
import org.springframework.web.filter.CorsFilter;
import springfox.documentation.swagger2.annotations.EnableSwagger2;

import java.io.IOException;
import java.net.URISyntaxException;

/**
 * amosiot启动类
 *
 * @author as-youjun
 */
@SpringBootApplication
@EnableTransactionManagement
@EnableConfigurationProperties
@EnableSwagger2
@ServletComponentScan
@EnableDiscoveryClient
@EnableFeignClients
//@Configuration
//@EnableApolloConfig
@EnableScheduling
@EnableJpaAuditing
@ComponentScan({"org.typroject","com.yeejoin.amos","com.yeejoin.amos.iot"})
public class AmosIotStart implements ApplicationContextAware {

    /**
     * 启动amosiot-server
     *
     * @param args
     * @throws IOException
     * @throws URISyntaxException
     */
    public static void main(String[] args) {
        System.out.println("amosiot server starting...");
        try {
            SpringApplication application = new SpringApplication(
                    AmosIotStart.class);
            Environment environment = application.run(args).getEnvironment();
            System.out.println("amosiot server is running...");
            System.out.println("SwaggerUI: 	http://localhost:"
                    + environment.getProperty("server.port")
                    + "/swagger-ui.html");
        } catch (Exception e) {
            System.out.println("error occur when run aomsiot server! " + e);
        }
    }

    /**
     * 获取并缓存上下文
     */
    @Override
    public void setApplicationContext(ApplicationContext applicationContext)
            throws BeansException {
        // 1.缓存spring上下文
        IotContext.getInstance().setApplicationContext(applicationContext);
        // 2.调度轮询job
        PollingJobStarter pollingJobStarter = (PollingJobStarter) applicationContext
                .getBean("pollingJobStarter");
        pollingJobStarter.start();
        // 3.启动车载终端数据处理线程
        VehicleFrameQueue.getInstance().start();
        VehicleDataQueue.getInstance().start();
        AlarmDataQueue.getInstance().start();
        TestVehicleDataQueue.getInstance().start();
        ReceiveEqpCache.getInstance().start();
        MqttFrameQueue.getInstance().start();
        new UsrTcpServer((ApplicationConfig) applicationContext
                .getBean("applicationConfig")).start();
        //4.调度设备连通性监测任务
//        EqpConnCheckService eqpConnCheckService = (EqpConnCheckService) applicationContext
//                .getBean("eqpConnCheckService");
//        eqpConnCheckService.start();
    }

    /**
     * 跨域过滤器
     *
     * @return
     */
    @Bean
    @Order(0)
    public CorsFilter corsFilter() {
        final UrlBasedCorsConfigurationSource source = new UrlBasedCorsConfigurationSource();
        final CorsConfiguration config = new CorsConfiguration();
        config.setAllowCredentials(true);// 允许cookies跨域
        config.addAllowedOrigin("*");// #允许向该服务器提交请求的URI，*表示全部允许，在SpringMVC中，如果设成*，会自动转成当前请求头中的Origin
        config.addAllowedHeader("*");// #允许访问的头信息,*表示全部
        config.setMaxAge(3600L);// 预检请求的缓存时间（秒），即在这个时间段里，对于相同的跨域请求不会再预检了
        config.addAllowedMethod("OPTIONS");
        config.addAllowedMethod("HEAD");
        config.addAllowedMethod("GET");
        config.addAllowedMethod("PUT");
        config.addAllowedMethod("POST");
        config.addAllowedMethod("DELETE");
        config.addAllowedMethod("PATCH");
        source.registerCorsConfiguration("/**", config);
        return new CorsFilter(source);
    }

}