package com.yeejoin.amos;

import org.apache.log4j.Logger;
import org.dom4j.Attribute;
import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.dom4j.io.OutputFormat;
import org.dom4j.io.SAXReader;
import org.dom4j.io.XMLWriter;
import org.dom4j.tree.DefaultElement;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.junit4.SpringRunner;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.StringReader;
import java.io.StringWriter;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

@RunWith(SpringRunner.class)
@SpringBootTest(classes = AmosIotStart.class, webEnvironment = SpringBootTest.WebEnvironment.RANDOM_PORT)
public class TransferXmlData {

    private static final Logger logger = Logger.getLogger(TransferXmlData.class);

    private static final String XmlPrePath = "/template/xml/";

    private static final String TARGET_XML_BAK = "/template/xml/xmlbak/";

    private static SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

    private String sourceId;

    private String fileName;

    @Test
    public void send() {
        TransferXmlData transferXmlData = new TransferXmlData();
        logger.info(transferXmlData.sendXml("123", "设备-网管系统-IP设备.xml"));
    }

    private void createXmlToTarget(TransferXmlData transferXmlData) {
        File dir = new File(TARGET_XML_BAK);
        String[] fileNames = dir.list();
        for (String fileName : fileNames) {
            logger.info(fileName);
            String sourceId = "sourceId";
            String xmlcontent = transferXmlData.sendXml(sourceId, fileName);
            FileWriter writer;
            try {
                File file = new File(TARGET_XML_BAK + "/" + fileName);
                if (file.exists())
                    file.delete();
                writer = new FileWriter(TARGET_XML_BAK + "/" + fileName);
                writer.write(xmlcontent);
                writer.flush();
                writer.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    public String sendXml(String sourceId, String fileName) {
        String sendxml = "";
        this.sourceId = sourceId;
        this.fileName = fileName;
        try {
            Document document = readXmlDataFromTemplate(fileName);
            Element rootElement = document.getRootElement();
            Element businessInfoElement = rootElement.element("data").element("businessInfo");
            businessInfoElement.remove(businessInfoElement.element("basicInfo"));
            Map<String, Object> xmlMap = multilayerXmlToMap(document.asXML());
            String xmlcontent = multilayerMapToXml((Map<String, Object>) xmlMap.get("root"), false);
            sendxml = xmlcontent;
        } catch (Exception e) {
            logger.error("创建xml报文异常", e);
        }
        return sendxml;
    }


    /**
     * 从文件读取xml文件
     */
    private Document readXmlDataFromTemplate(String fileName) throws Exception {
        Document doc;
        InputStream in = null;
        SAXReader reader = new SAXReader();
        try {
            String path = XmlPrePath + "/" + fileName;
            //in = new FileInputStream(path);
            in = TransferXmlData.class.getResourceAsStream(path);
            doc = reader.read(in);
        } finally {
            if (in != null) {
                in.close();
            }
        }
        return doc;
    }

    /**
     * 枚举值随机选取
     */
    private String getEnumValue(String value) {
        String[] valueArray = value.split(";");
        int count = (int) (Math.random() * valueArray.length);
        return valueArray[count];
    }


    private String getNumber() {
        int number = (int) (Math.random() * 300);
        //DecimalFormat df = new java.text.DecimalFormat("#.00");
        return String.valueOf(number);
    }


    private String getDateStr() {
        return sdf.format(new Date());
    }

    private String getRandomStr() {
        return UUID.randomUUID().toString();
    }

    private String getSourceId() {
        return this.sourceId;
    }

    private String getFileName() {
        return this.fileName;
    }


    /**
     * (多层)xml格式字符串转换为map
     *
     * @param xml xml字符串
     * @return 第一个为Root节点，Root节点之后为Root的元素，如果为多层，可以通过key获取下一层Map
     */
    private Map<String, Object> multilayerXmlToMap(String xml) {
        Document doc = null;
        try {
            doc = DocumentHelper.parseText(xml);
        } catch (DocumentException e) {
            logger.error("xml字符串解析，失败 --> {}", e);
        }
        Map<String, Object> map = new HashMap<>();
        if (null == doc) {
            return map;
        }
        // 获取根元素
        Element rootElement = doc.getRootElement();
        recursionXmlToMap(rootElement, map);
        return map;
    }

    /**
     * multilayerXmlToMap核心方法，递归调用
     *
     * @param element 节点元素
     * @param outmap  用于存储xml数据的map
     */
    @SuppressWarnings("unchecked")
    private void recursionXmlToMap(Element element, Map<String, Object> outmap) {
        // 得到根元素下的子元素列表
        List<Element> list = element.elements();

        List<Attribute> attributeList = element.attributes();
        if (attributeList.size() > 0)
            for (Attribute attribute : attributeList)
                outmap.put(attribute.getName() + "@" + element.getName(), attribute.getValue());
        int size = list.size();
        if (size == 0) {
            // 如果没有子元素,则将其存储进map中
            if (outmap.containsKey(element.getName())) {
                outmap.put(element.getName() + "#" + (outmap.size() + 1), element.getTextTrim());
                logger.info("*******重复的节点：" + element.getName());
            } else {
                outmap.put(element.getName(), element.getTextTrim());
            }

        } else {
            // innermap用于存储子元素的属性名和属性值
            Map<String, Object> innermap = new HashMap<>();
            // 遍历子元素
            list.forEach(childElement -> recursionXmlToMap(childElement, innermap));
            if (outmap.containsKey(element.getName())) {
                logger.info("*******重复的节点：" + element.getName());
                outmap.put(element.getName() + "#" + (outmap.size() + 1), innermap);
            } else {
                outmap.put(element.getName(), innermap);
            }


        }
    }


    /**
     * (多层)map转换为xml格式字符串
     *
     * @param map     需要转换为xml的map
     * @param isCDATA 是否加入CDATA标识符 true:加入 false:不加入
     * @return xml字符串
     */
    private String multilayerMapToXml(Map<String, Object> map, boolean isCDATA) {
        String parentName = "root";
        Document doc = DocumentHelper.createDocument();
        doc.addElement(parentName);
        String xml = recursionMapToXml(doc.getRootElement(), parentName, map, new HashMap<>(), isCDATA);
        return formatXML(xml);
    }

    /**
     * multilayerMapToXml核心方法，递归调用
     *
     * @param element    节点元素
     * @param parentName 根元素属性名
     * @param map        需要转换为xml的map
     * @param isCDATA    是否加入CDATA标识符 true:加入 false:不加入
     * @return xml字符串
     */
    @SuppressWarnings("unchecked")
    private String recursionMapToXml(Element element, String parentName, Map<String, Object> map, Map<String, Object> parentMap, boolean isCDATA) {
        Element xmlElement = element.addElement(parentName);
        map.keySet().forEach(key -> {
            Object obj = map.get(key);
            if (obj instanceof Map) {
                recursionMapToXml(xmlElement, key, (Map<String, Object>) obj, map, isCDATA);
            } else {
                String value = obj == null ? "" : obj.toString();
                if (!key.contains("@")) {
                    if (key.contains("#"))
                        key = key.substring(0, key.indexOf("#"));
                    if (isCDATA) {
                        xmlElement.addElement(key).addCDATA(value);
                    } else {

                        DefaultElement defaultElement = new DefaultElement(key);
                        String elementName = defaultElement.getName();
                        Map<String, String> attrValue = getAttrValue(map, elementName);

                        if (attrValue.size() > 0) {
                            DataType dataType = DataType.valueOf(attrValue.get("dataType"));
                            String enumValue = attrValue.get("enumValue");

                            value = this.genValue(dataType, value, enumValue);

                            //defaultElement.addAttribute(attrValue.get("attrName"), attrValue.get("attrValue"));
                        }
                        //defaultElement.addAttribute("dataType","number");
                        //defaultElement.addAttribute("enumValue","");
                        defaultElement.addText(value);
                        xmlElement.add(defaultElement);
                    }


                    String elementName = xmlElement.getName();
                    Map<String, String> attrValue = getAttrValue(parentMap, elementName);
                    if (attrValue.size() > 0)
                        for (String attrName : attrValue.keySet())
                            xmlElement.addAttribute(attrName, attrValue.get(attrName));
                }

            }
        });
        return xmlElement.asXML();
    }


    private String genValue(DataType dataType, String value, String enumValue) {
        switch (dataType) {
            //数字：直接生成随机数字
            //枚举：将所有枚举值写当前节点的textValue中，随机拿取一个。
            //随机字符串：直接随机一个字符串
            case number:
                value = getNumber();
                break;
            case date:
                value = getDateStr();
                break;
            case enum_str:
                value = getEnumValue(enumValue);
                break;
            case random_str:
                value = getRandomStr();
                break;
            case sourceId:
                value = getSourceId();
                break;
        }
        return value;
    }

    private Map<String, String> getAttrValue(Map<String, Object> parentMap, String elementName) {
        Map<String, String> attrValue = new HashMap<>();
        for (String key : parentMap.keySet()) {
            if (key.contains("@")) {
                String[] str = key.split("@");
                if (str.length == 2 && elementName.equals(str[1])) {
                    attrValue.put(str[0], String.valueOf(parentMap.get(key)));
                }
            }
        }
        return attrValue;
    }

    /**
     * 格式化xml,显示为容易看的XML格式
     *
     * @param xml 需要格式化的xml字符串
     */
    public String formatXML(String xml) {
        String requestXML = null;
        try {
            // 拿取解析器
            SAXReader reader = new SAXReader();
            Document document = reader.read(new StringReader(xml));
            if (null != document) {
                StringWriter stringWriter = new StringWriter();
                // 格式化,每一级前的空格
                OutputFormat format = new OutputFormat("    ", true);
                // xml声明与内容是否添加空行
                format.setNewLineAfterDeclaration(false);
                // 是否设置xml声明头部
                format.setSuppressDeclaration(false);
                // 是否分行
                format.setNewlines(true);
                XMLWriter writer = new XMLWriter(stringWriter, format);
                writer.write(document);
                writer.flush();
                writer.close();
                requestXML = stringWriter.getBuffer().toString();
            }
            return requestXML;
        } catch (Exception e) {
            logger.error("格式化xml，失败 --> {}", e);
            return null;
        }
    }


    enum DataType {

        number, //数字
        date,   //日期
        random_str,//随机字符串
        enum_str,    //枚举
        fixed,//固定不变值
        sourceId
    }
}
